package csbase.algorithms.flows;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;

import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.exception.BugException;

/**
 * 
 * 
 *
 * @author lmoreira
 */
public final class AlgorithmVersionTransferable implements Transferable {
  public static final DataFlavor DATA_FLAVOR;
  static {
    try {
      DATA_FLAVOR = new DataFlavor(DataFlavor.javaJVMLocalObjectMimeType
        + ";class=" + AlgorithmVersionInfo.class.getName());
    } catch (ClassNotFoundException e) {
      throw new BugException(e);
    }
  }

  private AlgorithmVersionInfo version;
  
  public AlgorithmVersionTransferable(AlgorithmVersionInfo version) {
    if (version == null) {
      throw new IllegalArgumentException("O parmetro version est nulo.");
    }
    this.version = version;
  }
  
  public DataFlavor[] getTransferDataFlavors() {
    return new DataFlavor[] { DATA_FLAVOR };
  }

  public boolean isDataFlavorSupported(DataFlavor flavor) {
    if (DATA_FLAVOR.equals(flavor)) {
      return true;
    }
    return false;
  }

  public AlgorithmVersionInfo getTransferData(DataFlavor flavor)
    throws UnsupportedFlavorException {
    if (!isDataFlavorSupported(flavor)) {
      throw new UnsupportedFlavorException(flavor);
    }
    return this.version;
  }
}
