package csbase.algorithms.flows.actions;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.KeyStroke;

import csbase.algorithms.flows.FlowApplication;
import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.GraphElement;
import csbase.algorithms.flows.graph.GraphFileDescriptor;
import csbase.algorithms.flows.graph.GraphLink;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.client.applications.ApplicationImages;

/**
 * Ao que executa o fluxo de algoritmos corrente.
 */
public final class ExecuteGraphAction extends FlowApplicationAction {
  /**
   * Cria a ao.
   * 
   * @param application A aplicao construtura de fluxos.
   */
  public ExecuteGraphAction(FlowApplication application) {
    super(application,
      ExecuteGraphAction.class.getName() + ".name",
      (ApplicationImages.ICON_PLAY_16), KeyStroke.getKeyStroke(
      KeyEvent.VK_ENTER, 0));
    setEnabled(false);
  }

  /**
   * {@inheritDoc}
   */
  public void actionPerformed(ActionEvent e) {
    getApplication().executeGraph();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void wasChangedWorkspace(Graph graph) {
    updateEnabledStatus(graph);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void wasElementCreated(Graph graph, GraphElement element) {
    updateEnabledStatus(graph);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void wasElementParametrized(Graph graph, GraphElement element) {
    updateEnabledStatus(graph);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void wasElementRemoved(Graph graph, GraphElement element) {
    updateEnabledStatus(graph);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void wasLinkAnchored(Graph graph, GraphLink link,
    GraphFileDescriptor fileDescriptor) {
    updateEnabledStatus(graph);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void wasLinkIncreased(Graph graph, GraphLink link) {
    updateEnabledStatus(graph);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void wasLinkStatusChanged(Graph graph, GraphLink link) {
    updateEnabledStatus(graph);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void wasLinkUnanchored(Graph graph, GraphLink link,
    GraphFileDescriptor fileDescriptor) {
    updateEnabledStatus(graph);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void wasParameterSetEnabled(Graph graph, GraphNode node,
    String parameterName, boolean isEnabled) {
    updateEnabledStatus(graph);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void wasParameterSetVisible(Graph graph, GraphNode node,
    String parameterName, boolean isVisible) {
    updateEnabledStatus(graph);
  }

  /**
   * Habilita/desabilita a ao, dependendo do estado do grafo.
   * 
   * @param graph O grafo.
   */
  private void updateEnabledStatus(Graph graph) {
    setEnabled(graph.isExecutable());
  }
}
