package csbase.algorithms.flows.filters;

import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;

import csbase.algorithms.flows.Workspace;
import csbase.algorithms.flows.actions.AskForParameterValuesAction;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.algorithms.flows.messages.PickNodeMessage;

/**
 * Filtro Vix para solicitar valores dos parmetros do configurador de 
 * algoritmos de um n. 
 */
public final class AskForParameterValuesActionFilter extends WorkspaceFilter {
  /**
   * A ao que solicita valores.
   */
  private AskForParameterValuesAction parameterAction;

  /**
   * Cria um filtro.
   * 
   * @param action A ao que solicita valores.
   * @param workspace A rea de trabalho.
   */
  public AskForParameterValuesActionFilter(
    AskForParameterValuesAction action, Workspace workspace) {
    this(workspace);
    setAction(action);
  }

  /**
   * Cria um filtro.
   * 
   * @param workspace A rea de trabalho.
   */
  public AskForParameterValuesActionFilter(Workspace workspace) {
    super(workspace);
  }

  @Override
  public void callbackButton(Point2D pt, MouseEvent ev) {
    if (
      ev.getButton() == MouseEvent.BUTTON1 &&
      ev.getID() == MouseEvent.MOUSE_CLICKED && ev.getClickCount() == 2) {
      PickNodeMessage pickNodeMessage = new PickNodeMessage(pt);
      pickNodeMessage.sendVO(this);
      GraphNode node = pickNodeMessage.getNode();
      if (node != null) {
        if (node.getAlgorithmConfiguratorView() != null) {
          node.askForParameterValues();
        }
        else {
          node.showError();
        }
      }
    }
    super.callbackButton(pt, ev);
  }

  /**
   * Obtm a ao que solicita os valores.
   * 
   * @return A ao que solicita os valores.
   */
  public AskForParameterValuesAction getAction() {
    return this.parameterAction;
  }

  /**
   * Atribui a ao que solicita os valores a esta instncia.
   * 
   * @param action A ao que solicita os valores.
   */
  private void setAction(AskForParameterValuesAction action) {
    if (action == null) {
      throw new IllegalArgumentException("O parmetro action est nulo.");
    }
    this.parameterAction = action;
    this.parameterAction.setEnabled(false);
  }
}
