package csbase.algorithms.flows.filters;

import csbase.algorithms.flows.Workspace;
import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.GraphElement;
import csbase.algorithms.flows.messages.PickGraphMessage;

import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;

import java.util.Collection;
import java.util.Iterator;

public final class HighlightElementFilter extends WorkspaceFilter {
  private Collection<GraphElement> elementCollection;

  public HighlightElementFilter(Workspace workspace) {
    super(workspace);
  }

  @Override
  public void callbackMove(Point2D pt, MouseEvent ev) {
    restart();
    PickGraphMessage message = new PickGraphMessage();
    message.sendVO(this);
    Graph graph = message.getGraph();
    this.elementCollection = graph.getElementCollection(pt);
    if (!this.elementCollection.isEmpty()) {
      Iterator<GraphElement> elementIterator = this.elementCollection.iterator();
      while (elementIterator.hasNext()) {
        GraphElement element = elementIterator.next();
        element.turnOnHighlight(pt);
      }
      repaint();
    }
    else {
      this.elementCollection = null;
    }
    super.callbackMove(pt, ev);
  }

  @Override
  public void wasElementRemoved(Graph graph, GraphElement element) {
    restart();
  }

  @Override
  public void wasReseted(Graph graph) {
    restart();
  }

  private void restart() {
    if (this.elementCollection != null) {
      Iterator<GraphElement> elementIterator = this.elementCollection.iterator();
      while (elementIterator.hasNext()) {
        GraphElement element = elementIterator.next();
        element.turnOffHighlight();
      }
      repaint();
      this.elementCollection = null;
    }
  }
}
