package csbase.algorithms.flows.filters;

import csbase.algorithms.flows.Workspace;
import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.GraphElement;
import csbase.algorithms.flows.graph.GraphFileDescriptor;
import csbase.algorithms.flows.graph.GraphLink;
import csbase.algorithms.flows.graph.GraphListener;
import csbase.algorithms.flows.graph.GraphNode;

import tecgraf.javautils.vix.Filter;
import tecgraf.javautils.vix.TypeVO;
import tecgraf.javautils.vix.TypeVS;

import java.awt.geom.Point2D;

public abstract class WorkspaceFilter extends Filter implements GraphListener {
  private Workspace workspace;

  protected WorkspaceFilter(Workspace workspace) {
    if (workspace == null) {
      throw new IllegalArgumentException("O parmetro workspace est nulo.");
    }
    this.workspace = workspace;
    this.workspace.getGraph().addGraphListener(this);
  }

  public void wasElementParametrized(Graph graph, GraphElement element) {
  }

  public void wasElementCreated(Graph graph, GraphElement element) {
  }

  public void wasElementDragged(
    Graph graph, GraphElement element, Point2D startPoint, Point2D endPoint) {
  }

  public void wasElementDragged(
    Graph graph, GraphElement element, double tx, double ty) {
  }

  public void wasElementDropped(
    Graph graph, GraphElement element, Point2D point) {
  }

  public void wasElementRemoved(Graph graph, GraphElement element) {
  }

  public void wasElementSelected(Graph graph, GraphElement element) {
  }

  public void wasLinkAnchored(
    Graph graph, GraphLink link, GraphFileDescriptor fileDescriptor) {
  }

  public void wasLinkIncreased(Graph graph, GraphLink link) {
  }

  public void wasLinkStatusChanged(Graph graph, GraphLink link) {
  }

  public void wasLinkUnanchored(
    Graph graph, GraphLink link, GraphFileDescriptor fileDescriptor) {
  }

  public void wasChangedWorkspace(Graph graph) {
  }

  public void wasNodeResized(Graph graph, GraphNode node) {
  }

  public void wasParameterSetEnabled(
    Graph graph, GraphNode node, String parameterName, boolean isEnabled) {
  }

  public void wasParameterSetVisible(
    Graph graph, GraphNode node, String parameterName, boolean isVisible) {
  }
  
  public void wasReseted(Graph graph) {
  }

  public final void attach() {
    this.workspace.attachFilter(this);
  }

  @Override
  public final boolean changeVS(TypeVS oldVS, TypeVS newVS) {
    if (super.changeVS(oldVS, newVS)) {
      updateListener();
      return true;
    }
    return false;
  }

  @Override
  public final boolean changeVO(TypeVO oldVO, TypeVO newVO) {
    if (super.changeVO(oldVO, newVO)) {
      updateListener();
      return true;
    }
    return false;
  }

  protected final Workspace getWorkspace() {
    return this.workspace;
  }

  @Override
  public final void repaint() {
    if (getVS() != null) {
      super.repaint();
    }
  }

  private void updateListener() {
    if (getVO() == null || getVS() == null) {
      this.workspace.getGraph().removeGraphListener(this);
    }
    else {
      this.workspace.getGraph().addGraphListener(this);
    }
  }
}
