package csbase.algorithms.flows.graph.actions;

import java.awt.event.ActionEvent;
import java.awt.geom.Point2D;

import csbase.algorithms.flows.Workspace;
import csbase.algorithms.flows.actions.Action;
import csbase.algorithms.flows.filters.AddLinkPopupActionFilter;
import csbase.algorithms.flows.filters.WorkspaceFilter;
import csbase.algorithms.flows.graph.GraphLink;
import csbase.client.applications.ApplicationImages;

/**
 * Ao de menu popup que quebra conexes entre ns do fluxo de algoritmos.
 *
 * @author lmoreira
 */
public final class BreakLinkAction extends GraphElementAction {
  private final Point2D point;

  /**
   * Cria ao.
   * 
   * @param link A conexo (No aceita {@code null}).
   * @param point O ponto onde o menu-popup foi pedido 
   * (No aceita {@code null}).
   */
  public BreakLinkAction(GraphLink link, Point2D point) {
    super(link, BreakLinkAction.class.getName()+".name",
      ApplicationImages.ICON_PAUSE_16);
    this.point = (Point2D) point.clone();
  }

  public void actionPerformed(ActionEvent e) {
    GraphLink link = (GraphLink) getElement();
    if (link.breakPoint(this.point)) {
      link.repaint();
    }
  }
  
  /**
   * Cria o filtro que cadastra a ao no menu popup.
   * 
   * @param workspace A rea de trabalho (No aceita {@code null}).
   * 
   * @return O filtro.
   */
  public static WorkspaceFilter createFilter(Workspace workspace) {
    return new AddLinkPopupActionFilter(workspace) {
                                                                                
      @Override
      protected Action createAction(GraphLink graphLink, Point2D point) {
        return new BreakLinkAction(graphLink, point);
      }
    };
  }
}
