package csbase.algorithms.flows.graph.actions;

import csbase.algorithms.flows.Workspace;
import csbase.algorithms.flows.actions.Action;
import csbase.algorithms.flows.filters.AddNodePopupActionFilter;
import csbase.algorithms.flows.filters.WorkspaceFilter;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.algorithms.flows.graph.ParametersTransferable;
import csbase.algorithms.flows.messages.CopyMessage;
import csbase.client.applications.ApplicationImages;

import java.awt.event.ActionEvent;
import java.awt.geom.Point2D;

/**
 * Ao de menu popup que copia os valores dos parmetros de um n de fluxo
 * de algoritmos.
 *
 * @author lmoreira
 */
public final class CopyParameterValuesAction extends GraphElementAction {
  /**
   * Cria a ao.
   * 
   * @param node O n (No aceita {@code null}).
   */
  public CopyParameterValuesAction(GraphNode node) {
    super(node,
      CopyParameterValuesAction.class.getName()+".name",
      ApplicationImages.ICON_COPY_16);
  }

  public void actionPerformed(ActionEvent e) {
    GraphNode node = (GraphNode) getElement();

    new CopyMessage(new ParametersTransferable(node)).sendVS(node);
  }
  
  /**
   * Cria o filtro que cadastra a ao no menu popup.
   * 
   * @param workspace A rea de trabalho (No aceita {@code null}).
   * 
   * @return O filtro.
   */
  public static WorkspaceFilter createFilter(Workspace workspace) {
    if (workspace == null) {
      throw new IllegalArgumentException("O parmetro workspace est nulo.");
    }
    return new AddNodePopupActionFilter(workspace) {

      @Override
      protected Action createAction(GraphNode graphNode, Point2D point) {
        return new CopyParameterValuesAction(graphNode);
      }
      
    };
  }
}
