package csbase.algorithms.flows.graph.actions;

import java.awt.Window;

import javax.swing.ImageIcon;
import javax.swing.KeyStroke;

import tecgraf.javautils.gui.StandardDialogs;

import csbase.algorithms.flows.actions.Action;
import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.GraphElement;
import csbase.client.util.StandardErrorDialogs;

/**
 * Ao abstrata de menu popup para o elementos do fluxo de algoritmos.
 *
 * @author lmoreira
 */
public abstract class GraphElementAction extends Action {
  private GraphElement element;

  /**
   * Cria a ao.
   * 
   * @param element O elemento do fluxo (No aceita {@code null}).
   * @param nameKey O nome da chave do mecanismo de internacionalizao com o 
   * nome da ao.
   * @param icon O cone da ao.
   * @param acceleratorKeyStroke A tecla que dispara a ao.
   */
  protected GraphElementAction(
    GraphElement element, String nameKey, ImageIcon icon,
    KeyStroke acceleratorKeyStroke) {
    super(nameKey, icon, acceleratorKeyStroke);
    this.element = element;
  }

  /**
   * Cria a ao.
   * 
   * @param element O elemento do fluxo (No aceita {@code null}).
   * @param nameKey O nome da chave do mecanismo de internacionalizao com o 
   * nome da ao.
   * @param icon O cone da ao.
   */
  protected GraphElementAction(
    GraphElement element, String nameKey, ImageIcon icon) {
    super(nameKey, icon);
    this.element = element;
  }

  /**
   * Obtm o elemento.
   * 
   * @return O elemento.
   */
  protected GraphElement getElement() {
    return this.element;
  }
  
  /**
   * Obtm a janela pai do menu popup.
   * 
   * @return A janela.
   */
  protected final Window getParentWindow() {
    Graph graph = element.getGraph();
    Window window = graph.getParentWindow();
    return window;
  }
                      
  /**
   * Exibe um dilogo com uma exceo.
   * 
   * @param exception A exceo (No aceita {@code null}).
   */
  protected final void showErrorDialog(Exception exception) {
    Window window = getParentWindow();
    StandardErrorDialogs.showErrorDialog(window,
      exception.getMessage(), exception);
  }

  /**
   * Exibe um dilogo com uma mensagem de erro.
   * 
   * @param message A mensagem de erro (No aceita {@code null}).
   */
  protected final void showErrorDialog(String message) {
    Window window = getParentWindow();
    StandardErrorDialogs.showErrorDialog(window,
      message);
  }
                                      
  /**
   * Exibe um dilogo com um aviso.
   * 
   * @param message O aviso (No aceita {@code null}).
   * @param title O ttulo do dilogo (No aceita {@code null}).
   */
  protected final void showWarning(String message, String title) {
    Window window = getParentWindow();
    StandardDialogs.showWarningDialog(window,
      title, message);
  }
}
