package csbase.algorithms.flows.graph.actions;

import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.geom.Point2D;

import csbase.algorithms.flows.Workspace;
import csbase.algorithms.flows.actions.Action;
import csbase.algorithms.flows.filters.AddNodePopupActionFilter;
import csbase.algorithms.flows.filters.WorkspaceFilter;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.algorithms.flows.graph.ParametersTransferable;
import csbase.algorithms.flows.graph.utils.CopyAndPasteUtil;
import csbase.algorithms.flows.graph.utils.PasteStatus;
import csbase.algorithms.flows.messages.PasteMessage;
import csbase.client.applications.ApplicationImages;

/**
 * Ao de menu popup que colar os valores dos parmetros de um n de fluxo de
 * algoritmos.
 * 
 * @author lmoreira
 */
public final class PasteParameterValuesAction extends GraphElementAction {
  /**
   * Cria a ao.
   * 
   * @param node O n (No aceita {@code null}).
   */
  public PasteParameterValuesAction(GraphNode node) {
    super(node, PasteParameterValuesAction.class.getName() + ".name", ApplicationImages.ICON_PASTE_16);
    boolean clipboardIsEmpty = CopyAndPasteUtil.isClipboardEmpty(node);
    setEnabled(clipboardIsEmpty);
  }

  /**
   * {@inheritDoc}
   */
  public void actionPerformed(ActionEvent event) {
    GraphNode node = (GraphNode) getElement();
    PasteMessage pasteMessage = new PasteMessage();
    pasteMessage.sendVS(node);
    Transferable transferable = pasteMessage.getTransferable();
    if (transferable instanceof ParametersTransferable) {
      ParametersTransferable parametersTransferable =
        (ParametersTransferable) transferable;
      PasteStatus pasteStatus =
        CopyAndPasteUtil.pasteParameters(parametersTransferable, node);
      if (pasteStatus.confirmPaste()
        && !(pasteStatus.getWarnings().isEmpty() && pasteStatus.getErrors()
          .isEmpty())) {
        node.askForParameterValues();
      }
    }

  }

  /**
   * Cria o filtro que cadastra a ao no menu popup.
   * 
   * @param workspace A rea de trabalho (No aceita {@code null}).
   * 
   * @return O filtro.
   */
  public static WorkspaceFilter createFilter(Workspace workspace) {
    if (workspace == null) {
      throw new IllegalArgumentException("O parmetro workspace est nulo.");
    }
    return new AddNodePopupActionFilter(workspace) {

      @Override
      protected Action createAction(GraphNode node, Point2D point) {
        if (!node.isWellFormed()) {
          return null;
        }
        return new PasteParameterValuesAction(node);
      }
    };
  }
}
