package csbase.algorithms.flows.graph.actions;

import java.awt.event.ActionEvent;

import csbase.algorithms.flows.Workspace;
import csbase.algorithms.flows.actions.Action;
import csbase.algorithms.flows.filters.AddGraphPopupActionFilter;
import csbase.algorithms.flows.filters.WorkspaceFilter;
import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.messages.RemoveElementsMessage;
import csbase.client.applications.ApplicationImages;

/**
 * Ao de menu popup que remove elementos que esto selecionados em um fluxo
 * de algoritmos.
 *
 * @author lmoreira
 */
public final class RemoveSelectedElementsAction extends GraphAction {
  /**
   * Cria a ao.
   * 
   * @param graph O fluxo (No aceita {@code null}).
   */
  public RemoveSelectedElementsAction(Graph graph) {
    super(graph,
      RemoveSelectedElementsAction.class.getName()+".name",
      ApplicationImages.ICON_DELETE_16);
    setEnabled(graph.hasSelectedElements());
  }

  public void actionPerformed(ActionEvent e) {
    new RemoveElementsMessage().sendVO(getGraph());
  }

  /**
   * Cria o filtro que cadastra a ao no menu popup.
   * 
   * @param workspace A rea de trabalho (No aceita {@code null}).
   * 
   * @return O filtro.
   */
  public static WorkspaceFilter createFilter(Workspace workspace) {
    if (workspace == null) {
      throw new IllegalArgumentException("O parmetro workspace est nulo.");
    }
    return new AddGraphPopupActionFilter(workspace) {

      @Override
      protected GraphAction createAction(Graph graph) {
        return new RemoveSelectedElementsAction(graph);
      }
    };
  }
}
