package csbase.algorithms.flows.graph.actions;

import java.awt.event.ActionEvent;
import java.awt.geom.Point2D;

import tecgraf.javautils.LNG;
import csbase.algorithms.flows.Workspace;
import csbase.algorithms.flows.actions.Action;
import csbase.algorithms.flows.filters.AddNodePopupActionFilter;
import csbase.algorithms.flows.filters.WorkspaceFilter;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.client.applications.ApplicationImages;
import csbase.logic.ClientProjectFile;

/**
 * Ao que exibe o arquivo de log de um n de fluxo de algoritmos.
 *
 * @author lmoreira
 */
public final class ShowLogFileAction extends GraphElementAction {

  /**
   * Cria a ao.
   * 
   * @param node O n (No aceita {@code null}).
   */
  public ShowLogFileAction(GraphNode node) {
    super(node, ShowLogFileAction.class.getName() + ".name",
      ApplicationImages.ICON_HISTORY_16);
    setEnabled(getElement().canCreateLogFile());
  }

  public void actionPerformed(ActionEvent ev) {
    if (!getElement().showLogFile()) {
      String message = LNG.get(getClass().getName() + ".error_msg");
      String title = LNG.get(getClass().getName() + ".error_title");
      showWarning(message, title);
      return;
    }
  }

  /**
   * Cria o filtro que cadastra a ao no menu popup.
   * 
   * @param workspace A rea de trabalho (No aceita {@code null}).
   * 
   * @return O filtro.
   */
  public static WorkspaceFilter createFilter(Workspace workspace) {
    if (workspace == null) {
      throw new IllegalArgumentException("O parmetro workspace est nulo.");
    }
    return new AddNodePopupActionFilter(workspace) {

      @Override
      protected Action createAction(GraphNode graphNode, Point2D point) {
        return new ShowLogFileAction(graphNode);
      }};
  }
  
  @Override
  protected GraphNode getElement() {
    return (GraphNode) super.getElement();
  }
}
