package csbase.algorithms.flows.graph.utils;

import java.util.List;

/**
 * Representa o resultado de uma operao de colar parmetros em um n Essa
 * operao pode gerar avisos {@link PasteStatus#warnings} e erros
 * {@link PasteStatus#errors} e pode ser cancelada pelo usurio
 * {@link PasteStatus#confirmPaste} .
 * 
 * @author isabella
 */
public class PasteStatus {

  /**
   * Construtor
   * 
   * @param warnings Lista de avisos geradas durante a operao.
   * @param errors Lista de erros gerados durante a operao.
   * @param confirmPaste Sinal de confirmao da operao. Verdadeiro se a
   *        operao no gerou avisos e erros ou se o usurio confirmou a
   *        operao mesmo com os avisos e erros. Falso caso o usurio tenha
   *        cancelado a operaco.
   */
  public PasteStatus(List<String> warnings, List<String> errors,
    boolean confirmPaste) {
    this.errors = errors;
    this.confirmPaste = confirmPaste;
    this.warnings = warnings;
  }

  /**
   * Lista de avisos geradas durante a operao.
   */
  private List<String> warnings;
  /**
   * Lista de erros gerados durante a operao.
   */
  private List<String> errors;
  /**
   * Sinal de confirmao da operao
   */
  private boolean confirmPaste;

  /**
   * Retorna a lista de avisos geradas durante a operao.
   * 
   * @return warnings Lista de avisos
   */
  public List<String> getWarnings() {
    return warnings;
  }

  /**
   * Retorna a lista de erros gerados durante a operao.
   * 
   * @return errors Lista de erros
   */
  public List<String> getErrors() {
    return errors;
  }

  /**
   * Retorna o sinal de confirmao da operao. Verdadeiro se a operao no
   * gerou avisos e erros ou se o usurio confirmou a operao mesmo com os
   * avisos e erros. Falso caso o usurio tenha cancelado a operaco.
   * 
   * @return confirmPaste O Sinal de confirmao.
   */
  public boolean confirmPaste() {
    return confirmPaste;
  }
}