/*
 * $Author:$
 * $Date:$
 * $Release:$
 */
package csbase.client.algorithms.parameters;

import java.util.LinkedList;
import java.util.List;

/**
 * Modelo de campo de texto nmero abstrato.
 * 
 * @param <N> O tipo do nmero.
 *
 * @author lmoreira
 */
public abstract class AbstractNumberTextFieldModel<N extends Number>
implements NumberTextFieldModel<N> {
  /**
   * Os observadores.
   */
  private List<NumberTextFieldModelListener> listeners;

  /**
   * Cria o modelo.
   */
  protected AbstractNumberTextFieldModel() {
    this.listeners = new LinkedList<NumberTextFieldModelListener>();
  }

  public final void addNumberTextFieldModelListener(
    NumberTextFieldModelListener listener) {
    if (listener == null) {
      throw new IllegalArgumentException("O parmetro listener est nulo.");
    }
    this.listeners.add(listener);
  }

  public final boolean removeNumberTextFieldListener(
    NumberTextFieldModelListener listener) {
    if (listener == null) {
      throw new IllegalArgumentException("O parmetro listener est nulo.");
    }
    return this.listeners.remove(listener);
  }

  /**
   * Dispara o evento de mudana de valor.
   */
  protected final void fireChangedValue() {
    for (NumberTextFieldModelListener listener : this.listeners) {
      listener.wasChangedValue();
    }
  }
}
