/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.client.algorithms.parameters;

import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JComponent;

import csbase.client.algorithms.parameters.validators.BooleanParameterValidator;
import csbase.logic.algorithms.parameters.BooleanParameter;

/**
 * Viso do {@link BooleanParameter Parmetro do Tipo Booleano}.
 * 
 * @author lmoreira
 */
public final class BooleanParameterView extends SimpleParameterView<Boolean> {

  /**
   * Cria a viso em modo {@link ParameterView.Mode#CONFIGURATION}.
   * 
   * @param window NO EST SENDO UTILIZADO. Existe somente para manter
   *        compatibilidade com o WebSintesi.
   * @param parameter O parmetro que  manipulado pela viso (No aceita
   *        {@code null}).
   * 
   * @deprecated para manter compatibilidade com o WebSintesi
   */
  @Deprecated
  public BooleanParameterView(Window window, BooleanParameter parameter) {
    this(parameter, Mode.CONFIGURATION);
  }

  /**
   * Cria a viso.
   * 
   * @param parameter O parmetro que  manipulado pela viso (No aceita
   *        {@code null}).
   * @param mode Modo de visualizao. No aceita {@code null}, os possveis
   *        valores so: {@link ParameterView.Mode#CONFIGURATION} ou
   *        {@link ParameterView.Mode#REPORT}.
   */
  public BooleanParameterView(BooleanParameter parameter, Mode mode) {
    super(parameter, mode);

    updateViewContents();
    updateCapabilityView();
    updateVisibilyView();
  }

  /**
   * @see csbase.client.algorithms.parameters.ParameterView#getParameter()
   */
  @Override
  public BooleanParameter getParameter() {
    return (BooleanParameter) super.getParameter();
  }

  @Override
  protected BooleanParameterValidator createParameterValidator() {
    return new BooleanParameterValidator();
  }

  @Override
  protected JComponent createConfigurationComponent(Object... componentArgs) {
    return new BooleanConfigurationParameter();
  }

  @Override
  protected JComponent createReportComponent(Object... componentArgs) {
    return new BooleanReportParameter();
  }

  @Override
  protected void updateViewContents() {
    ((IBooleanParameterComponent) getComponent()).updateViewContents();
  }

  private interface IBooleanParameterComponent {
    /**
     * Atualiza o contedo exibido pela viso.
     */
    void updateViewContents();
  }

  private final class BooleanReportParameter extends JCheckBox implements
    IBooleanParameterComponent {

    BooleanReportParameter() {
      setToolTipText(getParameter().getDescription());
      super.setEnabled(false);
      updateViewContents();
    }

    /**
     * Este mtodo no faz nada, pois este componente no pode ser habilitado.
     * 
     * @see javax.swing.AbstractButton#setEnabled(boolean)
     */
    @Override
    public void setEnabled(boolean enabled) {
    }

    /**
     * @see csbase.client.algorithms.parameters.IntegerParameterView.IIntegerParameterComponent#updateViewContents()
     */
    public void updateViewContents() {
      Boolean value = getParameter().getValue();
      setSelected(value.booleanValue());
    }
  }

  private final class BooleanConfigurationParameter extends JCheckBox implements
    IBooleanParameterComponent {

    BooleanConfigurationParameter() {
      setToolTipText(getParameter().getDescription());
      addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          getParameter().toggleValue();
        }
      });
    }

    /**
     * @see csbase.client.algorithms.parameters.IntegerParameterView.IIntegerParameterComponent#updateViewContents()
     */
    public void updateViewContents() {
      Boolean value = getParameter().getValue();
      setSelected(value.booleanValue());
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean fillVerticalSpace() {
    return false;
  }
}
