/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.client.algorithms.parameters;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.util.Arrays;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTextField;

import tecgraf.javautils.FileUtils;
import tecgraf.javautils.gui.GUIUtils;
import csbase.client.applicationmanager.ApplicationException;
import csbase.client.applicationmanager.ApplicationManager;
import csbase.client.applicationmanager.ApplicationType;
import csbase.client.applications.ApplicationImages;
import csbase.client.applications.ApplicationProject;
import csbase.client.desktop.DesktopComponentFrame;
import csbase.client.desktop.DesktopFrame;
import csbase.client.project.tasks.GetChildFromNameTask;
import csbase.client.project.tasks.GetFileTask;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.ClientProjectFile;
import csbase.logic.CommonClientProject;
import csbase.logic.ProjectFileType;
import csbase.logic.algorithms.FileParameterValue;
import csbase.logic.algorithms.parameters.FileParameter;
import csbase.logic.algorithms.parameters.FileParameterPipeAcceptance;

/**
 * A Viso para {@link FileParameter Parmetro do Tipo Arquivo}.
 * 
 * 
 * @author lmoreira
 */
public abstract class FileParameterView extends
  SimpleParameterView<FileParameterValue> {

  /**
   * Cria a viso em modo {@link ParameterView.Mode#CONFIGURATION}.
   * 
   * @param window NO EST SENDO UTILIZADO. Existe somente para manter
   *        compatibilidade com o WebSintesi.
   * @param parameter O parmetro (No aceita {@code null}).
   * 
   * @deprecated para manter compatibilidade com o WebSintesi
   */
  @Deprecated
  protected FileParameterView(Window window, FileParameter parameter) {
    this(parameter, Mode.CONFIGURATION);
  }

  /**
   * Cria a viso.
   * 
   * @param parameter O parmetro (No aceita {@code null}).
   * @param mode Modo de visualizao. No aceita {@code null}, os possveis
   *        valores so: {@link ParameterView.Mode#CONFIGURATION} ou
   *        {@link ParameterView.Mode#REPORT}.
   */
  protected FileParameterView(FileParameter parameter, Mode mode) {
    super(parameter, mode);

    updateViewContents();
    updateCapabilityView();
    updateVisibilyView();
  }

  /**
   * @see csbase.client.algorithms.parameters.ParameterView#getParameter()
   */
  @Override
  public FileParameter getParameter() {
    return (FileParameter) super.getParameter();
  }

  /**
   * Solicita um arquivo da usurio.
   * 
   * @return O arquivo ou {@code null} se o usurio cancelar ou houver um erro.
   */
  protected abstract FileParameterValue askForFile();

  @Override
  protected JComponent createConfigurationComponent(Object... componentArgs) {
    return new FileConfigurationParameter();
  }

  @Override
  protected JComponent createReportComponent(Object... componentArgs) {
    return new FileReportParameter();
  }

  @Override
  protected void setEnabled(boolean isEnabled) {
    getComponent().setEnabled(isEnabled);
  }

  @Override
  protected void updateVisibilyView() {
    boolean isVisible =
      getParameter().isVisible() && !getParameter().hasLink()
        && getParameter().usesPipe() != FileParameterPipeAcceptance.ALWAYS;
    if (isVisible() != isVisible) {
      setVisible(isVisible);
      fireVisibilityWasChanged();
    }
  }

  @Override
  protected void updateViewContents() {
    ((IFileParameterComponent) getComponent()).updateViewContents();
  }

  private interface IFileParameterComponent {
    /**
     * Atualiza o contedo exibido pela viso.
     */
    void updateViewContents();
  }

  private final class FileReportParameter extends JPanel implements
    IFileParameterComponent {

    /**
     * O campo de texto.
     */
    private JTextField textField;

    FileReportParameter() {
      setLayout(new GridBagLayout());

      textField = new JTextField();
      textField.setToolTipText(getParameter().getDescription());
      ComponentProperties.setProperties(textField, Mode.REPORT, true);
      textField.setEditable(false);

      GridBagConstraints constraints = new GridBagConstraints();
      constraints.anchor = GridBagConstraints.NORTHWEST;
      constraints.fill = GridBagConstraints.HORIZONTAL;
      constraints.gridheight = 1;
      constraints.gridwidth = 1;
      constraints.gridx = 1;
      constraints.gridy = 1;
      constraints.insets = new Insets(2, 2, 2, 2);
      constraints.weightx = 1.0;
      constraints.weighty = 0.0;
      add(textField, constraints);

      JButton button = createShowButton(textField);
      if (null != button) {
        constraints.fill = GridBagConstraints.NONE;
        constraints.gridx++;
        constraints.weightx = 0.0;
        add(button, constraints);
      }

      updateViewContents();
    }

    private JButton createShowButton(final JTextField textField) {
      final FileParameterValue value = getParameter().getValue();
      if (null != value && value.getType().equals("LOG")) {
        JButton browseButton =
          GUIUtils.createImageButton(ApplicationImages.ICON_VIEWLOG_16);
        browseButton.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent ev) {
            try {
              String applicationId =
                ApplicationManager.getInstance().getApplicationFromType(
                  value.getType());
              ApplicationType applicationType =
                ApplicationManager.getInstance().runApplication(applicationId);
              ClientProjectFile file = getClientProjectFile(value);
              applicationType.sendMessage(
                ApplicationProject.PROJECT_FILE_MESSAGE, file);
            }
            catch (ApplicationException e) {
              final DesktopFrame desktop = DesktopFrame.getInstance();
              final DesktopComponentFrame frame = desktop.getDesktopFrame();
              StandardErrorDialogs.showErrorDialog(frame, e);
            }
          }
        });
        browseButton.setToolTipText(getParameter().getDescription());
        return browseButton;
      }
      else {
        return null;
      }
    }

    private ClientProjectFile getClientProjectFile(final FileParameterValue log) {

      CommonClientProject project = DesktopFrame.getInstance().getProject();
      String[] path = log.getPathAsArray();

      /*
       * Extrai o caminho relativo do log a partir do caminho do projeto.
       * Devemos procurar do final para o incio do path pelo projeto pois o
       * path pode ser absoluto e o projeto ter o mesmo nome de algum diretrio
       * do sistema.
       */

      // Testa se o ltimo elemento do path no  o projeto.
      if (path[path.length - 1].equals(project.getName())) {
        path = new String[0];
      }
      else {
        // Caso o projeto no seja o ltimo elemento, procura nos demais.
        for (int inx = path.length - 2; inx >= 0; inx--) {
          if (path[inx].equals(project.getName())) {
            path = Arrays.copyOfRange(path, inx + 1, path.length);
            break;
          }
        }
      }

      final ClientProjectFile file = GetFileTask.runTask(project, path);
      return file;
    }

    /**
     * @see csbase.client.algorithms.parameters.FileParameterView.IFileParameterComponent#updateViewContents()
     */
    public void updateViewContents() {
      FileParameterValue value = getParameter().getValue();
      textField.setText(null == value ? " " : value.getPath());
    }
  }

  private final class FileConfigurationParameter extends JPanel implements
    IFileParameterComponent {

    /**
     * O campo de texto.
     */
    private JTextField textField;

    /**
     * O boto Navegar.
     */
    private JButton browseButton;

    FileConfigurationParameter() {
      setLayout(new GridBagLayout());

      browseButton =
        GUIUtils.createImageButton(ApplicationImages.ICON_BROWSEFILE_16);
      browseButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent ev) {
          FileParameterValue file = askForFile();
          if (file != null) {
            getParameter().setValue(file);
          }
        }
      });
      browseButton.setToolTipText(getParameter().getDescription());
      textField = new JTextField();
      textField.addFocusListener(new FocusListener() {
        public void focusGained(FocusEvent e) {
        }

        public void focusLost(FocusEvent e) {
          updateModel();
        }
      });
      textField.setToolTipText(getParameter().getDescription());

      GridBagConstraints constraints = new GridBagConstraints();
      constraints.anchor = GridBagConstraints.NORTHWEST;
      constraints.fill = GridBagConstraints.HORIZONTAL;
      constraints.gridheight = 1;
      constraints.gridwidth = 1;
      constraints.gridx = 1;
      constraints.gridy = 1;
      constraints.insets = new Insets(2, 2, 2, 2);
      constraints.weightx = 1.0;
      constraints.weighty = 0.0;
      add(textField, constraints);

      constraints.fill = GridBagConstraints.NONE;
      constraints.gridx++;
      constraints.weightx = 0.0;
      add(browseButton, constraints);
    }

    /**
     * @see csbase.client.algorithms.parameters.FileParameterView.IFileParameterComponent#updateViewContents()
     */
    public void updateViewContents() {
      FileParameterValue file = getParameter().getValue();
      if (file == null) {
        textField.setText("");
      }
      else {
        textField.setText(file.toString());
      }
    }

    /**
     * Habilita/Desabilita a viso.
     * 
     * @param isEnabled .
     */
    @Override
    public void setEnabled(boolean isEnabled) {
      super.setEnabled(isEnabled);
      textField.setEnabled(isEnabled);
      browseButton.setEnabled(isEnabled);
    }

    /**
     * Atualiza o contedo do modelo (L as informaes da viso e preenche o
     * parmetro).
     */
    private void updateModel() {
      String text = textField.getText();
      if (text.length() == 0) {
        getParameter().setValue(null);
      }
      else {
        if (text.equals("/")) {
          text = ".";
        }
        else if (text.startsWith("/")) {
          text = text.substring(1);
        }
        String[] path = FileUtils.splitPath(text, "/");
        CommonClientProject project = DesktopFrame.getInstance().getProject();
        ClientProjectFile rootProjectFile =
          (ClientProjectFile) project.getTree();
        ClientProjectFile clientProjectFile = rootProjectFile;
        String type = getParameter().getFileType();
        for (String component : path) {
          if (clientProjectFile == null) {
            break;
          }
          clientProjectFile =
            GetChildFromNameTask.runTask(clientProjectFile, component);
        }
        if (clientProjectFile != null) {
          if (clientProjectFile.isDirectory()) {
            type = ProjectFileType.DIRECTORY_TYPE;
          }
          else {
            type = clientProjectFile.getType();
          }
        }
        FileParameterValue file = new FileParameterValue(text, type);
        getParameter().setValue(file);
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean fillVerticalSpace() {
    return false;
  }
}
