package csbase.client.algorithms.parameters;

import java.awt.Window;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JComponent;
import javax.swing.JTextField;

import csbase.client.algorithms.parameters.validators.FormulaParameterValidator;
import csbase.logic.algorithms.parameters.FormulaParameter;

/**
 * A viso para o parmetro do tipo Frmula.
 * 
 * @author lmoreira
 */
public final class FormulaParameterView extends SimpleParameterView<String> {

  /**
   * Cria a viso em modo {@link ParameterView.Mode#CONFIGURATION}.
   * 
   * @param window NO EST SENDO UTILIZADO. Existe somente para manter
   *        compatibilidade com o WebSintesi.
   * @param formulaParameter O parmetro (No aceita {@code null}).
   * 
   * @deprecated para manter compatibilidade com o WebSintesi
   */
  @Deprecated
  public FormulaParameterView(Window window, FormulaParameter formulaParameter) {
    this(formulaParameter, Mode.CONFIGURATION);
  }

  /**
   * Cria a viso.
   * 
   * @param formulaParameter O parmetro (No aceita {@code null}).
   * @param mode Modo de visualizao. No aceita {@code null}, os possveis
   *        valores so: {@link ParameterView.Mode#CONFIGURATION} ou
   *        {@link ParameterView.Mode#REPORT}.
   */
  public FormulaParameterView(FormulaParameter formulaParameter, Mode mode) {
    super(formulaParameter, mode);

    updateCapabilityView();
    updateVisibilyView();
  }

  /**
   * @see csbase.client.algorithms.parameters.ParameterView#getParameter()
   */
  @Override
  public FormulaParameter getParameter() {
    return (FormulaParameter) super.getParameter();
  }

  @Override
  protected FormulaParameterValidator createParameterValidator() {
    return new FormulaParameterValidator(getParameter().isOptional());
  }

  @Override
  protected JComponent createConfigurationComponent(Object... componentArgs) {
    return new FormulaConfigurationParameter();
  }

  @Override
  protected JComponent createReportComponent(Object... componentArgs) {
    return new FormulaReportParameter();
  }

  @Override
  protected void updateViewContents() {
    ((IFormulaParameterComponent) getComponent()).updateViewContents();
  }

  private interface IFormulaParameterComponent {
    /**
     * Atualiza o contedo exibido pela viso.
     */
    void updateViewContents();
  }

  private final class FormulaReportParameter extends JTextField implements
    IFormulaParameterComponent {

    FormulaReportParameter() {
      setToolTipText(getParameter().getDescription());
      ComponentProperties.setProperties(this, Mode.REPORT, true);
      super.setEditable(false);
      updateViewContents();
    }

    /**
     * @see csbase.client.algorithms.parameters.TextParameterView.ITextParameterComponent#updateViewContents()
     */
    public void updateViewContents() {
      String value = getParameter().getValue();
      setText(null == value ? " " : value);
    }

    /**
     * Este mtodo no faz nada por que este componente no pode ser editvel.
     * 
     * @see javax.swing.text.JTextComponent#setEditable(boolean)
     */
    @Override
    public void setEditable(boolean enable) {
    }
  }

  private final class FormulaConfigurationParameter extends JTextField
    implements IFormulaParameterComponent {

    FormulaConfigurationParameter() {
      addFocusListener(new FocusListener() {
        public void focusGained(FocusEvent e) {
        }

        public void focusLost(FocusEvent e) {
          updateModel();
        }
      });

      setToolTipText(getParameter().getDescription());
      updateViewContents();
    }

    /**
     * @see csbase.client.algorithms.parameters.TextParameterView.ITextParameterComponent#updateViewContents()
     */
    public void updateViewContents() {
      String value = getParameter().getValue();
      setText(null == value ? "" : value);
    }

    /**
     * Atualiza o modelo (L informaes da viso e atrubui a informao ao
     * parmetro).
     */
    private void updateModel() {
      String text = getText();
      if (text.length() == 0) {
        getParameter().setValue(null);
      }
      else {
        getParameter().setValue(text);
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean fillVerticalSpace() {
    return false;
  }
}
