/**
 * $Id: InputFileListParameterView.java 97327 2009-10-09 01:14:44Z costa $
 */

package csbase.client.algorithms.parameters;

import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.text.MessageFormat;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import tecgraf.javautils.FileUtils;
import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.algorithms.parameters.validators.InputFileListParameterValidator;
import csbase.client.applications.ApplicationImages;
import csbase.client.desktop.DesktopFrame;
import csbase.client.kernel.ClientException;
import csbase.client.project.ProjectFileChooser;
import csbase.client.project.ProjectFileChooserOpen;
import csbase.client.project.ProjectTreePath;
import csbase.client.util.ClientUtilities;
import csbase.logic.ClientProjectFile;
import csbase.logic.ProjectFileType;
import csbase.logic.algorithms.FileParameterValue;
import csbase.logic.algorithms.parameters.FileParameter;
import csbase.logic.algorithms.parameters.FileParameterMode;
import csbase.logic.algorithms.parameters.InputFileListParameter;

/**
 * Viso para {@link FileParameter Parmetro do Tipo Arquivo} com Seleo
 * Mltipla.
 * 
 * @author lmoreira
 */
public final class InputFileListParameterView extends
  SimpleParameterView<List<FileParameterValue>> {

  /**
   * Prefixo para internacionalizao de idioma.
   */
  private static final String LNGPREFIX =
    InputFileListParameterView.class.getName() + ".";

  /**
   * Cria uma viso em modo {@link ParameterView.Mode#CONFIGURATION}.
   * 
   * @param window NO EST SENDO UTILIZADO. Existe somente para manter
   *        compatibilidade com o WebSintesi.
   * @param parameter O parmetro (No aceita {@code null}).
   * 
   * @deprecated para manter compatibilidade com o WebSintesi
   */
  @Deprecated
  public InputFileListParameterView(Window window,
    InputFileListParameter parameter) {
    this(parameter, Mode.CONFIGURATION);
  }

  /**
   * Cria uma viso.
   * 
   * @param parameter O parmetro (No aceita {@code null}).
   * @param mode Modo de visualizao. No aceita {@code null}, os possveis
   *        valores so: {@link ParameterView.Mode#CONFIGURATION} ou
   *        {@link ParameterView.Mode#REPORT}.
   */
  public InputFileListParameterView(InputFileListParameter parameter, Mode mode) {
    super(parameter, mode);

    updateCapabilityView();
    updateVisibilyView();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public InputFileListParameter getParameter() {
    return (InputFileListParameter) super.getParameter();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected InputFileListParameterValidator createParameterValidator() {
    return new InputFileListParameterValidator(getParameter().isOptional());
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected JComponent createConfigurationComponent(Object... componentArgs) {
    return new InputFileListConfigurationParameter();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected JComponent createReportComponent(Object... componentArgs) {
    return new InputFileListReportParameter();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void updateViewContents() {
    ((IInputFileListParameterComponent) getComponent()).updateViewContents();
  }

  /**
   * Interface do componente
   * 
   * @author Tecgraf/PUC-Rio
   */
  private interface IInputFileListParameterComponent {
    /**
     * Atualiza o contedo exibido pela viso.
     */
    void updateViewContents();
  }

  /**
   * Classe de implementao do modo report.
   * 
   * @author Tecgraf/PUC-Rio
   */
  private final class InputFileListReportParameter extends JPanel implements
    IInputFileListParameterComponent {

    /**
     * rea de texto de exibio
     */
    private JTextArea textArea;

    /**
     * Construtor
     */
    InputFileListReportParameter() {

      setLayout(new GridLayout());

      textArea = new JTextArea();
      textArea.setToolTipText(getParameter().getDescription());
      ComponentProperties.setProperties(textArea, Mode.REPORT, false);

      JScrollPane scroll = new JScrollPane(textArea);
      scroll
        .setBorder(ComponentProperties.getInstance(Mode.REPORT).getBorder());
      updateViewContents();
      add(scroll, new GBC(0, 0).both());
    }

    /**
     * @see csbase.client.algorithms.parameters.InputFileListParameterView.IInputFileListParameterComponent#updateViewContents()
     */
    public void updateViewContents() {
      List<FileParameterValue> values = getParameter().getValue();
      if (values == null) {
        textArea.setRows(1);
      }
      else {
        textArea.setRows(Math.max(1, Math.min(values.size(), 6)));

        StringBuffer sb = new StringBuffer();
        Iterator<FileParameterValue> valueIterator = values.iterator();
        while (valueIterator.hasNext()) {
          FileParameterValue value = valueIterator.next();
          sb.append(value.getPath()).append('\n');
        }
        // remove o ltimo breakLine
        String text = sb.substring(0, sb.length() - 1);
        textArea.setText(text);
      }
    }

    /**
     * @see javax.swing.JComponent#setEnabled(boolean)
     */
    @Override
    public void setEnabled(boolean enabled) {
      super.setEnabled(enabled);
      textArea.setEnabled(enabled);
    }
  }

  /**
   * Classe do parmetro
   * 
   * @author Tecgraf/PUC-Rio
   */
  private final class InputFileListConfigurationParameter extends JPanel
    implements IInputFileListParameterComponent {

    /**
     * O Boto Adicionar.
     */
    final private JButton addButton = new JButton();

    /**
     * Indica se a viso est habilitada.
     */
    private boolean isEnabledView;

    /**
     * A lista de arquivos.
     */
    private JList list;

    /**
     * O modelo da lista.
     */
    private DefaultListModel model;

    /**
     * O Boto Remover.
     */
    final private JButton removeButton = new JButton();

    /**
     * O Boto up.
     */
    final private JButton upButton = new JButton();

    /**
     * O Boto down.
     */
    final private JButton dnButton = new JButton();

    /**
     * Construtor
     */
    InputFileListConfigurationParameter() {
      setLayout(new BorderLayout());

      model = new DefaultListModel();
      list = new JList(model);
      list.getSelectionModel().setSelectionMode(
        ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
      list.setToolTipText(getParameter().getDescription());
      list.addKeyListener(new KeyAdapter() {
        @Override
        public void keyPressed(KeyEvent e) {
          int keyCode = e.getKeyCode();
          switch (keyCode) {
            case KeyEvent.VK_BACK_SPACE:
            case KeyEvent.VK_DELETE:
              removeFiles();
          }
        }
      });
      list.addListSelectionListener(new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent e) {
          updateRemoveButton();
          updateUpDownButton();
        }
      });
      list.setToolTipText(getParameter().getDescription());
      JPanel editPanel = new JPanel();
      editPanel.setLayout(new BorderLayout());

      add(new JScrollPane(list), BorderLayout.CENTER);
      add(createButtonPanel(), BorderLayout.EAST);
    }

    /**
     * @see csbase.client.algorithms.parameters.InputFileListParameterView.IInputFileListParameterComponent#updateViewContents()
     */
    public void updateViewContents() {
      model.removeAllElements();
      List<FileParameterValue> values = getParameter().getValue();
      if (values != null) {
        Iterator<FileParameterValue> valueIterator = values.iterator();
        while (valueIterator.hasNext()) {
          FileParameterValue value = valueIterator.next();
          model.addElement(value);
        }
      }
    }

    /**
     * @see javax.swing.JComponent#setEnabled(boolean)
     */
    @Override
    public void setEnabled(boolean isEnabled) {
      isEnabledView = isEnabled;
      list.setEnabled(isEnabled);
      addButton.setEnabled(isEnabled);
      updateRemoveButton();
      updateUpDownButton();
    }

    /**
     * Solicita os arquivos ao usurio.
     * 
     * @return O conjunto de arquivos selecionados (Se o usurio cancelar a
     *         seleo, o conjunto estar vazio; se houver um erro durante a
     *         seleo, retorna {@code null}).
     */
    private Set<FileParameterValue> askForFile() {
      try {
        Set<FileParameterValue> files = new HashSet<FileParameterValue>();
        ProjectFileChooser chooser =
          new ProjectFileChooserOpen(getWindow(), DesktopFrame.getInstance()
            .getProject(), true, getFileChooserMode(), getParameter()
            .getFileType());
        ProjectTreePath[] projectTreePaths = chooser.getSelectedPaths();
        if ((projectTreePaths == null) || (projectTreePaths.length == 0)) {
          return files;
        }
        for (ProjectTreePath projectTreePath : projectTreePaths) {
          ClientProjectFile clientProjectFile = projectTreePath.getFile();
          String type;
          if (clientProjectFile.isDirectory()) {
            type = ProjectFileType.DIRECTORY_TYPE;
          }
          else {
            type = clientProjectFile.getType();
          }
          FileParameterValue file =
            new FileParameterValue(FileUtils.joinPath('/', projectTreePath
              .getSimplePath()), type);
          files.add(file);
        }
        return files;
      }
      catch (ClientException e) {
        String errorMessage =
          "Ocorreu um erro ao abrir o dilogo de seleo de arquivos.";
        if (e.getLocalizedMessage() != null) {
          errorMessage += "\nDetalhes:\n";
          errorMessage += e.getLocalizedMessage();
        }
        StandardDialogs.showErrorDialog(getWindow(),
          "Erro ao abrir o dilogo de seleo de arquivo", errorMessage);
        return null;
      }
    }

    /**
     * Cria o painel com os botes Adicionar e Remover.
     * 
     * @return .
     */
    private JPanel createButtonPanel() {
      final InputFileListParameter parameter = getParameter();

      addButton.setIcon(ApplicationImages.ICON_BROWSEFILE_16);
      addButton.setText(LNG.get(LNGPREFIX + "add.button.text"));
      addButton.setToolTipText(LNG.get(LNGPREFIX + "add.button.tooltip"));
      addButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          Set<FileParameterValue> files = askForFile();
          if (files != null) {
            for (FileParameterValue file : files) {
              parameter.addFile(file);
            }
          }
        }
      });

      removeButton.setIcon(ApplicationImages.ICON_DELETE_16);
      removeButton.setText(LNG.get(LNGPREFIX + "del.button.text"));
      removeButton.setToolTipText(LNG.get(LNGPREFIX + "del.button.tooltip"));
      removeButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          removeFiles();
        }
      });
      removeButton.setEnabled(false);

      upButton.setIcon(ApplicationImages.ICON_UP_16);
      upButton.setText(LNG.get(LNGPREFIX + "up.button.text"));
      upButton.setToolTipText(LNG.get(LNGPREFIX + "up.button.tooltip"));
      upButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          moveUpFile();
        }
      });
      upButton.setEnabled(false);

      dnButton.setIcon(ApplicationImages.ICON_DOWN_16);
      dnButton.setText(LNG.get(LNGPREFIX + "down.button.text"));
      dnButton.setToolTipText(LNG.get(LNGPREFIX + "down.button.tooltip"));
      dnButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          moveDownFile();
        }
      });
      dnButton.setEnabled(false);

      ClientUtilities.adjustEqualSizes(addButton, removeButton, upButton,
        dnButton);

      final JPanel buttonPanel = new JPanel();
      buttonPanel.setLayout(new GridBagLayout());
      buttonPanel.add(addButton, new GBC(0, 0).center().none().insets(2));
      buttonPanel.add(removeButton, new GBC(0, 1).center().none().insets(2));
      if (!parameter.mustSort()) {
        buttonPanel.add(new JLabel(), new GBC(0, 2).center().vertical().insets(
          2));
        buttonPanel.add(upButton, new GBC(0, 3).center().none().insets(2));
        buttonPanel.add(dnButton, new GBC(0, 4).center().none().insets(2));
      }
      return buttonPanel;
    }

    /**
     * Obtm o modo do {@link ProjectFileChooser} que ser utilizado para
     * solicitar os arquivos ao usurio.
     * 
     * @return .
     */
    private int getFileChooserMode() {
      int mode;
      FileParameterMode parameterMode = getParameter().getMode();
      if (parameterMode.equals(FileParameterMode.REGULAR_FILE)) {
        mode = ProjectFileChooser.FILE_ONLY;
      }
      else if (parameterMode.equals(FileParameterMode.DIRECTORY)) {
        mode = ProjectFileChooser.DIRECTORY_ONLY;
      }
      else if (parameterMode
        .equals(FileParameterMode.DIRECTORY_AND_REGULAR_FILE)) {
        mode = ProjectFileChooser.FILE_AND_DIRECTORY;
      }
      else {
        String errorMessage =
          MessageFormat.format("O modo {0} no  vlido.", parameterMode);
        throw new IllegalStateException(errorMessage);
      }
      return mode;
    }

    /**
     * Remove os arquivos selecionados.
     */
    private void removeFiles() {
      int[] indexes = list.getSelectedIndices();
      FileParameterValue[] files = new FileParameterValue[indexes.length];
      for (int i = 0; i < indexes.length; i++) {
        final ListModel listModel = list.getModel();
        final int index = indexes[i];
        files[i] = (FileParameterValue) listModel.getElementAt(index);
      }

      final InputFileListParameter parameter = getParameter();
      for (FileParameterValue file : files) {
        parameter.removeFile(file);
      }
    }

    /**
     * Remove os arquivos selecionados.
     */
    private void moveUpFile() {
      final int[] indexes = list.getSelectedIndices();
      final int n = indexes.length;
      if (n != 1) {
        return;
      }
      final int index = indexes[0];
      final InputFileListParameter parameter = getParameter();
      parameter.moveUp(index);
      list.setSelectedIndex(index - 1);
    }

    /**
     * Remove os arquivos selecionados.
     */
    private void moveDownFile() {
      final int[] indexes = list.getSelectedIndices();
      final int n = indexes.length;
      if (n != 1) {
        return;
      }
      final int index = indexes[0];
      final InputFileListParameter parameter = getParameter();
      parameter.moveDown(index);
      list.setSelectedIndex(index + 1);
    }

    /**
     * Atualiza o boto Remover.
     */
    private void updateRemoveButton() {
      removeButton.setEnabled(isEnabledView && !list.isSelectionEmpty());
    }

    /**
     * Atualiza os botes Up/Down.
     */
    private void updateUpDownButton() {
      final int[] selection = list.getSelectedIndices();
      final boolean flag =
        isEnabledView && !list.isSelectionEmpty() && (selection.length == 1);
      upButton.setEnabled(flag);
      dnButton.setEnabled(flag);
    }

  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean fillVerticalSpace() {
    return true;
  }
}
