/*
 * $Id: InputFileParameterView.java 97328 2009-10-09 01:19:24Z costa $
 */

package csbase.client.algorithms.parameters;

import java.awt.Window;
import java.text.MessageFormat;

import tecgraf.javautils.FileUtils;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.algorithms.parameters.validators.InputFileParameterValidator;
import csbase.client.desktop.DesktopFrame;
import csbase.client.kernel.ClientException;
import csbase.client.project.ProjectFileChooser;
import csbase.client.project.ProjectFileChooserOpen;
import csbase.client.project.ProjectTreePath;
import csbase.logic.ClientProjectFile;
import csbase.logic.ProjectFileType;
import csbase.logic.algorithms.FileParameterValue;
import csbase.logic.algorithms.parameters.FileParameter;
import csbase.logic.algorithms.parameters.FileParameterMode;
import csbase.logic.algorithms.parameters.InputFileParameter;

/**
 * A Viso para {@link InputFileParameter Parmetro do Tipo Arquivo de Entrada}.
 * 
 * @author Tecgraf/PUC-Rio
 */
public final class InputFileParameterView extends FileParameterView {

  /**
   * Cria uma viso em modo {@link ParameterView.Mode#CONFIGURATION}.
   * 
   * @param window NO EST SENDO UTILIZADO. Existe somente para manter
   *        compatibilidade com o WebSintesi.
   * @param parameter O parmetro (No aceita {@code null}).
   * 
   * @deprecated para manter compatibilidade com o WebSintesi
   */
  @Deprecated
  public InputFileParameterView(Window window, InputFileParameter parameter) {
    this(parameter, Mode.CONFIGURATION);
  }

  /**
   * Cria a viso.
   * 
   * @param parameter O parmetro (No aceita {@code null}).
   * @param mode Modo de visualizao. No aceita {@code null}, os possveis
   *        valores so: {@link ParameterView.Mode#CONFIGURATION} ou
   *        {@link ParameterView.Mode#REPORT}.
   */
  public InputFileParameterView(InputFileParameter parameter, Mode mode) {
    super(parameter, mode);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public InputFileParameter getParameter() {
    return (InputFileParameter) super.getParameter();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected final FileParameterValue askForFile() {
    FileParameter parameter = getParameter();
    try {
      ProjectFileChooser chooser =
        new ProjectFileChooserOpen(getWindow(), DesktopFrame.getInstance()
          .getProject(), false, getFileChooserMode(), parameter.getFileType());
      ProjectTreePath projectTreePath = chooser.getSelectedPath();
      if (projectTreePath == null) {
        return null;
      }
      ClientProjectFile clientProjectFile = projectTreePath.getFile();
      String type;
      if (clientProjectFile.isDirectory()) {
        type = ProjectFileType.DIRECTORY_TYPE;
      }
      else {
        type = clientProjectFile.getType();
      }
      return new FileParameterValue(FileUtils.joinPath('/', projectTreePath
        .getSimplePath()), type);
    }
    catch (ClientException e) {
      String errorMessage =
        "Ocorreu um erro ao abrir o dilogo de seleo de arquivos.";
      if (e.getLocalizedMessage() != null) {
        errorMessage += "\nDetalhes:\n";
        errorMessage += e.getLocalizedMessage();
      }
      StandardDialogs.showErrorDialog(getWindow(),
        "Erro ao abrir o dilogo de seleo de arquivo", errorMessage);
      return null;
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected InputFileParameterValidator createParameterValidator() {
    return new InputFileParameterValidator(getParameter().isOptional());
  }

  /**
   * Obtm o modo do {@link ProjectFileChooser} que ser utilizado para
   * solicitar os arquivos ao usurio.
   * 
   * @return .
   */
  private int getFileChooserMode() {
    int mode;
    FileParameterMode parameterMode = getParameter().getMode();
    if (parameterMode.equals(FileParameterMode.REGULAR_FILE)) {
      mode = ProjectFileChooser.FILE_ONLY;
    }
    else if (parameterMode.equals(FileParameterMode.DIRECTORY)) {
      mode = ProjectFileChooser.DIRECTORY_ONLY;
    }
    else if (parameterMode.equals(FileParameterMode.DIRECTORY_AND_REGULAR_FILE)) {
      mode = ProjectFileChooser.FILE_AND_DIRECTORY;
    }
    else {
      String errorMessage =
        MessageFormat.format("O modo {0} no  vlido.", parameterMode);
      throw new IllegalStateException(errorMessage);
    }
    return mode;
  }
}
