/*
 * $Author:$
 * $Date:$
 * $Release:$
 */
package csbase.client.algorithms.parameters;

import java.awt.Window;

import csbase.client.algorithms.parameters.validators.IntegerParameterValidator;
import csbase.client.algorithms.parameters.validators.ListParameterValidator;
import csbase.logic.algorithms.parameters.IntegerListParameter;
import csbase.logic.algorithms.parameters.IntegerListParameterListener;

/**
 * Viso para o {@link IntegerListParameter Parmetro do Tipo Lista de Inteiros}.
 *
 * @author lmoreira
 */
public final class IntegerListParameterView
extends ListParameterView<Integer> {
  
  /**
   * Cria uma viso em modo {@link ParameterView.Mode#CONFIGURATION}.
   * 
   * @param window NO EST SENDO UTILIZADO. Existe somente para manter
   *        compatibilidade com o WebSintesi.
   * @param parameter O parmetro (No aceita {@code null}).
   * 
   * @deprecated para manter compatibilidade com o WebSintesi       
   */
  @Deprecated
  public IntegerListParameterView(Window window, IntegerListParameter parameter) {
    this(parameter, Mode.CONFIGURATION);
  }  
  
  /**
   * Cria a viso.
   * 
   * @param parameter O parmetro que  manipulado por esta viso 
   *        (No aceita {@code null}).
   * @param mode Modo de visualizao. No aceita {@code null}, os possveis
   *        valores so: {@link ParameterView.Mode#CONFIGURATION} ou
   *        {@link ParameterView.Mode#REPORT}.
   */
  public IntegerListParameterView(IntegerListParameter parameter, Mode mode) {
    super(
      parameter, mode,
      new Parser<Integer>() {
        public Integer parse(String text) {
          try {
            return Integer.parseInt(text);
          }
          catch (NumberFormatException e) {
            return null;
          }
        }
      },
      new Formatter<Integer>() {
        public String format(Integer element) {
          return element.toString();
        }
      }, new IntegerTextField());
    parameter.addIntegerListParameterListener(new IntegerListParameterListener() {

      public void minimumWasChanged(IntegerListParameter parameter) {
        ListParameterValidator<?> listParameterValidator = (ListParameterValidator) getParameterValidator();
        IntegerParameterValidator validator = (IntegerParameterValidator) listParameterValidator
          .getItemValidator();
        validator.setMinimum(parameter.getMinimum());
      }

      public void maximumWasChanged(IntegerListParameter parameter) {
        ListParameterValidator<?> listParameterValidator = (ListParameterValidator) getParameterValidator();
        IntegerParameterValidator validator = (IntegerParameterValidator) listParameterValidator
          .getItemValidator();
        validator.setMaximum(parameter.getMaximum());
      }
    });
  }

  /**
   * @see csbase.client.algorithms.parameters.ListParameterView#getParameter()
   */
  @Override
  public IntegerListParameter getParameter() {
    return (IntegerListParameter)super.getParameter();
  }

  @Override
  protected IntegerParameterValidator createItemValidator() {
    return new IntegerParameterValidator(
      getParameter().isOptional(), getParameter().getMaximum(), getParameter()
        .getMinimum());
  }
}
