package csbase.client.algorithms.parameters.validators;

import tecgraf.javautils.LNG;

/**
 * Resultado da Validao de Parmetros.
 * 
 * @author lmoreira
 */
public class ValidationResult {
  /**
   * Mensagem de erro (ou {@code null} em caso de sucesso).
   */
  private String errorMessage;

  /**
   * Cria o validador em caso de sucesso.
   */
  public ValidationResult() {
  }

  /**
   * Cria o validador em caso de erro.
   * 
   * @param errorMessage A mensagem de erro (No aceita {@code null}).
   */
  public ValidationResult(String errorMessage) {
    setErrorMessage(errorMessage);
  }

  /**
   * Obtm a mensagem de erro.
   * 
   * @return A mensagem de erro ou {@code null} se no houve erro.
   */
  public String getErrorMessage() {
    return this.errorMessage;
  }

  /**
   * Indica se houve sucesso ou falha.
   * 
   * @return {@code true} sucesso ou {@code false} falha.
   */
  public boolean isWellSucceded() {
    return (getErrorMessage() == null);
  }

  /**
   * Atribui a mensagem de erro.
   * 
   * @param errorMessage A mensagem de erro (No aceita {@code null}).
   */
  private void setErrorMessage(String errorMessage) {
    if (errorMessage == null) {
      throw new IllegalArgumentException("O parmetro errorMessage est nulo.");
    }
    this.errorMessage = errorMessage;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String toString() {
    if (errorMessage != null) {
      return getErrorMessage();
    }
    else {
      return LNG.get(ValidationResult.class.getName() + ".succeded");
    }
  }

}
