/*
 * $Id: XWindowsStub.java 76954 2008-06-06 19:24:00Z taciana $
 */
package csbase.client.applicationmanager;

import java.net.InetAddress;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.Client;
import csbase.client.desktop.DesktopFrame;
import csbase.client.remote.srvproxies.EventLogProxy;
import csbase.client.remote.srvproxies.XAppProxy;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.CommonClientProject;

/**
 * Stub para execuo de aplicao X
 * 
 * @author Tecgraf/PUC-Rio
 */
public class XWindowsStub extends ApplicationType {

  /**
   * {@inheritDoc}
   */
  @Override
  public void killApplication() {
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void startApplication() {
    final ApplicationManager manager = ApplicationManager.getInstance();
    final String[] aQueue = manager.getApplicationManagerEventQueue();
    final String[] initMsg = {
        LNG.get("application.manager.started.x.program"), getName(),
        "HASHCODE:" + this.hashCode(), getId() };
    EventLogProxy.addClientInformation(aQueue, initMsg);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void finishApplication() {
    final ApplicationManager manager = ApplicationManager.getInstance();
    final String[] aQueue = manager.getApplicationManagerEventQueue();
    final String[] initMsg = {
        LNG.get("application.manager.finished.x.program"), getName(),
        "HASHCODE:" + this.hashCode(), getId() };
    EventLogProxy.addClientInformation(aQueue, initMsg);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean closeApplication() {
    return true;
  }

  /**
   * Busca de nome do SGA de execuo.
   * 
   * @return o nome
   */
  protected String getSgaName() {
    return null;
  }

  /**
   * Construtor
   * 
   * @param id id da aplicao
   */
  public XWindowsStub(String id) {
    super(id);
    try {
      final DesktopFrame desktop = DesktopFrame.getInstance();
      if (desktop == null) {
        return;
      }
      final CommonClientProject project = desktop.getProject();
      if (project == null) {
        return;
      }
      final String xMsg = LNG.get("application.manager.x.server.confirm");
      final String appName = getName();
      final int answer = StandardDialogs.showYesNoDialog(null, appName, xMsg);
      if (answer == 1) {
        final String error = LNG.get("application.manager.x.error");
        StandardErrorDialogs.showErrorDialog(null, appName, error);
        return;
      }

      final String prjId = project.getId();
      final Object appId = id;
      String dspName = null;
      dspName = Client.getInstance().getParameter("client_address");
      if (dspName == null) {
        dspName = InetAddress.getLocalHost().getHostName();
      }
      dspName += ":0.0";
      XAppProxy.runApplication(prjId, appId, dspName, this
        .getSgaName());
    }
    catch (Exception e) {
      final String appName = getName();
      StandardErrorDialogs.showErrorDialog(null, appName, e.getMessage());
    }
  }
}
