/*
 * $Id: ApplicationComponentDialog.java 67747 2007-09-19 17:35:12Z valeria $
 */

package csbase.client.applications;

import java.awt.*;

import javax.swing.*;

/**
 * Modela um dialogo criado por uma aplicao.
 */
public class ApplicationComponentDialog extends JDialog {

  /**
   * A aplicao que chamou o Dialog
   */
  private final Application application;

  /**
   * Redefine o mtodo setVisible() do frame.  chamada tambm no caso
   * <code>dispose()</code>
   * 
   * @param visible indica se o dilogo deve ou no ser exibido.
   */
  @Override
  public void setVisible(boolean visible) {
    super.setVisible(visible);
    if (visible) {
      application.addWindow(this);
    }
    else {
      application.removeWindow(this);
    }
  }

  /**
   * Centraliza uma janela em relao a outra.
   * 
   * @param window a janela.
   */
  public void center(Window window) {
    Dimension wSize = this.getSize();
    Dimension sSize = window.getSize();
    int newX = (sSize.width - wSize.width) / 2;
    int newY = (sSize.height - wSize.height) / 2;
    this.setLocation(window.getX() + newX, window.getY() + newY);
  }

  /**
   * Retorna a aplicao que chamou o dilogo.
   * 
   * @return a aplicao.
   */
  public Application getApplication() {
    return application;
  }

  /**
   * Construtor que recebe uma Frame como referncia
   * 
   * @param owner .
   * @param title .
   * @param application .
   */
  public ApplicationComponentDialog(JFrame owner, String title,
    Application application) {
    super(owner, title, true);
    this.application = application;
  }

  /**
   * Construtor que recebe um Dialog como referncia
   * 
   * @param owner .
   * @param title .
   * @param application .
   */
  public ApplicationComponentDialog(JDialog owner, String title,
    Application application) {
    super(owner, title, true);
    this.application = application;
  }
}
