/*
 * $Id: ApplicationTask.java 90314 2009-04-09 14:51:45Z clinio $
 */

package csbase.client.applications;

import javax.swing.ImageIcon;
import csbase.client.desktop.Task;
import csbase.client.desktop.LocalTask;

/**
 * Tarefa de execuo de uma aplicao
 * 
 * @author Tecgraf/PUC-Rio
 * @param <R> tipo de retorno da task
 */
abstract public class ApplicationTask<R> extends LocalTask<R> {

  /**
   * Aplicao.
   */
  final Application application;

  /**
   * Mensagem
   */
  final String message;

  /**
   * Imagem associada
   */
  final ImageIcon image;

  /**
   * Tratador especfico da aplicao (usado em todos os momentos).
   * 
   * @param exception a excepo detectada.
   */
  @Override
  protected void handleError(final Exception exception) {
    if (!application.handleError(exception)) {
      super.handleError(exception);
    }
  }

  /**
   * Consulta  aplicao origem.
   * 
   * @return a aplicao
   */
  final public Application getApplication() {
    return application;
  }

  /**
   * Execuo da tarefa.
   * @return true se a tarefa foi executada at o final (h um resultado) false
   *         se a ocorrncia de um erro impediu a execuo da tarefa (no h
   *         resultado associado)
   * @see Task#execute(java.awt.Window, String, String, int, int, boolean, boolean, ImageIcon)
   */
  public final boolean executeTask() {
    return 
      execute(application.getApplicationFrame(), application.getName(), message,
      Task.DEFAULT_CANCEL_DELAY, Task.CANCEL_BUTTON, true, true, image);
  }

  /**
   * Construtor padro.
   * 
   * @param app a aplicao.
   * @param msg uma mensagem de espera.
   */
  public ApplicationTask(final Application app, final String msg) {
    this(app, msg, null);
  }

  /**
   * Construtor padro.
   * 
   * @param app a aplicao.
   * @param msg uma mensagem de espera.
   * @param img imagem do dilogo de espera.
   */
  public ApplicationTask(final Application app, final String msg,
    final ImageIcon img) {
    super();
    application = app;
    message = msg;
    image = img;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected abstract void performTask() throws Exception;
}
