package csbase.client.applications.algorithmsmanager.actions;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.ImageIcon;

import tecgraf.javautils.FileUtils;
import csbase.client.applications.algorithmsmanager.dialogs.CategoriesManagePanel;
import csbase.client.applications.algorithmsmanager.dialogs.CategoryCreateDialog;
import csbase.client.applications.algorithmsmanager.models.DataNodeInterface;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.logic.algorithms.Category;

/**
 * Representa a ao de criar novas categorias de algoritmos.
 * 
 */
public class CategoryCreateAction extends AlgorithmsManagerAction {
  /** Painel pai que criou essa ao */
  private CategoriesManagePanel parentPanel;

  /**
   * Dilogo de categorias de algoritmos
   */
  private CategoryCreateDialog categoryDialog;

  /**
   * Categoria pai onde sero criadas as novas categorias (pode ser CategoryData
   * ou DataTreeRootNode)
   */
  private DataNodeInterface<?, ?> parentCategory;

  /**
   * Constri a ao de criao de categorias de algoritmos.
   * 
   * @param managePanel painel pai que criou a ao
   * @param icon imagem da ao
   */
  public CategoryCreateAction(CategoriesManagePanel managePanel, ImageIcon icon) {
    super(managePanel.getApplication(), icon);
    this.parentPanel = managePanel;
  }

  /**
   * Obtm o identificador da categoria, a partir do nome da categoria.
   * 
   * @param categoryName nome da categoria
   * 
   * @return o identificador da categoria
   */
  private String getCategoryId(String categoryName) {
    String categoryId = FileUtils.fixDirectoryName(categoryName);
    return categoryId;
  }

  /**
   * Cria as categorias no servidor.
   * 
   * @param categoryNames lista com os nomes das categorias
   * @return as categorias que foram efetivamente criadas
   * 
   */
  public List<Category> createCategories(final List<String> categoryNames) {
    List<Category> categoryList = new ArrayList<Category>();
    for (String categoryName : categoryNames) {
      Category category = AlgorithmManagementProxy.createCategory(
        getParentCategoryId(), getCategoryId(categoryName), categoryName,
        getApplication().getApplicationFrame());
      if (category != null) {
        categoryList.add(category);
      }
    }
    return categoryList;
  }

  /**
   * Obtm o identificador da categoria pai.
   * 
   * @return retorna o identificador da categoria pai, ou null, caso as novas
   *         categorias devam ser criadas na raiz (primeiro nvel)
   */
  protected String getParentCategoryId() {
    if (parentCategory != null) {
      return parentCategory.getId();
    }
    return null;
  }

  /**
   * Obtm o nome da categoria pai.
   * 
   * @return retorna o nome da categoria pai, ou null, caso as novas categorias
   *         devam ser criadas na raiz (primeiro nvel)
   */
  public String getParentCategoryName() {
    return parentCategory.getFullName();
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.actions.AlgorithmsManagerAction#handleActionPerformed(java.awt.event.ActionEvent)
   */
  @Override
  protected void handleActionPerformed(ActionEvent ae) throws Exception {
    categoryDialog = new CategoryCreateDialog(this);
    parentCategory = parentPanel.getFirstSelectedCategory();
    categoryDialog.initDialog();
  }

}
