package csbase.client.applications.algorithmsmanager.actions;

import java.awt.event.ActionEvent;
import java.util.Arrays;
import java.util.List;

import javax.swing.ImageIcon;

import csbase.client.applications.algorithmsmanager.dialogs.CategoriesManagePanel;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.logic.algorithms.Category;

/**
 * Representa a ao de remover uma nova categoria de algoritmos.
 * 
 */
public class CategoryRemoveAction extends AlgorithmsManagerAction {
  /** Painel de gerncia de categorias que criou essa ao */
  private CategoriesManagePanel managePanel;

  /**
   * Constri a ao de remover categorias de algoritmos.
   * 
   * @param managePanel painel pai que criou a ao
   * @param icon imagem da ao
   */
  public CategoryRemoveAction(CategoriesManagePanel managePanel, ImageIcon icon) {
    super(managePanel.getApplication(), icon);
    this.managePanel = managePanel;
  }

  /**
   * Remove as categorias no servidor.
   * 
   * @param categories lista com as categorias a serem removidas
   * @return o nmero de categorias que foram efetivamente removidas
   * 
   */
  private int removeCategories(final List<Category> categories) {
    int removedCounter = 0;
    for (Category category : categories) {
      boolean removed = AlgorithmManagementProxy.removeCategory(category
        .getId(), getApplication().getApplicationFrame());
      if (removed) {
        removedCounter++;
      }
    }
    return removedCounter;
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.actions.AlgorithmsManagerAction#handleActionPerformed(java.awt.event.ActionEvent)
   */
  @Override
  protected void handleActionPerformed(ActionEvent ae) throws Exception {
    List<Category> categories = Arrays.asList(managePanel.getSelectedDataList()
      .toArray(new Category[0]));
    int option = managePanel.getApplication().showOptionDialog(
      "Confirma a remoo das categorias? ", new String[] { "Sim", "No" });
    if (option == 0) {
      int removedCounter = removeCategories(categories);
      managePanel.getApplication().showMessage(
        "Foram removidas " + removedCounter + " categorias com sucesso.");
    }
  }

}
