package csbase.client.applications.algorithmsmanager.dialogs;

import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.List;
import java.util.SortedSet;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import tecgraf.javautils.gui.GBC;
import csbase.client.applications.algorithmsmanager.actions.AlgorithmsAndCategoriesBindAction;
import csbase.client.applications.algorithmsmanager.models.CategoryData;
import csbase.client.applications.algorithmsmanager.models.DataNodeInterface;
import csbase.client.util.ClientUtilities;

/**
 * Dilogo para criao de uma ou mais categorias. As categorias sero
 * efetivamente criadas no servidor quando o usurio enviar o comando de
 * criao.
 * 
 */
public class AlgorithmsAndCategoriesBindDialog extends JDialog {

  /** Ao que criou o painel */
  private AlgorithmsAndCategoriesBindAction bindAction;

  /** Campo texto (read-only) com o dado selcionado no pai */
  // private JComboBox cb_parentData;
  private JTextField tf_parentData;

  /** Boto para adicionar categorias */
  private JButton bindButton;

  /** Boto para fechar a janela */
  private JButton closeButton;

  /** Ns selecionados na rvore de dados */
  private SortedSet<DataNodeInterface<?, ?>> parentNodes;

  /** Filhos disponveis na rvore de dados */
  private SortedSet<DataNodeInterface<?, ?>> availableChildren;

  /** Filhos associados aos ns selecionados */
  private SortedSet<DataNodeInterface<?, ?>> nodeChildren;

  /** Texto que representa o tipo do n selecionado na rvore de dados */
  private String parentDataLabel;

  /** Descrio do tipo do dado a ser feita a associao com os ns */
  private String dataBindingDescription;

  /** Painel de amarrao dos algoritmos a categorias */
  private DataBindingPanel dataBindPanel;

  /**
   * Constri a janela para associao de algoritmos e categorias.
   * 
   * @param action ao que criou o dilogo
   * @param parentNodes ns selecionados na rvore de dados
   * @param availableChildren filhos disponveis na rvore de dados
   * @param selectedChildren filhos associados aos ns selecionados
   * @param parentDataLabel texto que representa o tipo do n selecionado na
   *        rvore de dados
   * @param dataBindingDescription descrio do tipo do dado a ser feita a
   *        associao com os ns
   */
  public AlgorithmsAndCategoriesBindDialog(
    AlgorithmsAndCategoriesBindAction action,
    SortedSet<DataNodeInterface<?, ?>> parentNodes,
    SortedSet<DataNodeInterface<?, ?>> availableChildren,
    SortedSet<DataNodeInterface<?, ?>> selectedChildren,
    String parentDataLabel, String dataBindingDescription) {
    super(action.getApplication().getApplicationFrame(), action
      .getApplication().getString("AlgorithmsAndCategoriesBindDialog.title"),
      false);
    this.bindAction = action;
    this.parentNodes = parentNodes;
    this.availableChildren = availableChildren;
    this.nodeChildren = selectedChildren;
    this.parentDataLabel = parentDataLabel;
    this.dataBindingDescription = dataBindingDescription;
    createDialog();
  }

  /**
   * Obtm o texto correspondente a uma chave de internacionalizao.
   * 
   * @param key chave de internacionalizao
   * @return o texto correspondente a chave
   */
  private String getString(String key) {
    return bindAction.getApplication().getString(key);
  }

  /**
   * Cria o dilogo.
   */
  protected void createDialog() {
    getContentPane().setLayout(new GridBagLayout());

    JPanel mainPanel = getMainPanel();
    JPanel buttonsPanel = getButtonPanel();

    getContentPane().add(mainPanel, new GBC(0, 0).both().insets(5, 10, 5, 5));
    getContentPane().add(buttonsPanel,
      new GBC(0, 1).both().insets(5, 10, 5, 10));
    pack();
    setLocationRelativeTo(bindAction.getApplication().getApplicationFrame());
    setVisible(true);
  }

  /**
   * Obtm o painel com o modo de visualizao da rvore de categorias de
   * algoritmos.
   * 
   * @return o painel criado
   */
  private JPanel getMainPanel() {
    JPanel panel = new JPanel(new GridBagLayout());
    JLabel parentSelectedDataLabel = new JLabel(parentDataLabel);
    tf_parentData = new JTextField();
    tf_parentData.setEditable(false);

    JPanel fieldsPanel = new JPanel(new GridBagLayout());

    fieldsPanel.add(parentSelectedDataLabel, new GBC(0, 0).none().west()
      .insets(5, 5, 5, 5));
    fieldsPanel.add(tf_parentData, new GBC(1, 0).horizontal().west().insets(5,
      5, 5, 5));

    JPanel dataBindingPanel = getDataBindingPanel();
    panel.add(fieldsPanel, new GBC(0, 0).none().west().insets(5, 5, 5, 5));
    panel.add(dataBindingPanel, new GBC(0, 1).horizontal().west().insets(5, 5,
      5, 5));
    return panel;
  }

  /**
   * Obtm o painel que realiza a associao dos dados.
   * 
   * @return o painel
   */
  private JPanel getDataBindingPanel() {
    dataBindPanel = new DataBindingPanel(bindAction.getApplication(),
      availableChildren, nodeChildren, dataBindingDescription);
    return dataBindPanel;
  }

  /**
   * Cria o painel com os botes referentes ao modo de visualizao da rvore de
   * dados.
   * 
   * @return o painel criado
   */
  private JPanel getButtonPanel() {
    JPanel buttonPanel = new JPanel(new GridBagLayout());
    bindButton = new JButton(
      getString("AlgorithmsAndCategoriesBindDialog.button.bind"));
    bindButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        bindData();
      }

    });
    closeButton = new JButton(
      getString("AlgorithmsAndCategoriesBindDialog.button.close"));
    closeButton.addActionListener(new ActionListener() {

      @Override
      public void actionPerformed(ActionEvent e) {
        close();
      }
    });
    buttonPanel.add(bindButton, new GBC(0, 0).none().west().insets(5, 5, 5, 5));
    buttonPanel
      .add(closeButton, new GBC(1, 0).none().west().insets(5, 5, 5, 5));
    ClientUtilities.adjustEqualSizes(bindButton, closeButton);

    return buttonPanel;
  }

  /**
   * Fecha o dilogo.
   */
  public void close() {
    dispose();
  }

  /**
   * A partir dos nomes das categorias adicionadas pelo usurio, as categorias
   * sero efetivamente criadas no servidor.
   */
  protected void bindData() {
    List<Object> algorithmIds = getSelectedAlgorithmsIds();
    List<String> categoryIds = getSelectedCategoriesIds();
    boolean result = bindAction.bindAlgorithmsAndCategories(algorithmIds,
      categoryIds);
    if (result) {
      bindAction.getApplication().showMessage(
        getString("AlgorithmsAndCategoriesBindDialog.msg.create.sucess"));
    }
    else {
      bindAction
        .getApplication()
        .showError(
          getString("AlgorithmsAndCategoriesBindDialog.msg.error.create.no_sucess"));
    }
    close();
  }

  /**
   * Verifica se o tipo dos ns (pais) selecionados na rvore de dados para
   * realizar a associao so categorias.
   * 
   * @return retorna true se os ns pais selecionados na rvore de dados so
   *         categorias, caso contrrio, retorna false
   */
  private boolean isCategoryParent() {
    return parentNodes.first().getClass().equals(CategoryData.class);
  }

  private List<String> getSelectedCategoriesIds() {
    List<String> categoryIds = new Vector<String>();
    if (isCategoryParent()) {
      categoryIds = Arrays.asList(getParentIds().toArray(new String[0]));
    }
    else {
      categoryIds = Arrays.asList(getSelectedItemIds().toArray(new String[0]));
    }
    return categoryIds;
  }

  private List<Object> getSelectedAlgorithmsIds() {
    List<Object> algorithmIds = new Vector<Object>();
    if (!isCategoryParent()) {
      algorithmIds = getParentIds();
    }
    else {
      algorithmIds = getSelectedItemIds();
    }
    return algorithmIds;
  }

  private List<Object> getParentIds() {
    List<Object> parentIds = new Vector<Object>();
    for (DataNodeInterface<?, ?> parentNode : parentNodes) {
      parentIds.add(parentNode.getId());
    }
    return parentIds;
  }

  private List<Object> getSelectedItemIds() {
    List<Object> selectedIds = new Vector<Object>();
    List<DataNodeInterface<?, ?>> selectedItems = dataBindPanel
      .getSelectedItems();
    for (DataNodeInterface<?, ?> dataNode : selectedItems) {
      selectedIds.add(dataNode.getId());
    }
    return selectedIds;
  }

  /**
   * Inicializa o dilogo.
   */
  public void initDialog() {
    DataNodeInterface<?, ?>[] parentNodes = getParentSelectedNodes();
    tf_parentData.setText(parentNodes[0].getFullName());
    setVisible(true);
  }

  /**
   * Obtm os ns selecionados na rvore de dados no painel pai.
   * 
   * @return os ns selecionados na rvore de dados
   */
  private DataNodeInterface<?, ?>[] getParentSelectedNodes() {
    return parentNodes.toArray(new DataNodeInterface<?, ?>[0]);
  }
}
