package csbase.client.applications.algorithmsmanager.dialogs;

import java.awt.GridBagLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.tree.FilterableTreePanel;
import csbase.client.applications.AbstractSimpleApplicationPanel;
import csbase.client.applications.algorithmsmanager.AlgorithmsManager;
import csbase.client.applications.algorithmsmanager.actions.AlgorithmsAndCategoriesBindAction;
import csbase.client.applications.algorithmsmanager.actions.CategoryCreateAction;
import csbase.client.applications.algorithmsmanager.actions.CategoryRemoveAction;
import csbase.client.applications.algorithmsmanager.dialogs.DataTreeView.ViewChildrenMode;
import csbase.client.applications.algorithmsmanager.models.AlgorithmData;
import csbase.client.applications.algorithmsmanager.models.CategoryData;
import csbase.client.applications.algorithmsmanager.models.DataNodeInterface;
import csbase.client.applications.algorithmsmanager.models.DataTreeRootNode;
import csbase.client.util.ClientUtilities;
import csbase.logic.AlgoEvent;
import csbase.logic.algorithms.Category;

/**
 * Painel de gerncia de algoritmos.
 * 
 */
public class CategoriesManagePanel extends
  AbstractSimpleApplicationPanel<AlgorithmsManager> {

  /** Painel com o modo de visualizao, que contm a rvore de dados */
  private JPanel viewModePanel;

  /** Painel de gerncia das categorias de algoritmos */
  private JPanel managePanel;

  /** Viso da rvore de dados, a partir da qual se obtm a rvore com filtro */
  private DataTreeView<?, ?> dataTreeView;

  /** Painel com os botes para gerenciamento dos ns selecionados */
  private JPanel buttonPanel;

  /** Modo de visualizao da rvore por categorias de algoritmos */
  private JRadioButton rb_categoriesMode;

  /** Modo de visualizao da rvore por algoritmos */
  private JRadioButton rb_algorithmsMode;

  /** Boto checvel que exibe ou no os algoritmos das categorias */
  private JCheckBox cb_hideAlgorithms;

  /** Boto checvel que exibe ou no as categorias associadas aos algoritmos */
  private JCheckBox cb_hideCategories;

  /** Modo corrente de visualizao da rvore de dados */
  private TreeMode currentTreeMode;

  /** Boto para adicionar categorias */
  private JButton addButton;

  /** Boto para remover categorias */
  private JButton removeButton;

  /** Boto para associar filhos ao n selecionado */
  private JButton bindChildrenButton;

  /** Boto para desassociar filhos ao n selecionado */
  // private JButton unbindChildrenButton;

  /**
   * Modo de visualizao da rvore de dados: por categorias de algoritmos ou
   * por algoritmos.
   */
  public enum TreeMode {
    /** rvore organizada por categorias */
    BY_CATEGORY,
    /** rvore organizada por algoritmos */
    BY_ALGORITHMS;
  }

  /**
   * Constri o painel.
   * 
   * @param application aplicao de gerncia de algoritmos
   */
  public CategoriesManagePanel(AlgorithmsManager application) {
    super(application);
    setCurrentTreeMode(TreeMode.BY_CATEGORY);
    createButtons();
    buildPanel();
    addCategoryEventsObserver();
  }

  /**
   * Adiciona um observador para eventos de categorias de algoritmos que ocorram
   * no servidor.
   */
  private void addCategoryEventsObserver() {
    Category.addObserver(new Observer() {
      @Override
      public void update(Observable o, Object arg) {
        AlgoEvent event = (AlgoEvent) arg;
        CategoriesManagePanel.this.update(event);
      }
    });
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.AbstractSimpleApplicationPanel#buildPanel()
   */
  @Override
  protected void buildPanel() {
    setLayout(new GridBagLayout());
    setBorder(BorderFactory.createEtchedBorder());
    JPanel mainPanel = getMainPanel();
    add(mainPanel, new GBC(0, 0).both().west().insets(5, 5, 5, 5));
    changeAnotherChildrenState();
  }

  /**
   * Obtm o painel com o modo de visualizao da rvore de categorias de
   * algoritmos.
   * 
   * @return o painel criado
   */
  private JPanel getMainPanel() {
    JPanel panel = new JPanel(new GridBagLayout());
    panel.setBorder(BorderFactory
      .createTitledBorder(getString("CategoriesManagePanel.mainpanel.title")));

    panel.add(getViewModeGroupPanel(), new GBC(0, 0).none().west().insets(5, 5,
      5, 5));
    managePanel = getManagePanel();
    panel.add(managePanel, new GBC(0, 1).both().west().insets(5, 5, 5, 5));

    return panel;
  }

  /**
   * Obtm o painel de gerncia das categorias de algoritmos.
   * 
   * @return o painel criado
   */
  private JPanel getManagePanel() {
    managePanel = new JPanel(new GridBagLayout());
    FilterableTreePanel dataTreePanel = getDataTreePanel(getViewChildrenMode());
    buttonPanel = getButtonPanel();
    managePanel.add(dataTreePanel, new GBC(0, 0).both().west().insets(5, 5, 5,
      5));
    managePanel
      .add(buttonPanel, new GBC(1, 0).both().west().insets(5, 5, 5, 5));

    return managePanel;
  }

  /**
   * Obtm o painel com o grupo de modo de visualizao da rvore de dados.
   * 
   * @return o painel criado
   */
  private JPanel getViewModeGroupPanel() {
    viewModePanel = new JPanel(new GridBagLayout());
    ButtonGroup viewModeGroup = new ButtonGroup();

    ItemListener categoriesListener = createViewModeItemListener(TreeMode.BY_CATEGORY);
    ItemListener algorithmsListener = createViewModeItemListener(TreeMode.BY_ALGORITHMS);

    JLabel description = new JLabel(
      getString("CategoriesManagePanel.viewmode.panel.description"));
    rb_categoriesMode = getViewModeRadioButton(viewModeGroup,
      getString("CategoriesManagePanel.field.rb.categories"),
      categoriesListener, true);
    rb_algorithmsMode = getViewModeRadioButton(viewModeGroup,
      getString("CategoriesManagePanel.field.rb.algorithms"),
      algorithmsListener, false);
    cb_hideAlgorithms = createHideChildrenCheckBox(getString("CategoriesManagePanel.field.hideAlgorithms"));
    cb_hideCategories = createHideChildrenCheckBox(getString("CategoriesManagePanel.field.hideCategories"));

    int y = 0;
    int x = 0;
    viewModePanel.add(description, new GBC(x, y++).none().west().insets(5, 5,
      5, 5));
    viewModePanel.add(rb_categoriesMode, new GBC(x++, y).none().west().insets(
      5, 5, 5, 5));
    viewModePanel.add(cb_hideAlgorithms, new GBC(x, y++).none().west().insets(
      5, 5, 5, 5));
    // y = 1;
    x = 0;
    viewModePanel.add(rb_algorithmsMode, new GBC(x++, y).none().west().insets(
      5, 5, 5, 5));
    viewModePanel.add(cb_hideCategories, new GBC(x, y).none().west().insets(5,
      5, 5, 5));

    // viewModePanel.add(rb_categoriesMode, new GBC(0,
    // y++).none().west().insets(
    // 5, 5, 5, 10));
    // viewModePanel.add(cb_hideAlgorithms, new GBC(0,
    // y).none().west().insets(5,
    // 20, 5, 10));
    // y = 1;
    // viewModePanel.add(rb_algorithmsMode, new GBC(1,
    // y++).none().west().insets(
    // 5, 25, 5, 5));
    // viewModePanel.add(cb_hideCategories, new GBC(1,
    // y).none().west().insets(5,
    // 40, 5, 5));

    return viewModePanel;
  }

  /**
   * Cria um radio button de mode de visualizao da rvore de dados.
   * 
   * @param viewModeGroup grupo de botes a ser inserido
   * @param label texto do boto
   * @param itemListener listener de item associado
   * @param state se true, o estado inicial do boto  selecionado, caso
   *        contrrio, no fica selecionado
   * 
   * @return o radio button criado
   */
  private JRadioButton getViewModeRadioButton(ButtonGroup viewModeGroup,
    String label, ItemListener itemListener, boolean state) {
    JRadioButton rb = new JRadioButton(label, state);
    rb.addItemListener(itemListener);
    viewModeGroup.add(rb);
    return rb;
  }

  /**
   * Cria o listener de item para o boto de seleo de modo de visualizao da
   * rvore de dados.
   * 
   * @param mode modo de visualizao da rvore de dados
   * @return o listener de item criado
   */
  private ItemListener createViewModeItemListener(final TreeMode mode) {
    ItemListener categoriesListener = new ItemListener() {
      @Override
      public void itemStateChanged(ItemEvent e) {
        setCurrentTreeMode(mode);
        changeAnotherChildrenState();
        updateManagePanel();
      }
    };
    return categoriesListener;
  }

  /**
   * Atualiza o painel de gerncia de algoritmos.
   * 
   */
  protected void updateManagePanel() {
    managePanel.removeAll();
    FilterableTreePanel dataTreePanel = getDataTreePanel(getViewChildrenMode());
    buttonPanel = getButtonPanel();
    managePanel.add(dataTreePanel, new GBC(0, 0).both().west().insets(5, 5, 5,
      5));
    managePanel
      .add(buttonPanel, new GBC(1, 0).both().west().insets(5, 5, 5, 5));
    managePanel.revalidate();
  }

  /**
   * Altera o estado (habilitado ou no) dos botes para exibio de filhos, de
   * maneira que fique habilitado somente o correspondente ao modo corrente.
   * 
   */
  protected void changeAnotherChildrenState() {
    if (getCurrentTreeMode().equals(TreeMode.BY_CATEGORY)) {
      cb_hideCategories.setEnabled(false);
      cb_hideAlgorithms.setEnabled(true);
    }
    else {
      cb_hideCategories.setEnabled(true);
      cb_hideAlgorithms.setEnabled(false);
    }
  }

  /**
   * Altera o texto do boto checvel de exibio ou no dos filhos, de acordo
   * com o modo de visualizao da rvore de dados, que determina os tipos de
   * ns principais.
   * 
   * @param mode modo de visualizao da rvore de dados
   */
  protected void changeChildrenCheckBoxLabel(TreeMode mode) {
    if (mode.equals(TreeMode.BY_CATEGORY)) {
      cb_hideAlgorithms
        .setText(getString("CategoriesManagePanel.field.hideAlgorithms"));
    }
    else {
      cb_hideAlgorithms
        .setText(getString("CategoriesManagePanel.field.hideCategories"));
    }
  }

  /**
   * Obtm o boto checvel para ocultar ou no os filhos dos ns da rvore.
   * 
   * @param label texto do boto
   * 
   * @return o boto checvel
   */
  private JCheckBox createHideChildrenCheckBox(String label) {
    JCheckBox checkBox = new JCheckBox(label);
    checkBox.setSelected(false);
    checkBox.addItemListener(new ItemListener() {

      @Override
      public void itemStateChanged(ItemEvent e) {
        updateManagePanel();
      }
    });

    return checkBox;
  }

  /**
   * Cria o painel da rvore de dados.
   * 
   * @param viewMode modo de visualizao dos filhos dos ns da rvore
   * @return o painel criado
   */
  private FilterableTreePanel getDataTreePanel(ViewChildrenMode viewMode) {
    if (getCurrentTreeMode().equals(TreeMode.BY_CATEGORY)) {
      dataTreeView = new DataTreeView<CategoryData, AlgorithmData>(
        getApplication(), getApplication().getCategories(),
        getString("CategoriesManagePanel.root.label.categories"), viewMode);
    }
    else {
      dataTreeView = new DataTreeView<AlgorithmData, CategoryData>(
        getApplication(), getApplication().getAlgorithms(),
        getString("CategoriesManagePanel.root.label.algorithms"), viewMode);
    }
    FilterableTreePanel dataTreePanel = dataTreeView.getPanel();
    dataTreePanel.getTree().getSelectionModel().addTreeSelectionListener(
      new TreeSelectionListener() {

        @Override
        public void valueChanged(TreeSelectionEvent e) {
          verifyAndChangeButtonsState();
        }
      });
    dataTreeView.selectRootNode();
    return dataTreePanel;
  }

  /**
   * Cria os botes de gerncia dos ns.
   */
  private void createButtons() {
    if (getCurrentTreeMode().equals(TreeMode.BY_CATEGORY)) {
      addButton = new JButton(new CategoryCreateAction(this, null));
      removeButton = new JButton(new CategoryRemoveAction(this, null));
    }
    bindChildrenButton = new JButton(new AlgorithmsAndCategoriesBindAction(
      this, null));
  }

  /**
   * Obtm a categoria selecionada na rvore de dados.
   * 
   * @return retorna a categoria selecionada na rvore de dados. Se a seleo
   *         atual no corresponder a uma categoria ou no tiver nenhum item
   *         selecionado, retorna-se null
   */
  public DataNodeInterface<?, ?> getFirstSelectedCategory() {
    if (dataTreeView != null) {
      DataNodeInterface<?, ?> firstSelectedNode = dataTreeView
        .getFirstSelectedNode();
      if (dataTreeView.isRootNodeSelected()
        || verifySelectedItemIsCategory(firstSelectedNode)) {
        return firstSelectedNode;
      }
    }
    return null;
  }

  /**
   * Obtm a lista dos dados selecionados na rvore de dados.
   * 
   * @return retorna a lista de dados selecionados na rvore de dados. Se a
   *         seleo
   */
  public List<DataNodeInterface<?, ?>> getSelectedDataList() {
    List<DataNodeInterface<?, ?>> selectedDataList = new ArrayList<DataNodeInterface<?, ?>>();
    if (dataTreeView != null) {
      selectedDataList = dataTreeView.getSelectedDataList();
    }
    return selectedDataList;
  }

  /**
   * Verifica e modifica o estado dos botes de acordo com as selees feitas
   * pelo usurio na rvore de dados ou de acordo com o modo de visualizao da
   * rvore de dados.
   * 
   */
  private void verifyAndChangeButtonsState() {
    // boto bind
    boolean enableBindButton = false;
    if (!dataTreeView.isRootNodeSelected()) {
      if (dataTreeView.getSelectionCount() == 1) {
        enableBindButton = verifySelectedItemsModeCompliant(dataTreeView
          .getSelectedDataList(), getCurrentTreeMode());
      }
    }
    if (enableBindButton) {
      ((AlgorithmsAndCategoriesBindAction) bindChildrenButton.getAction())
        .setCurrentMode(getCurrentTreeMode());
    }
    bindChildrenButton.setEnabled(enableBindButton);

    if (getCurrentTreeMode().equals(TreeMode.BY_CATEGORY)) {
      // boto add
      boolean enableAddButton = false;
      if (dataTreeView.getSelectionCount() == 1) {
        enableAddButton = dataTreeView.isRootNodeSelected()
          || verifySelectedItemIsCategory(dataTreeView.getFirstSelectedNode());
      }
      addButton.setEnabled(enableAddButton);

      // boto remove
      boolean enableRemoveButton = false;
      if (!dataTreeView.isRootNodeSelected()) {
        if (dataTreeView.getSelectionCount() > 0) {
          enableRemoveButton = verifySelectedItemsModeCompliant(dataTreeView
            .getSelectedDataList(), getCurrentTreeMode());
        }
      }
      removeButton.setEnabled(enableRemoveButton);
    }

  }

  /**
   * Verifica se os itens selecionados na rvore de dados so compatveis com o
   * modo de visualizao da rvore.
   * 
   * @param selectedDataList dados selecionados
   * @param mode modo de visualizao da vore
   * @return retorna true, se os dados selecionados so compatveis com o modo
   *         de visualizao, caso contrrio, retorna false
   */
  private boolean verifySelectedItemsModeCompliant(
    List<DataNodeInterface<?, ?>> selectedDataList, TreeMode mode) {
    boolean isValid = false;
    if (mode.equals(TreeMode.BY_CATEGORY)) {
      for (DataNodeInterface<?, ?> dataNode : selectedDataList) {
        if (!verifySelectedItemIsCategory(dataNode)) {
          return false;
        }
      }
      isValid = true;
    }
    else {
      for (DataNodeInterface<?, ?> dataNode : selectedDataList) {
        if (!verifySelectedItemIsAlgorithm(dataNode)) {
          return false;
        }
      }
      isValid = true;
    }
    return isValid;
  }

  /**
   * Verifica se o caminho selecionado na rvore de dados corresponde a uma
   * categoria.
   * 
   * @param node n da rvore de dados
   * 
   * @return retorna true, se o item selecionado  uma categoria, caso
   *         contrrio, retorna true
   */
  private boolean verifySelectedItemIsCategory(DataNodeInterface<?, ?> node) {
    return node instanceof CategoryData;
  }

  /**
   * Verifica se o caminho selecionado na rvore de dados corresponde a uma
   * categoria.
   * 
   * @param node n da rvore de dados
   * 
   * @return retorna true, se o item selecionado  uma categoria, caso
   *         contrrio, retorna true
   */
  private boolean verifySelectedItemIsAlgorithm(DataNodeInterface<?, ?> node) {
    return node instanceof AlgorithmData;
  }

  /**
   * Verifica se o n corresponde a raiz da rvore de dados.
   * 
   * @param node n selecionado
   * @return retorna true, se for o n raiz, caso contrrio retorna false
   */
  public boolean isRootNode(DataNodeInterface<?, ?> node) {
    return node instanceof DataTreeRootNode<?, ?>;
  }

  /**
   * Cria o painel com os botes referentes ao modo de visualizao da rvore de
   * dados.
   * 
   * @return o painel criado
   */
  private JPanel getButtonPanel() {
    int y = 0;
    if (getCurrentTreeMode().equals(TreeMode.BY_CATEGORY)) {
      buttonPanel = new JPanel(new GridBagLayout());
      // bindChildrenButton.setText("Associar algoritmo...");
      // unbindChildrenButton.setText("Desassociar algoritmo");
      buttonPanel.add(addButton, new GBC(0, y++).none().east().insets(5, 5, 5,
        5));
      buttonPanel.add(removeButton, new GBC(0, y++).none().east().insets(5, 5,
        5, 5));
    }
    else {
      buttonPanel = new JPanel(new GridBagLayout());
      // bindChildrenButton.setText("Associar categoria...");
      // unbindChildrenButton.setText("Desassociar categoria");
    }
    ClientUtilities.adjustEqualSizes(addButton, removeButton,
      bindChildrenButton); // , unbindChildrenButton);

    buttonPanel.add(bindChildrenButton, new GBC(0, y++).none().northwest()
      .insets(5, 5, 5, 5));
    // buttonPanel.add(unbindChildrenButton, new GBC(0, y++).none().northwest()
    // .insets(5, 5, 5, 5));

    return buttonPanel;
  }

  /**
   * Estabelece o modo corrente de visualizao da rvore de dados.
   * 
   * @param treeMode modo de visualizao da rvore de dados
   */
  private void setCurrentTreeMode(TreeMode treeMode) {
    currentTreeMode = treeMode;
  }

  /**
   * Obtm o modo corrente de visualizao da rvore de dados.
   * 
   * @return o modo corrente de visualizao da rvore de dados.
   */
  private TreeMode getCurrentTreeMode() {
    return currentTreeMode;
  }

  /**
   * Obtm o modo de visualizao da rvore de dados.
   * 
   * @return o modo de visualizao da rvore, que exibe ou oculta os filhos
   */
  public ViewChildrenMode getViewChildrenMode() {
    if (getCurrentTreeMode().equals(TreeMode.BY_CATEGORY)) {
      if (cb_hideAlgorithms.isSelected()) {
        return ViewChildrenMode.HIDE_CHILDREN;
      }
    }
    else {
      if (cb_hideCategories.isSelected()) {
        return ViewChildrenMode.HIDE_CHILDREN;
      }
    }
    return ViewChildrenMode.SHOW_CHILDREN;
  }

  /**
   * Atualiza a representao da rvore, sempre que for recebida uma notificao
   * de alterao no servidor.
   * 
   * @param event
   * 
   * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
   */
  public void update(AlgoEvent event) {
    switch (event.type) {
      case AlgoEvent.CREATE:
      case AlgoEvent.MODIFY:
      case AlgoEvent.DELETE:
        updatePanelFromCategoryEvent();
        break;
      default:
        String msg = MessageFormat.format(LNG
          .get("CategoriesManagePanel.msg.error.unknown_event"),
          new Object[] { new Integer(event.type) });
        System.err.println(msg);
        break;
    }
  }

  /**
   * Atualiza o painel a partir do evento de categoria ocorrido.
   */
  private void updatePanelFromCategoryEvent() {
    getApplication().loadCategoryData();
    updateManagePanel();
  }
}
