package csbase.client.applications.algorithmsmanager.dialogs;

import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

import tecgraf.javautils.gui.GBC;
import csbase.client.applications.algorithmsmanager.actions.CategoryCreateAction;
import csbase.client.util.ClientUtilities;
import csbase.logic.algorithms.Category;

/**
 * Dilogo para criao de uma ou mais categorias. As categorias sero
 * efetivamente criadas no servidor quando o usurio enviar o comando de
 * criao.
 * 
 */
public class CategoryCreateDialog extends JDialog {

  /** Ao que criou o painel */
  private CategoryCreateAction createAction;

  /** Componente de lista com os nomes de todas as categorias a serem criadas */
  private JList categorySelectionList;

  /** Campo texto (read-only) com o nome da categoria pai */
  private JTextField tf_categoryParentName;

  /** Campo texto com o nome da categoria */
  private JTextField tf_categoryName;

  /** Boto para adicionar categorias */
  private JButton createButton;

  /** Boto para remover categorias */
  private JButton closeButton;

  /** Boto para adicionar uma nova categoria na lista */
  private JButton addButton;

  /** Boto para remover uma nova categoria na lista */
  private JButton removeButton;

  /**
   * Constri o painel.
   * 
   * @param action ao que criou o dilogo
   */
  public CategoryCreateDialog(CategoryCreateAction action) {
    super(action.getApplication().getApplicationFrame(), action
      .getApplication().getString("CategoryCreateDialog.title"), false);
    this.createAction = action;
    createDialog();
  }

  /**
   * Obtm o texto correspondente a uma chave de internacionalizao.
   * 
   * @param key chave de internacionalizao
   * @return o texto correspondente a chave
   */
  private String getString(String key) {
    return createAction.getApplication().getString(key);
  }

  /**
   * Cria o dilogo.
   */
  protected void createDialog() {
    getContentPane().setLayout(new GridBagLayout());

    JPanel mainPanel = getMainPanel();
    JPanel buttonsPanel = getButtonPanel();

    getContentPane().add(mainPanel, new GBC(0, 0).both().insets(5, 10, 5, 5));
    getContentPane().add(buttonsPanel,
      new GBC(0, 1).both().insets(5, 10, 5, 10));
    getRootPane().setDefaultButton(addButton);
    pack();
    setLocationRelativeTo(createAction.getApplication().getApplicationFrame());
    setVisible(true);
  }

  /**
   * Obtm o painel com o modo de visualizao da rvore de categorias de
   * algoritmos.
   * 
   * @return o painel criado
   */
  private JPanel getMainPanel() {
    JPanel panel = new JPanel(new GridBagLayout());
    JLabel categoryNameLabel = new JLabel(
      getString("CategoryCreateDialog.label.name"));
    JLabel parentCategoryNameLabel = new JLabel(
      getString("CategoryCreateDialog.label.parent.name"));
    tf_categoryParentName = new JTextField();
    tf_categoryParentName.setEditable(false);
    tf_categoryName = new JTextField(20);

    categorySelectionList = new JList(new DefaultListModel());
    categorySelectionList.getModel().addListDataListener(
      new ListDataListener() {

        @Override
        public void intervalRemoved(ListDataEvent e) {
          verifyAndChangeButtonsState();
        }

        @Override
        public void intervalAdded(ListDataEvent e) {
          verifyAndChangeButtonsState();
        }

        @Override
        public void contentsChanged(ListDataEvent e) {
          verifyAndChangeButtonsState();
        }
      });
    JScrollPane categoriesPanel = new JScrollPane(categorySelectionList);

    JPanel fieldsPanel = new JPanel(new GridBagLayout());

    fieldsPanel.add(parentCategoryNameLabel, new GBC(0, 0).none().west()
      .insets(5, 5, 5, 5));
    fieldsPanel.add(tf_categoryParentName, new GBC(1, 0).horizontal().west()
      .insets(5, 5, 5, 5));
    fieldsPanel.add(categoryNameLabel, new GBC(0, 1).none().west().insets(5, 5,
      5, 5));
    fieldsPanel.add(tf_categoryName, new GBC(1, 1).horizontal().west().insets(
      5, 5, 5, 5));
    fieldsPanel.add(categoriesPanel, new GBC(1, 2).horizontal().west().insets(
      5, 5, 5, 5));

    panel.add(fieldsPanel, new GBC(0, 0).none().west().insets(5, 5, 5, 5));
    panel.add(getCategoriesButtonPanel(), new GBC(1, 0).horizontal().west()
      .insets(5, 5, 5, 5));
    return panel;
  }

  /**
   * Obtm o painel com os botes para adio e remoo de categorias que
   * devero ser criadas no servidor. Esses botes gerenciam uma lista no
   * cliente.
   * 
   * @return o painel criado
   */
  private JPanel getCategoriesButtonPanel() {
    JPanel buttonPanel = new JPanel(new GridBagLayout());
    addButton = new JButton(getString("CategoryCreateDialog.button.add"));
    addButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        if (getCategoryName().trim().isEmpty()) {
          return;
        }
        if (!verifyAndAddCategoryName()) {
          createAction
            .getApplication()
            .showError(
              getString("CategoryCreateDialog.msg.error.category.name_duplicated"));
        }
        else {
          tf_categoryName.setText("");
        }
        tf_categoryName.requestFocus();
      }
    });
    removeButton = new JButton(getString("CategoryCreateDialog.button.remove"));
    removeButton.addActionListener(new ActionListener() {

      @Override
      public void actionPerformed(ActionEvent e) {
        removeCategoryName();
      }
    });
    buttonPanel.add(addButton, new GBC(0, 0).none().east().insets(5, 5, 5, 5));
    buttonPanel.add(removeButton, new GBC(0, 1).none().east()
      .insets(5, 5, 5, 5));
    ClientUtilities.adjustEqualSizes(addButton, removeButton);

    return buttonPanel;
  }

  /**
   * Remove o nome da categoria da lista das categorias que sero criadas.
   */
  protected void removeCategoryName() {
    int selectedIndex = categorySelectionList.getSelectedIndex();
    ((DefaultListModel) (categorySelectionList.getModel()))
      .remove(selectedIndex);
  }

  /**
   * Verifica se o nome da categoria digitado pelo usurio no existe na lista
   * das categorias que vo ser criadas. Se existir, o mtodo retorna false,
   * caso contrrio, o nome  adicionado na lista e o mtodo retorna true.
   * 
   * @return retorna true se o nome da categoria foi adicionado na lista, caso
   *         contrrio, retorna false
   */
  protected boolean verifyAndAddCategoryName() {
    boolean added = false;
    String categoryName = getCategoryName();
    DefaultListModel model = (DefaultListModel) categorySelectionList
      .getModel();
    if (!model.contains(categoryName)) {
      added = true;
      model.addElement(getCategoryName());
    }
    return added;
  }

  /**
   * Cria o painel com os botes referentes ao modo de visualizao da rvore de
   * dados.
   * 
   * @return o painel criado
   */
  private JPanel getButtonPanel() {
    JPanel buttonPanel = new JPanel(new GridBagLayout());
    createButton = new JButton(getString("CategoryCreateDialog.button.create"));
    createButton.setEnabled(false);
    createButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        createCategories();
      }

    });
    closeButton = new JButton(getString("CategoryCreateDialog.button.close"));
    closeButton.addActionListener(new ActionListener() {

      @Override
      public void actionPerformed(ActionEvent e) {
        close();
      }
    });
    buttonPanel.add(createButton, new GBC(0, 0).none().west()
      .insets(5, 5, 5, 5));
    buttonPanel
      .add(closeButton, new GBC(1, 0).none().west().insets(5, 5, 5, 5));
    ClientUtilities.adjustEqualSizes(createButton, closeButton);

    return buttonPanel;
  }

  /**
   * Fecha o dilogo.
   */
  public void close() {
    dispose();
  }

  /**
   * A partir dos nomes das categorias adicionadas pelo usurio, as categorias
   * sero efetivamente criadas no servidor.
   */
  protected void createCategories() {
    List<String> categoryNames = new ArrayList<String>();
    Enumeration<?> elements = ((DefaultListModel) (categorySelectionList
      .getModel())).elements();
    while (elements.hasMoreElements()) {
      categoryNames.add((String) elements.nextElement());
    }
    List<Category> categoriesCreated = createAction
      .createCategories(categoryNames);
    if (categoriesCreated.size() != categoryNames.size()) {
      createAction.getApplication().showError(
        getString("CategoryCreateDialog.msg.error.create.no_sucess"));
    }
    else {
      createAction.getApplication().showMessage(
        getString("CategoryCreateDialog.msg.create.sucess"));
    }

    close();
  }

  /**
   * Obtm o nome da categoria especificado pelo usurio.
   * 
   * @return o nome da categoria
   */
  private String getCategoryName() {
    return tf_categoryName.getText();
  }

  /**
   * Verifica e modifica o estado dos botes de acordo com as selees feitas
   * pelo usurio na rvore de dados ou de acordo com o modo de visualizao da
   * rvore de dados.
   * 
   */
  private void verifyAndChangeButtonsState() {
    boolean enableCreateButton = getNumberOfCategoriesToCreate() > 0 ? true
      : false;
    createButton.setEnabled(enableCreateButton);
  }

  /**
   * Obtm o nmero de categorias a serem criadas.
   * 
   * @return o nmero de categorias a serem criadas
   */
  private int getNumberOfCategoriesToCreate() {
    return categorySelectionList.getModel().getSize();
  }

  /**
   * Inicializa o dilogo.
   */
  public void initDialog() {
    tf_categoryParentName.setText(createAction.getParentCategoryName());
    tf_categoryName.requestFocus();
    setVisible(true);
  }
}
