package csbase.client.applications.algorithmsmanager.models;

import java.util.SortedSet;
import java.util.TreeSet;

import javax.swing.Icon;

import csbase.client.applications.ApplicationImages;
import csbase.client.applications.algorithmsmanager.AlgorithmsManager;
import csbase.logic.algorithms.AlgorithmInfo;

/**
 * Representa os dados do algoritmo na rvore.
 * 
 */
public class AlgorithmData extends AlgorithmInfo implements
  DataNodeInterface<AlgorithmData, CategoryData> {

  /**
   * Constri os dados do algoritmo da rvore, a partir do algoritmo
   * especificado.
   * 
   * @param algorithmInfo dados do algoritmo
   */
  public AlgorithmData(AlgorithmInfo algorithmInfo) {
    super(algorithmInfo);
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.models.DataNodeInterface#getDataChildren()
   */
  @Override
  public SortedSet<CategoryData> getDataChildren() {
    SortedSet<CategoryData> children = AlgorithmsManager
      .getAlgorithmCategories(this);
    return (children != null) ? children : new TreeSet<CategoryData>();
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.models.DataNodeInterface#getSubNodes()
   */
  @Override
  public SortedSet<AlgorithmData> getSubNodes() {
    return new TreeSet<AlgorithmData>();
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.models.DataNodeInterface#isVisible()
   */
  @Override
  public boolean isVisible() {
    return (!HIDE_ALGORITHM_VALUE
      .equals(getPropertyValue(HIDE_ALGORITHM_PROPERTY)));
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.models.DataNodeInterface#compare
   *      (java.lang.Object)
   */
  @Override
  public int compare(AlgorithmData node) {
    return compareTo(node);
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.models.DataNodeInterface#getIcon()
   */
  @Override
  public Icon getIcon() {
    return ApplicationImages.ICON_ALGORITHM_16;
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.models.DataInterface#getFullName()
   */
  @Override
  public String getFullName() {
    return getName();
  }
}
