package csbase.client.applications.algorithmsmanager.models;

import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.swing.Icon;

import csbase.client.applications.ApplicationImages;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.Category;

/**
 * Representa os dados da categoria na rvore.
 * 
 */
public class CategoryData extends Category implements
  DataNodeInterface<CategoryData, AlgorithmData> {

  /**
   * Nome completo da categoria pai, necessrio para fornecer o nome completo da
   * categoria, sem precisar navegar na rvore
   */
  private String parentFullName;

  /**
   * Constri os dados da categoria da rvore, a partir da categoria
   * especificada.
   * 
   * @param category categoria do algoritmo
   * @param parentFullName nome completo do pai dessa categoria
   */
  public CategoryData(Category category, String parentFullName) {
    super(category);
    this.parentFullName = parentFullName;
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.models.DataNodeInterface#getSubNodes()
   */
  @Override
  public SortedSet<CategoryData> getSubNodes() {
    SortedSet<CategoryData> subnodes = new TreeSet<CategoryData>();
    SortedSet<Category> categories = getCategories();
    for (Category category : categories) {
      subnodes.add(new CategoryData(category, getName()));
    }
    return subnodes;
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.models.DataNodeInterface#getDataChildren()
   */
  @Override
  public SortedSet<AlgorithmData> getDataChildren() {
    SortedSet<AlgorithmData> algorithms = new TreeSet<AlgorithmData>();
    Set<AlgorithmInfo> categoryAlgorithms = getAlgorithms();
    for (AlgorithmInfo algorithmInfo : categoryAlgorithms) {
      algorithms.add(new AlgorithmData(algorithmInfo));
    }
    return algorithms;
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.models.DataNodeInterface#isVisible()
   */
  @Override
  public boolean isVisible() {
    return true;
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.models.DataNodeInterface#compare(java.lang.Object)
   */
  @Override
  public int compare(CategoryData node) {
    return compareTo(node);
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.models.DataNodeInterface#getIcon()
   */
  @Override
  public Icon getIcon() {
    return ApplicationImages.ICON_FOLDER_16;
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.models.DataInterface#getFullName()
   */
  @Override
  public String getFullName() {
    return (parentFullName.isEmpty()) ? getName() : parentFullName + ":"
      + getName();
  }

  @Override
  public String toString() {
    return getFullName();
  }

}
