package csbase.client.applications.algorithmsmanager.models;

import java.util.SortedSet;

import javax.swing.Icon;

/**
 * Interface que modela os dados dos ns de uma rvore para gerncia de
 * categorias de algoritmos.
 * 
 * @param <N> classe do dado representado no n da rvore
 * @param <C> classe do filho dos ns da rvore
 * 
 */
public interface DataNodeInterface<N, C> extends DataInterface {

  /**
   * Obtm os sub-ns de um n, que sejam do mesmo tipo do n. O conjunto
   * retornado  imutvel. Caso no haja ns filhos, retornar um conjunto
   * vazio.
   * 
   * @return os sub-ns de um n ou um conjunto vazio
   */
  public SortedSet<N> getSubNodes();

  /**
   * Obtm os filhos de um n, que no sejam do mesmo tipo do n. O conjunto
   * retornado  imutvel. Caso no haja ns filhos, retornar um conjunto
   * vazio.
   * 
   * @return os filhos de um n ou um conjunto vazio
   */
  public SortedSet<C> getDataChildren();

  /**
   * Verifica se o n deve estar visvel na rvore de dados.
   * 
   * @return retorna true, se o n deve estar visvel, caso contrrio, retorna
   *         false
   */
  public boolean isVisible();

  /**
   * Compara um n com outro n especificado, segundo algum critrio definido
   * pelo n.
   * 
   * @param node n a ser comparado
   * 
   * @return o resultado da comparao entre os ns
   */
  public int compare(N node);

  /**
   * Obtm o cone do n a ser desenhado na rvore de dados.
   * 
   * @return o cone que representa o n
   */
  public Icon getIcon();
}
