package csbase.client.applications.algorithmsmanager.models;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.SortedSet;

import javax.swing.Icon;

import tecgraf.javautils.gui.tree.DefaultNode;
import csbase.client.applications.algorithmsmanager.dialogs.DataTreeView.ViewChildrenMode;

/**
 * Representa um n da rvore de dados. Cada n deve permitir exibir os filhos
 * associados a ele.
 * 
 * @param <N> classe do dado representado no n da rvore
 * @param <C> classe do filho dos ns da rvore
 * 
 */
public final class DataTreeNode<N extends DataNodeInterface<N, C>, C extends DataNodeInterface<C, N>>
  extends DefaultNode implements Comparable<DataTreeNode<N, C>> {
  /**
   * O n da rvore.
   */
  private N node;

  /**
   * Cria o n.
   * 
   * @param node n da rvore (No aceita {@code null})
   * @param viewMode modo de visualizao da rvore, que pode ser para exibir
   *        todos os filhos dos ns ou ocult-los
   */
  public DataTreeNode(N node, ViewChildrenMode viewMode) {
    super(node.getName());
    this.node = node;
    List<DataTreeNode<N, C>> treeNodes = new ArrayList<DataTreeNode<N, C>>();
    SortedSet<?> nodes = node.getSubNodes();
    for (Iterator<?> iterator = nodes.iterator(); iterator.hasNext();) {
      N childNode = (N) iterator.next();
      treeNodes.add(new DataTreeNode<N, C>(childNode, viewMode));
    }
    Collections.sort(treeNodes);

    // Adiciona as subcategorias que sempre devero ser exibidas
    List<DefaultNode> children = new LinkedList<DefaultNode>();
    children.addAll(treeNodes);

    if (viewMode == ViewChildrenMode.SHOW_CHILDREN) {
      // Adiciona os algoritmos como filhos
      List<DataTreeNode<C, N>> treeChildren =
        new ArrayList<DataTreeNode<C, N>>();
      Set<?> nodeChildren = node.getDataChildren();
      for (Iterator<?> iterator = nodeChildren.iterator(); iterator.hasNext();) {
        C child = (C) iterator.next();
        if (child.isVisible()) {
          DataTreeNode<C, N> childTreeNode =
            new DataTreeNode<C, N>(child, ViewChildrenMode.HIDE_CHILDREN);
          treeChildren.add(childTreeNode);
        }
      }
      Collections.sort(treeChildren);
      children.addAll(treeChildren);
    }

    for (DefaultNode child : children) {
      addChild(child);
    }
  }

  /**
   * (non-Javadoc)
   * 
   * @see java.lang.Comparable#compareTo(java.lang.Object)
   */
  public int compareTo(DataTreeNode<N, C> dataNode) {
    return getNode().compare(dataNode.getNode());
  }

  /**
   * (non-Javadoc)
   * 
   * @see tecgraf.javautils.gui.tree.DefaultNode#getIcon()
   */
  @Override
  public Icon getIcon() {
    return node.getIcon();
  }

  /**
   * Obtm o n da rvore de dados.
   * 
   * @return o n
   */
  public N getNode() {
    return node;
  }

}
