package csbase.client.applications.algorithmsmanager.models;

import java.util.SortedSet;

import tecgraf.javautils.gui.tree.DefaultNode;
import csbase.client.applications.algorithmsmanager.dialogs.DataTreeView.ViewChildrenMode;

/**
 * Representa o n raiz da rvore de dados, que efetivamente constri a rvore a
 * partir do seus ns e filhos.
 * 
 * @param <N> classe dos ns da rvore de dados
 * @param <C> classe dos filhos dos ns da rvore de dados
 */
public final class DataTreeRootNode<N extends DataNodeInterface<N, C>, C extends DataNodeInterface<C, N>>
  extends DefaultNode implements DataNodeInterface<N, C> {

  /** Texto que representa o n raiz da rvore de dados */
  private String rootLabel;

  /** Todos os ns do n raiz */
  private SortedSet<N> allNodes;

  /**
   * Constri a rvore de dados a partir do n raiz.
   * 
   * @param rootLabel texto a ser exibido no n raiz da rvore de dados
   * @param allNodes conjunto de ns da rvore de dados
   * @param viewMode modo de visualizao da rvore, que pode ser para exibir
   *        todos os filhos dos ns ou ocult-los
   */
  public DataTreeRootNode(String rootLabel, SortedSet<N> allNodes,
    ViewChildrenMode viewMode) {
    super(rootLabel, FiltrageMode.IGNORE);
    this.rootLabel = rootLabel;
    this.allNodes = allNodes;
    for (N node : allNodes) {
      addChild(new DataTreeNode<N, C>(node, viewMode));
    }
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.models.DataNodeInterface#compare(java.lang.Object)
   */
  @Override
  public int compare(N node) {
    return 0;
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.models.DataNodeInterface#getSubNodes()
   */
  @Override
  public SortedSet<N> getSubNodes() {
    return allNodes;
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.models.DataNodeInterface#isVisible()
   */
  @Override
  public boolean isVisible() {
    return true;
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.models.DataInterface#getFullName()
   */
  @Override
  public String getFullName() {
    return getName();
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.models.DataInterface#getName()
   */
  @Override
  public String getName() {
    return rootLabel;
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.models.DataNodeInterface#getDataChildren()
   */
  @Override
  public SortedSet<C> getDataChildren() {
    return null;
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.algorithmsmanager.models.DataInterface#getId()
   */
  @Override
  public String getId() {
    // return getName();
    return null;
  }
}