/*
 * $Id$
 */

package csbase.client.applications.commandsmonitor.table.column;

import java.rmi.RemoteException;
import java.util.Locale;

import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;

import csbase.client.algorithms.commands.cache.CommandsCache;
import csbase.client.applications.ApplicationImages;
import csbase.client.desktop.RemoteTask;
import csbase.logic.CommandInfo;
import csbase.logic.applicationservice.ApplicationRegistry;

/**
 * Coluna contendo a descrio do comando.
 * 
 * @author Tecgraf / PUC-Rio
 */
public class DescriptionColumn extends AbstractCommandsTableColumn {

  /**
   * Renderiza a coluna de descrio com um cone de um lpis no lado esquerdo,
   * indicando que a mesma  editvel.
   */
  private static final DefaultTableCellRenderer CELL_RENDERER;

  static {
    CELL_RENDERER = new DefaultTableCellRenderer();
    CELL_RENDERER.setIcon(ApplicationImages.ICON_EDITABLE_TABLE_CELL_14);
  }

  /**
   * Construtor.
   * 
   * @param id Identificador da coluna.
   * @param appRegistry registro da aplicao que ser utilizado para se obter o
   *        nome da coluna de acordo com o {@link Locale}.
   */
  public DescriptionColumn(String id, ApplicationRegistry appRegistry) {
    super(String.class, id, appRegistry, true);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Object getValue(CommandInfo cmd) {
    return cmd.getDescription();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void setValue(final CommandInfo cmd, Object value) {
    String previousDescription = cmd.getDescription();

    String description = value.toString();
    cmd.setDescription(0 == description.length() ? null : description);

    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      protected void performTask() throws RemoteException {
        CommandsCache.getInstance().updateDescription(cmd);
      }
    };
    String taskTitle = getGenericString("task.command.update.title");
    String taskMessage = getGenericString("task.command.update.message");
    boolean success = task.execute(null, taskTitle, taskMessage);
    if (!success) {
      cmd.setDescription(previousDescription);
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public TableCellRenderer createTableCellRenderer() {
    return CELL_RENDERER;
  }
}
