/*
 * $Id: FileExchangerAction.java 89383 2009-03-17 16:17:31Z juliana $
 */

package csbase.client.applications.fileexchanger.actions.core;

import java.awt.event.ActionEvent;

import javax.swing.Action;
import javax.swing.ImageIcon;

import csbase.client.applications.ApplicationAction;
import csbase.client.applications.ApplicationImages;
import csbase.client.applications.fileexchanger.FileExchanger;
import csbase.client.applications.fileexchanger.FileExchangerUI;

/**
 * Ao genrica do programa ProjectsManager
 * 
 * @author Tecgraf/PUC-Rio
 */
abstract public class FileExchangerAction extends ApplicationAction<FileExchanger> {

  /**
   * Consulta a aplicao
   * 
   * @return aplicao.
   */
  final public FileExchanger getFileExchanger() {
    return getApplication();
  }

  /**
   * Consulta se a ao possui uma imagem standard; se houver a imagem usada
   * aqui ser usada, seno valer a definida pelo mtodo
   * <code>getImageIcon()</code>.
   * 
   * @return uma imagem standard (ou null)
   */
  public ImageIcon getStandardImageIcon() {
    return null;
  }

  /**
   * Ajuste de proprieades default.
   */
  private void loadDefaultValues() {
    final String name = getName();
    if (name != null) {
      putValue(Action.NAME, name);
    }

    final Integer mnemonic = getMnemonic();
    if (mnemonic != null) {
      putValue(Action.MNEMONIC_KEY, mnemonic);
    }

    final ImageIcon icon = getImageIcon();
    if (icon != null) {
      putValue(Action.SMALL_ICON, icon);
    }

    final String tooltip = getToolTipText();
    if (tooltip != null) {
      putValue(Action.SHORT_DESCRIPTION, tooltip);
    }
  }

  /**
   * Mtodo de busca de uma string de internacionalizao dentro do escopo da
   * ao conforme o padro "<code>NomeDaClasse.TagDesejada</code>"
   * 
   * @param tag tag desejada
   * @return o texto de internacionalizao
   */
  final public String getString(final String tag) {
    return FileExchangerUI.getString(this.getClass(), tag);
  }
  
  /**
   * Mtodo de busca de uma string de internacionalizao dentro do escopo da
   * ao conforme o padro "<code>NomeDaClasse.TagDesejada</code>"
   * 
   * @param tag tag desejada
   * @return o texto de internacionalizao
   */
  final public boolean hasString(final String tag) {
    return FileExchangerUI.hasString(this.getClass(), tag);
  }

  /**
   * Mtodo de consulta ao texto de <i>tooltip</i> da ao montado no padro "<code>NomeDaClasse.NAME_SUFIX</code>"
   * 
   * @return o texto (nome) da ao.
   * @see #getString(String)
   */
  final public String getName() {
    final String TAG_NAME = "name";
    if (!hasString(TAG_NAME)) return null;
    final String name = getString(TAG_NAME);
    if (FileExchangerUI.isEmptyString(name)) return null;
    return name;
  }

  /**
   * Mtodo de consulta ao texto de <i>tooltip</i> da ao montado no padro "<code>NomeDaClasse.tooltip</code>"
   * 
   * @return o texto de tooltip
   * @see #getString(String)
   */
  final public String getToolTipText() {
    final String TAG_TOOLTIP = "tooltip";
    if (!hasString(TAG_TOOLTIP)) return null;
    final String tp = getString(TAG_TOOLTIP);
    if (FileExchangerUI.isEmptyString(tp)) return null;
    return tp;
  }

  /**
   * Consulta ao mnemnico da ao.
   * 
   * @return o mnemnico
   */
  final public Integer getMnemonic() {
    final String TAG_MNEMONIC = "mnemonic";
    if (!hasString(TAG_MNEMONIC)) return null;
    final String mn = getString(TAG_MNEMONIC);
    if (FileExchangerUI.isEmptyString(mn)) return null;
    return Integer.valueOf(mn.charAt(0));
  }

  /**
   * Mtodo de definio da imagem associada a ao.
   * 
   * @return um objeto do tipo <code>ImageIcon</code>.
   */
  final private ImageIcon getImageIcon() {
    final ImageIcon stdIcon = getStandardImageIcon();
    if (stdIcon != null) return stdIcon;

    final String IMG_ACTION_DIR = "actions";
    final String prefix = getClass().getSimpleName();
    final String name = prefix + ".16";
    final String pth = IMG_ACTION_DIR + "/" + name;
    // final String pth = name;
    
    final FileExchanger fileExchanger = getFileExchanger();
    final ImageIcon icon = fileExchanger.buildApplicationImage(pth);
    if (icon == ApplicationImages.ICON_NONE) return null;
    return icon;
  }

  /**
   * Resultado da ao encapsulada do Java.
   * 
   * @param ae evento Java
   * @see #actionDone()
   */
  @Override
  final public void actionPerformed(final ActionEvent ae) {
    try {
      actionDone();
    }
    catch (Exception e) {
      final FileExchanger fileExchanger = getFileExchanger();
      FileExchangerUI.handleException(fileExchanger, e);
    }
  }

  /**
   * @param fileExchanger A aplicao
   */
  public FileExchangerAction(final FileExchanger fileExchanger) {
    super(fileExchanger);
    loadDefaultValues();
  }

  /**
   * Ao efeitiva a ser determinada pelas sub-classes
   * 
   * @throws Exception se houver falha na operao.
   */
  abstract public void actionDone() throws Exception;
}

