package csbase.client.applications.fileexchanger.logic;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;

import csbase.logic.ClientProjectFile;
import csbase.logic.ClientProjectFileInputStream;
import csbase.logic.ProjectFileChannelLoadListener;

/**
 * @author Tecgraf/PUC-Rio
 */
public class ExchangeExportThread extends ExchangeThread {

  /**
   * {@inheritDoc}
   */
  @Override
  public void run() {
    final Exchange exchange = getExchange();
    final ExchangeMode transferMode = exchange.getTransferMode();
    switch (transferMode) {
      case STREAM:
        exportAsStream();
        break;
      case OPERATION:
        exportAsDownload();
        break;
    }
  }

  /**
   * Exportao como download.
   */
  private void exportAsDownload() {
    final Exchange exchange = getExchange();
    final int chunkSize = exchange.getBlockSize().getSize();
    try {
      final ClientProjectFile remoteFile = exchange.getRemoteFile();
      final File localFile = exchange.getLocalFile();

      remoteFile.download(localFile, chunkSize,
        new ProjectFileChannelLoadListener() {
          public void transferedBytes(long currentSize, long totalSize,
            long numBytes) {
            exchange.addCurrentTransferSize(numBytes);
            // System.out.println("" + numBytes + "...");
          }
        });
    }
    catch (Exception e) {
      exchange.signalEnded(e);
      return;
    }
    exchange.signalEnded(null);
  }

  /**
   * Exportao como stream.
   */
  private void exportAsStream() {
    final Exchange exchange = getExchange();
    final int bufferSize = exchange.getBlockSize().getSize();

    try {
      final ClientProjectFile remoteFile = exchange.getRemoteFile();
      final ClientProjectFileInputStream prjStream =
        new ClientProjectFileInputStream(remoteFile, bufferSize);
      final BufferedInputStream in = new BufferedInputStream(prjStream);
      BufferedOutputStream out = null;

      final File localFile = exchange.getLocalFile();
      final FileOutputStream fileStream = new FileOutputStream(localFile);
      out = new BufferedOutputStream(fileStream);
      byte[] buffer = new byte[bufferSize];
      int len = 0;
      while ((len = in.read(buffer)) > 0) {
        exchange.addCurrentTransferSize(len);
        out.write(buffer, 0, len);
      }
      out.flush();
      out.close();
      in.close();
    }
    catch (Exception e) {
      exchange.signalEnded(e);
      return;
    }
    exchange.signalEnded(null);
  }

  /**
   * Construtor
   * 
   * @param exchange operao
   */
  public ExchangeExportThread(final Exchange exchange) {
    super(exchange);
  }
}
