/*
 * Detalhes da ltima alterao:
 *
 * $Author: clinio $
 * $Date: 2008-07-02 16:39:27 -0300 (Wed, 02 Jul 2008) $
 * $Revision: 79484 $
 */
package csbase.client.applications.filetransfer;

import csbase.client.applications.Application;
import csbase.client.applications.ApplicationFrame;
import csbase.client.desktop.*;
import csbase.client.kernel.ClientException;
import csbase.client.project.ProjectTree;
import csbase.client.project.ProjectTreeFactory;
import csbase.client.project.ProjectTreePopupMenu;
import csbase.client.remote.ProjectAdminObserver;
import csbase.client.remote.ProjectObserver;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.CommonClientProject;
import csbase.logic.CommonProjectInfo;
import csbase.logic.ProjectUserEvent;
import csbase.logic.User;
import csbase.logic.UserProjectInfo;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.Observable;
import java.util.Observer;
import java.util.Set;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
 * A classe <code>FileTransfer</code> implementa o dilogo de  cpia de
 * arquivos entre projetos. No momento, as combos s mostram projetos do
 * usurio corrente e ProjectComboBox no est sendo notificada de
 * incluso/excluso do usurio em algum projeto de outro usurio.
 *
 * @author Ana Lodi
 * @version $Revision: 79484 $
 */
public class FileTransfer extends Application {
  /** Dilogo principal do visualizador. */
  private ApplicationFrame mainFrame = null;

  /** A rvore de diretrios do projeto origem */
  private ProjectTree sourceProjectTree;

  /** A rvore de diretrios do projeto destino */
  private ProjectTree targetProjectTree;

  /** Combo de Projeto origem */
  private ProjectComboBox sourceProjectCombo;

  /** Combo de Projeto destino */
  private ProjectComboBox targetProjectCombo;

  /** Projeto origem */
  private CommonClientProject sourceProject = null;

  /** Projeto destino */
  private CommonClientProject targetProject = null;

  /**
   * Exibe a janela
   */
  private void showFrame() {
    mainFrame.setSize(new Dimension(800, 600));
    mainFrame.center(DesktopFrame.getInstance().getDesktopFrame());
    mainFrame.setVisible(true);
  }

  /**
   * Cria a janela principal
   *
   * @return referncia para a janela criada
   */
  private ApplicationFrame createMainFrame() {
    mainFrame = getApplicationFrame();
    mainFrame.setTitle(getString("FILE_TRANSFER_TITLE"));
    Container cp = mainFrame.getContentPane();
    cp.setLayout(new BorderLayout());
    cp.add(createTextPanel(), BorderLayout.NORTH);
    cp.add(createTransferPanel(), BorderLayout.CENTER);
    cp.add(createButtonPanel(), BorderLayout.SOUTH);
    return mainFrame;
  }

  /**
   * Cria o painel de Transferncia
   *
   * @return .
   */
  private JPanel createTransferPanel() {
    JPanel tp = new JPanel(new GridLayout(1, 2));
    try {
      tp.add(createSourceTreePanel(mainFrame));
      tp.add(createTargetTreePanel(mainFrame));
    }
    catch (Exception e) {
      StandardErrorDialogs.showErrorDialog(mainFrame, mainFrame.getTitle(), e);
    }
    return tp;
  }

  /**
   * Cria o painel com o boto de Fechar
   *
   * @return .
   */
  private JPanel createButtonPanel() {
    JButton exitButton = new JButton(getString("FILE_TRANSFER_EXIT"));
    exitButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent ev) {
          close();
          finishApplication();
        }
      });

    JPanel bp = new JPanel(new FlowLayout());
    bp.add(exitButton);
    return bp;
  }

  /**
   * Cria o Painel com o Texto explicativo
   *
   * @return .
   */
  private JPanel createTextPanel() {
    JPanel tp = new JPanel(new BorderLayout());
    tp.setForeground(Color.GRAY);
    JLabel lb1 = new JLabel("  "+getString("FILE_TRANSFER_TEXT_MOVE"));
    JLabel lb2 = new JLabel("  "+getString("FILE_TRANSFER_TEXT_COPY"));
    lb1.setForeground(Color.GRAY);
    lb2.setForeground(Color.GRAY);
    tp.add(lb1, BorderLayout.NORTH);
    tp.add(lb2, BorderLayout.SOUTH);
    return tp;
  }

  /**
   * Cria o painel de projeto Origem.
   *
   * @param frame .
   *
   * @return o painel
   *
   * @throws ClientException .
   */
  private JComponent createSourceTreePanel(ApplicationFrame frame)
  throws ClientException {
    JPanel treePanel = new JPanel(new BorderLayout());
    treePanel.add(createSourceComboPanel(), BorderLayout.NORTH);
    try {
      // Criando a rvore de projeto origem   
      sourceProjectTree = createProjectTree(frame, sourceProject);
      treePanel.add(
        new JScrollPane(sourceProjectTree.getTree()), BorderLayout.CENTER);
    }
    catch (Exception e) {
      e.printStackTrace();
    }

    //  O Projeto Origem  o mesmo do projeto que est aberto 
    // (ou null, se no houver projeto aberto)
    CommonClientProject currentProject =
      DesktopFrame.getInstance().getProject();
    /* Se houver projeto aberto, ficar selecionado na combo */
    if (currentProject != null) {
      sourceProjectCombo.setSelectedItem(
        new UserProjectInfo(
          currentProject.getUserId(), currentProject.getName()));
    }
    return treePanel;
  }

  /**
   * Cria o painel de projeto Destino.
   *
   * @param frame .
   *
   * @return o painel
   *
   * @throws ClientException .
   */
  private JComponent createTargetTreePanel(ApplicationFrame frame)
  throws ClientException {
    JPanel treePanel = new JPanel(new BorderLayout());
    treePanel.add(createTargetComboPanel(), BorderLayout.NORTH);
    // Criando a rvore de projeto destino
    targetProjectTree = createProjectTree(frame, null);
    treePanel.add(
      new JScrollPane(targetProjectTree.getTree()), BorderLayout.CENTER);
    return treePanel;
  }

  /**
   * Cria a rvore de projeto.
   *
   * @param frame .
   * @param project .
   *
   * @return a ProjectTree
   *
   * @throws ClientException .
   */
  private ProjectTree createProjectTree(
    ApplicationFrame frame, CommonClientProject project)
  throws ClientException {
    ProjectTree projectTree;
    if (project != null) {
      projectTree = ProjectTreeFactory.getInstance()
        .makeProjectTree(frame, frame.getTitle(), project);
    }
    else {
      projectTree = ProjectTreeFactory.getInstance()
        .makeProjectTree(frame, frame.getTitle());
    }
    projectTree.enableRenameAction(false);
    projectTree.enableDeleteAction(false);
    /* Habilitando menu popup */
    projectTree.addMouseListener();

    /* Adaptando PopupMenu Root */
    ProjectTreePopupMenu popupRoot =
      (ProjectTreePopupMenu)projectTree.getPopupRoot();
    popupRoot.removeItem(getString("PRJ_DOWNLOAD"));
    popupRoot.removeItem(getString("PRJ_UPLOAD_FILE"));

    /* Adaptando PopupMenu Dir */
    ProjectTreePopupMenu popupDir =
      (ProjectTreePopupMenu)projectTree.getPopupDir();
    popupDir.removeItem(getString("PRJ_DOWNLOAD"));
    popupDir.removeItem(getString("PRJ_UPLOAD_FILE"));

    /* Adaptando PopupMenu File */
    ProjectTreePopupMenu popupFile =
      (ProjectTreePopupMenu)projectTree.getPopupFile();
    popupFile.removeItem(getString("PRJ_MANAGE_PUBLICATION"));
    popupFile.removeItem(getString("PRJ_CHANGE_TYPE"));
    popupFile.removeItem(getString("PRJ_DOWNLOAD_FILE"));

    /* Adaptando PopupMenu MultSelection */
    ProjectTreePopupMenu popupMult =
      (ProjectTreePopupMenu)projectTree.getPopupMultSelection();
    popupMult.removeItem(getString("PRJ_MANAGE_PUBLICATION"));
    popupMult.removeItem(getString("PRJ_CHANGE_TYPE"));
    return projectTree;
  }

  /*
   *  Cria o Painel com a combo de seleo de projetos Origem
   */
  private JComponent createSourceComboPanel() {
    JPanel comboPanel = new JPanel(new FlowLayout());
    // Criando combo de Projeto Origem
    sourceProjectCombo = new ProjectComboBox(mainFrame);
    sourceProjectCombo.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent ae) {
          sourceProject = openProjectAction(
              sourceProject, sourceProjectCombo, sourceProjectTree);
        }
      });
    comboPanel.add(new JLabel(getString("FILE_TRANSFER_PROJECT_SOURCE")));
    comboPanel.add(sourceProjectCombo);
    return comboPanel;
  }

  /*
   *  Cria o Painel com a combo de seleo de projetos Destino
   */
  private JComponent createTargetComboPanel() {
    JPanel comboPanel = new JPanel(new FlowLayout());
    // Criando combo de Projeto Destino
    targetProjectCombo = new ProjectComboBox(mainFrame);
    targetProjectCombo.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent ae) {
          targetProject = openProjectAction(
              targetProject, targetProjectCombo, targetProjectTree);
        }
      });
    comboPanel.add(new JLabel(getString("FILE_TRANSFER_PROJECT_TARGET")));
    comboPanel.add(targetProjectCombo);
    return comboPanel;
  }

  /*
   * Abre o projeto selecionado na combo e atualiza a rvore.
   * @return  o projeto aberto ou null em caso de erro.
   */
  private CommonClientProject openProjectAction(
    final CommonClientProject project, final ProjectComboBox projectCombo,
    final ProjectTree projectTree) {
    if (projectCombo.getSelectedItem() == null) {
      return null;
    }
    RemoteTask<CommonClientProject> task =
      new RemoteTask<CommonClientProject>() {
        @Override
        public void performTask()
        throws Exception {
          // Desinstalando o observador do projeto selecionado anteriormente
          if ((project != null)) {
            project.uninstallObserver();
          }

          // Abrindo o projeto correspondente ao item selecionado da combo.
          UserProjectInfo project =
            (UserProjectInfo)projectCombo.getSelectedItem();
          CommonClientProject newProject = CommonClientProject.openProject(
            project.getUserId(), project.getProjectName(), true);
          setResult(newProject);
          // Instalando o observador desse projeto.
          newProject.installObserver(new ProjectObserver(newProject));
          projectTree.setProject(newProject);
        }
      };
    if (task.execute(
      mainFrame, getName(), getString("FILE_TRANSFER_WAIT_MESSAGE"))) {
      return task.getResult();
    }
    return null;
  }

  /**
   * Trmino ASAP da aplicao.
   */
  public void killApplication() {
    close();
  }

  /**
   * Procedimento de finalizao da aplicao.
   */
  private void close() {
    try {
      // Desinstalando os observadores dos projeto selecionados
      if (sourceProject != null) {
        sourceProject.uninstallObserver();
      }
      if (targetProject != null) {
        targetProject.uninstallObserver();
      }
      // Removendo as combos de ProjectAdminObserver
      sourceProjectCombo.deleteObserver();
      targetProjectCombo.deleteObserver();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Mtodo que permite sempre ao usurio matar a aplicao.
   *
   * @return o flag indicativo.
   */
  public boolean userCanKillApplication() {
    return (true);
  }

  /**
   * Constri e exibe a frame de Transferncia de arquivos.
   *
   * @param id .
   */
  public FileTransfer(String id) {
    super(id);
    mainFrame = createMainFrame();
    showFrame();
  }

  /**
   * Classe que representa uma combo para seleo de projetos do usurio
   * corrente.  Se cadastra como observadora de ProjectAdminObserver para se
   * atualizar caso algum projeto seja removido ou criado.
   */
  private final class ProjectComboBox extends JComboBox implements Observer {
    /** Janela da aplicao */
    private JFrame mainFrame;

    /** Identificador do usurio corrente */
    private Object userId;

    public ProjectComboBox(JFrame mainFrame) {
      this.mainFrame = mainFrame;
      userId = User.getLoggedUser().getId();
      /* Carregando os projetos do usurio */
      loadProjects();
      setSelectedIndex(-1);
      /* Se cadastrando como observador de projetos em ProjectAdminObserver */
      ProjectAdminObserver.getInstance().addObserver(this);
    }

    /*
     * Atualiza a projectCombo com os projetos existentes para o usurio
     */
    private void loadProjects() {
      try {
        Set<UserProjectInfo> projects =
          CommonClientProject.getAllProjectsInfo(userId);
        setModel(
          new DefaultComboBoxModel(projects.toArray(new UserProjectInfo[0])));
      }
      catch (Exception e) {
        StandardErrorDialogs.showErrorDialog(mainFrame, mainFrame.getTitle(), e);
      }
    }

    /**
     * Chamado por notityObservers de ProjectAdminObserver  quando um projeto 
     * criado ou removido no servidor. arg ProjectUserEvent
     *
     * @param o .
     * @param arg .
     */
    public void update(Observable o, Object arg) {
      if (!(arg instanceof ProjectUserEvent)) {
        return;
      }
      ProjectUserEvent event = (ProjectUserEvent)arg;
      CommonProjectInfo info = (CommonProjectInfo)event.item;
      switch (event.type) {
        case ProjectUserEvent.CREATE:
          addItem(info);
          break;
        case ProjectUserEvent.DELETE:
          if ((getSelectedItem()).equals(info.name)) {
            setSelectedIndex(-1);
          }
          removeItem(info);
          break;
      }
    }

    /**
     * Remove a combo como observadora de ProjectAdminObserver
     */
    public void deleteObserver() {
      ProjectAdminObserver.getInstance().deleteObserver(this);
    }
  }
}
