package csbase.client.applications.projectsmanager;

import java.util.Observable;

import csbase.client.applications.projectsmanager.models.ProjectSpaceAllocation;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;
import csbase.client.remote.ProjectAdminObserver;
import csbase.logic.CommonProjectInfo;
import csbase.logic.ProjectUserEvent;

/**
 * Esta classe observa o servio de administrao de projetos, invocando mtodos
 * da aplicao ProjectsManager quando necessrio, para manter a lista de
 * projetos da aplicao coerente com o sistema.
 * 
 * @author jnlopes
 * 
 */
public class ProjectsCreationRemovalObserver extends
  AbstractProjectsManagerObserver {

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   */
  public ProjectsCreationRemovalObserver(final ProjectsManager projectsManager) {
    super(projectsManager);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void update(Observable o, Object arg) {

    if (o == null) {
      return;
    }

    if (!(o instanceof ProjectAdminObserver)
      || !(arg instanceof ProjectUserEvent)) {
      return;
    }

    final ProjectUserEvent event = (ProjectUserEvent) arg;
    if (!(event.item instanceof CommonProjectInfo)) {
      return;
    }

    final CommonProjectInfo cpi = (CommonProjectInfo) event.item;
    final ProjectsManagerData pmd = projectsManager.getProject(cpi);

    if (event.type == ProjectUserEvent.CREATE) {
      if (pmd != null) {
        /*
         * Nota - Por se tratar de um evento de criao de projeto, e dois
         * projetos de mesmo nome e owner no podem existir, esta condio se
         * limita aos casos onde um projeto  criado pelo prprio
         * projectsmanager, e em seguida, este mesmo evento  capturado aqui.
         */
        return;
      }
      addProject(cpi);
    }
    else if (event.type == ProjectUserEvent.DELETE) {
      if (pmd != null) {
        final ProjectSpaceAllocation psa = pmd.getProjectSpaceAllocation();
        if (psa != ProjectSpaceAllocation.WAITING_DEALLOCATION
          && psa != ProjectSpaceAllocation.ALLOCATED) {
          projectsManager.removeProject(cpi);
        }
      }
    }
    projectsManager.refreshProjectsTable();
  }
}
