/*
 * $Id$
 */

package csbase.client.applications.projectsmanager.actions;

import javax.swing.ImageIcon;

import csbase.client.applications.ApplicationImages;
import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.actions.core.ProjectsManagerAction;
import csbase.client.applications.projectsmanager.dialogs.ProjectCreationDialog;
import csbase.client.applications.projectsmanager.proxy.LocateMissingAllocatedProjectsTask;
import csbase.logic.CommonClientProject;

/**
 * Ao que abre o dilogo para criao de projetos.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class OpenProjectCreationDialogAction extends ProjectsManagerAction {

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   */
  public OpenProjectCreationDialogAction(final ProjectsManager projectsManager) {
    super(projectsManager);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionDone() throws Exception {
    final ProjectsManager projectsManager = getProjectsManager();
    final CommonClientProject ccp =
      ProjectCreationDialog.createProject(projectsManager);

    if (ccp == null) {
      if (projectsManager.isCurrentUserAdmin()) {
        final LocateMissingAllocatedProjectsTask lmapt =
          new LocateMissingAllocatedProjectsTask(projectsManager);

        final String title = getString("title");
        final String msg = getString("message");
        lmapt.execute(projectsManager.getApplicationFrame(), title, msg);

        if (lmapt.wasCancelled()) {
          final String err = getString("cancelled.message");
          lmapt.showError(err);
          return;
        }

        if (lmapt.getStatus() != true) {
          final Exception exception = lmapt.getError();
          throw exception;
        }

        final boolean result = lmapt.getResult();
        if (!result) {
          lmapt.showError(getString("project.creation.error.message"));
        }
      }
    }
    else {
      /*
       * TODO selecionar o projeto recm-criado
       */
      //      final ProjectsManagerData pmd = projectsManager.getProject(ccp);
      //      if (pmd != null) {
      //        projectsManager.setProjectsList(new ProjectsManagerData[] { pmd });
      //      }
    }

    projectsManager.refreshProjectsTable();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ImageIcon getStandardImageIcon() {
    return ApplicationImages.ICON_ADD_16;
  }

}
