package csbase.client.applications.projectsmanager.actions;

import java.util.List;

import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.ProjectsManagerUI;
import csbase.client.applications.projectsmanager.actions.core.ProjectsManagerAction;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;
import csbase.client.applications.projectsmanager.proxy.SingleProjectSpaceTask;

/**
 * Ao responsvel por solicitar clculo do espao ocupado pelos projetos
 * selecionados.
 * 
 * @author jnlopes
 * 
 */
public class SingleProjectSpaceAction extends ProjectsManagerAction {

  /**
   * Array de projetos.
   */
  private List<ProjectsManagerData> projectsManagerData;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectsManagerData Array de projetos.
   */
  public SingleProjectSpaceAction(final ProjectsManager projectsManager,
    final List<ProjectsManagerData> projectsManagerData) {
    super(projectsManager);
    this.projectsManagerData = projectsManagerData;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionDone() throws Exception {
    final ProjectsManager projectsManager = this.getProjectsManager();

    for (ProjectsManagerData pmd : projectsManagerData) {

      final String projectName = pmd.getProjectName();
      final String projectOwner = pmd.getOwnerId();

      final SingleProjectSpaceTask spst =
        new SingleProjectSpaceTask(projectsManager, projectName, projectOwner);

      spst.execute(projectsManager.getApplicationFrame(), getString("title"),
        getString("message"));
      if (spst.wasCancelled()) {
        final String err = getString("cancelled.message");
        spst.showError(err);
        return;
      }

      if (spst.getStatus() != true) {
        final Exception exception = spst.getError();
        ProjectsManagerUI.handleException(projectsManager, exception);
        return;
      }

      final double occupiedSpace = spst.getResult();

      if (occupiedSpace == -1) {
        final String errMsg = getString("occupied.space.error.message");
        final String errTitle = getString("occupied.space.error.title");
        StandardDialogs.showErrorDialog(projectsManager.getApplicationFrame(),
          errTitle, errMsg);
        return;
      }

      if (occupiedSpace != pmd.getOccupiedSpace()) {
        pmd.setOccupiedSpace(occupiedSpace);
        projectsManager.updateProjectOccupiedSpace(pmd);
        projectsManager.refreshProjectsTable();
      }
    }
  }

}
