package csbase.client.applications.projectsmanager.dialogs;

import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import tecgraf.javautils.gui.BorderUtil;
import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.GUIUtils;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.applications.ApplicationFrame;
import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.dialogs.core.ProjectsManagerDialog;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;
import csbase.client.applications.projectsmanager.panels.ProjectSharingInfoPanel;
import csbase.client.applications.projectsmanager.proxy.AddPersonalHistoryBlockTask;

/**
 * Dilogo de incluso de comentrios no histrico de projetos.
 * 
 * @author jnlopes
 */
public class ProjectHistoryDialog extends ProjectsManagerDialog {

  /**
   * Lista de projetos selecionados.
   */
  private List<ProjectsManagerData> projectsManagerData;

  /**
   * rea de texto onde o comentrio deve ser redigido.
   */
  private final JTextArea commentArea = new JTextArea();

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao.
   * @param projectsManagerData Lista de projetos selecionados.
   */
  public ProjectHistoryDialog(final ProjectsManager projectsManager,
    final List<ProjectsManagerData> projectsManagerData) {
    super(projectsManager);
    this.projectsManagerData = projectsManagerData;

    buildPanel();

    pack();
    center();
  }

  /**
   * Monta o painel com os botes.
   * 
   * @return o painel.
   */
  private JPanel buildButtonsPanel() {
    final JButton writeButton = new JButton();
    final JButton cancelButton = new JButton();

    final ProjectHistoryDialog phd = this;
    writeButton.setAction(new AbstractAction() {
      /**
       * {@inheritDoc}
       */
      @Override
      public void actionPerformed(ActionEvent e) {
        final ProjectsManager projectsManager = getProjectsManager();
        final String[] projectNames = new String[projectsManagerData.size()];
        final String[] ownerIds = new String[projectsManagerData.size()];

        int i = 0;
        for (ProjectsManagerData pmd : projectsManagerData) {
          projectNames[i] = pmd.getProjectName();
          ownerIds[i] = pmd.getOwnerId();
          i++;
        }

        final AddPersonalHistoryBlockTask aphbt =
          new AddPersonalHistoryBlockTask(projectsManager, projectNames,
            ownerIds, commentArea.getText());

        final ApplicationFrame appFrame = projectsManager.getApplicationFrame();
        final String taskTitle = getString("running.title");
        final String taskMsg = getString("running.message");
        aphbt.execute(appFrame, taskTitle, taskMsg);

        if (aphbt.wasCancelled()) {
          final String err = getString("cancelled.message");
          aphbt.showError(err);
          return;
        }

        if (aphbt.getStatus() != true) {
          final Exception exception = aphbt.getError();
          final String err = getString("comment.inclusion.error.message");
          final String msg = err + "\n" + exception.getMessage();
          StandardDialogs.showErrorDialog(appFrame, "", msg);
        }

        final String[] ret = aphbt.getResult();
        if (ret != null) {
          final StringBuilder projects = new StringBuilder();
          for (String str : ret) {
            projects.append(" - " + str + "\n");
          }
          final String err =
            getString("failure.inserting.some.comment.message");
          final String msg = err + "\n" + projects.toString();
          StandardDialogs.showWarningDialog(appFrame, "", msg);
        }

        phd.dispose();
      }

    });
    writeButton.setText(getString("insert.text"));

    cancelButton.setAction(new AbstractAction() {
      public void actionPerformed(ActionEvent e) {
        phd.dispose();
      }
    });
    cancelButton.setText(getString("cancel.text"));

    final JComponent[] buttons = { writeButton, cancelButton };
    GUIUtils.matchPreferredSizes(buttons);

    final JPanel buttonsPanel = new JPanel();
    buttonsPanel.add(writeButton);
    buttonsPanel.add(cancelButton);
    return buttonsPanel;
  }

  /**
   * Monta o componente de insero do texto.
   * 
   * @return o componente
   */
  private JScrollPane buildTextPane() {
    commentArea.setRows(5);
    commentArea.setColumns(40);
    commentArea.setLineWrap(true);

    final JScrollPane pane = new JScrollPane(commentArea);
    BorderUtil.setTitledBorder(pane, getString("comment.area.title"));
    return pane;
  }

  /**
   * Monta o componente dos projetos.
   * 
   * @return o componente
   */
  private ProjectSharingInfoPanel buildProjectPanel() {
    final ProjectsManager projectsManager = getProjectsManager();
    final ProjectSharingInfoPanel projectsPanel =
      new ProjectSharingInfoPanel(projectsManager, projectsManagerData, false);
    BorderUtil.setTitledBorder(projectsPanel, getString("projects.title"));
    return projectsPanel;
  }

  /**
   * Monta o painel.
   */
  private void buildPanel() {
    final JPanel buttonsPanel = buildButtonsPanel();
    final JScrollPane pane = buildTextPane();
    final ProjectSharingInfoPanel projectsPanel = buildProjectPanel();

    setLayout(new GridBagLayout());
    add(projectsPanel, new GBC(0, 0).horizontal());
    add(pane, new GBC(0, 1).both());
    add(buttonsPanel, new GBC(0, 2).horizontal());
  }
}
