/**
 * $Id: SimpleProjectTable.java 94617 2009-08-04 21:34:56Z jnlopes $
 */
package csbase.client.applications.projectsmanager.dialogs;

import java.util.Set;

import javax.swing.ListSelectionModel;

import tecgraf.javautils.gui.table.SortableTable;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;

/**
 * Tabela para exibio de uma lista de projetos.
 * 
 * @author Tecgraf
 */
class SimpleProjectTable extends SortableTable {

  /**
   * Construtor.
   * 
   * @param model - modelo para a tabela
   */
  public SimpleProjectTable(final ProjectListTableModel model) {
    super(model);
    setRowSelectionAllowed(true);
    setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    // preenche todo o espao vertical do scroll pane
    setFillsViewportHeight(true);
    setUseDisabledLook(true);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void setEnabled(final boolean enabled) {
    int rowCount = getRowCount();
    if (!enabled && rowCount > 0) {
      /*
       * de-seleciona todas as linhas se a tabela no est habilitada
       */
      removeRowSelectionInterval(0, rowCount - 1);
    }
    super.setEnabled(enabled);
  }

  /**
   * Preenche um conjunto com os projetos selecionados para cpia da lista de
   * permisses.
   * 
   * @param selectedProjects - conjunto a ser preenchido.
   */
  public void fillWithSelectedProjects(
      final Set<ProjectsManagerData> selectedProjects) {
    final int[] selectedRows = getSelectedRows();
    final ProjectListTableModel model = (ProjectListTableModel) getModel();
    for (int row : selectedRows) {
      selectedProjects.add(model.getProject(convertRowIndexToModel(row)));
    }
  }
}
