package csbase.client.applications.projectsmanager.models;

/**
 * Estrutura que armazena informaes de ocupao de projetos na rea de
 * projetos.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class ProjectGeneralAreaOccupationData {

  /**
   * Espao livre na rea de projetos.
   */
  private double freeSpace;

  /**
   * Espao ocupado na rea de projetos.
   */
  private double occupiedSpace;

  /**
   * Construtor
   * 
   * @param freeSpace Espao livre na rea de projetos.
   * @param occupiedSpace Espao ocupado na rea de projetos.
   */
  public ProjectGeneralAreaOccupationData(final double freeSpace,
    final double occupiedSpace) {
    this.freeSpace = freeSpace;
    this.occupiedSpace = occupiedSpace;
  }

  /**
   * Retorna porcentagem de espao livre na rea de projetos.
   * 
   * @return Porcentagem de espao livre na rea de projetos.
   */
  public double getFreeSpacePercentage() {
    return freeSpace / (freeSpace + occupiedSpace);
  }

  /**
   * Retorna porcentagem de espao ocupado na rea de projetos.
   * 
   * @return Porcentagem de espao ocupado na rea de projetos.
   */
  public double getOccupiedSpacePercentage() {
    return occupiedSpace / (freeSpace + occupiedSpace);
  }

  /**
   * Retorna espao livre na rea de projetos.
   * 
   * @return Espao livre na rea de projetos.
   */
  public double getFreeSpace() {
    return freeSpace;
  }

  /**
   * Define espao livre na rea de projetos.
   * 
   * @param freeSpace Espao livre na rea de projetos.
   */
  public void setFreeSpace(final double freeSpace) {
    this.freeSpace = freeSpace;
  }

  /**
   * Retorna espao ocupado na rea de projetos.
   * 
   * @return Espao ocupado na rea de projetos.
   */
  public double getOccupiedSpace() {
    return occupiedSpace;
  }

  /**
   * Define espao ocupado na rea de projetos.
   * 
   * @param occupiedSpace Espao ocupado na rea de projetos.
   */
  public void setOccupiedSpace(final double occupiedSpace) {
    this.occupiedSpace = occupiedSpace;
  }

}
