/*
 * $Id$
 */

package csbase.client.applications.projectsmanager.models;

import javax.swing.ImageIcon;

import csbase.client.applications.projectsmanager.ProjectsManagerUI;

/**
 * Status de alocacao de projeto.
 * 
 * @author Tecgraf/PUC-Rio
 */
public enum ProjectSpaceAllocation {

  /**
   * Sem alocao (projeto comum)
   */
  NO_ALLOCATION(ProjectsManagerUI.NO_ALLOCATION),

  /**
   * Aguardando alocao de rea.
   */
  WAITING_ALLOCATION(ProjectsManagerUI.ALLOCATED_BLOCKED),

  /**
   * Projeto removido, aguardando desalocao de rea.
   */
  WAITING_DEALLOCATION(ProjectsManagerUI.WAITING_DEALLOCATION),

  /**
   * rea alocada.
   */
  ALLOCATED(ProjectsManagerUI.ALLOCATED_FREE);

  /**
   * cone referente ao status do projeto.
   */
  final private ImageIcon icon;

  /**
   * Construtor
   * 
   * @param icon icone
   */
  private ProjectSpaceAllocation(final ImageIcon icon) {
    this.icon = icon;
  }

  /**
   * Retorna o item da enumeracao correspondente ao espaco informado.
   * 
   * @param space espaco possivelmente alocado.
   * @return O item de enumeracao referente ao espaco informado.
   */
  final static public ProjectSpaceAllocation getItemFromSpace(final long space) {
    if (space == 0) {
      return NO_ALLOCATION;
    }

    if (space == -1) {
      return WAITING_ALLOCATION;
    }

    return ALLOCATED;
  }

  /**
   * Imagem do escopo.
   * 
   * @return a imagem
   */
  final public ImageIcon getIcon() {
    return icon;
  }

  /**
   * Mtodo de busca de uma string de internacionalizao dentro do escopo da
   * ao conforme o padro "<code>NomeDaClasse.TagDesejada</code>"
   * 
   * @param tag tag desejada
   * @return o texto de internacionalizao
   */
  final public String getString(final String tag) {
    return ProjectsManagerUI.getString(this.getClass(), tag);
  }

  /**
   * Mtodo de teste de uma string de internacionalizao dentro do escopo da
   * ao conforme o padro "<code>NomeDaClasse.TagDesejada</code>"
   * 
   * @param tag tag desejada
   * @return o texto de internacionalizao
   */
  final public boolean hasString(final String tag) {
    return ProjectsManagerUI.hasString(this.getClass(), tag);
  }

  /**
   * @return O identificador deste campo da enumerao.
   */
  public String getDescription() {
    final String tag = this.name() + ".description";
    if (hasString(tag))
      return getString(tag);
    return null;
  }

  /**
   * Consulta ao ttulo
   * 
   * @return o ttulo
   */
  final public String getTitle() {
    final String tag = this.name() + ".title";
    if (hasString(tag))
      return getString(tag);
    return null;
  }

  /**
   * Indica se o projeto pode ser aberto. Projetos que esto aguardando alocao
   * / desalocao de espao no podem ser abertos.
   * 
   * @return Boolean indicando se o projeto pode ser aberto.
   */
  public boolean isOpenable() {
    if (this == NO_ALLOCATION || this == ALLOCATED) {
      return true;
    }

    return false;
  }

}
