/*
 * $Id$
 */

package csbase.client.applications.projectsmanager.models;

import java.util.List;
import java.util.Set;

/**
 * Estrutura que armazena lista de projetos no padro ProjectsManager
 * (ProjectsManagerData) e uma lista de strings que indicam quais projetos no
 * puderam ser abertos.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class ProjectsInfoData {

  /**
   * Lista de projetos abertos com sucesso.
   */
  final private List<ProjectsManagerData> projectsData;

  /**
   * Lista de nomes dos projetos que no puderam ser abertos.
   */
  final private Set<String> failedProjectNames;

  /**
   * Construtor
   * 
   * @param projectsData Lista de projetos abertos com sucesso.
   * @param failedProjectNames Lista de nomes dos projetos que no puderam ser
   *        abertos.
   */
  public ProjectsInfoData(final List<ProjectsManagerData> projectsData,
    final Set<String> failedProjectNames) {
    this.projectsData = projectsData;
    this.failedProjectNames = failedProjectNames;
  }

  /**
   * Retorna a lista de projetos abertos com sucesso.
   * 
   * @return a lista
   */
  public List<ProjectsManagerData> getProjectsData() {
    return projectsData;
  }

  /**
   * Retorna a lista de projetos que no puderam ser abertos.
   * 
   * @return a lista
   */
  public Set<String> getFailedProjectNames() {
    return failedProjectNames;
  }

}
