package csbase.client.applications.projectsmanager.panels.adapters;

import java.awt.Point;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;

import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.actions.AllocateAreaAction;
import csbase.client.applications.projectsmanager.actions.DeallocateAreaAction;
import csbase.client.applications.projectsmanager.models.ProjectSpaceAllocation;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;

/**
 * Listener que permite ao usurio alterar a alocao de um projeto especfico
 * atravs de click com o boto direito sobre a linha da tabela (na coluna que
 * representa o status da alocao).
 * 
 * @author Tecgraf/PUC-Rio
 */
public class ProjectsTableAllocationMouseAdapter extends ProjectsManagerAdapter {

  /**
   * A tabela de projetos
   */
  final private JTable projectsTable;

  /**
   * O menu de alocao
   */
  final private JPopupMenu allocationMenu;

  /**
   * O item de alocao de rea no menu
   */
  private JMenuItem allocateAreaItem;

  /**
   * O item de desalocao de rea no menu
   */
  private JMenuItem freeAreaItem;

  /**
   * Lista de projetos
   */
  private ProjectsManagerData project;

  /**
   * Construtor
   * 
   * @param projectsManager a aplicao
   * @param projectsTable a tabela da processos.
   */
  public ProjectsTableAllocationMouseAdapter(
    final ProjectsManager projectsManager, final JTable projectsTable) {
    super(projectsManager);
    this.projectsTable = projectsTable;
    allocationMenu = new JPopupMenu();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void mouseClicked(MouseEvent e) {
    handleClick(e);
  }

  /**
   * Mtodo responsvel por exibir o PopupMenu quando o usurio clicar com o
   * terceiro boto sobre determinadas colunas.
   * 
   * @param e O evento.
   */
  public void handleClick(MouseEvent e) {
    if (e.getButton() == MouseEvent.BUTTON3) {
      final Point p = e.getPoint();
      final Point general = e.getPoint();//e.getLocationOnScreen();
      final int row = projectsTable.rowAtPoint(p);
      final int column = projectsTable.columnAtPoint(p);

      final ProjectsManager projectsManager = getProjectsManager();
      final List<ProjectsManagerData> pl = projectsManager.getVisibleProjects();
      project = pl.get(row);

      if (column == 1) {
        allocationMenu.removeAll();
        final List<ProjectsManagerData> projectList =
          new ArrayList<ProjectsManagerData>();
        projectList.add(project);

        allocateAreaItem =
          new JMenuItem(new AllocateAreaAction(projectsManager, projectList));
        freeAreaItem =
          new JMenuItem(new DeallocateAreaAction(projectsManager, projectList));

        allocateAreaItem.setText(getString("allocate.text"));
        freeAreaItem.setText(getString("deallocate.text"));

        final ProjectSpaceAllocation spaceAllocation =
          project.getProjectSpaceAllocation();

        if (spaceAllocation == ProjectSpaceAllocation.WAITING_DEALLOCATION) {
          allocateAreaItem.setEnabled(false);
        }
        else if (spaceAllocation == ProjectSpaceAllocation.WAITING_ALLOCATION) {
          freeAreaItem.setEnabled(false);
        }
        else {
          allocateAreaItem.setEnabled(false);
          freeAreaItem.setEnabled(false);
        }

        allocationMenu.add(allocateAreaItem);
        allocationMenu.add(freeAreaItem);

        allocationMenu.show(this.projectsTable, general.x, general.y);
      }
    }
  }

}
