package csbase.client.applications.projectsmanager.panels.adapters;

import java.awt.Point;
import java.awt.event.MouseEvent;
import java.util.List;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;

import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.actions.SingleProjectSpaceAction;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;

/**
 * Listener que controla a apresentao de um JPopupMenu para calcular o espao
 * ocupado por um nico projeto.
 * 
 * @author jnlopes
 * 
 */
public class ProjectsTableOccupationMouseAdapter extends ProjectsManagerAdapter {

  /**
   * Tabela de projetos
   */
  final private JTable projectsTable;

  /**
   * Popup onde usurio pode selecionar recalcular espao ocupado por um projeto
   * especfico.
   */
  final private JPopupMenu occupationCalculusMenu;

  /**
   * Item do menu descrito acima.
   */
  private JMenuItem occupationItem;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectsTable Tabela de projetos.
   */
  public ProjectsTableOccupationMouseAdapter(
    final ProjectsManager projectsManager, final JTable projectsTable) {
    super(projectsManager);
    this.projectsTable = projectsTable;

    occupationCalculusMenu = new JPopupMenu();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void mouseClicked(MouseEvent e) {
    handleClick(e);
  }

  /**
   * Mtodo responsvel por exibir o PopupMenu quando o usurio clicar com o
   * terceiro boto sobre determinadas colunas.
   * 
   * @param e O evento.
   */
  public void handleClick(MouseEvent e) {
    if (e.getButton() == MouseEvent.BUTTON3) {

      final Point p = e.getPoint();
      final int column = projectsTable.columnAtPoint(p);
      if (column == 5) {

        occupationCalculusMenu.removeAll();
        final ProjectsManager projectsManager = getProjectsManager();
        final List<ProjectsManagerData> pmds =
          projectsManager.getSelectedProjects();
        occupationItem =
          new JMenuItem(
            new SingleProjectSpaceAction(getProjectsManager(), pmds));
        occupationItem.setText(getString("calculate.occupation.text"));
        occupationCalculusMenu.add(occupationItem);
        occupationCalculusMenu.show(this.projectsTable, p.x, p.y);
        //occupationCalculusMenu.setLocation(general);
      }
    }
  }

}
