package csbase.client.applications.projectsmanager.panels.adapters;

import java.awt.Point;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;

import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.actions.SetProjectPrivateAction;
import csbase.client.applications.projectsmanager.actions.SetProjectPublicAction;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;
import csbase.client.applications.projectsmanager.models.ProjectsManagerScope;

/**
 * Listener que permite ao usurio alterar o compartilhamento de um projeto
 * especfico atravs de click com o boto direito sobre a linha da tabela (na
 * coluna que representa o status de compartilhamento).
 * 
 * @author jnlopes
 * 
 */
public class ProjectsTableSharingMouseAdapter extends ProjectsManagerAdapter {

  /**
   * Tabela de projetos
   */
  final private JTable projectsTable;

  /**
   * Lista de projetos
   */
  private List<ProjectsManagerData> projects;

  /**
   * Popup onde usurio pode redefinir o status de compartilhamento de um
   * projeto especfico.
   */
  final private JPopupMenu sharingMenu;

  /**
   * Item do menu descrito acima.
   */
  private JMenuItem publicItem;

  /**
   * Item do menu descrito acima.
   */
  private JMenuItem privateItem;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectsTable A tabela de projetos
   */
  public ProjectsTableSharingMouseAdapter(
    final ProjectsManager projectsManager, final JTable projectsTable) {
    super(projectsManager);
    this.projectsTable = projectsTable;
    sharingMenu = new JPopupMenu();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void mouseClicked(MouseEvent e) {
    handleClick(e);
  }

  /**
   * Mtodo responsvel por exibir o PopupMenu quando o usurio clicar com o
   * terceiro boto sobre determinadas colunas.
   * 
   * @param e O evento.
   */
  public void handleClick(MouseEvent e) {
    if (e.getButton() == MouseEvent.BUTTON3) {

      final Point p = e.getPoint();

      final Point general = e.getPoint();//e.getLocationOnScreen();
      final int row = projectsTable.rowAtPoint(p);
      final int column = projectsTable.columnAtPoint(p);

      if (column == 0) {

        final ProjectsManager projectsManager = getProjectsManager();
        projects = projectsManager.getVisibleProjects();
        sharingMenu.removeAll();

        final ProjectsManagerData project = projects.get(row);

        final List<ProjectsManagerData> pmds =
          new ArrayList<ProjectsManagerData>();
        pmds.add(project);

        privateItem =
          new JMenuItem(new SetProjectPrivateAction(projectsManager, pmds));

        /*
         * TODO - O terceiro parmetro (true) para o construtor de
         * SetProjectPublicAction foi setado desta forma porque esta classe no
         * est em uso. Caso contrrio, seria necessrio criar dois itens. Um
         * para ao que torna o projeto pblico RO e outra que torna o projeto
         * pblico RW.
         */
        publicItem =
          new JMenuItem(new SetProjectPublicAction(projectsManager, pmds, true));

        privateItem.setText(getString("set.private.text"));
        publicItem.setText(getString("set.public.text"));

        final ProjectsManagerScope scope = project.getScope();
        if (scope == ProjectsManagerScope.PRIVATE) {
          privateItem.setEnabled(false);
        }
        else if (scope == ProjectsManagerScope.PUBLIC) {
          publicItem.setEnabled(false);
        }

        sharingMenu.add(privateItem);
        sharingMenu.add(publicItem);
        sharingMenu.show(this.projectsTable, general.x, general.y);
      }
    }
  }

}
