package csbase.client.applications.projectsmanager.proxy;

import java.util.ArrayList;
import java.util.List;

import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.models.ProjectHistoryBlock;
import csbase.client.applications.projectsmanager.models.ProjectHistoryBlockType;
import csbase.client.applications.projectsmanager.proxy.core.ProjectsManagerTask;
import csbase.logic.CommonClientProject;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectServiceInterface;

/**
 * Task que trata a incluso de comentrios pessoais ao histrico de projetos.
 * 
 * @author jnlopes
 */
public class AddPersonalHistoryBlockTask extends ProjectsManagerTask<String[]> {

  /**
   * Nomes dos projetos.
   */
  final private String[] projectNames;

  /**
   * Lista dos ids dos donos dos projetos.
   */
  final private String[] ownerIds;

  /**
   * Comentrio a ser adicionado ao histrico.
   */
  final private String personalComment;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectNames Nomes dos projetos.
   * @param ownerIds ids dos donos dos projetos
   * @param personalComment Comentrio a ser adicionado ao histrico.
   */
  public AddPersonalHistoryBlockTask(final ProjectsManager projectsManager,
    final String[] projectNames, final String[] ownerIds,
    final String personalComment) {
    super(projectsManager);
    this.projectNames = projectNames;
    this.ownerIds = ownerIds;
    this.personalComment = personalComment;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void performTask() throws Exception {

    final List<String> list = new ArrayList<String>();

    final ProjectServiceInterface psi = ClientRemoteLocator.projectService;

    for (int i = 0; i < projectNames.length; i++) {
      final CommonClientProject ccp =
        psi.openProject(ownerIds[i], projectNames[i], false);

      final ProjectHistoryBlock phb =
        new ProjectHistoryBlock(ProjectHistoryBlockType.PERSONAL_TEXT);
      phb.setUserComment("\n" + this.personalComment);

      try {
        ccp.getTree().appendDescription(phb.getHistoryBlock());
      }
      catch (Exception e) {
        list.add(projectNames[i]);
        //e.printStackTrace();
      }
    }

    if (list.size() > 0) {
      setResult(list.toArray(new String[list.size()]));
    }
    else {
      setResult(null);
    }
  }

}
