package csbase.client.applications.projectsmanager.proxy;

import javax.swing.JOptionPane;

import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.ProjectsManagerUI;
import csbase.client.applications.projectsmanager.proxy.core.ProjectsManagerTask;
import csbase.client.util.StandardErrorDialogs;
import csbase.exception.BugException;
import csbase.exception.CSBaseRuntimeException;
import csbase.exception.PermissionException;
import csbase.exception.ServiceFailureException;
import csbase.exception.UnavailableServiceException;

/**
 * Remote task para ser utilizada na criao de projetos.
 * 
 * @param <R> Tipo do resultado da remote task.
 * 
 * @author jnlopes
 * 
 */
abstract public class CreateProjectRemoteTask<R> extends ProjectsManagerTask<R> {

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao.
   */
  public CreateProjectRemoteTask(final ProjectsManager projectsManager) {
    super(projectsManager);
  }

  /**
   * Tratamento para erros repassados pelo proxy do servio remoto. Este mtodo
   * impede a exibio do dilogo de erro caso a exceo lanada seja uma
   * ServiceFailureException. Como esta task s deve ser usada para a criao de
   * projetos, excees deste tipo se referem a tentativa de criao de projetos
   * cujo diretrio j existe ou no pode ser criado. Nestes dois casos, ao
   * invs do dilogo de erro, a mensagem  exibida em um JOptionPane.
   * 
   * @param cserror exceo que representa esse erro
   * 
   */
  @Override
  protected void handleServerError(CSBaseRuntimeException cserror) {
    String msg = cserror.getMessage();
    if (msg == null) {
      if (cserror instanceof UnavailableServiceException) {
        msg =
          ProjectsManagerUI.getString(CreateProjectRemoteTask.class,
            "msg.unavailable");
      }
      else if (cserror instanceof ServiceFailureException) {
        msg =
          ProjectsManagerUI.getString(CreateProjectRemoteTask.class,
            "msg.servicefailure");
      }
      else if (cserror instanceof PermissionException) {
        msg =
          ProjectsManagerUI.getString(CreateProjectRemoteTask.class,
            "msg.permissionfailure");
      }
      else if (cserror instanceof BugException) {
        msg =
          ProjectsManagerUI.getString(CreateProjectRemoteTask.class,
            "msg.serverbug");
      }
      else {
        msg =
          ProjectsManagerUI.getString(CreateProjectRemoteTask.class,
            "msg.unknownerror");
      }
    }
    else {
      if (cserror instanceof ServiceFailureException) {
        JOptionPane.showMessageDialog(this.parentWindow, msg, ProjectsManagerUI
          .getString(CreateProjectRemoteTask.class,
            "create.project.error.title"), JOptionPane.ERROR_MESSAGE);
        return;
      }
    }
    StandardErrorDialogs.showErrorDialog(parentWindow, taskTitle, msg, cserror,
      getAdditionalInfo());
  }
}
