package csbase.client.applications.projectsmanager.proxy;

import java.rmi.RemoteException;

import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.proxy.core.ProjectsManagerTask;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectServiceInterface;

/**
 * Remote task para remoo de projetos.
 * 
 * @author jnlopes
 * 
 */
public class ProjectRemovalTask extends ProjectsManagerTask<Boolean> {

  /**
   * Nome dos projetos.
   */
  final private String[] projectNames;

  /**
   * Ids dos donos dos projetos.
   */
  final private String[] ownerIds;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectNames Nomes dos projetos.
   * @param ownerIds Ids dos donos dos projetos.
   */
  public ProjectRemovalTask(final ProjectsManager projectsManager,
    final String[] projectNames, final String[] ownerIds) {
    super(projectsManager);
    this.projectNames = projectNames;
    this.ownerIds = ownerIds;

  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void performTask() {

    boolean res = true;
    final ProjectServiceInterface psi = ClientRemoteLocator.projectService;
    for (int i = 0; i < projectNames.length; i++) {
      try {
        psi.removeProject(ownerIds[i], projectNames[i]);
      }
      catch (RemoteException re) {
        re.printStackTrace();
        res = false;
      }
    }
    setResult(res);
  }

}
