package csbase.client.applications.projectsmanager.proxy;

import java.util.List;

import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;
import csbase.client.applications.projectsmanager.proxy.core.ProjectsManagerTask;
import csbase.exception.ServiceFailureException;
import csbase.logic.CommonClientProject;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectServiceInterface;

/**
 * Task que obtm usurios compartilhados dos projetos.
 * 
 * @author jnlopes
 * 
 */
public class ReloadSharedUsersTask extends ProjectsManagerTask<Boolean> {

  /**
   * Lista de projetos.
   */
  final private List<ProjectsManagerData> projectsManagerData;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao.
   * @param projectsManagerData lista de projetos.
   */
  public ReloadSharedUsersTask(final ProjectsManager projectsManager,
    final List<ProjectsManagerData> projectsManagerData) {
    super(projectsManager);
    this.projectsManagerData = projectsManagerData;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void performTask() throws Exception {

    final ProjectServiceInterface psi = ClientRemoteLocator.projectService;

    for (ProjectsManagerData pmd : projectsManagerData) {
      try {
        final CommonClientProject ccp =
          psi.openProject(pmd.getOwnerId(), pmd.getProjectName(), false);

        if (ccp != null) {
          pmd.setUsers(ccp.getUsersRO(), ccp.getUsersRW());
          ccp.close(false);
        }
      }
      catch (ServiceFailureException sfe) {
        continue;
      }
    }

    setResult(true);
  }
}
