package csbase.client.applications.projectsmanager.proxy;

import java.rmi.RemoteException;

import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.ProjectsManagerUI;
import csbase.client.applications.projectsmanager.models.ProjectHistoryBlockType;
import csbase.client.applications.projectsmanager.models.ProjectsManagerScope;
import csbase.client.applications.projectsmanager.proxy.core.ProjectsManagerTask;
import csbase.logic.CommonClientProject;
import csbase.logic.ProjectPermissions.SharingType;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectServiceInterface;

/**
 * Ao que altera o status de compartilhamento de projetos.
 * 
 * @author jnlopes
 * 
 */
public class SetProjectSharingStatsTask extends ProjectsManagerTask<Boolean> {

  /**
   * Nomes dos projetos
   */
  final private String[] projectNames;

  /**
   * Ids dos donos dos projetos.
   */
  final private String[] ownerIds;

  /**
   * Escopo para o qual os projetos sero alterados.
   */
  final private ProjectsManagerScope scope;

  /**
   * Tipo de compartilhamento.
   */
  private final SharingType accessType;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectNames Nomes dos projetos
   * @param ownerIds Ids dos donos dos projetos.
   * @param scope Escopo para o qual os projetos sero alterados.
   * @param accessType tipo do compartilhamento
   */
  public SetProjectSharingStatsTask(final ProjectsManager projectsManager,
    final String[] projectNames, final String[] ownerIds,
    final ProjectsManagerScope scope, final SharingType accessType) {
    super(projectsManager);
    this.projectNames = projectNames;
    this.ownerIds = ownerIds;
    this.scope = scope;
    this.accessType = accessType;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void performTask() throws RemoteException {

    final ProjectServiceInterface psi = ClientRemoteLocator.projectService;
    if (scope == ProjectsManagerScope.PRIVATE) {
      for (int i = 0; i < projectNames.length; i++) {
        final CommonClientProject ccp =
          psi.openProject(ownerIds[i], projectNames[i], false);
        if (ccp == null) {
          setResult(false);
          return;
        }

        ccp.makePrivate();

        /*
         * Tratamento de adio de mensagens no histrico.
         */
        ProjectsManagerUI.addAutomaticHistoryBlock(ccp,
          ProjectHistoryBlockType.PROJECT_PRIVATE);

        ccp.close(true);
      }

    }
    else if (scope == ProjectsManagerScope.PUBLIC) {
      for (int i = 0; i < projectNames.length; i++) {
        final CommonClientProject ccp =
          psi.openProject(ownerIds[i], projectNames[i], false);
        if (ccp == null) {
          continue;
        }
        ccp.makePublic(accessType == SharingType.ALL_RO);

        /*
         * Tratamento de adio de mensagens no histrico.
         */
        ProjectsManagerUI.addAutomaticHistoryBlock(ccp,
          ProjectHistoryBlockType.PROJECT_PUBLIC);

        ccp.close(true);
      }
    }
    setResult(true);
  }
}
