package csbase.client.applications.projectsmanager.proxy;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;

import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.proxy.core.ProjectsManagerTask;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectServiceInterface;

/**
 * Task para modificao do status de alocao de projetos.
 * 
 * @author jnlopes
 */
public class StatusModificationTask extends ProjectsManagerTask<String[]> {

  /**
   * Nomes dos projetos.
   */
  final private String[] projectNames;

  /**
   * Lista dos ids dos donos dos projetos.
   */
  final private String[] ownerIds;

  /**
   * Espao (em GB) que foi alocado ao projeto.
   */
  final private int allocatedSpace;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectNames Nomes dos projetos.
   * @param ownerIds ids dos donos dos projetos
   * @param allocatedSpace Espao (em GB) que foi alocado ao projeto.
   */
  public StatusModificationTask(final ProjectsManager projectsManager,
      final String[] projectNames, final String[] ownerIds,
      final int allocatedSpace) {
    super(projectsManager);
    this.projectNames = projectNames;
    this.ownerIds = ownerIds;
    this.allocatedSpace = allocatedSpace;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void performTask() throws Exception {

    final List<String> list = new ArrayList<String>();
    final ProjectServiceInterface psi = ClientRemoteLocator.projectService;

    for (int i = 0; i < projectNames.length; i++) {
      try {
        if (allocatedSpace <= 0) {
          psi.setAllocatedProjectAsUnallocated(ownerIds[i], projectNames[i]);
        } else {
          psi.setUnallocatedProjectAsAllocated(ownerIds[i], projectNames[i],
              allocatedSpace);
        }
      } catch (RemoteException re) {
        list.add(projectNames[i]);
      }
    }

    if (list.size() > 0) {
      setResult(list.toArray(new String[list.size()]));
    } else {
      setResult(null);
    }
  }

}
