/**
 * $Id$
 */

package csbase.client.applications.projectsmanager.proxy;

import java.rmi.RemoteException;
import java.util.List;

import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.ProjectsManagerUI;
import csbase.client.applications.projectsmanager.models.ProjectHistoryBlockType;
import csbase.client.applications.projectsmanager.proxy.core.ProjectsManagerTask;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.CommonClientProject;
import csbase.logic.User;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectServiceInterface;

/**
 * Task para remover um usurio de um projeto (do qual ele no  dono).
 * 
 * @author jnlopes
 * 
 */
public class UserRemovalFromProjectTask extends ProjectsManagerTask<Boolean> {

  /**
   * Nomes dos projetos.
   */
  final private List<String> projectNames;

  /**
   * Ids dos donos dos projetos.
   */
  final private List<String> ownerIds;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectNames nomes dos projetos
   * @param ownerIds ids dos donos dos projetos.
   */
  public UserRemovalFromProjectTask(final ProjectsManager projectsManager,
    final List<String> projectNames, final List<String> ownerIds) {
    super(projectsManager);

    this.projectNames = projectNames;
    this.ownerIds = ownerIds;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void performTask() throws Exception {

    boolean error = false;
    final ProjectsManager projectsManager = getProjectsManager();
    final User user = projectsManager.getUser();
    final Object currentUserID = user.getId();

    final ProjectServiceInterface psi = ClientRemoteLocator.projectService;

    final int size = projectNames.size();
    for (int i = 0; i < size; i++) {
      try {
        final CommonClientProject ccp =
          psi.openProject(ownerIds.get(i), projectNames.get(i), false);

        /*
         * Tratamento de adio de mensagens no histrico.
         */
        ProjectsManagerUI.addAutomaticHistoryBlock(ccp,
          ProjectHistoryBlockType.PROJECT_SELF_REMOVAL);

        if (!ccp.removeUser(currentUserID)) {
          error = true;
        }
        ccp.close(false);
      }
      catch (RemoteException e) {
        StandardErrorDialogs.showErrorDialog(projectsManager
          .getApplicationFrame(), e);
        error = true;
      }
    }
    projectsManager.refreshProjectsTable();
    setResult(!error);
  }
}