package csbase.client.applications.projectsynchronization;

import csbase.client.desktop.*;
import csbase.client.util.StandardErrorDialogs;
import csbase.exception.ServiceFailureException;
import csbase.logic.*;
import csbase.logic.applicationservice.ApplicationRegistry;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectSynchronizationServiceInterface;

import tecgraf.javautils.gui.wizard.History;
import tecgraf.javautils.gui.wizard.WasChangedItemListener;
import tecgraf.javautils.gui.wizard.WizardException;

import java.awt.Container;

import java.util.Iterator;
import java.util.List;

public class StepSelectionProjects extends StepProjectSynchronization {
  private ContainerSelectionProjects containerSelectionProjects;
  protected ProjectSynchronizationData data;
  private boolean sameProjectFlag;
  private List listProjectsServerA;
  private List listProjectsServerB;
  private ApplicationRegistry registry;
  private String error;

  public StepSelectionProjects(ApplicationRegistry registry) {
    this.registry = registry;
    containerSelectionProjects = new ContainerSelectionProjects(registry);
  }

  private int getSelectIndexProjectA() {
    return containerSelectionProjects.getProjectsSelectionA().getCbProjetos()
    .getSelectedIndex();
  }

  private int getSelectIndexProjectB() {
    return containerSelectionProjects.getProjectsSelectionB().getCbProjetos()
    .getSelectedIndex();
  }

  private String getSelectItemProjectA() {
    UserProjectInfo project =
      (UserProjectInfo) containerSelectionProjects.getProjectsSelectionA()
        .getCbProjetos().getSelectedItem();
    return (project != null) ? project.getProjectName() : null;
  }

  private String getSelectItemProjectB() {
    UserProjectInfo project =
      (UserProjectInfo) containerSelectionProjects.getProjectsSelectionB()
        .getCbProjetos().getSelectedItem();
    return (project != null) ? project.getProjectName() : null;
  }

  protected void enter(History history)
  throws WizardException {
    error = null;
    data = (ProjectSynchronizationData)history.get(null, "data");
    sameProjectFlag = ((Boolean)history.get(null, "sameProjectFlag")).booleanValue();
    containerSelectionProjects.getProjectsSelectionA().getCbProjetos()
    .removeAllItems();
    containerSelectionProjects.getProjectsSelectionB().getCbProjetos()
    .removeAllItems();
    RemoteTask<ProjectSynchronizationListProjects> task =
      new RemoteTask<ProjectSynchronizationListProjects>() {
        @Override
        protected void performTask() throws Exception {
          ProjectSynchronizationServiceInterface psi =
            ClientRemoteLocator.projectSynchronizationService;
          ProjectSynchronizationListProjects pslp = psi.buildListProjects(data);
          setResult(pslp);
        }

        @Override
        protected void handleError(Exception e) {
          if (e instanceof ServiceFailureException) {
            StandardErrorDialogs.showErrorDialog(null, "SYNC", e.getMessage());
          }
          else {
            super.handleError(e);
          }
        }
      };
    if (!task.execute(null, getTitle(),
      registry.getString("step2.getProjectsContents"), 0,
        RemoteTask.CANCEL_BUTTON)) {
      if (!task.wasCancelled()) {
        Exception re = task.getError();
        error = re.getMessage();
        history.addGlobal("error", error);
      }
      return;
    }
    ProjectSynchronizationListProjects pslp = task.getResult();
    listProjectsServerA = pslp.getProjectsServerA();
    listProjectsServerB = pslp.getProjectsServerB();
    Integer indexA = (Integer)history.get(null, "indexA");
    Integer indexB = (Integer)history.get(null, "indexB");
    populateComboBox(indexA, indexB);
    containerSelectionProjects.getProjectsSelectionA().getCbProjetos()
    .addItemListener(new WasChangedItemListener(this));
    containerSelectionProjects.getProjectsSelectionB().getCbProjetos()
    .addItemListener(new WasChangedItemListener(this));
  }

  private void populateComboBox(Integer indexA, Integer indexB) {
    for (Iterator i = listProjectsServerA.iterator(); i.hasNext();) {
      containerSelectionProjects.getProjectsSelectionA().getCbProjetos()
      .addItem(i.next());
    }
    for (Iterator i = listProjectsServerB.iterator(); i.hasNext();) {
      containerSelectionProjects.getProjectsSelectionB().getCbProjetos()
      .addItem(i.next());
    }
    int sizeA =
      containerSelectionProjects.getProjectsSelectionA().getCbProjetos()
      .getItemCount();
    int sizeB =
      containerSelectionProjects.getProjectsSelectionB().getCbProjetos()
      .getItemCount();
    if (indexA == -1 && sizeA > 0) {
      indexA = 0;
    }
    if (indexB == -1 && sizeB > 0) {
      indexB = 0;
    }
    containerSelectionProjects.getProjectsSelectionA().getCbProjetos()
    .setSelectedIndex(indexA);
    containerSelectionProjects.getProjectsSelectionB().getCbProjetos()
    .setSelectedIndex(indexB);
  }

  protected Container getContainer() {
    return containerSelectionProjects.getRootPanel();
  }

  protected String getInstruction() {
    if (error == null) {
      return registry.getString("instruction.step2");
    }
    return error;
  }

  protected Class getNext(History arg0) {
    return StepSelectionFilesA.class;
  }

  protected String getTitle() {
    return registry.getString("title.step2");
  }

  protected boolean isReady() {
    if ((getSelectIndexProjectA() > -1) && (getSelectIndexProjectB() > -1)) {
      if (sameProjectFlag) {
        if (
          getSelectItemProjectA().equalsIgnoreCase(getSelectItemProjectB()) == false) {
          return true;
        }
      }
      else {
        return true;
      }
    }
    return false;
  }

  protected void exit(History history) {
    if ((getSelectItemProjectA() != null) && (getSelectItemProjectB() != null)) {
      data.getUnitA().setProjectName(getSelectItemProjectA());
      data.getUnitB().setProjectName(getSelectItemProjectB());
     RemoteTask<ProjectSynchronizationData> task =
        new RemoteTask<ProjectSynchronizationData>() {
          @Override
          protected void performTask() throws Exception {
            ProjectSynchronizationServiceInterface psi =
              ClientRemoteLocator.projectSynchronizationService;
            ProjectSynchronizationData psd = psi.buildTransferMaps(data);
            setResult(psd);
          }
        };
      if (!task.execute(null, getTitle(),
        registry.getString("step2.buildTransferList"), 0,
          RemoteTask.CANCEL_BUTTON)) {
        if (!task.wasCancelled()) {
          Exception re = task.getError();
          history.addGlobal("error", re.getMessage());
        }
        return;
      }
      data = task.getResult();
      history.addGlobal("data", data);
      history.addGlobal("indexA", new Integer(getSelectIndexProjectA()));
      history.addGlobal("indexB", new Integer(getSelectIndexProjectB()));
      history.addGlobal("projectNameA", getSelectItemProjectA());
      history.addGlobal("projectNameB", getSelectItemProjectB());
    }
  }
}
