/*
 * ClusterInfoPanel.java
 * $Revision: 75203 $
 */
package csbase.client.applications.sgamonitor;

import java.awt.BorderLayout;

import javax.swing.JScrollPane;
import csbase.client.applications.Application;
import csbase.client.desktop.DesktopErrorEvent;
import csbase.client.facilities.monitoringtable.MonitoringTableEvent;
import csbase.client.facilities.monitoringtable.MonitoringTableObserver;
import csbase.client.facilities.sgatable.SGAClusterTable;
import csbase.logic.SGAInfo;
import csbase.logic.SGASet;

/**
 * A classe <code>StarUsagePanel</code> modela o painel de monitorao que
 * exibe as cargas dos ns de um cluster como uma estrela de carga.
 *
 * @author Ana
 */
class ClusterInfoPanel extends DynamicMonitorPanel implements MonitoringTableObserver {
  /** Aplicao */
  protected Application app;

  /** Elementos de Interface */
  private SGAClusterTable sgaTable;
  private DesktopErrorEvent errorEvent = null;

  /** Gerenciador das informaes visualizadas */
  private ClusterCPULoadInfoHandler infoHandler = null;

  /**
   * Termina a monitorao.
   */
  public void close() {
    sgaTable.stopThread();
  }

  /**
   * Recebe notificaes da SGAClusterTable toda vez que esta  atualizada, ou
   * na ocorrncia de algum erro.
   *
   * @param sgaTableEvent Um evento do tipo MonitoringTableEvent.
   */
  public void update(final MonitoringTableEvent sgaTableEvent) {
    int event = sgaTableEvent.event;
    if (event == MonitoringTableEvent.REFRESH) {
      if (errorEvent != null) {
        //resetError(errorEvent);
        errorEvent = null;
      }
    }
  }

  /**
   * Retorna o ttulo do painel.
   *
   * @return ttulo do painel
   */
  @Override
  public String getTitle() {
    return app.getString("MON_CLUSTER_INFO_TITLE");
  }

  /**
   * Atualiza as informaes dinmicas exibidas.
   *
   * @param dynamicData informaes do servidor
   */
  @Override
  public void updateDynamicInfo(Object dynamicData) { // XXX ???
    infoHandler.updateInfo(((SGASet)dynamicData).getAllInfo());
  }

  /**
   * Constri o painel de monitorao.
   *
   * @param app .
   * @param server .
   */
  public ClusterInfoPanel(Application app, SGASet server) {
    super();
    SGAInfo[] clusterData = server.getAllInfo();
    infoHandler = new ClusterCPULoadInfoHandler(clusterData); // XXX ???
    this.app = app;
    setLayout(new BorderLayout(5, 5));
    sgaTable = new SGAClusterTable(server);
    sgaTable.addObserver(this);
    add(new JScrollPane(sgaTable), BorderLayout.CENTER);
  }
}
