package csbase.client.applications.sgamonitor;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.MessageFormat;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.applications.Application;
import csbase.client.applications.ApplicationFrame;
import csbase.client.desktop.DesktopErrorEvent;
import csbase.client.desktop.DesktopFrame;
import csbase.client.facilities.commandtable.CommandMonitoringTable;
import csbase.client.facilities.commandtable.CommandSubmittedTable;
import csbase.client.facilities.monitoringtable.MonitoringTableEvent;
import csbase.client.facilities.monitoringtable.MonitoringTableObserver;
import csbase.client.remote.srvproxies.SGAProxy;
import csbase.client.remote.srvproxies.SchedulerProxy;
import csbase.client.util.ClientUtilities;
import csbase.logic.CommandInfo;

/**
 * Aplicao CommandMonitor.
 */
public class CommandMonitorFrame implements MonitorFrame,
  MonitoringTableObserver {
  private Application app;
  private boolean showSubmittedTable;
  /** Elementos de Interface */
  private ApplicationFrame mainFrame;
  private JButton cancelButton;
  private JButton detailsButton;
  private CommandMonitoringPanel commandMonitoringPanel;
  private CommandSubmittedPanel commandSubmittedPanel;
  private DesktopErrorEvent errorEvent = null;

  /**
   * Cria a janela de monitorao dos comandos.
   */
  private void createMainFrame() {
    mainFrame = app.getApplicationFrame();
    Container cp = mainFrame.getContentPane();
    cp.setLayout(new BorderLayout(5, 5));

    commandMonitoringPanel = new CommandMonitoringPanel(app);

    if (showSubmittedTable) {

      commandSubmittedPanel = new CommandSubmittedPanel(app);
      commandSubmittedPanel.getSubmittedTable().addObserver(this);
      JSplitPane commandsPanel =
        new JSplitPane(JSplitPane.VERTICAL_SPLIT, commandMonitoringPanel,
          commandSubmittedPanel);
      commandsPanel.setOneTouchExpandable(true);
      commandsPanel.setResizeWeight(0.5);
      cp.add(commandsPanel, BorderLayout.CENTER);
    }
    else {
      JPanel commandsPanel = new JPanel(new GridBagLayout());
      GridBagConstraints constraints = new GridBagConstraints();
      constraints.anchor = GridBagConstraints.WEST;
      constraints.insets = new Insets(2, 2, 0, 2);
      constraints.weightx = 1.0;
      constraints.weighty = 1.0;
      constraints.gridx = 1;
      constraints.gridy = 1;
      constraints.fill = GridBagConstraints.BOTH;
      commandsPanel.add(commandMonitoringPanel, constraints);
      cp.add(commandsPanel, BorderLayout.CENTER);
    }
    commandMonitoringPanel.getMonitoringTable().addObserver(this);

    cp.add(createButtons(), BorderLayout.SOUTH);
  }

  /**
   * Exibe a janela de monitorao.
   */
  private void display() {
    mainFrame.pack();
    mainFrame.center(DesktopFrame.getInstance().getDesktopFrame());
    mainFrame.setVisible(true);
  }

  /**
   * Termina a monitorao.
   */
  public void close() {
    commandMonitoringPanel.getMonitoringTable().stop();
    // A monitorao da fila pode est desabilitada
    if (commandSubmittedPanel != null) {
      commandSubmittedPanel.getSubmittedTable().stop();
    }
  }

  /**
   * Cria o painel de botes.
   * 
   * @return o painel de botes criado
   */
  private JPanel createButtons() {
    JPanel panel = new JPanel(new FlowLayout());
    JButton closeButton =
      new JButton(app.getString("CommandMonitorFrame.close"));
    closeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        close();
        app.finishApplication();
      }
    });
    cancelButton =
      new JButton(app.getString("CommandMonitorFrame.cancel.multiple"));
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        int[] executingRows =
          commandMonitoringPanel.getMonitoringTable().getSelectedRows();
        if (((executingRows.length == 0) && (!showSubmittedTable))
          || ((executingRows.length == 0) && (commandSubmittedPanel
            .getSubmittedTable().getSelectedRows().length == 0))) {
          return;
        }
        String msg;
        if (executingRows.length != 0) {
          CommandInfo[] selectedRows = new CommandInfo[executingRows.length];
          for (int i = 0; i < executingRows.length; i++) {
            int modelRow =
              commandMonitoringPanel.getMonitoringTable()
                .convertRowIndexToModel(executingRows[i]);
            selectedRows[i] =
              commandMonitoringPanel.getMonitoringTable().getCommandInfo(
                modelRow);
          }
          if (executingRows.length == 1) {
            msg = app.getString("CommandMonitorFrame.cancel.confirm");
          }
          else {
            msg = app.getString("CommandMonitorFrame.cancel.multiple.confirm");
          }
          for (CommandInfo cmdInfo : selectedRows) {
            msg =
              msg
                + "\n"
                + MessageFormat.format(app
                  .getString("CommandMonitorFrame.cancel.algorithm"), cmdInfo
                  .getTip())
                + MessageFormat.format(app
                  .getString("CommandMonitorFrame.cancel.manualserver"),
                  cmdInfo.getSGAName());
            String descr = cmdInfo.getDescription();
            if (descr != null && descr.length() > 0) {
              msg =
                msg
                  + MessageFormat
                    .format(app
                      .getString("CommandMonitorFrame.cancel.description"),
                      descr);
            }
          }

          // Obtm confirmao para cancelar o(s) comando(s).
          int res =
            StandardDialogs.showYesNoDialog(mainFrame, app.getName(), msg);
          if (res == JOptionPane.YES_OPTION) {
            for (int i = 0; i < selectedRows.length; i++) {
              SGAProxy.killCommand(selectedRows[i].getSGAName(),
                selectedRows[i].getId());
            }
          }
        }
        else {
          int[] submittedRows =
            commandSubmittedPanel.getSubmittedTable().getSelectedRows();
          CommandInfo[] selectedRows = new CommandInfo[submittedRows.length];
          for (int i = 0; i < submittedRows.length; i++) {
            int modelRow =
              commandSubmittedPanel.getSubmittedTable().convertRowIndexToModel(
                submittedRows[i]);
            selectedRows[i] =
              commandSubmittedPanel.getSubmittedTable()
                .getCommandInfo(modelRow);
          }
          if (submittedRows.length == 1) {
            msg = app.getString("CommandMonitorFrame.cancel.confirm");
          }
          else {
            msg = app.getString("CommandMonitorFrame.cancel.multiple.confirm");
          }
          for (CommandInfo cmdInfo : selectedRows) {
            String sgaName = cmdInfo.getSGAName();
            msg =
              msg
                + "\n"
                + MessageFormat.format(app
                  .getString("CommandMonitorFrame.cancel.algorithm"), cmdInfo
                  .getTip());
            if (sgaName != null) {
              msg +=
                MessageFormat.format(app
                  .getString("CommandMonitorFrame.cancel.manualserver"),
                  sgaName);
            }
            else {
              msg += app.getString("CommandMonitorFrame.cancel.autoserver");
            }

            String descr = cmdInfo.getDescription();
            if (descr.length() > 0) {
              msg =
                msg
                  + MessageFormat
                    .format(app
                      .getString("CommandMonitorFrame.cancel.description"),
                      descr);
            }
          }

          // Obtm confirmao para cancelar o(s) comando(s).
          int res =
            StandardDialogs.showYesNoDialog(mainFrame, app.getName(), msg);
          if (res == JOptionPane.YES_OPTION) {
            for (int i = 0; i < selectedRows.length; i++) {
              SchedulerProxy.removeCommand(null, selectedRows[i].getId());
            }
          }
        }
      }
    });
    cancelButton.setEnabled(false);
    detailsButton = new JButton(app.getString("CommandMonitorFrame.details"));
    detailsButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        int selectedRow =
          commandMonitoringPanel.getMonitoringTable().getSelectedRow();
        int modelRow =
          commandMonitoringPanel.getMonitoringTable().convertRowIndexToModel(
            selectedRow);
        CommandInfo cmdInfo =
          commandMonitoringPanel.getMonitoringTable().getCommandInfo(modelRow);
        CommandDetailsFrame.showCommandDetailsFrame(cmdInfo.getProjectId(),
          cmdInfo.getId(), cmdInfo.getSGAName(), app);
      }
    });
    detailsButton.setEnabled(false);
    addSelectionListener();
    ClientUtilities.adjustEqualSizes(cancelButton, detailsButton);
    ClientUtilities.adjustEqualSizes(cancelButton, closeButton);
    panel.add(detailsButton);
    panel.add(cancelButton);
    panel.add(closeButton);
    return panel;
  }

  /**
   * Adiciona ouvidor para selecao de linhas, e habilita o botao de cancelar
   * execuo se estiver alguma linha em selecao, senao o botao e desabilitado.
   * Apenas uma tabela de comandos (em execuo ou submetidos) pode estar com
   * campos selecionados.
   */
  public void addSelectionListener() {
    ListSelectionModel selectModel =
      commandMonitoringPanel.getMonitoringTable().getSelectionModel();
    selectModel.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e) {
        int nRows =
          commandMonitoringPanel.getMonitoringTable().getSelectedRowCount();
        if (nRows > 0) {
          // Habilita o boto de detalhes apenas quando um nico
          // comando estiver selecionado.
          if (nRows == 1) {
            detailsButton.setEnabled(true);
          }
          else {
            detailsButton.setEnabled(false);
          }
          //?? HAbilitar o botao quando estiver funcionando no SGA
          cancelButton.setEnabled(true);
          if (showSubmittedTable)
            commandSubmittedPanel.getSubmittedTable().clearSelection();
        }
      }
    });
    if (showSubmittedTable) {
      ListSelectionModel selectSubmittedModel =
        commandSubmittedPanel.getSubmittedTable().getSelectionModel();
      selectSubmittedModel
        .addListSelectionListener(new ListSelectionListener() {
          public void valueChanged(ListSelectionEvent e) {
            if (commandSubmittedPanel.getSubmittedTable().getSelectedRowCount() > 0) {
              //?? HAbilitar o botao quando estiver funcionando no SGA
              cancelButton.setEnabled(true);
              // Habilita o boto de detalhes apenas quando algum comando da
              // tabela de monitorao estiver estiver selecionado.
              detailsButton.setEnabled(false);
              commandMonitoringPanel.getMonitoringTable().clearSelection();
            }
          }
        });
    }
  }

  /**
   * Recebe notificaes da CommandMonitoringTable toda vez que esta 
   * atualizada, ou na ocorrncia de algum erro.
   * 
   * @param cmdTableEvent Um evento do tipo MonitoringTableEvent.
   */
  public void update(MonitoringTableEvent cmdTableEvent) {
    int event = cmdTableEvent.event;
    final CommandMonitoringTable monTable =
      commandMonitoringPanel.getMonitoringTable();
    if (event == MonitoringTableEvent.REFRESH) {
      // Desabilita o boto Cancelar caso no existam linhas selecionadas
      if (showSubmittedTable) {
        final CommandSubmittedTable subTable =
          commandSubmittedPanel.getSubmittedTable();
        cancelButton.setEnabled(monTable.getSelectedRowCount() > 0
          || subTable.getSelectedRowCount() > 0);
      }
      else {
        cancelButton.setEnabled(monTable.getSelectedRowCount() > 0);
      }
      if (showSubmittedTable) {
        final CommandSubmittedTable subTable =
          commandSubmittedPanel.getSubmittedTable();
        detailsButton.setEnabled(monTable.getSelectedRowCount() == 1
          && subTable.getSelectedRowCount() == 0);
      }
      else {
        detailsButton.setEnabled(monTable.getSelectedRowCount() == 1);
      }
      if (errorEvent != null) {
        mainFrame.resetError(errorEvent);
        errorEvent = null;
      }
    }
    else if (event == MonitoringTableEvent.REMOTE_ERROR) {
      JComponent[] disabled;
      if (showSubmittedTable) {
        final CommandSubmittedTable subTable =
          commandSubmittedPanel.getSubmittedTable();
        disabled =
          new JComponent[] { monTable, subTable, detailsButton, cancelButton };
      }
      else {
        disabled = new JComponent[] { monTable, detailsButton, cancelButton, };
      }
      errorEvent =
        new DesktopErrorEvent(app
          .getString("CommandMonitorFrame.error.comm_server"), null, disabled);
      mainFrame.signalError(errorEvent);
    }
  }

  /**
   * Construtor padro.
   * 
   * @param app .
   * @param showSubmittedTable informa se a tabela de monitoramento dos comandos
   *        em espera por execuo deve ou no ser exibida.
   */
  public CommandMonitorFrame(Application app, boolean showSubmittedTable) {
    this.app = app;
    this.showSubmittedTable = showSubmittedTable;
    createMainFrame();
    display();
  }
}
