package csbase.client.applications.sgamonitor;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import csbase.client.applications.Application;
import csbase.client.facilities.commandtable.CommandMonitoringTable;

/**
 * Esta classe apresenta as tabelas de monitoramento de comandos em execuo.
 * 
 * @author valeria
 */
public class CommandMonitoringPanel extends JPanel {

  /**
   * A aplicao csbase.
   */
  private Application app;

  /**
   * Tabela de monitorao de comandos.
   */
  private CommandMonitoringTable cmdMonitoringTable;

  /**
   * Cria o painel para a apresentao das tabelas.
   * 
   * @param app - ref. para a aplicao.
   */
  public CommandMonitoringPanel(Application app) {
    super(new GridBagLayout());
    this.app = app;
    createGUIComponents();
  }

  /**
   * Cria os componentes da GUI.
   */
  private void createGUIComponents() {

    JLabel label =
      new JLabel(app.getString("CommandMonitoringPanel.executing.commands"));
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.anchor = GridBagConstraints.WEST;
    constraints.insets = new Insets(5, 5, 0, 5);
    constraints.gridx = 0;
    constraints.gridy = 0;
    constraints.fill = GridBagConstraints.BOTH;
    constraints.anchor = GridBagConstraints.WEST;
    add(label, constraints);

    constraints = new GridBagConstraints();
    constraints.anchor = GridBagConstraints.WEST;
    constraints.insets = new Insets(5, 5, 5, 5);
    constraints.gridx = 0;
    constraints.gridy = 1;
    constraints.weightx = 1.0;
    constraints.weighty = 1.0;
    constraints.fill = GridBagConstraints.BOTH;
    constraints.anchor = GridBagConstraints.WEST;

    // ver propriedade 'showProgress' ...
    boolean showProgress  = app.getBooleanSpecificProperty("showProgress", false);

    cmdMonitoringTable =
      new CommandMonitoringTable(
        ListSelectionModel.MULTIPLE_INTERVAL_SELECTION, showProgress);
    add(new JScrollPane(cmdMonitoringTable), constraints);
  }

  /**
   * Retorna a tabela de comandos em execuo.
   * 
   * @return a tabela.
   */
  public CommandMonitoringTable getMonitoringTable() {
    return cmdMonitoringTable;
  }
}
