package csbase.client.applications.sgamonitor;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import csbase.client.applications.Application;
import csbase.client.applications.ApplicationImages;
import csbase.client.desktop.RemoteTask;
import csbase.client.facilities.commandtable.CommandSubmittedTable;
import csbase.client.facilities.monitoringtable.MonitoringTableEvent;
import csbase.client.facilities.monitoringtable.MonitoringTableObserver;
import csbase.logic.User;
import csbase.remote.ClientRemoteLocator;

/**
 * Esta classe apresenta as tabelas de monitoramento de comandos em espera por 
 * execuo.
 * 
 * @author valeria
 */
public class CommandSubmittedPanel extends JPanel 
implements MonitoringTableObserver {
  private CommandSubmittedTable cmdSubmittedTable;
  private boolean queueBlocked;
  private final Application app;
  private JLabel queueStatusLabel;
  private JButton blockButton;
      
  /**
   * Cria o painel para a apresentao das tabelas.
   */
  public CommandSubmittedPanel(Application app) {
    super(new GridBagLayout());
    this.app = app;
    updateQueueStatus();
    createGUIComponents();
  }
  
  /**
   * Cria os componentes da GUI.
   */
  private void createGUIComponents() {
    JLabel label = new JLabel(
      app.getString("CommandSubmittedPanel.submitted.commands"));
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets = new Insets(5, 5, 0, 5);
    constraints.gridx = 0;
    constraints.gridy = 0;
    constraints.gridwidth = 2;
    constraints.fill = GridBagConstraints.BOTH;
    constraints.anchor = GridBagConstraints.WEST;
    add(label, constraints);
    
    constraints = new GridBagConstraints();
    constraints.insets = new Insets(5, 5, 5, 5);
    constraints.gridx = 0;
    constraints.gridy = 1;
    constraints.gridwidth = 2;
    constraints.weightx = 1.0;
    constraints.weighty = 1.0;
    constraints.fill = GridBagConstraints.BOTH;
    constraints.anchor = GridBagConstraints.WEST;
    
    cmdSubmittedTable = new CommandSubmittedTable(
      ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    cmdSubmittedTable.addObserver(this);
    add(new JScrollPane(cmdSubmittedTable), constraints);
    
    // Label para indicar bloqueio/desbloqueio
    queueStatusLabel = new JLabel(" ");
    queueStatusLabel.setHorizontalAlignment(JLabel.LEFT);
    Dimension d = queueStatusLabel.getPreferredSize();
    d.setSize(d.getWidth(), ApplicationImages.ICON_STOP_16.getIconHeight() + 14);
    queueStatusLabel.setPreferredSize(d);
    
    //if (queueBlocked) {
    //  queueStatusLabel.setText(app.getString("CommandSubmittedPanel.queue.blocked"));
    //} 
    constraints = new GridBagConstraints();
    constraints.insets = new Insets(5, 5, 5, 5);
    constraints.gridx = 0;
    constraints.gridy = 2;
    constraints.weightx = 1.0;
    constraints.weighty = 0;
    constraints.fill = GridBagConstraints.BOTH;
    constraints.anchor = GridBagConstraints.WEST;
    add(queueStatusLabel, constraints);
    
    blockButton = new JButton();
    blockButton.addActionListener(
      new ActionListener(){
        public void actionPerformed(ActionEvent e) {
          changeStatus();
        }
      });
    if (queueBlocked) {
      queueStatusLabel.setIcon(ApplicationImages.ICON_STOP_16);
      queueStatusLabel.setText(app.getString("CommandSubmittedPanel.queue.blocked"));
      blockButton.setText(app.getString("CommandSubmittedPanel.button.unblock"));
    } else {
      blockButton.setText(app.getString("CommandSubmittedPanel.button.block"));
    }
    constraints = new GridBagConstraints();
    constraints.insets = new Insets(5, 5, 5, 5);
    constraints.gridx = 1;
    constraints.gridy = 2;
    constraints.anchor = GridBagConstraints.EAST;
    add(blockButton, constraints);
    
    User loggedUser = User.getLoggedUser();
    if (!loggedUser.isAdmin()){
      blockButton.setVisible(false);
    }
  }
  
  /**
   * Altera os estado da fila. Se estiver bloqueada, desbloqueia.
   * Se estiver desbloqueada, bloqueia.
   */
  private void changeStatus(){
    final boolean newState = !queueBlocked;
    RemoteTask<Void> task = new RemoteTask<Void>(){

      @Override
      protected void performTask() throws Exception {
        ClientRemoteLocator.schedulerService.setBlocked(newState);
      }
      
    };
    boolean execute = task.execute(app.getApplicationFrame(), app.getName(),
      app.getString("CommandSubmittedPanel.msg.changeStatus"));
    if (execute){
      queueBlocked = newState;
      updateLabels();
    }
  }
  
  /**
   * Atualiza os labels.
   */
  private void updateLabels(){
    if (queueBlocked){
      queueStatusLabel.setIcon(ApplicationImages.ICON_STOP_16);
      queueStatusLabel.setText(
        app.getString("CommandSubmittedPanel.queue.blocked"));
      blockButton.setText(
        app.getString("CommandSubmittedPanel.button.unblock"));
    } else {
      queueStatusLabel.setIcon(null);
      queueStatusLabel.setText(" ");
      blockButton.setText(
        app.getString("CommandSubmittedPanel.button.block"));
    }
  }
  
  /**
   * Atualiza o status da fila: bloqueada ou desbloqueada.
   */
  private void updateQueueStatus(){
    RemoteTask<Boolean> task = new RemoteTask<Boolean>(){
      @Override
      protected void performTask() throws Exception {
        setResult(ClientRemoteLocator.schedulerService.isBlocked());
      }
    };
    boolean execute = task.execute(
      app.getApplicationFrame(), app.getName(), app.getString("CommandSubmittedPanel.msg.getStatus"));
    if (execute) {
      queueBlocked = task.getResult();
    }
  }
  
  /**
   * Retorna a tabela de comandos em espera por execuo.
   * 
   * @return a tabela.
   */
  public CommandSubmittedTable getSubmittedTable() {
    return cmdSubmittedTable;
  }

  /**
   * {@inheritDoc}
   */
  public void update(final MonitoringTableEvent cmdTableEvent) { 
    int event = cmdTableEvent.event;
    if (event == MonitoringTableEvent.REFRESH) {
      updateQueueStatus();
      updateLabels();
    }
  }  
}
