/*
 * GroupBarsUsagePanel.java $Revision: 65204 $
 */
package csbase.client.applications.sgamonitor;

import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.BorderFactory;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.CategoryAxis3D;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.NumberAxis3D;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.labels.StandardCategoryItemLabelGenerator;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.CategoryItemRenderer;
import org.jfree.data.category.DefaultCategoryDataset;

import csbase.client.applications.Application;
import csbase.client.util.MonitoringTheme;
import csbase.logic.SGAInfo;
import csbase.logic.SGASet;

/**
 * A classe <code>GroupBarsUsagePanel</code> modela o painel de monitorao
 * que exibe as cargas dos servidores de um grupo como barras de carga.
 * 
 * @author Anadr Clinio
 */
public class GroupBarsUsagePanel extends DynamicMonitorPanel {
  /** Aplicao */
  private Application app;
  /** Painel que contm o grfico */
  private ChartPanel chartPanel = null;
  /** Dataset */
  private DefaultCategoryDataset dataset = null;
  /** Grfico */
  private JFreeChart chart = null;
  /** Paint */
  private Color NORMAL_PAINT = Color.white;
  /** Paint */
  private Color WARNING_PAINT = Color.white;
  /** Paint */
  private Color ALERT_PAINT = Color.white;
  /** Paint */
  private Color PANIC_PAINT = Color.white;
  /** Paint */
  private Color DISABLED_PAINT = Color.white;
  /** Paint */
  private Color OUT_PAINT = Color.white;

  /**
   * Constri o painel que contm o canvas.
   * 
   * @return .
   */
  private ChartPanel createChartPanel() {
    dataset = new DefaultCategoryDataset();
    CategoryAxis xAxis = new CategoryAxis3D(app
      .getString("MON_BARS_GROUP_XAXIS_LABEL"));

    // xAxis.setSkipCategoryLabelsToFit(true);
    ValueAxis yAxis = new NumberAxis3D(app
      .getString("MON_BARS_GROUP_YAXIS_LABEL"));
    yAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());
    yAxis.setUpperMargin(0.10);
    yAxis.setRange(0, 100);
    CategoryItemRenderer renderer = new SGACategoryItemRenderer();
    renderer
      .setBaseItemLabelGenerator(new StandardCategoryItemLabelGenerator());
    CategoryPlot plot = new CategoryPlot(dataset, xAxis, yAxis, renderer);
    plot.setOrientation(PlotOrientation.VERTICAL);
    plot.setForegroundAlpha(0.75F);
    plot.setNoDataMessage(app.getString("MON_BARS_NO_DATA_LABEL"));
    // plot.setRenderer(new SGACategoryItemRenderer());
    chart = new JFreeChart("", JFreeChart.DEFAULT_TITLE_FONT, plot, false);
    // chart.setBackgroundPaint(BG_PAINT);
    chartPanel = new ChartPanel(chart);
    chartPanel.setPopupMenu(null);
    chartPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory
      .createLoweredBevelBorder(), BorderFactory.createEmptyBorder(10, 10, 10,
      10)));
    return chartPanel;
  }

  /**
   * Retorna o ttulo do painel.
   * 
   * @return ttulo do painel
   */
  @Override
  public String getTitle() {
    return app.getString("MON_BARS_GROUP_TITLE");
  }

  /**
   * Atualiza as informaes dinmicas exibidas.
   * 
   * @param objs informaes do servidor.
   */
  @Override
  public void updateDynamicInfo(Object objs) {
    SGASet[] sgaAll = (SGASet[]) objs;
    for (int r = 0; r < dataset.getRowCount(); r++) {
      dataset.removeRow(r);
    }
    CategoryItemRenderer renderer = chart.getCategoryPlot().getRenderer();
    for (int s = 0; s < sgaAll.length; s++) {
      boolean enabled = false;
      boolean alive = false;
      String name = "";
      double avg = 0;
      double actnodes = 0.0;
      SGASet sga = sgaAll[s];
      if (sga != null) {
        SGAInfo[] infos = sga.getAllInfo();
        for (int n = 0; n < infos.length; n++) {
          SGAInfo node = infos[n];
          if (node.getAlive()) {
            avg += node.getCPULoad1();
            actnodes += 1.0;
          }
        }
        name = sga.getName();
        enabled = sga.getEnabled();
        alive = sga.getAlive();
      }
      avg = (actnodes > 0) ? (avg / actnodes) : 0.0;
      dataset.setValue(avg, app.getString("MON_BARS_GROUP_CPU_USAGE"), name);
      Color paint = DISABLED_PAINT;
      if (avg <= 50) {
        paint = NORMAL_PAINT;
      }
      else if (avg <= 75) {
        paint = WARNING_PAINT;
      }
      else if (avg <= 100) {
        paint = ALERT_PAINT;
      }
      else {
        paint = PANIC_PAINT;
      }
      if (!enabled) {
        paint = DISABLED_PAINT;
      }
      if (!alive) {
        paint = OUT_PAINT;
      }
      renderer.setSeriesPaint(s, paint);
    }
    chartPanel.repaint();
  }

  /**
   * Constri o painel de monitorao.
   * 
   * @param app .
   */
  public GroupBarsUsagePanel(Application app) {
    super();
    this.app = app;

    NORMAL_PAINT = MonitoringTheme.getRangeNormalColor();
    WARNING_PAINT = MonitoringTheme.getRangeWarningColor();
    ALERT_PAINT = MonitoringTheme.getRangeAlertColor();
    PANIC_PAINT = MonitoringTheme.getRangePanicColor();
    DISABLED_PAINT = MonitoringTheme.getRangeDefaultColor();
    OUT_PAINT = MonitoringTheme.getOutColor();

    setLayout(new BorderLayout());
    add(createChartPanel(), BorderLayout.CENTER);
  }
}
