/*
 * GroupCPULoadInfoHandler.java
 * $Revision: 33587 $
 */
package csbase.client.applications.sgamonitor;

import csbase.logic.SGAInfo;
import csbase.logic.SGASet;

/**
 * A classe <code>GroupCPULoadInfoHandler</code> modela um gerenciador de
 * informaes de carga de um grupo de servidores.
 *
 * @author Ana
 */
class GroupCPULoadInfoHandler implements DynamicInfoHandler {
  /** Informaes do grupo */
  private SGASet[] groupData;

  /**
   * Atualiza as informaes dinmicas.
   *
   * @param dynamicData informaes do grupo
   */
  public void updateInfo(Object dynamicData) {
    this.groupData = (SGASet[])dynamicData;
  }

  /**
   * Obtm o nmero total de servidores.
   *
   * @return .
   */
  public int getTotalSize() {
    if (groupData == null) {
      return 0;
    }
    return groupData.length;
  }

  /**
   * Obtm o nmero de servidores vlidos. Os servidores vlidos so os que tem
   * informaes definidas (no null)
   *
   * @return .
   */
  public int getValidSize() {
    if (groupData == null) {
      return 0;
    }
    int size = 0;
    for (int i = 0; i < groupData.length; i++) {
      if ((groupData[i] != null) && (groupData[i].getAlive())) {
        size++;
      }
    }
    return size;
  }

  /**
   * Verifica se um servidor  vlido.
   *
   * @param index ndice do servidor
   *
   * @return true se o servidor  vlido, false caso contrrio
   */
  public boolean isValid(int index) {
    if (
      (index < groupData.length) && (groupData[index] != null) &&
        groupData[index].getAlive()) {
      return true;
    }
    else {
      return false;
    }
  }

  /**
   * Obtm o nome de um servidor.
   *
   * @param index ndice do servidor
   *
   * @return nome do servidor
   */
  public String getName(int index) {
    if ((index < groupData.length) && (groupData[index] != null)) {
      return groupData[index].getName();
    }
    else {
      return null;
    }
  }

  /**
   * Obtm a carga corrente de um servidor.
   *
   * @param index ndice do servidor
   *
   * @return carga do servidor (mdia se  um cluster)
   */
  public double getCurrentValue(int index) {
    if ((index < groupData.length) && (groupData[index] != null)) {
      SGAInfo[] serverData = groupData[index].getAllInfo();
      double sum = 0;
      for (int i = 0; i < serverData.length; i++) {
        double val = serverData[i].getCPULoad1();
        sum += val;
      }
      return sum / serverData.length;
    }
    else {
      return 0;
    }
  }

  /**
   * Constri o gerenciador.
   */
  public GroupCPULoadInfoHandler() {
    this.groupData = null;
  }
}
