/*
 * GroupSliceUsagePanel.java
 * $Revision: 61041 $
 */
package csbase.client.applications.sgamonitor;

import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.BorderFactory;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PiePlot;
import org.jfree.data.general.DefaultPieDataset;
import org.jfree.util.Rotation;

import csbase.client.applications.Application;
import csbase.client.util.MonitoringTheme;
import csbase.logic.SGASet;


/**
 * A classe <code>GroupSliceUsagePanel</code> modela o painel de monitorao
 * que exibe as cargas dos servidores de um grupo como uma torta de carga.
 *
 * @author Anadr Clinio
 */
public class GroupSliceUsagePanel extends DynamicMonitorPanel {
  
  /** Aplicao */
  final public Application app;

  /** Painel que contm o grfico */
  final private JFreeChart graph;
  
  
  /**
   * Constri o painel que contm o canvas.
   * @return um painel.
   */
  private ChartPanel createCanvasPanel() {
    final ChartPanel chartPanel = new ChartPanel(graph);
    chartPanel.setPopupMenu(null);
    chartPanel.setBorder(
      BorderFactory.createCompoundBorder(
        BorderFactory.createLoweredBevelBorder(),
        BorderFactory.createEmptyBorder(10, 10, 10, 10)));
    return chartPanel;
  }


  /**
   * Retorna o ttulo do painel.
   *
   * @return ttulo do painel
   */
  public String getTitle() {
    return app.getString("MON_SLICE_GROUP_TITLE");
  }

  /**
   * Atualiza as informaes dinmicas exibidas.
   *
   * @param obj informaes do servidor
   */
  public void updateDynamicInfo(final Object obj) {
    final SGASet[] info = (SGASet[])obj;
    double disabledSum = 0.0;
    double normalSum = 0.0;
    double warnSum = 0.0;
    double alertSum = 0.0;
    double panicSum = 0.0;
    if ((info == null) || (info.length == 0)) {
      disabledSum = 1.0;
    }
    else {
      for (int i = 0; i < info.length; i++) {
        final SGASet sga = (SGASet)info[i];
        if ((sga != null) && sga.getAlive()) {
          double load = sga.getCPULoad1();
          if (load <= 50) {
            normalSum += 1.0;
          }
          else if (load <= 75) {
            warnSum += 1.0;
          }
          else if (load <= 100) {
            alertSum += 1.0;
          }
          else {
            panicSum += 1.0;
          }
        }
        else {
          disabledSum += 1.0;
        }
      }
    }
    
    final String normalStr = app.getString("MON_SLICE_GROUP_NORMAL_TITLE");
    final String warnStr = app.getString("MON_SLICE_GROUP_WARNING_TITLE");
    final String alertStr = app.getString("MON_SLICE_GROUP_ALERT_TITLE");
    final String panicStr = app.getString("MON_SLICE_GROUP_PANIC_TITLE");
    final String disabledStr = app.getString("MON_SLICE_GROUP_DISABLED_TITLE");
    
    final Color normalColor = MonitoringTheme.getRangeNormalColor();
    final Color warnColor = MonitoringTheme.getRangeWarningColor();
    final Color alertColor = MonitoringTheme.getRangeAlertColor();
    final Color panicColor = MonitoringTheme.getRangePanicColor();
    final Color disabledColor = MonitoringTheme.getOutColor();
    
    final DefaultPieDataset dataset = new DefaultPieDataset();
    final PiePlot pieplot = (PiePlot)graph.getPlot();
    dataset.setValue(normalStr, normalSum);
    dataset.setValue(warnStr, warnSum);
    dataset.setValue(alertStr, alertSum);
    dataset.setValue(panicStr, panicSum);
    dataset.setValue(disabledStr, disabledSum);

    pieplot.setIgnoreZeroValues(false);
    pieplot.setBackgroundPaint(null);
    pieplot.setOutlinePaint(null);
    pieplot.setSectionPaint("0", normalColor);
    pieplot.setSectionPaint("1", warnColor);
    pieplot.setSectionPaint("2", alertColor);
    pieplot.setSectionPaint("3", panicColor);
    pieplot.setSectionPaint("4", disabledColor);
    
    pieplot.setDirection(Rotation.ANTICLOCKWISE);
    pieplot.setForegroundAlpha(0.8F);
    pieplot.setCircular(true);
    pieplot.setDataset(dataset);
  }

  /**
   * Constri o painel de monitorao.
   *
   * @param app .
   */
  public GroupSliceUsagePanel(final Application app) {
    super();
    this.app = app;
    graph = ChartFactory.createPieChart3D("", null, true, false, false);
    add(createCanvasPanel(), BorderLayout.CENTER);
  }
}
