/*
 * MainNodeUsagePanel.java
 *
 * $Author: valeria $
 * $Revision: 69426 $  - $Date: 2007-10-25 14:54:22 -0200 (Thu, 25 Oct 2007) $
 */
package csbase.client.applications.sgamonitor;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTextField;
import javax.swing.plaf.basic.BasicProgressBarUI;

import tecgraf.javautils.gui.GUIUtils;
import csbase.client.applications.Application;
import csbase.client.util.MonitoringTheme;
import csbase.client.util.sga.SGASelection;
import csbase.logic.CapacityType;
import csbase.logic.SGAInfo;
import csbase.logic.SGASet;

/**
 * A classe <code>MainNodeUsagePanel</code> modela o painel de monitorao que
 * exibe a ocupao do n principal (processador e memria) atravs de
 * "gauges".
 *
 * @author Ana
 * @version $Revision: 69426 $
 */
class MainNodeUsagePanel extends DynamicMonitorPanel {
  private Application app;

  private String title;
  private JProgressBar CPU1min;
  private JProgressBar CPU5min;
  private JProgressBar CPU15min;
  private JProgressBar RAMMemory;
  private JProgressBar swapMemory;

  private JTextField numProc;     /* Nmero de Processadores */
  private JTextField cpuCap;      /* Capacidade de processamento */
  private JTextField mRAM;        /* Tamanho da memria RAM  */
  private JTextField mSwap;       /* Tamanho da memria Swap */
  private JTextField platform;    /* Plataforma */
  private JTextField state;       /* State */

  private String selectedNode;
  private SGASet serverData;

  /**
   * Retorna o ttulo do painel.
   *
   * @return ttulo do painel
   */
  public String getTitle() {
    return title;
  }

  /**
   * Constroi um Gauge de monitorao
   *
   * @return .
   */
  private JProgressBar createGauge() {
    JProgressBar gauge = new JProgressBar(JProgressBar.VERTICAL, 0, 100);
    gauge.setUI(
      new BasicProgressBarUI() {
        protected Color getSelectionBackground() {
          return Color.BLACK;
        }

        protected Color getSelectionForeground() {
          return Color.BLACK;
        }
      });
    gauge.setStringPainted(true);
    return gauge;
  }

  public void updateDynamicInfo() {
    updateDynamicInfo(serverData); // Pra ser chamado do listener
  }
  /**
   * Atualiza as informaes dinmicas exibidas
   *
   * @param dynamicData informaes do servidor
   */
  public void updateDynamicInfo(Object dynamicData) {
    serverData = (SGASet)dynamicData;
    SGAInfo sga = serverData.getHost(selectedNode);
    updateGauge(CPU1min, (int)(sga.getCPULoad1()));
    updateGauge(CPU5min, (int)(sga.getCPULoad5()));
    updateGauge(CPU15min, (int)(sga.getCPULoad15()));
    int ramfree = (int)(sga.getRAMFreeMemory());
    int swapfree = (int)(sga.getSwapFreeMemory());
    updateGauge(RAMMemory, ramfree < 0 ? -1 : 100 - ramfree);
    updateGauge(swapMemory, swapfree < 0 ? -1 : 100 - swapfree);
  }

  private void updateGauge(JProgressBar gauge, int value) {
    if (value < 0) {
      gauge.setValue(0);
      gauge.setString("-----");
      gauge.setForeground(MonitoringTheme.getRangeErrorColor());
    }
    else {
      gauge.setValue(value);
      gauge.setString(value + " %");
      if (value < 50) {
        gauge.setForeground(MonitoringTheme.getRangeNormalColor());
      }
      else if (value < 75) {
        gauge.setForeground(MonitoringTheme.getRangeWarningColor());
      }
      else if (value < 100) {
        gauge.setForeground(MonitoringTheme.getRangeAlertColor());
      }
      else {
        gauge.setForeground(MonitoringTheme.getRangePanicColor());
      }
    }
  }

  /**
   * Atualiza as informaes estticas exibidas
   *
   * @param nodeName n selecionado
   */
  private void setNode(String nodeName) {
    selectedNode = nodeName;
    SGAInfo sga = serverData.getHost(nodeName);
    long cpuCapacity = sga.getCapacity(CapacityType.CPU);
    numProc.setText("" + sga.getNumProcessors());
    cpuCap.setText("" + (cpuCapacity == -1 ? "-----" : cpuCapacity ));
    mRAM.setText("" + sga.getRAMMemoryInfoMb() + " Mb");
    mSwap.setText("" + sga.getSwapMemoryInfoMb() + " Mb");
    platform.setText("" + sga.getPlatformId());
    if (sga.getAlive()) {
      state.setText("" + app.getString("MON_AVAILABLE"));
    }
    else {
      state.setText("" + app.getString("MON_INACCESSIBLE"));
    }
  }

  /**
   * Constri o painel que contm as informaes dinmicas do servidor.
   *
   * @param app .
   * @param serverData .
   */
  public MainNodeUsagePanel(Application app, SGASet serverData) {
    super();
    this.app = app;
    this.serverData = serverData;
    GridBagConstraints constraints = new GridBagConstraints();
    Insets defInsets = new Insets(2, 2, 2, 2);
    /*
     * Ocupao da CPU
     */
    constraints.weightx = 100;
    constraints.weighty = 0;
    constraints.gridheight = 1;
    constraints.gridwidth = 1;
    JPanel CPUpanel = new JPanel(new GridBagLayout());
    CPUpanel.setBorder(
      BorderFactory.createCompoundBorder(
        BorderFactory.createCompoundBorder(
          BorderFactory.createEmptyBorder(10, 10, 10, 10),
          BorderFactory.createTitledBorder(app.getString("MON_PROCESSOR"))),
        BorderFactory.createEmptyBorder(10, 10, 10, 10)));
    // Nmero de processadores
    numProc = new JTextField();
    numProc.setEditable(false);
    numProc.setText("" + serverData.getNumProcessors());
    // Capacidade de processamento
    long cpuCapacity = serverData.getCapacity(CapacityType.CPU);
    cpuCap = new JTextField();
    cpuCap.setEditable(false);
    cpuCap.setText("" + (cpuCapacity == -1 ? "-----" : cpuCapacity ));
    JPanel procPanel =
      GUIUtils.createBasicGridPanel(
        new JComponent[][] {
          { new JLabel(app.getString("MON_NUM_PROCESSORS") + " : "), numProc },
          { new JLabel(app.getString("MON_CPU_CAPACITY") + " : "), cpuCap }
        });
    constraints.anchor = GridBagConstraints.WEST;
    constraints.gridx = 0;
    constraints.gridy = 0;
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.insets = new Insets(0, 0, 2, 2);
    CPUpanel.add(procPanel, constraints);
    // Ttulo
    constraints.anchor = GridBagConstraints.WEST;
    constraints.gridx = 0;
    constraints.gridy = 1;
    constraints.fill = GridBagConstraints.NONE;
    constraints.insets = new Insets(11, 2, 5, 2);
    CPUpanel.add(
      new JLabel(app.getString("MON_USAGE_HISTORY") + " :"), constraints);
    // Gauges
    JPanel cgPanel = new JPanel(new GridBagLayout());
    cgPanel.setBackground(Color.WHITE);
    cgPanel.setBorder(
      BorderFactory.createCompoundBorder(
        BorderFactory.createLoweredBevelBorder(),
        BorderFactory.createEmptyBorder(15, 10, 15, 10)));
    CPU1min = createGauge();
    CPU5min = createGauge();
    CPU15min = createGauge();
    constraints.insets = defInsets;
    constraints.weighty = 100;
    constraints.anchor = GridBagConstraints.CENTER; // centrado
    constraints.fill = GridBagConstraints.VERTICAL;
    constraints.gridy = 0;
    constraints.gridx = 0;
    cgPanel.add(CPU1min, constraints);
    constraints.gridx = 1;
    cgPanel.add(CPU5min, constraints);
    constraints.gridx = 2;
    cgPanel.add(CPU15min, constraints);
    // Legendas dos Gauges
    constraints.weighty = 0;
    constraints.gridy = 1;
    constraints.gridx = 0;
    constraints.anchor = GridBagConstraints.CENTER; // centrado
    constraints.fill = GridBagConstraints.NONE;
    cgPanel.add(new JLabel("1 min"), constraints);
    constraints.gridx = 1;
    cgPanel.add(new JLabel("5 min"), constraints);
    constraints.gridx = 2;
    cgPanel.add(new JLabel("15 min"), constraints);
    constraints.gridx = 0;
    constraints.gridy = 2;
    constraints.fill = GridBagConstraints.BOTH;
    constraints.weighty = 100;
    CPUpanel.add(cgPanel, constraints);
    /*
     * Ocupao da Memria
     */
    constraints.weightx = 100;
    constraints.weighty = 0;
    constraints.gridheight = 1;
    constraints.gridwidth = 1;
    JPanel memPanel = new JPanel(new GridBagLayout());
    memPanel.setBorder(
      BorderFactory.createCompoundBorder(
        BorderFactory.createCompoundBorder(
          BorderFactory.createEmptyBorder(10, 10, 10, 10),
          BorderFactory.createTitledBorder(app.getString("MON_MEMORY"))),
        BorderFactory.createEmptyBorder(10, 10, 10, 10)));
    // Tamanho da memria
    // RAM
    mRAM = new JTextField();
    mRAM.setEditable(false);
    mRAM.setText("" + serverData.getRAMMemoryInfoMb() + " Mb");
    // Swap
    mSwap = new JTextField();
    mSwap.setEditable(false);
    mSwap.setText("" + serverData.getSwapMemoryInfoMb() + " Mb");
    JPanel msp =
      GUIUtils.createBasicGridPanel(
        new JComponent[][] {
          { new JLabel(app.getString("MON_RAM_MEMORY_SIZE") + " : "), mRAM },
          { new JLabel(app.getString("MON_SWAP_MEMORY_SIZE") + " : "), mSwap }
        });
    constraints.anchor = GridBagConstraints.WEST;
    constraints.gridx = 0;
    constraints.gridy = 0;
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.insets = new Insets(0, 0, 2, 2);
    memPanel.add(msp, constraints);
    // Ttulo
    constraints.anchor = GridBagConstraints.WEST;
    constraints.gridx = 0;
    constraints.gridy = 1;
    constraints.fill = GridBagConstraints.NONE;
    constraints.insets = new Insets(11, 2, 5, 2);
    memPanel.add(
      new JLabel(app.getString("MON_CURRENT_USAGE") + " :"), constraints);
    // Gauge
    JPanel mgPanel = new JPanel(new GridBagLayout());
    mgPanel.setBackground(Color.WHITE);
    mgPanel.setBorder(
      BorderFactory.createCompoundBorder(
        BorderFactory.createLoweredBevelBorder(),
        BorderFactory.createEmptyBorder(10, 10, 10, 10)));
    RAMMemory = createGauge();
    constraints.insets = defInsets;
    constraints.weighty = 100;
    constraints.anchor = GridBagConstraints.CENTER;
    constraints.fill = GridBagConstraints.VERTICAL;
    constraints.gridx = 0;
    constraints.gridy = 0;
    mgPanel.add(RAMMemory, constraints);
    constraints.weighty = 0;
    constraints.anchor = GridBagConstraints.CENTER;
    constraints.fill = GridBagConstraints.NONE;
    constraints.gridx = 0;
    constraints.gridy = 1;
    mgPanel.add(new JLabel("RAM"), constraints);
    swapMemory = createGauge();
    constraints.insets = defInsets;
    constraints.weighty = 100;
    constraints.anchor = GridBagConstraints.CENTER;
    constraints.fill = GridBagConstraints.VERTICAL;
    constraints.gridx = 1;
    constraints.gridy = 0;
    mgPanel.add(swapMemory, constraints);
    constraints.weighty = 0;
    constraints.anchor = GridBagConstraints.CENTER;
    constraints.fill = GridBagConstraints.NONE;
    constraints.gridx = 1;
    constraints.gridy = 1;
    mgPanel.add(new JLabel("Swap"), constraints);
    constraints.gridx = 0;
    constraints.gridy = 2;
    constraints.fill = GridBagConstraints.BOTH;
    constraints.weighty = 100;
    memPanel.add(mgPanel, constraints);
    /*
     * Painel de monitorao completo
     */
    JPanel completo = new JPanel(new GridBagLayout());
    constraints.anchor = GridBagConstraints.WEST; // alinhado  esquerda
    constraints.weighty = 100;
    constraints.weightx = 100;
    constraints.insets = new Insets(0, 0, 0, 0);
    constraints.gridwidth = 1;
    constraints.fill = GridBagConstraints.BOTH;
    constraints.gridy = 1;
    constraints.gridx = 0;
    completo.add(CPUpanel, constraints);
    constraints.gridx = 1;
    constraints.insets = new Insets(0, 2, 0, 0);
    completo.add(memPanel, constraints);
    /*
     * No caso de cluster, inclui seleo de ns.
     */
    setLayout(new BorderLayout());

    if (serverData.isCluster()) {
      final SGASelection nodeSelection =
        new SGASelection(false, new Vector<String>());
      Vector<String> nodesNames = new Vector<String>();
      String[] nodes = serverData.getAllNames();
      for (int i = 0; i < nodes.length; i++) {
        nodesNames.add(nodes[i]);
      }
      nodeSelection.loadItems(nodesNames);
      nodeSelection.addSGASelectionListener( new ActionListener() {
        public void actionPerformed(ActionEvent ae) {
          String nodeName = (String)nodeSelection.getSelectedItem();
          if (nodeName != null) {
            setNode(nodeName);
            updateDynamicInfo();
          }
        }
      });

      // Plataforma
      platform = new JTextField();
      platform.setEditable(false);

      // Estado
      state = new JTextField();
      state.setEditable(false);
      JPanel nodeInfo =
        GUIUtils.createBasicGridPanel(
          new JComponent[][] {
            { new JLabel(app.getString("MON_NODES") + " : "),
              nodeSelection.getPanel() },
            { new JLabel(app.getString("MON_PLATFORM") + " : "),
              platform},
            { new JLabel(app.getString("MON_STATE") + " : "),
              state}
          });
      add(nodeInfo,BorderLayout.NORTH);

      setNode(serverData.getInfo(0).getHostName()); // N Principal
      nodeSelection.setSelectedItem(selectedNode);
    }
    title = app.getString("MON_USAGE");
    add(completo, BorderLayout.CENTER);
  }
}
