/*
 * MemoryDistributionPanel.java
 *
 * $Author: moura $
 * $Revision: 36552 $  - $Date: 2006-02-02 18:13:31 -0200 (Thu, 02 Feb 2006) $
 */
package csbase.client.applications.sgamonitor;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.plaf.basic.BasicProgressBarUI;

import tecgraf.javautils.gui.GUIUtils;
import csbase.client.applications.Application;
import csbase.client.util.MonitoringTheme;
import csbase.logic.SGAInfo;
import csbase.logic.SGASet;

/**
 * A classe <code>MemoryDistributionPanel</code> modela o painel de monitorao
 * que exibe a distribuio de ocupao de memria nos ns de um cluster.
 *
 * @author Ana
 * @version $Revision: 36552 $
 */
class MemoryDistributionPanel extends DynamicMonitorPanel {
  private Application app;
  private int numNodes;
  private JProgressBar RAMUnavailableUsage;
  private JProgressBar RAMLightUsage;
  private JProgressBar RAMMediumUsage;
  private JProgressBar RAMHighUsage;
  private JProgressBar RAMVeryHighUsage;
  private JProgressBar swapUnavailableUsage;
  private JProgressBar swapLightUsage;
  private JProgressBar swapMediumUsage;
  private JProgressBar swapHighUsage;
  private JProgressBar swapVeryHighUsage;

  /**
   * Retorna o ttulo do painel.
   *
   * @return ttulo do painel
   */
  public String getTitle() {
    return app.getString("MON_CLUSTER_MEM_USAGE");
  }

  /**
   * Constroi um Gauge de monitorao
   *
   * @param c largura do Gauge
   *
   * @return .
   */
  private JProgressBar createGauge(Color c) {
    JProgressBar gauge = new JProgressBar(JProgressBar.HORIZONTAL, 0, numNodes);
    gauge.setUI(
      new BasicProgressBarUI() {
        protected Color getSelectionBackground() {
          return Color.BLACK;
        }

        protected Color getSelectionForeground() {
          return Color.BLACK;
        }
      });
    gauge.setStringPainted(true);
    gauge.setForeground(c);
    return gauge;
  }

  /**
   * Atualiza as informaes dinmicas exibidas
   *
   * @param obj informaes do servidor
   */
  public void updateDynamicInfo(Object obj) {
    SGASet sga = (SGASet)obj;
    SGAInfo[] dynamicData = sga.getAllInfo();
    int unavailable = 0;
    int light = 0;
    int medium = 0;
    int high = 0;
    int veryHigh = 0;

    // RAM
    for (int i = 0; i < numNodes; i++) {
      SGAInfo serverData = dynamicData[i];
      int freemem = (int)(serverData.getRAMFreeMemory());
      int usage = 100 - freemem;
      if (freemem < 0) {
        unavailable++;
      }
      else if (usage <= 25) {
        light++;
      }
      else if (usage <= 50) {
        medium++;
      }
      else if (usage <= 75) {
        high++;
      }
      else {
        veryHigh++;
      }
    }
    RAMUnavailableUsage.setValue(unavailable);
    RAMUnavailableUsage.setString("" + unavailable + " " +
      app.getString("MON_NODES"));
    RAMLightUsage.setValue(light);
    RAMLightUsage.setString("" + light + " " + app.getString("MON_NODES"));
    RAMMediumUsage.setValue(medium);
    RAMMediumUsage.setString("" + medium + " " + app.getString("MON_NODES"));
    RAMHighUsage.setValue(high);
    RAMHighUsage.setString("" + high + " " + app.getString("MON_NODES"));
    RAMVeryHighUsage.setValue(veryHigh);
    RAMVeryHighUsage.setString(
      "" + veryHigh + " " + app.getString("MON_NODES"));
    // Swap
    unavailable = 0;
    light = 0;
    medium = 0;
    high = 0;
    veryHigh = 0;
    for (int i = 0; i < numNodes; i++) {
      SGAInfo serverData = dynamicData[i];
      int freemem = (int)(serverData.getSwapFreeMemory());
      int usage = 100 - freemem;
      if (freemem < 0) {
        unavailable++;
      }
      else if (usage <= 25) {
        light++;
      }
      else if (usage <= 50) {
        medium++;
      }
      else if (usage <= 75) {
        high++;
      }
      else {
        veryHigh++;
      }
    }
    swapUnavailableUsage.setValue(unavailable);
    swapUnavailableUsage.setString("" + unavailable + " " + app.getString("MON_NODES"));
    swapLightUsage.setValue(light);
    swapLightUsage.setString("" + light + " " + app.getString("MON_NODES"));
    swapMediumUsage.setValue(medium);
    swapMediumUsage.setString("" + medium + " " + app.getString("MON_NODES"));
    swapHighUsage.setValue(high);
    swapHighUsage.setString("" + high + " " + app.getString("MON_NODES"));
    swapVeryHighUsage.setValue(veryHigh);
    swapVeryHighUsage.setString(
      "" + veryHigh + " " + app.getString("MON_NODES"));
  }

  /**
   * Constri o painel que contm as informaes dinmicas do servidor.
   *
   * @param app .
   * @param server .
   */
  public MemoryDistributionPanel(Application app, SGASet server) {
    super();
    this.app = app;
    numNodes = server.getNumNodes();
    setLayout(new GridLayout(2, 1));
    setBorder(
      BorderFactory.createCompoundBorder(
        BorderFactory.createCompoundBorder(
          BorderFactory.createEmptyBorder(10, 10, 10, 10),
          BorderFactory.createTitledBorder(
            app.getString("MON_MEM_DISTRIBUTION"))),
        BorderFactory.createEmptyBorder(10, 10, 10, 10)));
    
    // Painel RAM
    RAMUnavailableUsage = createGauge(MonitoringTheme.getRangeErrorColor());
    RAMLightUsage = createGauge(MonitoringTheme.getRangeNormalColor());
    RAMMediumUsage = createGauge(MonitoringTheme.getRangeNormalColor());
    RAMHighUsage = createGauge(MonitoringTheme.getRangeWarningColor());
    RAMVeryHighUsage = createGauge(MonitoringTheme.getRangeAlertColor());
    JComponent[][] rc =
    {
      { new JLabel(" 0  -  25 %"), RAMLightUsage },
      { new JLabel("25  -  50 %"), RAMMediumUsage },
      { new JLabel("50  -  75 %"), RAMHighUsage },
      { new JLabel("75  - 100 %"), RAMVeryHighUsage },
      { new JLabel("-----"), RAMUnavailableUsage }
    };
    JPanel mr = GUIUtils.createBasicGridPanel(rc);
    mr.setBackground(Color.WHITE);
    mr.setBorder(
      BorderFactory.createCompoundBorder(
        BorderFactory.createLoweredBevelBorder(),
        BorderFactory.createEmptyBorder(10, 10, 10, 10)));
    JPanel RAMp = new JPanel();
    RAMp.setLayout(new BorderLayout());
    RAMp.add(new JLabel("RAM"), BorderLayout.NORTH);
    RAMp.add(mr, BorderLayout.CENTER);
    add(RAMp);
    // Painel Swap
    swapUnavailableUsage = createGauge(MonitoringTheme.getRangeErrorColor());
    swapLightUsage = createGauge(MonitoringTheme.getRangeNormalColor());
    swapMediumUsage = createGauge(MonitoringTheme.getRangeNormalColor());
    swapHighUsage = createGauge(MonitoringTheme.getRangeWarningColor());
    swapVeryHighUsage = createGauge(MonitoringTheme.getRangeAlertColor());
    JComponent[][] sc =
    {
      { new JLabel(" 0  -  25 %"), swapLightUsage },
      { new JLabel("25  -  50 %"), swapMediumUsage },
      { new JLabel("50  -  75 %"), swapHighUsage },
      { new JLabel("75  - 100 %"), swapVeryHighUsage },
      { new JLabel("-----"), swapUnavailableUsage }
    };
    JPanel ms = GUIUtils.createBasicGridPanel(sc);
    ms.setBackground(Color.WHITE);
    ms.setBorder(
      BorderFactory.createCompoundBorder(
        BorderFactory.createLoweredBevelBorder(),
        BorderFactory.createEmptyBorder(10, 10, 10, 10)));
    JPanel swapp = new JPanel();
    swapp.setLayout(new BorderLayout());
    swapp.add(new JLabel("Swap"), BorderLayout.NORTH);
    swapp.add(ms, BorderLayout.CENTER);
    add(swapp);
  }
}
