package csbase.client.applications.sgamonitor;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.applications.Application;
import csbase.client.applications.ApplicationFrame;
import csbase.client.desktop.DesktopErrorEvent;
import csbase.client.desktop.DesktopFrame;
import csbase.client.facilities.monitoringtable.MonitoringTableEvent;
import csbase.client.facilities.monitoringtable.MonitoringTableObserver;
import csbase.client.facilities.sgatable.SGATable;
import csbase.client.remote.srvproxies.SGAProxy;
import csbase.client.util.ClientUtilities;
import csbase.logic.SGASet;

/**
 * A classe <code>SGADisableFrame</code> representa um dilogo de  desativao
 * dos servidores de algoritmos (SGAs).
 *
 * @author Andr Luiz Clinio
 * @version $Revision: 87641 $
 */
public class SGADisableFrame implements MonitorFrame, MonitoringTableObserver {
  /** Aplicao */
  private Application app;

  /** Elementos de Interface */
  private ApplicationFrame mainFrame;
  private JButton disableButton;
  private JButton monitorButton;  
  private JButton restartButton;
  private JButton selectAllButton;
  private SGATable sgaTable;
  private DesktopErrorEvent errorEvent = null;

  /**
   * Exibe a janela de monitorao.
   */
  private void display() {
    mainFrame.pack();
    mainFrame.center(DesktopFrame.getInstance().getDesktopFrame());
    mainFrame.setVisible(true);
  }

  /**
   * Termina a monitorao.
   */
  public void close() {
    sgaTable.stopThread();
  }

  /**
   * Cria a janela de monitorao dos servidores.
   */
  private void createMainFrame() {
    mainFrame = app.getApplicationFrame();
    Container cp = mainFrame.getContentPane();
    cp.setLayout(new BorderLayout(5, 5));
    sgaTable = new SGATable(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    sgaTable.addObserver(this);
    cp.add(new JScrollPane(sgaTable), BorderLayout.CENTER);
    cp.add(createButtons(), BorderLayout.SOUTH);
  }

  /**
   * Cria o painel de botes.
   *
   * @return o painel de botes criado
   */
  private JPanel createButtons() {
    JPanel panel = new JPanel(new FlowLayout());
    selectAllButton = new JButton(app.getString("MON_SELECT_ALL"));
    selectAllButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          sgaTable.selectAll();
        }
      });
    selectAllButton.setEnabled(true);
    this.restartButton = new JButton(app.getString("MON_RESTART"));
    this.restartButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          int[] rows = sgaTable.getSelectedRows();
          if ((rows != null) && (rows.length > 0)) {
            boolean confirmed = false;
            if (rows.length == 1) {
              int modelRow = sgaTable.convertRowIndexToModel(rows[0]);
              SGASet sga = sgaTable.getSGASet(modelRow);
              confirmed = confirmOneRestart(sga.getName());
            }
            else {
              confirmed = confirmManyRestarts();
            }
            if (confirmed) {
              if (sgaTable.getRowCount() == sgaTable.getSelectedRowCount()) {
                SGAProxy.restartAllSGAs();
              }
              else {
                for (int i = 0; i < rows.length; i++) {
                  int modelRow = sgaTable.convertRowIndexToModel(rows[i]);
                  SGASet sga = sgaTable.getSGASet(modelRow);
                  SGAProxy.restartSGA(sga.getName());
                }
              }
            }
          }
        }
      });
    this.restartButton.setEnabled(false);
    monitorButton = new JButton(app.getString("MON_DETAIL"));
    monitorButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          int viewRow = sgaTable.getSelectedRow();
          if (viewRow != -1) {
            int modelRow = sgaTable.convertRowIndexToModel(viewRow);
            SGASet sga = sgaTable.getSGASet(modelRow);
            new SGAInfoFrame(sga.getName(), app);
          }
        }
      });
    monitorButton.setEnabled(false);
    disableButton = new JButton(app.getString("MON_SHUTDOWN"));
    disableButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          int[] rows = sgaTable.getSelectedRows();
          if ((rows != null) && (rows.length > 0)) {
            boolean confirmed = false;
            if (rows.length == 1) {
              int modelRow = sgaTable.convertRowIndexToModel(rows[0]);
              SGASet sga = sgaTable.getSGASet(modelRow);
              confirmed = confirmOneShutdown(sga.getName());
            }
            else {
              confirmed = confirmManyShutdowns();
            }
            if (confirmed) {
              if (sgaTable.getRowCount() == sgaTable.getSelectedRowCount()) {
                SGAProxy.shutdownAllSGAs();
              }
              else {
                for (int i = 0; i < rows.length; i++) {
                  int modelRow = sgaTable.convertRowIndexToModel(rows[i]);
                  SGASet sga = sgaTable.getSGASet(modelRow);
                  SGAProxy.shutdownSGA(sga.getName());
                }
              }
            }
          }
        }
      });
    disableButton.setEnabled(false);
    JButton closeButton = new JButton(app.getString("MON_CLOSE"));
    closeButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent ev) {
          close();
          app.finishApplication();
        }
      });
    addSelectionListener();
    ClientUtilities.adjustEqualSizes(selectAllButton, monitorButton,
      this.restartButton, disableButton, closeButton);
    panel.add(selectAllButton);
    panel.add(monitorButton);
    panel.add(this.restartButton);
    panel.add(disableButton);
    panel.add(closeButton);
    return panel;
  }

  /**
   * Confirma a desativao de um SGA.
   *
   * @param sgaName nome do SGA
   *
   * @return true se a desativao foi confirmada
   */
  private boolean confirmOneShutdown(String sgaName) {
    Object[] options =
    {
      app.getString("SGADisableFrame.button.shutdown"),
      app.getString("SGADisableFrame.button.cancel")
    };
    int opt =
      StandardDialogs.showOptionDialog(
        mainFrame, app.getString("SGADisableFrame.title.short"),
        String.format(app.getString("SGADisableFrame.question.shutdown"), 
          sgaName), options);
    return (opt == 0);
  }

  /**
   * Confirma a reinicializao de um SGA.
   * 
   * @param sgaName nome do SGA
   * @return true se a reinicializao foi confirmada
   */
  private boolean confirmOneRestart(String sgaName) {
    Object[] options =
    {
      app.getString("SGADisableFrame.button.restart"),
      app.getString("SGADisableFrame.button.cancel")
    };
    int opt =
      StandardDialogs.showOptionDialog(
        mainFrame, app.getString("SGADisableFrame.title.short"),
        String.format(app.getString("SGADisableFrame.question.restart"),
          sgaName), options);
    return (opt == 0);
  }

  /**
   * Confirma a desativao dos SGAs selecionados
   *
   * @return true se a desativao foi confirmada
   */
  private boolean confirmManyShutdowns() {
    Object[] options =
    {
      app.getString("SGADisableFrame.button.shutdown"),
      app.getString("SGADisableFrame.button.cancel")
    };
    int opt =
      StandardDialogs.showOptionDialog(
        mainFrame, app.getString("SGADisableFrame.title.short"),
        app.getString("SGADisableFrame.question.shutdown.many"), options);
    return (opt == 0);
  }

  /**
   * Confirma o reincio dos SGAs selecionados
   * @return true se a reinicializao foi confirmada
   */
  private boolean confirmManyRestarts() {
    Object[] options =
      {
        app.getString("SGADisableFrame.button.restart"),
        app.getString("SGADisableFrame.button.cancel")
      };
    int opt =
      StandardDialogs.showOptionDialog(
        mainFrame, app.getString("SGADisableFrame.title.short"),
        app.getString("SGADisableFrame.question.restart.many"), options);
    return (opt == 0);
  }

  /**
   * Adiciona ouvidor para selecao de linhas, e habilita o botao  de visualizar
   * monitoracao se estiver alguma linha em selecao, senao o botao e
   * desabilitado.
   */
  public void addSelectionListener() {
    ListSelectionModel selectModel = sgaTable.getSelectionModel();
    selectModel.addListSelectionListener(
      new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent e) {
          if (sgaTable.getSelectedRowCount() == 0) {
            monitorButton.setEnabled(false);
            disableButton.setEnabled(false);
            restartButton.setEnabled(false);
          }
          else if (sgaTable.getSelectedRowCount() == 1) {
            monitorButton.setEnabled(true);
            disableButton.setEnabled(true);
            restartButton.setEnabled(true);
          }
          else {
            int[] viewRows = sgaTable.getSelectedRows();
            boolean isAlive = true;
            for (int i = 0; i < viewRows.length; i++) {
              int modelRow = sgaTable.convertRowIndexToModel(viewRows[i]);
              SGASet sgaSet = sgaTable.getSGASet(modelRow);
              if (!sgaSet.getAlive()) {
                isAlive = false;
                break;
              }
            }
            monitorButton.setEnabled(false);
            disableButton.setEnabled(isAlive);
            restartButton.setEnabled(isAlive);            
          }
        }
      });
  }

  /**
   * Recebe notificaes da SGATable toda vez que esta  atualizada, ou na
   * ocorrncia de algum erro.
   *
   * @param sgaTableEvent Um evento do tipo MonitoringTableEvent.
   */
  public void update(final MonitoringTableEvent sgaTableEvent) {
    int event = sgaTableEvent.event;
    if (event == MonitoringTableEvent.REFRESH) {
      if (sgaTable.isAnySgaAlive()) {
        selectAllButton.setEnabled(true);
      }
      else {
        selectAllButton.setEnabled(false);
        disableButton.setEnabled(false);
      }
      if (errorEvent != null) {
        mainFrame.resetError(errorEvent);
        disableButton.setEnabled(false); //Nenhum SGA est selecionado
        errorEvent = null;
      }
    }
    else if (event == MonitoringTableEvent.REMOTE_ERROR) {
      JComponent[] disabled =
        new JComponent[] { sgaTable, disableButton, selectAllButton };
      errorEvent =
        new DesktopErrorEvent(
          app.getString("MON_COMM_ERROR"), null, disabled);
      mainFrame.signalError(errorEvent);
    }
  }

  /**
   * Construtor padro. Constri um monitorador de servidores (SGA's) apenas
   * com as colunas padro.
   *
   * @param app .
   */
  public SGADisableFrame(Application app) {
    this.app = app;
    createMainFrame();
    display();
  }
}
