/*
 * ServerGroupDefinitionDialog.java
 *
 * $Author: ururahy $
 * $Revision: 56650 $  - $Date: 2006-09-18 17:54:25 -0300 (Mon, 18 Sep 2006) $
 */

package csbase.client.applications.sgamonitor;

import csbase.client.applications.Application;
import csbase.client.applications.ApplicationComponentDialog;
import csbase.client.remote.srvproxies.SGAProxy;
import csbase.client.util.ClientUtilities;
import csbase.logic.ServerGroupInfo;

import tecgraf.javautils.gui.GUIUtils;
import tecgraf.javautils.gui.selector.ContainerSelection;
import tecgraf.javautils.gui.table.ObjectTableProvider;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.*;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * A classe <code>ServerGroupDefinitionDialog</code>  responsvel pela
 * criao e alterao de grupos de servidores (SGAs).
 *
 * @author Glaucio e Ana
 * @version $Revision: 56650 $
 */
public class ServerGroupDefinitionDialog {
  /** Aplicao */
  private Application app;

  /** Grupo de Servidores */
  private ServerGroupInfo groupInfo;
  private boolean newGroup;

  /** Grupos j definidos */
  private Vector definedGroups;

  /** Elementos de interface */
  private ApplicationComponentDialog mainDialog;
  private String dialogTitle;
  private JTextField groupName;
  private ContainerSelection<String> serversSelection;

  /**
   * Exibe um dilogo de criao de grupo de servidores.
   *
   * @param definedGroups lista (vetor) de nomes dos grupos j definidos
   *
   * @return informaes do grupo criado, null se a operao tiver sido
   *         cancelada ou em caso de erro
   */
  public ServerGroupInfo showCreateDialog(Vector definedGroups) {
    this.definedGroups = definedGroups;
    this.newGroup = true;
    this.dialogTitle = app.getString("MON_GROUP_ADD_TITLE");
    Vector<String> servers = SGAProxy.getAllSGANames();
    showDialog(null, servers, new Vector<String>());
    return groupInfo;
  }

  /**
   * Exibe um dilogo de alterao de grupo de servidores.
   *
   * @param oldGroupInfo informaes do grupo a atualizar
   *
   * @return informaes atualizadas do grupo, null se a operao tiver sido
   *         cancelada ou em caso de erro
   */
  public ServerGroupInfo showUpdateDialog(ServerGroupInfo oldGroupInfo) {
    this.newGroup = false;
    this.dialogTitle = app.getString("MON_GROUP_UPDATE_TITLE");
    String[] selectedServers = oldGroupInfo.getServers();
    Vector<String> servers = SGAProxy.getAllSGANames();
    Vector<String> selectedServersVector = new Vector<String>();
    selectedServersVector.addAll(Arrays.asList(selectedServers));
    showDialog(oldGroupInfo.getName(), servers, selectedServersVector);
    return groupInfo;
  }

  /**
   * Cria o dilogo conforme a iterao (criao, alterao).
   *
   * @param name nome do grupo (alterar) ou null (criar)
   * @param selectableServers servidores disponveis para adio
   * @param selectedServers servidores selecionados no grupo
   */
  private void showDialog(
    String name, Vector<String> selectableServers, Vector<String> selectedServers) {
    mainDialog = new ApplicationComponentDialog(
        app.getApplicationFrame(), dialogTitle, app);
    Container cp = mainDialog.getContentPane();

    // Nome do grupo
    if (newGroup) {
      groupName = new JTextField();
    }
    else {
      groupName = new JTextField(name);
      groupName.setEditable(false);
    }
    JComponent[][] nameComp =
      new JComponent[][] {
        { new JLabel(app.getString("MON_GROUP")), groupName }
      };
    cp.add(GUIUtils.createBasicGridPanel(nameComp), BorderLayout.NORTH);
    // Painel para seleo de servidores
    JPanel selectionPanel =
      createSelectionPanel(selectableServers, selectedServers);
    cp.add(selectionPanel, BorderLayout.CENTER);
    // Botes
    JButton actionButton;
    if (newGroup) {
      actionButton = new JButton(app.getString("MON_CREATE"));
    }
    else {
      actionButton = new JButton(app.getString("MON_UPDATE"));
    }
    actionButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (validateGroup()) {
            //mainDialog.close();
            mainDialog.setVisible(false);
          }
        }
      });

    JButton cancelButton = new JButton(app.getString("MON_CANCEL"));
    cancelButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          mainDialog.setVisible(false);
          //mainDialog.close();
        }
      });
    ClientUtilities.adjustEqualSizes(actionButton, cancelButton);
    JPanel buttonsPanel = new JPanel();
    buttonsPanel.add(actionButton);
    buttonsPanel.add(cancelButton);
    cp.add(buttonsPanel, BorderLayout.SOUTH);
    mainDialog.pack();
    mainDialog.center(app.getApplicationFrame());
    mainDialog.setVisible(true);
  }

  /**
   * Cria o painel para seleo de servidores.
   *
   * @param selectableServers servidores disponveis para adio
   * @param selectedServers servidores selecionados no grupo
   *
   * @return o painel criado
   */
  private JPanel createSelectionPanel(
    Vector<String> selectableServers, Vector<String> selectedServers) {
    ObjectTableProvider selectableProvider =
      new ObjectTableProvider() {
        public Class[] getColumnClasses() {
          return new Class[] { String.class };
        }

        public Object[] getCellValues(Object arg0) {
          return new String[] {(String) arg0};
        }

        public String[] getColumnNames() {
          return new String[] { app.getString("MON_GROUP_SELECTABLE_SERVERS") };
        }
      };
      ObjectTableProvider selectedProvider =
      new ObjectTableProvider() {
        public Class[] getColumnClasses() {
          return new Class[] { String.class };
        }

        public Object[] getCellValues(Object arg0) {
          return new String[] {(String) arg0};
        }

        public String[] getColumnNames() {
          return new String[] { app.getString("MON_GROUP_SELECTED_SERVERS") };
        }
      };
    serversSelection = new ContainerSelection<String>(
      selectableProvider, selectedProvider, true, false);
    serversSelection.loadItems(selectableServers, selectedServers);
    serversSelection.adjustTableColumns();
    return serversSelection.getPanel();
  }

  /**
   * Valida as informaes do grupo criado/alterado.
   *
   * @return true se o grupo  vlido, false caso contrrio
   */
  private boolean validateGroup() {
    String name = groupName.getText();

    // Verifica o nome do grupo
    if (newGroup) {
      int nameSize = name.length();
      if (nameSize == 0) {
        ClientUtilities.showErrorByTextField(
          mainDialog, app.getString("MON_GROUP_NONAME_ERROR"), null, groupName);
        return false;
      }
      if (definedGroups.contains(name)) {
        ClientUtilities.showErrorByTextField(
          mainDialog, app.getString("MON_GROUP_DUPNAME_ERROR"), null, groupName);
        return false;
      }
    }

    // Verifica se lista de servidores selecionados est vazia
    List<String> selectedServers = serversSelection.getSelectedItems();
    if (selectedServers.size() == 0) {
      ClientUtilities.showErrorByTextField(
        mainDialog, app.getString("MON_GROUP_NO_SELECTED_SERVERS_ERROR"), null,
        groupName);
      return false;
    }
    // Temos que criar uma outra lista para ordenar porque o ContainerSelector
    // retorna uma lista no modificvel
    List<String> sortedServers = new ArrayList<String>();
    sortedServers.addAll(selectedServers);
    Collections.sort(sortedServers);
    String[] serverNames = new String[sortedServers.size()];
    sortedServers.toArray(serverNames);
    groupInfo = new ServerGroupInfo(name, serverNames);
    return true;
  }

/**
   * Construtor.
   *
   * @param app .
   */
  public ServerGroupDefinitionDialog(Application app) {
    this.app = app;
    this.groupInfo = null;
    this.definedGroups = null;
    this.newGroup = false;
  }
}
