/*
 * ServerGroupInfoFrame.java
 * $Author: ururahy $
 * $Revision: 56650 $  - $Date: 2006-09-18 17:54:25 -0300 (Mon, 18 Sep 2006) $
 */
package csbase.client.applications.sgamonitor;

import csbase.client.applications.*;
import csbase.client.remote.srvproxies.SGAProxy;
import csbase.logic.SGASet;
import csbase.logic.ServerGroupInfo;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.SGAServiceInterface;

import java.awt.*;
import java.awt.event.*;
import java.rmi.RemoteException;
import java.text.MessageFormat;
import java.util.Collections;
import java.util.Vector;

import javax.swing.*;
import javax.swing.event.*;

/**
 * A classe <code>ServerGroupInfoFrame</code>  responsvel por exibir as
 * informaes detalhadas de um grupo de servidores (SGAs).
 *
 * @author Ana
 * @version $Revision: 56650 $
 */
public class ServerGroupInfoFrame {
  private static int MON_PANEL = 0;
  private static int WARNING_NOSERVERS_PANEL = 1;
  private static int WARNING_SSI_PANEL = 2;

  /** Grupo monitorado */
  private ServerGroupInfo serverGroup;
  private String[] serverNames;
  private SGASet[] serversData;

  /** Thread responsvel pela atualizao das informaes dos servidores */
  private Thread updateThread = null;
  private boolean exitUpdateThread = false;

  /** Intervalo entre atualizaes de informao */
  private long updateInterval;

  /** Painis de monitorao */
  private Vector<JPanel> monPanels;

  /** Aplicao */
  private Application app;

  /** Elementos de Interface */
  private ApplicationComponentFrame mainFrame;
  private String mainFrameTitle;
  private JTextField serversNumber;
  private DynamicInfoPanel dynamicInfoPanel;
  private JButton detailButton;
  private JButton closeButton;
  private DefaultListModel serverListModel;
  private JList serverList;
  private String selectedServer;

  /**
   * Obtm a lista ordenada com os nomes dos servidores.
   *
   * @param allServers .
   *
   * @return .
   */
  private String[] getServerNames(Vector<String> allServers) {
    if ((allServers != null) && (allServers.size() > 0)) {
      Collections.sort(allServers);
      String[] names = new String[allServers.size()];
      allServers.copyInto(names);
      return names;
    }
    else {
      return new String[0];
    }
  }

  /**
   * Termina a monitorao.
   */
  private void close() {
    exitUpdateThread = true;
    try {
      updateThread.interrupt();
    }
    catch (Throwable t) {} // Applet nao pode fazer isto
  }

  /**
   * Cria a janela de monitorao de um servidor.
   *
   * @return referncia para a janela criada
   */
  private ApplicationComponentFrame createMainFrame() {
    ApplicationComponentFrame mainFrame =
      new ApplicationComponentFrame(mainFrameTitle, app);
    mainFrame.addWindowListener(
      new WindowAdapter() {
        public void windowClosing(WindowEvent we) {
          close();
        }
      });

    Container cp = mainFrame.getContentPane();
    cp.setLayout(new BorderLayout(5, 5));
    cp.add(createGeneralInfoPanel(), BorderLayout.NORTH);
    cp.add(createInfoPanel(), BorderLayout.CENTER);
    cp.add(createButtonsPanel(), BorderLayout.SOUTH);
    updateServerList();
    return mainFrame;
  }

  /**
   * Constri o painel que contm as informaes gerais do grupo.
   *
   * @return o painel criado
   */
  private JPanel createGeneralInfoPanel() {
    JPanel generalPanel = new JPanel(new GridBagLayout());
    GridBagConstraints constraints = new GridBagConstraints();

    // Atributos (constraints) comuns a todos os elementos
    Insets defInsets = new Insets(12, 2, 2, 2);
    Insets leftMarginInsets = new Insets(12, 20, 2, 2);
    constraints.anchor = GridBagConstraints.WEST; // alinhado  esquerda
    constraints.gridwidth = 1;
    constraints.gridheight = 1;

    // Nome do grupo
    constraints.fill = GridBagConstraints.NONE; // sem resize
    constraints.gridx = 0;
    constraints.gridy = 0;
    constraints.weightx = 0;
    constraints.weighty = 0;
    constraints.insets = new Insets(12, 12, 2, 2);
    generalPanel.add(new JLabel(app.getString("MON_GROUP")+" : "), constraints);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.gridx = 1;
    constraints.gridy = 0;
    constraints.weightx = 100;
    constraints.weighty = 100;
    constraints.insets = defInsets;
    JTextField name = new JTextField(serverGroup.getName());
    name.setEditable(false);
    generalPanel.add(name, constraints);

    // Nmero de servidores
    constraints.fill = GridBagConstraints.NONE; // sem resize
    constraints.gridx = 2;
    constraints.gridy = 0;
    constraints.weightx = 0;
    constraints.weighty = 0;
    constraints.insets = leftMarginInsets;
    generalPanel.add(
      new JLabel(app.getString("MON_GROUP_NUM")+" : "), constraints);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.gridx = 3;
    constraints.gridy = 0;
    constraints.weightx = 100;
    constraints.weighty = 100;
    constraints.insets = new Insets(12, 2, 2, 11);
    serversNumber = new JTextField();
    if ((serverNames != null) && (serverNames.length > 0)) {
      serversNumber.setText(""+serverNames.length);
    }
    serversNumber.setEditable(false);
    generalPanel.add(serversNumber, constraints);
    return generalPanel;
  }

  /**
   * Cria o painel de botes.
   *
   * @return painel de botes
   */
  private JPanel createButtonsPanel() {
    JPanel bp = new JPanel(new FlowLayout());

    // Boto de fechamento da janela
    closeButton = new JButton(app.getString("MON_CLOSE"));
    closeButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent ev) {
          close();
          mainFrame.setVisible(false);
        }
      });
    bp.add(closeButton);
    return bp;
  }

  /**
   * Cria o painel que contm a lista de servidores e as informaes.
   *
   * @return painel criado
   */
  private JPanel createInfoPanel() {
    /* Cria subpainel com lista de servidores */
    JPanel listPanel = new JPanel(new BorderLayout());
    listPanel.setBorder(
      BorderFactory.createTitledBorder(app.getString("MON_SERVERS")));
    serverListModel = new DefaultListModel();
    serverList = new JList(serverListModel);
    serverList.addListSelectionListener(
      new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent e) {
          if (!e.getValueIsAdjusting()) {
            int idx = serverList.getSelectedIndex();
            if (idx != -1) {
              detailButton.setEnabled(true);
              selectedServer = serverNames[idx];
            }
            else {
              detailButton.setEnabled(false);
              selectedServer = null;
            }
          }
        }
      });
    detailButton = new JButton(app.getString("MON_DETAIL"));
    detailButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (selectedServer != null) {
            new SGAInfoFrame(selectedServer, app);
          }
        }
      });
    detailButton.setEnabled(false);
    JPanel fooPanel = new JPanel();
    fooPanel.add(detailButton);
    listPanel.add(new JScrollPane(serverList), BorderLayout.CENTER);
    listPanel.add(fooPanel, BorderLayout.SOUTH);

    /* Painel completo */
    JPanel infoPanel = new JPanel(new GridBagLayout());
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.anchor = GridBagConstraints.WEST; // alinhado  esquerda
    constraints.gridwidth = 1;
    constraints.gridheight = 1;

    // Lista
    constraints.fill = GridBagConstraints.BOTH;
    constraints.gridx = 0;
    constraints.gridy = 0;
    constraints.weightx = 25;
    constraints.weighty = 100;
    constraints.insets = new Insets(2, 12, 2, 2);
    infoPanel.add(listPanel, constraints);
    constraints.fill = GridBagConstraints.BOTH;
    constraints.gridx = 1;
    constraints.gridy = 0;
    constraints.weightx = 75;
    constraints.weighty = 100;
    constraints.insets = new Insets(2, 11, 2, 11);
    infoPanel.add(dynamicInfoPanel, constraints);
    return infoPanel;
  }

  /**
   * Cria os painis de monitorao.
   *
   * @return um vetor com os painis criados
   */
  private Vector<JPanel> createMonitorPanels() {
    Vector<JPanel> panels = new Vector<JPanel>();
    panels.add(new GroupStarUsagePanel(app));
    panels.add(new GroupSliceUsagePanel(app));
    panels.add(new GroupBarsUsagePanel(app));
    return panels;
  }

  /**
   * Atualiza a lista de servidores.
   */
  private void updateServerList() {
    String sel = selectedServer;
    serverListModel.removeAllElements();
    if (serverNames != null) {
      for (int i = 0; i < serverNames.length; i++) {
        serverListModel.addElement(serverNames[i]);
      }
      if (sel != null) {
        serverList.setSelectedValue(sel, true);
      }
    }
  }

  /**
   * Painel que contm as informaes dinmicas do grupo.
   */
  private class DynamicInfoPanel extends JPanel {
    private JLabel warningMessage;
    private Application app;
    private int currentDynPanel;
    private DynamicMonitorPanel currentMonPanel;

    /**
     * Atualiza as informaes dinmicas do servidor. Essa atualizao 
     * comandada pela thread de tratamento de eventos do AWT.
     */
    public void updateInfo() {
      // Verifica o estado da comunicao com o servidor
      if (serversData == null) {
        if (currentDynPanel != WARNING_NOSERVERS_PANEL) {
          ((CardLayout)getLayout()).show(
            this, String.valueOf(WARNING_NOSERVERS_PANEL));
          currentDynPanel = WARNING_NOSERVERS_PANEL;
        }
        serversNumber.setText("0");
      }
      else {
        if (
          (currentDynPanel == WARNING_NOSERVERS_PANEL) ||
          (currentDynPanel == WARNING_SSI_PANEL)) {
          ((CardLayout)getLayout()).show(this, String.valueOf(MON_PANEL));
          currentDynPanel = MON_PANEL;
        }
        serversNumber.setText(""+serversData.length);
        if (serverGroup.containsAllServers()) {
          updateServerList();
        }

        // Atualiza informaes em todos os painis registrados
        for (int i = 0; i < monPanels.size(); i++) {
          DynamicMonitorPanel mp = (DynamicMonitorPanel)monPanels.get(i);
          mp.updateDynamicInfo(serversData);
        }

        // Redesenha o painel de monitorao corrente
        if (currentMonPanel != null) {
          currentMonPanel.repaint();
        }
      }
    }

    /**
     * Invalida a monitorao (mostrando painel de erro).
     */
    private void invalidateInfo() {
      if (currentDynPanel != WARNING_SSI_PANEL) {
        ((CardLayout)getLayout()).show(this, String.valueOf(WARNING_SSI_PANEL));
        currentDynPanel = WARNING_SSI_PANEL;
      }
    }

    /**
     * Constri o painel que contm as informaes dinmicas do grupo. Este
     * painel  diagramado com um CardLayout para permitir a  visualizao dos
     * painis de alarme em sobreposio ao painel de monitorao.
     *
     * @param app .
     */
    public DynamicInfoPanel(Application app) {
      super(new CardLayout());
      this.app = app;

      // Cria os painis de monitorao de acordo com o tipo de servidor
      monPanels = createMonitorPanels();
      final JTabbedPane monitorTabs = new JTabbedPane();
      if (monPanels != null) {
        for (int i = 0; i < monPanels.size(); i++) {
          DynamicMonitorPanel dp = (DynamicMonitorPanel)monPanels.get(i);
          monitorTabs.add(dp.getTitle(), (JPanel)dp);
        }
        currentMonPanel = (DynamicMonitorPanel)monPanels.get(0);
      }
      else {
        currentMonPanel = null;
      }
      monitorTabs.addChangeListener(
        new ChangeListener() {
          public void stateChanged(ChangeEvent ce) {
            DynamicMonitorPanel dp =
              (DynamicMonitorPanel)monitorTabs.getSelectedComponent();
            if (dp != null) {
              currentMonPanel = dp;
              currentMonPanel.repaint();
            }
          }
        });
      add(monitorTabs, String.valueOf(MON_PANEL));
      currentDynPanel = MON_PANEL;

      // Cria o painel de alarme (no h servidores registrados)
      JPanel warningNoServPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
      JLabel warningNoServMessage =
        new JLabel(app.getString("MON_GROUP_NO_SERVERS"));
      warningNoServMessage.setIcon(ApplicationImages.ICON_ERROR_32);
      warningNoServPanel.add(warningNoServMessage);
      add(warningNoServPanel, String.valueOf(WARNING_NOSERVERS_PANEL));

      // Cria o painel de alarme (erro comunicao com SSI)
      JPanel warningSSIPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
      JLabel warningSSIMessage =
        new JLabel(
          MessageFormat.format(
            app.getString("MON_GROUP_NO_INFO"),
            new Object[] { serverGroup.getName() }));
      warningSSIMessage.setIcon(ApplicationImages.ICON_ERROR_32);
      warningSSIPanel.add(warningSSIMessage);
      add(warningSSIPanel, String.valueOf(WARNING_SSI_PANEL));
    }
  }

  /**
   * Constri o monitor de um grupo de servidores
   *
   * @param Group grupo de servidores
   * @param app aplicao
   */
  public ServerGroupInfoFrame(ServerGroupInfo Group, Application app) {
    this.serverGroup = Group;
    this.app = app;
    serversData = null;
    if (serverGroup.containsAllServers()) {
      serverNames = getServerNames(SGAProxy.getAllSGANames());
    }
    else {
      serverNames = serverGroup.getServers();
    }
    selectedServer = null;

    /* Obtem o intervalo de atualizao */
    int updateIntervalAsInt = SGAProxy.getUpdateInterval();
    if (updateIntervalAsInt == -1) {
      return;
    }
    updateInterval = (long)updateIntervalAsInt*(long)1000;

    /* Constri e exibe os elementos de interface */
    mainFrameTitle =
      MessageFormat.format(
        app.getString("MON_GROUP_TITLE"), new Object[] { serverGroup.getName() });
    dynamicInfoPanel = new DynamicInfoPanel(app);
    mainFrame = createMainFrame();
    mainFrame.pack();
    mainFrame.center(app.getApplicationFrame());
    mainFrame.setVisible(true);

    /*
     * Thread responsvel pela atualizao das informaes dinmicas
     */
    updateThread =
      new Thread(
        new Runnable() {
          public void run() {
            while (!exitUpdateThread) {
              try {
                SGAServiceInterface sgaService = ClientRemoteLocator.sgaService;
                if (serverGroup.containsAllServers()) {
                  Vector<String> allServers = sgaService.getAllSGANames();
                  serverNames = getServerNames(allServers);
                }
                int numServers = serverNames.length;
                if (numServers == 0) {
                  serversData = null;
                }
                else {
                  serversData = new SGASet[numServers];
                  for (int i = 0; i < numServers; i++) {
                    serversData[i] = sgaService.getSGASet(serverNames[i]);
                  }
                }

                // Enfileira a atualizao das informaes dinmicas
                SwingUtilities.invokeLater(
                  new Runnable() {
                    public void run() {
                      dynamicInfoPanel.updateInfo();
                    }
                  });
                Thread.sleep(updateInterval);
              }
              catch (InterruptedException e) {}
              catch (RemoteException e) {
                dynamicInfoPanel.invalidateInfo();
              }
            }
          }
        });
    updateThread.start();
  }
}
