/*
 * ServerGroupMonitor.java
 *
 * $Author: ururahy $
 * $Revision: 56650 $  - $Date: 2006-09-18 17:54:25 -0300 (Mon, 18 Sep 2006) $
 */
package csbase.client.applications.sgamonitor;

import csbase.client.applications.Application;
import csbase.client.desktop.DesktopFrame;
import csbase.client.remote.srvproxies.SGAProxy;
import csbase.client.util.ClientUtilities;
import csbase.logic.ServerGroupInfo;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.text.MessageFormat;

import java.util.Collections;
import java.util.Comparator;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import tecgraf.javautils.gui.StandardDialogs;

/**
 * A classe <code>ServerGroupMonitorFrame</code> representa um dilogo de
 * monitorao de grupos de servidores de algoritmos
 *
 * @author Ana
 * @version $Revision: 56650 $
 */
public class ServerGroupMonitorFrame implements MonitorFrame {
  /** Controle para s permitir uma janela */

  //private static boolean isOpened = false;

  /** Aplicao */
  private Application app;

  /** Grupos definidos */
  private Vector<ServerGroupInfo> groups;

  /** Grupo selecionado */
  ServerGroupInfo selectedGroup;
  private String mainFrameTitle;
  private JButton monitorButton;
  private JButton updateButton;
  private JButton removeButton;
  private JList groupList;
  private DefaultListModel groupListModel;
  private DefaultListModel serverListModel;

  /**
   * Abre a janela somente se no est aberta.
   *
   * @return .
   */

  /*  public static void open() {
     if (!isOpened) {
       isOpened = true;
       new ServerGroupMonitorFrame();
     }
     else
      mainFrame.toFront();
     }
   */

  /**
   * Constri um monitor de grupos de servidores.
   *
   * @return .
   */

  //  private ServerGroupMonitorFrame() {
  //}   

  /**
   * Obtm os grupos definidos pelo usurio.
   *
   * @return um vetor com os grupos
   */
  private Vector<ServerGroupInfo> getServerGroups() {
    Vector<ServerGroupInfo> srvGroups = new Vector<ServerGroupInfo>();
    ServerGroupInfo[] defGroups = SGAProxy.getServerGroups();
    if (defGroups != null) {
      for (int i = 0; i < defGroups.length; i++) {
        srvGroups.add(defGroups[i]);
      }
    }
    srvGroups.add(
      ServerGroupInfo.createAllServersGroupInfo(
        app.getString("MON_GROUP_ALL_NAME")));
    return srvGroups;
  }

  /**
   * Atualiza os grupos definidos pelo usurio.
   *
   * @param srvGroups vetor com os grupos
   */
  private void setServerGroups(Vector srvGroups) {
    ServerGroupInfo[] info = new ServerGroupInfo[srvGroups.size()-1];
    int grp = 0;
    for (int i = 0; i < srvGroups.size(); i++) {
      ServerGroupInfo grpInfo = (ServerGroupInfo)srvGroups.get(i);
      if (!grpInfo.containsAllServers()) {
        info[grp++] = grpInfo;
      }
    }
    SGAProxy.setServerGroups(info);
  }

  /**
   * Termina a monitorao.
   */

  /*  private void close() {
     if (mainFrame != null) {
       mainFrame.close();
       mainFrame = null;
     }
     isOpened = false;
     } */

  /**
   * Cria a janela de monitorao dos grupos.
   */
  private void createMainFrame() {
    Container cp = this.app.getApplicationFrame().getContentPane();
    cp.add(createLists(), BorderLayout.CENTER);
    cp.add(createButtons(), BorderLayout.SOUTH);
    populateGroupList();
    groupList.setSelectedIndex(0);
  }

  /**
   * Cria o painel com as listas dos grupos.
   *
   * @return o painel criado
   */
  private JPanel createLists() {
    JPanel listPanel = new JPanel(new GridBagLayout());
    GridBagConstraints constraints = new GridBagConstraints();
    Insets defInsets = new Insets(2, 2, 2, 2);
    Insets leftMarginInsets = new Insets(2, 20, 2, 2);

    // Legendas
    constraints.anchor = GridBagConstraints.WEST; // alinhado  esquerda
    constraints.gridwidth = 1;
    constraints.gridheight = 1;
    constraints.fill = GridBagConstraints.NONE; // sem resize
    constraints.weightx = 0;
    constraints.weighty = 0;
    constraints.insets = defInsets;
    constraints.gridx = 0;
    constraints.gridy = 0;
    listPanel.add(new JLabel(app.getString("MON_GROUPS")+" : "), constraints);
    constraints.gridx = 1;
    constraints.gridy = 0;
    listPanel.add(
      new JLabel(app.getString("MON_GROUP_SERVERS")+" : "), constraints);

    // Listas
    groupListModel = new DefaultListModel();
    groupList = new JList(groupListModel);
    groupList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    groupList.addListSelectionListener(
      new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent e) {
          if (!e.getValueIsAdjusting()) {
            int idx = groupList.getSelectedIndex();
            updateGroupSelection(idx);
          }
        }
      });
    constraints.anchor = GridBagConstraints.WEST; // alinhado  esquerda
    constraints.gridwidth = 1;
    constraints.gridheight = 1;
    constraints.fill = GridBagConstraints.BOTH;
    constraints.weightx = 100;
    constraints.weighty = 100;
    constraints.insets = defInsets;
    constraints.gridx = 0;
    constraints.gridy = 1;
    listPanel.add(new JScrollPane(groupList), constraints);
    serverListModel = new DefaultListModel();
    JList serverList = new JList(serverListModel);
    serverList.setEnabled(false);
    constraints.gridx = 1;
    constraints.gridy = 1;
    listPanel.add(new JScrollPane(serverList), constraints);
    return listPanel;
  }

  /**
   * Cria o painel de botes.
   *
   * @return o painel de botes criado
   */
  private JPanel createButtons() {
    JPanel panel = new JPanel(new FlowLayout());
    monitorButton = new JButton(app.getString("MON_DETAIL"));
    monitorButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          new ServerGroupInfoFrame(selectedGroup, app);
        }
      });

    JButton createButton = new JButton(app.getString("MON_CREATE_GROUP"));
    createButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          Vector<String> definedGroups = new Vector<String>();
          for (int i = 0; i < groups.size(); i++) {
            definedGroups.add(((ServerGroupInfo)groups.get(i)).getName());
          }
          ServerGroupInfo newGroup =
            new ServerGroupDefinitionDialog(app).showCreateDialog(
              definedGroups);
          if (newGroup != null) {
            groups.add(newGroup);
            populateGroupList();
            groupList.setSelectedValue(newGroup.getName(), true);
            setServerGroups(groups);
          }
        }
      });
    updateButton = new JButton(app.getString("MON_UPDATE_GROUP"));
    updateButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          ServerGroupInfo newGroup =
            new ServerGroupDefinitionDialog(app).showUpdateDialog(
              selectedGroup);
          if (newGroup != null) {
            groups.set(groups.indexOf(selectedGroup), newGroup);
            selectedGroup = newGroup;
            populateServerList();
            setServerGroups(groups);
          }
        }
      });
    updateButton.setEnabled(false);
    removeButton = new JButton(app.getString("MON_REMOVE_GROUP"));
    removeButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          removeSelectedGroup();
          setServerGroups(groups);
        }
      });
    removeButton.setEnabled(false);
    JButton closeButton = new JButton(app.getString("MON_CLOSE"));
    closeButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent ev) {
          app.finishApplication();
        }
      });
    ClientUtilities.adjustEqualSizes(
      new JButton[] {
        monitorButton, closeButton, createButton, updateButton, removeButton
      });
    panel.add(monitorButton);
    panel.add(createButton);
    panel.add(updateButton);
    panel.add(removeButton);
    panel.add(closeButton);
    return panel;
  }

  /**
   * Altera o estado dos botes que dependem de seleo de grupo.
   *
   * @param state novo estado dos botes
   */
  private void changeButtonsState(boolean state) {
    monitorButton.setEnabled(state);
    updateButton.setEnabled(state);
    removeButton.setEnabled(state);
  }

  /**
   * Atualiza informaes do grupo selecionado.
   *
   * @param idx ndice do grupo selecionado
   */
  private void updateGroupSelection(int idx) {
    if (idx == -1) {
      monitorButton.setEnabled(false);
      updateButton.setEnabled(false);
      removeButton.setEnabled(false);
      selectedGroup = null;
    }
    else {
      monitorButton.setEnabled(true);
      selectedGroup = (ServerGroupInfo)groups.get(idx);
      if (selectedGroup.containsAllServers()) {
        updateButton.setEnabled(false);
        removeButton.setEnabled(false);
      }
      else {
        updateButton.setEnabled(true);
        removeButton.setEnabled(true);
      }
    }
    populateServerList();
  }

  /**
   * Preenche a lista de grupos com os grupos definidos.
   */
  private void populateGroupList() {
    Collections.sort(
      groups,
      new Comparator<ServerGroupInfo>() {
        public int compare(ServerGroupInfo sg1, ServerGroupInfo sg2) {
          return sg1.getName().compareTo(sg2.getName());
        }

        public boolean equals(Object o) {
          return super.equals(o);
        }
      });
    groupListModel.removeAllElements();
    for (int i = 0; i < groups.size(); i++) {
      groupListModel.addElement(((ServerGroupInfo)groups.get(i)).getName());
    }
  }

  /**
   * Preenche a lista de servidores de acordo com o grupo selecionado.
   */
  private void populateServerList() {
    serverListModel.removeAllElements();
    if (selectedGroup == null) {
      return;
    }
    String[] servers;
    if (selectedGroup.containsAllServers()) {
      Vector<String> allServers = SGAProxy.getAllSGANames();
      if ((allServers != null) && (allServers.size() > 0)) {
        Collections.sort(allServers);
        servers = new String[allServers.size()];
        allServers.copyInto(servers);
      }
      else {
        servers = null;
      }
    }
    else {
      servers = selectedGroup.getServers();
    }
    if ((servers == null) || (servers.length == 0)) {
      return;
    }
    for (int i = 0; i < servers.length; i++) {
      serverListModel.addElement(servers[i]);
    }
  }

  /**
   * Remove o grupo selecionado.
   */
  private void removeSelectedGroup() {
    int res =
      StandardDialogs.showYesNoDialog(
        this.app.getApplicationFrame(), mainFrameTitle,
        MessageFormat.format(
          app.getString("MON_GROUP_REMOVECONFIRM"),
          new Object[] { selectedGroup.getName() }));
    if (res != JOptionPane.YES_OPTION) {
      return;
    }
    int oldIndex = groups.indexOf(selectedGroup);
    groups.remove(selectedGroup);
    groupListModel.removeElement(selectedGroup.getName());
    if (oldIndex < groups.size()) {
      groupList.setSelectedIndex(oldIndex);
    }
    else {
      groupList.setSelectedIndex(oldIndex-1);
    }
  }

  /**
   * Termina a monitorao.
   */
  public void close() {}

  /**
   * Construtor padro. Constri um monitorador de servidores (SGA's) apenas
   * com as colunas padro.
   *
   * @param app .
   */
  public ServerGroupMonitorFrame(Application app) {
    this.app = app;
    groups = getServerGroups();
    selectedGroup = null;
    createMainFrame();
    this.app.getApplicationFrame().pack();
    this.app.getApplicationFrame().center(
      DesktopFrame.getInstance().getDesktopFrame());
    this.app.getApplicationFrame().setVisible(true);
  }
}
