/*
 * TrendUsagePanel.java
 * @version $Revision: 36552 $
 */
package csbase.client.applications.sgamonitor;

import csbase.client.applications.Application;
import csbase.client.util.MonitoringTheme;
import csbase.logic.SGASet;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.*;

/**
 * Classe que representa o elemento star-graph
 *
 * @author Andr Luiz Clinio
 */
class TrendCanvas extends JPanel {
  /*
   * A Aplicao
   */
  private Application app;
  /** Constante de definio do numero maximo de pontos no grafico */
  private final int maxPixels = 2000;
  /** Constante de definio da escala de tempo (pixels por segundo) */
  private final int pixelsPerSecond = 5;
  /** Constante de definio da escala de tempo (pixels por unid.) */
  private final int pixelsMargin = 30;
  /** Flag indicativo de desenho de rea de ocupao */
  private boolean drawRegions = true;
  /** Flag indicativo de desenho de grid horizontal */
  private boolean drawGridX = false;
  /** Flag indicativo de desenho de grid vertical */
  private boolean drawGridY = true;
  /**
   *
   */
  private int refreshUpdate = 5;
  /**
   *
   */
  private Vector<Double> RAMMemoVector = null;
  private Vector<Double> swapMemoVector = null;
  /**
   *
   */
  private Vector<Double> loadVector = null;

  /**
   *
   */
  private void drawBackground(Graphics graphics) {
    int width = getWidth();
    int height = getHeight();
    graphics.setColor(Color.WHITE);
    graphics.fillRect(0, 0, width, height);
  }

  /**
   *
   */
  private int timeToPixel(int time) {
    int width = getWidth();
    int zero = width - pixelsMargin;
    return zero - (pixelsPerSecond * time);
  }

  /**
   *
   */
  private int loadToPixel(double load) {
    int height = getHeight();
    double B = (height - pixelsMargin);
    double A = (pixelsMargin - B) / 100;
    return (int)((A * load) + B);
  }

  /**
   *
   */
  private int getNumTimes() {
    int width = getWidth();
    int size = width - (pixelsMargin * 2);
    return ((int)(size / (pixelsPerSecond))) - 1;
  }

  /**
   *
   */
  private void drawForeground(Graphics graphics) {
    int numTimes = getNumTimes();
    int zx = timeToPixel(0);
    int zy = loadToPixel(0);
    int lx = timeToPixel(numTimes);
    int ly = loadToPixel(100);
    int ly1;
    int ly2;
    if (drawRegions) {
      graphics.setColor(MonitoringTheme.getRangeNormalColor());
      ly1 = loadToPixel(50);
      ly2 = loadToPixel(0);
      graphics.fillRect(lx, ly1, Math.abs(zx - lx), Math.abs(ly1 - ly2));
      graphics.setColor(MonitoringTheme.getRangeWarningColor());
      ly1 = loadToPixel(75);
      ly2 = loadToPixel(50);
      graphics.fillRect(lx, ly1, Math.abs(zx - lx), Math.abs(ly1 - ly2));
      graphics.setColor(MonitoringTheme.getRangeAlertColor());
      ly1 = loadToPixel(100);
      ly2 = loadToPixel(75);
      graphics.fillRect(lx, ly1, Math.abs(zx - lx), Math.abs(ly1 - ly2));
    }
    else {
      graphics.setColor(MonitoringTheme.getRangeDefaultColor());
      graphics.fillRect(lx, ly, Math.abs(zx - lx), Math.abs(ly - zy));
    }
  }

  /**
   *
   */
  private void drawCurve(Graphics graphics, Color color, Vector vector) {
    int size = vector.size();
    if (size <= 2) {
      return;
    }
    graphics.setColor(color);
    for (int t = 0; t < (size - 1); t++) {
      int idx = size - t - 1;
      int x1 = timeToPixel(t);
      double v1 = ((Double)vector.get(idx)).doubleValue();
      if (v1 > 100) {
        v1 = 100;
      }
      int y1 = loadToPixel(v1);
      int x2 = timeToPixel(t + 1);
      double v2 = ((Double)vector.get(idx - 1)).doubleValue();
      if (v2 > 100) {
        v2 = 100;
      }
      int y2 = loadToPixel(v2);
      graphics.drawLine(x1, y1, x2, y1);
      graphics.drawLine(x1, y1 + 1, x2, y1 + 1);
      graphics.drawLine(x2, y1, x2, y2);
      graphics.drawLine(x2 + 1, y1, x2 + 1, y2);
    }
  }

  /**
   *
   */
  private void drawGrids(Graphics graphics) {
    int tsize = 2;
    int numTimes = getNumTimes();
    int zx = timeToPixel(0);
    int zy = loadToPixel(0);
    int lx = timeToPixel(numTimes);
    int ly = loadToPixel(100);
    /* Grid X */
    for (int i = 0; i < numTimes; i += 10) {
      int px = timeToPixel(i);
      graphics.setColor(MonitoringTheme.getAxisColor());
      graphics.drawLine(px, zy - tsize, px, zy + tsize);
      String str = Integer.toString(-i * refreshUpdate) + "s";
      if (i == 0) {
        str = app.getString("MON_TREND_NOW");
      }
      graphics.drawString(str, px, zy + (tsize * 6));
      if (drawGridX) {
        graphics.setColor(MonitoringTheme.getGridColor());
        graphics.drawLine(px, zy, px, ly);
      }
    }
    /* Grid Y */
    for (int i = 10; i <= 100; i += 10) {
      int py = loadToPixel(i);
      graphics.setColor(MonitoringTheme.getAxisColor());
      graphics.drawLine(zx - tsize, py, zx + tsize, py);
      graphics.drawString(Integer.toString(i), zx + (tsize * 2), py);
      if (drawGridY) {
        graphics.setColor(MonitoringTheme.getGridColor());
        graphics.drawLine(lx, py, zx, py);
      }
    }
  }

  /**
   *
   */
  private void drawAxis(Graphics graphics) {
    int numTimes = getNumTimes();
    int zx = timeToPixel(0);
    int zy = loadToPixel(0);
    int lx = timeToPixel(numTimes);
    int ly = loadToPixel(100);
    graphics.setColor(MonitoringTheme.getAxisColor());
    int asize = (int)(pixelsMargin / 2.5);
    /* Eixo X */
    graphics.drawLine(zx, zy, lx, zy);
    graphics.drawLine(zx, zy + 1, lx, zy + 1);
    graphics.fillPolygon(
      new Polygon(
        new int[] { lx, lx, lx - asize, lx },
        new int[] { zy, zy - (asize / 2), zy, zy + (asize / 2) }, 4));
    /* Eixo Y */
    graphics.drawLine(zx, zy, zx, ly);
    graphics.drawLine(zx + 1, zy, zx + 1, ly);
    graphics.fillPolygon(
      new Polygon(
        new int[] { zx, zx + (asize / 2), zx, zx - (asize / 2) },
        new int[] { ly, ly, ly - asize, ly }, 4));
    graphics.drawString("(%)", zx, ly - asize);
  }

  /**
   *
   */
  public void setRegionsMode(boolean mode) {
    drawRegions = mode;
  }

  /**
   *
   */
  public void setGridYMode(boolean mode) {
    drawGridY = mode;
  }

  /**
   *
   */
  public void setGridXMode(boolean mode) {
    drawGridX = mode;
  }

  /**
   *
   */
  public void paintComponent(Graphics g) {
    super.paintComponent(g);
    drawGraph(g.create());
  }

  /**
   *
   */
  public void addLoadData(double cpu, double RAMMemo, double swapMemo) {
    loadVector.add(new Double(cpu));
    RAMMemoVector.add(new Double(RAMMemo));
    swapMemoVector.add(new Double(swapMemo));
    if (loadVector.size() > maxPixels) {
      loadVector.removeElementAt(0);
      RAMMemoVector.removeElementAt(0);
      swapMemoVector.removeElementAt(0);
    }
  }

  /**
   *
   */
  public void drawGraph(Graphics graphics) {
    if (
      (loadVector == null) || (RAMMemoVector == null) ||
        (swapMemoVector == null)) {
      return;
    }
    int width = getWidth();
    int height = getHeight();
    drawBackground(graphics);
    drawForeground(graphics);
    drawAxis(graphics);
    drawGrids(graphics);
    clip(graphics);
    drawCurve(graphics, TrendUsagePanel.CPU_COLOR, loadVector);
    drawCurve(graphics, TrendUsagePanel.RAM_MEMORY_COLOR, RAMMemoVector);
    drawCurve(graphics, TrendUsagePanel.SWAP_MEMORY_COLOR, swapMemoVector);
    unClip(graphics);
  }

  /**
   *
   */
  public void unClip(Graphics graphics) {
    int width = getWidth();
    int height = getHeight();
    graphics.setClip(0, 0, width, height);
  }

  /**
   *
   */
  public void clip(Graphics graphics) {
    int numTimes = getNumTimes();
    int zx = timeToPixel(0);
    int zy = loadToPixel(0);
    int lx = timeToPixel(numTimes);
    int ly = loadToPixel(100);
    graphics.setClip(lx, ly, Math.abs(zx - lx), Math.abs(zy - ly));
  }

  /**
   * Construtor padro.
   *
   * @param app .
   * @param refreshUpdate .
   */
  public TrendCanvas(Application app, int refreshUpdate) {
    super();
    this.app = app;
    this.refreshUpdate = refreshUpdate;
    loadVector = new Vector<Double>();
    RAMMemoVector = new Vector<Double>();
    swapMemoVector = new Vector<Double>();
    Dimension size = new Dimension(500, 300);
    setMinimumSize(size);
    setPreferredSize(size);
  }
}


/**
 * A classe <code>TrendUsagePanel</code> modela o painel de monitorao que
 * exibe as cargas de um SGA como uma estrela de carga.
 *
 * @author Andr Luiz Clinio
 */
class TrendUsagePanel extends DynamicMonitorPanel {
  /** Constante de definio da cor da curva de memria. */
  public static final Color RAM_MEMORY_COLOR =
    MonitoringTheme.getRAMMemoryColor();
  public static final Color SWAP_MEMORY_COLOR =
    MonitoringTheme.getSwapMemoryColor();
  /** Constante de definio da cor da curva de carga na CPU. */
  public static final Color CPU_COLOR =
    MonitoringTheme.getProcessorColor();
  /*
   * A Aplicao
   */
  private Application app;
  /** Painel de exibio do grfico de tendncias. */
  private TrendCanvas canvasPanel = null;

  /**
   * Retorna o ttulo do painel.
   *
   * @return ttulo do painel
   */
  public String getTitle() {
    return app.getString("MON_SINGLE_TREND_USAGE");
  }

  /**
   * Retorna o painel que contm a legenda do grfico mostrado no canvas
   *
   * @return .
   */
  private JPanel createLegendPanel() {
    JPanel legendPanel = new JPanel();
    JLabel RAMMemoColorLabel = new JLabel("\u25a0");
    RAMMemoColorLabel.setForeground(RAM_MEMORY_COLOR);
    RAMMemoColorLabel.setBackground(RAM_MEMORY_COLOR);
    JTextArea RAMMemoLabel =
      new JTextArea(app.getString("MON_TREND_RAM_MEMORY") + "           ");
    RAMMemoLabel.setEditable(false);
    legendPanel.add(RAMMemoColorLabel);
    legendPanel.add(RAMMemoLabel);
    JLabel swapMemoColorLabel = new JLabel("\u25a0");
    swapMemoColorLabel.setForeground(SWAP_MEMORY_COLOR);
    swapMemoColorLabel.setBackground(SWAP_MEMORY_COLOR);
    JTextArea swapMemoLabel =
      new JTextArea(app.getString("MON_TREND_SWAP_MEMORY") + "           ");
    swapMemoLabel.setEditable(false);
    legendPanel.add(swapMemoColorLabel);
    legendPanel.add(swapMemoLabel);
    JLabel cpuColorLabel = new JLabel("\u25a0");
    cpuColorLabel.setForeground(CPU_COLOR);
    cpuColorLabel.setBackground(CPU_COLOR);
    JTextArea cpuLabel = new JTextArea(app.getString("MON_TREND_CPU"));
    cpuLabel.setEditable(false);
    legendPanel.add(cpuColorLabel);
    legendPanel.add(cpuLabel);
    legendPanel.setBackground(Color.WHITE);
    return legendPanel;
  }

  /**
   * Retorna o painel que contm o canvas e sua a legenda
   *
   * @param refreshUpdate .
   *
   * @return .
   */
  private JPanel createCanvasPanel(int refreshUpdate) {
    JPanel canvasLegendPanel = new JPanel();
    canvasLegendPanel.setLayout(new BorderLayout());
    JPanel legendPanel = createLegendPanel();
    canvasPanel = new TrendCanvas(app, refreshUpdate);
    canvasLegendPanel.add(canvasPanel, BorderLayout.CENTER);
    canvasLegendPanel.add(legendPanel, BorderLayout.SOUTH);
    canvasLegendPanel.setBorder(
      BorderFactory.createCompoundBorder(
        BorderFactory.createLoweredBevelBorder(),
        BorderFactory.createEmptyBorder(10, 10, 10, 10)));
    canvasLegendPanel.setBackground(Color.WHITE);
    return canvasLegendPanel;
  }

  /**
   *
   */
  private JPanel createTogglePanel() {
    JPanel togPanel = new JPanel();
    togPanel.setLayout(new GridLayout(2, 2));
    final JCheckBox gridXToggle =
      new JCheckBox(app.getString("MON_TREND_GRID_X_TOGGLE"), false);
    gridXToggle.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent ae) {
          canvasPanel.setGridXMode(gridXToggle.isSelected());
          canvasPanel.repaint();
        }
      });

    final JCheckBox gridYToggle =
      new JCheckBox(app.getString("MON_TREND_GRID_Y_TOGGLE"), true);
    gridYToggle.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent ae) {
          canvasPanel.setGridYMode(gridYToggle.isSelected());
          canvasPanel.repaint();
        }
      });

    final JCheckBox regionsToggle =
      new JCheckBox(app.getString("MON_TREND_REGIONS_TOGGLE"), true);
    regionsToggle.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent ae) {
          canvasPanel.setRegionsMode(regionsToggle.isSelected());
          canvasPanel.repaint();
        }
      });
    togPanel.add(regionsToggle);
    togPanel.add(gridXToggle);
    togPanel.add(gridYToggle);
    return togPanel;
  }

  /**
   * Atualiza as informaes dinmicas exibidas
   *
   * @param dynamicData informaes do servidor
   */
  public void updateDynamicInfo(Object dynamicData) {
    if (canvasPanel == null) {
      return;
    }
    SGASet serverData = (SGASet)dynamicData;
    double cpu = serverData.getCPULoad1();
    double RAMMemo = 100 - serverData.getRAMFreeMemory();
    double swapMemo = 100 - serverData.getSwapFreeMemory();
    canvasPanel.addLoadData(cpu, RAMMemo, swapMemo);
  }

  /**
   * Constri o painel que contm as informaes dinmicas do servidor.
   *
   * @param app .
   * @param updateIntervalInSec .
   */
  public TrendUsagePanel(Application app, long updateIntervalInSec) {
    this.app = app;
    this.setLayout(new BorderLayout());
    add(createCanvasPanel((int)updateIntervalInSec), BorderLayout.CENTER);
    // add( createLabelsPanel(), BorderLayout.CENTER );
    add(createTogglePanel(), BorderLayout.SOUTH);
    add(new JPanel(), BorderLayout.WEST);
    add(new JPanel(), BorderLayout.EAST);
  }
}
