/*
 * $Id: AboutDialog.java 91457 2009-05-13 02:09:54Z clinio $
 */

package csbase.client.desktop;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import tecgraf.javautils.LNG;
import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.configurationmanager.ConfigurationManagerException;
import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.GUIUtils;
import csbase.client.Client;
import csbase.client.login.LoginUI;
import csbase.exception.ConfigurationException;
import csbase.logic.Version;

/**
 * Dilogo de about do sistema.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class AboutDialog extends DesktopComponentDialog {

  /**
   * Boto de fechamento.
   */
  private JButton closeButton;

  /**
   * Cria o painel de botes.
   * 
   * @return painel
   */
  protected JPanel buildButtonsPanel() {
    final JPanel panel = new JPanel(new FlowLayout());

    closeButton = new JButton(getString("close.button"));

    closeButton.addKeyListener(new KeyAdapter() {
      @Override
      public void keyPressed(final KeyEvent evt) {
        final int pressed = evt.getKeyCode();
        // Se for apertado o botao ENTER
        if (pressed == KeyEvent.VK_ENTER) {
          close();
        }
      }
    });

    closeButton.addActionListener(new ActionListener() {
      public void actionPerformed(final ActionEvent ev) {
        close();
      }
    });

    panel.add(closeButton);
    return panel;
  }

  /**
   * Cria o painel de botes.
   * 
   * @return painel
   */
  protected JPanel buildMainPanel() {
    final JLabel versionLabel = new JLabel(getString("version.label"));
    final JLabel dateLabel = new JLabel(getString("date.label"));
    final JLabel serverLabel = new JLabel(getString("server.label"));
    final JLabel addrLabel = new JLabel(getString("address.label"));
    final JLabel systemLabel = new JLabel(getString("system.label"));

    final JTextField versionText = new JTextField(10);
    final JTextField dateText = new JTextField(10);
    final JTextField serverText = new JTextField(10);
    final JTextField addrText = new JTextField(10);
    final JTextField systemText = new JTextField(10);

    versionText.setEditable(false);
    dateText.setEditable(false);
    serverText.setEditable(false);
    addrText.setEditable(false);
    systemText.setEditable(false);

    final Client client = Client.getInstance();
    final String serverName = client.getServerName();
    String addrName;
    try {
      addrName = client.getServerAddress();
    }
    catch (final ConfigurationException e) {
      addrName = getString("address.value.error");
    }
    final String systemName = client.getSystemName();
    serverText.setText(serverName);
    addrText.setText(addrName);
    systemText.setText(systemName);

    final Version version = Version.getInstance();
    versionText.setText(version.getVersion());
    dateText.setText(version.getReleaseDate());

    final JComponent[][] cmps =
      new JComponent[][] { { systemLabel, systemText },
          { serverLabel, serverText }, { addrLabel, addrText },
          { versionLabel, versionText }, { dateLabel, dateText },

      };
    final JPanel fieldsPanel = GUIUtils.createBasicGridPanel(cmps);

    final JLabel imageLabel = createImageLabel();
    final JPanel panel = new JPanel();
    panel.setLayout(new GridBagLayout());

    if (imageLabel != null) {
      final GBC gbcImg = new GBC(0, 0).north();
      gbcImg.weighty(1);
      panel.add(imageLabel, gbcImg);
    }
    panel.add(fieldsPanel, new GBC(1, 0).horizontal().north());
    return panel;
  }

  /**
   * Cria a imagem do dilogo (com base no LoginUI, se tiver sendo usado)
   * 
   * @return A imagem , ou null, caso no tenha sido informada.
   */
  private JLabel createImageLabel() {
    final Configuration configuration;
    try {
      configuration =
        ConfigurationManager.getInstance().getConfiguration(LoginUI.class);
    }
    catch (final ConfigurationManagerException e) {
      return null;
    }
    final String propName = "image.path";
    final String imagePath = configuration.getOptionalProperty(propName);
    if (imagePath == null) {
      return null;
    }

    final URL imagePathUrl = this.getClass().getResource(imagePath);
    if (imagePathUrl == null) {
      return null;
    }

    final ImageIcon systemImage = new ImageIcon(imagePathUrl);
    final JLabel imageLabel = new JLabel();
    imageLabel.setIcon(systemImage);
    return imageLabel;
  }

  /**
   * Busca de string de internacionalizao.
   * 
   * @param tag o tag para a classe
   * @return o texto
   */
  final private String getString(final String tag) {
    final String className = this.getClass().getSimpleName();
    return LNG.get(className + "." + tag);
  }

  /**
   * Construtor
   * 
   * @param frame o desktop do sistema.
   */
  public AboutDialog(final JFrame frame) {
    super(frame);

    final Version version = Version.getInstance();
    final String versionName = version.getName();
    setTitle(String.format(getString("title"), Client.getInstance()
      .getSystemName(), versionName));

    final JPanel mainPanel = new JPanel();
    mainPanel.setLayout(new BorderLayout());
    mainPanel.add(buildMainPanel(), BorderLayout.CENTER);
    mainPanel.add(buildButtonsPanel(), BorderLayout.SOUTH);
    final Container container = this.getContentPane();
    container.add(mainPanel);
    pack();
    closeButton.requestFocus();
  }
}
