package csbase.client.desktop;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DecimalFormat;

import javax.swing.*;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.GUIUtils;
import csbase.client.remote.srvproxies.DiskUsageProxy;
import csbase.logic.CommonClientProject;
import csbase.logic.User;

/**
 * Dilogo de about do sistema.
 *
 * @author Tecgraf/PUC-Rio
 */
public class ProjectInfoDialog extends DesktopComponentDialog {

  /**
   * Texto com o nome do projeto
   */
  final private JTextField nameText = new JTextField(15);

  /**
   * Texto com a descrio.
   */
  final private JTextField descriptionText = new JTextField(15);

  /**
   * Texto com o id do usurio
   */
  final private JTextField userText = new JTextField(15);

  /**
  * Texto com o nome do usurio
  */
   final private JTextField userNameText = new JTextField(20);

  /**
   * Texto com o espao ocupado.
   */
  final private JTextField usageText = new JTextField(15);

  /**
   * Busca de string de internacionalizao.
   *
   * @param tag o tag para a classe
   * @return o texto
   */
  final private String getString(final String tag) {
    final String className = this.getClass().getSimpleName();
    return LNG.get(className + "." + tag);
  }


  /**
   * Cria o painel de botes.
   *
   * @return painel
   */
  protected JPanel buildMainPanel() {
    final JLabel nameLabel = new JLabel(getString("name.label"));
    final JLabel descriptionLabel = new JLabel(getString("description.label"));
    final JLabel usageLabel = new JLabel(getString("usage.label"));
    final JLabel userLabel = new JLabel(getString("user.label"));
    final JLabel userNameLabel = new JLabel(getString("user.name.label"));

    nameText.setEditable(false);
    descriptionText.setEditable(false);
    usageText.setEditable(false);
    userText.setEditable(false);
    userNameText.setEditable(false);

    final JComponent[][] cmps = new JComponent[][] {
        { nameLabel, nameText },
        { descriptionLabel, descriptionText },
        { userLabel, userText },
        { userNameLabel, userNameText },
        { usageLabel, usageText },
        };
    final JPanel fieldsPanel = GUIUtils.createBasicGridPanel(cmps);

    final JPanel panel = new JPanel();
    panel.setLayout(new GridBagLayout());
    GBC constraints = new GBC(0, 0).horizontal().center();
    // Este valores foram escolhidos para os insets porque o mtodo
    // GUIUtils.createBasicGridPanel possui insets (2, 2, 2, 5).
    // Ento, coloquei o complemento para termos (12,12,17,11).
    // O valor 17  usado para separar os botes.
    constraints.insets(10, 10, 15, 6);
    panel.add(fieldsPanel, constraints);
    return panel;
  }

  /**
   * Cria o painel de botes.
   *
   * @return painel
   */
  protected JPanel buildButtonsPanel() {
    final JPanel panel = new JPanel(new FlowLayout());

    final JButton closeButton = new JButton(getString("close.button"));
    closeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        close();
      }
    });

    final JButton refreshButton = new JButton(getString("refresh.button"));
    refreshButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        refresh();
      }
    });

    panel.add(refreshButton);
    panel.add(closeButton);
    GUIUtils.matchPreferredSizes(new JComponent[] { refreshButton, closeButton } );

    return panel;
  }

  /**
   * Atualizao dos dados.
   */
  final public void refresh() {
    final DesktopFrame desktopFrame = DesktopFrame.getInstance();
    final CommonClientProject prj = desktopFrame.getProject();
    final String projName = prj.getName();
    final Object ownerId = prj.getUserId();

    nameText.setText(projName);
    descriptionText.setText(prj.getDescription());
    userText.setText(ownerId.toString());

    RemoteTask<String> task = new RemoteTask<String>(){
      @Override
      protected void performTask() throws Exception {
        setResult(User.getName(ownerId));
      }
    };
    if (task.execute(desktopFrame.getDesktopFrame(),
      getTitle(), getString("get.user.name.msg"))) {
      userNameText.setText(task.getResult());
    }

    double usage =
      DiskUsageProxy.getUsedSpaceForProjectMb(ownerId, projName);
    String unitStr = "Mb";
    if (usage > 1024) {
      usage = usage / 1024;
      unitStr = "Gb";
      if (usage > 1024) {
        usage = usage / 1024;
        unitStr = "Tb";
      }
    }

    final DecimalFormat formatter = new DecimalFormat("0.000");
    usageText.setText(formatter.format(usage) + " " + unitStr);
  }

  /**
   * Construtor
   *
   * @param desktopFrame o desktop do sistema.
   */
  public ProjectInfoDialog(final DesktopFrame desktopFrame) {
    super(desktopFrame.getDesktopFrame());

    setTitle(getString("title"));
    final JPanel mainPanel = new JPanel();
    mainPanel.setLayout(new BorderLayout());
    mainPanel.add(buildMainPanel(), BorderLayout.CENTER);
    mainPanel.add(buildButtonsPanel(), BorderLayout.SOUTH);
    final Container container = this.getContentPane();
    container.add(mainPanel);
    pack();
  }
}
