/*
 * Created on Jun 18, 2004
 * 
 */
package csbase.client.desktop.dircontents;

import csbase.logic.ProjectFile;
import csbase.logic.ClientProjectFile;
import csbase.client.project.action.CommonFileOpenAction;

import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JTable;

/**
 * Classe que ativa os menus pop-up na tabela de contedo do diretrio.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class DirectoryContentsMouseAdapter extends MouseAdapter {
  /**
   * Linha selecionada
   */
  int selectedRow;

  /**
   * Referncia para o painel onde est a tabela de arquivos
   */
  DirectoryContentsPanel dirContentsPanel;

  /**
   * Referncia para a tabela de arquivos
   */
  JTable table;

  /**
   * Construtor.
   * 
   * @param panel componente que exibe o contedo do diretrio mostrado na
   *        rvore
   */
  public DirectoryContentsMouseAdapter(DirectoryContentsPanel panel) {
    dirContentsPanel = panel;
    table = dirContentsPanel.getTable();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void mousePressed(MouseEvent me) {
    if (me.isPopupTrigger()) {
      showPopupMenu(me);
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void mouseReleased(MouseEvent me) {
    if (me.isPopupTrigger()) {
      showPopupMenu(me);
    }
  }

  /**
   * Exibe o menu popup
   * 
   * @param me evento do mouse
   */
  private void showPopupMenu(MouseEvent me) {
    int X = me.getX();
    int Y = me.getY();
    if (dirContentsPanel.getSelectionCount() > 1) {
      // Mltiplos arquivos/diretrios foram selecionados
      if (dirContentsPanel.getPopupMultSelection() != null) {
        dirContentsPanel.configurePopupMultiSelection();
        dirContentsPanel.getPopupMultSelection().show(table, X, Y);
      }
    }
    else {
      int row = table.rowAtPoint(new Point(X, Y));

      // Seleciona a linha clicada
      table.setRowSelectionInterval(row, row);
      int fileColIndex = dirContentsPanel.getFileColIndex();
      ProjectFile file = (ProjectFile) table.getValueAt(row, fileColIndex);
      if (file.isDirectory()) {
        // Diretrio foi clicado
        if (((ClientProjectFile) file).getRenderingName().equals("../")) {
          return;
        }
        if (dirContentsPanel.getPopupDir() != null) {
          dirContentsPanel.configurePopupDir();
          dirContentsPanel.getPopupDir().show(table, X, Y);
        }
      }
      else {
        // Arquivo foi clicado
        if (dirContentsPanel.getPopupFile() != null) {
          dirContentsPanel.configurePopupFile(file);
          dirContentsPanel.getPopupFile().show(table, X, Y);
        }
      }
    }
  }

  /**
   * Permite a navegao dentro da tabela: se o usurio der 2 cliques sobre um
   * diretrio, a tabela passar a exibir o contedo deste diretrio.
   * 
   * @param e evento
   */
  @Override
  public void mouseClicked(MouseEvent e) {
    if (e.getClickCount() == 2) {
      int fileColIndex = dirContentsPanel.getFileColIndex();
      int row = table.getSelectedRow();
      ProjectFile file = (ProjectFile) table.getValueAt(row, fileColIndex);
      // Sincroniza com a seleo da rvore de diretrios
      if (file.isDirectory()) {
        dirContentsPanel.synchronizeSelection(file);
        return;
      }
      CommonFileOpenAction openAction = new CommonFileOpenAction(
        dirContentsPanel);
      openAction.openFile(file);
    }
  }
}
