/*
 * $Id: DirectoryContentsPanel.java 88986 2009-03-06 19:33:11Z ururahy $
 */
package csbase.client.desktop.dircontents;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Window;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Date;

import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.InputMap;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.TransferHandler;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import tecgraf.javautils.FormatUtils;
import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.table.ObjectTableModel;
import tecgraf.javautils.gui.table.ObjectTableProvider;
import tecgraf.javautils.gui.table.SortableTable;
import csbase.client.desktop.DesktopFrame;
import csbase.client.project.ProjectFileCellRenderer;
import csbase.client.project.ProjectFileContainer;
import csbase.client.project.ProjectTree;
import csbase.client.project.ProjectTreeAdapter;
import csbase.client.project.ProjectTreeSelectionEvent;
import csbase.client.project.ProjectTreeSelectionListener;
import csbase.client.project.action.CommonDirectoryCreationAction;
import csbase.client.project.action.CommonDirectoryRefreshAction;
import csbase.client.project.action.CommonFileChangeStateAction;
import csbase.client.project.action.CommonFileCopyAction;
import csbase.client.project.action.CommonFileCutAction;
import csbase.client.project.action.CommonFileDeleteAction;
import csbase.client.project.action.CommonFileDownloadAction;
import csbase.client.project.action.CommonFileImportAction;
import csbase.client.project.action.CommonFileOpenAction;
import csbase.client.project.action.CommonFilePasteAction;
import csbase.client.project.action.CommonFilePropertiesShowAction;
import csbase.client.project.action.CommonFilePublishAction;
import csbase.client.project.action.CommonFileRenameAction;
import csbase.client.project.action.CommonFileTypeChangeAction;
import csbase.client.project.action.CommonMultiSelectionFilePublishAction;
import csbase.client.project.action.CommonMultiSelectionFileTypeChangeAction;
import csbase.client.project.action.ExportFilesToDirAction;
import csbase.client.project.action.ProjectTreeUpdateAction;
import csbase.client.project.action.SingleFileExportAction;
import csbase.client.project.tasks.GetChildrenTask;
import csbase.client.util.DateTableCellRenderer;
import csbase.client.util.SizeTableCellRenderer;
import csbase.logic.ClientProjectFile;
import csbase.logic.ClientProjectFileComparator;
import csbase.logic.CommonClientProject;
import csbase.logic.NoHiddenFileFilter;
import csbase.logic.ProjectFile;
import csbase.logic.ProjectFileFilter;
import csbase.logic.ProjectFileType;
import csbase.logic.filters.ProjectFileNullFilter;

/**
 * Cria um painel que exibe o contedo de um diretrio, toda vez que o usurio
 * selecionar algum na rvore de projetos.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class DirectoryContentsPanel extends JPanel implements
  ProjectFileContainer {

  /**
   * ndice de nome de aqruivo
   */
  static final int NAME_COL_INDEX = 0;

  /**
   * Tabela que exibe lista de arquivos de um diretrio
   */
  private SortableTable dirContentsTable;

  /**
   * Modelo para a tabela de contedo de um diretrio
   */
  private ObjectTableModel<ProjectFile> dirModel;

  /**
   * Barra de status do diretrio
   */
  private JLabel statusBar;

  /**
   * ltimo texto exibido na barra de status
   */
  private String lastStatusBarText = " ";

  /**
   * Menu de popup de diretrio
   */
  private JPopupMenu popupDir;

  /**
   * Menu de popup de arquivo
   */
  private JPopupMenu popupFile;

  /**
   * Menu de popup de mltipla seleo (arquivos/diretrios)
   */
  private JPopupMenu popupMultSelection;

  /**
   * Item de menu "Colar" em diretrio
   */
  protected JMenuItem dirPasteFileMenuItem;

  /**
   * Controle de edio do nome do arquivo
   */
  private boolean currentEditingAllowed;

  /**
   * Seleo anterior de arquivo ou diretrio.  usado para controlar a
   * atualizao da SortableTable.
   */
  private ProjectFile previousSelectedFile;

  /**
   * Item de menu para diretrios: deleo
   */
  protected JMenuItem dirDeleteFileMenuItem;

  /**
   * Item de menu para diretrios: cut
   */
  protected JMenuItem dirCutFileMenuItem;

  /**
   * Item de menu para diretrios: rename
   */
  protected JMenuItem dirRenameFileMenuItem;

  /**
   * Item de menu para diretrios: criao de diretrio
   */
  protected JMenuItem dirCreateDirMenuItem;

  /**
   * Item de menu para diretrios: refresh
   */
  protected JMenuItem dirRefreshDirMenuItem;

  /**
   * Item de menu para diretrios: copy
   */
  protected JMenuItem dirCopyFileMenuItem;

  /**
   * Item de menu para diretrios: upload
   */
  protected JMenuItem dirUploadMenuItem;

  /**
   * Item de menu para diretrios: export
   */
  protected JMenuItem dirExportMenuItem;

  /**
   * Item de menu para arquivo: abrir arquivo com a aplicao associada
   */
  protected JMenuItem fileOpenMenuItem;

  /**
   * Item de menu para arquivo: abrir com bloco de notas
   */
  protected JMenuItem notepadFileOpenMenuItem;

  /**
   * Item de menu para arquivo: propriedaded
   */
  protected JMenuItem fileSeeFilePropertiesMenuItem;

  /**
   * Item de menu para arquivo: publicar
   */
  protected JMenuItem filePublishFileMenuItem;

  /**
   * Item de menu para arquivo: cut
   */
  protected JMenuItem fileCutFileMenuItem;

  /**
   * Item de menu para arquivo: copy
   */
  protected JMenuItem fileCopyFileMenuItem;

  /**
   * Item de menu para arquivo: renomear
   */
  protected JMenuItem fileRenameFileMenuItem;

  /**
   * Item de menu para arquivo: tipo
   */
  protected JMenuItem fileChangeFileTypeMenuItem;

  /**
   * Item de menu para arquivo: apagar
   */
  protected JMenuItem fileDeleteFileMenuItem;

  /**
   * Item de menu para arquivo: importao
   */
  protected JMenuItem fileDownloadFileMenuItem;

  /**
   * Item de menu para arquivo: exportao
   */
  protected JMenuItem fileExportFileMenuItem;

  /**
   * Item de menu para arquivo: avanado
   */
  protected JMenu advancedSubmenu;

  /**
   * Item de menu para arquivo: alterar estado
   */
  protected JMenuItem fileChangeStateFileMenuItem;

  /**
   * Item de menu para mltiplos arquivos/diretrios: tipo
   */
  protected JMenuItem msChangeFileTypeMenuItem;

  /**
   * Item de menu para mltiplos arquivos/diretrios: publicar
   */
  protected JMenuItem msPublishFileMenuItem;

  /**
   * Item de menu para mltiplos arquivos/diretrios: cut
   */
  protected JMenuItem msCutFileMenuItem;

  /**
   * Item de menu para mltiplos arquivos/diretrios: copy
   */
  protected JMenuItem msCopyFileMenuItem;

  /**
   * Item de menu para mltiplos arquivos/diretrios: deleo
   */
  protected JMenuItem msDeleteFileMenuItem;

  /**
   * Item de menu para mltiplos arquivos/diretrios: exportao
   */
  protected JMenuItem msExportFileMenuItem;

  /**
   * Janela me
   */
  private Window ownerWindow;

  /** */
  private String ownerTitle;

  /** */
  private ProjectTree projectTree;

  /** */
  private JScrollPane scrollPane;

  /**
   * Indica se a transferncia de arquivos est habilitada.
   */
  protected boolean transferEnabled;

  /**
   * Indica se a remoo de arquivos est habilitada.
   */
  protected boolean deleteEnabled;

  /**
   * Indica se a edio de nomes de arquivos est habilitada.
   */
  protected boolean renameEnabled;

  /**
   * Indica se a atualizao de diretrios e do projeto est habilitada.
   */
  protected boolean refreshEnabled;

  /**
   * Filtro para os arquivos exibidos na tabela do painel.
   */
  private ProjectFileFilter filter;

  /**
   * Consulta string de seleo de arquivo
   * 
   * @param file o arquivo
   * @return o texto
   */
  private String getStatusBarTextForFile(ProjectFile file) {
    long fileSize = file.size();
    String formattedFileSize = FormatUtils.formatSize(fileSize, 2);
    return MessageFormat.format(LNG.get("DIR_CONTENTS_STATUS_BAR"), 1,
      formattedFileSize);
  }

  /**
   * Consulta string de seleo de diretrio
   * 
   * @param children arquivos do diretrio
   * @return o texto
   */
  private String getStatusBarTextForDirectory(ProjectFile[] children) {
    return getStatusBarTextForFiles(children);
  }

  /**
   * Consulta string de seleo de mltiplos arquivos
   * 
   * @param files a lista de arquivos
   * @return o texto
   */
  private String getStatusBarTextForFiles(ProjectFile[] files) {
    int fileCount = files.length;

    // Calcula o volume total do diretrio para exibio na barra de status
    try {
      long dirSize = 0;
      for (int i = 0; i < fileCount; i++) {
        ProjectFile file = files[i];
        if (!file.isDirectory()) {
          dirSize += file.size();
        }
      }
      String formattedDirSize = FormatUtils.formatSize(dirSize, 2);
      return MessageFormat.format(LNG.get("DIR_CONTENTS_STATUS_BAR"),
        fileCount, formattedDirSize);
    }
    catch (Exception e) {
      e.printStackTrace();
      return LNG.get("DIR_CONTENTS_STATUS_BAR_ERROR");
    }
  }

  /**
   * Exibe o contedo do diretrio/arquivo selecionado pelo usurio no painel de
   * detalhes. S atualiza a tabela se o arquivo selecionado mudar.
   * 
   * @param file diretrio selecionado pelo usurio
   */
  public void display(final ProjectFile file) {
    if (file.isDirectory()) {
      displayDir(file);
    }
    else {
      displayFile(file);
    }
    statusBar.setText(lastStatusBarText);
  }

  /**
   * Exibe o diretrio com o ".." e os arquivos do diretrio dir dado.
   * 
   * @param dir diretrio
   */
  private void displayDir(ProjectFile dir) {
    ArrayList<ProjectFile> dirList = new ArrayList<ProjectFile>();
    ClientProjectFile parent = makeParentDir(dir);
    if (parent != null) {
      dirList.add(parent);
    }
    final ClientProjectFile clientDir = (ClientProjectFile)dir;
    final ClientProjectFile[] children = GetChildrenTask.runTask(clientDir);
    
    NoHiddenFileFilter noDotFileFilter = NoHiddenFileFilter.getInstance();
    boolean showHiddenFiles = DesktopFrame.getInstance()
      .shouldShowHiddenFiles();
    for (int i = 0; i < children.length; i++) {
      if (!showHiddenFiles && !noDotFileFilter.accept(children[i])) {
        continue;
      }
      if (filter.accept(children[i])) {
        dirList.add(children[i]);
      }
    }
    dirModel.setRows(dirList);
    lastStatusBarText = getStatusBarTextForDirectory(children);
  }

  /**
   * Cria sub-diretrio
   * 
   * @param dir diretrio
   * @return o novo diretrio
   */
  private ClientProjectFile makeParentDir(ProjectFile dir) {
    if (dir.getParent() == null) {
      return null;
    }
    final ClientProjectFile parent = (ClientProjectFile) dir.getParent();
    //final ClientProjectFile[] brothers = GetChildrenTask.runTask(parent);
    final ClientProjectFile parentDir = new ClientProjectFile(parent.getProjectId(),
      parent.getName(), parent.getParent(), parent.getType(), null, parent
        .isDirectory(), parent.isGlobalFile(), parent.isPublishedFile(), parent
        .isUnderConstruction(), parent.getAbsolutePath(),
      parent.whoPublished(), parent.whichProjectPublished(), parent
        .whoCreated(), parent.getCreationDate(), parent.size(), parent
        .getModificationDate(), parent.isUpdatable(), parent.isLocked());
    parentDir.setUpdated(parent.isUpdated());
    parentDir.setAsParentDir();
    return parentDir;
  }

  /**
   * Exibe dado (na status) do arquivo
   * 
   * @param file o arquivo
   */
  private void displayFile(ProjectFile file) {
    ArrayList<ProjectFile> fileList = new ArrayList<ProjectFile>();
    fileList.add(file);
    dirModel.setRows(fileList);
    lastStatusBarText = getStatusBarTextForFile(file);
  }

  /**
   * Exibe na status dados dos arquivso selecionados.
   * 
   * @param files lista de arquivos
   */
  public void display(ProjectFile[] files) {
    ArrayList<ProjectFile> fileList = new ArrayList<ProjectFile>();
    for (int i = 0; i < files.length; i++) {
      fileList.add(files[i]);
    }
    dirModel.setRows(fileList);
    lastStatusBarText = getStatusBarTextForFiles(files);
    statusBar.setText(lastStatusBarText);
  }

  /**
   * Sincroniza a seleo.
   * 
   * @param file o arquivo
   */
  public void synchronizeSelection(ProjectFile file) {
    if (file == null) {
      return;
    }
    if (!file.equals(projectTree.getSelectedFile())) {
      projectTree.setSelectedFile(file);
    }
  }

  /**
   * Limpa a tabela e a barra de status
   */
  public void clear() {
    dirModel.clear();
    lastStatusBarText = " ";
    statusBar.setText(lastStatusBarText);
  }

  /**
   * Criao de menu do contexto para diretrios
   * 
   * @return o menu
   */
  protected JPopupMenu createPopupDir() {
    /* Cria os tens de menu */
    dirCreateDirMenuItem = new JMenuItem(
      new CommonDirectoryCreationAction(this));
    dirRefreshDirMenuItem = new JMenuItem(
      new CommonDirectoryRefreshAction(this));
    dirCutFileMenuItem = new JMenuItem(new CommonFileCutAction(this));
    dirCopyFileMenuItem = new JMenuItem(new CommonFileCopyAction(this));
    dirPasteFileMenuItem = new JMenuItem(new CommonFilePasteAction(this));
    dirRenameFileMenuItem = new JMenuItem(new CommonFileRenameAction(this));
    dirDeleteFileMenuItem = new JMenuItem(new CommonFileDeleteAction(this));
    dirUploadMenuItem = new JMenuItem(new CommonFileImportAction(this));
    dirExportMenuItem = new JMenuItem(new ExportFilesToDirAction(this));
    /* Cria o menu */
    JPopupMenu menu = new JPopupMenu();
    menu.add(dirCreateDirMenuItem);
    menu.add(dirRefreshDirMenuItem);
    menu.addSeparator();
    menu.add(dirCutFileMenuItem);
    menu.add(dirCopyFileMenuItem);
    menu.add(dirPasteFileMenuItem);
    menu.addSeparator();
    menu.add(dirRenameFileMenuItem);
    menu.add(dirDeleteFileMenuItem);
    menu.addSeparator();
    menu.add(dirUploadMenuItem);
    menu.add(dirExportMenuItem);
    return menu;
  }

  /**
   * Criao de menu de contexto para arquivo
   * 
   * @return o menu
   */
  protected JPopupMenu createPopupFile() {
    /* Cria os tens de menu */
    fileOpenMenuItem = new JMenuItem(new CommonFileOpenAction(this));
    notepadFileOpenMenuItem = new JMenuItem(new CommonFileOpenAction(this,
      "notepad", LNG.get("BasicProjectTree.open.notepad")));
    fileSeeFilePropertiesMenuItem = new JMenuItem(
      new CommonFilePropertiesShowAction(this));
    filePublishFileMenuItem = new JMenuItem(new CommonFilePublishAction(this));
    fileCutFileMenuItem = new JMenuItem(new CommonFileCutAction(this));
    fileCopyFileMenuItem = new JMenuItem(new CommonFileCopyAction(this));
    fileRenameFileMenuItem = new JMenuItem(new CommonFileRenameAction(this));
    fileChangeFileTypeMenuItem = new JMenuItem(new CommonFileTypeChangeAction(
      this));
    fileDeleteFileMenuItem = new JMenuItem(new CommonFileDeleteAction(this));
    fileDownloadFileMenuItem = new JMenuItem(new CommonFileDownloadAction(this));
    fileExportFileMenuItem = new JMenuItem(new SingleFileExportAction(this));
    advancedSubmenu = new JMenu(LNG.get("BasicProjectTree.popupmenu.advanced"));
    fileChangeStateFileMenuItem = new JMenuItem(
      new CommonFileChangeStateAction(this));
    advancedSubmenu.add(fileChangeStateFileMenuItem);

    /* Cria o menu */
    JPopupMenu menu = new JPopupMenu();
    menu.add(fileOpenMenuItem);
    menu.add(notepadFileOpenMenuItem);
    menu.addSeparator();
    menu.add(fileSeeFilePropertiesMenuItem);
    menu.add(filePublishFileMenuItem);
    menu.addSeparator();
    menu.add(fileCutFileMenuItem);
    menu.add(fileCopyFileMenuItem);
    menu.addSeparator();
    menu.add(fileRenameFileMenuItem);
    menu.add(fileChangeFileTypeMenuItem);
    menu.add(fileDeleteFileMenuItem);
    menu.addSeparator();
    menu.add(fileDownloadFileMenuItem);
    menu.add(fileExportFileMenuItem);
    menu.addSeparator();
    menu.add(advancedSubmenu);
    return menu;
  }

  /**
   * Criao de menu de contexto para seleo mltipla
   * 
   * @return o menu
   */
  protected JPopupMenu createPopupMultSelection() {
    /* Cria os tens de menu */
    msPublishFileMenuItem = new JMenuItem(
      new CommonMultiSelectionFilePublishAction(this));
    msCutFileMenuItem = new JMenuItem(new CommonFileCutAction(this));
    msCopyFileMenuItem = new JMenuItem(new CommonFileCopyAction(this));
    msDeleteFileMenuItem = new JMenuItem(new CommonFileDeleteAction(this));
    msChangeFileTypeMenuItem = new JMenuItem(
      new CommonMultiSelectionFileTypeChangeAction(this));
    msExportFileMenuItem = new JMenuItem(new ExportFilesToDirAction(this));
    /* Cria o menu */
    JPopupMenu menu = new JPopupMenu();
    menu.add(msPublishFileMenuItem);
    menu.addSeparator();
    menu.add(msCutFileMenuItem);
    menu.add(msCopyFileMenuItem);
    menu.addSeparator();
    menu.add(msChangeFileTypeMenuItem);
    menu.add(msDeleteFileMenuItem);
    menu.addSeparator();
    menu.add(msExportFileMenuItem);
    return menu;
  }

  /**
   * Retorna o menu de diretrios
   * 
   * @return o menu
   */
  public JPopupMenu getPopupDir() {
    return popupDir;
  }

  /**
   * Retorna o menu de arquivo
   * 
   * @return o menu
   */
  public JPopupMenu getPopupFile() {
    return popupFile;
  }

  /**
   * Retorna o menu de seleo mltipla
   * 
   * @return o menu
   */
  public JPopupMenu getPopupMultSelection() {
    return popupMultSelection;
  }

  /**
   * {@inheritDoc}
   */
  public ClientProjectFile getSelectedFile() {
    int selectedRow = dirContentsTable.getSelectedRow();
    return (ClientProjectFile) dirContentsTable.getValueAt(selectedRow,
      NAME_COL_INDEX);
  }

  /**
   * {@inheritDoc}
   */
  public ClientProjectFile[] getSelectedFiles() {
    int[] selectedRows = dirContentsTable.getSelectedRows();
    if (selectedRows == null || selectedRows.length == 0) {
      return null;
    }
    ClientProjectFile[] selectedFiles = new ClientProjectFile[selectedRows.length];
    for (int i = 0; i < selectedRows.length; i++) {
      selectedFiles[i] = (ClientProjectFile) dirContentsTable.getValueAt(
        selectedRows[i], NAME_COL_INDEX);
    }
    return selectedFiles;
  }

  /**
   * {@inheritDoc}
   */
  public int getSelectionCount() {
    int[] selectedRows = dirContentsTable.getSelectedRows();
    if (selectedRows == null) {
      return 0;
    }
    return selectedRows.length;
  }

  /**
   * {@inheritDoc}
   */
  public Window getWindow() {
    return ownerWindow;
  }

  /**
   * {@inheritDoc}
   */
  public String getTitle() {
    return ownerTitle;
  }

  /**
   * {@inheritDoc}
   */
  public CommonClientProject getProject() {
    return this.projectTree.getProject();
  }

  /**
   * Retorna a tabela da listagem
   * 
   * @return a tabela
   */
  public SortableTable getTable() {
    return dirContentsTable;
  }

  /**
   * Consulta o ndice de arquivo
   * 
   * @return o ndice
   */
  public int getFileColIndex() {
    return NAME_COL_INDEX;
  }

  /**
   * {@inheritDoc}
   */
  public void clearSelection() {
    dirContentsTable.clearSelection();
  }

  /**
   * {@inheritDoc}
   */
  public void clearClipboard() {
    DirectoryContentsTransferHandler handler = (DirectoryContentsTransferHandler) dirContentsTable
      .getTransferHandler();
    handler.clearClipboard();
  }

  /**
   * {@inheritDoc}
   */
  public void startRenamingAction() {
    int selectedRow = dirContentsTable.getSelectedRow();
    ClientProjectFile file = (ClientProjectFile) dirContentsTable.getValueAt(
      selectedRow, NAME_COL_INDEX);
    if (!file.isParentDir()) {
      currentEditingAllowed = true;
      dirContentsTable.editCellAt(selectedRow, NAME_COL_INDEX);
      currentEditingAllowed = false;
    }
  }

  /**
   * {@inheritDoc}
   */
  public void startCutAction() {
    TransferHandler handler = dirContentsTable.getTransferHandler();
    handler.exportToClipboard(dirContentsTable, null, TransferHandler.MOVE);
  }

  /**
   * {@inheritDoc}
   */
  public void startCopyAction() {
    TransferHandler handler = dirContentsTable.getTransferHandler();
    handler.exportToClipboard(dirContentsTable, null, TransferHandler.COPY);
  }

  /**
   * {@inheritDoc}
   */
  public void startPasteAction() {
    TransferHandler handler = dirContentsTable.getTransferHandler();
    handler.importData(dirContentsTable, null);
    if (projectTree.getSelectionCount() == 1) {
      ProjectFile selectedFile = projectTree.getSelectedFile();
      display(selectedFile);
    }
  }

  /**
   * Indica se existem arquivos na rvore recortados.
   * 
   * @return indicativo
   */
  public boolean hasFileToPaste() {
    DirectoryContentsTransferHandler transferHandler = (DirectoryContentsTransferHandler) dirContentsTable
      .getTransferHandler();
    return (transferHandler != null && transferHandler
      .hasTransferableInClipboard());
  }

  /**
   * Habilita/desabilita alguns itens do menu para diretrios, de acordo com o
   * estado atual da janela de detalhes.
   */
  public void configurePopupDir() {
    if (getSelectedFile().getParent() == null) {
      dirCutFileMenuItem.setEnabled(false);
      dirRenameFileMenuItem.setEnabled(false);
      dirDeleteFileMenuItem.setEnabled(false);
    }
    else {
      dirCutFileMenuItem.setEnabled(transferEnabled);
      dirRenameFileMenuItem.setEnabled(renameEnabled);
      dirDeleteFileMenuItem.setEnabled(deleteEnabled);
    }
    dirRefreshDirMenuItem.setEnabled(refreshEnabled);
    dirCopyFileMenuItem.setEnabled(transferEnabled);
    dirPasteFileMenuItem.setEnabled(transferEnabled && hasFileToPaste());
  }

  /**
   * Habilita/desabilita alguns itens do menu para mltiplos
   * arquivos/diretrios, de acordo com o estado atual da janela de detalhes.
   */
  public void configurePopupMultiSelection() {
    if (hasDirectories(getSelectedFiles())) {
      msChangeFileTypeMenuItem.setEnabled(false);
    }
    else {
      msChangeFileTypeMenuItem.setEnabled(true);
    }
    msCutFileMenuItem.setEnabled(transferEnabled);
    msCopyFileMenuItem.setEnabled(transferEnabled);
    msDeleteFileMenuItem.setEnabled(deleteEnabled);
  }

  /**
   * Habilita/desabilita alguns itens do menu para arquivos, de acordo com o
   * estado atual da janela de detalhes.
   * 
   * @param file o arquivo selecionado.
   */
  public void configurePopupFile(ProjectFile file) {
    fileCopyFileMenuItem.setEnabled(transferEnabled);
    fileCutFileMenuItem.setEnabled(transferEnabled);
    fileDeleteFileMenuItem.setEnabled(deleteEnabled);
    fileRenameFileMenuItem.setEnabled(renameEnabled);
    ClientProjectFile projectFile = (ClientProjectFile) file;
    boolean isUnderConstruction = projectFile.isUnderConstruction();
    advancedSubmenu.setEnabled(isUnderConstruction);
  }

  /**
   * Consulta se h diretrios em uma lista de "arquivos"
   * 
   * @param projectFiles a lista de "arquivos"
   * @return indicativo
   */
  public boolean hasDirectories(ProjectFile[] projectFiles) {
    for (int i = 0; i < projectFiles.length; i++) {
      if (projectFiles[i].isDirectory()) {
        return true;
      }
    }
    return false;
  }

  /**
   * Atualiza os arquivos selecionados.
   * 
   * @see #update(ProjectFile[], boolean)
   */
  private void update() {
    this.update(DirectoryContentsPanel.this.projectTree.getSelectedFiles(),
      true);
  }

  /**
   * Atualiza o contedo da tabela se a seleo mudou ou se recebeu um evento da
   * rvore sobre alterao do modelo.
   * 
   * @param selectedFiles a seleo
   * @param refresh indicativo de atualizao
   */
  private void update(ProjectFile[] selectedFiles, boolean refresh) {
    if (selectedFiles == null || selectedFiles.length == 0) {
      this.clear();
      previousSelectedFile = null;
    }
    else {
      if (selectedFiles.length == 1) {
        ProjectFile selFile = selectedFiles[0];
        if (refresh || !selFile.equals(previousSelectedFile)) {
          display(selFile);
          previousSelectedFile = selFile;
        }
      }
      else {
        display(selectedFiles);
        previousSelectedFile = null;
      }
    }
  }

  /**
   * Habilita/desabilita a remoo de arquivos usando a tecla DELETE.
   * 
   * @param enable se <code>true</code>, habilita; caso contrrio,
   *        desabilita.
   */
  public void enableDeleteAction(boolean enable) {
    KeyStroke delKey = KeyStroke.getKeyStroke("DELETE");
    if (enable) {
      Action deleteAction = new CommonFileDeleteAction(this);
      String actionName = (String) deleteAction.getValue(Action.NAME);
      dirContentsTable.getInputMap().put(delKey, actionName);
      dirContentsTable.getActionMap().put(actionName, deleteAction);
    }
    else {
      String actionName = (String) dirContentsTable.getInputMap().get(delKey);
      dirContentsTable.getActionMap().remove(actionName);
      dirContentsTable.getInputMap().remove(delKey);
    }
    deleteEnabled = enable;
  }

  /**
   * Habilita/desabilita a atualizao da rvore de projetos usando a tecla F5.
   * 
   * @param enable se <code>true</code>, habilita; caso contrrio,
   *        desabilita.
   */
  public void enableRefreshAction(boolean enable) {
    KeyStroke refreshKey = KeyStroke.getKeyStroke("F5");
    if (enable) {
      Action refreshAction = new ProjectTreeUpdateAction(projectTree);
      String actionName = (String) refreshAction.getValue(Action.NAME);
      dirContentsTable.getInputMap().put(refreshKey, actionName);
      dirContentsTable.getActionMap().put(actionName, refreshAction);
    }
    else {
      String actionName = (String) dirContentsTable.getInputMap().get(
        refreshKey);
      dirContentsTable.getActionMap().remove(actionName);
      dirContentsTable.getInputMap().remove(refreshKey);
    }
    refreshEnabled = enable;
  }

  /**
   * Habilita/desabilita edio dos nomes dos arquivos.
   * 
   * @param enable <code>true</code> habilitar a edio, <code>false</code>
   *        desabilita.
   */
  public void enableRenameAction(boolean enable) {
    TableColumnModel columnModel = dirContentsTable.getColumnModel();
    TableColumn column = columnModel.getColumn(NAME_COL_INDEX);
    if (enable) {
      column.setCellEditor(new DirectoryContentsCellEditor(ownerWindow,
        ownerTitle));
    }
    else {
      column.setCellEditor(null);
    }
    renameEnabled = enable;
  }

  /**
   * Habilita/desabilita a transferncia de arquivos por drag&drop e clipboard.
   * 
   * @param enable true, para habilitar a transferncia, ou false, caso
   *        contrrio.
   */
  public void enableTransferAction(boolean enable) {
    if (enable) {
      dirContentsTable.setDragEnabled(true);
      dirContentsTable.setTransferHandler(new DirectoryContentsTransferHandler(
        ownerWindow, ownerTitle));
    }
    else {
      dirContentsTable.setDragEnabled(false);
      dirContentsTable.setTransferHandler(null);
    }
    transferEnabled = enable;
    enableTransferKeyBindings(enable);
  }

  /**
   * Habilita/desabilita o mapeamento de teclas de atalho para transferncia de
   * arquivos (Ctrl-X para Recortar, Ctrl-C para Copiar e Ctrl-V para Colar).
   * 
   * @param enable true, para habilitar a transferncia, ou false, caso
   *        contrrio.
   */
  private void enableTransferKeyBindings(boolean enable) {
    KeyStroke cutKey = KeyStroke.getKeyStroke("ctrl X");
    KeyStroke copyKey = KeyStroke.getKeyStroke("ctrl C");
    KeyStroke pasteKey = KeyStroke.getKeyStroke("ctrl V");
    InputMap inputMap = dirContentsTable.getInputMap();
    ActionMap actionMap = dirContentsTable.getActionMap();
    if (enable) {
      Action cutAction = new CommonFileCutAction(this);
      Action copyAction = new CommonFileCopyAction(this);
      Action pasteAction = new CommonFilePasteAction(this);
      inputMap.put(cutKey, cutAction.getValue(Action.NAME));
      inputMap.put(copyKey, copyAction.getValue(Action.NAME));
      inputMap.put(pasteKey, pasteAction.getValue(Action.NAME));
      actionMap.put(cutAction.getValue(Action.NAME), cutAction);
      actionMap.put(copyAction.getValue(Action.NAME), copyAction);
      actionMap.put(pasteAction.getValue(Action.NAME), pasteAction);
    }
    else {
      String actionName = (String) inputMap.get(cutKey);
      actionMap.remove(actionName);
      inputMap.remove(cutKey);
      actionName = (String) inputMap.get(copyKey);
      actionMap.remove(actionName);
      inputMap.remove(copyKey);
      actionName = (String) inputMap.get(pasteKey);
      actionMap.remove(actionName);
      inputMap.remove(pasteKey);
    }
  }

  /**
   * Formatador para as colunas da tabela.
   */
  private class MyObjectTableProvider implements ObjectTableProvider {
    /**
     * {@inheritDoc}
     */
    public Object[] getCellValues(Object row) {
      ProjectFile file = (ProjectFile) row;
      return new Object[] { file, getType(file), getSize(file), getDate(file) };
    }

    /**
     * {@inheritDoc}
     */
    public String[] getColumnNames() {
      return new String[] { LNG.get("DIR_CONTENTS_NAME"),
          LNG.get("DIR_CONTENTS_TYPE"), LNG.get("DIR_CONTENTS_SIZE"),
          LNG.get("DIR_CONTENTS_DATE") };
    }

    /**
     * {@inheritDoc}
     */
    public Class<?>[] getColumnClasses() {
      Class<?>[] colClasses = { ProjectFile.class, String.class, Double.class,
          Date.class };
      return colClasses;
    }

    /**
     * Obtm o tipo do arquivo.
     * 
     * @param file arquivo.
     * 
     * @return tipo do arquivo.
     */
    private String getType(ProjectFile file) {
      if (file.isDirectory()) {
        return LNG.get("DIR_CONTENTS_DIRECTORY");
      }
      return ProjectFileType.getFileType(file.getType()).getDescription();
    }

    /**
     * Obtm o tamanho do arquivo.
     * 
     * @param file arquivo.
     * 
     * @return tipo do arquivo.
     */
    private Double getSize(ProjectFile file) {
      if (file.isDirectory()) {
        return null;
      }
      return new Double(file.size());
    }

    /**
     * Obtm a data da ltima modificao do arquivo.
     * 
     * @param file arquivo.
     * 
     * @return data da ltima modificao do arquivo.
     */
    private Date getDate(ProjectFile file) {
      return new Date(file.getModificationDate());
    }
  }

  /**
   * <p>
   * Atribui o filtro para a tabela de arquivos deste painel.
   * </p>
   * ATENO:  importante que este mtodo seja chamado antes da rvore ser
   * filtrada, pois ela ir imediatamente notificar esta classe para se
   * redesenhar, e no momento do redesenho o filtro j dever estar atribudo.
   * 
   * @param filter novo filtro.
   */
  public void setFilter(ProjectFileFilter filter) {
    this.filter = filter;
  }

  /**
   * @param ownerWindow
   * @param ownerTitle
   * @param projectTree
   */
  public DirectoryContentsPanel(Window ownerWindow, String ownerTitle,
    ProjectTree projectTree) {
    this.ownerWindow = ownerWindow;
    this.ownerTitle = ownerTitle;
    this.filter = ProjectFileNullFilter.getInstance();
    this.projectTree = projectTree;
    this.projectTree.addTreeModelListener(new TreeModelListener() {
      public void treeNodesChanged(TreeModelEvent e) {
        DirectoryContentsPanel.this.update();
      }

      public void treeNodesInserted(TreeModelEvent e) {
        DirectoryContentsPanel.this.update();
      }

      public void treeNodesRemoved(TreeModelEvent e) {
        DirectoryContentsPanel.this.update();
      }

      public void treeStructureChanged(TreeModelEvent e) {
        DirectoryContentsPanel.this.update();
      }
    });
    this.projectTree
      .addTreeSelectionListener(new ProjectTreeSelectionListener() {
        public void update(ProjectTreeSelectionEvent event) {
          DirectoryContentsPanel.this.update(event.getSelectedFiles(), false);
        }
      });

    // Cria o formato da tabela
    ObjectTableProvider provider = new MyObjectTableProvider();

    // Cria o modelo da tabela
    dirModel = new ObjectTableModel<ProjectFile>(new ArrayList<ProjectFile>(),
      provider) {
      @Override
      public boolean isCellEditable(int rowIndex, int columnIndex) {
        if (!DirectoryContentsPanel.this.currentEditingAllowed) {
          return false;
        }
        return (columnIndex == NAME_COL_INDEX);
      }
    };
    dirContentsTable = new SortableTable(dirModel);
    dirContentsTable.setNoSortStateEnabled(true);
    // Seta o comparator para ClientProjectFile
    ClientProjectFileComparator comparator = new ClientProjectFileComparator();
    dirContentsTable.setComparator(0, comparator);
    // Renderizador da data e tamanho
    dirContentsTable
      .setDefaultRenderer(Date.class, new DateTableCellRenderer());
    dirContentsTable.setDefaultRenderer(Double.class,
      new SizeTableCellRenderer());
    TableColumnModel columnModel = dirContentsTable.getColumnModel();
    TableColumn column = columnModel.getColumn(NAME_COL_INDEX);
    column.setCellRenderer(new ProjectFileCellRenderer(this));

    // Remove as linhas da tabela
    dirContentsTable.setShowGrid(false);
    dirContentsTable.setRowMargin(0);
    dirContentsTable.addMouseListener(new DirectoryContentsMouseAdapter(this));
    // dirContentsTable.keepUserMarginChanges(true);

    // Se o usurio selecionar um ou mais arquivos, a barra de status 
    // atualizada com o tamanho total do(s) arquivo(s) selecionado(s).
    final ListSelectionModel selectModel = dirContentsTable.getSelectionModel();
    selectModel.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e) {
        if (dirContentsTable.getSelectedRowCount() > 0) {
          int[] indexes = dirContentsTable.getSelectedRows();
          int i;
          long selectedFilesSize = 0;
          boolean isParentDirSelected = false;
          for (i = 0; i < indexes.length; i++) {
            ClientProjectFile file = (ClientProjectFile) dirContentsTable
              .getValueAt(indexes[i], NAME_COL_INDEX);
            if (!file.isDirectory()) {
              selectedFilesSize += file.size();
            }
            if (file.isParentDir()) {
              isParentDirSelected = true;
            }
          }
          if (isParentDirSelected) {
            i--;
          }
          String formattedSelectedFilesSize = FormatUtils.formatSize(
            selectedFilesSize, 2);
          lastStatusBarText = MessageFormat.format(LNG
            .get("DIR_CONTENTS_STATUS_BAR"), i, formattedSelectedFilesSize);
        }
        statusBar.setText(lastStatusBarText);
      }
    });
    projectTree.addProjectTreeListener(new ProjectTreeAdapter() {
      @Override
      public void projectInfoModified() {
        DirectoryContentsPanel.this.update();
      }

      @Override
      public void projectRemoved() {
        DirectoryContentsPanel.this.clear();
      }

      @Override
      public void projectClosed() {
        /*
         * se o projeto fechado for o que estamos visualizando, no precisamos
         * atualizar o painel, basta o clear()
         */
        DirectoryContentsPanel.this.clear();
      }

      @Override
      public void projectChanged(CommonClientProject project) {
        DirectoryContentsPanel.this.update();
      }
    });
    scrollPane = new JScrollPane(dirContentsTable);
    scrollPane.getViewport().setBackground(Color.WHITE);
    statusBar = new JLabel();
    statusBar.setText(lastStatusBarText);
    setLayout(new BorderLayout());
    add(scrollPane, BorderLayout.CENTER);
    add(statusBar, BorderLayout.SOUTH);
    setPreferredSize(new Dimension(220, 100));

    // Criao dos menus de popup
    popupDir = createPopupDir();
    popupFile = createPopupFile();
    popupMultSelection = createPopupMultSelection();
    // Comportamento padro; os sistemas que desejarem podem desabilitar os
    // comportamentos conforme a necessidade chamando os mtodos abaixo com
    // parmetro "false".
    enableDeleteAction(true);
    enableRefreshAction(true);
    enableRenameAction(true);
    enableTransferAction(true);
  }

}
