/*
 * Detalhes da ltima alterao:
 *
 * $Author: lbarros $
 * $Date: 2007-06-06 11:40:29 -0300 (Wed, 06 Jun 2007) $
 * $Revision: 64254 $
 */
package csbase.client.externalresources;

import java.io.IOException;
import java.io.InputStream;

import javax.jnlp.FileContents;

/**
 * Implementao da <code>LocalFile</code> para execues em que a API JNLP
 * esteja disponvel.
 *
 * @author Tecgraf/PUC-Rio
 */
public final class JNLPLocalFile implements LocalFile {
  /**  a classe que representa o arquivo no sistema do cliente efetivamente. */
  private FileContents fileContents;

  /**
   * Cria um arquivo no sistema do cliente.
   *
   * @param fileContents O arquivo propriamente dito.
   *
   * @throws IllegalArgumentException Caso o arquivo recebido seja nulo.
   */
  JNLPLocalFile(FileContents fileContents) {
    if (fileContents == null) {
      throw new IllegalArgumentException(
        "No  permitido criar um arquivo local nulo.");
    }
    this.fileContents = fileContents;
  }

  /**
   * {@inheritDoc}
   */
  public long getLength()
  throws IOException {
    return this.fileContents.getLength();
  }

  /**
   * {@inheritDoc}
   */
  public boolean canRead()
  throws IOException {
    return this.fileContents.canRead();
  }

  /**
   * {@inheritDoc}
   */
  public boolean canWrite()
  throws IOException {
    return this.fileContents.canWrite();
  }

  /**
   * {@inheritDoc}
   */
  public InputStream getInputStream()
  throws IOException {
    return this.fileContents.getInputStream();
  }

  /**
   * {@inheritDoc}
   */
  public Object getOutputStream(boolean append) throws IOException {
    return this.fileContents.getOutputStream(append);
  }  
  
  /**
   * {@inheritDoc}
   */
  public String getName()
  throws IOException {
    return this.fileContents.getName();
  }

  /**
   * Obtm o arquivo do sistema do cliente.
   *
   * @return O arquivo do sistema do cliente.
   */
  FileContents getFileContents() {
    return this.fileContents;
  }

  /**
   * {@inheritDoc}
   */
  public boolean exists() {
    try {
      fileContents.getLength();
      return true;
    } catch (IOException e) {
      return false;
    }
  }
}
