package csbase.client.ias;

import java.io.File;

import javax.swing.filechooser.FileFilter;

/**
 * A classe <code>Filter</code>  usada pelo JFileChooser para filtrar o
 * conjunto de arquivos exibidos para o usurio.
 */
public class Filter extends FileFilter {
  private String extension;
  private String description;

  /**
   * Cria o filtro para um determinado arquivo.
   *
   * @param extension a extenso do arquivo que se deseja filtrar.
   * @param description descrio do tipo de arquivo.
   */
  public Filter(String extension, String description) {
    this.extension = extension;
    this.description = description;
  }

  @Override
  public boolean accept(File f) {
    if (f.isDirectory()) {
      return true;
    }
    if (isAccept(f)) {
      return true;
    }
    return false;
  }

  /**
   * Verifica se um determinado arquivo  do tipo aceito por este filtro.
   *
   * @param file o arquivo que se deseja verificar.
   *
   * @return true, se o arquivo for aceito pelo filtro, false caso contrrio.
   */
  public boolean isAccept(File file) {
    String ext = null;
    String fileName = file.getName();
    int i = fileName.lastIndexOf('.');
    if(i == -1) {
      return true;
    }
    if (i > 0 && i < fileName.length() - 1) {
      ext = fileName.substring(i + 1).toLowerCase();
      if (ext.equals(this.extension)) {
        return true;
      }
    }
    return false;
  }

  @Override
  public String getDescription() {
    return description;
  }
}
