/*
 * $Id$
 */

package csbase.client.ias.algorithm;

import java.util.Collection;

import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import csbase.client.applications.ApplicationImages;
import csbase.client.ias.algorithm.action.AddVersion;
import csbase.client.ias.algorithm.action.ImportVersion;
import csbase.client.ias.algorithm.action.RemoveAlgorithm;
import csbase.client.ias.algorithm.action.ShowAlgorithmProperties;
import csbase.client.ias.algorithm.action.ShowHistory;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionInfo;

/**
 * @author Tecgraf / PUC-Rio
 * 
 * N que representa um algoritmo. 
 */
final class AlgorithmNode extends AbstractAlgorithmTreeNode {

  /**
   * Construtor.
   * 
   * @param tree rvore que detm este n.
   * @param algorithm Informaes do algoritmo representado.
   */
  public AlgorithmNode(AlgorithmTree tree, AlgorithmInfo algorithm) {
    super(tree, algorithm);

    Collection<AlgorithmVersionInfo> versions = algorithm.getVersions();
    for (AlgorithmVersionInfo version : versions) {
      VersionNode node = new VersionNode(tree, version);
      add(node);
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ImageIcon getImageIcon() {
    return ApplicationImages.ICON_FOLDER_16;
  }

  /**
   * Cria um menu pop-up para este n.
   * 
   * @return um menu pop-up para este n.
   */
  @Override
  public JPopupMenu createPopupMenu() {
    JPopupMenu popupMenu = new JPopupMenu();
    popupMenu.add(new JMenuItem(new ShowHistory(getWindow(), this)));
    if (getTree().getMode().equals(Mode.WIZARD)) {
      popupMenu.add(new JMenuItem(new ImportVersion(getTree(),
        getAlgorithmInfo())));
    }
    popupMenu.add(new JMenuItem(new AddVersion(getTree(), getAlgorithmInfo())));
    popupMenu.add(new JMenuItem(new RemoveAlgorithm(getTree(),
      getAlgorithmInfo())));
    // Caso hajam atributos estendidos, adiciona um tem de propriedades.
    int propertySize =
      AlgorithmManagementProxy.getAlgorithmProperties(getWindow()).size();
    if (0 < propertySize) {
      popupMenu.add(new JMenuItem(new ShowAlgorithmProperties(getTree(),
        getAlgorithmInfo())));
    }

    return popupMenu;
  }

  private AlgorithmInfo getAlgorithmInfo() {
    return (AlgorithmInfo) getUserObject();
  }
}
