/*
 * $Id$
 */

package csbase.client.ias.algorithm;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import tecgraf.javautils.LNG;
import csbase.client.desktop.DesktopComponentFrame;

/**
 * @author Tecgraf / PUC-Rio
 * 
 * Especializao de um {@link DesktopComponentFrame} contendo uma
 * {@link AlgorithmTree rvore de algoritmos} e um boto para atualizao dos
 * algoritmos.
 */
public class AlgorithmTreeFrame extends DesktopComponentFrame {

  /**
   * Exibe janela contendo a rvore de algoritmos.
   * 
   * @param index identificador da janela
   * @param owner janela pai
   */
  private AlgorithmTreeFrame(Object index, DesktopComponentFrame owner) {
    super(index, owner, LNG.get("algomanager.title"));

    this.addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(WindowEvent ev) {
        close();
      }
    });

    getContentPane().add(createContentPane());

    pack();
    center(owner);
  }
  
  /**
   * Constri a janela de gerenciamento de algoritmos.
   * @param owner janela a partir da qual esta foi originada
   * @return a janela de gerenciamento de algoritmos
   */
  public static AlgorithmTreeFrame getInstance(DesktopComponentFrame owner) {
    String index = AlgorithmTreeFrame.class.getName();
    DesktopComponentFrame frame = getDesktopComponentFrame(index);
    if (frame == null) {
      frame = new AlgorithmTreeFrame(index, owner);
    }
    return (AlgorithmTreeFrame) frame;
  }

  /**
   * Constri uma painel contendo uma linha de instrues, uma rvore e um
   * painel de botes.
   * 
   * @return janela.
   */
  private JComponent createContentPane() {
    // Painel
    GridBagLayout layout = new GridBagLayout();
    JPanel panel = new JPanel(layout);
    panel.setPreferredSize(new Dimension(400, 400));
    GridBagConstraints c = new GridBagConstraints();
    // Instrues
    c.gridx = 0;
    c.gridy = 0;
    c.weightx = 0;
    c.weighty = 0;
    c.insets = new Insets(12, 12, 11, 11);
    c.fill = GridBagConstraints.NONE;
    c.anchor = GridBagConstraints.WEST;
    JLabel instruction = new JLabel(LNG.get("algomanager.msg.how_to_use"));
    layout.setConstraints(instruction, c);
    panel.add(instruction);
    // rvore
    c.gridx = 0;
    c.gridy = 1;
    c.weightx = 1;
    c.weighty = 1;
    c.fill = GridBagConstraints.BOTH;
    c.insets = new Insets(0, 12, 17, 11);
    AlgorithmTree tree = createTree();
    JScrollPane treeScrollPane = new JScrollPane(tree);
    layout.setConstraints(treeScrollPane, c);
    panel.add(treeScrollPane);
    // Painel de Botes
    c.gridx = 0;
    c.gridy = 2;
    c.weightx = 0;
    c.weighty = 0;
    c.fill = GridBagConstraints.NONE;
    c.insets = new Insets(0, 12, 0, 11);
    c.anchor = GridBagConstraints.CENTER;
    JPanel buttonPanel = createButtonPanel(tree);
    layout.setConstraints(buttonPanel, c);
    panel.add(buttonPanel);
    return panel;
  }

  /**
   * Cria a rvore de algoritmos.
   * 
   * @return uma rvore de algoritmos.
   */
  private AlgorithmTree createTree() {
    return new AlgorithmTree(this);
  }

  /**
   * Cria o painel de botes.
   * 
   * @return um painel de botes.
   */
  private JPanel createButtonPanel(final AlgorithmTree tree) {
    JPanel panel = new JPanel();
    JButton updateBtn = new JButton(LNG.get("algomanager.button.refresh.tree"));
    updateBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        tree.update();
      }
    });
    updateBtn.setToolTipText(LNG.get("algomanager.tooltip.button.update"));
    panel.add(updateBtn);
    JButton closeBtn = new JButton(LNG.get("algomanager.button.close"));
    closeBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        close();
      }
    });
    panel.add(closeBtn);
    return panel;
  }
}
