/*
 * $Id$
 */

package csbase.client.ias.algorithm;

import java.util.Set;

import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import csbase.client.applications.ApplicationImages;
import csbase.client.ias.algorithm.action.AddPlatform;
import csbase.client.ias.algorithm.action.ShowHistory;
import csbase.client.ias.algorithm.action.VersionWarningActionDecorator;
import csbase.logic.algorithms.AlgorithmVersionInfo;

/**
 * @author Tecgraf / PUC-Rio
 *
 * N que representa o diretrio bin que detm todas as plataformas. 
 */
final class ExecutableNode extends AbstractAlgorithmTreeNode {

  private AlgorithmVersionInfo version;

  /**
   * Construtor.
   * 
   * @param tree rvore que detm este n.
   * @param version Verso que detm este n.
   */
  public ExecutableNode(AlgorithmTree tree, AlgorithmVersionInfo version) {
    super(tree, version.getBinDirName());

    this.version = version;

    Set<String> platforms = version.getPlatforms().keySet();
    for (String platform : platforms) {
      add(new PlatformNode(tree, version, platform));
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ImageIcon getImageIcon() {
    return ApplicationImages.ICON_FOLDER_16;
  }

  /**
   * Cria um menu pop-up para este n.
   * 
   * @return um menu pop-up para este n.
   */
  @Override
  public JPopupMenu createPopupMenu() {
    JPopupMenu popupMenu = new JPopupMenu();
    popupMenu.add(new JMenuItem(new ShowHistory(this.getWindow(), this)));
    popupMenu.add(new JMenuItem(new VersionWarningActionDecorator(
      new AddPlatform(getTree(), version))));

    return popupMenu;
  }

}
