/*
 * $Id$
 */

package csbase.client.ias.algorithm;

import java.awt.datatransfer.DataFlavor;
import java.util.ArrayList;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import tecgraf.javautils.LNG;
import csbase.client.applications.ApplicationImages;
import csbase.client.ias.algorithm.ExecutableFileNode.ExecutableDTO;
import csbase.client.ias.algorithm.action.ImportExecutableFile;
import csbase.client.ias.algorithm.action.PasteFile;
import csbase.client.ias.algorithm.action.RemovePlatform;
import csbase.client.ias.algorithm.action.ShowHistory;
import csbase.client.ias.algorithm.action.VersionWarningActionDecorator;
import csbase.client.ias.algorithm.dnd.ITransferableTarget;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.logic.FileInfo;
import csbase.logic.IPathFactory;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.ExecutablePathFactory;

/**
 * @author Tecgraf / PUC-Rio
 * 
 * N que representa uma plataforma. 
 */
final class PlatformNode extends AbstractAlgorithmTreeNode implements
  ITransferableTarget {

  private final AlgorithmVersionInfo version;

  /**
   * Construtor.
   * 
   * @param tree rvore que detm este n.
   * @param version Informaes da verso que detm a plataforma representada.
   * @param platform Nome da plataforma representada por este n.
   */
  public PlatformNode(AlgorithmTree tree, AlgorithmVersionInfo version,
    String platform) {
    super(tree, platform);

    this.version = version;

    List<FileInfo> files = version.getPlatforms().get(platform);
    for (FileInfo file : files) {
      add(new ExecutableFileNode(tree, version, platform, file));
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ImageIcon getImageIcon() {
    return ApplicationImages.ICON_FOLDER_16;
  }

  /**
   * Cria um menu pop-up para este n.
   * 
   * @return um menu pop-up para este n.
   */
  @Override
  public JPopupMenu createPopupMenu() {
    JPopupMenu popupMenu = new JPopupMenu();
    popupMenu.add(new JMenuItem(new ShowHistory(this.getWindow(), this)));
    popupMenu.add(new JMenuItem(new VersionWarningActionDecorator(
      new ImportExecutableFile(getTree(), version, getPlatformName()))));
    popupMenu.add(new JMenuItem(new VersionWarningActionDecorator(
      new PasteFile(getTree(), LNG.get("algomanager.menu.bin_file.paste"),
        ExecutableFileNode.DATA_FLAVOR))));
    popupMenu.add(new JMenuItem(new VersionWarningActionDecorator(
      new RemovePlatform(getTree(), version, getPlatformName()))));

    return popupMenu;
  }

  private String getPlatformName() {
    return getUserObject().toString();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean canImport(Object[] data, DataFlavor flavor) {
    return null != data && 0 < data.length
      && ExecutableFileNode.DATA_FLAVOR.equals(flavor);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean importData(Object[] data) {
    ExecutableDTO dto = (ExecutableDTO) data[0];
    final AlgorithmVersionInfo sourceVersion = dto.getVersion();
    final String sourcePlatform = dto.getPlatformName();
    List<FileInfo> files = new ArrayList<FileInfo>(data.length);
    for (Object obj : data) {
      dto = (ExecutableDTO) obj;
      FileInfo file = dto.getFile();
      files.add(file);
    }

    IPathFactory sourcePathFactory =
      new ExecutablePathFactory(sourceVersion, sourcePlatform);
    IPathFactory targetPathFactory =
      new ExecutablePathFactory(version, getPlatformName());
    AlgorithmManagementProxy.copyFiles(sourceVersion, files, sourcePathFactory,
      version, null, targetPathFactory, true, getWindow());

    return true;
  }
}
