/*
 * $Id$
 */

package csbase.client.ias.algorithm;

import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import tecgraf.javautils.LNG;
import csbase.client.applications.ApplicationImages;
import csbase.client.ias.algorithm.action.AddAlgorithm;
import csbase.client.ias.algorithm.action.ShowHistory;
import csbase.logic.algorithms.AlgorithmInfo;

/**
 * @author Tecgraf / PUC-Rio
 * 
 *         Representa o n raiz da rvore.
 */
final class RootNode extends AbstractAlgorithmTreeNode {

  /**
   * Construtor.
   * 
   * @param tree rvore que detm este n.
   * @param algorithms Algoritmos da rvore.
   */
  public RootNode(AlgorithmTree tree, AlgorithmInfo[] algorithms) {
    super(tree, LNG.get("algomanager.tree.root"));

    for (AlgorithmInfo algorithm : algorithms) {
      AlgorithmNode node = new AlgorithmNode(tree, algorithm);
      this.add(node);
    }
  }

  /**
   * Obtm o ndice do n representando o algoritmo especificado.
   * 
   * @param algoId identificador do algoritmo sendo buscado.
   * 
   * @return o ndice do n ou -1 se este no for encontrado.
   */
  public int getNodeIndex(Object algoId) {
    if (children != null) {
      for (int i = 0; i < children.size(); i++) {
        AlgorithmNode childNode = (AlgorithmNode) children.get(i);
        AlgorithmInfo childAlgoInfo = (AlgorithmInfo) childNode.getUserObject();
        Object childAlgoId = childAlgoInfo.getId();
        if (algoId.equals(childAlgoId)) {
          return i;
        }
      }
    }
    return -1;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ImageIcon getImageIcon() {
    return ApplicationImages.ICON_FOLDER_16;
  }

  /**
   * Cria um menu pop-up para este n.
   * 
   * @return um menu pop-up para este n.
   */
  @Override
  public JPopupMenu createPopupMenu() {
    JPopupMenu popupMenu = new JPopupMenu();
    popupMenu.add(new JMenuItem(new ShowHistory(getWindow(), this)));
    popupMenu.add(new JMenuItem(new AddAlgorithm(getTree())));

    return popupMenu;
  }
}
