/*
 * $Id$
 */

package csbase.client.ias.algorithm;

import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import csbase.client.applications.ApplicationImages;
import csbase.client.ias.algorithm.action.DuplicateVersion;
import csbase.client.ias.algorithm.action.RemoveVersion;
import csbase.client.ias.algorithm.action.ShowHistory;
import csbase.client.ias.algorithm.action.ShowVersionProperties;
import csbase.client.ias.algorithm.action.VersionWarningActionDecorator;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.logic.algorithms.AlgorithmVersionInfo;

/**
 * @author Tecgraf / PUC-Rio
 * 
 * N que representa a verso de um algoritmo na rvore. 
 */
final class VersionNode extends AbstractAlgorithmTreeNode {

  /**
   * Construtor.
   * 
   * @param tree rvore que detm este n.
   * @param version Informaes da verso representada.
   */
  public VersionNode(AlgorithmTree tree, AlgorithmVersionInfo version) {
    super(tree, version);

    add(new ExecutableNode(tree, version));
    add(new ConfigurationNode(tree, version));
    add(new DocumentationNode(tree, version));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ImageIcon getImageIcon() {
    return ApplicationImages.ICON_FOLDER_16;
  }

  /**
   * Cria um menu pop-up para este n.
   * 
   * @return um menu pop-up para este n.
   */
  @Override
  public JPopupMenu createPopupMenu() {
    JPopupMenu popupMenu = new JPopupMenu();
    popupMenu.add(new JMenuItem(new ShowHistory(getWindow(), this)));
    popupMenu.add(new JMenuItem(new DuplicateVersion(getTree(),
      getAlgorithmVersionInfo())));
    popupMenu.add(new JMenuItem(new RemoveVersion(getTree(),
      getAlgorithmVersionInfo())));
    // Caso hajam atributos estendidos, adiciona um tem de propriedades.
    int propertySize =
      AlgorithmManagementProxy.getAlgorithmVersionProperties(getWindow())
        .size();
    if (0 < propertySize) {
      popupMenu.add(new JMenuItem(new VersionWarningActionDecorator(
        new ShowVersionProperties(getTree(), getAlgorithmVersionInfo()))));
    }

    return popupMenu;
  }

  /**
   * Obtm as informaes da verso representada por este n.
   * 
   * @return as informaes da verso representada por este n.
   */
  private AlgorithmVersionInfo getAlgorithmVersionInfo() {
    return (AlgorithmVersionInfo) getUserObject();
  }
}
