/*
 * $Id$
 */

package csbase.client.ias.algorithm.action;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.text.MessageFormat;
import java.util.Hashtable;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import tecgraf.javautils.LNG;
import csbase.client.desktop.DesktopComponentDialog;
import csbase.client.ias.algorithm.AlgorithmTree;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.client.util.ClientUtilities;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmProperty;

/**
 * @author Tecgraf / PUC-Rio
 * 
 * Ao de criao de algoritmo.
 */
public class AddAlgorithm extends AbstractAlgorithmTreeNodeAction {

  /**
   * Construtor.
   * 
   * @param tree rvore que detm o n fonte da ao.
   */
  public AddAlgorithm(AlgorithmTree tree) {
    super(tree, LNG.get("algomanager.menu.algorithm.add"));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionPerformed(ActionEvent e) {
    new AddAlgorithmDialog().show();
  }

  /**
   * Classe que representa um dilogo de incluso de algoritmos.
   * 
   * @author Leonardo Barros
   */
  private class AddAlgorithmDialog {

    /** Campo nome do algoritmo */
    private JTextField nameText;

    /**
     * Lista com as propriedades ordenadas dos algoritmos. Cada propriedade  um
     * mapeamento da chave ao Label (String de visualizaco).
     */
    private List<AlgorithmProperty> algorithmProperties;

    /** Painel de propriedades de algoritmos */
    private PropertiesPanel propertiesPanel;

    /**
     * Cria o dilogo.
     */
    public AddAlgorithmDialog() {
      this.nameText = new JTextField(30);
      algorithmProperties =
        AlgorithmManagementProxy.getAlgorithmProperties(getWindow());
      propertiesPanel =
        new PropertiesPanel(algorithmProperties,
          new Hashtable<String, String>(), LNG
            .get("algomanager.label.optional_properties"), getWindow(), true);
    }

    /**
     * Exibe o dilogo.
     */
    public void show() {
      String dialogTitle = LNG.get("algomanager.title.include");
      final DesktopComponentDialog dialog =
        DesktopComponentDialog.createDialog(getWindow());
      dialog.setTitle(dialogTitle);
      dialog.addWindowListener(new WindowAdapter() {
        @Override
        public void windowClosing(WindowEvent e) {
          dialog.close();
        }
      });

      JComponent[][] mainTextFields = makeMainTextFields();
      JPanel fullPanel = new JPanel();
      fullPanel.setLayout(new BoxLayout(fullPanel, BoxLayout.Y_AXIS));
      fullPanel.setBorder(BorderFactory.createEmptyBorder(12, 12, 12, 12));
      /* Cria e adiciona painel obrigatorio */
      JPanel mainTextPanel =
        ClientUtilities.createBasicGridPanel(mainTextFields);
      fullPanel.add(mainTextPanel);
      mainTextPanel.setBorder(BorderFactory.createTitledBorder(LNG
        .get("algomanager.label.required_properties")));
      if (this.algorithmProperties.size() > 0) {
        fullPanel.add(propertiesPanel.getPropertiesPanel());
      }
      JPanel buttonPanel = makeButtonPanel(dialog);
      dialog.add(fullPanel, BorderLayout.CENTER);
      dialog.add(buttonPanel, BorderLayout.SOUTH);
      dialog.pack();
      dialog.center(getWindow());
      dialog.setVisible(true);
    }

    /**
     * Constri um painel de botes para o dilogo.
     * 
     * @param dialog referncia para o dilogo.
     * 
     * @return painel de botes.
     */
    private JPanel makeButtonPanel(final DesktopComponentDialog dialog) {
      JPanel buttonsPanel = new JPanel();
      JButton includeButton =
        new JButton(LNG.get("algomanager.button.include"));
      includeButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (validateNewInfo(dialog)) {
            createAlgorithm();
            dialog.close();
          }
        }
      });

      JButton cancelButton = new JButton(LNG.get("algomanager.button.cancel"));
      cancelButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          dialog.close();
        }
      });
      ClientUtilities.adjustEqualSizes(includeButton, cancelButton);
      buttonsPanel.add(includeButton);
      buttonsPanel.add(cancelButton);
      return buttonsPanel;
    }

    /**
     * Constroi campos de texto do dialogo para atributos obrigatorios
     * 
     * @return array de componentes representando os campos de texto
     */
    private JComponent[][] makeMainTextFields() {
      String maxSize = Integer.toString(AlgorithmInfo.MAX_NAME_SIZE);
      String maxInLabel =
        MessageFormat.format(LNG.get("algomanager.msg.name_max"),
          new Object[] { maxSize });
      JLabel nameLabel =
        new JLabel(LNG.get("algomanager.label.name") + " " + maxInLabel);
      nameText = new JTextField(30);
      return new JComponent[][] { { nameLabel, nameText } };
    }

    /**
     * Valida os campos de texto para um novo algoritmo.
     * 
     * @param dialog janela de incluso de algoritmo que chamou o mtodo.
     * 
     * @return true se os campos estiverem corretamente preenchidos, false caso
     *         contrrio
     */
    private boolean validateNewInfo(DesktopComponentDialog dialog) {
      String name = nameText.getText().trim();
      int nameSize = name.length();
      if (nameSize == 0) {
        ClientUtilities.showErrorByTextField(dialog, LNG
          .get("algomanager.error.no_name"), null, nameText);
        return false;
      }
      if (getTree().contains(name)) {
        ClientUtilities.showErrorByTextField(dialog, LNG
          .get("algomanager.error.dup_name"), null, nameText);
        return false;
      }
      if (nameSize > AlgorithmInfo.MAX_NAME_SIZE) {
        String maxSize = Integer.toString(AlgorithmInfo.MAX_NAME_SIZE);
        ClientUtilities.showErrorByTextField(dialog, MessageFormat.format(LNG
          .get("algomanager.error.big_name"), new Object[] { maxSize }), null,
          nameText);
        return false;
      }
      return propertiesPanel.validateProperties();
    }

    /**
     * Cria um algoritmo no repositrio.
     * 
     * @return informaes acerca do algoritmo recm-includo.
     */
    private AlgorithmInfo createAlgorithm() {
      String name = nameText.getText().trim();
      AlgorithmInfo algoInfo =
        AlgorithmManagementProxy.createAlgorithm(name, propertiesPanel
          .getProperties(), getWindow());
      String dialogTitle = LNG.get("algomanager.title.include");
      if (algoInfo == null) {
        String title = LNG.get("algomanager.title.error") + " - " + dialogTitle;
        StandardErrorDialogs.showErrorDialog(getWindow(), title, MessageFormat
          .format(LNG.get("algomanager.error.include"), new Object[] { name }));
        return null;
      }
      return algoInfo;
    }
  }
}
