/*
 * $Id$
 */

package csbase.client.ias.algorithm.action;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import tecgraf.javautils.LNG;
import csbase.client.desktop.DesktopComponentDialog;
import csbase.client.ias.algorithm.AlgorithmTree;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.client.util.ClientUtilities;
import csbase.logic.algorithms.AlgorithmVersionInfo;

/**
 * @author Tecgraf / PUC-Rio
 * 
 * Ao que duplica uma verso.
 */
public class DuplicateVersion extends AbstractVersionFactoryAction {

  // Verso a ser duplicada.
  private final AlgorithmVersionInfo version;

  /**
   * Construtor.
   * 
   * @param tree rvore que detm o n fonte da ao.
   * @param version Verso a ser duplicada.
   */
  public DuplicateVersion(AlgorithmTree tree, AlgorithmVersionInfo version) {
    super(tree, LNG.get("algomanager.menu.version.duplicate"));

    this.version = version;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void createVersion() {
    new DuplicateVersionDialog().show();
  }

  /**
   * Classe que representa um dilogo de duplicao de verses.
   * 
   * @author Leonardo Barros
   */
  private class DuplicateVersionDialog extends VersionDialog {

    /**
     * Cria o dilogo.
     */
    public DuplicateVersionDialog() {
      super(version.getInfo());
    }

    /**
     * Exibe o dilogo.
     */
    public void show() {
      dialogTitle =
        String.format(LNG.get("algomanager.title.include_version"), algoInfo
          .getName());
      dialog = DesktopComponentDialog.createDialog(getWindow());
      dialog.setTitle(dialogTitle);
      dialog.addWindowListener(new WindowAdapter() {
        @Override
        public void windowClosing(WindowEvent e) {
          dialog.close();
        }
      });

      JPanel infoPanel = new JPanel();
      JLabel idLabel = new JLabel(LNG.get("algomanager.label.version_id"));
      majorField = new JTextField(3);
      minorField = new JTextField(3);
      patchField = new JTextField(3);
      infoPanel.add(idLabel);
      infoPanel.add(majorField);
      infoPanel.add(minorField);
      infoPanel.add(patchField);
      JPanel buttonsPanel = new JPanel();
      JButton includeButton =
        new JButton(LNG.get("algomanager.button.include"));
      includeButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          int[] vNumbers = getVersionNumbers();
          if (vNumbers != null) {
            duplicateVersion(version.getId(), vNumbers[0], vNumbers[1],
              vNumbers[2]);
            dialog.close();
          }
        }
      });

      JButton cancelButton = new JButton(LNG.get("algomanager.button.cancel"));
      cancelButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          dialog.close();
        }
      });
      ClientUtilities.adjustEqualSizes(includeButton, cancelButton);
      buttonsPanel.add(includeButton);
      buttonsPanel.add(cancelButton);
      Container cp = dialog.getContentPane();
      cp.add(infoPanel, BorderLayout.CENTER);
      cp.add(buttonsPanel, BorderLayout.SOUTH);
      dialog.pack();
      dialog.center(getWindow());
      dialog.setVisible(true);
    }

    /**
     * Duplica a verso especificada.
     * 
     * @param versionId identificador da verso a ser duplicada.
     * @param major nmero de verso da duplicata.
     * @param minor nmero de reviso da duplicata.
     * @param patch nmero de correo da duplicata.
     */
    private void duplicateVersion(Object versionId, int major, int minor,
      int patch) {
      Object algoId = algoInfo.getId();
      AlgorithmManagementProxy.duplicateVersion(algoId, versionId, major,
        minor, patch, getWindow());
    }
  }
}
