/*
 * $Id$
 */

package csbase.client.ias.algorithm.action;

import java.net.URL;
import java.rmi.RemoteException;
import java.text.MessageFormat;
import java.util.Collections;
import java.util.List;

import javax.swing.JFileChooser;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.desktop.Task;
import csbase.client.externalresources.ExternalResources;
import csbase.client.ias.algorithm.AlgorithmTree;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.FileInfo;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.niotransfer.RequestTransferInfo;

/**
 * @author Tecgraf / PUC-Rio
 * 
 * Ao de importao de executveis.
 */
public class ImportExecutableFile extends AbstractImportFileAction {

  // Verso que ir receber os executveis.
  private AlgorithmVersionInfo version;
  // Plataforma que ir receber os executveis.
  private String platformName;

  /**
   * Construtor.
   * 
   * @param tree Fonte da ao.
   * @param version verso de destino dos arquivos importados.
   * @param platformName plataforma de destino dos arquivos importados. 
   */
  public ImportExecutableFile(AlgorithmTree tree, AlgorithmVersionInfo version,
    String platformName) {

    super(tree, LNG.get("algomanager.menu.bin_file.add"),
      JFileChooser.FILES_AND_DIRECTORIES, true);

    this.version = version;
    this.platformName = platformName;
  }

  @Override
  protected Task<Boolean> createCheckFileExistenceTask(final String fileName) {
    return null;
  }

  @Override
  protected RequestTransferInfo prepareUpload(String fileName)
    throws RemoteException {
    AlgorithmInfo algorithm = version.getInfo();
    return ClientRemoteLocator.algorithmService.prepareUploadExecFile(algorithm
      .getId(), version.getId(), platformName, fileName);
  }

  @Override
  protected void appletUpload() {
    new UploadExecutableDialog().show();
  }

  /**
   * Classe que representa um dilogo de carregamento de executveis no servidor
   * a partir do cliente.
   * 
   * @author Leonardo Barros
   */
  private class UploadExecutableDialog {
    private String dialogTitle;

    /**
     * Cria o dilogo.
     */
    public UploadExecutableDialog() {
      List<FileInfo> execs = version.getExecutables(platformName);
      if (execs == null) {
        return;
      }
      Collections.sort(execs);
      dialogTitle =
        MessageFormat.format(LNG.get("algomanager.title.executable"),
          new Object[] { version.getInfo().getName(), version.getId(),
              platformName });
    }

    /**
     * Chama o browser para upload dos executveis. O upload s est disponivel
     * quando os recursos externos esto habilitados.
     */
    public void show() {
      if (ExternalResources.getInstance().isEnabled()) {
        String addr =
          AlgorithmManagementProxy.getExecutableURL(version.getInfo().getId(),
            version.getId(), platformName, getWindow());
        if (addr == null) {
          return;
        }
        try {
          final URL url = new URL(addr);
          ExternalResources.getInstance().showDocument(url);
        }
        catch (Exception ex) {
          StandardErrorDialogs.showErrorDialog(getWindow(), dialogTitle, LNG
            .get("PRJ_PROJECT_FILE_UPLOAD_IO_ERROR"), ex);
        }
      }
      else {
        StandardDialogs.showInfoDialog(getWindow(), dialogTitle, LNG
          .get("algomanager.error.upload_not_available"));
      }
    }
  }
}
