/*
 * $Id$
 */

package csbase.client.ias.algorithm.action;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;

import javax.swing.TransferHandler;
import javax.swing.TransferHandler.TransferSupport;

import csbase.client.ias.algorithm.AlgorithmTree;
import csbase.client.ias.algorithm.dnd.AlgorithmTreeClipboard;
import csbase.client.ias.algorithm.dnd.AlgorithmTreeTransferHandler;

/**
 * @author Tecgraf / PUC-Rio
 * 
 * Ao que importa dados atravs do
 * {@link AlgorithmTreeTransferHandler TransferHandler} da rvore.
 */
public class PasteFile extends AbstractAlgorithmTreeNodeAction {

  // Nome padro da ao.
  private static final String DEFAULT_NAME = "paste";

  /**
   * Contri uma instncia de {@link CopyFile} com o nome {@value #DEFAULT_NAME}
   * de {@link DataFlavor} <tt>null</tt>.
   * 
   * @param tree rvore que detm o n fonte da ao.
   */
  public PasteFile(AlgorithmTree tree) {
    this(tree, DEFAULT_NAME, null);
  }

  /**
   * Construtor.
   * 
   * @param tree rvore que detm o n fonte da ao.
   * @param name Nome da aao.
   * @param flavor {@link DataFlavor} dos objetos esperados. Se este valor for
   *        diferente de <tt>null</tt>, o {@link DataFlavor} dos objetos
   *        exportados atravs do
   *        {@link AlgorithmTreeTransferHandler TransferHandler} sero
   *        comparados com este e se no forem iguais, esta ao ser construida
   *        desabilitada.
   */
  public PasteFile(AlgorithmTree tree, String name, DataFlavor flavor) {
    super(tree, name);

    if (null != flavor) {
      Transferable transferable = AlgorithmTreeClipboard.getObject();
      setEnabled(null != transferable
        && transferable.isDataFlavorSupported(flavor));
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionPerformed(ActionEvent e) {
    Transferable transferable = AlgorithmTreeClipboard.getObject();
    if (null != transferable) {
      TransferHandler transferHandler = getTree().getTransferHandler();
      TransferSupport supp = new TransferSupport(getTree(), transferable);
      if (transferHandler.canImport(supp)) {
        transferHandler.importData(supp);
      }
    }
  }
}
