/* 
 * $Id$ 
 */

package csbase.client.ias.algorithm.action;

import java.awt.Window;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.util.ClientUtilities;
import csbase.logic.algorithms.AlgorithmProperty;

/**
 * A classe <code>AlgoPropertiesPanel</code> representa o painel de edio
 * das propriedades dos algoritmos e das verses.
 */
class PropertiesPanel {

  /** Componentes de entrada de dados das propriedades */
  private JTextField[] propertiesInputField;

  /** Configurao das propriedades do algoritmo ou verso */
  private List<AlgorithmProperty> algoProperties;

  /**
   * Valores das propriedades do algoritmo indexada pela chave
   * <code>AlgorithmProperty.key</code>
   */
  private Map<String, String> propertiesValues;

  /** Ttulo do painel */
  private String title;

  /** Janela em que o painel est colado */
  private Window owner;

  /** Painel com as propriedades */
  private JPanel mainPanel;

  /** Indica de o painel  editavel */
  private boolean editable;

  /**
   * Contrutor.
   * 
   * @param algoProperties Configurao das propriedades do algoritmo ou
   *        verso.
   * @param propertiesValues Valores das propriedades do algoritmo indexada
   *        pela chave <code>AlgorithmProperty.key</code>
   * @param title Ttulo do painel
   * @param owner Janela em que o painel est colado
   * @param editable Indica de o painel  editavel
   */
  PropertiesPanel(List<AlgorithmProperty> algoProperties,
    Map<String, String> propertiesValues, String title, Window owner,
    boolean editable) {
    this.algoProperties = algoProperties;
    this.propertiesValues = propertiesValues;
    this.title = title;
    this.owner = owner;
    this.editable = editable;
    this.propertiesInputField = new JTextField[algoProperties.size()];
    this.mainPanel = buildPanel();
  }

  /**
   * Retorna o painel de propriedades.
   * 
   * @return o painel de propriedades
   */
  protected JPanel getPropertiesPanel() {
    return mainPanel;
  }

  /**
   * Constri o painel de entrada de dados das propriedades.
   * 
   * @return o painel de entrada de dados das propriedades
   */
  private JPanel buildPanel() {
    int size = algoProperties.size();
    JComponent[][] components = new JComponent[size][2];
    for (int i = 0; i < size; i++) {
      String propKey = algoProperties.get(i).getKey();
      String propValue =
        (propertiesValues.get(propKey) == null) ? "" : propertiesValues
          .get(propKey);
      JTextField inputField = new JTextField(propValue, 30);
      inputField.setEditable(editable);
      propertiesInputField[i] = inputField;
      components[i][0] = new JLabel(algoProperties.get(i).getLabel());
      components[i][1] = inputField;
    }
    JPanel panel = ClientUtilities.createBasicGridPanel(components);
    panel.setBorder(BorderFactory.createTitledBorder(title));
    return panel;
  }

  /**
   * Indica se houve alterao nas propriedades
   * 
   * @return true se houver alterao das propriedades, ou false caso nenhuma
   *         propriedade tenha sido alterada.
   */
  protected boolean wasModified() {
    for (int i = 0; i < algoProperties.size(); i++) {
      AlgorithmProperty property = algoProperties.get(i);
      String newPropValue = propertiesInputField[i].getText().trim();
      String propKey = property.getKey();
      String propValue = propertiesValues.get(propKey);
      // Se a o valor existente para a propriedade for nulo e o usurio no
      // tiver preenchido, nada mudou
      if (newPropValue.length() == 0 && propValue == null) {
        continue;
      }
      if (!newPropValue.equals(propValue)) {
        return true;
      }
    }
    return false;
  }

  /**
   * Verifica se as propriedades esto nos seus em um formato vlido. Caso no
   * estejam, exibe um dilogo com o erro e retorna false
   * 
   * @return true se as propriedades esto no seu no formato vlido, ou false
   *         caso alguma no esteja em um formato vlido.
   */
  protected boolean validateProperties() {
    boolean result = true;
    List<String> invalidLabels = new ArrayList<String>();
    for (int i = 0; i < algoProperties.size(); i++) {
      AlgorithmProperty algoProp = algoProperties.get(i);
      String textFieldValue = propertiesInputField[i].getText().trim();
      if (!algoProp.isValidValue(textFieldValue)) {
        result = false;
        invalidLabels.add(algoProp.getLabel());
      }
    }
    if (!result) {
      String errorTitle = LNG.get("algomanager.title.error") + " - " + title;
      StringBuilder msg = new StringBuilder();
      msg.append(LNG.get("algomanager.error.validation.properties.1"));
      msg.append("\n");
      for (String label : invalidLabels) {
        msg.append("\n  - " + label);
      }
      msg.append("\n\n");
      msg.append(LNG.get("algomanager.error.validation.properties.2"));
      StandardDialogs.showErrorDialog(owner, errorTitle, msg.toString());
    }
    return result;
  }

  /**
   * Retorna os novos valores das propriedades.
   * 
   * @return os novos valores das propriedades.
   */
  protected Hashtable<String, String> getProperties() {
    Hashtable<String, String> propsUpdated = new Hashtable<String, String>();
    for (int i = 0; i < algoProperties.size(); i++) {
      AlgorithmProperty property = algoProperties.get(i);
      String propKey = property.getKey();
      String newPropValue = propertiesInputField[i].getText().trim();
      if (newPropValue.length() > 0) {
        propsUpdated.put(propKey, newPropValue);
      }
    }
    return propsUpdated;
  }
}
