/*
 * $Id$
 */

package csbase.client.ias.algorithm.action;

import java.awt.event.ActionEvent;
import java.text.MessageFormat;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.ias.algorithm.AlgorithmTree;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.algorithms.AlgorithmInfo;

/**
 * @author Tecgraf / PUC-Rio
 * 
 * Ao para remover um algoritmo.
 */
public class RemoveAlgorithm extends AbstractAlgorithmTreeNodeAction {

  // Algoritmo a ser removido.
  private final AlgorithmInfo algorithm;

  /**
   * Construtor.
   * 
   * @param tree rvore que detm o n fonte da ao.
   * @param algorithm Algoritmno a ser removido.
   */
  public RemoveAlgorithm(AlgorithmTree tree, AlgorithmInfo algorithm) {
    super(tree, LNG.get("algomanager.menu.algorithm.remove"));

    this.algorithm = algorithm;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionPerformed(ActionEvent ev) {
    new RemoveAlgorithmDialog().show();
  }

  /**
   * Classe que representa um dilogo de remoo de algoritmo.
   * 
   * @author Leonardo Barros
   */
  private class RemoveAlgorithmDialog {
    /**
     * Exibe o dilogo.
     */
    public void show() {
      String algoName = algorithm.getName();
      String dialogTitle = LNG.get("algomanager.title.remove");
      Object[] options =
        { LNG.get("algomanager.button.remove"),
            LNG.get("algomanager.button.cancel") };
      int opt =
        StandardDialogs.showOptionDialog(getWindow(), dialogTitle,
          MessageFormat.format(LNG.get("algomanager.msg.confirm.remove"),
            new Object[] { algoName }), options);
      if (opt != 0) {
        return;
      }
      boolean removed =
        AlgorithmManagementProxy
          .removeAlgorithm(algorithm.getId(), getWindow());
      if (!removed) {
        String title = LNG.get("algomanager.title.error") + " - " + dialogTitle;
        StandardErrorDialogs.showErrorDialog(getWindow(), title, MessageFormat
          .format(LNG.get("algomanager.error.remove"),
            new Object[] { algoName }));
      }
    }
  }
}
