/*
 * $Id$
 */

package csbase.client.ias.algorithm.action;

import java.awt.event.ActionEvent;
import java.text.MessageFormat;

import tecgraf.javautils.FormatUtils;
import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.ias.algorithm.AlgorithmTree;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.FileInfo;
import csbase.logic.algorithms.AlgorithmVersionInfo;

/**
 * @author Tecgraf / PUC-Rio
 * 
 * Ao para remover arquivos de documentao.
 */
public class RemoveDocumentationFiles extends AbstractAlgorithmTreeNodeAction {

  // Verso que detm os arquivos.
  private final AlgorithmVersionInfo version;
  // Arquivos a serem removidos.
  private final FileInfo[] files;

  /**
   * Construtor.
   * 
   * @param tree rvore que detm o n fonte da ao.
   * @param version Verso que detm os arquivos.
   * @param files Arquivos a serem removidos.
   */
  public RemoveDocumentationFiles(AlgorithmTree tree,
    AlgorithmVersionInfo version, FileInfo[] files) {
    super(tree, LNG.get("algomanager.menu.doc.remove"));

    this.version = version;
    this.files = files;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionPerformed(ActionEvent e) {
    removeDoc();
  }

  /**
   * Exibe um dilogo para confirmao de remoo de um arquivo de documentao.
   */
  private void removeDoc() {
    new RemoveDocDialog().show();
  }

  /**
   * Classe que representa um dilogo de remoo de arquivos de documentao.
   * 
   * @author Leonardo Barros
   */
  private class RemoveDocDialog {
    /**
     * Exibe o dilogo.
     */
    public void show() {
      String removeTitle = LNG.get("algomanager.title.remove_doc");
      Object[] options =
        { LNG.get("algomanager.button.remove"),
            LNG.get("algomanager.button.cancel") };
      int opt =
        StandardDialogs.showOptionDialog(getWindow(), removeTitle,
          MessageFormat.format(LNG.get("algomanager.msg.confirm.doc_remove"),
            new Object[] { FormatUtils.format(files) }), options);
      if (opt != 0) {
        return;
      }
      boolean success =
        AlgorithmManagementProxy.removeDocumentationFiles(version, files,
          getWindow());
      if (!success) {
        StandardErrorDialogs.showErrorDialog(getWindow(), LNG
          .get("algomanager.title.error")
          + " - " + removeTitle, LNG.get("algomanager.error.doc_remove"));
      }
    }
  }
}
