/*
 * $Id$
 */

package csbase.client.ias.algorithm.action;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.text.MessageFormat;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.desktop.DesktopComponentDialog;
import csbase.client.ias.algorithm.AlgorithmTree;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmProperty;

/**
 * @author Tecgraf / PUC-Rio
 * 
 * Ao para abrir um dilogo apresentando as propriedades de um algoritmo.
 */
public class ShowAlgorithmProperties extends AbstractAlgorithmTreeNodeAction {

  // Algoritmo a ter suas propriedades apresentadas.
  private AlgorithmInfo algorithm;

  /**
   * Construtor.
   * 
   * @param tree rvore que detm o n fonte da ao.
   * @param algorithm Algoritmo a ter suas propriedades apresentadas.
   */
  public ShowAlgorithmProperties(AlgorithmTree tree, AlgorithmInfo algorithm) {
    super(tree, LNG.get("algomanager.menu.algorithm.properties"));

    this.algorithm = algorithm;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionPerformed(ActionEvent e) {
    new AlgorithmPropertiesDialog().show();
  }

  /**
   * Classe que representa um dilogo com as propriedades de um algoritmo
   * (incluindo, caso existam, as propriedades estendidas)
   * 
   * @author Daniel de Paula
   */
  class AlgorithmPropertiesDialog {

    /** Dilogo em si */
    protected DesktopComponentDialog dialog;

    /**
     * Lista com as propriedades dos algoritmos.
     */
    private List<AlgorithmProperty> algorithmProperties;

    /** Painel de propriedades de algoritmos */
    private PropertiesPanel propertiesPanel;

    /**
     * Cria o dilogo.
     */
    public AlgorithmPropertiesDialog() {
      algorithmProperties =
        AlgorithmManagementProxy.getAlgorithmProperties(getWindow());
      propertiesPanel =
        new PropertiesPanel(algorithmProperties, algorithm.getPropertyValues(),
          LNG.get("algomanager.label.optional_properties"), getWindow(), true);
    }

    /**
     * Exibe dilogo
     */
    public void show() {
      String dialogTitle =
        String.format(LNG.get("algomanager.title.algo_properties"), algorithm
          .getName());
      dialog = DesktopComponentDialog.createDialog(getWindow());
      dialog.setTitle(dialogTitle);
      dialog.addWindowListener(new WindowAdapter() {
        @Override
        public void windowClosing(WindowEvent e) {
          close();
        }
      });

      /* Painel com os botoes */
      JPanel buttonsPanel = new JPanel();
      JButton includeButton = new JButton(LNG.get("algomanager.button.change"));
      JButton cancelButton = new JButton(LNG.get("algomanager.button.cancel"));
      buttonsPanel.add(includeButton);
      buttonsPanel.add(cancelButton);
      // TODO colocar abas com as versoes
      /* Listeners do dialog */
      includeButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (propertiesPanel.validateProperties()) {
            analyzePropertyChange();
            dialog.close();
          }
        }
      });
      cancelButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          close();
        }
      });
      /* ajustes */
      JPanel fullPanel = new JPanel();
      fullPanel.setLayout(new BoxLayout(fullPanel, BoxLayout.Y_AXIS));
      fullPanel.setBorder(BorderFactory.createEmptyBorder(12, 12, 12, 12));

      if (this.algorithmProperties.size() > 0) {
        fullPanel
          .add(propertiesPanel.getPropertiesPanel(), BorderLayout.CENTER);
      }
      fullPanel.add(buttonsPanel, BorderLayout.SOUTH);
      dialog.add(fullPanel, BorderLayout.CENTER);
      dialog.add(buttonsPanel, BorderLayout.SOUTH);
      dialog.pack();
      dialog.center(getWindow());
      dialog.setVisible(true);
    }

    /**
     * Fecha dilogo.
     */
    private void close() {
      if (propertiesPanel.wasModified()) {
        if (askConfirmation() == 0) {
          dialog.close();
        }
      }
      else {
        dialog.close();
      }
    }

    /**
     * Analisa se ouve ou nao mudanca em alguma das propriedades. Em caso
     * positivo, executa a atualizao.
     */
    private void analyzePropertyChange() {

      if (propertiesPanel.wasModified()) {
        String name = algorithm.getName();
        algorithm =
          AlgorithmManagementProxy.changeAlgorithmProperties(name,
            propertiesPanel.getProperties(), getWindow());
        String dialogTitle = LNG.get("algomanager.title.algo_properties");
        if (algorithm == null) {
          String title =
            LNG.get("algomanager.title.error") + " - " + dialogTitle;
          StandardErrorDialogs
            .showErrorDialog(getWindow(), title, MessageFormat.format(LNG
              .get("algomanager.error.update"), new Object[] { name }));
        }
        else {
          /*
           * Recarrega os algoritmos para que as alteraes se reflitam nos
           * outros usurios e no SchedulerService.
           */
          AlgorithmManagementProxy.reloadAlgorithms(getWindow());
        }
      }
    }

    /**
     * Exibe janela de confirmacao para atualizar propriedades do algoritmo
     * 
     * @return opcao selecionada
     */
    private int askConfirmation() {
      String algoName = algorithm.getName();
      String dialogTitle = LNG.get("algomanager.title.properties_change");
      int opt =
        StandardDialogs.showYesNoDialog(getWindow(), dialogTitle, MessageFormat
          .format(LNG.get("algomanager.msg.confirm.properties_update"),
            new Object[] { algoName }));
      return opt;
    }
  }
}
