package csbase.client.ias.algorithm.niotransfer;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.channels.SocketChannel;

import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.configurationmanager.ConfigurationManagerException;
import csbase.client.externalresources.LocalFile;
import csbase.exception.ConfigurationException;
import csbase.remote.niotransfer.RequestTransferInfo;

/**
 * Cliente para exportao de arquivos, semelhante a um cliente FTP, mas 
 * usando a API NIO do Java.
 * 
 * Cliente de uma aplicao simples cliente/servidor de transferncia de 
 * arquivos. 
 * 
 * @author Tecgraf
 */
public class FileClient {
  /** Objeto que implementa o protocolo de comunicao no cliente. */
  private ClientSimpleFileTransferProtocol protocol;
  /** Informaes necessrias para solicitar a transferncia. */  
  private RequestTransferInfo requestInfo;

  /**
   * Cria uma instncia de cliente, conectando-se  sua contraparte no servidor.
   *
   * @param serverName nome do servidor. 
   * @param info informaes necessrias para solicitar a transferncia.
   * 
   * @throws IOException caso ocorra algum problema durante a conexo.
   */
  public FileClient(String serverName, RequestTransferInfo info) 
  throws IOException {
    this.requestInfo = info;
    SocketChannel serverChannel = null;
    int port = requestInfo.getPort();
    int bufferSize;
    try {
      Configuration configuration = 
        ConfigurationManager.getInstance().getConfiguration(getClass());
      String value = 
        configuration.getOptionalProperty("buffer.size", "1048576");
      bufferSize = Integer.parseInt(value);
    } 
    catch (ConfigurationManagerException e) {
      throw new ConfigurationException(e);
    }
    serverChannel = SocketChannel.open();
    serverChannel.connect(new InetSocketAddress(serverName, port));
    protocol = 
      new ClientSimpleFileTransferProtocol(bufferSize, serverChannel);
  }

  /**
   * Envia um arquivo local para o servidor.  
   * 
   * @param sourceFile referncia para o arquivo no sistema de arquivos local.
   *        
   * @throws IOException
   */
  public void send(LocalFile sourceFile) throws IOException {
    protocol.writeFile(sourceFile, requestInfo);    
  }

  /**
   * Recebe um arquivo remoto e grava no sistema de arquivos local.
   * 
   * @param targetFile localizao do arquivo de destino (se existir, 
   *        sobrescreve).
   *        
   * @throws IOException
   */
  public void receive(LocalFile targetFile) throws IOException {
    protocol.readFile(requestInfo, targetFile);
  }
  
}
