/*
 * Detalhes da ltima alterao:
 *
 * $Author: moura $
 * $Date: 2006-08-16 17:31:23 -0300 (Qua, 16 Ago 2006) $
 * $Revision: 54958 $
 */
package csbase.client.ias.trustedserver;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.rmi.RemoteException;
import java.security.cert.Certificate;
import java.security.cert.CertificateFactory;
import java.security.cert.CertificateParsingException;
import java.text.MessageFormat;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;
import java.util.SortedSet;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import tecgraf.javautils.gui.table.ObjectTableModel;
import tecgraf.javautils.gui.table.ObjectTableProvider;
import csbase.client.desktop.RemoteTask;
import csbase.client.ias.ManagerPanel;
import csbase.client.remote.manager.trustedserver.TrustedServerManager;
import csbase.client.remote.manager.trustedserver.TrustedServerManagerListener;
import csbase.client.util.ClientUtilities;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.trustedserver.TrustedServer;

/**
 * Painel responsvel por exibir uma tabela com os servidores confiveis e opes
 * de gerenciamento dos mesmos.
 *
 * @author Tecgraf/PUC-Rio
 */
public final class TrustedServerManagerPanel extends ManagerPanel {
  /** A janela onde este painel est inserido. */
  private JFrame parent;

  /** O modelo contendo os servidores confiveis atualmente no repositrio. */
  private ObjectTableModel tableModel;

  /** Um ouvinte que captura mudanas no repositrio de servidores confiveis. */
  private TrustedServerManagerListener managerListener;

  /**
   * Cria um painel para gerenciamento de servidores confiveis.
   * 
   * @param parent A janela onde este painel est inserido.
   */
  public TrustedServerManagerPanel(JFrame parent) {
    this.parent = parent;

    final String[] columnNames =
      new String[] { null, LNG.get("IAS_TRUSTED_SERVER_NAME"),
          LNG.get("IAS_TRUSTED_SERVER_URI"),
          LNG.get("IAS_TRUSTED_SERVER_SUSPENDED"),
          LNG.get("IAS_TRUSTED_SERVER_LOCAL"),
          LNG.get("IAS_TRUSTED_SERVER_COMPLETED") };

    final Class<?>[] columnClasses =
      new Class[] { TrustedServer.class, String.class, String.class,
          String.class, String.class, String.class };

    if (TrustedServerManager.getInstance().isEnabled()) {
      ObjectTableProvider provider = new ObjectTableProvider() {
        public Object[] getCellValues(Object item) {
          TrustedServer trustedServer = (TrustedServer) item;
          String isSuspended;
          if (trustedServer.isSuspended()) {
            isSuspended = LNG.get("IAS_YES");
          }
          else {
            isSuspended = LNG.get("IAS_NO");
          }
          String isCompleted;
          if (trustedServer.isCompleted()) {
            isCompleted = LNG.get("IAS_YES");
          }
          else {
            isCompleted = LNG.get("IAS_NO");
          }
          String isLocal =
            trustedServer.isLocal() ? LNG.get("IAS_YES") : LNG.get("IAS_NO");
          ;
          return new Object[] { trustedServer, trustedServer.getName(),
              trustedServer.getURI(), isSuspended, isLocal, isCompleted };
        }

        public String[] getColumnNames() {
          return columnNames;
        }

        public Class<?>[] getColumnClasses() {
          return columnClasses;
        }
      };
      this.tableModel =
        new ObjectTableModel<TrustedServer>(this.getTrustedServers(), provider);
      Comparator[] comparators =
        {
          null, ClientUtilities.getStringComparatorIgnoreCase(),
          ClientUtilities.getStringComparatorIgnoreCase(),
          ClientUtilities.getStringComparatorIgnoreCase(),
          ClientUtilities.getStringComparatorIgnoreCase(),
          ClientUtilities.getStringComparatorIgnoreCase()
        };
      make(this.tableModel, comparators, false);
      this.managerListener =
        new TrustedServerManagerListener() {
            public void wasAddedTrustedServer(final TrustedServer addedLocalServer) {
              SwingUtilities.invokeLater(
                new Runnable() {
                  public void run() {
                    tableModel.add(addedLocalServer);
                    TrustedServerManagerPanel.this.getTable().adjustColumnWidth();
                  }
                });
            }

            public void wasRemovedTrustedServer(
              final TrustedServer removedLocalServer) {
              SwingUtilities.invokeLater(
                new Runnable() {
                  public void run() {
                    tableModel.remove(removedLocalServer);
                    TrustedServerManagerPanel.this.getTable().adjustColumnWidth();
                  }
                });
            }

            public void wasModifiedTrustedServer(
              final TrustedServer oldLocalServer, final TrustedServer newLocalServer) {
              SwingUtilities.invokeLater(
                new Runnable() {
                  public void run() {
                    tableModel.remove(oldLocalServer);
                    tableModel.add(newLocalServer);
                    TrustedServerManagerPanel.this.getTable().adjustColumnWidth();
                  }
                });
            }
          };
      TrustedServerManager.getInstance().addTrustedServerManagerListener(
        this.managerListener);
      
      this.getTable().adjustColumnWidth();
    }
    else {
      this.setLayout(new BorderLayout());
      JTextArea instructionsTextArea =
        new JTextArea(LNG.get("IAS_TRUSTED_SERVER_ADMIN_INSTRUCTIONS"));
      instructionsTextArea.setEditable(false);
      instructionsTextArea.setLineWrap(true);
      instructionsTextArea.setWrapStyleWord(true);
      this.add(instructionsTextArea, BorderLayout.CENTER);
    }
  }

  protected JPanel makeButtons() {
    JPanel panel = super.makeButtons();
    this.createUpdateButton(panel);
    this.createAddCertButton(panel);
    this.createRemCertButton(panel);
    return panel;
  }

  private void createUpdateButton(JPanel panel) {
    JButton updateButton = new JButton(LNG.get("IAS_TRUSTED_SERVER_UPDATE"));
    updateButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          tableModel.setRows(getTrustedServers());
        }
      });
    panel.add(updateButton); 
  }
  
  private void createAddCertButton(JPanel panel) {
    JButton addCertButton = new JButton(LNG.get("IAS_TRUSTED_SERVER_ADDCERT"));
    addCertButton.setEnabled(false);
    addCertButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        try {
          addCert();
          tableModel.setRows(getTrustedServers());
        }
        catch(CertificateParsingException cpe) {
          StandardErrorDialogs.showErrorDialog(parent, LNG
            .get("IAS_TRUSTED_SERVER_ADDCERT_TITLE"), LNG
            .get("IAS_TRUSTED_SERVER_ADDCERT_PARSE_EXCEPTION_ERROR"));
        }
        catch (Exception ex) {
          StandardErrorDialogs.showErrorDialog(parent, LNG
            .get("IAS_TRUSTED_SERVER_ADDCERT_TITLE"), LNG
            .get("IAS_TRUSTED_SERVER_ADDCERT_ERROR"), ex);
        }
      }
    });
    panel.add(addCertButton);
    this.addButtonToSelectionListener(addCertButton);
  }
  
  private void createRemCertButton(JPanel panel) {
    JButton remCertButton = new JButton(LNG.get("IAS_TRUSTED_SERVER_REMCERT"));
    remCertButton.setEnabled(false);
    remCertButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          try {
            remCert();
            tableModel.setRows(getTrustedServers());
          }
          catch (Exception ex) {
            StandardErrorDialogs.showErrorDialog(parent, LNG
              .get("IAS_TRUSTED_SERVER_REMCERT_TITLE"), LNG
              .get("IAS_TRUSTED_SERVER_REMCERT_ERROR"), ex);
          }
        }
      });
    panel.add(remCertButton);
    this.addButtonToSelectionListener(remCertButton);
  }
  
  private void remCert() throws Exception {
    int row = this.getTable().getSelectedRow();
    final TrustedServer trustedServer =
      (TrustedServer) this.getTable().getModel().getValueAt(row,
        IDENTIFIER_INDEX);
    
    if (!trustedServer.isCompleted())
      return;

    String msg =
      MessageFormat.format(LNG
        .get("IAS_TRUSTED_SERVER_REMCERT_CONFIRMATION_MESSAGE"), trustedServer
        .getName());
    int retVal =
      JOptionPane.showConfirmDialog(this.parent, msg, LNG
        .get("IAS_TRUSTED_SERVER_REMCERT_TITLE"), JOptionPane.YES_NO_OPTION);
    if (retVal == JOptionPane.NO_OPTION)
      return;

    RemoteTask<Boolean> remCertTask = new RemoteTask<Boolean>() {
      protected void performTask() throws Exception {
        TrustedServerManager.getInstance().removeCertificate(
          trustedServer.getName());
      }
    };

    if (!remCertTask.execute(this.parent, LNG
      .get("IAS_TRUSTED_SERVER_REMCERT_TITLE"), LNG
      .get("IAS_TRUSTED_SERVER_REMCERT_MESSAGE")))
      throw remCertTask.getError();
  }
  
  private void addCert() throws Exception {
    int row = this.getTable().getSelectedRow();
    final TrustedServer trustedServer =
      (TrustedServer) this.getTable().getModel().getValueAt(row,
        IDENTIFIER_INDEX);

    JFileChooser fileChooser = new JFileChooser();
    int retVal = fileChooser.showOpenDialog(this.parent);

    if (retVal == JFileChooser.APPROVE_OPTION) {
      File certFile = fileChooser.getSelectedFile();
      FileInputStream certfis = new FileInputStream(certFile);      
      CertificateFactory cf = CertificateFactory.getInstance("X.509");
      final Certificate cert = cf.generateCertificate(certfis);
      certfis.close();
      
      RemoteTask<Boolean> addCertTask = new RemoteTask<Boolean>() {
        protected void performTask() throws Exception {
          TrustedServerManager.getInstance().addCertificate(
            trustedServer.getName(), cert);
        }
      };

      if (!addCertTask.execute(this.parent, LNG
        .get("IAS_TRUSTED_SERVER_ADDCERT_TITLE"), LNG
        .get("IAS_TRUSTED_SERVER_ADDCERT_MESSAGE"))) 
        throw addCertTask.getError();
    }
  }
  
  public void add() {
    new TrustedServerInfoDialog(this.parent);
  }

  public void modify(Object id) {
    TrustedServer localServer = (TrustedServer)id;
    if (localServer == null) {
      throw new IllegalStateException(
        "No foi possvel achar um servidor confivel com o id "+id+".");
    }
    new TrustedServerInfoDialog(this.parent, localServer);
  }

  public void delete(Object id) {
    final TrustedServer localServer = (TrustedServer)id;
    int option =
      StandardDialogs.showYesNoDialog(
        this, LNG.get("IAS_TRUSTED_SERVER_DELETE_TITLE"),
        MessageFormat.format(
          LNG.get("IAS_TRUSTED_SERVER_DELETE_CONFIRMATION_MESSAGE"),
          new Object[] { localServer.getName() }));
    if (option == JOptionPane.YES_OPTION) {
      RemoteTask<Void> deleteLocalServerTask =
        new RemoteTask<Void>() {
          protected void performTask()
          throws Exception {
            TrustedServerManager.getInstance().removeTrustedServer(localServer);
          }
        };
      deleteLocalServerTask.execute(
        this.parent, LNG.get("IAS_TRUSTED_SERVER_DELETE_TITLE"),
        MessageFormat.format(
          LNG.get("IAS_TRUSTED_SERVER_DELETING_MESSAGE"),
          new Object[] { localServer.getName() }));
    }
  }

  /**
   * Obtm todos os servidores confiveis do repositrio.
   *
   * @return Um conjunto ordenado (por nome) com os servidores confiveis. Caso
   *         ocorra algum erro, ser retornado null.
   */
  private List<TrustedServer> getTrustedServers() {
    RemoteTask<SortedSet<TrustedServer>> getAllLocalServersTask =
      new RemoteTask<SortedSet<TrustedServer>>() {
        protected void performTask()
        throws RemoteException {
          this.setResult(TrustedServerManager.getInstance().getAllTrustedServers());
        }
      };
    SortedSet<TrustedServer> localServerSortedSet = null;
    if (
      getAllLocalServersTask.execute(
        this.parent, LNG.get("IAS_TRUSTED_SERVER_GET_ALL_TITLE"),
        LNG.get("IAS_TRUSTED_SERVER_GETTING_ALL_MESSAGE"))) {
      localServerSortedSet = getAllLocalServersTask.getResult();
    }
    if (localServerSortedSet == null) {
      return new LinkedList<TrustedServer>();
    }
    return new LinkedList<TrustedServer>(localServerSortedSet);
  }

  public void beforeClose() {
    if (TrustedServerManager.getInstance().isEnabled()) {
      TrustedServerManager.getInstance().removeTrustedServerManagerListener(
        this.managerListener);
    }
  }
}
