/*
 * $Id: BasicProjectTree.java 76954 2008-06-06 19:24:00Z taciana $
 */
package csbase.client.project;

import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;

import tecgraf.javautils.LNG;
import csbase.client.desktop.RemoteTask;
import csbase.client.kernel.ClientException;
import csbase.client.project.action.CommonDirectoryCreationAction;
import csbase.client.project.action.CommonDirectoryRefreshAction;
import csbase.client.project.action.CommonFileChangeStateAction;
import csbase.client.project.action.CommonFileCopyAction;
import csbase.client.project.action.CommonFileCutAction;
import csbase.client.project.action.CommonFileDeleteAction;
import csbase.client.project.action.CommonFileDownloadAction;
import csbase.client.project.action.CommonFileImportAction;
import csbase.client.project.action.CommonFileOpenAction;
import csbase.client.project.action.CommonFilePasteAction;
import csbase.client.project.action.CommonFilePropertiesShowAction;
import csbase.client.project.action.CommonFilePublishAction;
import csbase.client.project.action.CommonFileRenameAction;
import csbase.client.project.action.CommonFileTypeChangeAction;
import csbase.client.project.action.CommonFileUpdateAction;
import csbase.client.project.action.CommonMultiSelectionFilePublishAction;
import csbase.client.project.action.CommonMultiSelectionFileTypeChangeAction;
import csbase.client.project.action.DirectorySortExtAction;
import csbase.client.project.action.DirectorySortNameAction;
import csbase.client.project.action.DirectorySortTypeAction;
import csbase.client.project.action.ExportFilesToDirAction;
import csbase.client.project.action.FileUnlockAction;
import csbase.client.project.action.ProjectTreeUpdateAction;
import csbase.client.project.action.SingleFileExportAction;
import csbase.logic.ClientProjectFile;
import csbase.logic.CommonClientProject;
import csbase.logic.ProjectFile;
import csbase.logic.ProjectFileFilter;
import csbase.logic.User;

/**
 * Esta classe  uma implementao bsica (ou mnima) de uma rvore de projetos,
 * para uso por clientes de sistemas desenvolvidos a partir do CSBASE. Esta
 * implementao deve ser revista!
 *
 * @author Tecgraf/PUC-Rio
 */
public class BasicProjectTree extends ProjectTree {

  /**
   * Item de menu para criao de diretrio
   */
  protected JMenuItem rootCreateDirMenuItem;

  /**
   * Item de menu para ordenao de diretrio
   */
  protected JMenuItem rootSortDirMenuItem;

  /**
   * Item de menu para atualizar diretrio
   */
  protected JMenuItem rootRefreshProjectMenuItem;

  /**
   * Item de menu para cpia
   */
  protected JMenuItem rootCopyFileMenuItem;

  /**
   * Item de menu para colagem
   */
  protected JMenuItem rootPasteFileMenuItem;

  /**
   * Item de menu para importao no diretrio
   */
  protected JMenuItem rootImportMenuItem;

  /**
   * Item de menu para exportao do diretrio
   */
  protected JMenuItem rootExportMenuItem;

  /**
   * Boto para ordenao da rvore pelo nome
   */
  protected JRadioButtonMenuItem rootSortByNameRadio;

  /**
   * Boto para ordenao da rvore pela extenso
   */
  protected JRadioButtonMenuItem rootSortByExtRadio;

  /**
   * Boto para ordenao da rvore pelo tipo
   */
  protected JRadioButtonMenuItem rootSortByTypeRadio;

  /**
   * Item de menu para criao do diretrio
   */
  protected JMenuItem dirCreateDirMenuItem;

  /**
   * Item de menu para ordenao do diretrio
   */
  protected JMenuItem dirSortDirMenuItem;

  /**
   * Item de menu para atualizao do diretrio
   */
  protected JMenuItem dirRefreshDirMenuItem;

  /**
   * Item de menu para recortar o diretrio
   */
  protected JMenuItem dirCutFileMenuItem;

  /**
   * Item de menu para cpia do diretrio
   */
  protected JMenuItem dirCopyFileMenuItem;

  /**
   * Item de menu para colagem no diretrio
   */
  protected JMenuItem dirPasteFileMenuItem;

  /**
   * Item de menu para renomear o diretrio
   */
  protected JMenuItem dirRenameFileMenuItem;

  /**
   * Item de menu para apagar o diretrio
   */
  protected JMenuItem dirDeleteFileMenuItem;

  /**
   * Item de menu para importao no diretrio
   */
  protected JMenuItem dirImportMenuItem;

  /**
   * Item de menu para exportao do diretrio
   */
  protected JMenuItem dirExportMenuItem;

  /**
   * Boto para ordenao do diretrio pelo nome
   */
  protected JRadioButtonMenuItem dirSortByNameRadio;

  /**
   * Boto para ordenao do diretrio pela extenso
   */
  protected JRadioButtonMenuItem dirSortByExtRadio;

  /**
   * Boto para ordenao do diretrio pelo tipo
   */
  protected JRadioButtonMenuItem dirSortByTypeRadio;

  /**
   * Item de menu para abrir arquivo com a aplicao associada
   */
  protected JMenuItem fileOpenMenuItem;

  /**
   * Item de menu para abrir arquivo com o bloco de notas
   */
  protected JMenuItem notepadFileOpenMenuItem;

  /**
   * Item de menu para visualizar as propriedades do arquivo
   */
  protected JMenuItem fileSeeFilePropertiesMenuItem;

  /**
   * Item de menu para publicar o arquivo
   */
  protected JMenuItem filePublishFileMenuItem;

  /**
   * Item de menu para recortar o arquivo
   */
  protected JMenuItem fileCutFileMenuItem;

  /**
   * Item de menu para copiar o arquivo
   */
  protected JMenuItem fileCopyFileMenuItem;

  /**
   * Item de menu para renomear o arquivo
   */
  protected JMenuItem fileRenameFileMenuItem;

  /**
   * Item de menu para alterar o tipo do arquivo
   */
  protected JMenuItem fileChangeFileTypeMenuItem;

  /**
   * Item de menu para apagar o arquivo
   */
  protected JMenuItem fileDeleteFileMenuItem;

  /**
   * Item de menu para visualizar o arquivo com o navegador
   */
  protected JMenuItem fileDownloadFileMenuItem;

  /**
   * Item de menu para exportar o arquivo
   */
  protected JMenuItem fileExportFileMenuItem;

  /**
   * Item de menu para atualizar um arquivo
   */
  protected JMenuItem fileUpdateFileMenuItem;

  /**
   * Submenu para as opes avanado
   */
  protected JMenu advancedSubmenu;

  /**
   * Submenu de aes que dependem dos arquivos selecionados
   */
  protected JMenu msActionsSubmenu;

  /**
   * Submenu de aes que dependem do arquivo selecionado
   */
  protected JMenu fileActionsSubmenu;

  /**
   * Submenu para trocar o estado dos arquivo em construo
   */
  protected JMenuItem fileChangeStateFileMenuItem;

  /**
   * Submenu para desbloquear arquivo.
   */
  protected JMenuItem fileUnlockFileMenuItem;

  /**
   * Item de menu para cpia com seleo mltipla
   */
  protected JMenuItem msCutFileMenuItem;

  /**
   * Item de menu para recortar com seleo mltipla
   */
  protected JMenuItem msCopyFileMenuItem;

  /**
   * Item de menu para apagar com seleo mltipla
   */
  protected JMenuItem msDeleteFileMenuItem;

  /**
   * Item de menu para trocar o tipo dos arquivos com seleo mltipla
   */
  protected JMenuItem fileMultiSelectionChangeFileTypeMenuItem;

  /**
   * Item de menu para publicao com seleo mltipla
   */
  protected JMenuItem fileMultiSelectionPublishFileMenuItem;

  /**
   * Item de menu para exportao com seleo mltipla
   */
  protected JMenuItem msExportFileMenuItem;

  /**
   * Constri uma viso da rvore sem projeto associado.
   *
   * @param owner janela na qual a rvore  exibida
   * @param title ttulo da rvore
   *
   * @throws ClientException
   */
  public BasicProjectTree(Window owner, String title) throws ClientException {
    this(owner, title, null, null);
  }

  /**
   * Constri uma viso da rvore de um diretrio de um projeto.
   *
   * @param owner janela na qual a rvore  exibida
   * @param title ttulo da rvore
   * @param project o projeto cuja rvore  apresentada.
   *
   * @throws ClientException
   */
  public BasicProjectTree(Window owner, String title,
    CommonClientProject project) throws ClientException {
    this(owner, title, project, null);
  }

  /**
   * Constri uma viso da rvore de um diretrio de um projeto, usando um
   * filtro para montar a rvore.
   *
   * @param owner janela na qual a rvore  exibida
   * @param title ttulo da rvore
   * @param project o projeto cuja rvore  apresentada.
   * @param visualFilter filtro a ser utilizado para a construo da rvore.
   *
   * @throws ClientException
   */
  public BasicProjectTree(Window owner, String title,
    CommonClientProject project, ProjectFileFilter visualFilter)
    throws ClientException {
    this(owner, title, project, visualFilter, null);
  }

  /**
   * Constri uma viso da rvore de um diretrio de um projeto, usando filtros
   * para montar a rvore e para a seleo de arquivos.
   *
   * @param owner janela na qual a rvore  exibida
   * @param title ttulo da rvore
   * @param project o projeto cuja rvore  apresentada.
   * @param visualFilter filtro de visualizao
   * @param selectionFilter filtero de seleo
   *
   * @throws ClientException
   */
  public BasicProjectTree(Window owner, String title,
    CommonClientProject project, ProjectFileFilter visualFilter,
    ProjectFileFilter selectionFilter) throws ClientException {
    super(owner, title, project, visualFilter, selectionFilter);

    try {
      ClientProjectFileType.loadFileTypes(LNG.getLocale());
    }
    catch (Exception e) {
      throw new ClientException(LNG.get("desktop.file_types_error"), e);
    }
    addProjectTreeListener(new ProjectTreeAdapter() {
      @Override
      public void popupFileTriggered(ProjectFile file) {
        configureFileMenu(file);
      }

      @Override
      public void popupDirTriggered(ProjectFile dir) {
        configureDirMenu(dir);
      }

      @Override
      public void popupMultSelectionTriggered() {
        configureMultipleSelectionMenu();
      }

      @Override
      public void popupRootTriggered() {
        configureRootMenu();
      }
    });
    setPopupRoot(createPopupRoot());
    setPopupDir(createPopupDir());

    /*
     * Os menus do diretrio raiz e dos subdiretrios, que selecionam a
     * ordenao da rvore, precisam * ficar consistentes, pois a ordenao 
     * sempre * feita na rvore como um todo.
     */
    dirSortByNameRadio.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        rootSortByNameRadio.setSelected(true);
      }
    });
    dirSortByExtRadio.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        rootSortByExtRadio.setSelected(true);
      }
    });
    dirSortByTypeRadio.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        rootSortByTypeRadio.setSelected(true);
      }
    });

    rootSortByNameRadio.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        dirSortByNameRadio.setSelected(true);
      }
    });
    rootSortByExtRadio.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        dirSortByExtRadio.setSelected(true);
      }
    });
    rootSortByTypeRadio.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        dirSortByTypeRadio.setSelected(true);
      }
    });

    setPopupFile(createPopupFile());
    setPopupMultSelection(createPopupMultSelection());
    setSingleSelectionMode(false);

  }

  /**
   * Cria o menu para ordenao de um diretrio do projeto.
   *
   * @param nameRadio radioButton para ordenao por nome
   * @param extRadio radioButton para ordernao por extenso
   * @param typeRadio radioButton para ordernao por extenso
   *
   * @return o menu de ordenao
   */
  protected JMenuItem createSortDirMenuItem(
    final JRadioButtonMenuItem nameRadio, final JRadioButtonMenuItem extRadio,
    final JRadioButtonMenuItem typeRadio) {
    JMenuItem sortDirMenuItem = new JMenu(LNG.get("PRJ_SORT_DIR"));
    ButtonGroup sortGroup = new ButtonGroup();
    sortGroup.add(nameRadio);
    sortGroup.add(extRadio);
    sortGroup.add(typeRadio);
    nameRadio.setSelected(true);
    sortDirMenuItem.add(nameRadio);
    sortDirMenuItem.add(extRadio);
    sortDirMenuItem.add(typeRadio);
    return sortDirMenuItem;
  }

  /**
   * Cria o menu popup de um diretrio raiz de projeto.
   *
   * @return o menu popup do diretrio raiz
   */
  protected ProjectTreePopupMenu createPopupRoot() {
    /* Cria os tens de menu */
    rootCreateDirMenuItem =
      new JMenuItem(new CommonDirectoryCreationAction(this));
    rootSortByNameRadio =
      new JRadioButtonMenuItem(new DirectorySortNameAction(this));
    rootSortByExtRadio =
      new JRadioButtonMenuItem(new DirectorySortExtAction(this));
    rootSortByTypeRadio =
      new JRadioButtonMenuItem(new DirectorySortTypeAction(this));
    rootSortDirMenuItem =
      createSortDirMenuItem(rootSortByNameRadio, rootSortByExtRadio,
        rootSortByTypeRadio);
    rootRefreshProjectMenuItem =
      new JMenuItem(new ProjectTreeUpdateAction(this));
    rootCopyFileMenuItem = new JMenuItem(new CommonFileCopyAction(this));
    rootPasteFileMenuItem = new JMenuItem(new CommonFilePasteAction(this));
    rootImportMenuItem = new JMenuItem(new CommonFileImportAction(this));
    rootExportMenuItem = new JMenuItem(new ExportFilesToDirAction(this));

    /* Cria o menu */
    ProjectTreePopupMenu menu = new ProjectTreePopupMenu();
    menu.add(rootCreateDirMenuItem);
    menu.add(rootSortDirMenuItem);
    menu.add(rootRefreshProjectMenuItem);
    menu.addSeparator();
    menu.add(rootCopyFileMenuItem);
    menu.add(rootPasteFileMenuItem);
    menu.addSeparator();
    menu.add(rootImportMenuItem);
    menu.add(rootExportMenuItem);
    return menu;
  }

  /**
   * Cria o menu popup de um diretrio de projeto.
   *
   * @return o menu popup de um diretrio
   */
  protected ProjectTreePopupMenu createPopupDir() {
    /* Cria os tens de menu */
    dirCreateDirMenuItem =
      new JMenuItem(new CommonDirectoryCreationAction(this));

    dirSortByNameRadio =
      new JRadioButtonMenuItem(new DirectorySortNameAction(this));

    dirSortByExtRadio =
      new JRadioButtonMenuItem(new DirectorySortExtAction(this));

    dirSortByTypeRadio =
      new JRadioButtonMenuItem(new DirectorySortTypeAction(this));

    dirSortDirMenuItem =
      createSortDirMenuItem(dirSortByNameRadio, dirSortByExtRadio,
        dirSortByTypeRadio);

    dirRefreshDirMenuItem =
      new JMenuItem(new CommonDirectoryRefreshAction(this));
    dirCutFileMenuItem = new JMenuItem(new CommonFileCutAction(this));
    dirCopyFileMenuItem = new JMenuItem(new CommonFileCopyAction(this));
    dirPasteFileMenuItem = new JMenuItem(new CommonFilePasteAction(this));
    dirRenameFileMenuItem = new JMenuItem(new CommonFileRenameAction(this));
    dirDeleteFileMenuItem = new JMenuItem(new CommonFileDeleteAction(this));
    dirImportMenuItem = new JMenuItem(new CommonFileImportAction(this));
    dirExportMenuItem = new JMenuItem(new ExportFilesToDirAction(this));

    /* Cria o menu */
    ProjectTreePopupMenu menu = new ProjectTreePopupMenu();
    menu.add(dirCreateDirMenuItem);
    menu.add(dirSortDirMenuItem);
    menu.add(dirRefreshDirMenuItem);
    menu.addSeparator();
    menu.add(dirCutFileMenuItem);
    menu.add(dirCopyFileMenuItem);
    menu.add(dirPasteFileMenuItem);
    menu.addSeparator();
    menu.add(dirRenameFileMenuItem);
    menu.add(dirDeleteFileMenuItem);
    menu.addSeparator();
    menu.add(dirImportMenuItem);
    menu.add(dirExportMenuItem);
    return menu;
  }

  /**
   * Cria o menu popup de um arquivo do projeto.
   *
   * @return o menu popup do arquivo
   */
  protected ProjectTreePopupMenu createPopupFile() {
    /* Cria os tens de menu */
    fileOpenMenuItem = new JMenuItem(new CommonFileOpenAction(this));
    notepadFileOpenMenuItem =
      new JMenuItem(new CommonFileOpenAction(this, "notepad", LNG
        .get("BasicProjectTree.open.notepad")));

    fileSeeFilePropertiesMenuItem =
      new JMenuItem(new CommonFilePropertiesShowAction(this));
    filePublishFileMenuItem = new JMenuItem(new CommonFilePublishAction(this));
    fileUpdateFileMenuItem = new JMenuItem(new CommonFileUpdateAction(this));
    fileCutFileMenuItem = new JMenuItem(new CommonFileCutAction(this));
    fileCopyFileMenuItem = new JMenuItem(new CommonFileCopyAction(this));
    fileRenameFileMenuItem = new JMenuItem(new CommonFileRenameAction(this));
    fileChangeFileTypeMenuItem =
      new JMenuItem(new CommonFileTypeChangeAction(this));
    fileDeleteFileMenuItem = new JMenuItem(new CommonFileDeleteAction(this));
    fileDownloadFileMenuItem =
      new JMenuItem(new CommonFileDownloadAction(this));
    fileExportFileMenuItem = new JMenuItem(new SingleFileExportAction(this));

    advancedSubmenu = new JMenu(LNG.get("BasicProjectTree.popupmenu.advanced"));
    fileChangeStateFileMenuItem =
      new JMenuItem(new CommonFileChangeStateAction(this));
    advancedSubmenu.add(fileChangeStateFileMenuItem);
    fileUnlockFileMenuItem =
      new JMenuItem(new FileUnlockAction(this));
    advancedSubmenu.add(fileUnlockFileMenuItem);

    fileActionsSubmenu =
      new JMenu(LNG.get("BasicProjectTree.popupmenu.file.actions"));

    /* Cria o menu */
    ProjectTreePopupMenu menu = new ProjectTreePopupMenu();
    menu.add(fileOpenMenuItem);
    menu.add(notepadFileOpenMenuItem);
    menu.add(fileActionsSubmenu);
    menu.addSeparator();
    menu.add(fileSeeFilePropertiesMenuItem);
    menu.add(filePublishFileMenuItem);
    menu.addSeparator();
    menu.add(fileCutFileMenuItem);
    menu.add(fileCopyFileMenuItem);
    menu.addSeparator();
    menu.add(fileRenameFileMenuItem);
    menu.add(fileChangeFileTypeMenuItem);
    menu.add(fileDeleteFileMenuItem);
    menu.addSeparator();
    menu.add(fileDownloadFileMenuItem);
    menu.add(fileExportFileMenuItem);
    menu.addSeparator();
    menu.add(advancedSubmenu);
    return menu;
  }

  /**
   * Cria o menu popup para uma seleo mltipla de diretrios/arquivos.
   *
   * @return menu popup para uma seleo mltipla
   */
  protected ProjectTreePopupMenu createPopupMultSelection() {
    /* Cria os tens de menu */
    fileMultiSelectionPublishFileMenuItem =
      new JMenuItem(new CommonMultiSelectionFilePublishAction(this));
    msCutFileMenuItem = new JMenuItem(new CommonFileCutAction(this));
    msCopyFileMenuItem = new JMenuItem(new CommonFileCopyAction(this));
    msDeleteFileMenuItem = new JMenuItem(new CommonFileDeleteAction(this));
    fileMultiSelectionChangeFileTypeMenuItem =
      new JMenuItem(new CommonMultiSelectionFileTypeChangeAction(this));
    msExportFileMenuItem = new JMenuItem(new ExportFilesToDirAction(this));

    msActionsSubmenu =
      new JMenu(LNG.get("BasicProjectTree.popupmenu.multselection.actions"));

    /* Cria o menu */
    ProjectTreePopupMenu menu = new ProjectTreePopupMenu();
    menu.add(msActionsSubmenu);
    menu.add(fileMultiSelectionPublishFileMenuItem);
    menu.addSeparator();
    menu.add(msCutFileMenuItem);
    menu.add(msCopyFileMenuItem);
    menu.addSeparator();
    menu.add(fileMultiSelectionChangeFileTypeMenuItem);
    menu.add(msDeleteFileMenuItem);
    menu.addSeparator();
    menu.add(msExportFileMenuItem);
    return menu;
  }

  /**
   * Configura o menu de popup da raiz do projeto.
   */
  protected void configureRootMenu() {
    ProjectFile selectedFile = getSelectedFile();
    boolean rootSelected =
      (selectedFile != null && selectedFile.getParent() == null);
    rootCreateDirMenuItem.setEnabled(rootSelected);
    rootSortDirMenuItem.setEnabled(true);
    rootRefreshProjectMenuItem.setEnabled(refreshEnabled);
    rootCopyFileMenuItem.setEnabled(rootSelected && transferEnabled);
    if (rootSelected && transferEnabled && hasFileToPaste()) {
      rootPasteFileMenuItem.setEnabled(true);
    }
    else {
      rootPasteFileMenuItem.setEnabled(false);
    }
    rootImportMenuItem.setEnabled(rootSelected);
  }

  /**
   * Configura o menu de popup de um diretrio.
   *
   * @param dir o diretrio selecionado
   */
  protected void configureDirMenu(ProjectFile dir) {
    boolean dirSelected = (getSelectedFile() == dir);
    dirCreateDirMenuItem.setEnabled(dirSelected);
    dirSortDirMenuItem.setEnabled(true);
    dirRefreshDirMenuItem.setEnabled(refreshEnabled);
    dirCutFileMenuItem.setEnabled(dirSelected && transferEnabled);
    dirCopyFileMenuItem.setEnabled(dirSelected && transferEnabled);
    if (dirSelected && transferEnabled && hasFileToPaste()) {
      dirPasteFileMenuItem.setEnabled(true);
    }
    else {
      dirPasteFileMenuItem.setEnabled(false);
    }
    dirRenameFileMenuItem.setEnabled(dirSelected && renameEnabled);
    dirDeleteFileMenuItem.setEnabled(dirSelected && deleteEnabled);
    dirImportMenuItem.setEnabled(dirSelected);
  }

  /**
   * Configura o menu de popup de acordo com o arquivo.
   *
   * @param file arquivo selecionado
   */
  protected void configureFileMenu(ProjectFile file) {
    ClientProjectFile projectFile = (ClientProjectFile) file;
    boolean isUnderConstruction = projectFile.isUnderConstruction();
    fileSeeFilePropertiesMenuItem.setEnabled(true);
    fileUpdateFileMenuItem.setEnabled(projectFile.isUpdatable());
    fileDeleteFileMenuItem.setEnabled(deleteEnabled);
    fileCutFileMenuItem.setEnabled(!isUnderConstruction && transferEnabled);
    fileCopyFileMenuItem.setEnabled(!isUnderConstruction && transferEnabled);
    fileRenameFileMenuItem.setEnabled(!isUnderConstruction && renameEnabled);
    fileChangeFileTypeMenuItem.setEnabled(!isUnderConstruction);
    fileDownloadFileMenuItem.setEnabled(!isUnderConstruction);
    fileExportFileMenuItem.setEnabled(!isUnderConstruction);
    fileChangeStateFileMenuItem.setEnabled(isUnderConstruction);
    // Hibilita menu "Avanado".
    User user = User.getLoggedUser();
    boolean enableUnlock = file.isLocked() &&
      (getProject().getUserId().equals(user.getId()) || (user.isAdmin()));
    fileUnlockFileMenuItem.setEnabled(enableUnlock);
    advancedSubmenu.setEnabled(isUnderConstruction || enableUnlock);

    if (isUnderConstruction) {
      filePublishFileMenuItem.setEnabled(false);
    }
    else {
      filePublishFileMenuItem.setEnabled(projectFile.canPublishFile()
        || projectFile.canUnpublishFile());
    }

    if (fileActionsSubmenu != null) {
      fileActionsSubmenu.removeAll();
      fileActionsSubmenu.setEnabled(false);

      final ProjectFile selFile = getSelectedFile();
      if (selFile == null) {
        return;
      }
      final String type = file.getType();
      final boolean enabled =
        configureFileActionsSubMenu(fileActionsSubmenu, type, file);
      fileActionsSubmenu.setEnabled(enabled);
    }
  }

  /**
   * Ajusta aes padronizadas de acordo com o(s) arquivo(s) selecionados quando
   * todos so do mesmo tipo.
   *
   * @param menu o menu
   * @param type o tipo do arquivo.
   * @param selectedFile o arquivo selecionado.
   * @return se o submenu de aes deve ficar ativo ou no.
   */
  protected boolean configureFileActionsSubMenu(final JMenu menu,
    final String type, final ProjectFile selectedFile) {
    return false;
  }

  /**
   * Ajusta aes padronizadas de acordo com o(s) arquivo(s) selecionados quando
   * todos so do mesmo tipo.
   *
   * @param menu o menu
   * @param type o tipo dos arquivos.
   * @param selectedFiles um array dos arquivos selecionados.
   * @return se o submenu de aes deve ficar ativo ou no.
   */
  protected boolean configureMultSelectionActionsSubMenu(final JMenu menu,
    final String type, final ProjectFile[] selectedFiles) {
    return false;
  }

  /**
   * Configura o menu de popup para uma seleo mltipla de arquivos/diretrios
   */
  protected void configureMultipleSelectionMenu() {
    ProjectFile[] selectedFiles = getSelectedFiles();
    if (ClientProjectFile.canPublishFiles((ClientProjectFile[]) selectedFiles)
      || canUnpublishFiles(selectedFiles)) {
      fileMultiSelectionPublishFileMenuItem.setEnabled(true);
    }
    else {
      fileMultiSelectionPublishFileMenuItem.setEnabled(false);
    }
    msCutFileMenuItem.setEnabled(!ClientProjectFile
      .hasFileUnderConstruction(selectedFiles)
      && transferEnabled);
    msCopyFileMenuItem.setEnabled(!ClientProjectFile
      .hasFileUnderConstruction(selectedFiles)
      && transferEnabled);
    msDeleteFileMenuItem.setEnabled(deleteEnabled);
    fileMultiSelectionChangeFileTypeMenuItem.setEnabled(!ClientProjectFile
      .hasDirectories(selectedFiles));

    if (msActionsSubmenu != null) {
      msActionsSubmenu.removeAll();
      msActionsSubmenu.setEnabled(false);
      final ProjectFile[] selFiles = getSelectedFiles();
      if (selFiles == null || selFiles.length == 0) {
        return;
      }

      final ProjectFile file0 = selFiles[0];
      final String type0 = file0.getType();
      for (ProjectFile f : selFiles) {
        final String type = f.getType();
        if (!type.equals(type0)) {
          return;
        }
      }
      final boolean enabled =
        configureMultSelectionActionsSubMenu(msActionsSubmenu, type0, selFiles);
      msActionsSubmenu.setEnabled(enabled);
    }
  }

  /**
   * Verifica se  possvel despublicar todos os arquivos de projetos.
   *
   * @param selectedFiles Os arquivos de projeto que se deseja despublicar.
   *
   * @return true, se for possvel despublicar todos os arquivos de projeto, ou
   *         false, caso contrrio.
   */
  private boolean canUnpublishFiles(ProjectFile[] selectedFiles) {
    CanUnpublishFileTask task =
      new CanUnpublishFileTask((ClientProjectFile[]) selectedFiles);
    return ((task.execute(this.getWindow(), this.getTitle(), LNG
      .get("PRJ_WAITING_CAN_UNPUBLISH_FILES"))) && (task.getResult()));
  }

  /**
   * <p>
   * Classe utilizada para verificar se um grupo de arquivos pode ser
   * despublicado.
   * </p>
   *
   * <p>
   * O Resultado da execuo dessa task  um {@link Boolean} que indica se 
   * possvel, ou no, despublicar todo o grupo de arquivos.
   * </p>
   */
  private static class CanUnpublishFileTask extends RemoteTask<Boolean> {
    /** Arquivos cuja publicao ser revertida */
    private final ClientProjectFile[] projectFileArray;

    /**
     * Constri a <code>CanUnpublishFileTask</code>
     *
     * @param projectFileArray array de arquivos cuja publicao ser revertida
     */
    CanUnpublishFileTask(ClientProjectFile[] projectFileArray) {
      this.projectFileArray = projectFileArray;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void performTask() throws Exception {
      setResult(ClientProjectFile.canUnpublishFiles(this.projectFileArray));
    }
  }
}
