/*
 * ClientProjectFileType.java
 *
 * $Author$
 * $Revision: 66445 $ - $Date$
 */
package csbase.client.project;

import java.rmi.RemoteException;
import java.util.Locale;

import csbase.logic.ProjectFileType;
import csbase.logic.ProjectFileTypeInfo;

import javax.swing.ImageIcon;

/**
 * A classe <code>ClientProjectFileType</code> mantm todos os tipos  de
 * arquivos conhecidos pelo sistema.
 *
 * @author Cristina Ururahy
 */
public class ClientProjectFileType extends ProjectFileType {
  /** cone de um arquivo regular */
  private ImageIcon baseIcon;

  /** cone de um arquivo recortado */
  private ImageIcon cutIcon;

  /** cone de um arquivo proveniente da base */
  private ImageIcon dbIcon;

  /** cone de um arquivo publicado pelo  prprio usurio */
  private ImageIcon pubMeIcon;

  /** cone de um arquivo publicado por outro usurio  */
  private ImageIcon pubOtherIcon;

  /** cone de um arquivo em construo  */
  private ImageIcon ucIcon;

  /**
   * {@inheritDoc}
   * @see csbase.logic.ProjectFileType#createProjectFileType(csbase.logic.ProjectFileTypeInfo)
   */
  @Override
  protected ProjectFileType createProjectFileType(
    ProjectFileTypeInfo info) {
    return new ClientProjectFileType(info);
  }

  /**
   * Obtm todos os tipos de arquivos definidos.
   * 
   * @return um array com todos os tipos de arquivos
   */
  public static ClientProjectFileType[] getAllFileTypes() {
    return (ClientProjectFileType[]) ProjectFileType.getAllFileTypes();
  }

  /**
   * Obtm o tipo de arquivo que possui um determinado cdigo.
   * 
   * @param typeCode identificador do arquivo
   * @return o tipo de arquivo que possui um determinado cdigo
   */
  public static ClientProjectFileType getFileType(String typeCode) {
    projectFileTypePrototype = new ClientProjectFileType();
    return (ClientProjectFileType) ProjectFileType.getFileType(typeCode);
  }

  /**
   * Chama o mtodo remoto que carrega os tipos de arquivos a partir do
   * locale
   * 
   * @param locale configurao de idioma
   * @throws RemoteException erro na chamada remota 
   */
  public static void loadFileTypes(Locale locale) throws RemoteException {
    projectFileTypePrototype = new ClientProjectFileType();
    ProjectFileType.loadFileTypes(locale);
  }

  /**
   * Cria um ClientProjectFileType
   *
   * @param info .
   */
  public ClientProjectFileType(ProjectFileTypeInfo info) {
    super(info);
    this.baseIcon = new ImageIcon(info.baseIcon);
    this.ucIcon = new ImageIcon(info.ucIcon);
    this.dbIcon = new ImageIcon(info.dbIcon);
    this.cutIcon = new ImageIcon(info.cutIcon);
    this.pubMeIcon = new ImageIcon(info.pubMeIcon);
    this.pubOtherIcon = new ImageIcon(info.pubOtherIcon);
  }

/**
 * Cria um prottipo do objeto ProjectFileType, para que o mtodo
 * createProjectFileType possa ser chamado de forma apropriada.
 */
  protected ClientProjectFileType() {
  }

  /**
   * Obtm o cone que representa um arquivo desse tipo no estado regular.
   *
   * @return .
   */
  public ImageIcon getBaseIcon() {
    return this.baseIcon;
  }

  /**
   * Obtm o cone que representa um arquivo desse tipo, no estado recortado.
   *
   * @return .
   */
  public ImageIcon getCutIcon() {
    return this.cutIcon;
  }

  /**
   * Obtm o cone que representa um arquivo desse tipo, proveniente da base de
   * dados.
   *
   * @return .
   */
  public ImageIcon getDataBaseIcon() {
    return this.dbIcon;
  }

  /**
   * Obtm o cone que representa um arquivo desse tipo, publicado pelo prprio
   * usurio.
   *
   * @return .
   */
  public ImageIcon getPublishedByMeIcon() {
    return this.pubMeIcon;
  }

  /**
   * Obtm o cone que representa um arquivo desse tipo, publicado por outro
   * usurio.
   *
   * @return .
   */
  public ImageIcon getPublishedByOtherIcon() {
    return this.pubOtherIcon;
  }

  /**
   * Obtm o cone que representa um arquivo desse tipo no estado em
   * construo.
   *
   * @return .
   */
  public ImageIcon getUnderConstructionIcon() {
    return this.ucIcon;
  }
}
