/*
 * ProjectInfoDialog.java
 * 
 * $Author: taciana $ $Date: 2007-10-15 10:54:41 -0200 (Mon, 15 Oct 2007) $
 * $Revision: 96246 $
 */
package csbase.client.project;

import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GUIUtils;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.desktop.RemoteTask;
import csbase.client.util.ClientUtilities;
import csbase.client.util.NumericTextField;
import csbase.logic.CommonClientProject;
import csbase.logic.CommonProjectInfo;
import csbase.logic.User;
import csbase.logic.UserProjectInfo;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectServiceInterface;

/**
 * A classe <code>ProjectInfoDialog</code> mostra um dilogo para criao ou
 * modificao das informaes de um projeto. Na criao de um projeto, o
 * usurio pode acionar o boto de criao, que confirma a operao, ou pode
 * acionar o boto de cancelamento, que desiste da operao. O boto de criao
 * somente  habilitado quando o usurio tiver informado um nome do projeto. Na
 * modificao de um projeto, o usurio pode acionar o boto de alterar, que
 * confirma a operao, ou pode acionar o boto de cancelamento, que desiste da
 * operao. O boto de alterao somente  habilitado quando o usurio tiver
 * alterado algum dado de projeto. No caso, o usurio pode modificar apenas a
 * descrio de um projeto. Ao confirmar as operao, seja de criao ou de
 * alterao, o servio  executado no servidor e o usurio  notificado de
 * qualquer problema que tenha ocorrido.
 */
public class ProjectCreationWithAllocationDialog extends
  CommonProjectInfoDialog {
  /** O campo da projeo adotado pelo projeto. */
  protected JComboBox projectionField;
  /** O campo do datum adotado pelo projeto. */
  protected JComboBox datumField;
  /** O campo do meridiano adotado pelo projeto. */
  protected JComboBox mcField;
  /** O campo que indica se o projeto quer ou no uma reserva de rea. */
  protected JCheckBox lockingOption;
  /** O campo com o tamanho da rea reservada. */
  protected NumericTextField sizeField;

  /**
   * Mostra um dilogo com as informaes de um projeto existente. O usurio
   * pode alterar os dados do projeto. Caso a operao seja confirmada, os dados
   * so alterados no servidor.
   * 
   * @param owner A janela a partir da qual esse dilogo foi solicitado.
   * @param project O projeto a ser alterado.
   * 
   * @return O projeto modificado.
   */
  public static CommonClientProject updateProject(Window owner,
    CommonClientProject project) {
    return new ProjectModificationDialog(owner, project).project;
  }

  /**
   * Mostra um dilogo para que o usurio crie um projeto. O usurio informa os
   * dados do projeto. Caso a operao seja confirmada, o servidor  requisitado
   * para executar o servio de criao do projeto.
   * 
   * @param owner A janela a partir da qual esse dilogo foi solicitado.
   * 
   * @return O projeto criado.
   */
  public static CommonClientProject createProject(Window owner) {
    return new ProjectCreationDialog(owner).project;
  }

  /**
   * Cria um painel com as opes para o sistema de coordenadas de projeto e
   * para a reserva de rea em disco.
   * 
   * @return O painel recm-criado.
   */
  @Override
  protected JPanel makeAditionalPanel() {
    JComponent[][] components = { { makeLockingAreaPanel() }, };
    JPanel panel = GUIUtils.createBasicGridPanel(components);
    return panel;
  }

  /**
   * Cria um painel com as opes de reserva de rea para o projeto.
   * 
   * @return O painel recm-criado.
   */
  private JPanel makeLockingAreaPanel() {

    // Check-box com a opo de reservar ou no uma rea de projeto
    lockingOption =
      new JCheckBox(LNG
        .get("ProjectCreationWithAllocationDialog.reserve.label"));
    lockingOption.setSelected(false);

    // Tamanho da rea a ser reservada
    final JLabel sizeLabel =
      new JLabel(LNG.get("ProjectCreationWithAllocationDialog.size.label"));
    sizeLabel.setEnabled(false);

    sizeField = new NumericTextField(4);
    sizeField.setEnabled(false);

    final JLabel gbLabel =
      new JLabel(LNG.get("ProjectCreationWithAllocationDialog.unit.label"));
    gbLabel.setEnabled(false);

    final JLabel note =
      new JLabel(LNG.get("ProjectCreationWithAllocationDialog.note.size.label"));
    note.setEnabled(false);

    lockingOption.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        sizeLabel.setEnabled(e.getStateChange() == ItemEvent.SELECTED);
        sizeField.setEnabled(e.getStateChange() == ItemEvent.SELECTED);
        gbLabel.setEnabled(e.getStateChange() == ItemEvent.SELECTED);
        note.setEnabled(e.getStateChange() == ItemEvent.SELECTED);
      }
    });

    final JComponent[][] components =
      { { lockingOption }, { sizeLabel, sizeField, gbLabel }, { null, note } };
    JPanel panel = GUIUtils.createBasicGridPanel(components);
    panel.setBorder(BorderFactory.createTitledBorder(LNG
      .get("ProjectCreationWithAllocationDialog.project.locking.title")));
    return panel;
  }

  /**
   * Cria um painel com os botes que vo ser usados no dilogo requisitado.
   * Esse mtodo  abstrato. Depende da ao que originou o dilogo.
   * 
   * @return O painel recm-criado.
   */
  @Override
  protected JPanel makeButtonPanel() {
    return new JPanel();
  }

  /**
   * Informa o ttulo que vai ser utilizado na janela de dilogo. Esse mtodo 
   * abstrato. Depende da ao que originou o dilogo.
   * 
   * @return O ttulo da janela.
   */
  @Override
  protected String getTitle() {
    return "";
  }

  /**
   * Constri uma viso dos campos de dados de um projeto. A viso serve tanto
   * para criao de um projeto novo bem como para atualizao dos dados de um
   * projeto existente.
   * 
   * @param owner A janela a partir da qual esse dilogo foi solicitado.
   */
  protected ProjectCreationWithAllocationDialog(Window owner) {
    super(owner);
  }
}

/**
 * A classe <code>ProjectCreationDialog</code> usa o dilogo com informaes de
 * projeto, para o propsito de criao de um novo projeto no servidor.
 */
class ProjectCreationDialog extends ProjectCreationWithAllocationDialog {
  /**
   * Limpa os campos do dilogo e atribui os listeners a esses campos.
   */
  private void clearDialog() {
    // Inicializa os campos
    nameField.setText("");
    descrField.setText("");
    // Adiciona um listener ao campo de nome para que,
    // quando um nome for digitado, o boto principal
    // seja habilitado.
    nameField.addKeyListener(new KeyAdapter() {
      @Override
      public void keyReleased(KeyEvent e) {
        String name = nameField.getText().trim();
        if (name.equals("")) {
          defaultButton.setEnabled(false);
        }
        else {
          defaultButton.setEnabled(true);
        }
      }
    });
  }

  /**
   * Cria um painel com os botes que vo ser usados no dilogo de criao de
   * projeto. Na criao de um projeto, o usurio pode acionar o boto de
   * criao, que confirma a operao, ou pode acionar o boto de cancelamento,
   * que desiste da operao. O boto de criao somente  habilitado quando o
   * usurio tiver informado um nome do projeto.
   * 
   * @return O painel recm-criado.
   */
  @Override
  protected JPanel makeButtonPanel() {
    JPanel panel = new JPanel();

    // Boto de criao de projeto
    defaultButton =
      (JButton) panel.add(new JButton(LNG
        .get("ProjectCreationWithAllocationDialog.create.button")));
    JButton cancelButton =
      (JButton) panel.add(new JButton(LNG
        .get("ProjectCreationWithAllocationDialog.cancel.button")));
    defaultButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        handleAction();
      }
    });
    defaultButton.setEnabled(false);
    // Boto de cancelamento da operao de criao.
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        dialog.close();
      }
    });
    ClientUtilities.adjustEqualSizes(defaultButton, cancelButton);
    return panel;
  }

  /**
   * Informa o ttulo que vai ser utilizado na janela de criao de projeto.
   * 
   * @return O ttulo da janela.
   */
  @Override
  protected String getTitle() {
    return LNG.get("ProjectCreationWithAllocationDialog.title");
  }

  /**
   * Executa a ao de criao de um projeto. O servio  chamado no servidor.
   * Caso algum problema ocorra, o usurio  informado com uma mensagem em uma
   * outra janela de dilogo.
   */
  private void handleAction() {
    if (lockingOption.isSelected()) {
      createLockedProject();
    }
    else {
      super.createProject();
    }
  }

  /**
   * Cria um projeto bloqueado.
   */
  private void createLockedProject() {
    final CommonProjectInfo info = new CommonProjectInfo();
    info.userId = User.getLoggedUser().getId();
    info.name = nameField.getText();
    info.description = descrField.getText();

    if (!ClientUtilities.isValidFileName(info.name)) {
      final String err = LNG.get("PRJ_PROJECT_FILE_NAME_CHARACTERES_ERROR");
      StandardDialogs.showErrorDialog(dialog, getTitle(), err);
      return;
    }
    final int size;
    try {
      if (lockingOption.isSelected()) {
        final String txt = sizeField.getText();
        size = Integer.parseInt(txt);
      }
      else
        size = 0;
    }
    catch (Exception e) {
      final String err =
        LNG.get("ProjectCreationWithAllocationDialog.invalid.size");
      StandardDialogs.showErrorDialog(dialog, getTitle(), err);
      return;
    }

    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      public void performTask() throws Exception {
        info.setAttribute(CommonClientProject.LOCKING_AREA_SIZE, size);
        CommonClientProject.createProjectWithAllocation(info, size);
        ClientRemoteLocator.server.setServerSessionProperty(
          ProjectServiceInterface.CURRENT_PROJECT_SESSION_PROPERTY,
          new UserProjectInfo(info.userId, info.name));
      }
    };
    if (task.execute(dialog, getTitle(), LNG
      .get("ProjectCreationWithAllocationDialog.creating.msg"))) {
      /* Informa que o projeto foi criado, mas est bloqueado */
      StandardDialogs.showInfoDialog(owner, getTitle(), String.format(LNG
        .get("ProjectCreationWithAllocationDialog.creation.success.msg"),
        info.name));
      dialog.close();
    }
  }

  /**
   * Instancia a viso de informaes de projeto usada para criao de projeto.
   * O dilogo  criado e apresentado.
   * 
   * @param owner A janela a partir da qual esse dilogo foi solicitado.
   */
  ProjectCreationDialog(Window owner) {
    super(owner);
    clearDialog();
    dialog.pack();
    dialog.setVisible(true);
  }
}

/**
 * A classe <code>ProjectModificationDialog</code> usa o dilogo com informaes
 * de projeto, para o propsito de atualizao dos dados de um projeto.
 * Atualmente, a nica informao que o usurio pode alterar  a descrio de um
 * projeto.
 */
class ProjectModificationDialog extends ProjectCreationWithAllocationDialog {
  /**
   * Inicializa os campos do dilogo com os dados do projeto que vai ser
   * atualizado. Impede a modificao dos campos de nome e dos valores do
   * sistema de coordenadas usado pelo projeto. Atribui os listeners aos campos
   * que podem ser modificados para que o boto principal seja habilitado ou
   * no.
   * 
   * @param proj O projeto usado para o preenchimento dos campos.
   */
  private void loadDialog(CommonClientProject proj) {
    // Preenche o nome e a descrio do projeto
    nameField.setText(proj.getName());
    descrField.setText(proj.getDescription());
    long size = proj.getLockingAreaSize();
    if (size != 0) {
      lockingOption.setSelected(true);
      sizeField.setText(String.valueOf(size));
    }

    // Apenas a descrio do projeto pode ser alterada.
    nameField.setEnabled(false);
    lockingOption.setEnabled(false);
    sizeField.setEnabled(false);
    // Adiciona ao campo de descrio, um listener que habilita e
    // desabilita o boto de alterao, dependendo se a descrio
    // tiver sido ou no alterada.
    ChangeListener changeListener = new ChangeListener();
    descrField.addKeyListener(changeListener);
  }

  /**
   * Cria um painel com os botes que vo ser usados no dilogo de modificao
   * de dados de um projeto. Na modificao de um projeto, o usurio pode
   * acionar o boto de alterar, que confirma a operao, ou pode acionar o
   * boto de cancelamento, que desiste da operao. O boto de alterao
   * somente  habilitado quando o usurio tiver alterado algum dado de projeto.
   * No caso, o usurio pode modificar apenas a descrio de um projeto.
   * 
   * @return O painel recm-criado.
   */
  @Override
  protected JPanel makeButtonPanel() {
    JPanel panel = new JPanel();
    defaultButton =
      (JButton) panel.add(new JButton(LNG
        .get("ProjectCreationWithAllocationDialog.update.button")));
    JButton cancelButton =
      (JButton) panel.add(new JButton(LNG
        .get("ProjectCreationWithAllocationDialog.cancel.button")));
    defaultButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        handleAction();
      }
    });
    defaultButton.setEnabled(false);
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        dialog.close();
      }
    });
    return panel;
  }

  /**
   * Informa o ttulo que vai ser utilizado na janela de atualizao de projeto.
   * 
   * @return O ttulo da janela.
   */
  @Override
  protected String getTitle() {
    return LNG.get("ProjectCreationWithAllocationDialog.update.title");
  }

  /**
   * Informa se os dados do projeto foi alterado. O nico campo que permite
   * alterao  a descrio.
   * 
   * @return true of false, caso o projeto tenha sido alterado ou no.
   */
  protected boolean projectHasChanged() {
    return !project.getDescription().equals(descrField.getText());
  }

  /**
   * Executa a ao de modificao de um projeto. O servio  chamado no
   * servidor. Caso algum problema ocorra, o usurio  informado com uma
   * mensagem em uma outra janela de dilogo.
   */
  private void handleAction() {
    project.setDescription(descrField.getText());
    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      public void performTask() throws Exception {
        project.modify();
      }
    };
    if (task.execute(dialog, getTitle(), LNG
      .get("ProjectCreationWithAllocationDialog.updating.msg"))) {
      /* Informa que a modificao ocorreu com sucesso */
      tecgraf.javautils.gui.StandardDialogs.showInfoDialog(dialog, getTitle(),
        LNG.get("ProjectCreationWithAllocationDialog.update.success.msg"));
      dialog.close();
    }
  }

  /**
   * A classe <code>ChangeListener</code>  responsvel por verificar se um
   * campo sofreu uma alterao, mantendo o boto principal do dilogo
   * habilitado ou desabilitado, de acordo.
   */
  class ChangeListener extends KeyAdapter {
    /**
     * {@inheritDoc}
     */
    @Override
    public void keyReleased(KeyEvent e) {
      checkChange();
    }

    /**
     * Check de modificao.
     */
    private void checkChange() {
      if (projectHasChanged()) {
        defaultButton.setEnabled(true);
      }
      else {
        defaultButton.setEnabled(false);
      }
    }
  }

  /**
   * Instancia a viso de informaes de projeto usada para modificao de um
   * projeto existente. O dilogo  criado e apresentado.
   * 
   * @param owner A janela a partir da qual esse dilogo foi solicitado.
   * @param project O projeto que vai sofrer a alterao.
   */
  ProjectModificationDialog(Window owner, CommonClientProject project) {
    super(owner);
    this.project = project;
    loadDialog(project);
    dialog.pack();
    dialog.setVisible(true);
  }
}
