/*
 * $Id: ProjectPreferences.java 95893 2009-09-07 14:23:52Z mjulia $
 */
package csbase.client.project;

import java.util.HashMap;
import java.util.Map;

/**
 * Classe que representa as preferncias da configurao dos projetos do usurio.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class ProjectPreferences {
  
  /**
   * Nome da preferncia referente a atualizao automtica da rvore de projetos
   */
  private final static String AUTO_UPDATE = "AUTO_UPDATE";
  
  /**
   * Nome da preferncia referente a atualizao automtica da rvore de projetos
   */
  private final static String TREE_FILTER = "TREE_FILTER";
  
  /**
   * Atributo da preferncia referente a atualizao automtica da rvore de projetos
   * @see #AUTO_UPDATE
   */
  private boolean autoUpdate = true;
  
  /**
   * Atributo da preferncia referente a atualizao automtica da rvore de projetos
   * @see #TREE_FILTER
   */
  private boolean treeFilter = true;
  
  /**
   * Constri o <code>ProjectPreferences</code> a partir do mapa de valores
   * lido do <code>SharedObject</code>.
   *
   * @param preferences mapa de valores
   */
  public ProjectPreferences(Map<String, Object> preferences) {
    Boolean autoUpdatePreference = (Boolean) preferences.get(AUTO_UPDATE);
    this.autoUpdate = autoUpdatePreference != null ? autoUpdatePreference : true;
    Boolean treeFilterPreference = (Boolean) preferences.get(TREE_FILTER);
    this.treeFilter = treeFilterPreference != null ? treeFilterPreference : true;
  }

  /**
   * Constri o <code>ProjectPreferences</code>.
   * 
   * @param autoUpdate indicao se a rvore de projetos deve ser
   * atualizada automaticamente.
   * @param treeFilter indicao se a rvore de projetos deve exibir um filtro
   */
  public ProjectPreferences(boolean autoUpdate, boolean treeFilter) {
    this.autoUpdate = autoUpdate;
    this.treeFilter = treeFilter;
  }
  
  /**
   * Obtm um mapa com os valores das preferncias para persistir no
   * <code>SharedObject</code>.
   * @return um mapa com os valores das preferncias
   */
  public Map<String,Object> getContent() {
    Map<String, Object> map = new HashMap<String, Object>();
    map.put(AUTO_UPDATE, autoUpdate);
    map.put(TREE_FILTER, treeFilter);
    return map;
  }

  /**
   * Obtm a indicao se a rvore de projetos deve ser
   * atualizada automaticamente.
   *
   * @return .
   */
  public boolean getAutoUpdate() {
    return autoUpdate;
  }
  
  /**
   * Obtm a indicao se a rvore de projetos deve exibir um filtro para 
   *
   * @return .
   */
  public boolean getTreeFilter() {
    return treeFilter;
  }
}
