/**
 * ProjectTreeMouseAdapter.java $Author: taciana $ $Revision: 66445 $  - $Date:
 * 2004/01/12 18:51:50 $
 */
package csbase.client.project;

import csbase.client.project.action.CommonFileOpenAction;
import csbase.logic.ProjectFile;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import java.util.Enumeration;

import javax.swing.JTree;
import javax.swing.tree.TreePath;

/**
 * A classe <code>ProjectTreeMouseAdapter</code> modela os eventos de uma
 * rvore de diretrios de um projeto.
 *
 * @author Taciana
 */
public class ProjectTreeMouseAdapter extends MouseAdapter {
  /** Arvore sobre a qual este Adapter funciona */
  private ProjectTree projectTree;
  JTree tree;
  int X;
  int Y;
  TreePath path;
  ProjectFile file;
  private boolean isValidPath;

  @Override
  public void mouseReleased(MouseEvent me) {
    if (isValidPath && me.isPopupTrigger()) {
      showPopupMenu();
    }
  }

  /**
   * Processa o evento de mouseClicked e caso tenha ocorrido um
   * double-click executa a ao correspondente.
   * O double-click tem que ser capturado neste evento, seno no Windows
   * o efeito  que a qualquer janela aberta em decorrncia de um
   * double-click aparece atrs da janela em que o evento
   * aconteceu.
   */
  @Override
  public void mouseClicked(MouseEvent me) {
    if (isValidPath && (me.getClickCount() == 2)) {
      doubleClickAction();
    }
  }

  @Override
  public void mousePressed(MouseEvent me) {
    isValidPath = isValidPath(me);
    if (isValidPath && me.isPopupTrigger()) {
      showPopupMenu();
    }
  }

  /**
   * Trata o duplo clique do mouse.
   */
  public void doubleClickAction() {
    if (file.isDirectory()) {
      return;
    }
    CommonFileOpenAction openAction = new CommonFileOpenAction(projectTree);
    openAction.openFile(file);
  }

  /**
   * Verifica se  um caminho vlido.
   * 
   * @param me evento do mouse
   * @return true se clicou em um n da rvore
   */
  private boolean isValidPath(MouseEvent me) {
    tree = (JTree)me.getSource();
    X = me.getX();
    Y = me.getY();
    path = tree.getPathForLocation(X, Y);
    if (path == null) {
      return false;
    }
    ProjectTreeNode lastPathComponent =
      (ProjectTreeNode)path.getLastPathComponent();
    file = lastPathComponent.getClientProjectFile();
    return true;
  }

  /**
   * Exibe o menu popup
   */
  private void showPopupMenu() {
    if (tree.getSelectionCount() > 1) {
      if (projectTree.getPopupMultSelection() != null) {
        notifyPopupMultSelectionWasTriggered();
        projectTree.getPopupMultSelection().show(tree, X, Y);
      }
    }
    else {
      tree.setSelectionPath(path);
      if (file.isDirectory()) {
        if ((file.getParent() == null) && (projectTree.getPopupRoot() != null)) {
          notifyPopupRootWasTriggered();
          projectTree.getPopupRoot().show(tree, X, Y);
        }
        else {
          if (projectTree.getPopupDir() != null) {
            notifyPopupDirWasTriggered(file);
            projectTree.getPopupDir().show(tree, X, Y);
          }
        }
        return;
      }
      if (projectTree.getPopupFile() != null) {
        notifyPopupFileWasTriggered(file);
        projectTree.getPopupFile().show(tree, X, Y);
      }
    }
  }

  /**
   * Notifica os listeners sobre a exibio do menu de popup de seleo
   * mltipla.
   */
  private void notifyPopupMultSelectionWasTriggered() {
    ProjectTreeListener listener;
    for (
      Enumeration elem = projectTree.getListeners().elements();
      elem.hasMoreElements();) {
      listener = (ProjectTreeListener)elem.nextElement();
      listener.popupMultSelectionTriggered();
    }
  }

  /**
   * Notifica os listeners sobre a exibio do menu de popup da raiz do
   * projeto.
   */
  private void notifyPopupRootWasTriggered() {
    ProjectTreeListener listener;
    for (
      Enumeration elem = projectTree.getListeners().elements();
      elem.hasMoreElements();) {
      listener = (ProjectTreeListener)elem.nextElement();
      listener.popupRootTriggered();
    }
  }

  /**
   * Notifica os listeners sobre a exibio do menu de popup de diretrio.
   *
   * @param dir O diretrio sobre o qual o menu de popup foi chamado.
   */
  private void notifyPopupDirWasTriggered(ProjectFile dir) {
    ProjectTreeListener listener;
    for (
      Enumeration elem = projectTree.getListeners().elements();
      elem.hasMoreElements();) {
      listener = (ProjectTreeListener)elem.nextElement();
      listener.popupDirTriggered(dir);
    }
  }

  /**
   * Notifica os listeners sobre a exibio do menu de popup de arquivo.
   *
   * @param file O arquivo sobre o qual o menu de popup foi chamado.
   */
  private void notifyPopupFileWasTriggered(ProjectFile file) {
    ProjectTreeListener listener;
    for (
      Enumeration elem = projectTree.getListeners().elements();
      elem.hasMoreElements();) {
      listener = (ProjectTreeListener)elem.nextElement();
      listener.popupFileTriggered(file);
    }
  }

  /**
   * Construtor
   *
   * @param projectTree .
   */
  public ProjectTreeMouseAdapter(ProjectTree projectTree) {
    super();
    this.projectTree = projectTree;
  }
}
