/*
 * $Id: CommonProjectAction.java 83891 2008-10-06 22:41:03Z clinio $
 */

package csbase.client.project.action;

import java.awt.Window;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JFileChooser;

import csbase.client.project.ProjectFileContainer;
import csbase.client.util.SingletonFileChooser;
import csbase.logic.CommonClientProject;
import csbase.logic.ProjectFile;

/**
 * Ao sobre projetos.
 * 
 * @author Tecgraf/PUC-Rio
 */
public abstract class CommonProjectAction extends AbstractAction {

  /**
   * O container.
   */
  private ProjectFileContainer container;

  /**
   * Conulta o arquivo selecionado.
   * 
   * @return o arquivo
   */
  protected ProjectFile getSelectedFile() {
    return container.getSelectedFile();
  }

  /**
   * Consulta os arquivos selecionados.
   * 
   * @return os arquivos
   */
  protected ProjectFile[] getSelectedFiles() {
    return container.getSelectedFiles();
  }

  /**
   * Consulta o nmero de arquivos selecionados.
   * 
   * @return o nmero
   */
  protected int getSelectionCount() {
    return container.getSelectionCount();
  }

  /**
   * Consulta a window do container.
   * 
   * @return a window
   */
  protected Window getWindow() {
    return container.getWindow();
  }

  /**
   * Consulta o ttulo do container.
   * 
   * @return o ttulo
   */
  protected String getTitle() {
    return container.getTitle();
  }

  /**
   * Consulta o projeto corrente.
   * 
   * @return o projeto
   */
  protected CommonClientProject getProject() {
    return container.getProject();
  }

  /**
   * Reseta a seleo do container
   */
  protected void clearSelection() {
    container.clearSelection();
  }

  /**
   * ativa a limpeza do clipboard.
   */
  protected void clearClipboard() {
    container.clearClipboard();
  }

  /**
   * Inicia a ao de rename.
   */
  protected void startRenamingAction() {
    container.startRenamingAction();
  }

  /**
   * Inicia a ao de cut..
   */
  protected void startCutAction() {
    container.startCutAction();
  }

  /**
   * Inicia a ao de copy.
   */
  protected void startCopyAction() {
    container.startCopyAction();
  }

  /**
   * Inicia a ao de paste
   */
  protected void startPasteAction() {
    container.startPasteAction();
  }

  /**
   * Retorna uma instncia nica (singleton) de um dilogo de seleo de
   * arquivos locais.
   * 
   * @return dilogo de seleo de arquivos locais.
   */
  protected JFileChooser getFileChooser() {
    return SingletonFileChooser.getInstance();
  }

  /**
   * Construtor
   * 
   * @param container
   */
  public CommonProjectAction(ProjectFileContainer container) {
    this.container = container;
    putValue(Action.NAME, getName());
  }

  /**
   * {@inheritDoc}
   */
  public abstract void actionPerformed(ActionEvent e);

  /**
   * Consulta o nome da ao.
   * 
   * @return o nome
   */
  public abstract String getName();
}
