/*
 * ProjectCreationAction.java
 * 
 * $Author: jnlopes $ $Revision: 96246 $ - $Date: 2008-09-17 10:05:53 -0300 (Wed,
 * 17 Sep 2008) $
 */
package csbase.client.project.action;

import java.awt.Window;
import java.awt.event.ActionEvent;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.project.CommonProjectInfoDialog;
import csbase.client.project.ProjectCreationWithAllocationDialog;
import csbase.client.project.ProjectTree;
import csbase.client.project.tasks.CheckAreaReservedTask;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.CommonClientProject;

/**
 * Ao que cria um projeto de um usurio.
 */
public class CommonProjectCreationAction extends ProjectAction {

  /**
   * Indica se habilita a criao de projeto com alocao.
   */

  private boolean allocation = false;

  /**
   * Prefixo para internacionalizao.
   */
  final private static String prefix = "CommonProjectCreationAction.";

  /**
   * Executa a ao.No csbase, criamos um projeto apenas com as informacoes
   * comuns.
   * 
   * @param e .
   */
  @Override
  public void actionPerformed(ActionEvent e) {
    final CommonClientProject newProject;
    final Window window = getWindow();
    if (allocation) {
      newProject = ProjectCreationWithAllocationDialog.createProject(window);
    }
    else {
      newProject = CommonProjectInfoDialog.createProject(window);
    }
    if (newProject == null) {
      return;
    }
    /* Fecha o projeto corrente salvando as configuracoes */
    final ProjectCloseAction closeAction =
      new ProjectCloseAction(getProjectTree());
    closeAction.actionPerformed(e);
    try {
      setCurrentProject(newProject);
    }
    catch (Exception ex) {
      StandardErrorDialogs.showErrorDialog(window, getTitle(), ex);
    }
  }

  /**
   * Obtm o nome da ao.
   * 
   * @return .
   */
  @Override
  public String getName() {
    return LNG.get("PRJ_CREATION");
  }

  /**
   * 
   */
  private void checkAreaReserved() {

    final Window window = getWindow();
    final CheckAreaReservedTask task = new CheckAreaReservedTask();
    task.execute(window, LNG.get(prefix + "area.reserved.title"), LNG
      .get(prefix + "area.reserved.msg"));

    if (task.wasCancelled()) {
      final String title = LNG.get(prefix + "area.reserved.cancelled.title");
      final String msg = LNG.get(prefix + "area.reserved.cancelled.msg");
      StandardDialogs.showErrorDialog(window, title, msg);
      return;
    }

    if (task.getStatus() != true) {
      final Exception exception = task.getError();
      final String title = LNG.get(prefix + "area.reserved.exception.title");
      final String msg =
        LNG.get(prefix + "area.reserved.exception.msg") + "\n"
          + exception.getMessage();
      StandardDialogs.showErrorDialog(window, title, msg);
      return;
    }

    this.allocation = task.getResult();
  }

  /**
   * Constri a ao.
   * 
   * @param treeView .
   */
  public CommonProjectCreationAction(ProjectTree treeView) {
    super(treeView);
    checkAreaReserved();
  }

  /**
   * Constri a ao.
   * 
   * @param treeView .
   * @param allocation indica se habilita a criao com alocao
   * 
   * @deprecated - A definio de utilizao de reserva de rea agora  feita
   *             diretamente atravs de propriedade booleana definida em
   *             ProjectService.properties. Utilize o outro construtor.
   */
  @Deprecated
  public CommonProjectCreationAction(ProjectTree treeView, boolean allocation) {
    this(treeView);
  }
}
