/*
 * FileTypeChangeDialog.java
 *
 * $Author: taciana $
 * $Revision: 68848 $ - $Date: 2007-10-10 15:45:37 -0300 (Wed, 10 Oct 2007) $
 */
package csbase.client.project.action;

import csbase.client.desktop.*;
import csbase.client.project.ClientProjectFileProperties;
import csbase.client.util.ClientUtilities;
import csbase.logic.ClientProjectFile;
import csbase.logic.ProjectFile;
import csbase.logic.ProjectFileType;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.text.MessageFormat;

import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GUIUtils;

/**
 * A classe <code>FileTypeChangeDialog</code> modela uma janela que  permite ao
 * usurio modificar o tipo de um <code>ProjectFile</code>.
 */
public class FileTypeChangeDialog {

  /** Dilogo principal. */
  private DesktopComponentDialog dialog = null;

  /** Janela a partir do qual esse dilogo foi chamado. */
  private Window owner = null;

  /** O boto default que fecha o dilogo. */
  private JButton defaultButton;

  /** O arquivo cujo tipo vai ser modificado. */
  private ClientProjectFile file;

  /** Propriedades do arquivo: */
  private String fileName; // Nome do arquivo
  private Object[] filePath; // Caminho do arquivo
//  private String fileType; // Tipo do arquivo

  /** Campo de tipos de arquivo. */
  private JComboBox fileTypeList;

  /**
   * Mostra a janela que permite ao usurio modificar o tipo do arquivo.
   *
   * @param owner A janela a partir do qual esse dilogo foi chamado.
   * @param file O arquivo cujo tipo vai ser alterado.
   */
  public static void show(Window owner, ProjectFile file) {
    new FileTypeChangeDialog(owner, file);
  }

  /**
   * Cria o dilogo.
   *
   * @return .
   */
  private DesktopComponentDialog makeDialog() {
    /* Cria um dilogo no modal */
    DesktopComponentDialog dialog;
    if (owner instanceof JDialog) {
      dialog = new DesktopComponentDialog((JDialog)owner, getTitle());
    }
    else {
      dialog = new DesktopComponentDialog((JFrame)owner, getTitle());
    }
    dialog.setModal(false);

    /* Cria o painel principal do dilogo */
    JPanel mainPanel = new JPanel(new BorderLayout());
    mainPanel.add(makePropertiesPanel(dialog.getBackground()));
    mainPanel.add(makeButtonPanel(), BorderLayout.SOUTH);
    dialog.getContentPane().add(mainPanel);
    dialog.pack();
    dialog.center(owner);
    dialog.getRootPane().setDefaultButton(defaultButton);
    return dialog;
  }

  /**
   * Cria o painel com as propriedades do arquivo.
   *
   * @param background A cor de fundo usada pelos componentes.
   *
   * @return .
   */
  private JPanel makePropertiesPanel(Color background) {
    ImageIcon icon = ClientProjectFileProperties.getImageIcon(this.file);
    // #TODO As variveis source e accessMode nunca so lidas. Os mtodos
    // associados a elas necessitam ser executados?
    String source = ClientProjectFileProperties.getSourceDescription(this.file);
    String accessMode = ClientProjectFileProperties.getAccessMode(this.file);
    String typeName;
    ProjectFileType type = null;
    try {
      type = ProjectFileType.getFileType(file.getType());
      typeName = type.getDescription();
    }
    catch (Exception e) {
      typeName = " ";
    }

    /* Campo com o nome do arquivo */
    JTextField nameTextField = new JTextField(fileName);
    nameTextField.setEditable(false);

    /* Campo com a localizao */
    String fileLocalization = buildLocalization(filePath);
    JTextField pathTextField = new JTextField(fileLocalization);
    pathTextField.setEditable(false);

    /* Tipo atual do arquivo */
    JLabel typeLabel = new JLabel(icon, JLabel.LEFT);
    JTextArea typeTextArea = new JTextArea(" "+typeName);
    typeTextArea.setBackground(background);
    typeTextArea.setEditable(false);
    JPanel typePanel = new JPanel(new FlowLayout(FlowLayout.LEADING, 0, 0));
    typePanel.add(typeLabel);
    typePanel.add(typeTextArea);

    /* Tipos de arquivos */
    ProjectFileType[] fTypes = ProjectFileType.getAllFileTypes();
    fileTypeList = new JComboBox(new DefaultComboBoxModel(fTypes));
    if (type != null) {
      fileTypeList.setSelectedItem(type);
    }

    /* Monta o panel usando grid layout */
    JComponent[][] rows =
      {
        { new JLabel(LNG.get("PRJ_FILE_NAME")), nameTextField },
        { new JLabel(LNG.get("PRJ_FILE_LOCALIZATION")), pathTextField },
        { new JLabel(LNG.get("PRJ_FILE_TYPE")), typePanel },
        { new JLabel(LNG.get("PRJ_FILE_NEW_TYPE")), fileTypeList },
      };
    return GUIUtils.createBasicGridPanel(rows);
  }

  /**
   * Cria o painel com os botes.
   *
   * @return .
   */
  private JPanel makeButtonPanel() {
    JPanel panel = new JPanel();
    defaultButton = new JButton(LNG.get("PRJ_FILE_TYPE_CHANGE_CONFIRMATION"));
    panel.add(defaultButton);
    defaultButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          changeType();
        }
      });

    JButton cancelButton = new JButton(LNG.get("PRJ_CANCEL"));
    panel.add(cancelButton);
    cancelButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          dialog.close();
        }
      });

    JButton[] allButtons = { defaultButton, cancelButton };
    ClientUtilities.adjustEqualSizes(allButtons);
    return panel;
  }

  /**
   * Informa o ttulo que vai ser utilizado na janela de exibio.
   *
   * @return O ttulo da janela.
   */
  protected String getTitle() {
    return LNG.get("PRJ_PROJECT_FILE_CHANGE_TYPE_TITLE");
  }

  /**
   * Recupera as propriedades do arquivo.
   */
  private void retrieveFileProperties() {
    RemoteTask<Void> task = new RemoteTask<Void>() {
        @Override
        public void performTask()
        throws Exception {
          fileName = file.getName();
          filePath = file.getPath();
        }
      };
    task.execute(
      owner, getTitle(), LNG.get("PRJ_WAITING_OBTAIN_FILE_PROPERTIES"));
  }

  /**
   * Consulta aos dados digitados pelo usurio.
   *
   * @return Novo tipo de arquivo.
   */
  private String getNewType() {
    ProjectFileType selectedType =
      (ProjectFileType)fileTypeList.getSelectedItem();
    return selectedType.getCode();
  }

  /**
   * Muda o tipo do arquivo.
   */
  private void changeType() {
    final String newType = getNewType();
    if (newType.equals(file.getType())) {
      dialog.close();
      return;
    }
    RemoteTask<Void> task =
      new RemoteTask<Void>() {
        @Override
        public void performTask() throws Exception {
          file.changeType(newType);
        }
      };
    if (task.execute(
      owner, getTitle(), MessageFormat.format(
        LNG.get("PRJ_WAITING_CHANGE_FILE_TYPE"),
        new Object[] {file.getName()}))){
      dialog.close();
    }
  }

  /**
   * Constri a representao de onde se encontra um determinado  arquivo. A
   * String resultante possui a seguinte forma: [nome-do-projeto]:
   * caminho-dos-diretrios
   *
   * @param filePath .
   *
   * @return A string que representa a localizao de um arquivo na rvore do
   *         projeto.
   */
  private String buildLocalization(Object[] filePath) {
    String path = "["+filePath[0]+"]"+": ";
    for (int i = 1; i < (filePath.length-1); i++) {
      path += (filePath[i]+"/");
    }
    return path;
  }

  /**
   * Constri o dilogo.
   *
   * @param owner A janela pai a partir da qual esse dilogo  chamado.
   * @param file O arquivo cujo tipo vai ser alterado.
   */
  private FileTypeChangeDialog(Window owner, ProjectFile file) {
    this.owner = owner;
    this.file = (ClientProjectFile)file;
    retrieveFileProperties();
    dialog = makeDialog();
    dialog.setVisible(true);
  }
}
