/**
 * ProjectAction.java
 * 
 * $Id: ProjectAction.java 65709 2007-07-24 18:07:12Z costa $
 */
package csbase.client.project.action;

import csbase.client.desktop.DesktopFrame;
import csbase.client.project.ProjectTree;
import csbase.client.project.ProjectTreeAdapter;
import csbase.client.remote.ProjectObserver;
import csbase.logic.CommonClientProject;
import csbase.logic.ProjectFile;

import java.awt.Window;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;

/**
 * A classe <code>ProjectAction</code> modela as aes que um usurio pode
 * efetuar sobre um <code>Project</code> ou sobre um <code>ProjectFile</code>.
 */
public abstract class ProjectAction extends AbstractAction implements
  ClipboardOwner {
  /** A instncia da rvore de projetos associada a essa ao. */
  private ProjectTree treeView;

  /**
   * Obtm a janela sobre a qual essa ao atua.
   * 
   * @return .
   */
  public Window getWindow() {
    return treeView.getWindow();
  }

  /**
   * Obtm o ttulo da janela sobre a qual essa ao atua.
   * 
   * @return .
   */
  public String getTitle() {
    return treeView.getTitle();
  }

  /**
   * Obtm a rvore de projeto sobre a qual essa ao atua.
   * 
   * @return .
   */
  public ProjectTree getProjectTree() {
    return treeView;
  }

  /**
   * Obtm o projeto sobre o qual essa ao atua.
   * 
   * @return .
   */
  public CommonClientProject getCurrentProject() {
    return treeView.getProject();
  }

  /**
   * Altera o projeto sobre o qual essa ao atua.
   * 
   * @param project .
   * 
   * @throws Exception
   */
  public void setCurrentProject(CommonClientProject project) throws Exception {
    if (project == null) {
      return;
    }
    CommonClientProject currentProject = getCurrentProject();
    if (currentProject != null) {
      currentProject.uninstallObserver();
    }
    project.installObserver(new ProjectObserver(project));
    treeView.setProject((CommonClientProject) project);
  }

  /**
   * Fecha o projeto sobre o qual essa ao atua.
   */
  public void closeProject() {
    treeView.closeProject();
    try {
      getCurrentProject().uninstallObserver();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Obtm o arquivo sobre o qual essa ao atua.
   * 
   * @return .
   */
  public ProjectFile getSelectedFile() {
    return treeView.getSelectedFile();
  }

  /**
   * Obtm os arquivos sobre os quais essa ao atua.
   * 
   * @return .
   */
  public ProjectFile[] getSelectedFiles() {
    return treeView.getSelectedFiles();
  }

  /**
   * Limpa a rea de clipboard.
   */
  public void clearClipboard() {
    treeView.clearClipboard();
  }

  public void lostOwnership(Clipboard clipboard, Transferable contents) {
  }

  /**
   * Habilita a ao se houver um projeto aberto na rvore.
   * 
   * @param enable .
   */
  public void enableIfProjectOpened(boolean enable) {
    treeView.addProjectTreeListener(new ProjectTreeAdapter() {
      @Override
      public void projectChanged(CommonClientProject project) {
        setEnabled(project != null);
      }
    });
  }

  /**
   * Habilita a selecao de projetos depois que remove algum.
   * 
   * @param enable .
   */
  public void enableIfProject(boolean enable) {
    treeView.addProjectTreeListener(new ProjectTreeAdapter() {
      @Override
      public void projectRemoved() {
        setEnabled(DesktopFrame.getInstance().userHasProject());
      }
    });
  }

  /**
   * Executa essa ao.
   * 
   * @param event .
   */
  public abstract void actionPerformed(ActionEvent event);

  /**
   * Obtm o nome da ao.
   * 
   * @return .
   */
  public abstract String getName();

  /**
   * Constri uma ao.
   * 
   * @param treeView .
   */
  public ProjectAction(ProjectTree treeView) {
    this.treeView = treeView;
    putValue(Action.NAME, getName());
  }
}
