/*
 * $Id: ProjectCloseAction.java 93285 2009-07-03 20:02:55Z clinio $
 */
package csbase.client.project.action;

import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import tecgraf.javautils.LNG;
import csbase.client.applicationmanager.ApplicationManager;
import csbase.client.applicationmanager.ApplicationType;
import csbase.client.desktop.RemoteTask;
import csbase.client.project.ProjectTree;
import csbase.exception.PermissionException;
import csbase.logic.CommonClientProject;
import csbase.logic.User;
import csbase.logic.applicationservice.ApplicationRegistry;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectServiceInterface;

/**
 * Ao que atualiza a rvore de projeto.
 */
public class ProjectCloseAction extends ProjectAction implements WindowListener {

  /**
   * {@inheritDoc}
   * 
   * @see csbase.client.project.action.ProjectAction#actionPerformed(java.awt.event.ActionEvent)
   */
  @Override
  public void actionPerformed(ActionEvent e) {
    close();
  }

  /**
   * Executa a ao relativa a janela
   * 
   * @param e .
   */
  public void windowClosing(WindowEvent e) {
    close();
  }

  /**
   * Tenta salvar as modificaes no projeto?
   * 
   * @return false Caso tenha ocorrido algum erro ao fechar o projeto.
   */
  public boolean close() {
    final CommonClientProject project = getCurrentProject();
    if (project == null) {
      return true;
    }
    /*
     * Tenta fechar cada uma das aplicaes em execuo. Caso alguma no
     * termine, cancela o fechamento do projeto.
     */
    ApplicationManager appMgr = ApplicationManager.getInstance();
    Hashtable runningApps = appMgr.getRunningApplications();
    Enumeration appKeys = runningApps.keys();
    String key = null;
    while (appKeys.hasMoreElements()) {
      key = (String) appKeys.nextElement();
      Vector originalVector = (Vector) runningApps.get(key);

      /*
       *  medida que as aplicaes vo sendo fechadas, o vetor vai diminuindo;
       * para evitar problemas no loop, trabalha com um clone.
       */
      Vector copyVector = (Vector) originalVector.clone();
      if (copyVector != null) {
        for (int i = 0; i < copyVector.size(); i++) {
          ApplicationType app = (ApplicationType) copyVector.get(i);
          ApplicationRegistry appReg =
            appMgr.getApplicationRegistry(app.getId());
          if (appReg.requireProject()) {
            if (!app.closeApplication()) {
              return false;
            }
            else {
            }
          }
          else {
          }
        }
      }
    }
    /* Notifica observadores do fechamento do projeto */
    closeProject();

    /*
     * se o usurio tem permisso de escrita no projeto, grava modificaes
     */
    Object ownerID = User.getLoggedUser().getId();
    CommonClientProject currentProject = getCurrentProject();
    if (currentProject.userHasAccesRW(ownerID)) {
      if (!saveProjectModifications()) {
        return false;
      }
    }
    /*
     * agora fecha efetivamente o projeto
     */
    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      protected void performTask() throws Exception {
        project.close(true);
      }
    };
    if (!task.execute(getWindow(), LNG.get("PRJ_PROJECT_CLOSE_TITLE"), LNG
      .get("PRJ_WAITING_CLOSE_PROJECT"))) {
      return false;
    }
    this.getProjectTree().resetProject();
    return true;
  }

  /**
   * Persiste eventuais modificaes nas propriedades do projeto.
   * 
   * @return true se a gravao foi bem-sucedida
   */
  private boolean saveProjectModifications() {
    RemoteTask<Void> saveProjectTask = new RemoteTask<Void>() {
      @Override
      public void performTask() throws Exception {
        try {
          getCurrentProject().modify();
        }
        finally {
          ClientRemoteLocator.server.setServerSessionProperty(
            ProjectServiceInterface.CURRENT_PROJECT_SESSION_PROPERTY, null);
        }
      }
    };
    if (!saveProjectTask.execute(getWindow(), LNG
      .get("PRJ_PROJECT_CLOSE_TITLE"), LNG.get("PRJ_WAITING_CLOSE_PROJECT"))) {
      Exception e = saveProjectTask.getError();
      // Isto ser removido quando a pendncia [CSBASE-351] for resolvida
      if (!(e instanceof PermissionException)) {
        return false;
      }
    }
    return true;
  }

  /**
   * Obtm o nome da ao.
   * 
   * @return .
   */
  @Override
  public String getName() {
    return LNG.get("PRJ_CLOSE");
  }

  /**
   * {@inheritDoc}
   * 
   * @see java.awt.event.WindowListener#windowActivated(java.awt.event.WindowEvent)
   */
  public void windowActivated(WindowEvent e) {
  }

  /**
   * {@inheritDoc}
   * 
   * @see java.awt.event.WindowListener#windowClosed(java.awt.event.WindowEvent)
   */
  public void windowClosed(WindowEvent e) {
  }

  /**
   * {@inheritDoc}
   * 
   * @see java.awt.event.WindowListener#windowDeactivated(java.awt.event.WindowEvent)
   */
  public void windowDeactivated(WindowEvent e) {
  }

  /**
   * {@inheritDoc}
   * 
   * @see java.awt.event.WindowListener#windowDeiconified(java.awt.event.WindowEvent)
   */
  public void windowDeiconified(WindowEvent e) {
  }

  /**
   * {@inheritDoc}
   * 
   * @see java.awt.event.WindowListener#windowIconified(java.awt.event.WindowEvent)
   */
  public void windowIconified(WindowEvent e) {
  }

  /**
   * {@inheritDoc}
   * 
   * @see java.awt.event.WindowListener#windowOpened(java.awt.event.WindowEvent)
   */
  public void windowOpened(WindowEvent e) {
  }

  /**
   * Constri a ao.
   * 
   * @param treeView .
   */
  public ProjectCloseAction(ProjectTree treeView) {
    super(treeView);
  }
}
