/*
 * $Id: OpenSharedProjectDialog.java 84381 2008-10-16 18:13:14Z clinio $
 */
package csbase.client.project.dialogs;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.rmi.RemoteException;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import tecgraf.javautils.LNG;
import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.GUIUtils;
import tecgraf.javautils.gui.StandardDialogs;
import tecgraf.javautils.gui.table.ObjectTableModel;
import tecgraf.javautils.gui.table.ObjectTableProvider;
import tecgraf.javautils.gui.table.SortableTable;
import csbase.client.desktop.DesktopComponentDialog;
import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.RemoteTask;
import csbase.client.project.ProjectTree;
import csbase.client.project.tasks.OpenProjectOnTreeRemoteTask;
import csbase.client.remote.ProjectAdminObserver;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.CommonClientProject;
import csbase.logic.CommonProjectInfo;
import csbase.logic.ProjectUserEvent;
import csbase.logic.SharedProjectEvent;
import csbase.logic.User;
import csbase.logic.UserProjectInfo;

/**
 * A classe <code>UsersCommonProjectOpenDialog</code> mostra um dilogo modal
 * para seleo de projetos de outros usurios. Na seleo de um projeto, o
 * usurio pode acionar o boto de seleo, que confirma a operao, ou pode
 * acionar o boto de cancelamento, que desiste da operao. O boto de seleo
 * somente  habilitado quando o usurio tiver selecionado algum projeto. Aps a
 * seleo, o projeto  aberto e um observador  instalado para ser notificado
 * de qualquer alterao remota que ocorrer sobre as informaes desse projeto.
 *  observador de ProjectAdminObserver, para acrescentar ou remover uma linha
 * da tabela, caso o usurio seja adicionado ou removido de algum projeto.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class OpenSharedProjectDialog implements Observer {

  /**
   * O identificador do usurio
   */
  protected Object userId;

  /**
   * A janela que deu origem a esse dialogo
   */
  protected Window owner;

  /**
   * A lista com os nomes de projetos dos demais usurios
   */
  protected SortableTable table;

  /**
   * O modelo da tabela de usurios
   */
  protected ObjectTableModel<UserProjectInfo> model;

  /**
   * A janela de seleo de projeto.
   */
  protected DesktopComponentDialog dialog;

  /**
   * O boto default da janela
   */
  protected JButton defaultButton;

  /**
   * Vetor com os projetos compartilhados
   */
  protected Vector<UserProjectInfo> projects;

  /**
   * O projeto selecionado
   */
  protected CommonClientProject project;

  /**
   * Mostra um dilogo com a lista de nomes dos projetos de outros usurio e
   * aguarda que um deles seja selecionado. O dilogo  modal. No caso da
   * confimao da seleo, o projeto  aberto.
   * 
   * @param owner A janela a partir da qual esse dilogo foi chamado.
   * @return O projeto selecionado.
   */
  public static CommonClientProject openProject(Window owner) {
    try {
      return new OpenSharedProjectDialog(owner).project;
    }
    catch (Exception e) {
      StandardErrorDialogs.showErrorDialog(owner, LNG
        .get("OpenSharedProjectDialog.title"), LNG
        .get("OpenSharedProjectDialog.error.no.project"));
    }
    return null;
  }

  /**
   * Cria um dilogo que aguarda a seleo de um projeto. Solicita ao servidor
   * que o projeto selecionado seja aberto.
   * 
   * @return .
   */
  private DesktopComponentDialog makeDialog() {
    // Cria o painel principal da janela
    JPanel mainPanel = new JPanel(new BorderLayout());
    mainPanel.add(makeProjectListPanel(), BorderLayout.CENTER);
    mainPanel.add(makeButtonPanel(), BorderLayout.SOUTH);

    // Cria um dilogo modal
    final DesktopComponentDialog dialog;
    if (owner instanceof JFrame) {
      dialog = new DesktopComponentDialog((JFrame) owner, getTitle());
    }
    else if (owner instanceof JDialog) {
      dialog = new DesktopComponentDialog((JDialog) owner, getTitle());
    }
    else {
      return null;
    }
    dialog.addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(WindowEvent e) {
        dialog.close();
      }
    });

    dialog.getContentPane().add(mainPanel);
    dialog.pack();
    dialog.center(owner);
    dialog.getRootPane().setDefaultButton(defaultButton);
    return dialog;
  }

  /**
   * Obtem a lista de projetos de um usurio.
   * 
   * @return .
   */
  private Vector<UserProjectInfo> getProjects() {
    RemoteTask<Vector<UserProjectInfo>> task =
      new RemoteTask<Vector<UserProjectInfo>>() {
        @Override
        public void performTask() throws Exception {
          setResult(getAllUserProjects(userId));
        }
      };
    if (task.execute(owner, getTitle(), LNG
      .get("OpenSharedProjectDialog.info.get.projects"))) {
      return task.getResult();
    }
    return null;
  }

  /**
   * Cria um painel com a lista dos projetos apresentados no dilogo. O contedo
   * da lista  requisitado ao servidor.
   * 
   * @return .
   */
  private JPanel makeProjectListPanel() {
    JPanel panel = new JPanel(new GridBagLayout());

    /* Prepara a tabela */
    ObjectTableProvider columnFormatter = getColumnFormatter();
    model = new ObjectTableModel<UserProjectInfo>(projects, columnFormatter);
    table = new SortableTable(model);
    table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    table.setRowSelectionInterval(0, 0);
    table.adjustColumnWidth();
    adjustTableSize();

    // Associa o duplo clique a ao principal do dilogo
    MouseListener mouseListener = new MouseAdapter() {
      @Override
      public void mouseClicked(MouseEvent e) {
        if (e.getClickCount() == 2) {
          handleAction();
        }
      }
    };
    table.addMouseListener(mouseListener);

    final JScrollPane scrollPane = new JScrollPane(table);
    panel.add(scrollPane, new GBC(0, 0).both());
    return panel;
  }

  /**
   * Ajusta o tamanho inicial da tabela com base na configurao da classe.
   */
  private void adjustTableSize() {
    int width;
    int height;
    final int defaultWidth = 500;
    final int defaultHeight = 150;
    try {
      final ConfigurationManager mng = ConfigurationManager.getInstance();
      final Configuration cnf = mng.getConfiguration(this.getClass());
      width = cnf.getOptionalIntegerProperty("table.width", defaultWidth);
      height = cnf.getOptionalIntegerProperty("table.height", defaultHeight);
    }
    catch (Exception e) {
      width = defaultWidth;
      height = defaultHeight;
    }
    final Dimension dim = new Dimension(width, height);
    table.setPreferredScrollableViewportSize(dim);
  }

  /**
   * Devolve o formatador dos elementos disponveis para seleo.
   * 
   * @return .
   */
  private ObjectTableProvider getColumnFormatter() {
    return new ObjectTableProvider() {
      public String[] getColumnNames() {
        return new String[] { LNG.get("OpenSharedProjectDialog.users"),
            LNG.get("OpenSharedProjectDialog.projects") };
      }

      public Class<?>[] getColumnClasses() {
        return new Class[] { String.class, String.class };
      }

      public Object[] getCellValues(Object item) {
        UserProjectInfo userProj = (UserProjectInfo) item;
        if (userProj == null)
          return null;
        String userName = "";
        try {
          userName = userProj.getUserName();
        }
        catch (Exception e) {
        }
        String[] format = { userName, userProj.getProjectName() };
        return format;
      }
    };
  }

  /**
   * Seleciona os projetos compartilhados dos usurios.
   * 
   * @param userId o identificador do usurio
   * @return os projetos compartilhados do usurio
   * @throws RemoteException falha de rmi
   */
  protected Vector<UserProjectInfo> getAllUserProjects(Object userId)
    throws RemoteException {
    return CommonClientProject.getAllUserProjects(userId);
  }

  /**
   * Cria um painel com os botes que vo ser usados no dilogo requisitado.
   * Esse mtodo deve ser sobrescrito dependendo acao da selecao.
   * 
   * @return O painel recm-criado.
   */
  protected JPanel makeButtonPanel() {
    JPanel panel = new JPanel();
    final String openText = LNG.get("OpenSharedProjectDialog.open");
    defaultButton = (JButton) panel.add(new JButton(openText));
    defaultButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        handleAction();
      }
    });

    final String cancelText = LNG.get("OpenSharedProjectDialog.cancel");
    final JButton cancelButton = (JButton) panel.add(new JButton(cancelText));
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        dialog.close();
      }
    });

    final JComponent[] components =
      new JComponent[] { defaultButton, cancelButton };
    GUIUtils.matchPreferredSizes(components);

    return panel;
  }

  /**
   * Executa a ao de abrir o projeto selecionado e instalar um observador para
   * ele.
   */
  private void handleAction() {
    int index = table.getSelectedRow();
    if (index == -1) {
      final String err = LNG.get("OpenSharedProjectDialog.error.no.selection");
      StandardDialogs.showErrorDialog(owner, getTitle(), err);
      return;
    }
    int modelIndex = table.convertRowIndexToModel(index);
    UserProjectInfo userProject = model.getRow(modelIndex);
    final Object userAdminId = userProject.getUserId();
    final String projName = userProject.getProjectName();
    final ProjectTree tree = DesktopFrame.getInstance().getTree();
    OpenProjectOnTreeRemoteTask task =
      new OpenProjectOnTreeRemoteTask(projName, userAdminId, tree);
    final String taskMsg = LNG.get("OpenSharedProjectDialog.info.open.project");
    task.execute(dialog, getTitle(), taskMsg);

    /* Se o resultado for null  porque a operao no pode ser efetuada. */
    project = task.getResult();
    if (project != null) {
      dialog.close();
    }
  }

  /**
   * Abre o projeto. Este metodo deve ser sobrescrito para abrir projeto
   * especificos do cada sistema.
   * 
   * @param userId .
   * @param projectName .
   * @return .
   * @throws RemoteException falha de rmi
   */
  protected CommonClientProject openProject(Object userId, String projectName)
    throws RemoteException {
    return CommonClientProject.openProject(userId, projectName, true);
  }

  /**
   * Informa o ttulo que vai ser utilizado na janela de dilogo. Esse mtodo
   * depende da ao que originou o dilogo.
   * 
   * @return O ttulo da janela.
   */
  protected String getTitle() {
    return LNG.get("OpenSharedProjectDialog.title");
  }

  /**
   * Constri uma viso de dilogo para seleo de projetos de um um usurio.
   * 
   * @param owner A janela a partir da qual esse dilogo foi chamado.
   */
  protected OpenSharedProjectDialog(Window owner) {
    this.userId = User.getLoggedUser().getId();
    this.owner = owner;
    this.projects = getProjects();
    // Se cadastrando como observador de projetos em ProjectAdminObserver
    ProjectAdminObserver.getInstance().addObserver(this);

    dialog = makeDialog();
    dialog.setVisible(true);
  }

  /**
   * Verifica se o usurio foi adicionado ou removido de algum projeto, ou se
   * algum projeto da qual ele participava foi removido; Insere ou remove linha
   * da tabela, conforme o caso. Se todas as linhas forem removidas, desabilita
   * a boto Confirmar.
   * 
   * @param o observvel
   * @param arg argumento
   */
  public void update(Observable o, Object arg) {
    if (arg instanceof SharedProjectEvent) {
      SharedProjectEvent event = (SharedProjectEvent) arg;
      CommonProjectInfo commonInfo = event.getProjectInfo();
      UserProjectInfo projectRow =
        new UserProjectInfo(commonInfo.userId, commonInfo.name);
      if (event.contains(User.getLoggedUser().getId())) {
        model.add(projectRow);
      }
      else {
        model.remove(projectRow);
      }
    }
    else if (arg instanceof ProjectUserEvent) {
      ProjectUserEvent event = (ProjectUserEvent) arg;
      if (event.type == ProjectUserEvent.DELETE) {
        // Pegando informaes do projeto que vem no event
        CommonProjectInfo commonInfo = (CommonProjectInfo) event.item;
        // Gerando um UserProjectInfo (uma linha de table)
        UserProjectInfo projectRow =
          new UserProjectInfo(commonInfo.userId, commonInfo.name);
        model.remove(projectRow);
      }
    }
    if (table.getRowCount() == 0) {
      defaultButton.setEnabled(false);
    }
  }

}
