/*
 * Detalhes da ltima alterao:
 * 
 * $Author: rodrigoh $ $Date$ $Revision: 86684 $
 */
package csbase.client.project.dialogs;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.rmi.RemoteException;
import java.text.MessageFormat;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.GUIUtils;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.desktop.DesktopComponentDialog;
import csbase.client.desktop.RemoteTask;
import csbase.logic.ClientProjectFile;
import csbase.remote.ClientRemoteLocator;

/**
 * Representa um dilogo de agendamento de atualizaes de um arquivo.
 * 
 * @author Tecgraf/PUC-Rio
 */
public final class UpdateFileDialog extends DesktopComponentDialog {
  /**
   * Prefixo das chaves de traduo.
   */
  private static final String LNG_PREFIX = "UpdateFileDialog.";
  /**
   * O arquivo objeto da atualizao.
   */
  private ClientProjectFile file;
  /**
   * Campo texto utilizado para se obter o intervalo da atualizao.
   */
  private JTextField intervalField;
  /**
   * Boto responsvel por parar as atualizaes.
   */
  private JButton stopButton;

  /**
   * Cria o dilogo.
   * 
   * @param owner A janela dona do dilogo.
   * @param file O arquivo cuja atualizao ser agendada.
   */
  public UpdateFileDialog(JFrame owner, ClientProjectFile file) {
    super(owner, MessageFormat.format(LNG.get(LNG_PREFIX + "title"),
      new Object[] { file.getName() }));
    this.file = file;
    this.createGui();
  }

  /**
   * Cria a interface grfica.
   */
  private void createGui() {
    this.setModal(true);
    this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    this.addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(WindowEvent e) {
        close();
      }
    });

    JPanel mainPanel = new JPanel(new BorderLayout());
    mainPanel.add(makeDataPanel(), BorderLayout.CENTER);
    mainPanel.add(makeButtonPanel(), BorderLayout.SOUTH);

    this.getContentPane().add(mainPanel);
    this.pack();
    this.center(this.getOwner());
  }

  /**
   * Cria o painel de dados.
   * 
   * @return O painel de dados.
   */
  private JPanel makeDataPanel() {
    JPanel panel = new JPanel();
    panel.setLayout(new GridBagLayout());

    GBC gbc =
      new GBC(1, 1, GridBagConstraints.WEST, GridBagConstraints.NONE,
        new Insets(12, 12, 6, 3));
    panel.add(new JLabel(LNG.get(LNG_PREFIX + "interval")), gbc);

    int interval = this.file.getUpdateInterval();
    String intervalText = "";
    if (interval > 0) {
      intervalText = String.valueOf(interval);
    }
    this.intervalField = new JTextField(intervalText, 5);
    gbc =
      new GBC(2, 1, GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
        new Insets(12, 3, 6, 11));
    gbc.gridwidth = 2;
    gbc.weightx = 1;
    panel.add(this.intervalField, gbc);

    return panel;
  }

  /**
   * Cria o painel de botes.
   * 
   * @return O painel de botes.
   */
  private JPanel makeButtonPanel() {
    JPanel panel = new JPanel();
    String startButtonTitle;
    if (this.file.getUpdateInterval() == 0) {
      startButtonTitle = LNG.get(LNG_PREFIX + "start");
    }
    else {
      startButtonTitle = LNG.get(LNG_PREFIX + "update");
    }
    JButton startButton = (JButton) panel.add(new JButton(startButtonTitle));
    startButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (handleStartAction()) {
          close();
        }
      }
    });

    this.stopButton =
      (JButton) panel.add(new JButton(LNG.get(LNG_PREFIX + "stop")));
    this.stopButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (handleStopAction()) {
          close();
        }
      }
    });

    JButton cancelButton =
      (JButton) panel.add(new JButton(LNG.get(LNG_PREFIX + "close")));
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        close();
      }
    });

    GUIUtils.matchPreferredSizes(new JComponent[] { startButton,
        this.stopButton, cancelButton });
    this.getRootPane().setDefaultButton(startButton);
    if (this.file.getUpdateInterval() == 0) {
      this.stopButton.setEnabled(false);
    }
    else {
      this.stopButton.setEnabled(true);
    }
    return panel;
  }

  /**
   * Executa a ao do boto iniciar.
   * 
   * @return {@code true} caso a aao tenha sido executada, ou {@code false},
   *         caso contrrio.
   */
  private boolean handleStartAction() {
    String text = this.intervalField.getText();
    int interval;
    try {
      interval = Integer.parseInt(text);
    }
    catch (NumberFormatException e) {
      interval = -1;
    }
    if (interval <= 0) {
      StandardDialogs.showErrorDialog(this, MessageFormat.format(LNG
        .get(LNG_PREFIX + "invalid.interval.title"), new Object[] { this
        .getTitle() }), LNG.get(LNG_PREFIX + "invalid.interval.message"));
      return false;
    }
    final int updateInterval = interval;
    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      protected void performTask() throws RemoteException {
        if (ClientRemoteLocator.projectService.startUpdate(file.getProjectId(),
          file.getPath(), updateInterval)) {
          file.setUpdateInterval(updateInterval);
        }
      }
    };
    if (task.execute(this.getOwner(), LNG.get(LNG_PREFIX + "task.title"),
      MessageFormat.format(LNG.get(LNG_PREFIX + "task.message"),
        new Object[] { this.file.getName() }))) {
      this.stopButton.setEnabled(true);
      return true;
    }
    return false;
  }

  /**
   * Executa a ao do boto parar.
   * 
   * @return {@code true} caso a aao tenha sido executada, ou {@code false},
   *         caso contrrio.
   */
  private boolean handleStopAction() {
    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      protected void performTask() throws RemoteException {
        if (ClientRemoteLocator.projectService.stopUpdate(file.getProjectId(),
          file.getPath())) {
          file.setUpdateInterval(0);
        }
      }
    };
    if (task.execute(this.getOwner(), LNG.get(LNG_PREFIX + "task.title"),
      MessageFormat.format(LNG.get(LNG_PREFIX + "task.message"),
        new Object[] { this.file.getName() }))) {
      this.stopButton.setEnabled(false);
      return true;
    }
    return false;
  }
}
