/*
 * $Id: UsersPermissionsTable.java 91455 2009-05-13 01:54:56Z clinio $
 */
package csbase.client.project.dialogs;

import java.util.List;
import java.util.Set;

import javax.swing.ListSelectionModel;

import tecgraf.javautils.gui.table.SortableTable;
import csbase.logic.UserOutline;

/**
 * Tabela de permissoes de usurios.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class UsersPermissionsTable extends SortableTable {
  /**
   * Preenche os conjuntos de usurios com acesso RO e com acesso RW definidos
   * na tabela. Os conjuntos so preenchidos independente do tipo de
   * compartilhamento (i.e. mesmo que a tabela esteja desabilitada).
   * 
   * @param usersRO - conjunto de usurios com acesso RO ao projeto, a ser
   *        preenchido pelo mtodo
   * @param usersRW - conjunto de usurios com acesso RW ao projeto, a ser
   *        preenchido pelo mtodo
   */
  public void fillUserSets(final Set<Object> usersRO, final Set<Object> usersRW) {
    final UsersPermissionsTableModel model =
      (UsersPermissionsTableModel) getModel();
    model.fillUserSets(usersRO, usersRW);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void setEnabled(final boolean enabled) {
    final int rowCount = getRowCount();
    if (!enabled && rowCount > 0) {
      // De-seleciona todas as linhas se a tabela no est habilitada
      removeRowSelectionInterval(0, rowCount - 1);
    }
    super.setEnabled(enabled);
  }

  /**
   * Construtor.
   * 
   * @param allUsers - todos os usurios do sistema
   * @param usersRO - todos os usurios que tm acesso RO ao projeto
   * @param usersRW - todos os usurios que tm acesso RW ao projeto
   */
  public UsersPermissionsTable(final List<UserOutline> allUsers,
    final Set<Object> usersRO, final Set<Object> usersRW) {
    setModel(new UsersPermissionsTableModel(allUsers, usersRO, usersRW));
    setRowSelectionAllowed(true);
    // TODO permitir seleo mltipla para operaes com o boto direito
    setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    setUseDisabledLook(true);
  }
}
