/*
 * $Id: GetChildrenTask.java 79318 2008-06-27 22:25:25Z clinio $
 */
package csbase.client.project.tasks;

import csbase.logic.ClientProjectFile;
import csbase.logic.ProjectFile;

/**
 * Classe que implementa a tarefa de consultar os filhos de um diretrio em um
 * projeto.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class GetChildrenTask extends CoreTask<ClientProjectFile[]> {

  /**
   * Arquivo consultado.
   */
  private ClientProjectFile clientProjectFile;

  /**
   * Construtor.
   * 
   * @param projectFile o arquivo consultado.
   */
  public GetChildrenTask(final ProjectFile projectFile) {
    this.clientProjectFile = (ClientProjectFile) projectFile;
  }

  /**
   * {@inheritDoc}
   * 
   * Faz a chamada remota a (lado cliente).
   * 
   * @see ClientProjectFile#getChildren()
   */
  @Override
  protected void performTask() throws Exception {
    ClientProjectFile[] newFile = clientProjectFile.getChildren();
    setResult(newFile);
  }

  /**
   * Mtodo de convenincia para chamada automtica da task.
   * 
   * @param parent
   * @return o arquivo ou null
   */
  static final public ClientProjectFile[] runTask(final ClientProjectFile parent) {
    final GetChildrenTask task = new GetChildrenTask(parent);
    final String[] args = new String[] { parent.toString() };
    final boolean hasResult = task.execute(args);
    if (!hasResult) {
      return null;
    }

    final ClientProjectFile[] result = task.getResult();
    return result;
  }
}
