/*
 * Detalhes da ltima alterao:
 * 
 * $Author: ururahy $
 * $Date: 2009-03-06 14:50:04 -0300 (Fri, 06 Mar 2009) $
 * $Revision: 88977 $
 */
package csbase.client.project.tasks;

import javax.swing.SwingUtilities;

import csbase.client.desktop.DesktopFrame;
import csbase.client.project.ProjectTree;
import csbase.client.project.action.ProjectCloseAction;
import csbase.client.remote.ProjectObserver;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.CommonClientProject;
import csbase.logic.ProjectFile;
import csbase.logic.UserProjectInfo;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectServiceInterface;

/**
 * Abre um projeto no servidor e atualiza numa rvore de projetos.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class OpenProjectOnTreeRemoteTask extends OpenProjectRemoteTask {
  /**
   * A rvore de projetos.
   */
  private ProjectTree tree;

  /**
   * Abre um projeto no servidor e atualiza numa rvore de projetos.
   * 
   * @param projectName O nome do projeto.
   * @param userId O identificador do usurio que est abrinco o projeto.
   * @param tree A rvore de projetos.
   */
  public OpenProjectOnTreeRemoteTask(String projectName, Object userId,
    ProjectTree tree) {
    super(projectName, userId);
    this.tree = tree;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected final void performTask() throws Exception {
    CommonClientProject project = this.tree.getProject();
    if (project != null) {
      ProjectCloseAction closeAction = new ProjectCloseAction(this.tree);
      if (!closeAction.close()) {
        return;
      }
    }
    super.performTask();
    project = this.getResult();
    ClientRemoteLocator.server.setServerSessionProperty(
      ProjectServiceInterface.CURRENT_PROJECT_SESSION_PROPERTY,
      new UserProjectInfo(project.getUserId(), project.getName()));
    try {
      project.installObserver(new ProjectObserver(project));
    }
    catch (Exception e) {
      DesktopFrame desktop = DesktopFrame.getInstance();
      StandardErrorDialogs.showErrorDialog(desktop.getDesktopFrame(), desktop
        .getTitle(), e);
    }
    // Antecipa a carga dos filhos e netos na task da viso inicial da rvore.
    ProjectFile[] children = project.getTree().getChildren();
    for (ProjectFile child : children) {
      try {
        child.getChildren();
      }
      catch (Exception e) {
        // Devemos abrir o projeto mesmo que ocorra um erro durante a abertura.
        DesktopFrame desktop = DesktopFrame.getInstance();
        StandardErrorDialogs.showErrorDialog(desktop.getDesktopFrame(), desktop
          .getTitle(), e.getMessage(), e);
      }
    }
    SwingUtilities.invokeLater(new TreeUpdater(this.tree, project));
  }

  /**
   * Atualiza uma rvore de projetos com um projeto.
   * 
   * @author Tecgraf/PUC-Rio
   */
  private static class TreeUpdater implements Runnable {
    /**
     * A rvore de projetos.
     */
    private ProjectTree tree;
    /**
     * O projeto.
     */
    private CommonClientProject project;

    /**
     * Atualiza uma rvore de projetos com um projeto.
     * 
     * @param tree A rvore de projetos.
     * @param project O projeto.
     */
    TreeUpdater(ProjectTree tree, CommonClientProject project) {
      this.tree = tree;
      this.project = project;
    }

    /**
     * {@inheritDoc}
     */
    public void run() {
      this.tree.setProject(this.project);
    }
  }
}
