/*
 * Detalhes da ltima alterao:
 * 
 * $Author: clinio $
 * $Date: 2008-07-02 16:39:27 -0300 (Wed, 02 Jul 2008) $
 * $Revision: 79484 $
 */
package csbase.client.project.tasks;

import csbase.client.desktop.RemoteTask;
import csbase.logic.CommonClientProject;

/**
 * <p>
 * Tarefa que define a abertura de um projeto no servidor.
 * </p>
 * <p>
 * OBS:  instalado um observador remoto para um projeto aberto.
 * </p>
 * 
 * @author Tecgraf/PUC-Rio
 */
public class OpenProjectRemoteTask extends RemoteTask<CommonClientProject> {
  /** O nome do projeto a ser aberto. */
  private String projectName;

  /** O identificador do usurio que est abrindo o projeto. */
  private Object userId;

  /**
   * Cria uma tarefa para abertura de um projeto no servidor.
   * 
   * @param projectName O nome do projeto a ser aberto.
   * @param userId O identificador do usurio que est abrindo o projeto.
   */
  public OpenProjectRemoteTask(String projectName, Object userId) {
    this.projectName = projectName;
    this.userId = userId;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void performTask() throws Exception {
    setResult(openProject());
  }

  /**
   * Abre um projeto no servidor.
   * 
   * @return O projeto aberto, ou null, caso no seja possvel cri-lo.
   * 
   * @throws Exception Caso ocorra algum erro no procedimento.
   */
  protected CommonClientProject openProject() throws Exception {
    return CommonClientProject.openProject(this.userId, this.projectName, true);
  }

  /**
   * Obtm o nome do projeto a ser aberto.
   * 
   * @return O nome do projeto a ser aberto.
   */
  protected final String getProjectName() {
    return this.projectName;
  }

  /**
   * Obtm o identificador do usurio que est abrindo o projeto.
   * 
   * @return O identificador do usurio que est abrindo o projeto.
   */
  protected final Object getUserId() {
    return this.userId;
  }
}
