/*
 * $Id$
 */
package csbase.client.remote;

import java.awt.Window;
import java.util.Date;

import javax.swing.SwingUtilities;

import tecgraf.javautils.FormatUtils;
import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.remote.srvproxies.EventLogProxy;
import csbase.logic.MonitoredServerListener;
import csbase.logic.ServerURI;

/**
 * Possui uma implementao padro de listener do cliente para monitorao de
 * conexo com o servidor. Exibe um JOptioPane quando a comunicao  perdida ou
 * restabelecida.
 * 
 */
public class ClientRemoteMonitorListener implements MonitoredServerListener {

  /** Janela que partir da qual o JOptioPane ser exibido. */
  private final Window parent;

  /** Fila de log de aes do cliente */
  private static String[] desktopQueue = new String[] { "ConnectionMonitoring" };

  /**
   * Construtor.
   * 
   * @param parent Janela que partir da qual o JOptioPane.
   */
  public ClientRemoteMonitorListener(Window parent) {
    this.parent = parent;
  }

  /**
   * {@inheritDoc}
   */
  public void notifyConnectionLost(final ServerURI serverURI) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        String serverName = serverURI.getHost();
        String msg =
          String.format(LNG
            .get("ClientRemoteMonitorListener.connection.lost"),
            serverName, FormatUtils.format(new Date()) );
        String title = String
        .format(LNG.get("ClientRemoteMonitorListener.connection.lost.title"),
          serverName);
        StandardDialogs.showErrorDialog(parent, title, msg);
      }
    });
  }

  /**
   * {@inheritDoc} Aps o restabelecimento da conexo, verifica se a verso 
   * compatvel com o servidor. Caso no seja, informa ao usurio.
   */
  public void notifyConnectionReestablished(final ServerURI serverURI) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        ClientRemoteMonitor monitor = ClientRemoteMonitor.getInstance();
        String serverName = serverURI.getHost();
        if (monitor.isSameVersion()) {
          EventLogProxy.addClientInformation(desktopQueue, LNG
            .get("ClientRemoteMonitorListener.connection.reestablished.log"));

          String msg =
            String.format(LNG
              .get("ClientRemoteMonitorListener.connection.reestablished"),
              serverName, FormatUtils.format(new Date()) );
          String title = 
            String.format(LNG
              .get("ClientRemoteMonitorListener.connection.reestablished.title"),
              serverName);
          StandardDialogs.showInfoDialog(parent, title, msg);

        }
        else {
          EventLogProxy.addClientInformation(desktopQueue, LNG
            .get("ClientRemoteMonitorListener.different.version.log"));

          String msg =
            String.format(LNG
              .get("ClientRemoteMonitorListener.different.version"),
              serverName, FormatUtils.format(new Date()));
          String title = String.format(LNG
              .get("ClientRemoteMonitorListener.different.version.title"),
              serverName);
          StandardDialogs.showErrorDialog(parent, title, msg);
        }
      }
    });
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void notifyLoggedIn(ServerURI serverURI) {
    /** nada a fazer */
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void notifyLoggedOut(ServerURI serverURI) {
    /** nada a fazer */
  }

}
