/*
 * NotificationObserver.java @author Andr Luiz Clinio $Revision: 76101 $ -
 * $Date: 2008-05-19 15:47:18 -0300 (Mon, 19 May 2008) $
 */
package csbase.client.remote;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Observable;

import csbase.remote.NotificationServiceInterface;
import csbase.remote.RemoteObserver;

/**
 * Gateway de observao no cliente para um observador remoto do servio de
 * notificao e um observavl das notificaes no lado cliente.
 */
public class NotificationObserver extends Observable implements RemoteObserver {
  /** Nmero de referncias ao gateway */
  private int numRefs = 0;

  /**
   * Mtodo de update remoto que repassa aos observadores locais.
   * 
   * @param arg .
   * 
   * @throws RemoteException exceo.
   */
  public void update(final Object arg) throws RemoteException {
    setChanged();
    notifyObservers(arg);
  }

  /**
   * Check de existncia de alguma referncia ao gateway no cliente.
   * 
   * @return uma flag indicativo de observadores no cliente.
   */
  public boolean hasNoRefs() {
    return numRefs <= 0;
  }

  /**
   * Decremento de referncias.
   */
  public void decRefs() {
    numRefs--;
  }

  /**
   * Incremento de referncias.
   */
  public void incRefs() {
    numRefs++;
  }

  /**
   * Adiciona o observador no servidor para o identificador do usurio.
   * 
   * @param userId
   * 
   * @throws RemoteException exceo de rmi
   */
  public void addUserObserver(Object userId) throws RemoteException {
    ClientRemoteMonitor.getInstance().addObserver(
      NotificationServiceInterface.SERVICE_NAME, this, userId);
  }

  /**
   * Remove o observador no servidor para o identificador do usurio.
   * 
   * @param userId
   * 
   * @throws RemoteException exceo de rmi
   */
  public void deleteUserObserver(Object userId) throws RemoteException {
    ClientRemoteMonitor.getInstance().deleteObserver(
      NotificationServiceInterface.SERVICE_NAME, this, userId);
  }

  /**
   * Exportao do objeto remoto.
   * 
   * @throws RemoteException exceo de rmi
   */
  public void install() throws RemoteException {
    UnicastRemoteObject.exportObject(this, 0);
  }

  /**
   * Retirada da exportao do objeto remoto.
   * 
   * @throws RemoteException exceo de rmi
   */
  public void uninstall() throws RemoteException {
    UnicastRemoteObject.unexportObject(this, true);
  }

  /**
   * Construtor de registro do objeto remoto.
   * 
   * @throws RemoteException exceo de rmi
   */
  public NotificationObserver() throws RemoteException {
    numRefs = 0;
    install();
  }
}
