/*
 * ProjectAdminObserver.java
 *
 * $Author: costa $
 * $Revision: 36914 $  - $Date: 2006-05-19 15:18:39 -0300 (Fri, 19 May 2006) $
 */
package csbase.client.remote;

import csbase.logic.AdministrationEvent;
import csbase.logic.AllProjectsArgObserver;
import csbase.logic.ProjectAdminInfo;
import csbase.logic.ProjectAdminPermission;
import csbase.logic.User;
import csbase.logic.UserProjectsArgObserver;
import csbase.remote.ProjectServiceInterface;
import csbase.remote.RemoteObserver;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import java.util.Observable;

/**
 * A classe <code>ProjectAdminObserver</code> implementa um observador  remoto
 * do servio de administrao de projetos.  Possui apenas uma nica
 * instncia. Interage com a classe {@link csbase.logic.ProjectAdminInfo
 * ProjectAdminInfo} para manter as informaes de projetos atualizadas.
 *
 * @author Maria Julia Lima
 * @version $Revision: 36914 $
 */
public class ProjectAdminObserver extends Observable implements RemoteObserver {
  /** Instncia nica do observador */
  private static ProjectAdminObserver instance = null;
  private Object arg;

  /**
   * Inicializa os servios desta classe: cria o nico objeto deste tipo e o
   * registra como observador do ProjectService e do ServerMonitor.
   *
   * @throws RemoteException .
   */
  public static void start()
  throws RemoteException {
    if (instance != null) {
      return;
    }
    instance = new ProjectAdminObserver();
  }

  public static void stop()
  throws RemoteException {
    if (instance == null) {
      return;
    }
    instance.uninstall();
  }

  /**
   * Retorna a nica instancia desse Objeto
   *
   * @return .
   */
  public static ProjectAdminObserver getInstance() {
    return instance;
  }

  /**
   * Registra no servidor este objeto como observador de projectAdmin.
   *
   * @throws RemoteException .
   */
  public void install()
  throws RemoteException {
    User loggedUser = User.getLoggedUser();
    ProjectAdminPermission projAdmPermission =
      (ProjectAdminPermission)loggedUser.getPermission(
        ProjectAdminPermission.class);
    if (loggedUser.isAdmin() || (projAdmPermission != null)) {
      arg = new AllProjectsArgObserver();
    }
    else {
      arg = new UserProjectsArgObserver(loggedUser.getId());
    }
    ClientRemoteMonitor.getInstance().addObserver(
      ProjectServiceInterface.SERVICE_NAME, this, arg);
  }

  /**
   * Remove do servidor o registro desse observador.
   *
   * @throws RemoteException .
   */
  public void uninstall()
  throws RemoteException {
    ClientRemoteMonitor.getInstance().deleteObserver(
      ProjectServiceInterface.SERVICE_NAME, this, arg);
    instance = null;
    UnicastRemoteObject.unexportObject(this, true);
  }

  /**
   * Esse mtodo  chamado sempre que alguma alterao ocorre nas informaes
   * administrativas dos projetos. Informa  classe {@link
   * csbase.logic.ProjectAdminInfo ProjectAdminInfo}  sobre a ao ocorrida no
   * provedor. Essa ao  representada por um objeto da classe {@link
   * csbase.logic.AdministrationEvent AdministrationEvent}
   *
   * @param event Descreve a ao ocorrida no provedor.
   */
  public void update(Object event) {
    if (event instanceof AdministrationEvent) {
      AdministrationEvent action = (AdministrationEvent)event;
      ProjectAdminInfo.update(action);
    }

    // event pode ser ProjectUserEvent ou ProjectUsersUpdateEvent
    else {
      setChanged();
      notifyObservers(event);
    }
  }

  /**
   * Cria um observador remoto do servio de administrao de projetos. Exporta
   * esse objeto para o sistema rmi.
   *
   * @throws RemoteException
   */
  protected ProjectAdminObserver()
  throws RemoteException {
    super();
    UnicastRemoteObject.exportObject(this, 0);
    install();
  }
}
