package csbase.client.remote.manager.trustedserver;

import csbase.client.remote.ClientRemoteMonitor;
import csbase.remote.TrustedServerServiceInterface;
import csbase.remote.RemoteObserver;

import java.rmi.NoSuchObjectException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import java.util.Observable;

/**
 * Representa um observador do servio de servidores confiveis.  responsvel por
 * atualizar manter caches sincronizados em relao ao repositrio de
 * servidores confiveis do servio.
 *
 * @author Tecgraf/PUC-Rio
 */
final class TrustedServerServiceRemoteObserver extends Observable
implements RemoteObserver {
  public void update(Object event) {
    setChanged();
    notifyObservers(event);
  }

  public void install()
  throws RemoteException {
    UnicastRemoteObject.exportObject(this, 0);
    ClientRemoteMonitor.getInstance().addObserver(
      TrustedServerServiceInterface.SERVICE_NAME, this, null);
  }

  public void uninstall() {
    ClientRemoteMonitor.getInstance().deleteObserver(
      TrustedServerServiceInterface.SERVICE_NAME, this, null);
    try {
      UnicastRemoteObject.unexportObject(this, true);
    }
    catch (NoSuchObjectException e) {
      IllegalStateException exception =
        new IllegalStateException("Erro ao remover objeto do registro do RMI.");
      exception.initCause(e);
      throw exception;
    }
  }
}
