/*
 * NotificationTranslator.java $Author: clinio $ $Revision: 94187 $ - $Date:
 * 2006-12-07 15:53:32 -0200 (Thu, 07 Dec 2006) $
 */
package csbase.client.util;

import java.text.MessageFormat;

import tecgraf.javautils.FormatUtils;
import tecgraf.javautils.LNG;
import csbase.logic.AlgorithmsReloadNotificationData;
import csbase.logic.CloseProjectNotificationData;
import csbase.logic.CommandEndNotificationData;
import csbase.logic.CommandErrorNotificationData;
import csbase.logic.CommandFailedNotificationData;
import csbase.logic.CommandKilledNotificationData;
import csbase.logic.CommandLostNotificationData;
import csbase.logic.CommandSuccessNotificationData;
import csbase.logic.CommandWithNoExitCodeNotificationData;
import csbase.logic.FileSystemSpaceNotificationData;
import csbase.logic.NotificationData;
import csbase.logic.OpenProjectNotificationData;
import csbase.logic.ProjectAdminInfo;
import csbase.logic.ProjectNotificationData;
import csbase.logic.SGANotificationData;
import csbase.logic.SGASet;
import csbase.logic.ServerNotificationData;
import csbase.logic.UserNotificationData;
import csbase.logic.UsersNotificationData;

/**
 * Singleton do tradutor de notificaes vindas do servidor vindas do arquivo
 * <code> csbase/client/resources/properties/language/idiom_pt_BR.properties
 * </code>
 * 
 * @author Andr Luiz Clinio
 */
public class NotificationTranslator {
  /**
   * Traduo de comando sem classe definida. Retorna uma string de lixo para
   * depurao.
   * 
   * @param data dados de fim de comando.
   * 
   * @return .
   */
  public String translate(Object data) {
    return LNG.get("notification.object.internal.error");
  }

  /**
   * Traduo de comando sem classe especfica de notificao definida. Retorna
   * uma string de lixo para depurao.
   * 
   * @param data dados de fim de comando.
   * 
   * @return .
   */
  public String translate(NotificationData data) {
    return LNG.get("notification.untyped.object.internal.error");
  }

  /**
   * Traduo de notificao de usurio. Retorna uma string.
   * 
   * @param data objeto contedo de notificao
   * 
   * @return .
   */
  public String translate(UserNotificationData data) {
    return data.toString();
  }

  /**
   * Traduo de reload de servio de algoritmos.
   * 
   * @param data objeto contedo de notificao
   * 
   * @return uma string.
   */
  public String translate(AlgorithmsReloadNotificationData data) {
    return LNG.get("notification.data.algorithms.reload");
  }

  /**
   * Traduo de de/registro de SGA Retorna uma string.
   * 
   * @param data objeto contedo de notificao
   * 
   * @return .
   */
  public String translate(SGANotificationData data) {
    SGASet info = data.getMainInfo();
    String sgaFormat = null;
    String sgaName = info.getName() + "/" + info.getPlatformId();
    if (data.getFlag() == SGANotificationData.SGA_START_UP) {
      sgaFormat = LNG.get("notification.data.sga.startup");
    }
    else if (data.getFlag() == SGANotificationData.SGA_SHUTDOWN) {
      sgaFormat = LNG.get("notification.data.sga.shutdown");
    }
    else if (data.getFlag() == SGANotificationData.SGA_REINIT) {
      sgaFormat = LNG.get("notification.data.sga.reinit");
    }
    else {
      return "!!!";
    }
    return MessageFormat.format(sgaFormat, new Object[] { sgaName });
  }

  /**
   * Traduo de start-up do SSI Retorna uma string.
   * 
   * @param data objeto contedo de notificao
   * 
   * @return .
   */
  public String translate(ServerNotificationData data) {
    if ( data.getState()) {
      final String startStr = LNG.get("notification.data.server.startup");
      return startStr;
    }
    final String shutdownStr = LNG.get("notification.data.server.shutdown");
    return shutdownStr;
  }

  /**
   * Traduo de comando terminado no SGA.
   * 
   * @param data dados de fim de comando.
   * 
   * @return .
   */
  public String translate(CommandEndNotificationData data) {
    if (data.hasTimeInfo()) {
      return MessageFormat.format(LNG.get("notification.data.command.end"),
        new Object[] { data.getCommandId(), data.getCmdDesc(), data.getTip(),
            formatInterval(data.getUserTime()),
            formatInterval(data.getCPUTime()),
            formatInterval(data.getElapsedTime()), data.getStartTime(),
            data.getEndTime(), data.getExecHost() });
    }

   return MessageFormat.format(LNG.get("notification.data.command.end.no.data"), new Object[] {
          data.getCommandId(), data.getCmdDesc(), data.getTip(),
          data.getStartTime(), data.getEndTime(), data.getExecHost() });
  }

  /**
   * Traduo de comando terminado com erro no SGA.
   * 
   * @param data dados de fim de comando.
   * 
   * @return .
   */
  public String translate(CommandErrorNotificationData data) {
    return MessageFormat.format(LNG.get("notification.data.command.error"),
      new Object[] { data.getCommandId(), data.getCmdDesc(), data.getTip(),
          data.getStartTime(), data.getExecHost() });

  }

  /**
   * Traduo de comando terminado no SGA, cujo cdigo de retorno no foi
   * encontrado.
   * 
   * @param data dados de fim de comando.
   * 
   * @return .
   */
  public String translate(CommandWithNoExitCodeNotificationData data) {
    return MessageFormat.format(LNG.get("notification.data.command.no_code"),
      new Object[] { data.getCommandId(), data.getCmdDesc(), data.getTip(),
          data.getStartTime(), data.getExecHost() });

  }

  /**
   * Traduo de comando terminado com sucesso no SGA.
   * 
   * @param data dados de fim de comando.
   * 
   * @return .
   */
  public String translate(CommandSuccessNotificationData data) {
    return MessageFormat.format(LNG.get("notification.data.command.success"),
      new Object[] { data.getCommandId(), data.getCmdDesc(), data.getTip(),
          formatInterval(data.getUserTime()),
          formatInterval(data.getCPUTime()),
          formatInterval(data.getElapsedTime()), data.getStartTime(),
          data.getEndTime(), data.getExecHost() });

  }

  /**
   * Faz a formatao de um tempo em segundos
   * 
   * @param timesec o tempo
   * @return o texto
   */
  private String formatInterval(int timesec) {
    if (timesec < 0) {
      return "---";
    }
    return FormatUtils.formatInterval(timesec);
  }

  /**
   * Traduo de comando perdido no SGA.
   * 
   * @param data dados do comando.
   * 
   * @return .
   */
  public String translate(CommandLostNotificationData data) {
    return MessageFormat.format(LNG.get("notification.data.command.lost"),
      new Object[] { data.getCommandId(), data.getCmdDesc(), data.getTip(),
          data.getStartTime(), data.getExecHost() });
  }

  /**
   * Traduo de comando nao iniciado no SGA.
   * 
   * @param data dados do comando.
   * 
   * @return .
   */
  public String translate(CommandFailedNotificationData data) {
    return MessageFormat.format(LNG.get("notification.data.command.failed"),
      new Object[] { data.getCommandId(), data.getCmdDesc(), data.getTip(),
          data.getStartTime(), data.getExecHost() });
  }

  /**
   * Traduo de comando interrompido no SGA.
   * 
   * @param data dados do comando.
   * 
   * @return .
   */
  public String translate(CommandKilledNotificationData data) {
    return MessageFormat.format(LNG.get("notification.data.command.killed"),
      new Object[] { data.getCommandId(), data.getCmdDesc(), data.getTip(),
          data.getStartTime(), data.getExecHost() });
  }

  /**
   * Traduo de incluso/remoo de usurios em um projeto. Retorna uma string.
   * 
   * @param data objeto contedo de notificao
   * 
   * @return .
   */
  public String translate(UsersNotificationData data) {
    String text = null;
    Object projectAdmin = data.getUserId();
    String projectName = data.getProjectName();
    if (data.getType() == UsersNotificationData.INSERT) {
      text = LNG.get("notification.data.project.manager.user.inserted");
    }
    else if (data.getType() == UsersNotificationData.REMOVE) {
      text = LNG.get("notification.data.project.manager.user.removed");
    }
    else if (data.getType() == UsersNotificationData.REMOVE_PROJECT) {
      text = LNG.get("notification.data.project.manager.prj.deleted");
    }
    else {
      // FIXME ??! certamente no  a melhor coisa se fazer...
      return "!!!";
    }
    return MessageFormat.format(text,
      new Object[] { projectName, projectAdmin });
  }

  /**
   * Traduo da notificao de administrao de projetos.
   * 
   * @param data O contedo de notificao
   * 
   * @return O texto correspondente  notificao que ser exibida ao usurio.
   */
  public String translate(ProjectNotificationData data) {
    ProjectAdminInfo info = data.getProjectAdminInfo();
    int state = info.getState();
    switch (state) {
      case ProjectAdminInfo.WAITING_AREA_ALLOCATION:
        return MessageFormat.format(LNG
          .get("notification.data.area.alloc.wait"), new Object[] {
            info.getProjectName(), info.getUserId() });
      case ProjectAdminInfo.UNLOCKED_WITH_AREA_ALLOCATED:
        return MessageFormat.format(LNG.get("notification.data.area.unlock"),
          new Object[] { info.getProjectName(),
              new Long(info.getAreaRequestSize()),
              new Long(info.getAreaLockedSize()) });
      case ProjectAdminInfo.WAITING_AREA_FREE:
        return MessageFormat.format(
          LNG.get("notification.data.area.free.wait"), new Object[] {
              info.getProjectName(), info.getUserId() });
      default:
        return LNG.get("notification.data.area.no.msg.error");
    }
  }

  /**
   * Faz a traduo de uma notificao.
   * 
   * @param data a notificao
   * @return o texto
   */
  public String translate(OpenProjectNotificationData data) {
    return MessageFormat.format(LNG.get("notification.data.project.open"),
      new Object[] { data.getProjectName() });
  }

  /**
   * Faz a traduo de uma notificao.
   * 
   * @param data a notificao
   * @return o texto
   */
  public String translate(FileSystemSpaceNotificationData data) {
    final String fsName = data.getFileSystemName();
    final String key = "notification.data.file.system." + fsName;
    final String fsText =  !LNG.hasKey(key) ? fsName : LNG.get(key);
    final String msg = LNG.get("notification.data.file.system.space");
    return MessageFormat.format(msg, new Object[] { fsText, data.getPercentage(), data.getScale() });
  }

  /**
   * Faz a traduo de uma notificao.
   * 
   * @param data a notificao
   * @return o texto
   */
  public String translate(CloseProjectNotificationData data) {
    return MessageFormat.format(LNG.get("notification.data.project.close"),
      new Object[] { data.getProjectName() });
  }
}
