/*
 * $Author: ururahy $
 * $Date: 2009-01-15 16:03:57 -0200 (Thu, 15 Jan 2009) $
 * $Release: $
 */
package csbase.client.util.sga;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;

import javax.swing.JComboBox;

import csbase.client.algorithms.AlgorithmConfiguratorView;
import csbase.client.kernel.ClientException;
import csbase.logic.SGASet;

/**
 * Define um painel para seleo de SGAs.
 * 
 * @author ururahy
 *
 */
public class SGASingleSelectionPanel extends SGASelectionPanel {
  private JComboBox sgaServerComboBox;

  /**
   * Define um painel para seleo de SGAs.
   * 
   * @param algorithmConfigurator a viso do configurador de algoritmos, que informa
   *        se o algoritmo  de execuo simples.
   */
  public SGASingleSelectionPanel(AlgorithmConfiguratorView algorithmConfigurator) {
    super(algorithmConfigurator);
    this.sgaServerComboBox = makeSGAServerComboBox();
  }

  @Override
  public Collection<SGASet> getManualServerCollection() {
    Collection<SGASet> selectedServerCollection = new LinkedList<SGASet>();
    SGASet selectedServer = getSelectedServer();
    if (selectedServer != null) {
      selectedServerCollection.add(selectedServer);
    }
    return Collections.unmodifiableCollection(selectedServerCollection);
  }
  
  @Override
  public SGASet getManualSelectedServer() {
    return (SGASet) this.sgaServerComboBox.getSelectedItem();
  }

  @Override
  public boolean hasAvailableServers() {
    if(this.sgaServerComboBox.getItemCount() == 0)
      return false;
    return true;
  }

  /**
   * <p>
   * Verifica se a seleo de servidores est pronta.
   * </p>
   * 
   * <p>
   * A seleo  considerada pronta, se e somente se 1 dos seguintes critrios
   * for atendido:
   * 
   * <ul>
   * <li>
   * A seleo de servidor estiver no modo automtico.
   * </li>
   * <li>
   * A seleo de servidor estiver no modo manual e existir ao menos um servidor
   * selecionado.
   * </li>
   * </ul>
   * </p>
   *
   * @return .
   */
  @Override
  public boolean isSelectionReady() {
    if(!this.hasAvailableServers())
      return false;
    return true;
  }
  
  @Override
  public void populateGUIComponents() {
    this.removeAll();
    this.setLayout(new GridBagLayout());
    GridBagConstraints c = new GridBagConstraints();
    c.anchor = GridBagConstraints.WEST;
    c.gridx = 1;
    c.gridy = 1;
    c.gridwidth = 1;
    c.gridheight = 1;
    c.insets = new Insets(2, 2, 2, 2);
    c.weightx = 1.0;
    c.weighty = 0.0;
    c.fill = GridBagConstraints.HORIZONTAL;
    this.add(this.platformFilterPanel, c);
    c.insets = new Insets(2, 2, 2, 2);
    c.gridx = 1;
    c.gridy++;
    this.add(this.automaticServerRadioButton, c);
    c.gridy++;
    this.add(this.manualServerRadioButton, c);
    c.weightx = 0.0;
    c.gridy++;
    c.insets = new Insets(2, 20, 2, 2);
    this.add(this.sgaServerComboBox, c);
    this.revalidate();
    
    this.updateEnabledServers();
  }
  
  @Override
  public void updateEnabledServers() {
    if (this.hasAvailableServers()) {
      if (this.manualServerRadioButton.isSelected()) {
        this.sgaServerComboBox.setEnabled(true);
      }
      else {
        this.sgaServerComboBox.setEnabled(false);
      }
      this.manualServerRadioButton.setEnabled(true);
      this.automaticServerRadioButton.setEnabled(true);
    }
    else {
      this.manualServerRadioButton.setEnabled(false);
      this.automaticServerRadioButton.setEnabled(false);
      this.sgaServerComboBox.setEnabled(false);
    }
  }
  
  @Override
  public void reloadSGAs() {
    try {
      sgaServerComboBox.removeAllItems();
      for (SGASet sgaSet : this.loadServers()) {
        sgaServerComboBox.addItem(sgaSet);
      }
      updateEnabledServers();
    } catch (ClientException e) {
      e.printStackTrace();
    }
  }

  /**
   * Cria um componente do tipo JComboBox para a seleo de SGAs.
   * 
   * @return O componente JComboBox criado e populado com os SGAs disponveis.
   */
  private JComboBox makeSGAServerComboBox() {
    JComboBox combo = null;
    try {
      combo = new JComboBox(this.loadServers());
      combo.addItemListener(
        new ItemListener() {
          public void itemStateChanged(ItemEvent e) {
            fireChangedServer();
          }
        });
    } 
    catch (ClientException e) {
        e.printStackTrace();
    }
    return combo;
  }
}
