/**
 * Tecgraf
 * websintesi
 * $Author: taciana $
 * $Revision: 68848 $ - $Date: 2007-10-10 15:45:37 -0300 (Wed, 10 Oct 2007) $ * 
 */
package csbase.logic;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.ByteBuffer;
import java.rmi.RemoteException;

/**
 * DOCUMENT ME!
 *
 * @author pizzol Implementa um output stream a partir de um arquivo da rea de
 *         projetos (ClientProjectFile).
 */
public class ClientProjectFileOutputStream extends OutputStream {
  /** Arquivo a ser lido */
  private final ClientProjectFile file;

  /** Posio corrente no arquivo */
  private long currentFilePosition = 0;

  /**
   * Escreve um byte especfico.
   *
   * @param b Byte a ser escrito.
   *
   * @throws IOException 
   */
  @Override
  public void write(int b) throws IOException {
    try {
      byte[] array = new byte[1];
      array[0] = (byte)b;
      ByteBuffer auxBuffer = ByteBuffer.wrap(array);
      file.write(auxBuffer, currentFilePosition);
      currentFilePosition++;
    }
    catch (IOException e) {
      currentFilePosition = 0;
      throw e;
    }
  }

  /**
   * Escreve os bytes de um array no output stream.
   *
   * @param b Array de bytes a serem escritos.
   *
   * @throws IOException 
   */
  @Override
  public void write(byte[] b) throws IOException {
    try {
      ByteBuffer auxBuffer = ByteBuffer.wrap(b);
      file.write(auxBuffer, currentFilePosition);
      currentFilePosition += b.length;
    }
    catch (IOException e) {
      currentFilePosition = 0;
      throw e;
    }
  }

  /**
   * {@inheritDoc}
   * @see java.io.OutputStream#write(byte[], int, int)
   */
  @Override
  public void write(byte[] b, int off, int len)
  throws IOException {
    try {
      ByteBuffer auxBuffer = ByteBuffer.wrap(b, off, len);
      file.write(auxBuffer, currentFilePosition);
      currentFilePosition += len;
    }
    catch (IOException e) {
      currentFilePosition = 0;
      throw e;
    }
  }

  /**
   * Fora a escrita imediata dos bytes de sada bufferizados.
   *
   * @throws IOException 
   */
  @Override
  public void flush()
  throws IOException {
    // vazio
  }

  /**
   * Fecha o output stream.
   *
   * @throws IOException 
   */
  @Override
  public void close() throws IOException {
    truncate();
    file.close();
  }

  /**
   * Trunca o arquivo de acordo o seu tamanho, e reinicia o controle
   * interno da posio corrente do arquivo.
   *
   * @throws IOException 
   */
  private void truncate() throws IOException {
    file.truncate(currentFilePosition);
    currentFilePosition = 0;
  }

  /**
   * @return o {@link ClientProjectFile} associado a este stream
   */
  public ClientProjectFile getClientProjectFile() {
    return file;
  }

/**
   * Cria um output stream a partir de um arquivo da rea de projetos,
   * abrido-o para escrita.
   * @param file Arquivo da rea de projetos (um ClientProjectFile).
   * @throws RemoteException falha de rmi 
   */
  public ClientProjectFileOutputStream(final ClientProjectFile file)
  throws RemoteException {
    if (file == null) {
      throw new IllegalArgumentException("O parmetro file est nulo.");
    }
    this.file = file;
    file.openFileChannel(false);
  }
}
