/*
 * CommandEndNotificationData.java
 * 
 * $Author: isabella $ $Date: 2008-10-31 18:19:56 -0200 (Fri, 31 Oct 2008) $
 * $Revision: 85068 $
 */

package csbase.logic;

/**
 * Classe que representa os dados de mensagem de fim de comando em um SGA,
 * quando o comando no d informaes sobre como a execuo terminou (com
 * sucesso ou com erro).
 */
public class CommandEndNotificationData extends ExtendedCommandFinishedNotificationData {

  /**
   * Indica se os tempos de execuo dos comandos esto definidos na notificao
   */
  private boolean timeInfo;

  /**
   * Retorna verdadeiro se possui os tempos de execuo dos comandos.
   * 
   * @return verdadeiro se possui os tempos de execuo dos comandos.
   */
  public boolean hasTimeInfo() {
    return timeInfo;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected String getStatusString() {
    return "END";
  }

  /**
   * Constri o evento de trmino normal de comando.
   * 
   * @param commandId identificador do comando.
   * @param cmdDesc descrio do comando.
   * @param tip Dica para auxiliar a identificao do algoritmo do comando.
   * @param elapsedTimeSec tempo de parede.
   * @param userTimeSec tempo em modo usurio.
   * @param cpuTimeSec tempo em modo sistema.
   * @param startTime Hora de incio
   * @param endTime Hora de trmino.
   * @param execHost Servidor de execuo.
   * @param projectId identificador do projeto
   */
  public CommandEndNotificationData(Object commandId, String cmdDesc,
    String tip, int elapsedTimeSec, int userTimeSec, int cpuTimeSec,
    long startTime, long endTime, String execHost, Object projectId) {
    super(commandId, cmdDesc, tip, elapsedTimeSec, userTimeSec, cpuTimeSec,
      startTime, endTime, execHost, projectId);
    if ((elapsedTimeSec >= 0) || (userTimeSec >= 0) || (cpuTimeSec >= 0))
      this.timeInfo = true;
    else
      this.timeInfo = false;
  }

  /**
   * Constri o evento de trmino de comando sem dados de tempo.
   * 
   * @param commandId identificador do comando.
   * @param cmdDesc descrio do comando.
   * @param tip Dica para auxiliar a identificao do algoritmo do comando.
   * @param startTime Hora de incio
   * @param endTime Hora de trmino.
   * @param execHost Servidor de execuo.
   * @param projectId identificador do projeto
   */
  public CommandEndNotificationData(Object commandId, String cmdDesc,
    String tip, long startTime, long endTime, String execHost, Object projectId) {
    super(commandId, cmdDesc, tip, -1, -1, -1, startTime, endTime, execHost,
      projectId);
    this.timeInfo = false;
  }

}
