/*
 * $Id$
 */

package csbase.logic;

/**
 * 
 * Indica o que levou o comando a terminar.<br>
 * Pode ser vista como uma classificao do comando dentro do estado
 * {@link CommandStatus#FINISHED}. Caso ele no esteja neste estado, seu
 * {@link CommandFinalizationType} deve ser
 * {@link CommandFinalizationType#NOT_FINISHED}.
 * 
 * @author Tecgraf / PUC-Rio
 */
public enum CommandFinalizationType {
  /**
   * O comando no est no estado {@link CommandStatus#FINISHED}
   */
  NOT_FINISHED,
  /**
   * No se sabe em que condies o comando terminou, pois o comando  antigo.<br>
   * Ser utilizado na visualizao de dados de comandos persistidos antes da
   * criao desta classificao.
   */
  UNKNOWN,
  /**
   * No se sabe em que condies o comando terminou, pois o comando no
   * informa.<br>
   * Ser utilizado na visualizao de dados de comandos que no informam o
   * cdigo de sada durante a execuo.
   */
  END,
  /**
   * O comando terminou com sucesso (retornou cdigo de sada de execuo igual
   * a zero).
   */
  SUCCESS,
  /**
   * O comando terminou com erro (retornou cdigo de sada de execuo diferente
   * de zero).
   */
  EXECUTION_ERROR,
  /**
   * Houve uma falha na inicializao do comando.
   */
  FAILED,
  /**
   * O comando foi cancelado.
   */
  KILLED,
  /**
   * O comando foi perdido.
   */
  LOST,
  /**
   * O comando terminou, mas no foi possvel ler o cdigo de retorno, para
   * determinar se foi bem sucedido.
   */
  NO_EXIT_CODE;

}
