/*
 * CommandLostNotificationData.java
 * 
 * $Author: isabella $ $Date: 2008-10-31 18:19:56 -0200 (Fri, 31 Oct 2008) $
 * $Revision: 85068 $
 */

package csbase.logic;

/**
 * Classe que representa os dados de mensagem de comando perdido em um SGA.<br>
 * Isso acontece, por exemplo, na seguinte situao:<br>
 * Digamos que o SGAD seja desligado. Quando reiniciado, ele verifica o contedo
 * de sua tabela de algoritmos em execuo [que  persistida]. Havendo entradas,
 * verifica-se suas situaes. Se um comando no estiver mais executando e no
 * tiver terminado da forma usual, o SGAD no saber o que fazer, notificando
 * ento uma perda de comando.
 */
public class CommandLostNotificationData extends DefaultCommandFinishedNotificationData {

  /**
   * Constri o evento de perda de comando sem dados de tempo.
   * 
   * @param commandId identificador do comando.
   * @param cmdDesc descrio do comando.
   * @param tip Dica para auxiliar a identificao do algoritmo do comando.
   * @param startTime Hora de incio
   * @param execHost Servidor de execuo.
   * @param projectId identificador do projeto
   */
  public CommandLostNotificationData(Object commandId, String cmdDesc,
    String tip, long startTime, String execHost, Object projectId) {
    super(commandId, cmdDesc, tip, startTime, execHost, projectId);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected String getStatusString() {
    return "LOST";
  }

}
