package csbase.logic;

/**
 * Modela um projeto.
 */
public interface CommonProject {
  /**
   * Fornece o identificador do usurio ao qual o projeto pertence.
   *
   * @return Identificador do usurio.
   */
  Object getUserId();

  /**
   * Fornece o nome do projeto.
   *
   * @return Nome do projeto.
   */
  String getName();

  /**
   * Fornece a dscrio do projeto.
   *
   * @return A descrio do projeto.
   */
  String getDescription();

  /**
   * Altera a descrio do projeto. Essa modificao s ser persistida quando
   * o mtodo {@link modify} for chamado.
   *
   * @param Nova descrio.
   */
  void setDescription(String description);

  /**
   * Retorna a rvore do projeto, na forma de um {@link ProjectFile}.
   *
   * @return A rvore do projeto.
   */
  ProjectFile getTree();

  /**
   * Persiste as modificaes feitas nas informaes do projeto. Caso alguma
   * informao no possa ser alterada, seu valor original ser
   * reestabelecido.
   */
  void modify() throws Exception;

  /**
   * Fecha um projeto. Aps chamado este mtodo, o objeto que representa o
   * projeto no poder mais ser utilizado.
   * @param notify indicativo de notificao aos clientes.
   */
  void close(boolean notify) throws Exception;
}
