/*
 * CommandEndNotificationData.java
 * 
 * $Author: taciana $ $Date: 2007-06-26 12:18:26 -0300 (Tue, 26 Jun 2007) $
 * $Revision: 64808 $
 */

package csbase.logic;

/**
 * Classe que representa os dados mais detalhados de mensagem de fim de comando
 * em um SGA. Essa classe deve ser estendida pela notificaes de comando que
 * contm esses dados mais detalhados.
 */
public abstract class ExtendedCommandFinishedNotificationData extends
  CommandNotificationData {

  /** Tempo de parede do comando. */
  protected int elapsedTimeSec;

  /** Tempo de usurio do comando. */
  protected int userTimeSec;

  /** Tempo de CPU do comando. */
  protected int cpuTimeSec;

  /** hora de trmino */
  protected long endTime;

  /**
   * Constri o evento de trmino de comando.
   * 
   * @param commandId identificador do comando.
   * @param cmdDesc descrio do comando.
   * @param tip Dica para auxiliar a identificao do algoritmo do comando.
   * @param elapsedTimeSec tempo de parede.
   * @param userTimeSec tempo em modo usurio.
   * @param cpuTimeSec tempo em modo sistema.
   * @param startTime Hora de incio
   * @param endTime Hora de trmino.
   * @param execHost Servidor de execuo.
   * @param projectId identificador do projeto
   */
  public ExtendedCommandFinishedNotificationData(Object commandId, String cmdDesc,
    String tip, int elapsedTimeSec, int userTimeSec, int cpuTimeSec,
    long startTime, long endTime, String execHost, Object projectId) {
    super(commandId, cmdDesc, tip, startTime, execHost, projectId);
    this.elapsedTimeSec = elapsedTimeSec;
    this.userTimeSec = userTimeSec;
    this.cpuTimeSec = cpuTimeSec;
    this.endTime = endTime;
  }

  /**
   * String para auditagem.
   * 
   * @return uma string
   */
  @Override
  public String toString() {
    String msg =
      "CMD/" + getStatusString() + " (" + cmdDesc + ") (" + getTip() + ")"
        + " - " + commandId + " : ";
    if (userTimeSec >= 0) {
      msg = msg + "user " + userTimeSec + " s, ";
    }
    else {
      msg = msg + "user <no informado>, ";
    }
    if (cpuTimeSec >= 0) {
      msg = msg + "cpu " + cpuTimeSec + " s, ";
    }
    else {
      msg = msg + "cpu <no informado>, ";
    }
    if (elapsedTimeSec >= 0) {
      msg = msg + "total " + elapsedTimeSec + " s.";
    }
    else {
      msg = msg + "total <no informado>.";
    }
    msg = msg + "\n";
    if (startTime >= 0) {
      msg = msg + "hora de incio " + startTime + ".";
    }
    else {
      msg = msg + "hora de incio <no informado>.";
    }
    if (endTime >= 0) {
      msg = msg + "hora de trmino " + endTime + ".";
    }
    else {
      msg = msg + "hora de trmino <no informado>.";
    }
    if ((execHost != null) && (!execHost.equals(""))) {
      msg = msg + "executado por " + execHost + ".";
    }
    else {
      msg = msg + "executado por <no informado>.";
    }
    return msg;
  }

  /**
   * Retorna o tempo de parede do comando.
   * 
   * @return o tempo
   */
  public int getElapsedTime() {
    return elapsedTimeSec;
  }

  /**
   * Retorna o tempo em modo usurio.
   * 
   * @return o tempo
   */
  public int getUserTime() {
    return userTimeSec;
  }

  /**
   * Retorna o tempo de sistema.
   * 
   * @return o tempo
   */
  public int getCPUTime() {
    return cpuTimeSec;
  }

  /**
   * Retorna a hora de trmino da execuo do comando.
   * 
   * @return o identificador
   */
  public long getEndTime() {
    return endTime;
  }

}
