package csbase.logic;

import java.io.Serializable;

/**
 * Esta classe representa as informaes de um arquivo que foi obtido como
 * resultado da pesquisa de arquivos. Este objeto ser utilizado tanto pelo
 * servidor (que fornecer o resultado) como pelo cliente (que exibir este
 * resultado em uma tabela).
 * 
 * @author Tecgraf
 */
public class FileInfoSearchResult implements Serializable {

  /**
   * Nome do arquivo.
   */
  private String fileName;

  /**
   * Path do arquivo relativo ao projeto.
   */
  private String filePath;

  /**
   * Nome do projeto que est associado ao arquivo.
   */
  private String projectName;

  /**
   * Construtor.
   * 
   * @param fileName - nome do arquivo.
   * @param filePath - path do arquivo.
   * @param projectName - nome do projeto associado.
   */
  public FileInfoSearchResult(String fileName, String filePath,
    String projectName) {
    this.fileName = fileName;
    this.filePath = filePath;
    this.projectName = projectName;
  }

  /**
   * Obtm o nome do projeto.
   * 
   * @return nome do projeto
   */
  public String getProjectName() {
    return projectName;
  }

  /**
   * Obtm o nome do arquivo.
   * 
   * @return nome do arquivo
   */
  public String getFileName() {
    return fileName;
  }

  /**
   * Obtm o path do arquivo.
   * 
   * @return path do arquivo
   */
  public String getFilePath() {
    return filePath;
  }

}
