/*
 * $Id: FileSystemSpaceNotificationData.java 80681 2008-07-30 22:10:26Z clinio $
 */

package csbase.logic;

/**
 * Notificao de espao em disco utilizado.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class FileSystemSpaceNotificationData extends NotificationData {

  /**
   * Nome do file system.
   */
  final private String fileSystemName;

  /**
   * Percentual
   */
  final private double percentage;

  /**
   * Indicativo de volatilidade;
   */
  final private boolean isVolatile;

  /**
   * A escala comparativa.
   */
  final private double scale;

  /**
   * Consulta o valor do nome do file system
   * 
   * @return o valor
   */
  public final String getFileSystemName() {
    return fileSystemName;
  }

  /**
   * Consulta o valor de scale
   * 
   * @return o valor
   */
  public final double getScale() {
    return scale;
  }

  /**
   * Consulta o valor do percentual
   * 
   * @return o valor
   */
  public final double getPercentage() {
    return percentage;
  }

  /**
   * {@inheritDoc}
   * 
   * A volatilidade  dada na construo da mensagem; isto , quem notifica
   * determina se a mensagem ser ou no voltil.
   */
  @Override
  public boolean isVolatile() {
    return isVolatile;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String toString() {
    final String prefix = "Aviso de espao no file system ";
    final String perc = String.format("%.2f%%", percentage);
    return prefix + fileSystemName + " em: " + perc;
  }

  /**
   * Construtor
   * 
   * @param fileSystemName nome do file system
   * @param percentage percentual infomado
   * @param scale a escala de aviso.
   * @param isVolatile volatilidade da mensagem
   */
  public FileSystemSpaceNotificationData(final String fileSystemName,
    final double percentage, final double scale, final boolean isVolatile) {
    this.fileSystemName = fileSystemName;
    this.percentage = percentage;
    this.isVolatile = isVolatile;
    this.scale = scale;
  }

}
