/*
 * Notification.java
 * $Author: taciana $
 * $Revision: 76954 $  - $Date: 2008-06-06 16:24:00 -0300 (Fri, 06 Jun 2008) $
 */
package csbase.logic;

import java.io.Serializable;
import java.util.Date;

/**
 * Classe que define uma notificao a um usurio.
 * @author Andr Luiz Clinio
 */
public class Notification implements Serializable {

  /** Identificador da mensagem */
  private Object id = null;

  /** Nome do remetente */
  private String senderName = null;

  /** Contedo da mensagem */
  private NotificationData data = null;

  /** Data de envio da mensagem */
  private Date date = null;

  /**
   * Flag indicando se esta notificao deve ser visualizada pelo usurio de 
   * destino.
   */
  private boolean display;
  
  /**
   * Consulta a data de envio da mensagem
   *
   * @return a data
   */
  public Date getDate() {
    return date;
  }

  /**
   * Consulta o identificador da mensagem
   *
   * @return o identificador
   */
  public Object getId() {
    return id;
  }

  /**
   * Consulta ao nome do remetente
   *
   * @return uma string
   */
  public String getSenderName() {
    return senderName;
  }

  /**
   * Consulta o contedo da mensagem
   *
   * @return um objeto com os dados
   */
  public NotificationData getData() {
    return data;
  }

  /**
   * Verifica se uma notificao  considerada voltil pelo seu contedo.
   * @return flag booleano indicativo.
   */
  public boolean isVolatile() {
    if (data == null) return true;
    return data.isVolatile();
  }

  /**
   * Indica se essa notificao deve ser visualizada pelo usurio de destino.
   * 
   * @return <tt>true</tt> se o cliente deve expor essa notificao ao usurio.
   */
  public boolean display() {
    return display;
  }

  /**
   * Constroi uma instncia que ser visualizada pelo usurio de destino.
   * 
   * @param id Identificador da mensagem.
   * @param senderName Nome do remetente.
   * @param data Contedo da mensagem.
   * 
   * @throws Exception .
   */
  public Notification(Object id, String senderName, NotificationData data)
    throws Exception {
    this(id, senderName, data, true);
  }

  /**
   * Construtor padro.
   * 
   * @param id Identificador da mensagem.
   * @param senderName Nome do remetente.
   * @param data Contedo da mensagem.
   * @param display <tt>true</tt> se o cliente deve expor essa notificao ao 
   *        usurio.
   * 
   * @throws Exception .
   */
  public Notification(Object id, String senderName, NotificationData data,
    boolean display) throws Exception {
    this.date = new Date();
    this.data = data;
    this.id = id;
    this.senderName = senderName;
    this.display = display;
  }
}
