/*
 * $Id: NotificationData.java 97098 2009-10-05 19:01:29Z clinio $
 */

package csbase.logic;

import java.io.Serializable;

/**
 * Representa o contedo da mensagem de notificao.
 * 
 * @author Tecgraf/PUC-Rio
 */
public abstract class NotificationData implements Serializable {
  /**
   * Indicativo de popup de aviso
   */
  private final boolean mustPopUp;

  /**
   * Requer esta implementao para quem enviar a mensagem definir se essa
   * mensagem pode ser considerada voltil.
   * 
   * @return indicativo
   */
  public abstract boolean isVolatile();

  /**
   * Indica de um popup deve ser feito para a notificao.
   * 
   * @return indicativo
   */
  public boolean mustPopUp() {
    return this.mustPopUp;
  }

  /**
   * Requer esta implementao apenas para auditagem nos logs do servidor. Este
   * mtodo <u>no</u> define o que vai aparacer na ISI (texto da mensagem para
   * o usurio).
   * 
   * @see csbase.client.util.NotificationTranslator
   */
  @Override
  public abstract String toString();

  /**
   * Construtor
   */
  protected NotificationData() {
    this(false);
  }

  /**
   * Construtor
   * 
   * @param mustPopUp indicaitvo de popup.
   */
  protected NotificationData(final boolean mustPopUp) {
    this.mustPopUp = mustPopUp;
  }
}