/*
 * ProjectEvent.java $author$ $Revision: 86441 $ - $Date: 2008-09-05 15:12:59
 * -0300 (sex, 05 set 2008) $
 */
package csbase.logic;

import java.io.Serializable;

/**
 * Representa eventos que podem ocorrer em um projeto. Objetos dessa classe so
 * criados pelo servidor sempre que acontecer alguma modificao em um projeto.
 */
public abstract class ProjectEvent implements Serializable {
  /** O evento de modificar informaes do projeto. */
  public static final int INFO_MODIFIED = 1;
  /** O evento de remoo do projeto. */
  public static final int PROJECT_DELETED = 2;
  /** O evento de incluir um arquivo na rvore do projeto. */
  public static final int NEW_FILE = 3;
  /** O evento de remover um arquivo da rvore do projeto. */
  public static final int FILE_DELETED = 4;
  /** O evento de renomear um arquivo. */
  public static final int NEW_FILE_NAME = 5;
  /**
   * O evento de mover um arquivo para um outro diretrio da rvore do projeto.
   */
  public static final int FILE_MOVED = 6;
  /** O evento de alterao do estado de um arquivo. */
  public static final int FILE_STATE_CHANGED = 7;
  /** O evento de atualizao da rvore do projeto. */
  public static final int TREE_REFRESHED = 8;
  /** O evento de fechamento de um projeto */
  public static final int PROJECT_CLOSED = 9;
  /** O evento de remover vrios arquivos da rvore do projeto. */
  public static final int FILES_DELETED = 10;
  /**
   * O evento de incluir vrios arquivos simultaneamente na rvore do projeto.
   */
  public static final int NEW_FILES = 11;
  /** O evento de atualizao de um diretrio do projeto. */
  public static final int DIR_REFRESHED = 12;

  /**
   * Indica qual foi o evento ocorrido no servidor. Pode ser uma das constantes
   * definidas nessa classe.
   */
  public int event;

  /**
   * Cria um evento, deduzindo os parmetros de acordo com o tipo de evento.
   * 
   * @param type - tipo de evento.
   * @param args - parmetros de acordo com o tipo de evento.
   * @return o evento criado.
   */
  public static ProjectEvent makeEvent(int type, Object[] args) {
    try {
      switch (type) {
        case INFO_MODIFIED:
          return new ProjectInfoModifiedEvent((CommonProjectInfo) args[0]);
        case PROJECT_DELETED:
          return new ProjectDeletedEvent((String) args[0]);
        case NEW_FILE:
          return new NewProjectFileEvent((String[]) args[0],
            (ProjectFile) args[1]);
        case NEW_FILES:
          return new NewProjectFilesEvent((String[][]) args[0],
            (ProjectFile[]) args[1]);
        case FILE_DELETED:
          return new ProjectFileDeletedEvent((String[]) args[0]);
        case NEW_FILE_NAME:
          return new ProjectFileRenamedEvent((String[]) args[0],
            (String) args[1], (String) args[2]);
        case FILE_MOVED:
          return null;
        case FILE_STATE_CHANGED:
          return new ProjectFileStateChangedEvent((String[]) args[0],
            (String) args[1], (boolean[]) args[2], args[3], args[4],
            ((Long) args[5]).longValue(), ((Long) args[6]).longValue(),
            (Boolean) args[7]);
        case TREE_REFRESHED:
          return new TreeRefreshedEvent((ProjectFile) args[0]);
        case DIR_REFRESHED:
          return new DirRefreshedEvent((String[]) args[0],
            (ProjectFile) args[1]);
        case PROJECT_CLOSED:
          return new ProjectClosedEvent();
        case FILES_DELETED:
          return new ProjectFilesDeletedEvent((String[][]) args[0]);
        default:
          return null;
      }
    }
    catch (Exception e) {
      return null;
    }
  }
}
