/*
 * PublicationInfo.java
 *
 * $Author: rodrigoh $
 * $Revision: 56574 $  - $Date: 2006-09-15 13:43:55 -0300 (Fri, 15 Sep 2006) $
 */
package csbase.logic;

import java.io.Serializable;

import java.util.Arrays;

/**
 * Representa os dados de publicao de um arquivo.
 *
 * @author Julia
 * @version $Revision: 56574 $
 */
public class PublicationInfo implements Serializable {
  /** Identificador do SSI no qual o arquivo est publicado.  sempre null. */
  private Object SSIId;

  /** Nome do SSI no qual o arquivo est publicado. */
  private String SSIName;

  /** Identificador do usrio que publicou o arquivo. */
  private Object whoPublishedId;

  /** Identificador do projeto */
  private Object projectId;

  /** Nome do projeto */
  private String projectName;

  /** Localizao do arquivo na rea de publicao. */
  private String[] path;

  /** Caminho real do arquivo. */
  private String absolutePath;

  /** Tamanho do arquivo. */
  private long fileSize;

  /**
   * Obtm o identificador do SSI no qual o arquivo est publicado.
   *
   * @return O identificador do SSI.
   */
  public Object getSSIId() {
    return SSIId;
  }

  /**
   * Obtm o nome do SSI no qual o arquivo est publicado.
   *
   * @return O nome do SSI.
   */
  public String getSSIName() {
    return SSIName;
  }

  /**
   * Obtm o identificador do usurio que publicou o arquivo.
   *
   * @return O identificador do usurio responsvel por essa publicao.
   */
  public Object getUserWhoPublished() {
    return whoPublishedId;
  }

  /**
   * Obtm o identificador do projeto de origem.
   *
   * @return O identificador do projeto de origem.
   */
  public Object getProjectId() {
    return this.projectId;
  }

  /**
   * Obtm o nome do projeto originrio do arquivi publicado.
   *
   * @return O nome do projeto.
   */
  public String getProjectName() {
    return projectName;
  }

  /**
   * Obtm o caminho de localizao do arquivo relativo ao projeto.
   *
   * @return O caminho de localizao do arquivo.
   */
  public String[] getPath() {
    return path;
  }

  /**
   * Obtm o caminho real do arquivo no servidor.
   *
   * @return O caminho real do arquivo no servidor.
   */
  public String getAbsolutePath() {
    return absolutePath;
  }

  /**
   * Obtm o tamanho do arquivo publicado.
   *
   * @return O tamanho (em bytes) do arquivo publicado.
   */
  public long getFileSize() {
    return fileSize;
  }

  /**
   * Testa se um <code>PublicationInfo</code>  igual a um outro  objeto.
   * Dois objetos <code>PublicationInfo</code> so iguais de forem da mesma
   * classe e se possuirem as mesmas informaes.
   *
   * @param obj O objeto a ser comparado.
   *
   * @return Verdadeiro, se os objetos forem iguais ou falso, caso contrrio.
   */
  @Override
  public boolean equals(Object obj) {
    if (!(obj instanceof PublicationInfo)) {
      return false;
    }
    PublicationInfo info = (PublicationInfo)obj;
    return ((this.SSIId == info.SSIId) || this.SSIId.equals(info.SSIId)) &&
    ((this.SSIName == info.SSIName) || this.SSIName.equals(info.SSIName)) &&
    ((this.whoPublishedId == info.whoPublishedId) ||
    this.whoPublishedId.equals(info.whoPublishedId)) &&
    ((this.projectName == info.projectName) ||
    this.projectName.equals(info.projectName)) &&
    ((this.path == info.path) || Arrays.equals(this.path, info.path)) &&
    (this.fileSize == info.fileSize);
  }

  /**
   * Cdigo hash do objeto.
   *
   * @return inteiro representando o cdigo hash.
   */
  @Override
  public int hashCode() {
    StringBuffer buffer = new StringBuffer();
    buffer.append(
      (SSIId == null) ? "SSIIdIsNull" : Integer.toString(SSIId.hashCode()));
    buffer.append(
      (whoPublishedId == null) ? "WhoPublishedIdIsNull"
      : Integer.toString(whoPublishedId.hashCode()));
    buffer.append((projectName == null) ? "ProjectNameIsNull" : projectName);
    buffer.append(
      (path == null || path.length == 0) ? "ThereAreNoElementsOrPathIsNull"
      : (path[0])); // se path[0] for nulo, a string "null"  adicionada
    buffer.append(Long.toString(fileSize));
    return buffer.toString().hashCode();
  }

/**
   * Constri um <code>PublicationInfo</code> com as informaes sobre a
   * publicao de um arquivo.
   *
   * @param SSIId O identificador do SSI no qual o arquivo est publicado.
   * @param SSIName O nome do SSI no qual o arquivo est publicado.
   * @param whoPublishedId O identificador do usurio que publicou o arquivo.
   * @param projectName O nome do projeto que originou o arquivo.
   * @param projectId O identificador do projeto de origem.
   * @param path A localizao (caminho) do arquivo relativo ao projeto.
   * @param absolutePath A localizao (caminho) do arquivo na rea de
   *        publicados.
   * @param fileSize O tamanho do arquivo publicado.
   */
  public PublicationInfo(
    Object SSIId, String SSIName, Object whoPublishedId, Object projectId,
    String projectName, String[] path, String absolutePath, long fileSize) {
    this.SSIId = SSIId;
    this.SSIName = SSIName;
    this.whoPublishedId = whoPublishedId;
    this.projectId = projectId;
    this.projectName = projectName;
    this.path = path;
    this.absolutePath = absolutePath;
    this.fileSize = fileSize;
  }
}
