package csbase.logic;

/**
 * Classe para ordenar arrays, em ordem crescente ou decrescente. Por enquanto,
 * s trabalha com inteiros, mas pode ser extendida para trabalhar com
 * qualquer tipo primitivo.
 *
 * @author Leonardo Barros
 */
public class QuickSort {
  /**
   * Ordena um array de inteiros, em ordem crescente ou decrescente.
   *
   * @param a array a ser ordenado.
   * @param ascendingOrder true se ordem ascendente; false se decrescente.
   */
  public static void sort(int[] a, boolean ascendingOrder) {
    sort(a, 0, a.length-1, ascendingOrder);
  }

  private static void sort(int[] a, int low, int high, boolean ascendingOrder) {
    if (low >= high) {
      return;
    }
    int p = partition(a, low, high, ascendingOrder);
    sort(a, low, p, ascendingOrder);
    sort(a, p+1, high, ascendingOrder);
  }

  private static int partition(
    int[] a, int low, int high, boolean ascendingOrder) {
    int pivot = a[low];
    int i = low-1;
    int j = high+1;
    while (i < j) {
      i++;
      if (ascendingOrder) {
        while (a[i] < pivot) {
          i++;
        }
      }
      else {
        while (a[i] > pivot) {
          i++;
        }
      }
      j--;
      if (ascendingOrder) {
        while (a[j] > pivot) {
          j--;
        }
      }
      else {
        while (a[j] < pivot) {
          j--;
        }
      }
      if (i < j) {
        swap(a, i, j);
      }
    }
    return j;
  }

  private static void swap(int[] a, int i, int j) {
    int temp = a[i];
    a[i] = a[j];
    a[j] = temp;
  }
}
