/*
 * $Id: UserProjectInfo.java 92186 2009-06-05 19:34:07Z ururahy $
 */
package csbase.logic;

import java.io.Serializable;

/**
 * Representa os dados de publicao de um arquivo.
 * 
 * @author Taciana
 * @version $Revision: 92186 $
 */
public class UserProjectInfo implements Serializable, Comparable {
  /** Identificador do usurio administrador do projeto */
  private Object userId;

  /** Nome do projeto */
  private String projectName;

  /**
   * Obtm o identificador do usurio dono do projeto.
   * 
   * @return O identificador do usurio.
   */
  public Object getUserId() {
    return userId;
  }

  /**
   * Obtm o nome do usurio dono do projeto.
   * 
   * @return O nome do usurio.
   * 
   * @throws Exception
   */
  public String getUserName() throws Exception {
    return User.getName(userId);
  }

  /**
   * Obtm o nome do projeto.
   * 
   * @return O nome do projeto.
   */
  public String getProjectName() {
    return projectName;
  }

  /**
   * Testa se um <code>UserProjectInfo</code>  igual a um outro objeto. Dois
   * objetos <code>UserProjectInfo</code> so iguais de forem da mesma classe e
   * se possuirem as mesmas informaes.
   * 
   * @param obj .
   * 
   * @return Verdadeiro, se os objetos forem iguais ou falso, caso contrrio.
   */
  @Override
  public boolean equals(Object obj) {
    if (!(obj instanceof UserProjectInfo)) {
      return false;
    }
    UserProjectInfo info = (UserProjectInfo) obj;
    return ((this.userId == info.userId) || this.userId.equals(info.userId))
      && ((this.projectName == info.projectName) || this.projectName
        .equals(info.projectName));
  }

  /**
   * Cdigo hash do objeto.
   * 
   * @return inteiro representando o cdigo hash.
   */
  @Override
  public int hashCode() {
    // XXX atributos podem ser nulos?
    String str = (projectName == null) ? "projectNameNull" : projectName;
    str +=
      (userId == null) ? "userIdNull" : Integer.toString(userId.hashCode());
    return str.hashCode();
  }

  /**
   * Constri um <code>UserProjectInfo</code> com as informaes sobre um
   * projeto e seu dono.
   * 
   * @param userId Identificador do usurio adminitrador do projeto.
   * @param projectName O nome do projeto.
   */
  public UserProjectInfo(Object userId, String projectName) {
    this.userId = userId;
    this.projectName = projectName;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String toString() {
    return this.projectName + " (" + this.userId + ")";
  }

  public int compareTo(Object info) {
    return this.toString().compareTo(info.toString());
  }
}
