/*
 * XApplicationInfo.java
 * $Author: cassino $
 * $Revision: 33587 $  - $Date: 2004-04-15 11:21:24 -0300 (Thu, 15 Apr 2004) $
 */
package csbase.logic;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * Representa os dados de uma aplicao para ser mostrada na ISI.
 *
 * @author Andr Luiz Clinio
 * @version $Revision: 33587 $
 */
public class XApplicationInfo implements Serializable {
  /**
   * Nome do diretrio que contm os executveis para as plataformas (contido
   * no diretrio de uma verso).
   */
  public static String BIN_DIR_NAME = "bin";
  /** Tamanho mximo do nome de um aplicao. */
  public static final int MAX_NAME_SIZE = 50;
  /** Identificador do aplicao. */
  private Object id;
  /** Nome do aplicao. */
  private String name;
  /** Data da ltima atualizao do aplicao. */
  private long lastUpdate;

  /**
   * Obtm o identificador.
   *
   * @return a identificao
   */
  public Object getId() {
    return id;
  }

  /**
   * Obtm o nome do aplicao.
   *
   * @return nome do aplicao
   */
  public String getName() {
    return name;
  }

  /**
   * Impresso default para a classe.
   *
   * @return umastring como nome da aplicao.
   */
  public String toString() {
    return getName();
  }

  /**
   * Obtm a data de ltima atualizao do aplicao.
   *
   * @return a data (nmero de milissegundos desde 1/1/1970)
   */
  public long getLastUpdate() {
    return lastUpdate;
  }

  /**
   * Altera a data de atualizao do aplicao.
   *
   * @param lastUpdate .
   */
  protected void setLastUpdate(long lastUpdate) {
    this.lastUpdate = lastUpdate;
  }

  /**
   * Construtor de um XApplicationInfo.
   *
   * @param id identificador do algorimo
   * @param name nome do aplicao
   * @param lastUpdate data de ltima atualizao
   */
  public XApplicationInfo(Object id, String name, long lastUpdate) {
    this.id = id;
    this.name = name;
    this.lastUpdate = lastUpdate;
  }
}
