package csbase.logic.algorithms;

import java.text.MessageFormat;

import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.configurationmanager.ConfigurationManagerException;
import tecgraf.javautils.configurationmanager.MissingPropertyException;
import csbase.exception.ConfigurationException;

public final class CommandLineParameterFormatter {

  private static CommandLineParameterFormatter instance;
  private String defaultCommandLinePattern;
  
  private CommandLineParameterFormatter() {
  }
  
  public static final CommandLineParameterFormatter getInstance() {
    if (instance == null) {
      instance = new CommandLineParameterFormatter();
    }
    return instance;
  }

  public String formatCommandLine(String parameterName,
    String parameterValue, String commandLinePattern) {
    if ((parameterValue == null) || (commandLinePattern == null)) {
      return null;
    }
    String[] variables = new String[] { "$NOME_DO_PARAMETRO",
        "$VALOR_DO_PARAMETRO" };
    for (int i = 0; i < variables.length; i++) {
      String format = "{" + i + "}";
      commandLinePattern = commandLinePattern.replace(variables[i], format);
    }
    return MessageFormat.format(commandLinePattern, new Object[] {
        parameterName, parameterValue });
  }

  public String getDefaultCommandLinePattern() {
    if (defaultCommandLinePattern == null) {
      Configuration configuration;
      try {
        configuration = ConfigurationManager.getInstance().getConfiguration(
          getClass());
      } catch (ConfigurationManagerException e) {
        throw new ConfigurationException(e);
      }
      String commandLinePatternKey = "commandLinePattern";
      try {
        String commandLinePattern = configuration
          .getMandatoryProperty(commandLinePatternKey);
        defaultCommandLinePattern = commandLinePattern;  
      } catch (MissingPropertyException e) {
        throw new ConfigurationException(e);
      }
    }
    return defaultCommandLinePattern;
  }
}
