package csbase.logic.algorithms.commands;

import csbase.logic.NotificationData;

/**
 * Evento que indica que algo ocorreu no servio de persistncia de comando.
 */
public class CommandPersistenceCommandNotificationData extends NotificationData {

	/**
	 * Indica o tipo do evento.
	 * 
	 * @author Tecgraf / PUC-Rio
	 */
	public enum Type {
		// O comando foi removido.
		REMOVED,
		// O comando foi salvo.
		SAVED,
		// O comando foi atualizado.
		UPDATED
	}

	/**
	 * O identificador do comando.
	 */
	private String commandId;

	/**
	 * O identificador do projeto.
	 */
	private Object projectId;

	/**
	 * Tipo do evento.
	 */
	private Type type;

	/**
	 * Cria o evento.
	 * 
	 * @param projectId
	 *            O identificador do projeto (no pode ser {@code null}).
	 * @param commandId
	 *            O identificador do comando (no pode ser {@code null}).
	 */
	public CommandPersistenceCommandNotificationData(Object projectId, String commandId,
			Type type) {
		if (commandId == null)
			throw new NullPointerException("O parmetro projectId est nulo.");
		if (projectId == null)
			throw new NullPointerException("O parmetro commandId est nulo.");
		if (type == null)
			throw new NullPointerException("O parmetro type est nulo.");

		this.commandId = commandId;
		this.projectId = projectId;
		this.type = type;
	}

	/**
	 * Obtm o identificador do comando.
	 * 
	 * @return O identificador do comando.
	 */
	public String getCommandId() {
		return commandId;
	}

	/**
	 * Obtm o identificador do projeto.
	 * 
	 * @return O identificador do projeto.
	 */
	public Object getProjectId() {
		return projectId;
	}

	/**
	 * Obtm o identificador do tipo de evento.
	 * 
	 * @return O identificador do tipo de evento.
	 */
	public Type getType() {
		return type;
	}

	/**
	 * Definio da volatilidade da notificao.
	 * 
	 * @return Flag indicativo da volatilidade da notificao.
	 */
	@Override
	public boolean isVolatile() {
		return true;
	}

	@Override
	public String toString() {
		return CommandPersistenceCommandNotificationData.class.getName() + "{projectId:"
				+ projectId + ", commandId:" + commandId + ", type:" + type
				+ "}";
	}
}
