package csbase.logic.algorithms.commands;

import java.io.Serializable;

import csbase.logic.algorithms.flows.configurator.FlowAlgorithmConfigurator;

/**
 * Comando de Fluxo de Algoritmos.
 */
public final class FlowCommand implements Serializable {

  /**
   * O configurador de fluxo de algoritmos.
   */
  private FlowAlgorithmConfigurator flowAlgorithmConfigurator;

  /**
   * Cria o comando de fluxo de algoritmos.
   * 
   * @param commandProperties As propriedades do comando
   *                          (no aceita {@code null}).
   * @param flowAlgorithmConfigurator O configurador de fluxo de algoritmos
   *                                  (no aceita {@code null}).
   */
  public FlowCommand(CommandProperties commandProperties,
    FlowAlgorithmConfigurator flowAlgorithmConfigurator) {
    setCommandProperties(commandProperties);
    setFlowAlgorithmConfigurator(flowAlgorithmConfigurator);
  }


  /**
   * Obtm o configurador de fluxo de algoritmos.
   * 
   * @return O configurador de fluxo de algoritmos.
   */
  public FlowAlgorithmConfigurator getFlowAlgorithmConfigurator() {
    return flowAlgorithmConfigurator;
  }

  /**
   * Atribui as propriedades de comando deste comando.
   * 
   * @param commandProperties As proprieades do comando 
   *                          (no aceita {@code null}).
   */
  private void setCommandProperties(CommandProperties commandProperties) {
    if (commandProperties == null) {
      throw new IllegalArgumentException(
        "O parmetro commandProperties est nulo.");
    }
  }

  /**
   * Atribui o configurador de fluxos de algoritmos a este comando.
   * 
   * @param flowAlgorithmConfigurator O configurador de fluxo de algoritmos
   *                                  (no aceita {@code null}).
   */
  private void setFlowAlgorithmConfigurator(
    FlowAlgorithmConfigurator flowAlgorithmConfigurator) {
    if (flowAlgorithmConfigurator == null) {
      throw new IllegalArgumentException(
        "O parmetro flowAlgorithmConfigurator est nulo.");
    }
    this.flowAlgorithmConfigurator = flowAlgorithmConfigurator;
  }
}
