package csbase.logic.algorithms.parameters;

import csbase.util.parser.FiniteAutomaton;
import csbase.util.parser.Parser;
import csbase.util.parser.State;
import csbase.util.parser.Token;
import csbase.util.parser.actions.AppendAction;
import csbase.util.parser.actions.DiscardAction;
import csbase.util.parser.actions.GenerateTokenAction;
import csbase.util.parser.exception.AutomatonException;
import csbase.util.parser.iterators.CharSymbolIterator;
import csbase.util.parser.symbols.CharSymbol;
import csbase.util.parser.symbols.Symbol;

import java.io.IOException;
import java.io.Reader;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

final class ListParameterParser extends Parser {
  static final char START_LINE_CHAR = '{';
  static final char ELEMENT_SEPARATOR_CHAR = ',';
  static final char END_LINE_CHAR = '}';

  ListParameterParser() {
    super(new ListParameterFiniteAutomaton());
  }

  List parseText(Reader reader)
  throws IOException, AutomatonException {
    StringBuilder buffer = new StringBuilder();
    int current;
    while((current = reader.read()) != -1) {
        buffer.append((char)current);
    }
    String text = buffer.toString();
    return parseText(text);
  }
  
  List parseText(String text)
  throws AutomatonException {
    List tokenList = parse(new CharSymbolIterator(text));
    return this.createStringList(tokenList);
  }

  private List<String> createStringList(List tokens) {
    List<String> stringList = new ArrayList<String>(tokens.size());
    Iterator tokenIterator = tokens.iterator();
    while (tokenIterator.hasNext()) {
      Token token = (Token)tokenIterator.next();
      List symbolList = token.getSymbolList();
      Iterator symbolListIterator = symbolList.iterator();
      StringBuffer buffer = new StringBuffer();
      while (symbolListIterator.hasNext()) {
        CharSymbol symbol = (CharSymbol)symbolListIterator.next();
        Character object = (Character)symbol.getObject();
        buffer.append(object);
      }
      stringList.add(buffer.toString());
    }
    return stringList;
  }

  private static class ListParameterFiniteAutomaton extends FiniteAutomaton {
    private static final Symbol START_LINE_SYMBOL =
      new CharSymbol(ListParameterParser.START_LINE_CHAR);
    private static final Symbol ELEMENT_SEPARATOR_SYMBOL =
      new CharSymbol(ListParameterParser.ELEMENT_SEPARATOR_CHAR);
    private static final Symbol END_LINE_SYMBOL =
      new CharSymbol(ListParameterParser.END_LINE_CHAR);

    private ListParameterFiniteAutomaton() {
      super(new State(false));
      State startLineState = getInitialState();
      State elementState = new State(false);
      State endLineState = new State(true);
      startLineState.addTransition(
        START_LINE_SYMBOL, DiscardAction.getInstance(), elementState);
      elementState.addTransition(
        ELEMENT_SEPARATOR_SYMBOL, GenerateTokenAction.getInstance(),
        elementState);
      elementState.addTransition(
        END_LINE_SYMBOL, DiscardAction.getInstance(), endLineState);
      elementState.setDefaultTransition(
        AppendAction.getInstance(), elementState);
    }
  }
}
