/*
 * $Author:$ 
 * $Date:$ 
 * $Release:$
 */
package csbase.logic.algorithms.parameters;

import java.io.Serializable;

import java.util.Map;

public abstract class Parameter<V> implements Comparable<Parameter<V>>, Serializable {
  private String name;

  protected Parameter(String name) {
    setName(name);
  }

  public abstract Map<String, Object> exportValue();

  public abstract String getCommandLine(String projectPath, String fileSeparator);

  public abstract void importValue(Map<String, Object> values);

  public abstract boolean isSetDefaultValue();

  public abstract boolean isVisible();

  public abstract void resetValue();

  public final int compareTo(Parameter<V> parameter) {
    return getName().compareTo(parameter.getName());
  }

  @Override
  public final boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (!getClass().equals(obj.getClass())) {
      return false;
    }
    Parameter parameter = (Parameter)obj;
    return getName().equals(parameter.getName());
  }

  public final String getName() {
    return this.name;
  }

  @Override
  public final int hashCode() {
    return getName().hashCode();
  }

  @Override
  public final String toString() {
    return getName();
  }

  private void setName(String name) {
    if (name == null) {
      throw new IllegalArgumentException("name == null");
    }
    this.name = name;
  }
}
