package csbase.logic.algorithms.parameters;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.text.MessageFormat;

import csbase.exception.algorithms.ExpressionFunctionExecutionException;

public final class ReflectionExpressionFunction extends ExpressionFunction {

  private Method method;

  public ReflectionExpressionFunction(Method method) {
    super(method.getName(), method.getParameterTypes());
    if (!Modifier.isPublic(method.getModifiers())) {
      String errorMessage = MessageFormat.format(
        "O mtodo {0} no  pblico.\n"
          + "Apenas mtodos pblicos podem ser utilizados.", method.getName());
      throw new IllegalArgumentException(errorMessage);
    }
    if (!Modifier.isStatic(method.getModifiers())) {
      String errorMessage = MessageFormat.format(
        "O mtodo {0} no  esttico.\n"
          + "Apenas mtodos estticos podem ser utilizados.", method.getName());
      throw new IllegalArgumentException(errorMessage);
    }
    this.method = method;
  }

  @Override
  protected Object doOperation(SimpleAlgorithmConfigurator configurator,
                               Object... parameters)
    throws ExpressionFunctionExecutionException {
    try {
      return this.method.invoke(null, parameters);
    } catch (IllegalAccessException e) {
      throw new ExpressionFunctionExecutionException(e);
    } catch (InvocationTargetException e) {
      throw new ExpressionFunctionExecutionException(e.getTargetException());
    }
  }
}
