package csbase.logic.algorithms.parameters;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

public final class RowValue implements Serializable {
  private List<Object> cellValues;
  private List<RowValueListener> listeners;
    
  public RowValue(Object... cellValues) {
    if (cellValues == null) {
    	throw new IllegalArgumentException(
    			"O parmetro cellValues est nulo.\n");
    }
    this.cellValues = new ArrayList<Object>(cellValues.length);
    for (Object cellValue : cellValues) {
      this.cellValues.add(cellValue);
    }
    listeners = new LinkedList<RowValueListener>();
  }

  public void addRowValueListener(RowValueListener listener) {
    if (listener == null) {
      throw new IllegalArgumentException("O parmetro listener est nulo.");
    }
    listeners.add(listener);
  }

  public Object getCellValue(int columnIndex) {
    return cellValues.get(columnIndex);
  }

  public boolean removeRowValueListener(RowValueListener listener) {
    if (listener == null) {
      throw new IllegalArgumentException("O parmetro listener est nulo.");
    }
    return listeners.remove(listener);
  }

  public void setCellValue(Object cellValue, int columnIndex) {
    cellValues.set(columnIndex, cellValue);
    for (RowValueListener listener : listeners) {
      listener.valueWasChanged(cellValue, columnIndex);
    }
  }

  public List<Object> getCellValues() {
    return Collections.unmodifiableList(cellValues);
  }
}
