/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.logic.algorithms.parameters;

import java.io.Serializable;
import java.util.Set;

import csbase.exception.ParseException;
import csbase.exception.algorithms.FormulaCreationException;
import csbase.exception.algorithms.FormulaEvaluationException;

public final class ValidationExpression implements Serializable {
  private String errorMessage;
  private Expression expression;
  private SimpleAlgorithmConfigurator configurator;

  public ValidationExpression(
    SimpleAlgorithmConfigurator configurator, String expressionText,
    String errorMessage)
  throws FormulaCreationException, ParseException {
    setConfigurator(configurator);
    setErrorMessage(errorMessage);
    this.expression = new Expression(expressionText);
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (!getClass().equals(obj.getClass())) {
      return false;
    }
    ValidationExpression expression = (ValidationExpression)obj;
    if (!getConfigurator().equals(expression.getConfigurator())) {
      return false;
    }
    return toString().equals(obj.toString());
  }

  public boolean evaluate()
  throws FormulaEvaluationException {
    return this.expression.evaluate(getConfigurator());
  }

  public SimpleAlgorithmConfigurator getConfigurator() {
    return this.configurator;
  }

  public String getErrorMessage() {
    return this.errorMessage;
  }

  public Set<SimpleParameter> getParameters() {
    return this.expression.getParameters();
  }

  @Override
  public int hashCode() {
    return toString().hashCode();
  }

  @Override
  public String toString() {
    return this.expression.toString();
  }

  private void setConfigurator(SimpleAlgorithmConfigurator version) {
    if (version == null) {
      throw new IllegalArgumentException(
        "O parmetro configurator est null.\n");
    }
    this.configurator = version;
  }

  private void setErrorMessage(String errorMessage) {
    if (errorMessage == null) {
      throw new IllegalArgumentException(
        "O parmetro expressionText est nulo.\n");
    }
    this.errorMessage = errorMessage;
  }
}
