/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.logic.algorithms.parameters.conditions;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

public abstract class CompositeOperatorCondition implements Condition {
  private List<Condition> conditions;

  protected CompositeOperatorCondition(Condition... conditions) {
    this(Arrays.asList(conditions));
  }
  
  protected CompositeOperatorCondition(List<Condition> conditions) {
    if (conditions == null) {
      throw new IllegalArgumentException("O parmetro conditions est nulo.");
    }
    if (conditions.size() < 2) {
      String errorMessage = String.format(
        "O nmero de condies tem que ser pelo menos 2, "
          + "porm o nmero atual : %d.\n", conditions.size());
      throw new IllegalArgumentException(errorMessage);
    }
    this.conditions = new ArrayList<Condition>(conditions);
  }

  @Override
  public final boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (!getClass().equals(obj.getClass())) {
      return false;
    }
    CompositeOperatorCondition condition = (CompositeOperatorCondition) obj;
    return conditions.equals(condition.conditions);
  }

  @Override
  public final int hashCode() {
    return conditions.hashCode();
  }

  protected final List<Condition> getConditions() {
    return Collections.unmodifiableList(conditions);
  }
}
