/*
 * $Author:$
 * $Date:$
 * $Release:$
 */
package csbase.logic.algorithms.parameters.conditions;

import csbase.logic.algorithms.parameters.SimpleAlgorithmConfigurator;
import csbase.logic.algorithms.parameters.SimpleParameter;

import java.text.MessageFormat;

public final class SimpleCondition implements Condition {
  private String parameterName;
  private Object value;

  public SimpleCondition(String parameterName, Object value) {
    setParameterName(parameterName);
    this.value = value;
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (!getClass().equals(obj.getClass())) {
      return false;
    }
    SimpleCondition condition = (SimpleCondition)obj;
    if (!getParameterName().equals(condition.getParameterName())) {
      return false;
    }
    if (getValue() == null && condition.getValue() == null) {
      return true;
    }
    if (getValue() != null && getValue().equals(condition.getValue())) {
      return true;
    }
    return false;
  }

  public boolean evaluate(SimpleAlgorithmConfigurator configurator) {
    if (configurator == null) {
      throw new IllegalArgumentException("O parmetro version est nulo.");
    }
    SimpleParameter<?> parameter =
      configurator.getSimpleParameter(getParameterName());
    if (parameter == null) {
      throw new IllegalStateException(
        MessageFormat.format(
          "O parmetro {1} do algoritmo {0} no foi encontrado.",
          getParameterName(), configurator));
    }
    Object currentValue = parameter.getValue();
    if (this.value == null && currentValue == null) {
      return true;
    }
    if (this.value != null && this.value.equals(currentValue)) {
      return true;
    }
    return false;
  }

  public String getParameterName() {
    return this.parameterName;
  }

  public Object getValue() {
    return this.value;
  }

  @Override
  public int hashCode() {
    return getParameterName().hashCode();
  }

  @Override
  public String toString() {
    String toString = "----------\n";
    toString += "Classe = "+getClass().getName()+"\n";
    toString += "Parametro = "+getParameterName()+"\n";
    toString += "Valor = "+getValue()+"\n";
    toString += "----------\n";
    return toString;
  }

  private void setParameterName(String parameterName) {
    if (parameterName == null) {
      throw new IllegalArgumentException(
        "O parmetro parameterName est nulo.");
    }
    this.parameterName = parameterName;
  }
}
