package csbase.logic.algorithms.parsers;

import csbase.exception.ParseException;
import csbase.logic.algorithms.parameters.InputFileListParameter;
import csbase.logic.algorithms.parameters.InputFileParameter;
import csbase.logic.algorithms.parameters.Parameter;

/**
 * 
 * <p>
 * Fbrica de parmetros do tipo "arquivo de entrada":
 * {@link InputFileParameter} e {@link InputFileListParameter}. Como esses dois
 * parmetros podem ser criados a partir de uma mesma tag XML, essa fbrica foi
 * criada para direcionar a anlise para o parser apropriado para cada um dos
 * casos.
 * </p>
 * 
 * <p>
 * O elemento corrente do {@link XmlParser analisador de XML} precisa ser um
 * elemento {@link InputFileParameterParser#INPUT_FILE_PARAMETER_ELEMENT}.
 * </p>
 * 
 */
public class InputFileParameterFactory implements ParameterFactory {

  /**
   * <p>
   * O atributo
   * {@value #INPUT_FILE_PARAMETER_ELEMENT_MULTIPLE_SELECTION_ATTRIBUTE} dos
   * elementos {@link InputFileParameter} e {@link InputFileListParameter}.
   * Indica se o parmetro aceita seleo mltipla. Caso positivo, o parmetro
   * deve ser instanciado como {@link InputFileListParameter}, caso contrrio,
   * ser um {@link InputFileParameter}.
   * </p>
   * 
   * Indica se o arquivo de entrada aceita seleo mltipla,  opcional, o seu
   * valor-padro 
   * {@link #INPUT_FILE_PARAMETER_ELEMENT_MULTIPLE_SELECTION_DEFAULT_VALUE} e 
   * do tipo booleano. </p>
   */
  protected static final String INPUT_FILE_PARAMETER_ELEMENT_MULTIPLE_SELECTION_ATTRIBUTE =
    "selecao_multipla";

  /**
   * <p>
   * O valor-padro para o atributo
   * {@link #INPUT_FILE_PARAMETER_ELEMENT_MULTIPLE_SELECTION_ATTRIBUTE} dos
   * elementos {@link InputFileParameter} e {@link InputFileListParameter}.
   * </p>
   * <p>
   * O seu valor 
   * {@value #INPUT_FILE_PARAMETER_ELEMENT_MULTIPLE_SELECTION_DEFAULT_VALUE}.
   * </p>
   */
  protected static final boolean INPUT_FILE_PARAMETER_ELEMENT_MULTIPLE_SELECTION_DEFAULT_VALUE =
    false;

  /**
   * {@inheritDoc}
   */
  @Override
  public Parameter<?> createParameter(XmlParser parser,
    String commandLinePattern) throws ParseException {
    boolean isMultipleSelection =
      parser.extractAttributeValueAsBoolean(
        INPUT_FILE_PARAMETER_ELEMENT_MULTIPLE_SELECTION_ATTRIBUTE,
        INPUT_FILE_PARAMETER_ELEMENT_MULTIPLE_SELECTION_DEFAULT_VALUE);
    AbstractFileParameterParser<?> inputFileParser = null;
    if (isMultipleSelection) {
      inputFileParser = new InputFileListParameterParser();
    }
    else {
      inputFileParser = new InputFileParameterParser();
    }
    return inputFileParser.createParameter(parser, commandLinePattern);
  }

}
