package csbase.logic.algorithms.parsers;

import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.configurationmanager.ConfigurationManagerException;
import csbase.exception.ConfigurationException;
import csbase.exception.ParseException;
import csbase.logic.algorithms.parameters.OutputFileParameter;
import csbase.logic.algorithms.parameters.Parameter;

/**
 * <p>
 * Analisador de {@link OutputFileParameter#createLogFile(String) arquivo de
 * log}.
 * </p>
 * 
 * <p>
 * Este parser l os atributos de parmetros do tipo "arquivo de log". O
 * elemento corrente do {@link XmlParser analisador de XML} precisa ser um
 * elemento que representa um arquivo de log.
 * </p>
 */
public class LogFileParameterFactory extends ParameterParser implements
  ParameterFactory {

  /**
   * Indica a visibilidade com a qual o elemento de log deve ser construdo. O
   * valor deste atributo ser resgatado a partir da propriedade
   * 'element.arquivo_de_log.visible' do arquivo de configuraes desta classe.
   * Caso o arquivo ou a propriedade no exista, o default  false.
   */
  private final boolean isLogElementVisible;

  /**
   * Construtor.
   */
  public LogFileParameterFactory() {
    Configuration configuration;
    try {
      configuration =
        ConfigurationManager.getInstance().getConfiguration(getClass());
    }
    catch (ConfigurationManagerException e) {
      throw new ConfigurationException(e,
        "Erro ao ler as configuraes do parser.");
    }

    if (null != configuration) {
      isLogElementVisible =
        configuration.getOptionalBooleanProperty("arquivo_de_log.visible",
          false);
    }
    else {
      isLogElementVisible = false;
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Parameter<?> createParameter(XmlParser parser,
    String defaultCommandLinePattern) throws ParseException {
    String commandLinePattern =
      loadCommandLinePattern(parser, defaultCommandLinePattern);
    OutputFileParameter parameter =
      OutputFileParameter.createLogFile(commandLinePattern);
    parameter.setVisible(isLogElementVisible);
    return parameter;
  }

}
