package csbase.logic.trustedserver;

import java.io.Serializable;

/**
 * Representa uma enumerao com os tipos de evento possveis gerados pelo
 * servio de servidores confiveis.
 *
 * @author Tecgraf/PUC-Rio
 */
public class TrustedServerEventType implements Serializable {
  /** O identificador do tipo adicionar. */
  private static final int ADD_ID = 0;

  /** O identificador do tipo modificar. */
  private static final int MODIFY_ID = 1;

  /** O identificador do tipo remover. */
  private static final int REMOVE_ID = 2;

  /**
   * O tipo adicionar.  gerado quando um servidor local  adicionado ao
   * repositrio de servidores locais.
   */
  public static final TrustedServerEventType ADD =
    new TrustedServerEventType(ADD_ID);

  /**
   * O tipo modificar.  gerado quando um servidor local  modificado no
   * repositrio de servidores locais.
   */
  public static final TrustedServerEventType MODIFY =
    new TrustedServerEventType(MODIFY_ID);

  /**
   * O tipo remover.  gerado quando um servidor local  removido do
   * repositrio de servidores locais.
   */
  public static final TrustedServerEventType REMOVE =
    new TrustedServerEventType(REMOVE_ID);

  /** O identificador do tipo de evento. */
  private Integer id;

  /**
   * Cria um tipo de evento de servidor confivel.
   *
   * @param id O identificador do tipo de evento.
   */
  private TrustedServerEventType(int id) {
    this.id = new Integer(id);
  }

  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (!this.getClass().equals(obj.getClass())) {
      return false;
    }
    TrustedServerEventType type = (TrustedServerEventType)obj;
    return this.id.equals(type.id);
  }

  public int hashCode() {
    return this.id.hashCode();
  }
}
