package csbase.logic.trustedserver;

import java.io.Serializable;

import csbase.logic.ServerURI;

/**
 * <p>
 * Representa um conjunto de informaes usadas na criao ou modificao de um
 * servidor confivel. 
 * </p>
 *
 * @author Tecgraf/PUC-Rio
 */
public final class TrustedServerInfo implements Serializable {
  /** O nome que ser usado na criao/modificao do servidor confivel. */
  private String name;

  /** Indica se o servidor local criado/modificado est suspenso. */
  private boolean suspended;

  /** Indica se esse servidor  tambm um servidor local */
  private boolean isLocal;
  
  /** URI do servidor */
  private ServerURI uri;

  /**
   * Cria um conjunto de informaes que ser usado na criao/modificao de
   * um servidor.
   *
   * @param name O nome do servidor
   * @param suspended Indica se o servidor est suspenso.
   * @param certificate O certificado do servidor
   * @param isLocal Indica se o servidor tambm  local
   * @param uri A URI para conexo com o servidor
   */
  public TrustedServerInfo(String name, boolean suspended, boolean isLocal, ServerURI uri) {
    this.name = name;
    this.suspended = suspended;
    this.isLocal = isLocal;
    this.uri = uri;
  }

  /**
   * Obtm o nome que ser utilizada para adicionar/modificar um servidor
   * 
   *
   * @return O nome do servidor 
   */
  public String getName() {
    return this.name;
  }

  /**
   * Verifica se o servidor que ser criado/modificado est suspenso ou
   * no.
   *
   * @return true, caso o servidor esteja suspenso, ou false, caso
   *         contrrio.
   */
  public boolean isSuspended() {
    return this.suspended;
  }

  /**
   * Informa se o servidor  local
   * 
   * @return true se representa um servidor local
   */
  public boolean isLocal() {
    return this.isLocal;
  }
  
  /**
   * Obtm a URI do servidor
   * 
   * @return A URI para do servidor
   */
  public ServerURI getURI() {
    return this.uri;
  }
}
