package csbase.logic.trustedserver;

/**
 * Representa um evento de modificao de um servidor confivel no repositrio de
 * servidores confiveis.
 *
 * @author Tecgraf/PUC-Rio
 */
public final class TrustedServerModifyEvent extends TrustedServerEvent {
  /** O servidor no estado anterior  modificao. */
  private TrustedServer oldTrustedServer;

  /** O servidor no estado resultante da modificao. */
  private TrustedServer newTrustedServer;

  /**
   * Cria um evento de modificao em um servidor local.
   *
   * @param source A origem do evento (onde o evento foi gerado).
   * @param oldTrustedServer O servidor local no estado anterior  modificao.
   * @param newTrustedServer O servidor local no estado resultante da
   *        modificao.
   */
  public TrustedServerModifyEvent(
    Object source, TrustedServer oldTrustedServer, TrustedServer newTrustedServer) {
    super(source, TrustedServerEventType.MODIFY);
    this.oldTrustedServer = oldTrustedServer;
    this.newTrustedServer = newTrustedServer;
  }

  /**
   * Obtm o servidor no estado anterior  modificao.
   *
   * @return O servidor no estado anterior  modificao.
   */
  public TrustedServer getOldTrustedServer() {
    return this.oldTrustedServer;
  }

  /**
   * Obtm o servidor no estado resultante da modificao.
   *
   * @return O servidor no estado resultante da modificao.
   */
  public TrustedServer getNewTrustedServer() {
    return this.newTrustedServer;
  }
}
