/*
 * $Id: DiskUsageServiceInterface.java 83625 2008-10-01 19:59:50Z clinio $
 */
package csbase.remote;

import java.rmi.RemoteException;
import java.util.Hashtable;

import csbase.exception.InvalidRequestException;
import csbase.logic.diskusageservice.DiskOccupation;

/**
 * Interface remota para acesso ao Servio de notificao.
 * 
 * @author Tecgraf/PUC-Rio
 */
public interface DiskUsageServiceInterface extends ServiceInterface {

  /**
   * Nome do servio para o <code>ServiceManager</code>
   */
  public static final String SERVICE_NAME = "DiskUsageService";

  /**
   * Consulta a lista de (usurios/espao ocupado) da rea de projetos
   * 
   * @return a hash.
   * @throws RemoteException se houver falha de comunicao.
   */
  public Hashtable<Object, Double> getUsersList() throws RemoteException;

  /**
   * Consulta o espao (em Mb) ocupado por um usurio.
   * 
   * @param userId id do usurio
   * @return o espao ocupado.
   * @throws RemoteException se houver falha de comunicao.
   */
  public double getUsedSpaceForUserMb(Object userId) throws RemoteException;

  /**
   * Consulta o espao (em Mb) ocupado por um projeto.
   * 
   * @param ownerId id do owner
   * @param projName id do projeto
   * @return o espao ocupado.
   * @throws RemoteException se houver falha de comunicao.
   * @throws InvalidRequestException caso algum parmetro seja nulo
   */
  public double getUsedSpaceForProjectMb(final Object ownerId,
    final String projName) throws RemoteException;

  /**
   * Consulta o espao da rea do servidor.
   * 
   * @return espao
   * @throws RemoteException falha de rmi
   */
  public DiskOccupation getServerOccupation() throws RemoteException;

  /**
   * Consulta o espao da rea de projetos.
   * 
   * @return espao
   * @throws RemoteException falha de rmi
   */
  public DiskOccupation getProjectOccupation() throws RemoteException;
}
