/*
 * MailServiceInterface.java
 * $Author: clinio $
 * $Revision: 36281 $  - $Date: 2005-12-02 11:22:57 -0200 (Fri, 02 Dec 2005) $
 */
package csbase.remote;

import java.rmi.RemoteException;
import java.util.Hashtable;

/**
 * Interface remota para acesso ao servio de e-mail.
 * @author Andr Luiz Clinio.
 */
public interface MailServiceInterface extends ServiceInterface {

  /** Nome do servio para o <code>ServiceManager</code> */
  public static final String SERVICE_NAME = "MailService";

  /**
   * Mtodo para envio de email para um usurio.
   * @return um flag indicativo de sucesso.
   */
  public boolean mailTo(final String senderName, final Object userId, 
  final String content) throws RemoteException;

  /**
   * Mtodo para envio de email para o suporte.
   * @return um flag indicativo de sucesso.
   */
  public boolean mailSupport(final String content) throws RemoteException;

  /**
   * Mtodo de envio de email.
   *
   * @param from o remetente
   * @param to o destinatrio
   * @param subject o assunto
   * @param content a string enviada
   *
   * @return flag indicativo de sucesso.
   */
  public boolean sendMail(final String senderName, final String to, 
  final String content) throws RemoteException;

  /**
   * Mtodo de envio de email com flexibilidade para customizar o assunto,
   * o campo de resposta (reply) e o campo de "copia para" (cc).
   *
   * @param senderName o remetente
   * @param to o destinatrio
   * @param content a string enviada
   * @param subject o assunto do email
   * @param replyTo o endereo que ir receber resposta (reply)
   * @param copyTo os endereos que iro receber uma cpia da mensagem
   *
   * @return flag indicativo de sucesso.
   */
  public boolean sendMail(final String senderName, final String to, 
  final String content, final String subject, final String replyTo, 
  final String[] copyTo) throws RemoteException; 
                                          
}
