/*
 * PublicationServiceInterface.java
 *
 * $Author: taciana $
 * $Revision: 72732 $  - $Date: 2008-02-15 10:15:26 -0200 (Fri, 15 Feb 2008) $
 */
package csbase.remote;

import java.rmi.RemoteException;
import java.util.Vector;

import csbase.logic.PublicationInfo;

/**
 * Interface remota para acesso ao servio de publicao de dados.
 */
public interface PublicationServiceInterface extends ServiceInterface {
  /** Nome do servio para o <code>ServiceManager</code> */
  public static final String SERVICE_NAME = "PublicationService";

  /**
   * Obtm todos as informaes de publicao de dados para um usurio.
   *
   * @param userId O identificador do usurio que deseja saber os dados de
   *        publicaes feitas para ele.
   *
   * @return Um vetor com <code>PublicationInfo</code>.
   * @throws RemoteException falha de rmi
   */
  public Vector<PublicationInfo> getAllPublicationInfo(Object userId)
  throws RemoteException;

  /**
   * Publica um arquivo.
   *
   * @param projectId Identificador do projeto.
   * @param path Caminho do arquivo a ser publicado.
   * @param userIds Lista contendo os identificadores dos usurios que tero
   *        acesso ao arquivo publicado.
   * @param whoPublishedId Identificador do usurio que publicou o arquivo.
   *
   * @return .
   * @throws RemoteException falha de rmi
   */
  boolean publishFile(
    Object projectId, String[] path, Object[] userIds, Object whoPublishedId)
    throws RemoteException;

  /**
   * Despublica um arquivo. Despublicar significa retirar todos os usurios da
   * lista de acesso ao arquivo publicado. Assim, quem tem link para ele
   * continua tendo, quem no tem, no consegue mais.
   *
   * @param id Identificador do projeto.
   * @param path Caminho do arquivo a ser despublicado.
   * @param userId Identificador do usurio que quer publicar o arquivo
   *
   * @return Verdadeiro em caso de sucesso e falso caso a operao falhe.
   */

/*
  boolean unPublishFile(Object id, String[] path, Object userId)
    throws RemoteException;
*/

  /**
   * Atualiza a lista de publicao do arquivo.
   * @param projectId identificador do projeto
   * @param path caminho para o arquivo
   * @param userIds lista de usurios que possuem acesso ao arquivo
   * @param userId Identificador do usurio que quer publicar o arquivo.
   *
   * @return Verdadeiro em caso de sucesso e falso caso a operao falhe.
   * @throws RemoteException falha de rmi
   */
  public boolean updatePublishAccess(
    Object projectId, String[] path, Object[] userIds, Object userId)
    throws RemoteException;

  /**
   * Retorna a lista de usurios para os quais um arquivo foi publicado. Se o
   * arquivo no estiver publicado ou no for encontrado, retorna null.
   *
   * @param projectId Identificador do projeto.
   * @param path Caminho do arquivo a ser despublicado.
   *
   * @return Um array com os identificadores dos usurios para os quais o
   *         arquivo foi publicado.
   * @throws RemoteException falha de rmi
   */
  public Object[] getPublishedFileAccess(Object projectId, String[] path)
    throws RemoteException;

  /**
   * Cria um link para um arquivo publicado. Dessa forma, o arquivo  publicado
   * pode ser manipulado na rea de projeto.
   *
   * @param projectId Identificador do projeto.
   * @param path Caminho do arquivo publicado.
   * @param info Informaes sobre a publicao a ser referenciada.
   * @throws RemoteException falha de rmi
   */
  public void linkToPublishedFile(
    Object projectId, String[] path, PublicationInfo info)
    throws RemoteException;
}
