package csbase.remote.niotransfer;

import java.io.Serializable;

/**
 * Classe que armazena informaes necessrias para uma solicitao de 
 * transferncia de arquivos entre cliente e servidor. Objetos dessa classe
 * so usados pela classe 
 * {@link csbase.client.ias.algorithm.niotransfer.FileClient} como entrada para
 * o protocolo de transferncia.
 *
 * @author Tecgraf
 */
public class RequestTransferInfo implements Serializable {
  /** Chave representando a transferncia. */
  private String key;
  /** Porta em que o socket de transferncias est escutando. */
  private int port;

  /**
   * Cria um objeto para armazenar informaes necessrias para a transferncia.
   * 
   * @param key chave representando a transferncia.
   * @param port porta em que o socket de transferncias est escutando.
   */
  public RequestTransferInfo(String key, int port) {
    this.key = key;
    this.port = port;
  }

  /**
   * Retorna a chave que representa a transferncia.
   * 
   * @return chave que representa a transferncia.
   */
  public String getKey() {
    return key;
  }
  
  /**
   * Retorna a porta em que o socket de transferncias est escutando.
   * 
   * @return porta em que o socket de transferncias est escutando.
   */
  public int getPort() {
    return port;
  }

}
