/*
 * $Id: ServerLevel.java 61414 2007-03-12 17:08:04Z lbarros $
 */
package csbase.server;

import java.util.logging.Level;


/**
 * Classe que estende a {@link Level} para acrescentar o nvel DEBUG, para
 * depurao.
 *
 * @author Tecgraf
 */
public class ServerLevel extends Level {

  /** 
   * DEBUG indica que a mensagem traz informao relevante somente para 
   * desenvolvedores, para depurao. Este nvel apresenta valor 450, o que
   * garante sua unicidade na hierarquia e o dispe imediatamente aps o FINE
   * e imediatamente antes do FINER, no sentido decrescente de hierarquia. Obs:
   * A escolha do valor 450 foi arbitrria, com a nica restrio de ser um
   * valor inferior a INFO na hierarquia.
   */
  public static final Level DEBUG = new ServerLevel("DEBUG", 450);
  
  /**
   * Permite a criao de novos nveis.
   * 
   * @param name nome para o novo nvel.
   * @param value valor para o novo nvel.
   */
  public ServerLevel(String name, int value) {
    super(name, value);
  }

  /**
   * Permite a criao de novos nveis, cujo nome seja internacionalizado.
   * 
   * @param name nome para o novo nvel.
   * @param value valor para o novo nvel.
   * @param resourceBundleName bundle que permite a internacionalizao do nome.
   */
  public ServerLevel(String name, int value, String resourceBundleName) {
    super(name, value, resourceBundleName);
  }
  
  /**
   * <p>
   * Garante a existncia de uma nica instncia do nvel DEBUG caso um objeto
   * desta classe seja desserializado.
   * <p>
   * <p>
   * A unicidade das demais instncias de {@code Level} pertencentes  classe
   * pai so garantidas pela implementao do mtodo {@code readResolve} na
   * mesma.
   * </p>
   * 
   * @return objeto desserializado.
   */
  protected Object readResolve() {
    boolean sameName = this.getName().equals(DEBUG.getName()); 
    boolean sameValue = this.intValue() == DEBUG.intValue();
    boolean sameResourceBundleName = 
      this.getResourceBundleName() == DEBUG.getResourceBundleName() || 
        ( this.getResourceBundleName() != null && 
          this.getResourceBundleName().equals(DEBUG.getResourceBundleName()) );
    if (sameName && sameValue && sameResourceBundleName) {
      return DEBUG;
    }
    return this;
  }
}
