/*
 * CentralAdministrationObserver.java
 * $Author: rodrigoh $
 * $Revision: 35859 $ - $Date: 2005-08-16 10:36:38 -0300 (Tue, 16 Aug 2005) $
 */
package csbase.server.services.administrationservice;

import csbase.logic.AdministrationEvent;
import csbase.logic.Permission;
import csbase.logic.Platform;
import csbase.logic.Role;
import csbase.logic.User;
import csbase.logic.UserGroup;
import csbase.remote.AdministrationServiceInterface;
import csbase.remote.RemoteObserver;
import csbase.remote.TransactionCallbackInterface;
import csbase.server.LocalServerRemoteMonitor;
import csbase.server.Service;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

/**
 * A classe <code>CentralAdministrationObserver</code> implementa um observador
 * remoto do servio central de administrao. Os eventos recebidos devem ser
 * refletidos no servio local. Por comodidade, ele tambm implementa a
 * interface TransactionCallbackInterface, para que possa ser utilizado como
 * 'callback' durante transaes com o servio central de administrao.
 */
class CentralAdministrationObserver extends UnicastRemoteObject
implements RemoteObserver, TransactionCallbackInterface {
  /** Servio de administrao local. */
  LocalAdministrationService service;

  /** Indica se o lock deve ou no ser mantido. */
  boolean lockMode;

  CentralAdministrationObserver(LocalAdministrationService service)
  throws RemoteException {
    super();
    this.service = service;
    lockMode = false;
  }

  void setLockMode(boolean lockMode) {
    this.lockMode = lockMode;
  }

  public boolean isAlive() {
    return lockMode;
  }

  public void install()
  throws RemoteException {
    LocalServerRemoteMonitor.getInstance().addObserver(
      AdministrationServiceInterface.SERVICE_NAME, this, null);
  }

  public void uninstall()
  throws RemoteException {
    LocalServerRemoteMonitor.getInstance().deleteObserver(
      AdministrationServiceInterface.SERVICE_NAME, this, null);
    unexportObject(this, true);
  }

  /**
   * Esse mtodo  chamado sempre que alguma alterao ocorre no servio de
   * administrao. Essa alterao deve ser informada ao servidor local.
   *
   * @param remoteEvent Descreve o evento ocorrido no servidor central.
   *
   * @throws RemoteException .
   */
  public void update(Object remoteEvent)
  throws RemoteException {
    try {
      // Registra que o usurio deste thread  o administrador
      Service.setUserId(User.getAdmin().getId());
      AdministrationEvent event = (AdministrationEvent)remoteEvent;
      if (event.item instanceof User) {
        service.updateUser(event);
      }
      else if (event.item instanceof UserGroup) {
        service.updateUserGroup(event);
      }
      else if (event.item instanceof Role) {
        service.updateRole(event);
      }
      else if (event.item instanceof Permission) {
        service.updatePermission(event);
      }
      else if (event.item instanceof Platform) {
        service.updatePlatform(event);
      }
      else {
        service.logSevereMessage(
          "Evento no reconhecido: "+event+" - item="+event.item);
      }
    }
    catch (Exception e) {
      String msg = "Falha na atualizao de usurio em servidor local.";
      service.logSevereMessage(msg, e);
      throw new RemoteException(msg, e);
    }
  }
}
