package csbase.server.services.algorithmservice.niotransfer;

import java.util.Map;

import csbase.remote.niotransfer.SimpleFileTransferProtocol.Operation;

/**
 * Classe representando as informaes necessrias para a execuo de uma
 * operao de leitura/escrita entre cliente e servidor.
 *
 * @author Tecgraf
 */
public class DoTransferInfo {
  
  /** Operao a ser executada. */
  private Operation operation;
  /** Caminho do arquivo no servidor. */
  private String path;
  /** Informaes a serem devolvidas ao servio quando do trmino da operao.*/
  private Map notifyArgs;

  /**
   * Cria um objeto contendo as informaes necessrias para a execuo da
   * operao.
   * 
   * @param operation operao a ser executada.
   * @param path caminho do arquivo no servidor.
   */
  public DoTransferInfo(Operation operation, String path) {
    this.operation = operation;
    this.path = path;
  }

  /**
   * Cria um objeto contendo as informaes necessrias para a execuo da
   * operao, alm de permitir que um servio seja notificado ao trmino.
   * 
   * @param operation operao a ser executada.
   * @param path caminho do arquivo no servidor.
   * @param notifyArgs informaes a serem devolvidas ao servio quando do 
   *        trmino da operao.
   */
  public DoTransferInfo(Operation operation, String path, Map notifyArgs) {
    this(operation, path);
    this.notifyArgs = notifyArgs;
  }

  /**
   * Retorna a operao.
   * 
   * @return operao.
   */
  public Operation getOperation() {
    return operation;
  }

  /**
   * Retorna o caminho do arquivo no servidor.
   * 
   * @return caminho do arquivo no servidor.
   */
  public String getPath() {
    return path;
  }

  /**
   * Retorna as informaes a serem devolvidas ao servio quando do trmino da 
   * operao.
   * 
   * @return informaes a serem repassadas ao servio.
   */
  public Map getNotifyArgs() {
    return notifyArgs;
  }
  
}
