package csbase.server.services.algorithmservice.niotransfer;

import java.util.Map;

/**
 * Interface para entidades (atualmente, somente servios) que desejam 
 * transferir arquivos entre o cliente e o servidor.
 *
 * @author Tecgraf
 */
public interface FileTransferrer {

  /**
   * Callback invocada pelo {@link FileServer} quando um canal com o cliente 
   * est para ser fechado, indicando que uma importao acabou de ser realizada 
   * (este mtodo no  invocado na exportao).
   * 
   * @param notifyArgs informaes acerca da importao recm-realizada. Estas
   *        informaes foram passadas pelo objeto que implementa esta interface
   *        no incio da importao, o {@link FileServer} apenas as devolve
   *        ao trmino da importao.
   */
  void channelClosing(Map notifyArgs);

  /**
   * Callback para erros ocorridos durante a transferncia, cujo log  de
   * responsabilidade da entidade que solicitou a transferncia. 
   * 
   * @param message mensagem descrevendo o erro.
   * @param error erro ocorrido.
   */
  void logSevereMessage(String message, Throwable error);

}
