package csbase.server.services.commandpersistenceservice;

import java.rmi.RemoteException;
import java.util.List;
import java.util.Set;

import csbase.exception.BugException;
import csbase.exception.CSBaseRuntimeException;
import csbase.exception.ServiceFailureException;
import csbase.logic.CommandInfo;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.remote.CommandPersistenceServiceInterface;
import csbase.remote.RemoteObservable;
import csbase.remote.RemoteObserver;
import csbase.server.Service;
import csbase.server.ServiceProxy;
import csbase.util.callbak.ICallback;

public final class CommandPersistenceServiceProxy extends ServiceProxy
implements CommandPersistenceServiceInterface, RemoteObservable {

  public CommandPersistenceServiceProxy(Object key) throws RemoteException {
    super(key);
  }

  public void addObserver(RemoteObserver observer, Object arg) {
    try {
      Service.setKey(this.key);
      CommandPersistenceService.getInstance().addObserver(observer, arg);
    } catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    } catch (CSBaseRuntimeException re) {
      throw re;
    } catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }

  }

  public int countObservers() {
    try {
      Service.setKey(this.key);
      return CommandPersistenceService.getInstance().countObservers();
    } catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    } catch (CSBaseRuntimeException re) {
      throw re;
    } catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }

  }

  public boolean deleteObserver(RemoteObserver observer, Object arg) {
    try {
      Service.setKey(this.key);
      return CommandPersistenceService.getInstance().deleteObserver(observer,
        arg);
    } catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    } catch (CSBaseRuntimeException re) {
      throw re;
    } catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  public void deleteObservers() {
    try {
      Service.setKey(this.key);
      CommandPersistenceService.getInstance().deleteObservers();
    } catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    } catch (CSBaseRuntimeException re) {
      throw re;
    } catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  public void getCommandInfos(Object projectId,
    long desiredResponseTime, final ICallback<Set<CommandInfo>> callback) {
    try {
      Service.setKey(key);
      CommandPersistenceService.getInstance().getCommandInfos(projectId,
        desiredResponseTime, callback);
    }
    catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    }
    catch (CSBaseRuntimeException re) {
      throw re;
    }
    catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  public Set<CommandInfo> getCommandInfos(Object projectId) {
    try {
      Service.setKey(key);
      return CommandPersistenceService.getInstance().getCommandInfos(
        projectId);
    } catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    } catch (CSBaseRuntimeException re) {
      throw re;
    } catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  public csbase.logic.CommandInfo getCommandInfo(
    Object projectId, String commandId) {
    try {
      Service.setKey(key);
      return CommandPersistenceService.getInstance().getCommandInfo(
        projectId, commandId);
    } catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    } catch (CSBaseRuntimeException re) {
      throw re;
    } catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  public boolean removeCommandInfo(Object projectId, String commandId) {
    try {
      Service.setKey(key);
      return CommandPersistenceService.getInstance().removeCommandInfo(
        projectId, commandId);
    } catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    } catch (CSBaseRuntimeException re) {
      throw re;
    } catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  public boolean[] removeCommandInfos(List<Object> projectIds,
    List<String> commandIds) {
    try {
      Service.setKey(key);
      return CommandPersistenceService.getInstance().removeCommandInfos(
        projectIds, commandIds);
    }
    catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    }
    catch (CSBaseRuntimeException re) {
      throw re;
    }
    catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  public boolean hasChanged() {
    try {
      Service.setKey(this.key);
      return CommandPersistenceService.getInstance().hasChanged();
    } catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    } catch (CSBaseRuntimeException re) {
      throw re;
    } catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  public boolean isAlive() {
    try {
      Service.setKey(key);
      return CommandPersistenceService.getInstance().isAlive();
    } catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    } catch (CSBaseRuntimeException re) {
      throw re;
    } catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  public void notifyObservers(Object event) {
    try {
      Service.setKey(this.key);
      CommandPersistenceService.getInstance().notifyObservers(event);
    } catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    } catch (CSBaseRuntimeException re) {
      throw re;
    } catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  public void updateCommandDescription(java.lang.Object param_0, java.lang.String param_1, java.lang.String param_2) {
    try {
      Service.setKey(key);
      CommandPersistenceService.getInstance().updateCommandDescription(param_0, param_1, param_2);
    } catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    } catch (CSBaseRuntimeException re) {
      throw re;
    } catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  /*
   * (non-Javadoc)
   *
   * @see csbase.remote.CommandPersistenceServiceInterface#readAlgorithmConfigurator(java.lang.Object,
   *      java.lang.String)
   */
  public AlgorithmConfigurator readAlgorithmConfigurator(Object projectId,
    String commandId) {
    try {
      Service.setKey(key);
      return CommandPersistenceService.getInstance().readAlgorithmConfigurator(
        projectId, commandId);
    }
    catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    }
    catch (CSBaseRuntimeException re) {
      throw re;
    }
    catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }
}
