package csbase.server.services.filetransferservice;

import java.rmi.RemoteException;
import java.util.List;

import csbase.exception.BugException;
import csbase.exception.CSBaseRuntimeException;
import csbase.exception.ServiceFailureException;
import csbase.logic.filetransferservice.FileTransferConnection;
import csbase.logic.filetransferservice.FileTransferElement;
import csbase.logic.filetransferservice.FileTransferPredefinedConnection;
import csbase.logic.filetransferservice.FileTransferRequest;
import csbase.remote.FileTransferServiceInterface;
import csbase.server.Service;

/**
 * @author Tecgraf/PUC-Rio
 */
public class FileTransferServiceProxy extends csbase.server.ServiceProxy
  implements FileTransferServiceInterface {

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean checkConnection(final FileTransferConnection connection)
    throws RemoteException {
    try {
      Service.setKey(this.key);
      return FileTransferService.getInstance().checkConnection(connection);
    }
    catch (final ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    }
    catch (final CSBaseRuntimeException re) {
      throw re;
    }
    catch (final Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void download(final FileTransferConnection connection,
    final String remoteFilePath, final String[] localFilePath,
    final long size) throws RemoteException {
    try {
      Service.setKey(this.key);
      FileTransferService.getInstance().download(connection, remoteFilePath,
        localFilePath, size);
    }
    catch (final ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    }
    catch (final CSBaseRuntimeException re) {
      throw re;
    }
    catch (final Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public List<FileTransferRequest> getAllRequests(final csbase.logic.User user)
    throws RemoteException {
    try {
      Service.setKey(this.key);
      return FileTransferService.getInstance().getAllRequests(user);
    }
    catch (final ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    }
    catch (final CSBaseRuntimeException re) {
      throw re;
    }
    catch (final Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void interruptRequest(final List<String> i) throws RemoteException {
    try {
      Service.setKey(this.key);
      FileTransferService.getInstance().interruptRequest(i);
    }
    catch (final ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    }
    catch (final CSBaseRuntimeException re) {
      throw re;
    }
    catch (final Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean isAlive() throws RemoteException {
    try {
      Service.setKey(this.key);
      return FileTransferService.getInstance().isAlive();
    }
    catch (final ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    }
    catch (final CSBaseRuntimeException re) {
      throw re;
    }
    catch (final Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  /**
   * {@inheritDoc}return true;
   */
  @Override
  public List<FileTransferElement> listContent(
    final FileTransferConnection connection, final String remoteDirPath)
    throws RemoteException {
    try {
      Service.setKey(this.key);
      return FileTransferService.getInstance().listContent(connection,
        remoteDirPath);
    }
    catch (final ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    }
    catch (final CSBaseRuntimeException re) {
      throw re;
    }
    catch (final Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void removeRequest(final List<String> i) throws RemoteException {
    try {
      Service.setKey(this.key);
      FileTransferService.getInstance().removeRequest(i);
    }
    catch (final ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    }
    catch (final CSBaseRuntimeException re) {
      throw re;
    }
    catch (final Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void upload(final FileTransferConnection connection,
    final String remoteFilePath, final String[] localFilePath, final long size)
    throws RemoteException {
    try {
      Service.setKey(this.key);
      FileTransferService.getInstance().upload(connection, remoteFilePath,
        localFilePath, size);
    }
    catch (final ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    }
    catch (final CSBaseRuntimeException re) {
      throw re;
    }
    catch (final Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  /**
   * Construtor
   * 
   * @param param_0 bla
   * @throws java.rmi.RemoteException bla
   */
  public FileTransferServiceProxy(final Object param_0) throws RemoteException {
    super(param_0);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public List<FileTransferPredefinedConnection> getPredefinedConnections() throws RemoteException {
    try {
      Service.setKey(this.key);
      return FileTransferService.getInstance().getPredefinedConnections();
    }
    catch (final ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    }
    catch (final CSBaseRuntimeException re) {
      throw re;
    }
    catch (final Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }

  }

}