/*
 * $Id:$
 */

package csbase.server.services.filetransferservice.ftp;

import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;

import org.apache.commons.net.ftp.FTPClient;

import csbase.exception.ServiceFailureException;
import csbase.logic.ProjectFileType;
import csbase.logic.filetransferservice.FileTransferConnection;
import csbase.logic.filetransferservice.FileTransferRequest;
import csbase.logic.filetransferservice.FileTransferRequestStatus;
import csbase.server.services.filetransferservice.FileTransferJob;
import csbase.server.services.filetransferservice.FileTransferService;
import csbase.server.services.projectservice.ServerProject;
import csbase.server.services.projectservice.ServerProjectFile;

/**
 * Job de download no protocolo FTP.
 * 
 * @author Tecgraf/PUC-Rio
 */
class FTPDownloadJob extends FileTransferJob {

  /**
   * Construtor
   * 
   * @param request requisio.
   */
  FTPDownloadJob(final FileTransferRequest request) {
    super(request);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  final protected void transfer() throws Exception {

    final FileTransferRequest request = getRequest();
    final FileTransferConnection connection = request.getConnection();
    final Object projectId = connection.getProjectId();
    final ServerProject project = ServerProject.getProject(projectId);
    final String[] localPath = request.getLocalFilePath();
    final String remoteFilePath = request.getRemoteFilePath();
    final String password = connection.getPassword();
    final String userName = connection.getUserName();
    final String serverName = connection.getServerName();
    final Object userId = connection.getUserId();

    if (!project.exists(localPath)) {
      final String fileType =
        ProjectFileType.getFileType(ProjectFileType.UNKNOWN).getCode();
      project.createFile(localPath, fileType, userId, false);
    }
    final String[] fullPath = project.makeFullPath(localPath);
    final ServerProjectFile spf =
      ServerProjectFile.findFile(projectId, fullPath);
    spf.setUnderConstruction(true);

    final FTPClient ftp = new FTPClient();
    InputStream inStream = null;
    OutputStream outStream = null;

    try {
      ftp.connect(serverName);
      final boolean loginOk = ftp.login(userName, password);
      if (!loginOk) {
        throw new ServiceFailureException("bad login");
      }

      inStream = ftp.retrieveFileStream(remoteFilePath);
      if(inStream == null) {
        final String err = "null download input stream for: " + remoteFilePath;
        throw new ServiceFailureException(err);
      }
      outStream = new FileOutputStream(spf.getRealFile());

      final int chunkSize = FileTransferService.CHUNK_SIZE;
      byte[] b = new byte[chunkSize];
      int bytesRead = 0;
      while ((bytesRead = inStream.read(b)) != -1) {
        outStream.write(b, 0, bytesRead);
        request.addTransferredSize(bytesRead);
        if (request.getStatus() == FileTransferRequestStatus.INTERRUPTED) {
          return;
        }
      }
    }
    catch (Exception e) {
      e.printStackTrace();
      try {
        if (ftp.isConnected()) {
          ftp.disconnect();
        }
      }
      catch (Exception e2) {
        // No faz nada, pois a falha foi justamente no disconnect.
      }
      throw new ServiceFailureException(e.getMessage());
    }
    finally {
      if (outStream != null) {
        outStream.close();
      }
      if (inStream != null) {
        inStream.close();
      }
    }
    spf.setUnderConstruction(false);
  }
}

//final FileTransferRequest request = getRequest();
//final FileTransferConnection connection = request.getConnection();
//final Object projectId = connection.getProjectId();
//final ServerProject project = ServerProject.getProject(projectId);
//final String[] localPath = request.getLocalFilePath();
//final Object userId = connection.getUserId();
//final String rfp = request.getRemoteFilePath();
//final String password = connection.getPassword();
//final String userName = connection.getUserName();
//final String serverName = connection.getServerName();
//if (!project.exists(localPath)) {
//  final String fileType =
//    ProjectFileType.getFileType(ProjectFileType.UNKNOWN).getCode();
//  project.createFile(localPath, fileType, userId, false);
//}
//
//final String[] fullPath = project.makeFullPath(localPath);
//final ServerProjectFile spf =
//  ServerProjectFile.findFile(projectId, fullPath);
//spf.setUnderConstruction(true);
//
//final String remoteFilePath = rfp;
////    while (remoteFilePath.startsWith("/")) {
////      remoteFilePath = remoteFilePath.substring(1);
////    }
//final String pwd = FTPUtils.encode(password);
//final String efp = FTPUtils.encodeFilePath(remoteFilePath);
//final String rpath =
//  "ftp://" + userName + ":" + pwd + "@" + serverName + "/" + efp;
//
//BufferedInputStream in = null;
//BufferedOutputStream out = null;
//try {
//  final URL url = new URL(rpath);
//  final URLConnection conn = url.openConnection();
//  final InputStream connStream = conn.getInputStream();
//  in = new BufferedInputStream(connStream);
//  final int chunkSize = 16 * 1024;
//  byte[] b = new byte[chunkSize];
//  final FileOutputStream fileStream =
//    new FileOutputStream(spf.getRealFile());
//  out = new BufferedOutputStream(fileStream);
//  int bytesRead = 0;
//  while ((bytesRead = in.read(b)) != -1) {
//    out.write(b, 0, bytesRead);
//    request.addTransferredSize(bytesRead);
//    final FileTransferRequestStatus status = request.getStatus();
//    if (status == FileTransferRequestStatus.INTERRUPTED) {
//      return;
//    }
//  }
//  out.flush();
//  out.close();
//  in.close();
//}
//catch (Exception e) {
//  spf.remove();
//  throw e;
//}
//finally {
//  if (out != null) {
//    out.close();
//  }
//  if (in != null) {
//    in.close();
//  }
//}
//spf.setUnderConstruction(false);
//}
