/*
 * $Id: SFTPUploadJob.java 96878 2009-09-30 17:50:41Z clinio $
 */

package csbase.server.services.filetransferservice.sftp;

import java.io.File;
import java.io.FileInputStream;
import java.io.OutputStream;

import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;
import com.jcraft.jsch.SftpProgressMonitor;
import com.jcraft.jsch.UserInfo;

import csbase.exception.ServiceFailureException;
import csbase.logic.filetransferservice.FileTransferConnection;
import csbase.logic.filetransferservice.FileTransferRequest;
import csbase.logic.filetransferservice.FileTransferRequestStatus;
import csbase.server.services.filetransferservice.FileTransferJob;
import csbase.server.services.filetransferservice.FileTransferService;
import csbase.server.services.projectservice.ServerProject;
import csbase.server.services.projectservice.ServerProjectFile;

/**
 * Job de download no protocolo FTP.
 * 
 * @author Tecgraf/PUC-Rio
 */
class SFTPUploadJob extends FileTransferJob {

  /**
   * {@inheritDoc}
   */
  @Override
  final protected void transfer() throws Exception {
    
    
    final FileTransferRequest request = getRequest();
    final FileTransferConnection connection = request.getConnection();
    final String remoteFilePath = request.getRemoteFilePath();
    final String serverName = connection.getServerName();
    final String userName = connection.getUserName();
    final String password = connection.getPassword();

    JSch jsch = new JSch();
    final int port = 22;
    final Session session;
    try {
      session = jsch.getSession(userName, serverName, port);
    }
    catch (JSchException e) {
      throw new ServiceFailureException(e.getMessage());
    }

    final UserInfo ui = new StubUserInfo(password);
    session.setUserInfo(ui);


    final OutputStream outputStream;
    try {
      session.connect();
      final Channel c = session.openChannel("sftp");
      c.connect();
      final ChannelSftp channel = (ChannelSftp) c;
      final String[] splittedPath = remoteFilePath.split(File.separator);
      final String file = splittedPath[splittedPath.length-1];
      final String dir = remoteFilePath.substring(0, remoteFilePath.lastIndexOf(File.separator));

      channel.cd(dir);
      outputStream = channel.put(file, (SftpProgressMonitor)null, 0);
    }
    catch (JSchException e) {
      throw new ServiceFailureException(e.getMessage());
    }
    
    
    final Object projectId = connection.getProjectId();
    final ServerProject project = ServerProject.getProject(projectId);
    final String[] localPath = request.getLocalFilePath();

    final String[] fullPath = project.makeFullPath(localPath);
    final ServerProjectFile spf =
      ServerProjectFile.findFile(projectId, fullPath);

    FileInputStream inputStream = null;
    try {
      final File realFile = spf.getRealFile();
      inputStream = new FileInputStream(realFile);
    }
    catch (Exception e) {
      throw new ServiceFailureException(e.getMessage());
    }

    try {
      final int chunkSize = FileTransferService.CHUNK_SIZE;
      byte[] b = new byte[chunkSize];
      int bytesRead = 0;
      while ((bytesRead = inputStream.read(b)) != -1) {
        outputStream.write(b, 0, bytesRead);
        request.addTransferredSize(bytesRead);
        if (request.getStatus() == FileTransferRequestStatus.INTERRUPTED) {
          return;
        }
      }
      outputStream.flush();
    }
    catch (final Exception e) {
      spf.remove();
      throw e;
    }
    finally {
      outputStream.close();
    }

  }

  /**
   * Construtor
   * 
   * @param request requisio.
   */
  public SFTPUploadJob(final FileTransferRequest request) {
    super(request);
  }

}
