/*
 * SGAHandler.java
 *
 * $Author: ururahy $
 * $Revision: 67318 $ - $Date: 2007-09-10 17:03:59 -0300 (Mon, 10 Sep 2007) $
 */
package csbase.server.services.sgaservice;

import org.omg.CORBA.IntHolder;

import sgaidl.*;

/**
 * A classe <code>SGAHandler</code> implementa o <i>servant</i> associado 
 * interface idl <code>SGAManager</code>, atravs da qual os SGAs interagem
 * com o servio de gerncia de SGAs. As requisies recebidas dos SGAs so
 * repassadas pelo SGAHandler  implementao do servio de gerncia de SGAs
 * (SGAService).
 */
public class SGAHandler extends SGAManagerPOA {
  /** Referncia para a implementao do servio SGAService */
  private SGAService service = null;

  /**
   * SGA avisa ao Manager que est ativo.
   *
   * @param sga referncia para o SGA
   * @param sganame nome do SGA
   * @param staticNodesInfo informaes estticas dos ns
   * @param updateInterval informa de quanto em quanto tempo o SGA
   *        deve dizer que est vivo.
   *
   * @return true se o SGA foi registrado com sucesso junto ao SSI
   */
  public boolean registerSGA(SGAServer sga, String sganame,
    StaticNodeInfo[] staticNodesInfo, IntHolder updateInterval) {
    return service.registerSGA(sga, sganame, staticNodesInfo, updateInterval);
  }

  /**
   * SGA avisa ao Manager que sair do ar
   *
   * @param sga referncia para o SGA
   * @param sganame nome do SGA
   */

  public void unregisterSGA(SGAServer sga, String sganame) {
    service.unregisterSGA(sga, sganame);
  }

  /**
   * SGA avisa ao Manager que est apto a receber novos comandos
   *
   * @param sga referncia para o SGA
   * @param sganame nome do SGA
   */
  public void setSGAEnabled(SGAServer sga, String sganame) {
    service.setSGAEnabled(sganame);
  }

  /**
   * SGA avisa ao Manager que no est apto a receber novos comandos, apesar de
   * continuar ativo.
   *
   * @param sga referncia para o SGA
   * @param sganame nome do SGA
   */
  public void setSGADisabled(SGAServer sga, String sganame) {
    service.setSGADisabled(sganame);
  }

  /**
   * SGA avisa ao Manager que um comando terminou
   *
   * @param sgaName .
   * @param cmd referncia para o gerenciador do comando
   * @param cmdId .
   * @param info informaes sobre o trmino de execuo do comando
   * @param cmdString .
   *
   * @return .
   */
  public boolean commandCompleted(
    String sgaName, SGACommand cmd, String cmdId,
    CompletedCommandInfo info, String cmdString) {
    return service.commandCompleted(sgaName, cmd, cmdId, info);
  }

  /**
   * SGA avisa ao Manager que um comando no foi recuperado
   *
   * @param sgaName .
   * @param cmdId .
   * @param cmdString .
   *
   * @return .
   */
  public boolean commandLost(
    String sgaName, String cmdId, String cmdString) {
    return service.commandLost(sgaName, cmdId);
  }

  /**
   * SGA avisa ao Manager que um comando foi recuperado
   *
   * @param sgaName .
   *
   * @return .
   */
  public boolean commandRetrieved(
    String sgaName, RetrievedInfo[] cmds) {
    return service.commandRetrieved(sgaName, cmds);
  }

  /**
   * SGA verifica se o Manager est ativo
   *
   * @param sga referncia para o SGA
   * @param sganame nome do SGA
   *
   * @return true
   */

  public boolean isRegistered(SGAServer sga, String sganame) {
    return service.isRegistered(sga, sganame);
  }

  /**
   * SGA verifica se o Manager est ativo
   *
   * @param sga referncia para o SGA
   * @param sganame nome do SGA
   * @param dynamicNodesInfo informaes dinmicas dos ns
   *
   * @return true
   */

  public boolean updateSGAInfo(SGAServer sga, String sganame,
    SGADynamicInfo dynamicInfo) {
    return service.updateSGAInfo(sga, sganame, dynamicInfo);
  }

  /**
   * Constri um servant para a interface SGAManager
   *
   * @param srv referncia para a implementao do servio SGAService
   */
  public SGAHandler(SGAService srv) {
    service = srv;
  }
}
