package csbase.server.services.trustedserverservice;

import csbase.exception.BugException;
import csbase.exception.CSBaseRuntimeException;
import csbase.exception.ServiceFailureException;
import csbase.logic.trustedserver.TrustedServerInfo;
import csbase.logic.trustedserver.TrustedServer;
import csbase.remote.TrustedServerServiceInterface;
import csbase.remote.RemoteObserver;
import csbase.server.Service;
import csbase.server.ServiceProxy;

import java.rmi.RemoteException;
import java.security.cert.Certificate;

import java.util.SortedSet;

public class TrustedServerServiceProxy extends ServiceProxy
implements TrustedServerServiceInterface {
  /**
   * Constri um proxy para o servio de servidores locais.
   *
   * @param key A chave do usurio que est requisitando o proxy.
   *
   * @throws RemoteException Caso ocorra algum problema na comunicao com o
   *         servidor.
   */
  public TrustedServerServiceProxy(Object key)
  throws RemoteException {
    super(key);
  }

  public SortedSet<TrustedServer> getAllTrustedServers() {
    try {
      Service.setKey(this.key);
      return TrustedServerService.getInstance().getAllTrustedServers();
    }
    catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getMessage());
    }
    catch (CSBaseRuntimeException re) {
      throw re;
    }
    catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  public boolean addTrustedServer(TrustedServerInfo localServerInfo)
  throws RemoteException {
    try {
      Service.setKey(this.key);
      return TrustedServerService.getInstance().addTrustedServer(localServerInfo);
    }
    catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getMessage());
    }
    catch (CSBaseRuntimeException re) {
      throw re;
    }
    catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  public void removeTrustedServer(TrustedServer localServer)
  throws RemoteException {
    try {
      Service.setKey(this.key);
      TrustedServerService.getInstance().removeTrustedServer(localServer);
    }
    catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getMessage());
    }
    catch (CSBaseRuntimeException re) {
      throw re;
    }
    catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  public boolean modifyTrustedServer(
    TrustedServer localServer, TrustedServerInfo localServerInfo)
  throws RemoteException {
    try {
      Service.setKey(this.key);
      return TrustedServerService.getInstance().modifyTrustedServer(
        localServer, localServerInfo);
    }
    catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getMessage());
    }
    catch (CSBaseRuntimeException re) {
      throw re;
    }
    catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  public void addObserver(RemoteObserver observer, Object arg) {
    try {
      Service.setKey(this.key);
      TrustedServerService.getInstance().addObserver(observer, arg);
    }
    catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getMessage());
    }
    catch (CSBaseRuntimeException re) {
      throw re;
    }
    catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  public boolean deleteObserver(RemoteObserver observer, Object arg) {
    try {
      Service.setKey(this.key);
      return TrustedServerService.getInstance().deleteObserver(observer, arg);
    }
    catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getMessage());
    }
    catch (CSBaseRuntimeException re) {
      throw re;
    }
    catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  public void notifyObservers(Object event) {
    try {
      Service.setKey(this.key);
      TrustedServerService.getInstance().notifyObservers(event);
    }
    catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getMessage());
    }
    catch (CSBaseRuntimeException re) {
      throw re;
    }
    catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  public void deleteObservers() {
    try {
      Service.setKey(this.key);
      TrustedServerService.getInstance().deleteObservers();
    }
    catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getMessage());
    }
    catch (CSBaseRuntimeException re) {
      throw re;
    }
    catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  public boolean hasChanged() {
    try {
      Service.setKey(this.key);
      return TrustedServerService.getInstance().hasChanged();
    }
    catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getMessage());
    }
    catch (CSBaseRuntimeException re) {
      throw re;
    }
    catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  public int countObservers() {
    try {
      Service.setKey(this.key);
      return TrustedServerService.getInstance().countObservers();
    }
    catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getMessage());
    }
    catch (CSBaseRuntimeException re) {
      throw re;
    }
    catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  public boolean isAlive() throws RemoteException {
    Service.setKey(this.key);
    return TrustedServerService.getInstance().isAlive();
  }

  public boolean addCert(String serverName, Certificate c)
    throws RemoteException {
    try {
      Service.setKey(this.key);
      return TrustedServerService.getInstance().addCert(serverName, c);
    }
    catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getMessage());
    }
    catch (CSBaseRuntimeException re) {
      throw re;
    }
    catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  public void removeCert(String serverName) throws RemoteException {
    try {
      Service.setKey(this.key);
      TrustedServerService.getInstance().removeCert(serverName);
    }
    catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getMessage());
    }
    catch (CSBaseRuntimeException re) {
      throw re;
    }
    catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
}
}
