/*
 * $Author:$
 * $Date:$
 * $Release:$
 */
package csbase.client.algorithms.parameters.validators;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Locale;

import tecgraf.javautils.LNG;

import csbase.client.algorithms.parameters.validators.ValidationResult;
import csbase.logic.algorithms.parameters.DoubleParameter;

import junit.framework.TestCase;

public final class DoubleParameterValidatorTest extends TestCase {
  static {
    LNG.load("csbase.client.resources.properties.language.idiom", new Locale(
      "pt", "BR"));
  }
  
  public void testDoubleParameterValidatorWithOptionalTrue() {
    boolean isOptional = true;
    Double maximum = new Double(5.5);
    boolean isMaximumIncluded = true;
    Double minimum = new Double(1.1);
    boolean isMinimumIncluded = true;
    DoubleParameterValidator validator =
      new DoubleParameterValidator(
        isOptional, maximum, isMaximumIncluded, minimum, isMinimumIncluded);
    assertEquals(isOptional, validator.isOptional());
    assertEquals(maximum, validator.getMaximum());
    assertEquals(isMaximumIncluded, validator.isMaximumIncluded());
    assertEquals(minimum, validator.getMinimum());
    assertEquals(isMinimumIncluded, validator.isMinimumIncluded());
  }

  public void testDoubleParameterValidatorWithOptionalFalse() {
    boolean isOptional = false;
    Double maximum = new Double(5.5);
    boolean isMaximumIncluded = true;
    Double minimum = new Double(1.1);
    boolean isMinimumIncluded = true;
    DoubleParameterValidator validator =
      new DoubleParameterValidator(
        isOptional, maximum, isMaximumIncluded, minimum, isMinimumIncluded);
    assertEquals(isOptional, validator.isOptional());
    assertEquals(maximum, validator.getMaximum());
    assertEquals(isMaximumIncluded, validator.isMaximumIncluded());
    assertEquals(minimum, validator.getMinimum());
    assertEquals(isMinimumIncluded, validator.isMinimumIncluded());
  }

  public void testDoubleParameterValidatorWithNullMaximum() {
    boolean isOptional = true;
    boolean isMaximumIncluded = true;
    Double minimum = new Double(1.1);
    boolean isMinimumIncluded = true;
    DoubleParameterValidator validator =
      new DoubleParameterValidator(
        isOptional, null, isMaximumIncluded, minimum, isMinimumIncluded);
    assertEquals(isOptional, validator.isOptional());
    assertNull(validator.getMaximum());
    assertFalse(validator.isMaximumIncluded());
    assertEquals(minimum, validator.getMinimum());
    assertEquals(isMinimumIncluded, validator.isMinimumIncluded());
  }

  public void testDoubleParameterValidatorWithNullMinimum() {
    boolean isOptional = true;
    Double maximum = new Double(5.5);
    boolean isMaximumIncluded = true;
    boolean isMinimumIncluded = true;
    DoubleParameterValidator validator =
      new DoubleParameterValidator(
        isOptional, maximum, isMaximumIncluded, null, isMinimumIncluded);
    assertEquals(isOptional, validator.isOptional());
    assertEquals(maximum, validator.getMaximum());
    assertEquals(isMaximumIncluded, validator.isMaximumIncluded());
    assertNull(validator.getMinimum());
    assertFalse(validator.isMinimumIncluded());
  }

  public void testDoubleParameterValidatorWithIsMaximumIncludedFalse() {
    boolean isOptional = true;
    Double maximum = new Double(5.5);
    boolean isMaximumIncluded = false;
    Double minimum = new Double(1.1);
    boolean isMinimumIncluded = true;
    DoubleParameterValidator validator =
      new DoubleParameterValidator(
        isOptional, maximum, isMaximumIncluded, minimum, isMinimumIncluded);
    assertEquals(isOptional, validator.isOptional());
    assertEquals(maximum, validator.getMaximum());
    assertEquals(isMaximumIncluded, validator.isMaximumIncluded());
    assertEquals(minimum, validator.getMinimum());
    assertEquals(isMinimumIncluded, validator.isMinimumIncluded());
  }

  public void testDoubleParameterValidatorWithIsMinimumIncludedFalse() {
    boolean isOptional = true;
    Double maximum = new Double(5.5);
    boolean isMaximumIncluded = true;
    Double minimum = new Double(1.1);
    boolean isMinimumIncluded = false;
    DoubleParameterValidator validator =
      new DoubleParameterValidator(
        isOptional, maximum, isMaximumIncluded, minimum, isMinimumIncluded);
    assertEquals(isOptional, validator.isOptional());
    assertEquals(maximum, validator.getMaximum());
    assertEquals(isMaximumIncluded, validator.isMaximumIncluded());
    assertEquals(minimum, validator.getMinimum());
    assertEquals(isMinimumIncluded, validator.isMinimumIncluded());
  }

  public void testDoubleParameterValidatorWithMinimumLargerThanMaximum() {
    boolean isOptional = true;
    Double maximum = new Double(5.5);
    boolean isMaximumIncluded = true;
    Double minimum = new Double(11.11);
    boolean isMinimumIncluded = true;
    try {
      new DoubleParameterValidator(
        isOptional, maximum, isMaximumIncluded, minimum, isMinimumIncluded);
      fail(
        "No deveria ser possvel criar um validador com mnimo maior "+
        "do que o mximo.");
    }
    catch (IllegalArgumentException e) {}
  }

  public void testValidateValueWithTrueIsOptionalAndNullValue() {
    boolean isOptional = true;
    DoubleParameterValidator validator =
      new DoubleParameterValidator(isOptional, null, false, null, false);
    DoubleParameter parameter =
      new DoubleParameter(
        "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
        null, isOptional, "{0} {1}", null, false, null, false);
    ValidationResult result = validator.validateValue(parameter, null);
    assertNotNull(result);
    assertTrue(result.isWellSucceded());
    assertNull(result.getErrorMessage());
  }

  public void testValidateValueWithFalseIsOptionalAndNullValue() {
    boolean isOptional = false;
    DoubleParameterValidator validator =
      new DoubleParameterValidator(isOptional, null, false, null, false);
    DoubleParameter parameter =
      new DoubleParameter(
        "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
        null, isOptional, "{0} {1}", null, false, null, false);
    ValidationResult result = validator.validateValue(parameter, null);
    assertNotNull(result);
    assertFalse(result.isWellSucceded());
    assertNotNull(result.getErrorMessage());
  }

  public void testValidateValueWithValueEqualsMaximumWithTrueIsMaximumIncluded() {
    boolean isMaximumIncluded = true;
    Double maximum = new Double(10);
    DoubleParameterValidator validator =
      new DoubleParameterValidator(
        false, maximum, isMaximumIncluded, null, false);
    DoubleParameter parameter =
      new DoubleParameter(
        "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
        null, false, "{0} {1}", maximum, isMaximumIncluded, null, false);
    ValidationResult result = validator.validateValue(parameter, maximum);
    assertNotNull(result);
    assertTrue(result.isWellSucceded());
    assertNull(result.getErrorMessage());
  }

  public void testValidateValueWithValueEqualsMaximumWithFalseIsMaximumIncluded() {
    boolean isMaximumIncluded = false;
    Double maximum = new Double(10);
    DoubleParameterValidator validator =
      new DoubleParameterValidator(
        false, maximum, isMaximumIncluded, null, false);
    DoubleParameter parameter =
      new DoubleParameter(
        "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
        null, false, "{0} {1}", maximum, isMaximumIncluded, null, false);
    ValidationResult result = validator.validateValue(parameter, maximum);
    assertNotNull(result);
    assertFalse(result.isWellSucceded());
    assertNotNull(result.getErrorMessage());
  }

  public void testValidateValueWithValueEqualsMinimumWithTrueIsMinimumIncluded() {
    boolean isMinimumIncluded = true;
    Double minimum = new Double(10);
    DoubleParameterValidator validator =
      new DoubleParameterValidator(
        false, null, false, minimum, isMinimumIncluded);
    DoubleParameter parameter =
      new DoubleParameter(
        "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
        null, false, "{0} {1}", null, false, minimum, isMinimumIncluded);
    ValidationResult result = validator.validateValue(parameter, minimum);
    assertNotNull(result);
    assertTrue(result.isWellSucceded());
    assertNull(result.getErrorMessage());
  }

  public void testValidateValueWithValueEqualsMinimumWithFalseIsMinimumIncluded() {
    boolean isMinimumIncluded = false;
    Double minimum = new Double(10);
    DoubleParameterValidator validator =
      new DoubleParameterValidator(
        false, null, false, minimum, isMinimumIncluded);
    DoubleParameter parameter =
      new DoubleParameter(
        "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
        null, false, "{0} {1}", null, false, minimum, isMinimumIncluded);
    ValidationResult result = validator.validateValue(parameter, minimum);
    assertNotNull(result);
    assertFalse(result.isWellSucceded());
    assertNotNull(result.getErrorMessage());
  }

  public void testValidateValueWithValueLargerThanMaximum() {
    boolean isMaximumIncluded = true;
    Double maximum = new Double(10);
    DoubleParameterValidator validator =
      new DoubleParameterValidator(
        false, maximum, isMaximumIncluded, null, false);
    DoubleParameter parameter =
      new DoubleParameter(
        "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
        null, false, "{0} {1}", maximum, isMaximumIncluded, null, false);
    ValidationResult result = validator.validateValue(parameter, new Double(maximum.doubleValue() + 0.1));
    assertNotNull(result);
    assertFalse(result.isWellSucceded());
    assertNotNull(result.getErrorMessage());
  }

  public void testValidateValuewithValueLesserThanMinimum() {
    boolean isMinimumIncluded = true;
    Double minimum = new Double(10);
    DoubleParameterValidator validator =
      new DoubleParameterValidator(
        false, null, false, minimum, isMinimumIncluded);
    DoubleParameter parameter =
      new DoubleParameter(
        "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
        null, false, "{0} {1}", null, false, minimum, isMinimumIncluded);
    ValidationResult result = validator.validateValue(parameter, new Double(minimum.doubleValue()-1));
    assertNotNull(result);
    assertFalse(result.isWellSucceded());
    assertNotNull(result.getErrorMessage());
  }

  public void testGetErrorMessage() {
    boolean isMinimumIncluded = true;
    Double minimum = new Double(10);
    DoubleParameterValidator validator =
      new DoubleParameterValidator(
        false, null, false, minimum, isMinimumIncluded);
    DoubleParameter parameter =
      new DoubleParameter(
        "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
        null, false, "{0} {1}", null, false, minimum, isMinimumIncluded);
    ValidationResult result = validator.validateValue(parameter, new Double(minimum.doubleValue()-1));
    assertNotNull(result);
    assertFalse(result.isWellSucceded());
    assertNotNull(result.getErrorMessage());
  }
}
