/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.client.algorithms.parameters.validators;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Locale;

import tecgraf.javautils.LNG;

import csbase.client.algorithms.parameters.validators.ValidationResult;
import csbase.logic.algorithms.parameters.IntegerParameter;

import junit.framework.TestCase;

public final class IntegerParameterValidatorTest extends TestCase {
  static {
    LNG.load("csbase.client.resources.properties.language.idiom", new Locale(
      "pt", "BR"));
  }
  
  public void testIntegerParameterValidatorWithTrueIsOptional() {
    boolean isOptional = true;
    Integer maximum = null;
    Integer minimum = null;
    IntegerParameterValidator validator =
      new IntegerParameterValidator(isOptional, maximum, minimum);
    assertValidator(isOptional, maximum, minimum, validator);
  }

  public void testIntegerParameterValidatorWithFalseIsOptional() {
    boolean isOptional = false;
    Integer maximum = null;
    Integer minimum = null;
    IntegerParameterValidator validator =
      new IntegerParameterValidator(isOptional, maximum, minimum);
    assertValidator(isOptional, maximum, minimum, validator);
  }

  public void testIntegerParameterValidatorWithMaximum() {
    boolean isOptional = false;
    Integer maximum = new Integer(1000);
    Integer minimum = null;
    IntegerParameterValidator validator =
      new IntegerParameterValidator(isOptional, maximum, minimum);
    assertValidator(isOptional, maximum, minimum, validator);
  }

  public void testIntegerParameterValidatorWithMinimum() {
    boolean isOptional = false;
    Integer maximum = null;
    Integer minimum = new Integer(100);
    IntegerParameterValidator validator =
      new IntegerParameterValidator(isOptional, maximum, minimum);
    assertValidator(isOptional, maximum, minimum, validator);
  }

  public void testIntegerParameterValidatorWithMaximumLesserThanMinimum() {
    Integer maximum = new Integer(10);
    Integer minimum = new Integer(10000);
    boolean isOptional = true;
    try {
      new IntegerParameterValidator(isOptional, maximum, minimum);
      fail("Esperada IllegalArgumentException");
    }
    catch (IllegalArgumentException e) {}
  }

  public void testValidateValueOptionalAndWithoutMaximumNorMinimum() {
    IntegerParameter parameter =
      new IntegerParameter(
        "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
        new Integer(10), false, "{0}{1}", null, null);
    IntegerParameterValidator validator =
      new IntegerParameterValidator(true, null, null);
    Integer value = new Integer(3);
    ValidationResult result = validator.validateValue(parameter, value);
    assertNotNull(result);
    assertTrue(result.isWellSucceded());
    assertNull(result.getErrorMessage());
  }

  public void testValidateValueWithValidWithAllRestrictions() {
    IntegerParameter parameter =
      new IntegerParameter(
        "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
        new Integer(10), false, "{0}{1}", null, null);
    Integer maximum = new Integer(200);
    Integer minimum = new Integer(0);
    IntegerParameterValidator validator =
      new IntegerParameterValidator(false, maximum, minimum);
    Integer value = new Integer(100);
    ValidationResult result = validator.validateValue(parameter, value);
    assertNotNull(result);
    assertTrue(result.isWellSucceded());
    assertNull(result.getErrorMessage());
  }

  public void testValidateValueWithNullValueOptional() {
    IntegerParameter parameter =
      new IntegerParameter(
        "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
        new Integer(10), false, "{0}{1}", null, null);
    IntegerParameterValidator validator =
      new IntegerParameterValidator(true, null, null);
    ValidationResult result = validator.validateValue(parameter, null);
    assertNotNull(result);
    assertTrue(result.isWellSucceded());
    assertNull(result.getErrorMessage());
  }

  public void testValidateValueWithValidValueEqualsMaximum() {
    IntegerParameter parameter =
      new IntegerParameter(
        "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
        new Integer(10), false, "{0}{1}", null, null);
    Integer maximum = new Integer(100);
    IntegerParameterValidator validator =
      new IntegerParameterValidator(false, maximum, null);
    ValidationResult result = validator.validateValue(parameter, maximum);
    assertNotNull(result);
    assertTrue(result.isWellSucceded());
    assertNull(result.getErrorMessage());
  }

  public void testValidateValueWithValueEqualsMinimum() {
    IntegerParameter parameter =
      new IntegerParameter(
        "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
        new Integer(10), false, "{0}{1}", null, null);
    Integer minimum = new Integer(0);
    IntegerParameterValidator validator =
      new IntegerParameterValidator(false, null, minimum);
    ValidationResult result = validator.validateValue(parameter, minimum);
    assertNotNull(result);
    assertTrue(result.isWellSucceded());
    assertNull(result.getErrorMessage());
  }

  public void testValidateValueWithNullValueNonOptional() {
    IntegerParameter parameter =
      new IntegerParameter(
        "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
        new Integer(10), false, "{0}{1}", null, null);
    IntegerParameterValidator validator =
      new IntegerParameterValidator(false, null, null);
    ValidationResult result = validator.validateValue(parameter, null);
    assertNotNull(result);
    assertFalse(result.isWellSucceded());
    assertNotNull(result.getErrorMessage());
  }

  public void testValidateWithValueLargerThanMaximum() {
    IntegerParameter parameter =
      new IntegerParameter(
        "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
        new Integer(10), false, "{0}{1}", null, null);
    Integer maximum = new Integer(0);
    IntegerParameterValidator validator =
      new IntegerParameterValidator(false, maximum, null);
    Integer value = new Integer(maximum.intValue()+1);
    ValidationResult result = validator.validateValue(parameter, value);
    assertNotNull(result);
    assertFalse(result.isWellSucceded());
    assertNotNull(result.getErrorMessage());
  }

  public void testValidateWithValueLesserThanMinimum() {
    IntegerParameter parameter =
      new IntegerParameter(
        "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
        new Integer(10), false, "{0}{1}", null, null);
    Integer minimum = new Integer(0);
    IntegerParameterValidator validator =
      new IntegerParameterValidator(false, null, minimum);
    Integer value = new Integer(minimum.intValue()-1);
    ValidationResult result = validator.validateValue(parameter, value);
    assertNotNull(result);
    assertFalse(result.isWellSucceded());
    assertNotNull(result.getErrorMessage());
  }

  private void assertValidator(
    boolean isOptional, Integer maximum, Integer minimum,
    IntegerParameterValidator validator) {
    assertEquals(isOptional, validator.isOptional());
    assertEquals(maximum, validator.getMaximum());
    assertEquals(minimum, validator.getMinimum());
  }
}
