/*
 * $Author$ $Date$ $Release:$
 */
package csbase.logic.algorithms;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import junit.framework.TestCase;

public final class AlgorithmVersionIdTest extends TestCase {
  public static AlgorithmVersionId create() {
    return new AlgorithmVersionId(1, 2, 3);
  }

  public void testAlgorithmVersion() {
    int major = 1;
    int minor = 2;
    int patch = 3;
    AlgorithmVersionId id = new AlgorithmVersionId(major, minor, patch);
    assertEquals(major, id.getMajor());
    assertEquals(minor, id.getMinor());
    assertEquals(patch, id.getPatch());
  }

  public void testEqualsObject() {
    int major = 1;
    int minor = 2;
    int patch = 3;
    AlgorithmVersionId id = new AlgorithmVersionId(major, minor, patch);
    AlgorithmVersionId equalsId = new AlgorithmVersionId(major, minor, patch);
    assertTrue(id.equals(equalsId));
  }

  public void testEqualsObjectDifferentMajor() {
    int major = 1;
    int minor = 2;
    int patch = 3;
    AlgorithmVersionId id = new AlgorithmVersionId(major, minor, patch);
    int differentMajor = major+10;
    AlgorithmVersionId differentMajorId =
      new AlgorithmVersionId(differentMajor, minor, patch);
    assertFalse(id.equals(differentMajorId));
  }

  public void testEqualsObjectDifferentMinor() {
    int major = 1;
    int minor = 2;
    int patch = 3;
    AlgorithmVersionId id = new AlgorithmVersionId(major, minor, patch);
    int differentMinor = minor+10;
    AlgorithmVersionId differentMinorId =
      new AlgorithmVersionId(major, differentMinor, patch);
    assertFalse(id.equals(differentMinorId));
  }

  public void testEqualsObjectDifferentPatch() {
    int major = 1;
    int minor = 2;
    int patch = 3;
    AlgorithmVersionId id = new AlgorithmVersionId(major, minor, patch);
    int differentPatch = patch+10;
    AlgorithmVersionId differentPatchId =
      new AlgorithmVersionId(major, minor, differentPatch);
    assertFalse(id.equals(differentPatchId));
  }

  public void testEqualsObjectObjectNull() {
    int major = 1;
    int minor = 2;
    int patch = 3;
    AlgorithmVersionId id = new AlgorithmVersionId(major, minor, patch);
    assertFalse(id.equals(null));
  }

  public void testEqualsObjectAnotherClassObject() {
    int major = 1;
    int minor = 2;
    int patch = 3;
    AlgorithmVersionId id = new AlgorithmVersionId(major, minor, patch);
    String anotherClassObject = "Another Class Object";
    assertFalse(id.equals(anotherClassObject));
  }

  public void testHashCode() {
    int major = 1;
    int minor = 2;
    int patch = 3;
    AlgorithmVersionId id = new AlgorithmVersionId(major, minor, patch);
    AlgorithmVersionId equalsId = new AlgorithmVersionId(major, minor, patch);
    assertEquals(id.hashCode(), equalsId.hashCode());
  }

  public void testCompareToEquals() {
    int major = 1;
    int minor = 2;
    int patch = 3;
    AlgorithmVersionId id = new AlgorithmVersionId(major, minor, patch);
    AlgorithmVersionId equalsId = new AlgorithmVersionId(major, minor, patch);
    assertEquals(0, id.compareTo(equalsId));
  }

  public void testCompareToMajorGreaterThan() {
    int major = 1;
    int minor = 2;
    int patch = 3;
    AlgorithmVersionId id = new AlgorithmVersionId(major, minor, patch);
    int majorGreaterThan = major+1;
    int minorLesserThan = minor-1;
    int patchLesserThan = patch-1;
    AlgorithmVersionId majorGreaterThanId =
      new AlgorithmVersionId(
        majorGreaterThan, minorLesserThan, patchLesserThan);
    assertTrue(majorGreaterThanId.compareTo(id) > 0);
  }

  public void testCompareToMajorLesserThan() {
    int major = 1;
    int minor = 2;
    int patch = 3;
    AlgorithmVersionId id = new AlgorithmVersionId(major, minor, patch);
    int majorLesserThan = major-1;
    int minorGreaterThan = minor+1;
    int patchGreaterThan = patch+1;
    AlgorithmVersionId majorLesserThanId =
      new AlgorithmVersionId(
        majorLesserThan, minorGreaterThan, patchGreaterThan);
    assertTrue(majorLesserThanId.compareTo(id) < 0);
  }

  public void testCompareToMinorGreaterThan() {
    int major = 1;
    int minor = 2;
    int patch = 3;
    AlgorithmVersionId id = new AlgorithmVersionId(major, minor, patch);
    int minorGreaterThan = minor+1;
    int patchLesserThan = patch-1;
    AlgorithmVersionId minorGreaterThanId =
      new AlgorithmVersionId(major, minorGreaterThan, patchLesserThan);
    assertTrue(minorGreaterThanId.compareTo(id) > 0);
  }

  public void testCompareToMinorLesserThan() {
    int major = 1;
    int minor = 2;
    int patch = 3;
    AlgorithmVersionId id = new AlgorithmVersionId(major, minor, patch);
    int minorLesserThan = minor-1;
    int patchGreaterThan = patch+1;
    AlgorithmVersionId minorGreaterThanId =
      new AlgorithmVersionId(major, minorLesserThan, patchGreaterThan);
    assertTrue(minorGreaterThanId.compareTo(id) < 0);
  }

  public void testCompareToPatchGreaterThan() {
    int major = 1;
    int minor = 2;
    int patch = 3;
    AlgorithmVersionId id = new AlgorithmVersionId(major, minor, patch);
    int patchGreaterThan = patch+1;
    AlgorithmVersionId patchGreaterThanId =
      new AlgorithmVersionId(major, minor, patchGreaterThan);
    assertTrue(patchGreaterThanId.compareTo(id) > 0);
  }

  public void testCompareToPatchLesserThan() {
    int major = 1;
    int minor = 2;
    int patch = 3;
    AlgorithmVersionId id = new AlgorithmVersionId(major, minor, patch);
    int patchGreaterThan = patch-1;
    AlgorithmVersionId patchLesserThanId =
      new AlgorithmVersionId(major, minor, patchGreaterThan);
    assertTrue(patchLesserThanId.compareTo(id) < 0);
  }

  public void testToString() {
    int major = 1;
    int minor = 2;
    int patch = 3;
    AlgorithmVersionId id = new AlgorithmVersionId(major, minor, patch);
    assertNotNull(id.toString());
    String separator = ".";
    String expectedStrindId = major+separator+minor+separator+patch;
    assertEquals(expectedStrindId, id.toString());
  }

  public void testSerialize()
  throws IOException, ClassNotFoundException {
    AlgorithmVersionId expectedId = new AlgorithmVersionId(1, 2, 3);
    ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
    ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
    objectOutput.writeObject(expectedId);
    ByteArrayInputStream inputSource =
      new ByteArrayInputStream(outputSource.toByteArray());
    ObjectInputStream objectInput = new ObjectInputStream(inputSource);
    AlgorithmVersionId loadedId = (AlgorithmVersionId)objectInput.readObject();
    assertEquals(expectedId.getMajor(), loadedId.getMajor());
    assertEquals(expectedId.getMinor(), loadedId.getMinor());
    assertEquals(expectedId.getPatch(), loadedId.getPatch());
  }
}
