/*
 * $Author:$
 * $Date:$
 * $Release:$
 */
package csbase.logic.algorithms.parameters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import csbase.exception.ParseException;

import junit.framework.TestCase;

/**
 * Testes unitrios para a classe {@link BooleanColumn}.
 *
 * @author lmoreira
 */
public final class BooleanColumnTest extends TestCase {
  /**
   * Testando criar colunas de booleanos com valor-padro falso e
   * editvel.
   */
  public void testBooleanColumn1() {
    String id = "Idenficador da coluna";
    String label = "Nome da coluna";
    boolean defaultValue = false;
    boolean isEditable = true;
    BooleanColumn column =
      new BooleanColumn(id, label, defaultValue, isEditable, null, null);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertFalse(column.isOptional());
    assertEquals(new Boolean(defaultValue), column.getDefaultValue(0));
    assertEquals(isEditable, column.isEditable());
    assertEquals("1", column.getTrueCommand());
    assertEquals("0", column.getFalseCommand());
  }

  /**
   * Testando criar colunas de booleanos com valor-padro verdadeiro e
   * editvel.
   */
  public void testBooleanColumn2() {
    String id = "Idenficador da coluna";
    String label = "Nome da coluna";
    boolean defaultValue = true;
    boolean isEditable = true;
    BooleanColumn column =
      new BooleanColumn(id, label, defaultValue, isEditable, null, null);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertFalse(column.isOptional());
    assertEquals(new Boolean(defaultValue), column.getDefaultValue(0));
    assertEquals(isEditable, column.isEditable());
    assertEquals("1", column.getTrueCommand());
    assertEquals("0", column.getFalseCommand());
  }

  /**
   * Testando criar colunas de booleanos com valor-padro falso e
   * no-editvel.
   */
  public void testBooleanColumn3() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean defaultValue = false;
    boolean isEditable = false;
    BooleanColumn column =
      new BooleanColumn(id, label, defaultValue, isEditable, null, null);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertFalse(column.isOptional());
    assertEquals(new Boolean(defaultValue), column.getDefaultValue(0));
    assertEquals(isEditable, column.isEditable());
    assertEquals("1", column.getTrueCommand());
    assertEquals("0", column.getFalseCommand());
  }

  /**
   * Testando criar colunas de booleanos com valor-padro verdadeiro e
   * no-editvel.
   */
  public void testBooleanColumn4() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean defaultValue = true;
    boolean isEditable = false;
    BooleanColumn column =
      new BooleanColumn(id, label, defaultValue, isEditable, null, null);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertFalse(column.isOptional());
    assertEquals(new Boolean(defaultValue), column.getDefaultValue(0));
    assertEquals(isEditable, column.isEditable());
    assertEquals("1", column.getTrueCommand());
    assertEquals("0", column.getFalseCommand());
  }

  
  /**
   * Testando criar colunas de booleanos com valor-padro falso,
   * editvel e com valor para comando verdadeiro definido.
   */
  public void testBooleanColumn5() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean defaultValue = true;
    boolean isEditable = true;
    String trueCommand = "COMANDO_VERDADEIRO";
    BooleanColumn column =
      new BooleanColumn(id, label, defaultValue, isEditable, null, trueCommand);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertFalse(column.isOptional());
    assertEquals(new Boolean(defaultValue), column.getDefaultValue(0));
    assertEquals(trueCommand, column.getTrueCommand());
    assertEquals("0", column.getFalseCommand());
  }

  /**
   * Testando criar colunas de booleanos com valor-padro verdadeiro,
   * editvel e com valor para comando falso definido.
   */
  public void testBooleanColumn6() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean defaultValue = false;
    boolean isEditable = true;
    String falseCommand = "COMANDO_FALSO";
    BooleanColumn column =
      new BooleanColumn(id, label, defaultValue, isEditable, falseCommand, 
        null);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertFalse(column.isOptional());
    assertEquals(new Boolean(defaultValue), column.getDefaultValue(0));
    assertEquals("1", column.getTrueCommand());
    assertEquals(falseCommand, column.getFalseCommand());
  }

  /**
   * Tentando criar um columa com rtulo {@code null}.
   */
  public void testBooleanColumn7() {
    try {
      new BooleanColumn("ID", null, false, true, null, null);
      fail("No deveria ser possvel criar uma coluna com nome nulo.");
    }
    catch (IllegalArgumentException e) {}
  }
  
  /**
   * Tentando criar um columa com identificador {@code null}.
   */
  public void testBooleanColumn8() {
    try {
      new BooleanColumn(null, "Rtulo", false, true, null, null);
      fail("No deveria ser possvel criar uma coluna com nome nulo.");
    }
    catch (IllegalArgumentException e) {}
  }
  
  /**
   * Testando o mtodo {@link BooleanColumn#getCommandValue(Boolean)} utilizando
   * o valor {@code true}.
   */
  public void testGetCommandValue1() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean defaultValue = false;
    String falseCommand = "COMANDO_FALSO";
    String trueCommand = "COMANDO_VERDADEIRO";
    BooleanColumn column =
      new BooleanColumn(id, label, defaultValue, true, falseCommand, trueCommand);
    assertEquals(trueCommand, column.getTrueCommand());
    assertEquals(falseCommand, column.getFalseCommand());
    assertEquals(trueCommand, column.getCommandValue(new Boolean(true)));
  }

  /**
   * Testando o mtodo {@link BooleanColumn#getCommandValue(Boolean)} utilizando
   * o valor {@code false}.
   */
  public void testGetCommandValue2() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean defaultValue = false;
    String falseCommand = "COMANDO_FALSO";
    String trueCommand = "COMANDO_VERDADEIRO";
    BooleanColumn column =
      new BooleanColumn(id, label, defaultValue, true, falseCommand, trueCommand);
    assertEquals(trueCommand, column.getTrueCommand());
    assertEquals(falseCommand, column.getFalseCommand());
    assertEquals(falseCommand, column.getCommandValue(new Boolean(false)));
  }

  /**
   * Testando o mtodo {@link BooleanColumn#getCommandValue(Boolean)} utilizando
   * o valor {@code null}.
   */
  public void testGetCommandValue3() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean defaultValue = false;
    String falseCommand = "COMANDO_FALSO";
    String trueCommand = "COMANDO_VERDADEIRO";
    BooleanColumn column =
      new BooleanColumn(id, label, defaultValue, true, falseCommand, trueCommand);
    try {
      assertNull(column.getCommandValue(null));
      fail("Esperada IllegalArgumentException");
    }
    catch (IllegalArgumentException e) {}
  }

  /**
   * Testando o mtodo {@link BooleanColumn#getItemValueAsText(Boolean)} com
   * valor verdadeiro.
   */
  public void testGetItemValueAsText1() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean defaultValue = false;
    String falseCommand = "COMANDO_FALSO";
    String trueCommand = "COMANDO_VERDADEIRO";
    BooleanColumn column =
      new BooleanColumn(id, label, defaultValue, true, falseCommand, trueCommand);
    assertEquals("true", column.getItemValueAsText(true));
  }
  
  /**
   * Testando o mtodo {@link BooleanColumn#getItemValueAsText(Boolean)} com
   * valor falso.
   */
  public void testGetItemValueAsText2() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean defaultValue = false;
    String falseCommand = "COMANDO_FALSO";
    String trueCommand = "COMANDO_VERDADEIRO";
    BooleanColumn column =
      new BooleanColumn(id, label, defaultValue, true, falseCommand, trueCommand);
    assertEquals("false", column.getItemValueAsText(false));
  }
  
  /**
   * Testando o mtodo {@link BooleanColumn#getItemValueAsText(Boolean)} com
   * valor {@code null}.
   */
  public void testGetItemValueAsText3() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean defaultValue = false;
    String falseCommand = "COMANDO_FALSO";
    String trueCommand = "COMANDO_VERDADEIRO";
    BooleanColumn column =
      new BooleanColumn(id, label, defaultValue, true, falseCommand, trueCommand);
    assertNull(column.getItemValueAsText(null));
  }
  
  /**
   * Testando o mtodo {@link BooleanColumn#getItemValueFromText(String)} com
   * valor verdadeiro.
   * @throws ParseException 
   */
  public void testGetItemValueFromText1() throws ParseException {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean defaultValue = false;
    String falseCommand = "COMANDO_FALSO";
    String trueCommand = "COMANDO_VERDADEIRO";
    BooleanColumn column =
      new BooleanColumn(id, label, defaultValue, true, falseCommand, trueCommand);
    assertEquals(new Boolean(true), column.getItemValueFromText("true"));
  }
  
  /**
   * Testando o mtodo {@link BooleanColumn#getItemValueFromText(String)} com
   * valor falso.
   * @throws ParseException 
   */
  public void testGetItemValueFromText2() throws ParseException {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean defaultValue = false;
    String falseCommand = "COMANDO_FALSO";
    String trueCommand = "COMANDO_VERDADEIRO";
    BooleanColumn column =
      new BooleanColumn(id, label, defaultValue, true, falseCommand, trueCommand);
    assertEquals(new Boolean(false), column.getItemValueFromText("false"));
  }
  
  /**
   * Testando o mtodo {@link BooleanColumn#getItemValueFromText(String)} com
   * valor {@code null}.
   */
  public void testGetItemValueFromText3() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean defaultValue = false;
    String falseCommand = "COMANDO_FALSO";
    String trueCommand = "COMANDO_VERDADEIRO";
    BooleanColumn column =
      new BooleanColumn(id, label, defaultValue, true, falseCommand, trueCommand);
    try {
      column.getItemValueFromText(null);
      fail();
    }
    catch (ParseException e) {}
  }

  /**
   * Testando o mtodo {@link BooleanColumn#getItemValueFromText(String)} com
   * valor que no representa um valor booleano.
   */
  public void testGetItemValueFromText4() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean defaultValue = false;
    String falseCommand = "COMANDO_FALSO";
    String trueCommand = "COMANDO_VERDADEIRO";
    BooleanColumn column =
      new BooleanColumn(id, label, defaultValue, true, falseCommand, trueCommand);
    try {
      column.getItemValueFromText("valor no-booleano");
      fail();
    }
    catch (ParseException e) {}
  }
  
  /**
   * Testando a serializao e desserializao.
   * 
   * @throws IOException Bug.
   * @throws ClassNotFoundException Bug.
   */
  public void testSerialize()
  throws IOException, ClassNotFoundException {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean defaultValue = true;
    String trueCommand = "COMANDO_VERDADEIRO";
    String falseCommand = "COMANDO_FALSO";
    BooleanColumn outputColumn =
      new BooleanColumn(
         id, label, defaultValue, true, falseCommand, trueCommand);
    ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
    ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
    objectOutput.writeObject(outputColumn);
    ByteArrayInputStream inputSource =
      new ByteArrayInputStream(outputSource.toByteArray());
    ObjectInputStream objectInput = new ObjectInputStream(inputSource);
    BooleanColumn inputColumn = (BooleanColumn)objectInput.readObject();
    assertEquals(outputColumn.getId(), inputColumn.getId());
    assertEquals(outputColumn.getLabel(), inputColumn.getLabel());
    assertEquals(outputColumn.isOptional(), inputColumn.isOptional());
    assertEquals(outputColumn.getDefaultValue(0), inputColumn
      .getDefaultValue(0));
    assertEquals(outputColumn.getFalseCommand(), inputColumn.getFalseCommand());
    assertEquals(outputColumn.getTrueCommand(), inputColumn.getTrueCommand());
  }
}
