/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.logic.algorithms.parameters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import junit.framework.TestCase;
import csbase.exception.ParseException;

public final class BooleanParameterTest extends TestCase {
	public void testBooleanParameter() {
		String name = "Nome do parmetro";
		String label = "Rtulo do parmetro";
		String description = "Descrio do parmetro";
		boolean defaultValue = false;
    String commandLinePattern = "{0}={1}";
		String falseCommand = "FALSO";
		String trueCommand = "VERDADEIRO";
		BooleanParameter parameter = new BooleanParameter(name, label, description,
      defaultValue, commandLinePattern, falseCommand, trueCommand);
		assertNotNull("O nome do parmetro no deveria ser nulo.", parameter
				.getName());
		assertEquals("O nome do parmetro no  igual ao nome esperado.", name,
				parameter.getName());
		assertNotNull("A descrio do parmetro no deveria ser nula.",
				parameter.getDescription());
		assertEquals(
				"A descrio do parmetro no  igual  descrio esperada.",
				description, parameter.getDescription());
		assertNotNull("O rtulo do parmetro no deveria ser nulo.", parameter
				.getLabel());
		assertEquals("O rtulo do parmetro no  igual ao rtulo esperado.",
				label, parameter.getLabel());
		assertNotNull("O valor-padro no deveria ser nulo.", parameter
				.getDefaultValue());
		assertEquals("O valor-padro deveria no  igual ao esperado.",
				new Boolean(defaultValue), parameter.getDefaultValue());
    assertEquals(commandLinePattern, parameter.getCommandLinePattern());
		assertEquals(trueCommand, parameter.getTrueCommand());
		assertEquals(falseCommand, parameter.getFalseCommand());
		assertFalse(parameter.isOptional());
	}

	public void testBooleanParameterWithNullName() {
		String label = "Rtulo do parmetro";
		String description = "Descrio do parmetro";
		boolean defaultValue = false;
    String commandLinePattern = "{0}={1}";
		String falseCommand = "FALSO";
		String trueCommand = "VERDADEIRO";
		try {
			new BooleanParameter(null, label, description, defaultValue,
        commandLinePattern, falseCommand, trueCommand);
			fail("No deveria ser possvel criar um parmetro sem um nome.");
		} catch (IllegalArgumentException e) {
		}
	}

	public void testBooleanParameterWithNullLabel() {
		String name = "Nome do parmetro";
		String description = "Descrio do parmetro";
		boolean defaultValue = false;
    String commandLinePattern = "{0}={1}";
		String falseCommand = "FALSO";
		String trueCommand = "VERDADEIRO";
		try {
			new BooleanParameter(name, null, description, defaultValue,
        commandLinePattern, falseCommand, trueCommand);
			fail("No deveria ser possvel criar um parmetro no-oculto sem um rtulo.");
		} catch (IllegalArgumentException e) {
		}
	}

	public void testBooleanParameterWithNullDescription() {
		String name = "Nome do parmetro";
		String label = "Rtulo do parmetro";
		boolean defaultValue = false;
    String commandLinePattern = "{0}={1}";
		String falseCommand = "FALSO";
		String trueCommand = "VERDADEIRO";
		try {
			new BooleanParameter(name, label, null, defaultValue, commandLinePattern,
        falseCommand, trueCommand);
			fail("No deveria ser possvel criar um parmetro no-oculto sem uma descrio.");
		} catch (IllegalArgumentException e) {
		}
	}

	public void testBooleanParameterWithNullFalseCommand() {
		String name = "Nome do parmetro";
		String label = "Rtulo do parmetro";
		String description = "Descrio do parmetro";
		boolean defaultValue = false;
    String commandLinePattern = "{0}={1}";
    String trueCommand = "VERDADEIRO";
		BooleanParameter parameter = new BooleanParameter(name, label,
				description, defaultValue, commandLinePattern, null, trueCommand);
		assertNotNull("O nome do parmetro no deveria ser nulo.", parameter
				.getName());
		assertEquals("O nome do parmetro no  igual ao nome esperado.", name,
				parameter.getName());
		assertNotNull("A descrio do parmetro no deveria ser nula.",
				parameter.getDescription());
		assertEquals(
				"A descrio do parmetro no  igual  descrio esperada.",
				description, parameter.getDescription());
		assertNotNull("O rtulo do parmetro no deveria ser nulo.", parameter
				.getLabel());
		assertEquals("O rtulo do parmetro no  igual ao rtulo esperado.",
				label, parameter.getLabel());
		assertNotNull("O valor-padro no deveria ser nulo.", parameter
				.getDefaultValue());
		assertEquals("O valor-padro deveria no  igual ao esperado.",
				new Boolean(defaultValue), parameter.getDefaultValue());
    assertEquals(commandLinePattern, parameter.getCommandLinePattern());
		assertEquals(trueCommand, parameter.getTrueCommand());
		assertEquals("0", parameter.getFalseCommand());
		assertFalse(parameter.isOptional());
	}

	public void testBooleanParameterWithNullTrueCommand() {
		String name = "Nome do parmetro";
		String label = "Rtulo do parmetro";
		String description = "Descrio do parmetro";
		boolean defaultValue = false;
    String commandLinePattern = "{0}={1}";
		String falseCommand = "FALSO";
		BooleanParameter parameter = new BooleanParameter(name, label,
				description, defaultValue, commandLinePattern, falseCommand, null);
		assertNotNull("O nome do parmetro no deveria ser nulo.", parameter
				.getName());
		assertEquals("O nome do parmetro no  igual ao nome esperado.", name,
				parameter.getName());
		assertNotNull("A descrio do parmetro no deveria ser nula.",
				parameter.getDescription());
		assertEquals(
				"A descrio do parmetro no  igual  descrio esperada.",
				description, parameter.getDescription());
		assertNotNull("O rtulo do parmetro no deveria ser nulo.", parameter
				.getLabel());
		assertEquals("O rtulo do parmetro no  igual ao rtulo esperado.",
				label, parameter.getLabel());
		assertNotNull("O valor-padro no deveria ser nulo.", parameter
				.getDefaultValue());
		assertEquals("O valor-padro deveria no  igual ao esperado.",
				new Boolean(defaultValue), parameter.getDefaultValue());
		assertEquals("1", parameter.getTrueCommand());
		assertEquals(falseCommand, parameter.getFalseCommand());
    assertEquals(commandLinePattern, parameter.getCommandLinePattern());
		assertFalse(parameter.isOptional());
	}

	public void testGetValueAsText() {
		String name = "Nome do parmetro";
		String label = "Rtulo do parmetro";
		String description = "Descrio do parmetro";
		boolean[] defaultValues = new boolean[] { true, false };
    String commandLinePattern = "{0}={1}";
		String[] textValues = new String[] { Boolean.TRUE.toString(),
				Boolean.FALSE.toString() };
		for (int i = 0; i < defaultValues.length; i++) {
			BooleanParameter parameter = new BooleanParameter(name, label,
					description, defaultValues[i], commandLinePattern, null, null);
			assertEquals(textValues[i], parameter.getValueAsText());
		}
	}

	public void testSetValueAsText() throws ParseException {
		String name = "Nome do parmetro";
		String label = "Rtulo do parmetro";
		String description = "Descrio do parmetro";
		boolean defaultValue = false;
    String commandLinePattern = "{0}={1}";
		BooleanParameter parameter = new BooleanParameter(name, label,
				description, defaultValue, commandLinePattern, null, null);
		parameter.setValueAsText(Boolean.TRUE.toString());
		assertEquals(new Boolean(true), parameter.getValue());
		parameter.setValueAsText(Boolean.FALSE.toString());
		assertEquals(new Boolean(false), parameter.getValue());
	}

	public void testSetValueAsTextWithIllegalTextValue() {
		String name = "Nome do parmetro";
		String label = "Rtulo do parmetro";
		String description = "Descrio do parmetro";
		boolean defaultValue = false;
    String commandLinePattern = "{0}={1}";
		BooleanParameter parameter = new BooleanParameter(name, label,
				description, defaultValue, commandLinePattern, null, null);
		try {
			parameter.setValueAsText("Valor invlido.");
			fail("Esperada ParseException");
		} catch (ParseException e) {
		}
	}

	public void testSetValueAsTextWithNullValue() throws ParseException {
		String name = "Nome do parmetro";
		String label = "Rtulo do parmetro";
		String description = "Descrio do parmetro";
		boolean defaultValue = false;
    String commandLinePattern = "{0}={1}";
    BooleanParameter parameter = new BooleanParameter(name, label,
				description, defaultValue, commandLinePattern, null, null);
    assertEquals(new Boolean(defaultValue), parameter.getValue());
    parameter.setValue(new Boolean(!defaultValue));
    assertEquals(new Boolean(!defaultValue), parameter.getValue());
		parameter.setValueAsText(null);
    assertEquals(new Boolean(defaultValue), parameter.getValue());
	}

	public void testGetCommandValueWithFalseValue() {
		String name = "Nome do parmetro";
		String label = "Rtulo do parmetro";
		String description = "Descrio do parmetro";
		String falseCommand = "Comando falso";
		String trueCommand = "Comando verdadeiro";
		boolean defaultValue = false;
    String commandLinePattern = "{0}={1}";
		BooleanParameter parameter = new BooleanParameter(name, label, description,
      defaultValue, commandLinePattern, falseCommand, trueCommand);
		assertEquals(falseCommand, parameter.getCommandValue("/", null));
	}

	public void testGetCommandValueWithTrueValue() {
		String name = "Nome do parmetro";
		String label = "Rtulo do parmetro";
		String description = "Descrio do parmetro";
		String falseCommand = "Comando falso";
		String trueCommand = "Comando verdadeiro";
		boolean defaultValue = true;
    String commandLinePattern = "{0}={1}";
		BooleanParameter parameter = new BooleanParameter(name, label, description,
      defaultValue, commandLinePattern, falseCommand, trueCommand);
		assertEquals(trueCommand, parameter.getCommandValue("/", null));
	}

	public void testSerialize() throws IOException, ClassNotFoundException {
		String name = "Nome do parmetro";
		String label = "Rtulo do parmetro";
		String description = "Descrio do parmetro";
		boolean defaultValue = true;
    String commandLinePattern = "--{0} {1}";
		String trueCommand = "s";
		String falseCommand = "n";
		BooleanParameter outputParameter = new BooleanParameter(name, label,
      description, defaultValue, commandLinePattern, falseCommand, trueCommand);
		ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
		ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
		objectOutput.writeObject(outputParameter);
		ByteArrayInputStream inputSource = new ByteArrayInputStream(
				outputSource.toByteArray());
		ObjectInputStream objectInput = new ObjectInputStream(inputSource);
		BooleanParameter inputParameter = (BooleanParameter) objectInput
				.readObject();
		assertEquals(outputParameter, inputParameter);
		assertEquals(outputParameter.getName(), inputParameter.getName());
		assertEquals(outputParameter.getLabel(), inputParameter.getLabel());
		assertEquals(outputParameter.getDescription(), inputParameter
				.getDescription());
		assertEquals(outputParameter.getDefaultValue(), inputParameter
				.getDefaultValue());
		assertEquals(outputParameter.isOptional(), inputParameter.isOptional());
		assertEquals(outputParameter.getCommandLinePattern(), inputParameter
      .getCommandLinePattern());
    assertEquals(outputParameter.getTrueCommand(), inputParameter
				.getTrueCommand());
		assertEquals(outputParameter.getFalseCommand(), inputParameter
				.getFalseCommand());
	}
}
