package csbase.logic.algorithms.parameters;

import junit.framework.Assert;

/**
 * <p>Observador Verificador de Eventos de {@link IntegerListParameter}:</p>
 * 
 * <ul>Ele  responsvel por:
 *   <li>Verificar se eventos inesperados esto ocorrendo.</li>
 *   <li>Verificar se eventos esperados esto ocorrendo.</li>
 *   <li>Verificar se a quantidade de eventos esperados est dentro do esperado.
 *   </li>
 * </ul> 
 *
 * @author lmoreira
 */
final class CheckIntegerListParameterListener implements IntegerListParameterListener {

  /**
   * Libera o evento 
   * {@link IntegerListParameterListener#maximumWasChanged(IntegerListParameter)}.
   */
  private boolean allowMaximumWasChangedEvent;
  /**
   * Libera o evento 
   * {@link IntegerListParameterListener#minimumWasChanged(IntegerListParameter)}.
   */
  private boolean allowMinimumWasChangedEvent;
  
  /**
   * Contador de eventos ocorridos.
   */
  private int eventCount;
  
  /**
   * <p>Cria um verificador.</p>
   * 
   * <p>Ele proibe todos os eventos.</p>
   *
   */
  CheckIntegerListParameterListener() {
    allowMaximumWasChangedEvent = false;
    allowMinimumWasChangedEvent = false;
    eventCount = 0;
  }
  
  /**
   * Permite que haja eventos 
   * {@link IntegerListParameterListener#maximumWasChanged(IntegerListParameter)}. 
   */
  public void allowMaximumWasChangedEvent() {
    allowMaximumWasChangedEvent = true;
  }
  
  /**
   * Permite que haja eventos 
   * {@link IntegerListParameterListener#minimumWasChanged(IntegerListParameter)}. 
   */
  public void allowMinimumWasChangedEvent() {
    allowMinimumWasChangedEvent = true;
  }

  /**
   * Obtm o nmero de eventos ocorridos.
   * 
   * @return .
   */
  public int getEventCount() {
    return eventCount;
  }

  public void maximumWasChanged(IntegerListParameter parameter) {
    Assert.assertTrue(allowMaximumWasChangedEvent);
    eventCount++;
  }

  public void minimumWasChanged(IntegerListParameter parameter) {
    Assert.assertTrue(allowMinimumWasChangedEvent);
    eventCount++;
  }
}
