/*
 * $Author:$
 * $Date:$
 * $Release:$
 */
package csbase.logic.algorithms.parameters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import junit.framework.TestCase;

public final class DoubleListParameterTest extends TestCase {
  public void testDoubleListParameterWithTrueIsOptional() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    String commandLinePattern = "--{0} {1}";
    Double maximum = new Double(10.10);
    boolean isMaximumIncluded = true;
    Double minimum = new Double(1.1);
    boolean isMinimumIncluded = true;
    DoubleListParameter parameter = new DoubleListParameter(name, label,
      description, isOptional, commandLinePattern, maximum, isMaximumIncluded,
      minimum, isMinimumIncluded);
    assertEquals(name, label, description, isOptional, commandLinePattern, 
      maximum, isMaximumIncluded, minimum, isMinimumIncluded, parameter);
  }

  public void testDoubleListParameterWithFalseIsOptional() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Double maximum = new Double(10.10);
    boolean isMaximumIncluded = true;
    Double minimum = new Double(1.1);
    boolean isMinimumIncluded = true;
    DoubleListParameter parameter =
      new DoubleListParameter(name, label,
      description, isOptional, commandLinePattern, maximum, isMaximumIncluded,
      minimum, isMinimumIncluded);
    assertEquals(name, label, description, isOptional, commandLinePattern,
      maximum, isMaximumIncluded, minimum, isMinimumIncluded, parameter);
  }

  public void testDoubleListParameterWithNullMaximum() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Double maximum = null;
    boolean isMaximumIncluded = false;
    Double minimum = new Double(1.1);
    boolean isMinimumIncluded = true;
    DoubleListParameter parameter = new DoubleListParameter(name, label,
      description, isOptional, commandLinePattern, maximum, isMaximumIncluded,
      minimum, isMinimumIncluded);
    assertEquals(name, label, description, isOptional, commandLinePattern,
      maximum, isMaximumIncluded, minimum, isMinimumIncluded, parameter);
  }

  public void testDoubleListParameterWithNullMinimum() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Double maximum = new Double(10.10);
    boolean isMaximumIncluded = true;
    Double minimum = null;
    boolean isMinimumIncluded = false;
    DoubleListParameter parameter = new DoubleListParameter(name, label,
      description, isOptional, commandLinePattern, maximum, isMaximumIncluded,
      minimum, isMinimumIncluded);
    assertEquals(name, label, description, isOptional, commandLinePattern,
      maximum, isMaximumIncluded, minimum, isMinimumIncluded, parameter);
  }

  public void testDoubleListParameterWithFalseIsMaximumIncluded() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Double maximum = new Double(10.10);
    boolean isMaximumIncluded = false;
    Double minimum = new Double(1.1);
    boolean isMinimumIncluded = true;
    DoubleListParameter parameter = new DoubleListParameter(name, label,
      description, isOptional, commandLinePattern, maximum, isMaximumIncluded,
      minimum, isMinimumIncluded);
    assertEquals(name, label, description, isOptional, commandLinePattern,
      maximum, isMaximumIncluded, minimum, isMinimumIncluded, parameter);
  }

  public void testDoubleListParameterWithFalseIsMinimumIncluded() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";    
    Double maximum = new Double(10.10);
    boolean isMaximumIncluded = true;
    Double minimum = new Double(1.1);
    boolean isMinimumIncluded = false;
    DoubleListParameter parameter = new DoubleListParameter(name, label,
      description, isOptional, commandLinePattern, maximum, isMaximumIncluded,
      minimum, isMinimumIncluded);
    assertEquals(name, label, description, isOptional, commandLinePattern,
      maximum, isMaximumIncluded, minimum, isMinimumIncluded, parameter);
  }

  public void testDoubleListParameterWithMinimumLargerThanMaximum() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Double maximum = new Double(10.10);
    boolean isMaximumIncluded = true;
    Double minimum = new Double(12.12);
    boolean isMinimumIncluded = false;
    try {
      new DoubleListParameter(name, label, description, isOptional,
        commandLinePattern, maximum, isMaximumIncluded, minimum,
        isMinimumIncluded);
      fail(
        "No deveria ser possvel criar uma parmetro com o mnimo maior do que o mximo.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  public void testSerialize()
  throws IOException, ClassNotFoundException {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    String commandLinePattern = "--{0} {1}";
    Double maximum = new Double(10.10);
    boolean isMaximumIncluded = true;
    Double minimum = new Double(1.1);
    boolean isMinimumIncluded = true;
    DoubleListParameter outputParameter = new DoubleListParameter(name, label,
      description, isOptional, commandLinePattern, maximum, isMaximumIncluded,
      minimum, isMinimumIncluded);
    ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
    ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
    objectOutput.writeObject(outputParameter);
    ByteArrayInputStream inputSource =
      new ByteArrayInputStream(outputSource.toByteArray());
    ObjectInputStream objectInput = new ObjectInputStream(inputSource);
    DoubleListParameter inputParameter =
      (DoubleListParameter)objectInput.readObject();
    assertEquals(outputParameter.getName(), outputParameter.getLabel(),
      outputParameter.getDescription(), outputParameter.isOptional(),
      outputParameter.getCommandLinePattern(), outputParameter.getMaximum(),
      outputParameter.isMaximumIncluded(), outputParameter.getMinimum(),
      outputParameter.isMinimumIncluded(), inputParameter);
  }

  /**
   * Verifica se todos os atributos fornecidos so iguais aos respectivos
   * atributos do parmetro.
   * 
   * @param name O nome (Aceita {@code null}).
   * @param label O rtulo (Aceita {@code null}).
   * @param description A descrio (Aceita {@code null}).
   * @param isOptional Indica se o valor do parmetro  opcional/obrigatrio.
   * @param commandLinePattern O padro da linha de comando (Aceita {@code null}).
   * @param maximum O mximo (Aceita {@code null}).
   * @param isMaximumIncluded Indica se o valor mximo  considerado vlido.
   * @param minimum O mnimo (Aceita {@code null}).
   * @param isMinimumIncluded Indica se o valor mnimo  considerado vlido.
   * @param parameter O parmetro (No aceita {@code null})..
   */
  private void assertEquals(String name, String label, String description,
    boolean isOptional, String commandLinePattern, Double maximum,
    boolean isMaximumIncluded, Double minimum, boolean isMinimumIncluded,
    DoubleListParameter parameter) {
    assertEquals(name, parameter.getName());
    assertEquals(label, parameter.getLabel());
    assertEquals(description, parameter.getDescription());
    assertEquals(isOptional, parameter.isOptional());
    assertEquals(commandLinePattern, parameter.getCommandLinePattern());
    assertEquals(maximum, parameter.getMaximum());
    assertEquals(isMaximumIncluded, parameter.isMaximumIncluded());
    assertEquals(minimum, parameter.getMinimum());
    assertEquals(isMinimumIncluded, parameter.isMinimumIncluded());
    assertNull(parameter.getDefaultValue());
  }
}
