/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.logic.algorithms.parameters;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import junit.framework.TestCase;

/**
 * Testes unitrios para a classe {@link EnumerationParameter}.
 */
public final class EnumerationParameterTest extends TestCase {
  /**
   * Testando o construtor: criando um parmetro com um nico item.
   */
  public void testEnumerationParameter1() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String itemId = "ID do item";
    String itemName = "Nome do item";
    String itemValue = "Valor do item";
    String itemDescription = "Descrio do item";
    EnumerationItem defaultValue = new EnumerationItem(itemId, itemName,
      itemValue, itemDescription);
    String commandLinePattern = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
    boolean mustSortItems = true;
    List<EnumerationItem> items = new LinkedList<EnumerationItem>();
    items.add(defaultValue);
    EnumerationParameter parameter = new EnumerationParameter(name, label,
      description, defaultValue, commandLinePattern, items, mustSortItems);
    assertEquals(name, label, description, defaultValue, commandLinePattern,
      items, mustSortItems, parameter);
  }

  /**
   * Testando o construtor: criando um parmetro com mais do que um item sem
   * ordenao.
   */
  public void testEnumerationParameter2() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String itemId1 = "Identificador do item 1";
    String itemName1 = "Nome do item 1";
    String itemValue1 = "Valor do item 1";
    String itemDescription1 = "Descrio do item 1";
    EnumerationItem item1 = new EnumerationItem(itemId1, itemName1, itemValue1,
      itemDescription1);
    String itemId2 = "Identificador do item 2";
    String itemName2 = "Nome do item 2";
    String itemValue2 = "Valor do item 2";
    String itemDescription2 = "Descrio do item 2";
    EnumerationItem item2 = new EnumerationItem(itemId2, itemName2, itemValue2,
      itemDescription2);
    String commandLinePattern = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
    boolean mustSortItems = false;
    List<EnumerationItem> items = new LinkedList<EnumerationItem>();
    items.add(item2);
    items.add(item1);
    EnumerationParameter parameter = new EnumerationParameter(name, label,
      description, item1, commandLinePattern, items, mustSortItems);
    assertEquals(name, label, description, item1, commandLinePattern, items,
      mustSortItems, parameter);
  }

  /**
   * Testando o construtor: criando um parmetro com mais do que um item com
   * ordenao.
   */
  public void testEnumerationParameter3() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String itemId1 = "Identificador do item 1";
    String itemName1 = "Nome do item 1";
    String itemValue1 = "Valor do item 1";
    String itemDescription1 = "Descrio do item 1";
    EnumerationItem item1 = new EnumerationItem(itemId1, itemName1, itemValue1,
      itemDescription1);
    String itemId2 = "Identificador do item 2";
    String itemName2 = "Nome do item 2";
    String itemValue2 = "Valor do item 2";
    String itemDescription2 = "Descrio do item 2";
    EnumerationItem item2 = new EnumerationItem(itemId2, itemName2, itemValue2,
      itemDescription2);
    String commandLinePattern = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
    boolean mustSortItems = true;
    List<EnumerationItem> items = new LinkedList<EnumerationItem>();
    items.add(item2);
    items.add(item1);
    EnumerationParameter parameter = new EnumerationParameter(name, label,
      description, item1, commandLinePattern, items, mustSortItems);
    Collections.sort(items);
    assertEquals(name, label, description, item1, commandLinePattern, items,
      mustSortItems, parameter);
  }

  /**
   * Testando o construtor: criando um parmetro sem valor-padro.
   */
  public void testEnumerationParameter4() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String itemId1 = "Identificador do item 1";
    String itemName1 = "Nome do item 1";
    String itemValue1 = "Valor do item 1";
    String itemDescription1 = "Descrio do item 1";
    EnumerationItem item1 = new EnumerationItem(itemId1, itemName1, itemValue1,
      itemDescription1);
    String itemId2 = "Identificador do item 2";
    String itemName2 = "Nome do item 2";
    String itemValue2 = "Valor do item 2";
    String itemDescription2 = "Descrio do item 2";
    EnumerationItem item2 = new EnumerationItem(itemId2, itemName2, itemValue2,
      itemDescription2);
    String commandLinePattern = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
    boolean mustSortItems = true;
    List<EnumerationItem> items = new LinkedList<EnumerationItem>();
    items.add(item2);
    items.add(item1);
    EnumerationParameter parameter = new EnumerationParameter(name, label,
      description, null, commandLinePattern, items, mustSortItems);
    Collections.sort(items);
    assertEquals(name, label, description, null, commandLinePattern, items,
      mustSortItems, parameter);
  }

  /**
   * Testando o construtor: criando um parmetro sem formato de linha de
   * comando.
   */
  public void testEnumerationParameter5() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String itemId1 = "Identificador do item 1";
    String itemName1 = "Nome do item 1";
    String itemValue1 = "Valor do item 1";
    String itemDescription1 = "Descrio do item 1";
    EnumerationItem item1 = new EnumerationItem(itemId1, itemName1, itemValue1,
      itemDescription1);
    String itemId2 = "Identificador do item 2";
    String itemName2 = "Nome do item 2";
    String itemValue2 = "Valor do item 2";
    String itemDescription2 = "Descrio do item 2";
    EnumerationItem item2 = new EnumerationItem(itemId2, itemName2, itemValue2,
      itemDescription2);
    String commandLinePattern = null;
    boolean mustSortItems = true;
    List<EnumerationItem> items = new LinkedList<EnumerationItem>();
    items.add(item2);
    items.add(item1);
    EnumerationParameter parameter = new EnumerationParameter(name, label,
      description, null, commandLinePattern, items, mustSortItems);
    Collections.sort(items);
    assertEquals(name, label, description, null, commandLinePattern, items,
      mustSortItems, parameter);
  }

  /**
   * Testando o construtor: criando um parmetro sem itens.
   */
  public void testEnumerationParameter6() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String commandLinePattern = null;
    boolean mustSortItems = true;
    List<EnumerationItem> items = new LinkedList<EnumerationItem>();
    EnumerationParameter parameter = new EnumerationParameter(name, label,
      description, null, commandLinePattern, items, mustSortItems);
    assertEquals(name, label, description, null, commandLinePattern, items,
      mustSortItems, parameter);
  }

  /**
   * Testando o construtor: com nome {@code null}.
   */
  public void testEnumerationParameter7() {
    String name = null;
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String commandLinePattern = null;
    boolean mustSortItems = true;
    List<EnumerationItem> items = new LinkedList<EnumerationItem>();
    try {
      new EnumerationParameter(name, label, description, null,
        commandLinePattern, items, mustSortItems);
      fail();
    }
    catch (IllegalArgumentException e) {
      assertEquals("name == null", e.getMessage());
    }
  }

  /**
   * Testando o construtor: com rtulo {@code null}.
   */
  public void testEnumerationParameter8() {
    String name = "Nome do parmetro";
    String label = null;
    String description = "Descrio do parmetro";
    String commandLinePattern = null;
    boolean mustSortItems = true;
    List<EnumerationItem> items = new LinkedList<EnumerationItem>();
    try {
      new EnumerationParameter(name, label, description, null,
        commandLinePattern, items, mustSortItems);
      fail();
    }
    catch (IllegalArgumentException e) {
      assertEquals("label == null", e.getMessage());
    }
  }

  /**
   * Testando o construtor: com descrio {@code null}.
   */
  public void testEnumerationParameter9() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = null;
    String commandLinePattern = null;
    boolean mustSortItems = true;
    List<EnumerationItem> items = new LinkedList<EnumerationItem>();
    try {
      new EnumerationParameter(name, label, description, null,
        commandLinePattern, items, mustSortItems);
      fail();
    }
    catch (IllegalArgumentException e) {
      assertEquals("description == null", e.getMessage());
    }
  }

  /**
   * Testando o construtor: com itens {@code null}.
   */
  public void testEnumerationParameter10() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String commandLinePattern = null;
    boolean mustSortItems = true;
    List<EnumerationItem> items = null;
    try {
      new EnumerationParameter(name, label, description, null,
        commandLinePattern, items, mustSortItems);
      fail();
    }
    catch (IllegalArgumentException e) {
      assertEquals("items == null", e.getMessage());
    }
  }

  /**
   * Verifica se os atributos fornecidos so iguais aos atributos do parmetro.
   * 
   * @param name O nome (Aceita {@code null}).
   * @param label O rtulo (Aceita {@code null}).
   * @param description A descrio (Aceita {@code null}).
   * @param defaultValue O valor-padro (Aceita {@code null}).
   * @param commandLinePattern O padro da linha de comando (Aceita {@code null}).
   * @param items Os itens (Aceita {@code null}).
   * @param mustSortItems indica se a enumerao deve estar ordenada.
   * @param parameter O parmetro (No aceita {@code null}).
   */
  private void assertEquals(String name, String label, String description,
    EnumerationItem defaultValue, String commandLinePattern,
    List<EnumerationItem> items, boolean mustSortItems,
    EnumerationParameter parameter) {
    assertEquals(name, parameter.getName());
    assertEquals(label, parameter.getLabel());
    assertEquals(description, parameter.getDescription());
    assertEquals(defaultValue, parameter.getDefaultValue());
    assertFalse(parameter.isOptional());
    assertEquals(commandLinePattern, parameter.getCommandLinePattern());
    assertEquals(items, parameter.getItems());
    assertEquals(mustSortItems, parameter.mustSortItems());
  }
}
