package csbase.logic.algorithms.parameters;

import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.exception.algorithms.ExpressionFunctionExecutionException;
import csbase.exception.algorithms.ExpressionFunctionNotFoundException;
import csbase.logic.algorithms.ExecutionType;

import junit.framework.TestCase;

import java.lang.reflect.Field;

import java.util.HashSet;
import java.util.Hashtable;
import java.util.Set;

import tecgraf.javautils.configurationmanager.ConfigurationManager;

public final class ExpressionFunctionExecutorTest extends TestCase {
  
  /**
   * Armazena o estado do ExpressionFunctionExecutor para ser recuperado no 
   * {@link ExpressionFunctionExecutorTest#tearDow()}. 
   */
  private Set<ExpressionFunction> executorInitialFunctions = 
    new HashSet<ExpressionFunction>(); 
  
  @Override
  protected void setUp() throws Exception {
    ExpressionFunctionExecutor expressionFunctionExecutor = 
      ExpressionFunctionExecutor.getInstance();
    
    executorInitialFunctions.clear();
    executorInitialFunctions.addAll(
      expressionFunctionExecutor.getExpressionFunctions());
        
    expressionFunctionExecutor.clear();
  }

  @Override
  protected void tearDown() throws Exception {
    ExpressionFunctionExecutor expressionFunctionExecutor = 
      ExpressionFunctionExecutor.getInstance();
    
    expressionFunctionExecutor.clear();    
    expressionFunctionExecutor.addAllExpressionFunction(executorInitialFunctions);    
  }
  
  static {
    try {
      ConfigurationManager.createInstance();
    }
    catch (IllegalStateException e) {}
  }

  public void testGetInstance1() {
    assertNotNull(ExpressionFunctionExecutor.getInstance());
    assertSame(ExpressionFunctionExecutor.getInstance(),
      ExpressionFunctionExecutor.getInstance());
  }

  public void testAddExpressionFunction1() {
    MockExpressionFunction mockExpressionFunction = new MockExpressionFunction(
      "Nome da funo", new Class[] {});
    assertTrue(ExpressionFunctionExecutor.getInstance().addExpressionFunction(
      mockExpressionFunction));
    Set<ExpressionFunction> expressionFunctions = new HashSet<ExpressionFunction>();
    expressionFunctions.add(mockExpressionFunction);
    assertEquals(expressionFunctions, ExpressionFunctionExecutor.getInstance()
      .getExpressionFunctions());
  }

  public void testAddExpressionFunction2() {
    MockExpressionFunction mockExpressionFunction = new MockExpressionFunction(
      "Nome da funo", new Class[] {});
    assertTrue(ExpressionFunctionExecutor.getInstance().addExpressionFunction(
      mockExpressionFunction));
    Set<ExpressionFunction> expressionFunctions = new HashSet<ExpressionFunction>();
    expressionFunctions.add(mockExpressionFunction);
    assertEquals(expressionFunctions, ExpressionFunctionExecutor.getInstance()
      .getExpressionFunctions());
    assertFalse(ExpressionFunctionExecutor.getInstance().addExpressionFunction(
      mockExpressionFunction));
    assertEquals(expressionFunctions, ExpressionFunctionExecutor.getInstance()
      .getExpressionFunctions());
  }

  public void testAddExpressionFunction3() {
    try {
      ExpressionFunctionExecutor.getInstance().addExpressionFunction(null);
      fail("Esperada IllegalArgumentException.");
    } catch (IllegalArgumentException e) {
    }
  }

  public void testGetExpressionFunctions1() {
    Set<ExpressionFunction> expressionFunctions = ExpressionFunctionExecutor
      .getInstance().getExpressionFunctions();
    assertNotNull(expressionFunctions);
    assertNotNull(expressionFunctions.isEmpty());
  }

  public void testGetExpressionFunctions2() {
    MockExpressionFunction mockExpressionFunction = new MockExpressionFunction(
      "Nome da funo", new Class[] {});
    assertTrue(ExpressionFunctionExecutor.getInstance().addExpressionFunction(
      mockExpressionFunction));
    Set<ExpressionFunction> expressionFunctions = new HashSet<ExpressionFunction>();
    expressionFunctions.add(mockExpressionFunction);
    assertEquals(expressionFunctions, ExpressionFunctionExecutor.getInstance()
      .getExpressionFunctions());
    try {
      ExpressionFunctionExecutor.getInstance().getExpressionFunctions().clear();
      fail("Esperada UnsupportedOperationException.");
    } catch (UnsupportedOperationException e) {
    }
    assertEquals(expressionFunctions, ExpressionFunctionExecutor.getInstance()
      .getExpressionFunctions());
  }

  public void testExecute1() throws ExpressionFunctionExecutionException,
    ExpressionFunctionNotFoundException {
    AlgorithmInfo algorithm = new AlgorithmInfo("Identificador do algoritmo",
      "Nome do algoritmo", "Diretro do algoritmo", new Hashtable());
    AlgorithmVersionInfo version = new AlgorithmVersionInfo(algorithm,
      new AlgorithmVersionId(1, 2, 3), new Hashtable(), new Hashtable());
    SimpleAlgorithmConfigurator configurator = new SimpleAlgorithmConfigurator(
      version, null, ExecutionType.SIMPLE, null, false, "Comando", null, null,
      true);
    String functionName = "Nome da funo";
    Class[] parameterTypes = new Class[] { String.class, Integer.class };
    Object[] parameters = new Object[] { "Parmetro 1", 2 };
    Object result = "Resultado";
    CheckExpressionFunction function = new CheckExpressionFunction(
      functionName, parameterTypes, configurator, parameters, result);
    assertFalse(function.wasDoneOperation());
    ExpressionFunctionExecutor.getInstance().addExpressionFunction(function);
    assertEquals(result, ExpressionFunctionExecutor.getInstance().execute(
      functionName, configurator, parameters));
  }

  public void testExecute2() throws ExpressionFunctionExecutionException {
    AlgorithmInfo algorithm = new AlgorithmInfo("Identificador do algoritmo",
      "Nome do algoritmo", "Diretro do algoritmo", new Hashtable());
    AlgorithmVersionInfo version = new AlgorithmVersionInfo(algorithm,
      new AlgorithmVersionId(1, 2, 3), new Hashtable(), new Hashtable());
    SimpleAlgorithmConfigurator configurator = new SimpleAlgorithmConfigurator(
      version, null, ExecutionType.SIMPLE, null, false, "Comando", null, null,
      true);
    try {
      ExpressionFunctionExecutor.getInstance().execute("Funo que no existe",
        configurator, new Object[] {});
      fail(ExpressionFunctionNotFoundException.class.getName());
    } catch (ExpressionFunctionNotFoundException e) {
    }
  }

  public void testExecute3() throws ExpressionFunctionExecutionException {
    AlgorithmInfo algorithm = new AlgorithmInfo("Identificador do algoritmo",
      "Nome do algoritmo", "Diretro do algoritmo", new Hashtable());
    AlgorithmVersionInfo version = new AlgorithmVersionInfo(algorithm,
      new AlgorithmVersionId(1, 2, 3), new Hashtable(), new Hashtable());
    SimpleAlgorithmConfigurator configurator = new SimpleAlgorithmConfigurator(
      version, null, ExecutionType.SIMPLE, null, false, "Comando", null, null,
      true);
    String functionName = "Nome da funo";
    Class[] parameterTypes = new Class[] { String.class, Integer.class };
    Object[] parameters = new Object[] { "Parmetro 1", 2 };
    Object result = "Resultado";
    CheckExpressionFunction function = new CheckExpressionFunction(
      functionName, parameterTypes, configurator, parameters, result);
    assertFalse(function.wasDoneOperation());
    ExpressionFunctionExecutor.getInstance().addExpressionFunction(function);
    try {
      ExpressionFunctionExecutor.getInstance().execute(
        "Nome de funo que no existe", configurator, parameters);
      fail(ExpressionFunctionNotFoundException.class.getName());
    } catch (ExpressionFunctionNotFoundException e) {
    }
  }

  public void testExecute4() throws ExpressionFunctionExecutionException {
    AlgorithmInfo algorithm = new AlgorithmInfo("Identificador do algoritmo",
      "Nome do algoritmo", "Diretro do algoritmo", new Hashtable());
    AlgorithmVersionInfo version = new AlgorithmVersionInfo(algorithm,
      new AlgorithmVersionId(1, 2, 3), new Hashtable(), new Hashtable());
    SimpleAlgorithmConfigurator configurator = new SimpleAlgorithmConfigurator(
      version, null, ExecutionType.SIMPLE, null, false, "Comando", null, null,
      true);
    String functionName = "Nome da funo";
    Class[] parameterTypes = new Class[] { String.class, Integer.class };
    Object[] parameters = new Object[] { "Parmetro 1", 2 };
    Object result = "Resultado";
    CheckExpressionFunction function = new CheckExpressionFunction(
      functionName, parameterTypes, configurator, parameters, result);
    assertFalse(function.wasDoneOperation());
    ExpressionFunctionExecutor.getInstance().addExpressionFunction(function);
    try {
      ExpressionFunctionExecutor.getInstance().execute(functionName,
        configurator, new Object[] { 1.3, 'a', true });
      fail(ExpressionFunctionNotFoundException.class.getName());
    } catch (ExpressionFunctionNotFoundException e) {
    }
  }

  public void testExecute5() throws ExpressionFunctionExecutionException,
    ExpressionFunctionNotFoundException {
    AlgorithmInfo algorithm = new AlgorithmInfo("Identificador do algoritmo",
      "Nome do algoritmo", "Diretro do algoritmo", new Hashtable());
    AlgorithmVersionInfo version = new AlgorithmVersionInfo(algorithm,
      new AlgorithmVersionId(1, 2, 3), new Hashtable(), new Hashtable());
    SimpleAlgorithmConfigurator configurator = new SimpleAlgorithmConfigurator(
      version, null, ExecutionType.SIMPLE, null, false, "Comando", null, null,
      true);
    try {
      ExpressionFunctionExecutor.getInstance().execute(null, configurator,
        new Object[] { 1.3, 'a', true });
      fail(IllegalArgumentException.class.getName());
    } catch (IllegalArgumentException e) {
    }
  }

  public void testExecute6() throws ExpressionFunctionExecutionException,
    ExpressionFunctionNotFoundException {
    try {
      ExpressionFunctionExecutor.getInstance().execute("Nome de funo", null,
        new Object[] { 1.3, 'a', true });
      fail(IllegalArgumentException.class.getName());
    } catch (IllegalArgumentException e) {
    }
  }

  public void testExecute7() throws ExpressionFunctionExecutionException,
    ExpressionFunctionNotFoundException {
    AlgorithmInfo algorithm = new AlgorithmInfo("Identificador do algoritmo",
      "Nome do algoritmo", "Diretro do algoritmo", new Hashtable());
    AlgorithmVersionInfo version = new AlgorithmVersionInfo(algorithm,
      new AlgorithmVersionId(1, 2, 3), new Hashtable(), new Hashtable());
    SimpleAlgorithmConfigurator configurator = new SimpleAlgorithmConfigurator(
      version, null, ExecutionType.SIMPLE, null, false, "Comando", null, null,
      true);
    try {
      ExpressionFunctionExecutor.getInstance().execute("Nome de funo",
        configurator, (Object[]) null);
      fail(IllegalArgumentException.class.getName());
    } catch (IllegalArgumentException e) {
    }
  }

  public void testCreateReflexiveExpressionFunctions1()
    throws NoSuchMethodException {
    ExpressionFunctionExecutor.getInstance()
      .createReflexiveExpressionFunctions(
        ClassWithOnlyOnePublicStaticMethod.class);
    String functionName = "method1";
    Class[] parameterTypes = new Class[] { Integer.class, Float.class, Object.class };
    ReflectionExpressionFunction expressionFunction = new ReflectionExpressionFunction(
      ClassWithOnlyOnePublicStaticMethod.class.getMethod(functionName,
        parameterTypes));
    Set<ExpressionFunction> expressionFunctions = new HashSet<ExpressionFunction>();
    expressionFunctions.add(expressionFunction);
    assertEquals(expressionFunctions, ExpressionFunctionExecutor.getInstance()
      .getExpressionFunctions());
  }

  public void testCreateReflexiveExpressionFunctions2() {
    ExpressionFunctionExecutor.getInstance()
      .createReflexiveExpressionFunctions(ClassWithoutStaticMethods.class);
    assertNotNull(ExpressionFunctionExecutor.getInstance()
      .getExpressionFunctions());
    assertTrue(ExpressionFunctionExecutor.getInstance()
      .getExpressionFunctions().isEmpty());
  }

  public void testCreateReflexiveExpressionFunctions3() {
    ExpressionFunctionExecutor.getInstance()
      .createReflexiveExpressionFunctions(ClassWithoutPublicMethods.class);
    assertNotNull(ExpressionFunctionExecutor.getInstance()
      .getExpressionFunctions());
    assertTrue(ExpressionFunctionExecutor.getInstance()
      .getExpressionFunctions().isEmpty());
  }

  public void testCreateReflexiveExpressionFunctions4()
    throws NoSuchMethodException {
    ExpressionFunctionExecutor.getInstance()
      .createReflexiveExpressionFunctions(
        ClassWithStaticAndNonStaticMethods.class);
    String functionName = "method1";
    Class[] parameterTypes = new Class[] { Integer.class, Float.class, Object.class };
    ReflectionExpressionFunction expressionFunction = new ReflectionExpressionFunction(
      ClassWithOnlyOnePublicStaticMethod.class.getMethod(functionName,
        parameterTypes));
    Set<ExpressionFunction> expressionFunctions = new HashSet<ExpressionFunction>();
    expressionFunctions.add(expressionFunction);
    assertEquals(expressionFunctions, ExpressionFunctionExecutor.getInstance()
      .getExpressionFunctions());
  }

  public void testCreateReflexiveExpressionFunctions5()
    throws NoSuchMethodException {
    ExpressionFunctionExecutor.getInstance()
      .createReflexiveExpressionFunctions(
        ClassWithPublicAndNonPublicMethods.class);
    String functionName = "method1";
    Class[] parameterTypes = new Class[] { Integer.class, Float.class, Object.class };
    ReflectionExpressionFunction expressionFunction = new ReflectionExpressionFunction(
      ClassWithOnlyOnePublicStaticMethod.class.getMethod(functionName,
        parameterTypes));
    Set<ExpressionFunction> expressionFunctions = new HashSet<ExpressionFunction>();
    expressionFunctions.add(expressionFunction);
    assertEquals(expressionFunctions, ExpressionFunctionExecutor.getInstance()
      .getExpressionFunctions());
  }

  public void testCreateReflexiveExpressionFunctions6() {
    ExpressionFunctionExecutor.getInstance()
      .createReflexiveExpressionFunctions(ClassWithoutMethods.class);
    assertNotNull(ExpressionFunctionExecutor.getInstance()
      .getExpressionFunctions());
    assertTrue(ExpressionFunctionExecutor.getInstance()
      .getExpressionFunctions().isEmpty());
  }

  private static class ClassWithOnlyOnePublicStaticMethod {
    @SuppressWarnings("unused")
    public static String method1(Integer param1, Float param2, Object param3) {
      return null;
    }
  }

  private static class ClassWithoutStaticMethods {
    @SuppressWarnings("unused")
    public String method1(Integer param1, Float param2, Object param3) {
      return null;
    }
  }

  private static class ClassWithoutPublicMethods {
    @SuppressWarnings("unused")
    static String method1(Integer param1, Float param2, Object param3) {
      return null;
    }
  }

  private static class ClassWithStaticAndNonStaticMethods {
    @SuppressWarnings("unused")
    public static String method1(Integer param1, Float param2, Object param3) {
      return null;
    }

    public int method2() {
      return 0;
    }
  }

  private static class ClassWithPublicAndNonPublicMethods {
    @SuppressWarnings("unused")
    public static String method1(Integer param1, Float param2, Object param3) {
      return null;
    }

    static int method2() {
      return 0;
    }
  }

  private static class ClassWithoutMethods {

  }
}
