package csbase.logic.algorithms.parameters;

import java.util.Arrays;
import java.util.Collections;
import java.util.Hashtable;

import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.exception.algorithms.ExpressionFunctionExecutionException;
import csbase.logic.algorithms.ExecutionType;

import junit.framework.TestCase;

public final class ExpressionFunctionTest extends TestCase {

  public void testExpressionFunction1() {
    String functionName = "Nome da funo";
    Class[] parameterTypes = new Class[] { Object.class, String.class, Integer.class };
    ExpressionFunction expressionFunction = new MockExpressionFunction(
      functionName, parameterTypes);
    assertEquals(functionName, expressionFunction.getName());
    assertEquals(Arrays.asList(parameterTypes), expressionFunction
      .getParameterTypes());
  }

  public void testExpressionFunction2() {
    String functionName = "Nome da funo";
    ExpressionFunction expressionFunction = new MockExpressionFunction(
      functionName);
    assertEquals(functionName, expressionFunction.getName());
    assertEquals(Collections.EMPTY_LIST, expressionFunction.getParameterTypes());
  }

  public void testExpressionFunction3() {
    try {
      new MockExpressionFunction(null);
      fail("Esperada IllegalArgumentException.");
    } catch (IllegalArgumentException e) {
    }
  }

  public void testExpressionFunction4() {
    try {
      new MockExpressionFunction("Nome da funo", (Class[]) null);
      fail("Esperada IllegalArgumentException.");
    } catch (IllegalArgumentException e) {
    }
  }

  public void testExpressionFunction5() {
    try {
      new MockExpressionFunction("Nome da funo", new Class[] { null });
      fail("Esperada IllegalArgumentException.");
    } catch (IllegalArgumentException e) {
    }
  }

  public void testGetParameterTypes1() {
    String functionName = "Nome da funo";
    Class[] parameterTypes = new Class[] { Object.class, String.class, Integer.class };
    ExpressionFunction expressionFunction = new MockExpressionFunction(
      functionName, parameterTypes);
    assertEquals(functionName, expressionFunction.getName());
    assertEquals(Arrays.asList(parameterTypes), expressionFunction
      .getParameterTypes());
    try {
      expressionFunction.getParameterTypes().clear();
      fail("Esperada UnsupportedOperationException.");
    } catch (UnsupportedOperationException e) {
    }
    assertEquals(Arrays.asList(parameterTypes), expressionFunction
      .getParameterTypes());
  }

  public void testEquals1() {
    String name = "Nome da funo";
    Class[] parameterTypes = new Class[] { Object.class, String.class };

    ExpressionFunction function1 = new MockExpressionFunction(name,
      parameterTypes);
    ExpressionFunction function2 = new MockExpressionFunction(name,
      parameterTypes);

    assertTrue(function1.equals(function2));
  }

  public void testEquals2() {
    String name = "Nome da funo";

    Class[] parameterTypes1 = new Class[] { Object.class, String.class };
    Class[] parameterTypes2 = new Class[] { Integer.class, Float.class };

    ExpressionFunction function1 = new MockExpressionFunction(name,
      parameterTypes1);
    ExpressionFunction function2 = new MockExpressionFunction(name,
      parameterTypes2);

    assertFalse(function1.equals(function2));
  }

  public void testEquals3() {
    String name1 = "Nome da funo 1";
    String name2 = "Nome da funo 2";

    Class[] parameterTypes = new Class[] { Object.class, String.class };

    ExpressionFunction function1 = new MockExpressionFunction(name1,
      parameterTypes);
    ExpressionFunction function2 = new MockExpressionFunction(name2,
      parameterTypes);

    assertFalse(function1.equals(function2));
  }

  public void testEquals4() {
    String functionName = "Nome da funo";
    Class[] parameterTypes = new Class[] { Object.class, String.class, Integer.class };
    ExpressionFunction expressionFunction = new MockExpressionFunction(
      functionName, parameterTypes);
    assertFalse(expressionFunction.equals(null));
  }

  public void testEquals5() {
    String functionName = "Nome da funo";
    Class[] parameterTypes = new Class[] { Object.class, String.class, Integer.class };
    ExpressionFunction expressionFunction = new MockExpressionFunction(
      functionName, parameterTypes);
    assertFalse(expressionFunction.equals(functionName));
  }

  public void testHashCode() {
    String functionName = "Nome da funo";
    Class[] parameterTypes = new Class[] { Object.class, String.class, Integer.class };
    ExpressionFunction expressionFunction1 = new MockExpressionFunction(
      functionName, parameterTypes);
    ExpressionFunction expressionFunction2 = new MockExpressionFunction(
      functionName, parameterTypes);
    assertEquals(expressionFunction1.hashCode(), expressionFunction2.hashCode());
  }

  public void testAcceptParameters1() {
    String functionName = "Nome da funo";
    Class[] parameterTypes = new Class[] { Float.class, String.class, Integer.class };
    ExpressionFunction expressionFunction = new MockExpressionFunction(
      functionName, parameterTypes);
    assertTrue(expressionFunction
      .acceptParameters(new Object[] { 1f, "String", 10 }));

  }

  public void testAcceptParameters2() {
    String functionName = "Nome da funo";
    Class[] parameterTypes = new Class[] { Float.class, String.class, Integer.class };
    ExpressionFunction expressionFunction = new MockExpressionFunction(
      functionName, parameterTypes);
    assertFalse(expressionFunction
      .acceptParameters(new Object[] { 1f, "String" }));

  }

  public void testAcceptParameters3() {
    String functionName = "Nome da funo";
    Class[] parameterTypes = new Class[] { Float.class, String.class, Integer.class };
    ExpressionFunction expressionFunction = new MockExpressionFunction(
      functionName, parameterTypes);
    assertFalse(expressionFunction
      .acceptParameters(new Object[] { "String", 10, 1f }));
  }

  public void testAcceptParameters4() {
    String functionName = "Nome da funo";
    Class[] parameterTypes = new Class[] { ExpressionFunction.class };
    ExpressionFunction expressionFunction = new MockExpressionFunction(
      functionName, parameterTypes);
    assertTrue(expressionFunction
      .acceptParameters(new Object[] { expressionFunction }));

  }

  public void testAcceptParameters5() {
    String functionName = "Nome da funo";
    Class[] parameterTypes = new Class[] { ExpressionFunction.class };
    ExpressionFunction expressionFunction = new MockExpressionFunction(
      functionName, parameterTypes);
    try {
      expressionFunction.acceptParameters((Object[]) null);
      fail("Esperada IllegalArgumentException.");
    } catch (IllegalArgumentException e) {
    }
  }

  public void testAcceptParameters6() {
    String functionName = "Nome da funo";
    Class[] parameterTypes = new Class[] { String.class, String.class };
    ExpressionFunction expressionFunction = new MockExpressionFunction(
      functionName, parameterTypes);
    assertTrue(expressionFunction
      .acceptParameters(new Object[] { "Parmetro 1", null }));
  }

  public void testExecute1() throws ExpressionFunctionExecutionException {
    String functionName = "Nome da funo";
    Class[] parameterTypes = new Class[] { Float.class, String.class, Integer.class };
    String result = "Resultado";
    Object[] parameters = new Object[] { 1f, "Texto", 3 };
    AlgorithmInfo algorithm = new AlgorithmInfo("Identificador do algoritmo",
      "Nome do algoritmo", "Diretro do algoritmo", new Hashtable());
    AlgorithmVersionInfo version = new AlgorithmVersionInfo(algorithm,
      new AlgorithmVersionId(1, 2, 3), new Hashtable(), new Hashtable());
    SimpleAlgorithmConfigurator configurator = new SimpleAlgorithmConfigurator(
      version, null, ExecutionType.SIMPLE, null, false, "Comando", null, null,
      true);
    CheckExpressionFunction expressionFunction = new CheckExpressionFunction(
      functionName, parameterTypes, configurator, parameters, result);
    assertEquals(result, expressionFunction.execute(configurator, parameters));
    assertTrue(expressionFunction.wasDoneOperation());
  }

  public void testExecute2() {
    String functionName = "Nome da funo";
    Class[] parameterTypes = new Class[] { Float.class, String.class, Integer.class };
    CheckExpressionFunction expressionFunction = new CheckExpressionFunction(
      functionName, parameterTypes);
    AlgorithmInfo algorithm = new AlgorithmInfo("Identificador do algoritmo",
      "Nome do algoritmo", "Diretro do algoritmo", new Hashtable());
    AlgorithmVersionInfo version = new AlgorithmVersionInfo(algorithm,
      new AlgorithmVersionId(1, 2, 3), new Hashtable(), new Hashtable());
    SimpleAlgorithmConfigurator configurator = new SimpleAlgorithmConfigurator(
      version, null, ExecutionType.SIMPLE, null, false, "Comando", null, null,
      true);
    try {
      expressionFunction.execute(configurator, new Object[] { 1f, "Texto" });
      fail(ExpressionFunctionExecutionException.class.getName());
    } catch (ExpressionFunctionExecutionException e) {
    }
    assertFalse(expressionFunction.wasDoneOperation());
  }

  public void testExecute3() {
    String functionName = "Nome da funo";
    Class[] parameterTypes = new Class[] { Float.class, String.class, Integer.class };
    CheckExpressionFunction expressionFunction = new CheckExpressionFunction(
      functionName, parameterTypes);
    AlgorithmInfo algorithm = new AlgorithmInfo("Identificador do algoritmo",
      "Nome do algoritmo", "Diretro do algoritmo", new Hashtable());
    AlgorithmVersionInfo version = new AlgorithmVersionInfo(algorithm,
      new AlgorithmVersionId(1, 2, 3), new Hashtable(), new Hashtable());
    SimpleAlgorithmConfigurator configurator = new SimpleAlgorithmConfigurator(
      version, null, ExecutionType.SIMPLE, null, false, "Comando", null, null,
      true);
    try {
      expressionFunction.execute(configurator, new Object[] { "Texto", 1, 1f });
      fail(ExpressionFunctionExecutionException.class.getName());
    } catch (ExpressionFunctionExecutionException e) {
    }
    assertFalse(expressionFunction.wasDoneOperation());
  }

  public void testExecute4() throws ExpressionFunctionExecutionException {
    String functionName = "Nome da funo";
    Class[] parameterTypes = new Class[] { Float.class, String.class, Integer.class };
    CheckExpressionFunction expressionFunction = new CheckExpressionFunction(
      functionName, parameterTypes);
    AlgorithmInfo algorithm = new AlgorithmInfo("Identificador do algoritmo",
      "Nome do algoritmo", "Diretro do algoritmo", new Hashtable());
    AlgorithmVersionInfo version = new AlgorithmVersionInfo(algorithm,
      new AlgorithmVersionId(1, 2, 3), new Hashtable(), new Hashtable());
    SimpleAlgorithmConfigurator configurator = new SimpleAlgorithmConfigurator(
      version, null, ExecutionType.SIMPLE, null, false, "Comando", null, null,
      true);
    try {
      expressionFunction.execute(configurator, (Object[]) null);
      fail(IllegalArgumentException.class.getName());
    } catch (IllegalArgumentException e) {
    }
    assertFalse(expressionFunction.wasDoneOperation());
  }
}
