package csbase.logic.algorithms.parameters;

import junit.framework.TestCase;
import csbase.logic.ProjectFileType;
import csbase.logic.algorithms.FileParameterValue;

/**
 * Testes unitrios para a classe {@link FileParameter}.
 */
public final class FileParameterTest extends TestCase {
  /**
   * Testando criar um parmetro do tipo arquivo opcional, com padro para linha
   * de comando padro,.com o modo arquivo regular, com seleo simples, que
   * aceite pipe e que precisa ordenar os caminhos para os arquivos.
   */
  public void testFileParameter1() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    String commandLinePattern = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
    String type = "Tipo de arquivo";
    FileParameterMode mode = FileParameterMode.REGULAR_FILE;
    boolean isMultipleSelection = false;
    FileParameterPipeAcceptance canUsePipe = FileParameterPipeAcceptance.TRUE;
    boolean mustSort = true;
    FileParameter parameter =
      new MockFileParameter(name, label, description, isOptional,
        commandLinePattern, type, mode, isMultipleSelection, canUsePipe,
        mustSort);
    assertFileParameter(name, label, description, isOptional,
      commandLinePattern, type, mode, canUsePipe, parameter);
  }

  /**
   * Testando criar um parmetro do tipo arquivo obrigatrio.
   */
  public void testFileParameter2() {
    boolean isOptional = false;
    FileParameter parameter =
      new MockFileParameter("Nome do parmetro", "Rtulo do parmetro",
        "Descrio do parmetro", isOptional,
        "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO", "Tipo de arquivo",
        FileParameterMode.REGULAR_FILE, false,
        FileParameterPipeAcceptance.TRUE, true);
    assertEquals(isOptional, parameter.isOptional());
  }

  /**
   * Testando criar um parmetro do tipo arquivo sem tipo.
   */
  public void testFileParameter3() {
    boolean isOptional = false;
    FileParameter parameter =
      new MockFileParameter("Nome do parmetro", "Rtulo do parmetro",
        "Descrio do parmetro", isOptional,
        "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO", null,
        FileParameterMode.REGULAR_FILE, false,
        FileParameterPipeAcceptance.TRUE, true);
    assertNull(parameter.getFileType());
  }

  /**
   * Testando criar um parmetro com nome {@code null}.
   */
  public void testFileParameter4() {
    try {
      new MockFileParameter(null, "Rtulo do parmetro",
        "Descrio do parmetro", false,
        "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO", null,
        FileParameterMode.REGULAR_FILE, false,
        FileParameterPipeAcceptance.TRUE, true);
      fail("No deveria ser possvel criar um parmetro sem nome.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um parmetro com rtulo {@code null}.
   */
  public void testFileParameter5() {
    try {
      new MockFileParameter("Nome do parmetro", null,
        "Descrio do parmetro", false,
        "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO", null,
        FileParameterMode.REGULAR_FILE, false,
        FileParameterPipeAcceptance.TRUE, true);
      fail("No deveria ser possvel criar um parmetro sem rtulo.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um parmetro com descrio {@code null}.
   */
  public void testFileParameter6() {
    try {
      new MockFileParameter("Nome do parmetro", "Rtulo do parmetro", null,
        false, "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO", null,
        FileParameterMode.REGULAR_FILE, false,
        FileParameterPipeAcceptance.TRUE, true);
      fail("No deveria ser possvel criar um parmetro sem descrio.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um parmetro com padro de linha de comando diferente do
   * padro.
   */
  public void testFileParameter7() {
    String commandLinePattern = "$VALOR_DO_PARAMETRO";
    FileParameter parameter =
      new MockFileParameter("Nome", "Rtulo", "Descrio", false,
        commandLinePattern, null, FileParameterMode.REGULAR_FILE, false,
        FileParameterPipeAcceptance.TRUE, true);
    assertEquals(commandLinePattern, parameter.getCommandLinePattern());
  }

  /**
   * Testando criar um parmetro com modo {@link FileParameterMode#DIRECTORY}.
   */
  public void testFileParameter8() {
    FileParameterMode mode = FileParameterMode.DIRECTORY;
    FileParameter parameter =
      new MockFileParameter("Nome", "Rtulo", "Descrio", false,
        "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO", null, mode, false,
        FileParameterPipeAcceptance.TRUE, true);
    assertEquals(mode, parameter.getMode());
  }

  /**
   * Testando criar um parmetro com modo
   * {@link FileParameterMode#DIRECTORY_AND_REGULAR_FILE}.
   */
  public void testFileParameter9() {
    FileParameterMode mode = FileParameterMode.DIRECTORY_AND_REGULAR_FILE;
    FileParameter parameter =
      new MockFileParameter("Nome", "Rtulo", "Descrio", false,
        "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO", null, mode, false,
        FileParameterPipeAcceptance.TRUE, true);
    assertEquals(mode, parameter.getMode());
  }

  /**
   * Testando criar um parmetro que no aceite pipe.
   */
  public void testFileParameter11() {
    FileParameterPipeAcceptance canUsePipe = FileParameterPipeAcceptance.FALSE;
    FileParameter parameter =
      new MockFileParameter("Nome", "Rtulo", "Descrio", false,
        "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO", null,
        FileParameterMode.REGULAR_FILE, false, canUsePipe, true);
    assertEquals(canUsePipe, parameter.usesPipe());
  }

  /**
   * Testando o mtodo {@link FileParameter#getValueAsText()}: Sem valor
   * preenchido.
   */
  public void testGetValueAsText1() {
    FileParameter parameter =
      new MockFileParameter("Nome do parmetro", "Rtulo do parmetro",
        "Descrio do parmetro", false,
        "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO", "TEXT",
        FileParameterMode.REGULAR_FILE, false,
        FileParameterPipeAcceptance.TRUE, true);
    assertNull(parameter.getValueAsText());
  }

  /**
   * Testando o mtodo {@link FileParameter#getValueAsText()}: Com 1 arquivo
   * (com tipo).
   */
  public void testGetValueAsText2() {
    FileParameter parameter =
      new MockFileParameter("Nome do parmetro", "Rtulo do parmetro",
        "Descrio do parmetro", false,
        "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO", "TEXT",
        FileParameterMode.REGULAR_FILE, false,
        FileParameterPipeAcceptance.TRUE, true);
    assertNull(parameter.getValueAsText());
    FileParameterValue file = new FileParameterValue("dir1/dir2/arq", "DOC");
    parameter.setValue(file);
    assertEquals("dir1/dir2/arq:DOC", parameter.getValueAsText());
  }

  /**
   * Testando o mtodo {@link FileParameter#getValueAsText()}: Com 1 arquivo
   * (sem tipo).
   */
  public void testGetValueAsText3() {
    FileParameter parameter =
      new MockFileParameter("Nome do parmetro", "Rtulo do parmetro",
        "Descrio do parmetro", false,
        "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO", "TEXT",
        FileParameterMode.REGULAR_FILE, false,
        FileParameterPipeAcceptance.TRUE, true);
    assertNull(parameter.getValueAsText());
    FileParameterValue file = new FileParameterValue("dir1/dir2/arq");
    parameter.setValue(file);
    assertEquals("dir1/dir2/arq:" + ProjectFileType.UNKNOWN, parameter
      .getValueAsText());
  }

  /**
   * Testando o mtodo {@link FileParameter#setValueAsText(String)}: Sem valor.
   */
  public void testSetValueAsText1() {
    FileParameter parameter =
      new MockFileParameter("Nome do parmetro", "Rtulo do parmetro",
        "Descrio do parmetro", false,
        "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO", "TEXT",
        FileParameterMode.REGULAR_FILE, false,
        FileParameterPipeAcceptance.TRUE, true);
    FileParameterValue file = new FileParameterValue("dir1/dir2/arq", "DOC");
    parameter.setValue(file);
    assertEquals(file, parameter.getValue());
    parameter.setValueAsText(null);
    assertNull(parameter.getValue());
  }

  /**
   * Testando o mtodo {@link FileParameter#setValueAsText(String)}: Com 1 valor
   * (sem tipo).
   */
  public void testSetValueAsText2() {
    FileParameter parameter =
      new MockFileParameter("Nome do parmetro", "Rtulo do parmetro",
        "Descrio do parmetro", false,
        "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO", "TEXT",
        FileParameterMode.REGULAR_FILE, false,
        FileParameterPipeAcceptance.TRUE, true);
    parameter.setValueAsText("dir1/dir2/arq:" + ProjectFileType.UNKNOWN);
    FileParameterValue file = new FileParameterValue("dir1/dir2/arq");
    assertEquals(file, parameter.getValue());
  }

  /**
   * Testando o mtodo {@link FileParameter#setValueAsText(String)}: Com 1 valor
   * (com tipo).
   */
  public void testSetValueAsText3() {
    FileParameter parameter =
      new MockFileParameter("Nome do parmetro", "Rtulo do parmetro",
        "Descrio do parmetro", false,
        "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO", "TEXT",
        FileParameterMode.REGULAR_FILE, false,
        FileParameterPipeAcceptance.TRUE, true);
    parameter.setValueAsText("dir1/dir2/arq:DOC");
    FileParameterValue file = new FileParameterValue("dir1/dir2/arq", "DOC");
    assertEquals(file, parameter.getValue());
  }

  /**
   * Testando o mtodo {@link FileParameter#getCommandLine(String, String)}: Com
   * 1 valor e com separador Unix.
   */
  public void testGetCommandLine1() {
    FileParameter parameter =
      new MockFileParameter("PARAM_NAME", "Rtulo do parmetro",
        "Descrio do parmetro", false,
        "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO", "TEXT",
        FileParameterMode.DIRECTORY, false, FileParameterPipeAcceptance.TRUE,
        true);
    parameter.setValueAsText("dir1/dir2/dir3:DIRECTORY_TYPE");
    assertEquals("PARAM_NAME=../../../../../../project/usuario/"
      + "Projeto/dir1/dir2/dir3", parameter.getCommandLine(
      "project/usuario/Projeto", "/"));
  }

  /**
   * Testando o mtodo {@link FileParameter#getCommandLine(String, String)}: Com
   * separador Windows.
   */
  public void testGetCommandLine2() {
    FileParameter parameter =
      new MockFileParameter("PARAM_NAME", "Rtulo do parmetro",
        "Descrio do parmetro", false,
        "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO", "TEXT",
        FileParameterMode.DIRECTORY, false, FileParameterPipeAcceptance.TRUE,
        true);
    parameter.setValueAsText("dir1/dir2/dir3:DIRECTORY_TYPE");
    assertEquals("PARAM_NAME=..\\..\\..\\..\\..\\..\\project\\usuario\\Projeto"
      + "\\dir1\\dir2\\dir3", parameter.getCommandLine(
      "project\\usuario\\Projeto", "\\"));
  }

  /**
   * Testando o mtodo {@link FileParameter#getCommandLine(String, String)}: Sem
   * valor.
   */
  public void testGetCommandLine3() {
    FileParameter parameter =
      new MockFileParameter("PARAM_NAME", "Rtulo do parmetro",
        "Descrio do parmetro", false,
        "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO", "TEXT",
        FileParameterMode.REGULAR_FILE, false,
        FileParameterPipeAcceptance.TRUE, true);
    assertNull(parameter.getCommandLine("project/usuario/Projeto", "/"));
  }

  /**
   * Verifica se todos os atributos do parmetro informado so iguais aos
   * valores esperados.
   * 
   * @param name O nome (Aceita {@code null}).
   * @param label O rtulo (Aceita {@code null}).
   * @param description A descrio (Aceita {@code null}).
   * @param isOptional Indica se o parmetro  opcional.
   * @param commandLinePattern O padro para a linha de comando (Aceita {@code
   *        null}).
   * @param type O tipo (Aceita {@code null}).
   * @param mode O modo (Aceita {@code null}).
   * @param canUsePipe Indica se o parmetro aceita pipe.
   * @param parameter O parmetro.(No aceita {@code null}).
   */
  private void assertFileParameter(String name, String label,
    String description, boolean isOptional, String commandLinePattern,
    String type, FileParameterMode mode,
    FileParameterPipeAcceptance canUsePipe, FileParameter parameter) {
    assertNotNull(parameter);
    assertEquals(name, parameter.getName());
    assertEquals(label, parameter.getLabel());
    assertEquals(description, parameter.getDescription());
    assertEquals(isOptional, parameter.isOptional());
    assertEquals(commandLinePattern, parameter.getCommandLinePattern());
    assertEquals(type, parameter.getFileType());
    assertEquals(mode, parameter.getMode());
    assertEquals(canUsePipe, parameter.usesPipe());
    assertNull(parameter.getDefaultValue());
  }
}
