/*
 * $Author:$
 * $Date:$
 * $Release:$
 */
package csbase.logic.algorithms.parameters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.LinkedList;
import java.util.List;

import junit.framework.Assert;
import junit.framework.TestCase;
import csbase.exception.ParseException;

/**
 * Testes unitrios para a classe {@link IntegerParameter}.
 *
 * @author lmoreira
 */
public final class IntegerParameterTest extends TestCase {
  /**
   * Testando criar um objeto parmetro inteiro obrigatrio.
   */
  public void testIntegerParameter1() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    Integer defaultValue = null;
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Integer maximum = null;
    Integer minimum = null;
    IntegerParameter parameter = new IntegerParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, maximum, minimum);
    assertEquals(name, label, description, defaultValue, isOptional,
      commandLinePattern, maximum, minimum, true,
      new LinkedList<SimpleParameterListener<Integer>>(), parameter);
  }

  /**
   * Testando criar um objeto parmetro inteiro opcional.
   */
  public void testIntegerParameter2() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    Integer defaultValue = null;
    boolean isOptional = true;
    String commandLinePattern = "--{0} {1}";
    Integer maximum = null;
    Integer minimum = null;
    IntegerParameter parameter = new IntegerParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, maximum, minimum);
    assertEquals(name, label, description, defaultValue, isOptional,
      commandLinePattern, maximum, minimum, true,
      new LinkedList<SimpleParameterListener<Integer>>(), parameter);
  }

  /**
   * Testando criar um objeto parmetro inteiro com mximo.
   */
  public void testIntegerParameter3() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    Integer defaultValue = null;
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Integer maximum = new Integer(1000);
    Integer minimum = null;
    IntegerParameter parameter = new IntegerParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, maximum, minimum);
    assertEquals(name, label, description, defaultValue, isOptional,
      commandLinePattern, maximum, minimum, true,
      new LinkedList<SimpleParameterListener<Integer>>(), parameter);
  }

  /**
   * Testando criar um objeto parmetro inteiro com mnimo.
   */
  public void testIntegerParameter4() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    Integer defaultValue = null;
    boolean isOptional = true;
    String commandLinePattern = "--{0} {1}";
    Integer maximum = null;
    Integer minimum = new Integer(-1000);
    IntegerParameter parameter = new IntegerParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, maximum, minimum);
    assertEquals(name, label, description, defaultValue, isOptional,
      commandLinePattern, maximum, minimum, true,
      new LinkedList<SimpleParameterListener<Integer>>(), parameter);
  }

  /**
   * Testando criar um objeto parmetro inteiro com valor-padro.
   */
  public void testIntegerParameter5() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    Integer defaultValue = new Integer(10);
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Integer maximum = null;
    Integer minimum = null;
    IntegerParameter parameter = new IntegerParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, maximum, minimum);
    assertEquals(name, label, description, defaultValue, isOptional,
      commandLinePattern, maximum, minimum, true,
      new LinkedList<SimpleParameterListener<Integer>>(), parameter);
  }

  /**
   * Testando criar um objeto parmetro inteiro sem nome.
   */
  public void testIntegerParameter6() {
    try {
      new IntegerParameter(
        null, "Rtulo do parmetro", "Descrio do parmetro", new Integer(10),
        false, "--{0} {1}", null, null);
      fail("Esperada IllegalArgumentException");
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um objeto parmetro inteiro sem rtulo.
   */
  public void testIntegerParameter7() {
    try {
      new IntegerParameter(
        "Nome do parmetro", null, "Descrio do parmetro", new Integer(10),
        false, "--{0} {1}", null, null);
      fail("Esperada IllegalArgumentException");
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um objeto parmetro inteiro sem descrio.
   */
  public void testIntegerParameter8() {
    try {
      new IntegerParameter("Nome do parmetro", "Rtulo do parmetro", null,
        new Integer(10), false, "--{0} {1}", null, null);
      fail("Esperada IllegalArgumentException");
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um objeto parmetro inteiro com mnimo maior do que
   * o mximo.
   */
  public void testIntegerParameter9() {
    try {
      new IntegerParameter(
        "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
        null, false,  "--{0} {1}", new Integer(-1000), new Integer(1000));
      fail("Esperada IllegalArgumentException");
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando o mtodo {@link IntegerParameter#setValueAsText(String)} com
   * nmero inteiro como argumento.
   * 
   * @throws ParseException Bug.
   */
  public void testSetValueAsText1()
  throws ParseException {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    Integer defaultValue = null;
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Integer maximum = null;
    Integer minimum = null;
    IntegerParameter parameter = new IntegerParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, maximum, minimum);
    Integer value = new Integer(10);
    parameter.setValueAsText(value.toString());
    assertEquals(value.toString(), parameter.getValueAsText());
    assertEquals(value, parameter.getValue());
  }

  /**
   * Testando o mtodo {@link IntegerParameter#setValueAsText(String)} com
   * o valor {@code null} como argumento.
   * 
   * @throws ParseException Bug.
   */
  public void testSetValueAsText2()
  throws ParseException {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    Integer defaultValue = new Integer(10);
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Integer maximum = null;
    Integer minimum = null;
    IntegerParameter parameter = new IntegerParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, maximum, minimum);
    parameter.setValueAsText(null);
    assertNull(parameter.getValueAsText());
    assertNull(parameter.getValue());
  }

  /**
   * Testando o mtodo {@link IntegerParameter#setValueAsText(String)} com um
   * texto que no representa um nmero inteiro como argumento.
   */
  public void testSetValueAsText3() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    Integer defaultValue = new Integer(10);
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Integer maximum = null;
    Integer minimum = null;
    IntegerParameter parameter = new IntegerParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, maximum, minimum);
    try {
      parameter.setValueAsText("Valor invlido.");
      fail("Esperada ParseException");
    }
    catch (ParseException e) {
    }
  }

  /**
   * Testando o mtodo {@link IntegerParameter#setMaximum(Integer)} com
   * um mximo diferente do mximo anterior.
   */
  public void testSetMaximum1() {
    CheckIntegerParameterListener checkListener = new CheckIntegerParameterListener();
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    int defaultValue = 500;
    boolean isOptional = false;
    Integer maximum = 1000;
    Integer minimum = 0;
    String commandLinePattern = "--{0} {1}";
    IntegerParameter integerParameter = new IntegerParameter(name, label,
      description, defaultValue, isOptional, commandLinePattern, maximum,
      minimum);
    integerParameter.addIntegerParameterListener(checkListener);
    checkListener.allowMaximumWasChangedEvent();
    Integer newMaximum = 2000;
    assertTrue(integerParameter.setMaximum(newMaximum));
    assertEquals(1, checkListener.getEventCount());
    assertEquals(newMaximum, integerParameter.getMaximum());
  }
  
  /**
   * Testando o mtodo {@link IntegerParameter#setMaximum(Integer)} com
   * um mximo e sem mximo anteriormente.
   */
  public void testSetMaximum2() {
    CheckIntegerParameterListener checkListener = new CheckIntegerParameterListener();
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    int defaultValue = 500;
    boolean isOptional = false;
    Integer maximum = null;
    Integer minimum = 0;
    String commandLinePattern = "--{0} {1}";
    IntegerParameter integerParameter = new IntegerParameter(name, label,
      description, defaultValue, isOptional, commandLinePattern, maximum,
      minimum);
    integerParameter.addIntegerParameterListener(checkListener);
    checkListener.allowMaximumWasChangedEvent();
    Integer newMaximum = 2000;
    assertTrue(integerParameter.setMaximum(newMaximum));
    assertEquals(1, checkListener.getEventCount());
    assertEquals(newMaximum, integerParameter.getMaximum());
  }
  
  /**
   * Testando o mtodo {@link IntegerParameter#setMaximum(Integer)} sem
   * um mximo e com um mximo anterior.
   */
  public void testSetMaximum3() {
    CheckIntegerParameterListener checkListener = new CheckIntegerParameterListener();
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    int defaultValue = 500;
    boolean isOptional = false;
    Integer maximum = 1000;
    Integer minimum = 0;
    String commandLinePattern = "--{0} {1}";
    IntegerParameter integerParameter = new IntegerParameter(name, label,
      description, defaultValue, isOptional, commandLinePattern, maximum,
      minimum);
    integerParameter.addIntegerParameterListener(checkListener);
    checkListener.allowMaximumWasChangedEvent();
    Integer newMaximum = null;
    assertTrue(integerParameter.setMaximum(newMaximum));
    assertEquals(1, checkListener.getEventCount());
    assertEquals(newMaximum, integerParameter.getMaximum());
  }
  
  /**
   * Testando o mtodo {@link IntegerParameter#setMaximum(Integer)} com
   * um mximo igual ao anterior.
   */
  public void testSetMaximum4() {
    CheckIntegerParameterListener checkListener = new CheckIntegerParameterListener();
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    int defaultValue = 500;
    boolean isOptional = false;
    Integer maximum = 1000;
    Integer minimum = 0;
    String commandLinePattern = "--{0} {1}";
    IntegerParameter integerParameter = new IntegerParameter(name, label,
      description, defaultValue, isOptional, commandLinePattern, maximum,
      minimum);
    integerParameter.addIntegerParameterListener(checkListener);
    Integer newMaximum = maximum;
    assertFalse(integerParameter.setMaximum(newMaximum));
    assertEquals(0, checkListener.getEventCount());
    assertEquals(newMaximum, integerParameter.getMaximum());
  }
    
  /**
   * Testando o mtodo {@link IntegerParameter#setMaximum(Integer)} com
   * um mximo atual e anterior {@code null}.
   */
  public void testSetMaximum5() {
    CheckIntegerParameterListener checkListener = new CheckIntegerParameterListener();
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    int defaultValue = 500;
    boolean isOptional = false;
    Integer maximum = null;
    Integer minimum = 0;
    String commandLinePattern = "--{0} {1}";
    IntegerParameter integerParameter = new IntegerParameter(name, label,
      description, defaultValue, isOptional, commandLinePattern, maximum,
      minimum);
    integerParameter.addIntegerParameterListener(checkListener);
    Integer newMaximum = maximum;
    assertFalse(integerParameter.setMaximum(newMaximum));
    assertEquals(0, checkListener.getEventCount());
    assertEquals(newMaximum, integerParameter.getMaximum());
  }
  

  /**
   * Testando o mtodo {@link IntegerParameter#setMinimum(Integer)} com
   * um mnimo diferente do mnimo anterior.
   */
  public void testSetMinimum1() {
    CheckIntegerParameterListener checkListener = new CheckIntegerParameterListener();
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    int defaultValue = 500;
    boolean isOptional = false;
    Integer maximum = 1000;
    Integer minimum = -1000;
    String commandLinePattern = "--{0} {1}";
    IntegerParameter integerParameter = new IntegerParameter(name, label,
      description, defaultValue, isOptional, commandLinePattern, maximum,
      minimum);
    integerParameter.addIntegerParameterListener(checkListener);
    checkListener.allowMinimumWasChangedEvent();
    Integer newMinimum = -2000;
    assertTrue(integerParameter.setMinimum(newMinimum));
    assertEquals(1, checkListener.getEventCount());
    assertEquals(newMinimum, integerParameter.getMinimum());
  }
  
  /**
   * Testando o mtodo {@link IntegerParameter#setMinimum(Integer)} com
   * um mnimo e sem mnimo anteriormente.
   */
  public void testSetMinimum2() {
    CheckIntegerParameterListener checkListener = new CheckIntegerParameterListener();
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    int defaultValue = 500;
    boolean isOptional = false;
    Integer maximum = 1000;
    Integer minimum = null;
    String commandLinePattern = "--{0} {1}";
    IntegerParameter integerParameter = new IntegerParameter(name, label,
      description, defaultValue, isOptional, commandLinePattern, maximum,
      minimum);
    integerParameter.addIntegerParameterListener(checkListener);
    checkListener.allowMinimumWasChangedEvent();
    Integer newMinimum = -2000;
    assertTrue(integerParameter.setMinimum(newMinimum));
    assertEquals(1, checkListener.getEventCount());
    assertEquals(newMinimum, integerParameter.getMinimum());
  }
  
  /**
   * Testando o mtodo {@link IntegerParameter#setMinimum(Integer)} sem
   * um mnimo e com um mnimo anterior.
   */
  public void testSetMinimum3() {
    CheckIntegerParameterListener checkListener = new CheckIntegerParameterListener();
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    int defaultValue = 500;
    boolean isOptional = false;
    Integer maximum = 1000;
    Integer minimum = -1000;
    String commandLinePattern = "--{0} {1}";
    IntegerParameter integerParameter = new IntegerParameter(name, label,
      description, defaultValue, isOptional, commandLinePattern, maximum,
      minimum);
    integerParameter.addIntegerParameterListener(checkListener);
    checkListener.allowMinimumWasChangedEvent();
    Integer newMinimum = null;
    assertTrue(integerParameter.setMinimum(newMinimum));
    assertEquals(1, checkListener.getEventCount());
    assertEquals(newMinimum, integerParameter.getMinimum());
  }
  
  /**
   * Testando o mtodo {@link IntegerParameter#setMinimum(Integer)} com
   * um mnimo igual ao anterior.
   */
  public void testSetMinimum4() {
    CheckIntegerParameterListener checkListener = new CheckIntegerParameterListener();
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    int defaultValue = 500;
    boolean isOptional = false;
    Integer maximum = 1000;
    Integer minimum = 0;
    String commandLinePattern = "--{0} {1}";
    IntegerParameter integerParameter = new IntegerParameter(name, label,
      description, defaultValue, isOptional, commandLinePattern, maximum,
      minimum);
    integerParameter.addIntegerParameterListener(checkListener);
    Integer newMinimum = minimum;
    assertFalse(integerParameter.setMinimum(newMinimum));
    assertEquals(0, checkListener.getEventCount());
    assertEquals(newMinimum, integerParameter.getMinimum());
  }
    
  /**
   * Testando o mtodo {@link IntegerParameter#setMinimum(Integer)} com
   * um mnimo atual e anterior {@code null}.
   */
  public void testSetMinimum5() {
    CheckIntegerParameterListener checkListener = new CheckIntegerParameterListener();
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    int defaultValue = 500;
    boolean isOptional = false;
    Integer maximum = null;
    Integer minimum = 0;
    String commandLinePattern = "--{0} {1}";
    IntegerParameter integerParameter = new IntegerParameter(name, label,
      description, defaultValue, isOptional, commandLinePattern, maximum,
      minimum);
    integerParameter.addIntegerParameterListener(checkListener);
    Integer newMinimum = minimum;
    assertFalse(integerParameter.setMinimum(newMinimum));
    assertEquals(0, checkListener.getEventCount());
    assertEquals(newMinimum, integerParameter.getMinimum());
  }
  
  /**
   * Testando serializao.
   *  
   * @throws IOException Bug.
   * @throws ClassNotFoundException Bug.
   */
  public void testSerialize()
  throws IOException, ClassNotFoundException {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    Integer defaultValue = new Integer(5);
    boolean isOptional = false;
    Integer maximum = new Integer(10);
    Integer minimum = new Integer(0);
    String commandLinePattern = "--{0} {1}";
    IntegerParameter outputParameter = new IntegerParameter(name, label,
      description, defaultValue, isOptional, commandLinePattern, maximum,
      minimum);
    ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
    ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
    objectOutput.writeObject(outputParameter);
    ByteArrayInputStream inputSource =
      new ByteArrayInputStream(outputSource.toByteArray());
    ObjectInputStream objectInput = new ObjectInputStream(inputSource);
    IntegerParameter inputParameter =
      (IntegerParameter)objectInput.readObject();
    assertEquals(outputParameter, inputParameter);
  }

  /**
   * Verifica se os atributos do parmetro inteiros so iguais aos atributos
   * especificados.
   * 
   * @param name O nome (Aceita {@code null}).
   * @param label O rtulo (Aceita {@code null}).
   * @param description A descrio (Aceita {@code null}).
   * @param defaultValue O valor-padro (Aceita {@code null}).
   * @param isOptional O indicador parmetro opcional/obrigatrio.
   * @param commandLinePattern O padro para a linha de comando 
   *        (Aceita {@code null}).
   * @param maximum O mximo (Aceita {@code null}).
   * @param minimum O mnimo (Aceita {@code null}).
   * @param isEnabled O indicador parmetro habilitado/desabilitado.
   * @param listeners Os observadores (No aceita {@code null}).
   * @param parameter O parmetro (No aceita {@code null}).
   */
  private static void assertEquals(String name, String label,
    String description, Object defaultValue, boolean isOptional,
    String commandLinePattern, Integer maximum, Integer minimum,
    boolean isEnabled, List<SimpleParameterListener<Integer>> listeners,
    IntegerParameter parameter) {
    SimpleParameterTest.assertEquals(name, label, description, defaultValue,
      isOptional, commandLinePattern, isEnabled, listeners, parameter);
    Assert.assertEquals(maximum, parameter.getMaximum());
    Assert.assertEquals(minimum, parameter.getMinimum());
  }

  /**
   * Verifica se todos os atributos dos 2 parmetros especificados so iguais.
   * 
   * @param expectedParameter O parmetro esperado (Aceita {@code null}).
   * @param foundParameter O parmetro encontrado (Aceita {@code null}).
   */
  private static void assertEquals(
    IntegerParameter expectedParameter, IntegerParameter foundParameter) {
    SimpleParameterTest.assertEquals(expectedParameter, foundParameter);
    Assert.assertEquals(
      expectedParameter.getMaximum(), foundParameter.getMaximum());
    Assert.assertEquals(
      expectedParameter.getMinimum(), foundParameter.getMinimum());
  }
}
