package csbase.logic.algorithms.parameters;

import junit.framework.TestCase;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.exception.algorithms.ExpressionFunctionExecutionException;
import csbase.logic.algorithms.ExecutionType;

public final class ListExpressionFunctionTest extends TestCase {
  public void testListExpressionFunction1() {
    String name = "Nome da funo";
    Class[] parameterTypes = new Class[] { Integer.class, Double.class };
    Class[] expectedParameterTypes = new Class[] { String.class, Integer.class, Double.class };
    ListExpressionFunction function = new MockListExpressionFunction(name,
      parameterTypes);
    assertEquals(name, function.getName());
    List<Class> parameterTypeList = function.getParameterTypes();
    assertParameterTypes(expectedParameterTypes, parameterTypeList);
  }

  public void testListExpressionFunction2() {
    Class[] parameterTypes = new Class[] { String.class, Object.class };
    try {
      new MockListExpressionFunction(null, parameterTypes);
      fail(IllegalArgumentException.class.getName());
    } catch (IllegalArgumentException e) {
    }
  }

  public void testListExpressionFunction3() {
    String name = "Nome da funo";
    try {
      new MockListExpressionFunction(name, (Class[]) null);
      fail(IllegalArgumentException.class.getName());
    } catch (IllegalArgumentException e) {
    }
  }

  public void testAcceptParameters4() {
    String name = "Nome da funo";
    Class[] parameterTypes = new Class[] { Integer.class, Double.class };
    ListExpressionFunction function = new MockListExpressionFunction(name,
      parameterTypes);
    Object[] parameters = new Object[] { "Nome da lista", 1, 2.0 };
    assertTrue(function.acceptParameters(parameters));
  }

  public void testAcceptParameters5() {
    String name = "Nome da funo";
    Class[] parameterTypes = new Class[] { Integer.class, Double.class };
    ListExpressionFunction function = new MockListExpressionFunction(name,
      parameterTypes);
    Object[] parameters = new Object[] { 1, 2.0 };
    assertFalse(function.acceptParameters(parameters));
  }

  public void testDoOperation1() throws ExpressionFunctionExecutionException {
    String name = "Nome da funo";
    Class[] parameterTypes = new Class[] { Integer.class, Double.class };
    CheckListExpressionFunction function = new CheckListExpressionFunction(
      name, parameterTypes);
    Object[] parameters = new Object[] { 1, 2.0 };
    List<Object> parameterList = new LinkedList<Object>();
    for (Object parameter : parameters) {
      parameterList.add(parameter);
    }
    Object result = "Resultado";
    function.setExpected(parameterList, parameters, result);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    ListParameter<Object> parameter = new MockListParameter(
      parameterName, parameterLabel, parameterDescription, false, "{0}={1}");
    parameter.setValue(parameterList);
    ParameterGroup group = new ParameterGroup("Grupo");
    group.addParameter(parameter);
    SimpleAlgorithmConfigurator configurator = createConfigurator();
    configurator.addGroup(group);
    assertEquals(result, function.doOperation(configurator, parameterName, 1, 2.0));
    assertTrue(function.wasDoneOperation());
  }

  public void testDoOperation2() {
    String name = "Nome da funo";
    Class[] parameterTypes = new Class[] { Integer.class, Double.class };
    CheckListExpressionFunction function = new CheckListExpressionFunction(
      name, parameterTypes);
    Object[] parameters = new Object[] { 1, 2.0 };
    List<Object> parameterList = new LinkedList<Object>();
    for (Object parameter : parameters) {
      parameterList.add(parameter);
    }
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    ListParameter<Object> parameter = new MockListParameter(
      parameterName, parameterLabel, parameterDescription, false, "{0}={1}");
    parameter.setValue(parameterList);
    ParameterGroup group = new ParameterGroup("Grupo");
    group.addParameter(parameter);
    SimpleAlgorithmConfigurator configurator = createConfigurator();
    configurator.addGroup(group);
    try {
      function.doOperation(configurator, "Nome de funo que no existe", 1, 2.0);
      fail(ExpressionFunctionExecutionException.class.getName());
    } catch (ExpressionFunctionExecutionException e) {
    }
    assertFalse(function.wasDoneOperation());
  }
  
  public void testDoOperation3() {
    String name = "Nome da funo";
    Class[] parameterTypes = new Class[] { Integer.class, Double.class };
    CheckListExpressionFunction function = new CheckListExpressionFunction(
      name, parameterTypes);
    Object[] parameters = new Object[] { 1, 2.0 };
    List<Object> parameterList = new LinkedList<Object>();
    for (Object parameter : parameters) {
      parameterList.add(parameter);
    }
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    SimpleParameter<Object> parameter = new MockSimpleParameter(
      parameterName, parameterLabel, parameterDescription, null, false,
      "{0}{1}");
    parameter.setValue(parameterList);
    ParameterGroup group = new ParameterGroup("Grupo");
    group.addParameter(parameter);
    SimpleAlgorithmConfigurator configurator = createConfigurator();
    configurator.addGroup(group);
    try {
      function.doOperation(configurator, parameterName, 1, 2.0);
      fail(ExpressionFunctionExecutionException.class.getName());
    }
    catch (ExpressionFunctionExecutionException e) {}
    assertFalse(function.wasDoneOperation());
  }
  
  public void testDoOperation4() throws ExpressionFunctionExecutionException {
    String name = "Nome da funo";
    Class[] parameterTypes = new Class[] { Integer.class, Double.class };
    CheckListExpressionFunction function = new CheckListExpressionFunction(
      name, parameterTypes);
    Object[] parameters = new Object[] { 1, 2.0 };
    List<Object> parameterList = new LinkedList<Object>();
    for (Object parameter : parameters) {
      parameterList.add(parameter);
    }
    String parameterName = "Nome do parmetro";
    try {
      function.doOperation((SimpleAlgorithmConfigurator)null, parameterName, 1, 2.0);
      fail(IllegalArgumentException.class.getName());
    }
    catch (IllegalArgumentException e) {}
    assertFalse(function.wasDoneOperation());
  }
  
  public void testDoOperation5() {
    String name = "Nome da funo";
    Class[] parameterTypes = new Class[] { Integer.class, Double.class };
    CheckListExpressionFunction function = new CheckListExpressionFunction(
      name, parameterTypes);
    Object[] parameters = new Object[] { 1, 2.0 };
    List<Object> parameterList = new LinkedList<Object>();
    for (Object parameter : parameters) {
      parameterList.add(parameter);
    }
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    ListParameter<Object> parameter = new MockListParameter(
      parameterName, parameterLabel, parameterDescription, false, "{0}={1}");
    parameter.setValue(parameterList);
    ParameterGroup group = new ParameterGroup("Grupo");
    group.addParameter(parameter);
    SimpleAlgorithmConfigurator configurator = createConfigurator();
    configurator.addGroup(group);
    try {
      function.doOperation(configurator, null, 1, 2.0);
      fail(ExpressionFunctionExecutionException.class.getName());
    }
    catch (ExpressionFunctionExecutionException e) {}
    assertFalse(function.wasDoneOperation());
  }
  
  private SimpleAlgorithmConfigurator createConfigurator() {
    String algorithmId = "Identificador do algoritmo";
    String algorithmName = "Nome do algoritmo";
    String algorithmDirectory = "Diretrio do algoritmo";
    AlgorithmInfo algorithmInfo = new AlgorithmInfo(algorithmId, algorithmName,
      algorithmDirectory, new Hashtable(), new Hashtable());
    AlgorithmVersionInfo versionInfo = new AlgorithmVersionInfo(algorithmInfo,
      new AlgorithmVersionId(1, 2, 3), new Hashtable(), new Hashtable());
    SimpleAlgorithmConfigurator configurator = new SimpleAlgorithmConfigurator(
      versionInfo, null, ExecutionType.SIMPLE, null, false, "Comando", null, null
      , true);
    return configurator;
  }

  private void assertParameterTypes(Class[] expectedParameterTypes,
                                    List<Class> foundParameterTypes) {
    Iterator<Class> parameterTypeIterator = foundParameterTypes.iterator();
    for (Class parameterType : expectedParameterTypes) {
      assertEquals(parameterTypeIterator.next(), parameterType);
    }
  }
}
