package csbase.logic.algorithms.parameters;

import java.util.Hashtable;

import junit.framework.TestCase;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.exception.algorithms.ExpressionFunctionExecutionException;
import csbase.logic.algorithms.ExecutionType;

public final class ListMinExpressionFunctionTest extends TestCase {

  public void testListMinExpressionFunction1() {
    ListMinExpressionFunction function = new ListMinExpressionFunction();
    assertEquals("list_min", function.getName());
    assertNotNull(function.getParameterTypes());
    assertEquals(1, function.getParameterTypes().size());
    assertEquals(String.class, function.getParameterTypes().get(0));
  }

  public void testExecute1() throws ExpressionFunctionExecutionException {
    ListMinExpressionFunction function = new ListMinExpressionFunction();
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    DoubleListParameter parameter = new DoubleListParameter(parameterName,
      parameterLabel, parameterDescription, false, null, null, false, null,
      false);
    parameter.addElement(2.0);
    parameter.addElement(1.0);
    parameter.addElement(3.0);
    ParameterGroup group = new ParameterGroup("Grupo");
    group.addParameter(parameter);
    SimpleAlgorithmConfigurator configurator = createConfigurator();
    configurator.addGroup(group);
    assertEquals(1.0, function.execute(configurator, parameterName));
  }

  public void testExecute2() {
    ListMinExpressionFunction function = new ListMinExpressionFunction();
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    DoubleListParameter parameter = new DoubleListParameter(parameterName,
      parameterLabel, parameterDescription, false, null, null, false, null,
      false);
    ParameterGroup group = new ParameterGroup("Grupo");
    group.addParameter(parameter);
    SimpleAlgorithmConfigurator configurator = createConfigurator();
    configurator.addGroup(group);
    try {
      function.execute(configurator, "Nome de parmetro que no existe");
      fail(ExpressionFunctionExecutionException.class.getName());
    }
    catch (ExpressionFunctionExecutionException e) {}
  }
  
  public void testExecute3() {
    ListMinExpressionFunction function = new ListMinExpressionFunction();
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    DoubleListParameter parameter = new DoubleListParameter(parameterName,
      parameterLabel, parameterDescription, false, null, null, false, null,
      false);
    ParameterGroup group = new ParameterGroup("Grupo");
    group.addParameter(parameter);
    SimpleAlgorithmConfigurator configurator = createConfigurator();
    configurator.addGroup(group);
    try {
      function.execute(configurator, parameterName);
      fail(ExpressionFunctionExecutionException.class.getName());
    }
    catch (ExpressionFunctionExecutionException e) {}
  }
  
  public void testExecute4() throws ExpressionFunctionExecutionException {
    ListMinExpressionFunction function = new ListMinExpressionFunction();
    String parameterName = "Nome do parmetro";
    try {
      function.execute(null, parameterName);
      fail(IllegalArgumentException.class.getName());
    }
    catch (IllegalArgumentException e) {}
  }

  public void testExecute5() {
    ListMinExpressionFunction function = new ListMinExpressionFunction();
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    DoubleListParameter parameter = new DoubleListParameter(parameterName,
      parameterLabel, parameterDescription, false, null, null, false, null,
      false);
    ParameterGroup group = new ParameterGroup("Grupo");
    group.addParameter(parameter);
    SimpleAlgorithmConfigurator configurator = createConfigurator();
    configurator.addGroup(group);
    try {
      function.execute(configurator, (String)null);
      fail(ExpressionFunctionExecutionException.class.getName());
    }
    catch (ExpressionFunctionExecutionException e) {}
  }
  
  private SimpleAlgorithmConfigurator createConfigurator() {
    String algorithmId = "Identificador do algoritmo";
    String algorithmName = "Nome do algoritmo";
    String algorithmDirectory = "Diretrio do algoritmo";
    AlgorithmInfo algorithmInfo = new AlgorithmInfo(algorithmId, algorithmName,
      algorithmDirectory, new Hashtable(), new Hashtable());
    AlgorithmVersionInfo versionInfo = new AlgorithmVersionInfo(algorithmInfo,
      new AlgorithmVersionId(1, 2, 3), new Hashtable(), new Hashtable());
    SimpleAlgorithmConfigurator configurator = new SimpleAlgorithmConfigurator(
      versionInfo, null, ExecutionType.SIMPLE, null, false, "Comando", null,
      null, true);
    return configurator;
  }
}
