/*
 * $Author:$
 * $Date:$
 * $Release:$
 */
package csbase.logic.algorithms.parameters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.LinkedList;
import java.util.List;

import junit.framework.TestCase;

//@todo Testar setElement
public final class ListParameterTest extends TestCase {
  public void testListParameterTestWithOptionalTrue() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    ListParameter parameter =
      new MockListParameter(name, label, description, isOptional, "{0}={1}");
    assertEquals(name, parameter.getName());
    assertEquals(label, parameter.getLabel());
    assertEquals(description, parameter.getDescription());
    assertNull(parameter.getDefaultValue());
    assertTrue(parameter.isOptional());
  }

  public void testListParameterTestWithOptionalFalse() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    ListParameter parameter =
      new MockListParameter(name, label, description, isOptional, "{0}={1}");
    assertEquals(name, parameter.getName());
    assertEquals(label, parameter.getLabel());
    assertEquals(description, parameter.getDescription());
    assertNull(parameter.getDefaultValue());
    assertFalse(parameter.isOptional());
  }

  public void testListParameterTestWithNameNull() {
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    try {
      new MockListParameter(null, label, description, isOptional, "{0}={1}");
      fail(
        "No deveria ser possvel criar um parmetro do tipo lista com nome nulo.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  public void testListParameterTestWithLabelNull() {
    String name = "Nome do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    try {
      new MockListParameter(name, null, description, isOptional, "{0}={1}");
      fail(
        "No deveria ser possvel criar um parmetro do tipo lista com rtulo nulo.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  public void testListParameterTestWithDescriptionNull() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    boolean isOptional = true;
    try {
      new MockListParameter(name, label, null, isOptional, "{0}={1}");
      fail(
        "No deveria ser possvel criar um parmetro do tipo lista com descrio nula.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  public void testAddElement1() {
    String name = "LST";
    String label = "Lista";
    String description = "Parmetro do tipo lista";
    boolean isOptional = false;
    ListParameter parameter =
      new MockListParameter(name, label, description, isOptional, "{0}={1}");
    assertNull(parameter.getValue());
    CheckParameterListener listener = new CheckParameterListener();
    listener.allowChangeValue();
    parameter.addSimpleParameterListener(listener);
    String value1 = "valor 1";
    parameter.addElement(value1);
    assertEquals(1, listener.getHowManyEventsOccurred());
    String value2 = "valor 2";
    parameter.addElement(value2);
    assertEquals(2, listener.getHowManyEventsOccurred());
    List<String> values = new LinkedList<String>();
    values.add(value1);
    values.add(value2);
    assertEquals(values, parameter.getValue());
  }

  public void testAddElement2() {
    String name = "LST";
    String label = "Lista";
    String description = "Parmetro do tipo lista";
    boolean isOptional = false;
    ListParameter parameter =
      new MockListParameter(name, label, description, isOptional, "{0}={1}");
    assertNull(parameter.getValue());
    CheckParameterListener listener = new CheckParameterListener();
    listener.allowChangeValue();
    parameter.addSimpleParameterListener(listener);
    String value = "valor";
    parameter.addElement(value);
    assertEquals(1, listener.getHowManyEventsOccurred());
    parameter.addElement(value);
    assertEquals(2, listener.getHowManyEventsOccurred());
    List<String> values = new LinkedList<String>();
    values.add(value);
    values.add(value);
    assertEquals(values, parameter.getValue());
  }

  public void testAddElement3() {
    String name = "LST";
    String label = "Lista";
    String description = "Parmetro do tipo lista";
    boolean isOptional = false;
    ListParameter parameter =
      new MockListParameter(name, label, description, isOptional, "{0}={1}");
    assertNull(parameter.getValue());
    CheckParameterListener listener = new CheckParameterListener();
    parameter.addSimpleParameterListener(listener);
    try {
      parameter.addElement(null);
      fail("Esperada IllegalArgumentException");
    }
    catch (IllegalArgumentException e) {
    }
    assertEquals(0, listener.getHowManyEventsOccurred());
    assertNull(parameter.getValue());
  }

  public void testRemoveElement1() {
    String name = "LST";
    String label = "Lista";
    String description = "Parmetro do tipo lista";
    boolean isOptional = false;
    ListParameter parameter =
      new MockListParameter(name, label, description, isOptional, "{0}={1}");
    assertNull(parameter.getValue());
    CheckParameterListener listener = new CheckParameterListener();
    listener.allowChangeValue();
    parameter.addSimpleParameterListener(listener);
    String value1 = "valor 1";
    parameter.addElement(value1);
    assertEquals(1, listener.getHowManyEventsOccurred());
    String value2 = "valor 2";
    parameter.addElement(value2);
    assertEquals(2, listener.getHowManyEventsOccurred());
    parameter.removeElement(0);
    assertEquals(3, listener.getHowManyEventsOccurred());
    List<Object> values = new LinkedList<Object>();
    values.add(value2);
    assertEquals(values, parameter.getValue());
  }

  public void testRemoveElement2() {
    String name = "LST";
    String label = "Lista";
    String description = "Parmetro do tipo lista";
    boolean isOptional = false;
    ListParameter parameter =
      new MockListParameter(name, label, description, isOptional, "{0}={1}");
    assertNull(parameter.getValue());
    CheckParameterListener listener = new CheckParameterListener();
    listener.allowChangeValue();
    parameter.addSimpleParameterListener(listener);
    String value1 = "valor 1";
    parameter.addElement(value1);
    assertEquals(1, listener.getHowManyEventsOccurred());
    String value2 = "valor 2";
    parameter.addElement(value2);
    assertEquals(2, listener.getHowManyEventsOccurred());
    parameter.removeElement(1);
    assertEquals(3, listener.getHowManyEventsOccurred());
    parameter.removeElement(0);
    assertEquals(4, listener.getHowManyEventsOccurred());
    assertNull(parameter.getValue());
  }

  public void testRemoveElement3() {
    String name = "LST";
    String label = "Lista";
    String description = "Parmetro do tipo lista";
    boolean isOptional = false;
    ListParameter parameter =
      new MockListParameter(name, label, description, isOptional, "{0}={1}");
    assertNull(parameter.getValue());
    CheckParameterListener listener = new CheckParameterListener();
    parameter.addSimpleParameterListener(listener);
    try {
      parameter.removeElement(0);
      fail("UnsupportedOperationException");
    }
    catch (UnsupportedOperationException e) {
    }
    assertEquals(0, listener.getHowManyEventsOccurred());
    assertNull(parameter.getValue());
  }

  public void testRemoveElement4() {
    String name = "LST";
    String label = "Lista";
    String description = "Parmetro do tipo lista";
    boolean isOptional = false;
    ListParameter parameter =
      new MockListParameter(name, label, description, isOptional, "{0}={1}");
    assertNull(parameter.getValue());
    String value1 = "valor 1";
    parameter.addElement(value1);
    String value2 = "valor 2";
    parameter.addElement(value2);
    CheckParameterListener listener = new CheckParameterListener();
    parameter.addSimpleParameterListener(listener);
    try {
      parameter.removeElement(2);
      fail("IllegalArgumentException");
    }
    catch (IllegalArgumentException e) {
    }
    assertEquals(0, listener.getHowManyEventsOccurred());
    List<Object> values = new LinkedList<Object>();
    values.add(value1);
    values.add(value2);
    assertEquals(values, parameter.getValue());
  }

  public void testRemoveElement5() {
    String name = "LST";
    String label = "Lista";
    String description = "Parmetro do tipo lista";
    boolean isOptional = false;
    ListParameter parameter =
      new MockListParameter(name, label, description, isOptional, "{0}={1}");
    assertNull(parameter.getValue());
    String value1 = "valor 1";
    parameter.addElement(value1);
    String value2 = "valor 2";
    parameter.addElement(value2);
    CheckParameterListener listener = new CheckParameterListener();
    parameter.addSimpleParameterListener(listener);
    try {
      parameter.removeElement(-1);
      fail("IllegalArgumentException");
    }
    catch (IllegalArgumentException e) {
    }
    assertEquals(0, listener.getHowManyEventsOccurred());
    List<Object> values = new LinkedList<Object>();
    values.add(value1);
    values.add(value2);
    assertEquals(values, parameter.getValue());
  }

  public void testSerialize()
  throws IOException, ClassNotFoundException {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    MockListParameter outputParameter =
      new MockListParameter(name, label, description, isOptional, "{0}={1}");
    List<Object> defaultValues = new LinkedList<Object>();
    defaultValues.add("Valor-padro 1");
    defaultValues.add("Valor-padro 2");
    ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
    ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
    objectOutput.writeObject(outputParameter);
    ByteArrayInputStream inputSource =
      new ByteArrayInputStream(outputSource.toByteArray());
    ObjectInputStream objectInput = new ObjectInputStream(inputSource);
    ListParameter inputParameter = (ListParameter)objectInput.readObject();
    assertEquals(outputParameter, inputParameter);
    assertEquals(outputParameter.getName(), inputParameter.getName());
    assertEquals(outputParameter.getLabel(), inputParameter.getLabel());
    assertEquals(
      outputParameter.getDescription(), inputParameter.getDescription());
    assertEquals(
      outputParameter.getDefaultValue(), inputParameter.getDefaultValue());
    assertEquals(outputParameter.isOptional(), inputParameter.isOptional());
  }
}
