package csbase.logic.algorithms.parameters;

import java.lang.reflect.Method;
import java.util.Hashtable;

import junit.framework.Assert;
import junit.framework.TestCase;
import csbase.exception.algorithms.ExpressionFunctionExecutionException;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.ExecutionType;

public final class ReflectionExpressionFunctionTest extends TestCase {
  public void testReflectionExpressionFunction1() throws NoSuchMethodException,
    ExpressionFunctionExecutionException {
    AlgorithmInfo algorithm = new AlgorithmInfo("Identificador do algoritmo",
      "Nome do algoritmo", "Diretro do algoritmo", new Hashtable());
    AlgorithmVersionInfo version = new AlgorithmVersionInfo(algorithm,
      new AlgorithmVersionId(1, 2, 3), new Hashtable(), new Hashtable());
    SimpleAlgorithmConfigurator configurator = new SimpleAlgorithmConfigurator(
      version, null, ExecutionType.SIMPLE, null, false, "Comando", null, null,
      true);
    Method method = Method1.class.getMethod("publicStaticMethod",
      new Class[] { String.class, Float.class, Integer.class });
    ReflectionExpressionFunction function = new ReflectionExpressionFunction(
      method);
    String expectedParameter1 = "Texto 1";
    Float expectedParameter2 = 2f;
    Integer expectedParameter3 = 3;
    Double expectedResult = 3.0;
    Method1.allowExecute(expectedParameter1, expectedParameter2,
      expectedParameter3, expectedResult);
    Object result = function
      .execute(
        configurator,
        new Object[] { expectedParameter1, expectedParameter2, expectedParameter3 });
    assertEquals(expectedResult, result);
  }

  public void testReflectionExpressionFunction2() throws NoSuchMethodException {
    Method method = Method1.class.getDeclaredMethod("privateStaticMethod",
      new Class[] { Integer.class });
    try {
      new ReflectionExpressionFunction(method);
      fail(IllegalArgumentException.class.getName());
    } catch (IllegalArgumentException e) {
    }
  }

  public void testReflectionExpressionFunction3() throws NoSuchMethodException {
    Method method = Method1.class.getMethod("publicInstanceMethod",
      new Class[] { String.class });
    try {
      new ReflectionExpressionFunction(method);
      fail(IllegalArgumentException.class.getName());
    } catch (IllegalArgumentException e) {
    }
  }

  public void testReflectionExpressionFunction4() {
    try {
      new ReflectionExpressionFunction(null);
      fail(NullPointerException.class.getName());
    } catch (NullPointerException e) {
    }
  }

  @Override
  protected void setUp() {
    Method1.denyExecute();
  }

  @Override
  protected void tearDown() {
    Method1.denyExecute();
  }

  static class Method1 {
    private static boolean canExecute;
    private static Double result;
    private static String expectedParameter1;
    private static Float expectedParameter2;
    private static Integer expectedParameter3;

    static void denyExecute() {
      canExecute = false;
      Method1.expectedParameter1 = null;
      Method1.expectedParameter2 = null;
      Method1.expectedParameter3 = null;
      Method1.result = null;
    }

    static void allowExecute(String expectedParameter1,
                             Float expectedParameter2,
                             Integer expectedParameter3, Double result) {
      Method1.expectedParameter1 = expectedParameter1;
      Method1.expectedParameter2 = expectedParameter2;
      Method1.expectedParameter3 = expectedParameter3;
      Method1.result = result;
      canExecute = true;
    }

    public static Double publicStaticMethod(String parameter1,
                                            Float parameter2, Integer parameter3) {
      Assert.assertTrue(canExecute);
      Assert.assertEquals(expectedParameter1, parameter1);
      Assert.assertEquals(expectedParameter2, parameter2);
      Assert.assertEquals(expectedParameter3, parameter3);
      return result;
    }

    @SuppressWarnings("unused")
    private static Boolean privateStaticMethod(Integer parameter) {
      return false;
    }

    public String publicInstanceMethod(String parameter) {
      return null;
    }
  }
}
