/*
 * $Author:$
 * $Date:$
 * $Release:$
 */
package csbase.logic.algorithms.parameters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.LinkedList;
import java.util.List;

import junit.framework.TestCase;

public final class TextListParameterTest extends TestCase {
  public void testTextListParameterWithTrueIsOptional() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    String commandLinePattern = "{0}={1}";
    Integer maxCharacters = new Integer(50);
    TextListParameter parameter =
      new TextListParameter(
        name, label, description, isOptional, commandLinePattern, maxCharacters);
    assertEquals(name, parameter.getName());
    assertEquals(label, parameter.getLabel());
    assertEquals(description, parameter.getDescription());
    assertEquals(isOptional, parameter.isOptional());
    assertNull(parameter.getDefaultValue());
    assertEquals(maxCharacters, parameter.getMaxCharacters());
    
  }

  public void testTextListParameterWithFalseIsOptional() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "{0}={1}";
    Integer maxCharacters = new Integer(50);
    TextListParameter parameter = new TextListParameter(name, label,
      description, isOptional, commandLinePattern, maxCharacters);
    assertEquals(name, parameter.getName());
    assertEquals(label, parameter.getLabel());
    assertEquals(description, parameter.getDescription());
    assertEquals(isOptional, parameter.isOptional());
    assertNull(parameter.getDefaultValue());
    assertEquals(maxCharacters, parameter.getMaxCharacters());
    
  }

  public void testTextListParameterWithNullMaxCharacters() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "{0}={1}";
    TextListParameter parameter = new TextListParameter(name, label,
      description, isOptional, commandLinePattern, null);
    assertEquals(name, parameter.getName());
    assertEquals(label, parameter.getLabel());
    assertEquals(description, parameter.getDescription());
    assertEquals(isOptional, parameter.isOptional());
    assertNull(parameter.getDefaultValue());
    assertNull(parameter.getMaxCharacters());
    
  }

  public void testTextListParameterWithNullName() {
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    String commandLinePattern = "{0}={1}";
    try {
      new TextListParameter(null, label, description, isOptional,
        commandLinePattern, null);
      fail("No deveria ser possvel criar um parmetro sem nome.");
    }
    catch (IllegalArgumentException e) {}
  }

  public void testTextListParameterWithNullLabel() {
    String name = "Nome do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    String commandLinePattern = "{0}={1}";
    try {
      new TextListParameter(name, null, description, isOptional,
        commandLinePattern, null);
      fail("No deveria ser possvel criar um parmetro sem rtulo.");
    }
    catch (IllegalArgumentException e) {}
  }

  public void testTextListParameterWithNullDescription() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    boolean isOptional = true;
    String commandLinePattern = "{0}={1}";
    try {
      new TextListParameter(name, label, null, isOptional, commandLinePattern,
        null);
      fail("No deveria ser possvel criar um parmetro sem descrio.");
    }
    catch (IllegalArgumentException e) {}
  }

  public void testSerialize()
  throws IOException, ClassNotFoundException {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    MockListParameter outputParameter =
      new MockListParameter(name, label, description, isOptional, "{0}={1}");
    List<String> defaultValues = new LinkedList<String>();
    defaultValues.add("Valor-padro 1");
    defaultValues.add("Valor-padro 2");
    ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
    ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
    objectOutput.writeObject(outputParameter);
    ByteArrayInputStream inputSource =
      new ByteArrayInputStream(outputSource.toByteArray());
    ObjectInputStream objectInput = new ObjectInputStream(inputSource);
    MockListParameter inputParameter =
      (MockListParameter)objectInput.readObject();
    assertEquals(outputParameter, inputParameter);
    assertEquals(outputParameter.getName(), inputParameter.getName());
    assertEquals(outputParameter.getLabel(), inputParameter.getLabel());
    assertEquals(
      outputParameter.getDescription(), inputParameter.getDescription());
    assertEquals(
      outputParameter.getDefaultValue(), inputParameter.getDefaultValue());
    assertEquals(outputParameter.isOptional(), inputParameter.isOptional());
  }
}
