/*
 * $Author:$
 * $Date:$
 * $Release:$
 */
package csbase.logic.algorithms.parameters.conditions;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Arrays;
import java.util.Hashtable;
import java.util.List;

import junit.framework.TestCase;
import csbase.logic.FileInfo;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.ExecutionType;
import csbase.logic.algorithms.parameters.SimpleAlgorithmConfigurator;

public final class AndOperatorConditionTest extends TestCase {
  public static AndOperatorCondition create() {
    return new AndOperatorCondition(
      new MockCondition(true), new MockCondition(false));
  }

  public void testAndOperatorCondition() {
    Condition firstCondition = new MockCondition(true);
    Condition secondCondition = new MockCondition(false);
    Condition thirdCondition = new MockCondition(false);
    assertFalse(firstCondition.equals(secondCondition));
    AndOperatorCondition condition = new AndOperatorCondition(firstCondition,
      secondCondition, thirdCondition);
    assertEquals(
      Arrays.asList(firstCondition, secondCondition, thirdCondition), condition
        .getConditions());
  }

  public void testEquals() {
    Condition firstCondition = new MockCondition(true);
    Condition secondCondition = new MockCondition(false);
    AndOperatorCondition condition1 =
      new AndOperatorCondition(firstCondition, secondCondition);
    AndOperatorCondition condition2 =
      new AndOperatorCondition(firstCondition, secondCondition);
    assertTrue(condition1.equals(condition2));
  }

  public void testEqualsWithNull() {
    AndOperatorCondition condition = create();
    assertFalse(condition.equals(null));
  }

  public void testEqualsWithObjectOfAnotherClass() {
    AndOperatorCondition condition1 = create();
    OrOperatorCondition condition2 = OrOperatorConditionTest.create();
    assertFalse(condition1.equals(condition2));
  }

  public void testHashCode() {
    Condition firstCondition = new MockCondition(true);
    Condition secondCondition = new MockCondition(false);
    AndOperatorCondition condition1 =
      new AndOperatorCondition(firstCondition, secondCondition);
    AndOperatorCondition condition2 =
      new AndOperatorCondition(firstCondition, secondCondition);
    assertEquals(condition1.hashCode(), condition2.hashCode());
  }

  public void testEvaluateWithTrueFirstConditionAndTrueSecondCondition() {
    Condition firstCondition = new MockCondition(true);
    Condition secondCondition = new MockCondition(true);
    AndOperatorCondition condition =
      new AndOperatorCondition(firstCondition, secondCondition);
    assertTrue(condition.evaluate(createAlgorithmVersion()));
  }

  public void testEvaluateWithTrueFirstConditionAndFalseSecondCondition() {
    Condition firstCondition = new MockCondition(true);
    Condition secondCondition = new MockCondition(false);
    AndOperatorCondition condition =
      new AndOperatorCondition(firstCondition, secondCondition);
    assertFalse(condition.evaluate(createAlgorithmVersion()));
  }

  public void testEvaluateWithFalseFirstConditionAndTrueSecondCondition() {
    Condition firstCondition = new MockCondition(false);
    Condition secondCondition = new MockCondition(true);
    AndOperatorCondition condition =
      new AndOperatorCondition(firstCondition, secondCondition);
    assertFalse(condition.evaluate(createAlgorithmVersion()));
  }

  public void testEvaluateWithFalseFirstConditionAndFalseSecondCondition() {
    Condition firstCondition = new MockCondition(false);
    Condition secondCondition = new MockCondition(false);
    AndOperatorCondition condition =
      new AndOperatorCondition(firstCondition, secondCondition);
    assertFalse(condition.evaluate(createAlgorithmVersion()));
  }

  public void testSerialize()
  throws IOException, ClassNotFoundException {
    String parameterName = "Nome do parmetro";
    String value = "Valor";
    Condition firstCondition = new SimpleCondition(parameterName, value);
    Condition secondCondition = new SimpleCondition(parameterName, value);
    AndOperatorCondition outputCondition =
      new AndOperatorCondition(firstCondition, secondCondition);
    ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
    ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
    objectOutput.writeObject(outputCondition);
    ByteArrayInputStream inputSource =
      new ByteArrayInputStream(outputSource.toByteArray());
    ObjectInputStream objectInput = new ObjectInputStream(inputSource);
    AndOperatorCondition inputCondition =
      (AndOperatorCondition)objectInput.readObject();
    assertEquals(outputCondition, inputCondition);
    assertEquals(
      outputCondition.getConditions(), inputCondition.getConditions());
  }

  private SimpleAlgorithmConfigurator createAlgorithmVersion() {
    AlgorithmInfo algorithmInfo = new AlgorithmInfo("ID Algoritmo",
      "Algoritmo", "DIR Algoritmo", new Hashtable<String, String>());
    AlgorithmVersionId algorithmVersionId = new AlgorithmVersionId(1, 2, 3);
    AlgorithmVersionInfo algorithmVersionInfo = new AlgorithmVersionInfo(
      algorithmInfo, algorithmVersionId,
      new Hashtable<String, List<FileInfo>>(),
      new Hashtable<String, String>());
    return new SimpleAlgorithmConfigurator(algorithmVersionInfo, null,
      ExecutionType.SIMPLE, null, false, "Comando", null, null, true);
  }
}
