package csbase.logic.algorithms.parameters.triggers;

import java.util.Hashtable;
import java.util.List;

import junit.framework.TestCase;
import csbase.logic.FileInfo;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.ExecutionType;
import csbase.logic.algorithms.parameters.IntegerListParameter;
import csbase.logic.algorithms.parameters.IntegerParameter;
import csbase.logic.algorithms.parameters.ParameterGroup;
import csbase.logic.algorithms.parameters.SimpleAlgorithmConfigurator;
import csbase.logic.algorithms.parameters.conditions.Condition;
import csbase.logic.algorithms.parameters.conditions.SimpleCondition;

/**
 * Testes unitrios para a classe {@link ChangeMaximumForIntegersTrigger}.
 * 
 * @author lmoreira
 */
public final class ChangeMaximumForIntegersTriggerTest extends TestCase {

  /**
   * Testando criar um objeto da classe {@link ChangeMaximumForIntegersTrigger} 
   * utilizando um parmetro da classe {@link IntegerParameter} com mximo.
   */
  public void testChangeMaximumTrigger1() {
    IntegerParameter parameter = createMockIntegerParameter();
    Condition condition = createMockCondition();
    Integer maximum = 10;
    ChangeMaximumForIntegersTrigger trigger = new ChangeMaximumForIntegersTrigger(parameter,
      condition, maximum);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(maximum, trigger.getMaximum());
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMaximumForIntegersTrigger} utilizando
   * um parmetro da classe {@link IntegerParameter} sem mximo.
   */
  public void testChangeMaximumTrigger2() {
    IntegerParameter parameter = createMockIntegerParameter();
    Condition condition = createMockCondition();
    Integer maximum = null;
    ChangeMaximumForIntegersTrigger trigger = new ChangeMaximumForIntegersTrigger(parameter,
      condition, maximum);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(maximum, trigger.getMaximum());
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMaximumForIntegersTrigger} utilizando
   * um parmetro da classe {@link IntegerListParameter} com mximo.
   */
  public void testChangeMaximumTrigger3() {
    IntegerListParameter parameter = createMockIntegerListParameter();
    Condition condition = createMockCondition();
    Integer maximum = 10;
    ChangeMaximumForIntegersTrigger trigger = new ChangeMaximumForIntegersTrigger(parameter,
      condition, maximum);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(maximum, trigger.getMaximum());    
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMaximumForIntegersTrigger} utilizando
   * um parmetro da classe {@link IntegerListParameter} sem mximo.
   */
  public void testChangeMaximumTrigger4() {
    IntegerListParameter parameter = createMockIntegerListParameter();
    Condition condition = createMockCondition();
    Integer maximum = null;
    ChangeMaximumForIntegersTrigger trigger = new ChangeMaximumForIntegersTrigger(parameter,
      condition, maximum);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(maximum, trigger.getMaximum());    
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMaximumForIntegersTrigger} com
   * parmetro {@link IntegerParameter} {@code null}.
   */
  public void testChangeMaximumTrigger5() {
    IntegerParameter parameter = null;
    Condition condition = createMockCondition();
    Integer maximum = 10;
    try {
      new ChangeMaximumForIntegersTrigger(parameter, condition, maximum);
      fail();
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMaximumForIntegersTrigger} com
   * parmetro {@link IntegerListParameter} {@code null}.
   */
  public void testChangeMaximumTrigger6() {
    IntegerListParameter parameter = null;
    Condition condition = createMockCondition();
    Integer maximum = 10;
    try {
      new ChangeMaximumForIntegersTrigger(parameter, condition, maximum);
      fail();
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMaximumForIntegersTrigger} com
   * parmetro {@link IntegerParameter} e condio {@code null}.
   */
  public void testChangeMaximumTrigger7() {
    IntegerParameter parameter = createMockIntegerParameter();
    Condition condition = null;
    Integer maximum = 10;    
    try {
      new ChangeMaximumForIntegersTrigger(parameter, condition, maximum);
      fail();
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMaximumForIntegersTrigger} com
   * parmetro {@link IntegerListParameter} e condio {@code null}.
   */
  public void testChangeMaximumTrigger8() {
    IntegerListParameter parameter = createMockIntegerListParameter();
    Condition condition = null;
    Integer maximum = 10;    
    try {
      new ChangeMaximumForIntegersTrigger(parameter, condition, maximum);
      fail();
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForIntegersTrigger#doAction()} com parmetro do
   * tipo {@link IntegerParameter} com condio verdadeira.
   */
  public void testDoAction1() {
    String parameterName = "Nome do parmetro";
    Integer defaultValue = 10;
    Integer maximum = 21;
    IntegerParameter parameter = createMockIntegerParameter(parameterName,
      defaultValue, maximum);
    assertEquals(maximum, parameter.getMaximum());
    Condition condition = new SimpleCondition(parameterName, defaultValue);
    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Grupo de parmetros");
    group.addParameter(parameter);
    configurator.addGroup(group);
    assertTrue(condition.evaluate(configurator));
    Integer newMaximum = 31;
    ChangeMaximumForIntegersTrigger trigger = new ChangeMaximumForIntegersTrigger(parameter,
      condition, newMaximum);
    assertTrue(trigger.update(configurator));
    assertEquals(newMaximum, parameter.getMaximum());
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForIntegersTrigger#doAction()} com parmetro do
   * tipo {@link IntegerListParameter} com condio verdadeira.
   */
  public void testDoAction2() {
    String parameterName = "Nome do parmetro";
    Integer maximum = 21;
    IntegerListParameter parameter = createMockIntegerListParameter(
      parameterName, maximum);
    assertEquals(maximum, parameter.getMaximum());
    Condition condition = new SimpleCondition(parameterName, null);
    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Grupo de parmetros");
    group.addParameter(parameter);
    configurator.addGroup(group);
    assertTrue(condition.evaluate(configurator));
    Integer newMaximum = 31;
    ChangeMaximumForIntegersTrigger trigger = new ChangeMaximumForIntegersTrigger(parameter,
      condition, newMaximum);
    assertTrue(trigger.update(configurator));
    assertEquals(newMaximum, parameter.getMaximum());
  }


  /**
   * Testando o mtodo {@link ChangeMaximumForIntegersTrigger#doAction()} com parmetro do
   * tipo {@link IntegerParameter} com condio falsa.
   */
  public void testDoAction3() {
    String parameterName = "Nome do parmetro";
    Integer defaultValue = 10;
    Integer maximum = 21;
    IntegerParameter parameter = createMockIntegerParameter(parameterName,
      defaultValue, maximum);
    assertEquals(maximum, parameter.getMaximum());
    Condition condition = new SimpleCondition(parameterName, null);
    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Grupo de parmetros");
    group.addParameter(parameter);
    configurator.addGroup(group);
    assertFalse(condition.evaluate(configurator));
    Integer newMaximum = 31;
    ChangeMaximumForIntegersTrigger trigger = new ChangeMaximumForIntegersTrigger(parameter,
      condition, newMaximum);
    assertFalse(trigger.update(configurator));
    assertEquals(maximum, parameter.getMaximum());
    
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForIntegersTrigger#doAction()} com parmetro do
   * tipo {@link IntegerListParameter} com condio falsa.
   */
  public void testDoAction4() {
    String parameterName = "Nome do parmetro";
    Integer maximum = 21;
    IntegerListParameter parameter = createMockIntegerListParameter(
      parameterName, maximum);
    assertEquals(maximum, parameter.getMaximum());
    Integer value = 34;
    Condition condition = new SimpleCondition(parameterName, value);
    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Grupo de parmetros");
    group.addParameter(parameter);
    configurator.addGroup(group);
    assertFalse(condition.evaluate(configurator));
    Integer newMaximum = 31;
    ChangeMaximumForIntegersTrigger trigger = new ChangeMaximumForIntegersTrigger(parameter,
      condition, newMaximum);
    assertFalse(trigger.update(configurator));
    assertEquals(maximum, parameter.getMaximum());
    
  }
  
  /**
   * Testando o mtodo {@link ChangeMaximumForIntegersTrigger#equals(Object)} com parmetro
   * do tipo {@link IntegerParameter} e com gatilhos iguais.
   */
  public void testEquals1() {
    String parameterName = "Nome do parmetro";
    IntegerParameter parameter = createMockIntegerParameter(parameterName);
    Integer maximum = 10;
    Condition condition1 = new SimpleCondition("Nome do parmetro 1", "Valor 1");
    Condition condition2 = new SimpleCondition("Nome do parmetro 2", "Valor 2");
    ChangeMaximumForIntegersTrigger trigger1 = new ChangeMaximumForIntegersTrigger(parameter,
      condition1, maximum);
    ChangeMaximumForIntegersTrigger trigger2 = new ChangeMaximumForIntegersTrigger(parameter,
      condition2, maximum);
    assertTrue(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForIntegersTrigger#equals(Object)} com parmetro
   * do tipo {@link IntegerListParameter} e com gatilhos iguais.
   */
  public void testEquals2() {
    String parameterName = "Nome do parmetro";
    IntegerListParameter parameter = createMockIntegerListParameter(parameterName);
    Integer maximum = 10;
    Condition condition1 = new SimpleCondition("Nome do parmetro 1", "Valor 1");
    Condition condition2 = new SimpleCondition("Nome do parmetro 2", "Valor 2");
    ChangeMaximumForIntegersTrigger trigger1 = new ChangeMaximumForIntegersTrigger(parameter,
      condition1, maximum);
    ChangeMaximumForIntegersTrigger trigger2 = new ChangeMaximumForIntegersTrigger(parameter,
      condition2, maximum);
    assertTrue(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForIntegersTrigger#equals(Object)} com parmetro
   * do tipo {@link IntegerParameter} e com nomes de parmetro diferentes.
   */
  public void testEquals3() {
    String parameterName1 = "Nome do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    IntegerParameter parameter1 = createMockIntegerParameter(parameterName1);
    IntegerParameter parameter2 = createMockIntegerParameter(parameterName2);
    Integer maximum = 10;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMaximumForIntegersTrigger trigger1 = new ChangeMaximumForIntegersTrigger(parameter1,
      condition, maximum);
    ChangeMaximumForIntegersTrigger trigger2 = new ChangeMaximumForIntegersTrigger(parameter2,
      condition, maximum);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForIntegersTrigger#equals(Object)} com parmetro
   * do tipo {@link IntegerListParameter} e com nomes de parmetro diferentes.
   */
  public void testEquals4() {
    String parameterName1 = "Nome do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    IntegerListParameter parameter1 = createMockIntegerListParameter(parameterName1);
    IntegerListParameter parameter2 = createMockIntegerListParameter(parameterName2);
    Integer maximum = 10;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMaximumForIntegersTrigger trigger1 = new ChangeMaximumForIntegersTrigger(parameter1,
      condition, maximum);
    ChangeMaximumForIntegersTrigger trigger2 = new ChangeMaximumForIntegersTrigger(parameter2,
      condition, maximum);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForIntegersTrigger#equals(Object)} com parmetro
   * do tipo {@link IntegerParameter} e com mximos diferentes.
   */
  public void testEquals5() {
    String parameterName = "Nome do parmetro";
    IntegerParameter parameter = createMockIntegerParameter(parameterName);
    Integer maximum1 = 11;
    Integer maximum2 = 22;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMaximumForIntegersTrigger trigger1 = new ChangeMaximumForIntegersTrigger(parameter,
      condition, maximum1);
    ChangeMaximumForIntegersTrigger trigger2 = new ChangeMaximumForIntegersTrigger(parameter,
      condition, maximum2);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForIntegersTrigger#equals(Object)} com parmetro
   * do tipo {@link IntegerListParameter} e com mximos diferentes.
   */
  public void testEquals6() {
    String parameterName = "Nome do parmetro";
    IntegerListParameter parameter = createMockIntegerListParameter(parameterName);
    Integer maximum1 = 11;
    Integer maximum2 = 22;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMaximumForIntegersTrigger trigger1 = new ChangeMaximumForIntegersTrigger(parameter,
      condition, maximum1);
    ChangeMaximumForIntegersTrigger trigger2 = new ChangeMaximumForIntegersTrigger(parameter,
      condition, maximum2);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForIntegersTrigger#equals(Object)} com parmetro
   * com parmetro de tipos diferentes.
   */
  public void testEquals7() {
    String parameterName = "Nome do parmetro";
    IntegerListParameter parameter1 = createMockIntegerListParameter(parameterName);
    IntegerParameter parameter2 = createMockIntegerParameter(parameterName);
    Integer maximum = 10;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMaximumForIntegersTrigger trigger1 = new ChangeMaximumForIntegersTrigger(parameter1,
      condition, maximum);
    ChangeMaximumForIntegersTrigger trigger2 = new ChangeMaximumForIntegersTrigger(parameter2,
      condition, maximum);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForIntegersTrigger#equals(Object)} com parmetro
   * do tipo {@link IntegerParameter} e com argumento {@code null}.
   */
  public void testEquals8() {
    String parameterName = "Nome do parmetro";
    IntegerParameter parameter = createMockIntegerParameter(parameterName);
    Integer maximum = 10;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMaximumForIntegersTrigger trigger = new ChangeMaximumForIntegersTrigger(parameter,
      condition, maximum);
    assertFalse(trigger.equals(null));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForIntegersTrigger#equals(Object)} com parmetro
   * do tipo {@link IntegerParameter} e com argumento de outra classe.
   */
  public void testEquals9() {
    String parameterName = "Nome do parmetro";
    IntegerParameter parameter = createMockIntegerParameter(parameterName);
    Integer maximum = 10;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMaximumForIntegersTrigger trigger = new ChangeMaximumForIntegersTrigger(parameter,
      condition, maximum);
    assertFalse(trigger.equals("Outra classe"));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForIntegersTrigger#equals(Object)} com parmetro
   * do tipo {@link IntegerListParameter} e com argumento {@code null}.
   */
  public void testEquals10() {
    String parameterName = "Nome do parmetro";
    IntegerListParameter parameter = createMockIntegerListParameter(parameterName);
    Integer maximum = 10;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMaximumForIntegersTrigger trigger = new ChangeMaximumForIntegersTrigger(parameter,
      condition, maximum);
    assertFalse(trigger.equals(null));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForIntegersTrigger#equals(Object)} com parmetro
   * do tipo {@link IntegerListParameter} e com argumento de outra classe.
   */
  public void testEquals13() {
    String parameterName = "Nome do parmetro";
    IntegerListParameter parameter = createMockIntegerListParameter(parameterName);
    Integer maximum = 10;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMaximumForIntegersTrigger trigger = new ChangeMaximumForIntegersTrigger(parameter,
      condition, maximum);
    assertFalse(trigger.equals("Outra classe"));
  }

  /**
   * Cria um configurador de algorithms qualquer.
   * 
   * @return .
   */
  private SimpleAlgorithmConfigurator createMockAlgorithmConfigurator() {
    AlgorithmInfo algorithmInfo = new AlgorithmInfo("ID Algoritmo",
      "Algoritmo", "DIR Algoritmo", new Hashtable<String, String>());
    AlgorithmVersionId algorithmVersionId = new AlgorithmVersionId(1, 2, 3);
    AlgorithmVersionInfo algorithmVersionInfo = new AlgorithmVersionInfo(
      algorithmInfo, algorithmVersionId,
      new Hashtable<String, List<FileInfo>>(),
      new Hashtable<String, String>());
    return new SimpleAlgorithmConfigurator(algorithmVersionInfo, null,
      ExecutionType.SIMPLE, null, false, "Comando", null, null, true);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link IntegerListParameter}.
   * 
   * @return .
   */
  private IntegerListParameter createMockIntegerListParameter() {
    String parameterName = "Nome do parmetro";
    return createMockIntegerListParameter(parameterName);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link IntegerListParameter}.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * 
   * @return .
   */
  private IntegerListParameter createMockIntegerListParameter(String parameterName) {
    Integer maximum = null;
    return createMockIntegerListParameter(parameterName, maximum);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link IntegerListParameter}.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * @param maximum O mximo (Aceita {@code null}).
   * 
   * @return .
   */
  private IntegerListParameter createMockIntegerListParameter(
    String parameterName, Integer maximum) {
    return new IntegerListParameter(parameterName, "Rtulo do parmetro",
      "Descrio do parmetro", false, "{0}={1}", maximum, null);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link IntegerParameter}.
   * 
   * @return .
   */
  private IntegerParameter createMockIntegerParameter() {
    String parameterName = "Nome do parmetro";
    return createMockIntegerParameter(parameterName);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link IntegerParameter}.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * 
   * @return .
   */
  private IntegerParameter createMockIntegerParameter(String parameterName) {
    Integer defaultValue = null;
    Integer maximum = null;
    return createMockIntegerParameter(parameterName, defaultValue, maximum);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link IntegerParameter}.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * @param defaultValue O valor-padro (Aceita {@code null}).
   * @param maximum O mximo (Aceita {@code null}).
   * 
   * @return .
   */
  private IntegerParameter createMockIntegerParameter(String parameterName,
    Integer defaultValue, Integer maximum) {
    return new IntegerParameter(parameterName, "Rtulo do parmetro",
      "Descrio do parmetro", defaultValue, false, "{0}={1}", maximum, null);
  }

  /**
   * Cria uma condio qualquer.
   * 
   * @return .
   */
  private Condition createMockCondition() {
    return new SimpleCondition("Nome do parmetro", "Valor qualquer");
  }
}
