package csbase.logic.algorithms.parameters.triggers;

import java.util.Hashtable;
import java.util.List;

import junit.framework.TestCase;
import csbase.logic.FileInfo;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.ExecutionType;
import csbase.logic.algorithms.parameters.DoubleListParameter;
import csbase.logic.algorithms.parameters.DoubleParameter;
import csbase.logic.algorithms.parameters.ParameterGroup;
import csbase.logic.algorithms.parameters.SimpleAlgorithmConfigurator;
import csbase.logic.algorithms.parameters.conditions.Condition;
import csbase.logic.algorithms.parameters.conditions.SimpleCondition;

/**
 * Testes unitrios para a classe {@link ChangeMinimumForDoublesTrigger}.
 * 
 * @author lmoreira
 */
public final class ChangeMinimumForDoublesTriggerTest extends TestCase {

  /**
   * Testando criar um objeto da classe {@link ChangeMinimumForDoublesTrigger} utilizando
   * um parmetro da classe {@link DoubleParameter} com mnimo e incluir mnimo
   * verdadeiro.
   */
  public void testChangeMinimumTrigger1() {
    DoubleParameter parameter = createMockDoubleParameter();
    Condition condition = createMockCondition();
    Double minimum = 10.5;
    boolean includeMinimum = true;
    ChangeMinimumForDoublesTrigger trigger = new ChangeMinimumForDoublesTrigger(parameter,
      condition, minimum, includeMinimum);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(minimum, trigger.getMinimum());
    assertEquals(includeMinimum, trigger.isMinimumIncluded());
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMinimumForDoublesTrigger} utilizando
   * um parmetro da classe {@link DoubleParameter} com mnimo e incluir mnimo
   * falso.
   */
  public void testChangeMinimumTrigger2() {
    DoubleParameter parameter = createMockDoubleParameter();
    Condition condition = createMockCondition();
    Double minimum = 10.5;
    boolean includeMinimum = false;
    ChangeMinimumForDoublesTrigger trigger = new ChangeMinimumForDoublesTrigger(parameter,
      condition, minimum, includeMinimum);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(minimum, trigger.getMinimum());
    assertEquals(includeMinimum, trigger.isMinimumIncluded());
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMinimumForDoublesTrigger} utilizando
   * um parmetro da classe {@link DoubleParameter} sem mnimo.
   */
  public void testChangeMinimumTrigger3() {
    DoubleParameter parameter = createMockDoubleParameter();
    Condition condition = createMockCondition();
    Double minimum = null;
    boolean includeMinimum = false;
    ChangeMinimumForDoublesTrigger trigger = new ChangeMinimumForDoublesTrigger(parameter,
      condition, minimum, includeMinimum);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(minimum, trigger.getMinimum());
    assertEquals(includeMinimum, trigger.isMinimumIncluded());
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMinimumForDoublesTrigger} utilizando
   * um parmetro da classe {@link DoubleListParameter} com mnimo e incluir
   * mnimo verdadeiro.
   */
  public void testChangeMinimumTrigger4() {
    DoubleListParameter parameter = createMockDoubleListParameter();
    Condition condition = createMockCondition();
    Double minimum = 10.5;
    boolean includeMinimum = true;
    ChangeMinimumForDoublesTrigger trigger = new ChangeMinimumForDoublesTrigger(parameter,
      condition, minimum, includeMinimum);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(minimum, trigger.getMinimum());
    assertEquals(includeMinimum, trigger.isMinimumIncluded());
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMinimumForDoublesTrigger} utilizando
   * um parmetro da classe {@link DoubleListParameter} com mnimo e incluir
   * mnimo falso.
   */
  public void testChangeMinimumTrigger5() {
    DoubleListParameter parameter = createMockDoubleListParameter();
    Condition condition = createMockCondition();
    Double minimum = 10.5;
    boolean includeMinimum = false;
    ChangeMinimumForDoublesTrigger trigger = new ChangeMinimumForDoublesTrigger(parameter,
      condition, minimum, includeMinimum);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(minimum, trigger.getMinimum());
    assertEquals(includeMinimum, trigger.isMinimumIncluded());
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMinimumForDoublesTrigger} utilizando
   * um parmetro da classe {@link DoubleListParameter} sem mnimo.
   */
  public void testChangeMinimumTrigger6() {
    DoubleListParameter parameter = createMockDoubleListParameter();
    Condition condition = createMockCondition();
    Double minimum = null;
    boolean includeMinimum = false;
    ChangeMinimumForDoublesTrigger trigger = new ChangeMinimumForDoublesTrigger(parameter,
      condition, minimum, includeMinimum);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(minimum, trigger.getMinimum());
    assertEquals(includeMinimum, trigger.isMinimumIncluded());
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMinimumForDoublesTrigger} com
   * parmetro {@link DoubleParameter} {@code null}.
   */
  public void testChangeMinimumTrigger7() {
    DoubleParameter parameter = null;
    Condition condition = createMockCondition();
    Double minimum = 10.5;
    boolean includeMinimum = true;
    try {
      new ChangeMinimumForDoublesTrigger(parameter, condition, minimum, includeMinimum);
      fail();
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMinimumForDoublesTrigger} com
   * parmetro {@link DoubleListParameter} {@code null}.
   */
  public void testChangeMinimumTrigger8() {
    DoubleListParameter parameter = null;
    Condition condition = createMockCondition();
    Double minimum = 10.5;
    boolean includeMinimum = true;
    try {
      new ChangeMinimumForDoublesTrigger(parameter, condition, minimum, includeMinimum);
      fail();
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMinimumForDoublesTrigger} com
   * parmetro {@link DoubleParameter} e condio {@code null}.
   */
  public void testChangeMinimumTrigger9() {
    DoubleParameter parameter = createMockDoubleParameter();
    Condition condition = null;
    Double minimum = 10.5;
    boolean includeMinimum = true;
    try {
      new ChangeMinimumForDoublesTrigger(parameter, condition, minimum, includeMinimum);
      fail();
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMinimumForDoublesTrigger} com
   * parmetro {@link DoubleListParameter} e condio {@code null}.
   */
  public void testChangeMinimumTrigger10() {
    DoubleListParameter parameter = createMockDoubleListParameter();
    Condition condition = null;
    Double minimum = 10.5;
    boolean includeMinimum = true;
    try {
      new ChangeMinimumForDoublesTrigger(parameter, condition, minimum, includeMinimum);
      fail();
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForDoublesTrigger#doAction()} com parmetro do
   * tipo {@link DoubleParameter} com condio verdadeira.
   */
  public void testDoAction1() {
    String parameterName = "Nome do parmetro";
    Double defaultValue = 10.5;
    Double minimum = 21.0;
    boolean isMinimumIncluded = true;
    DoubleParameter parameter = createMockDoubleParameter(parameterName,
      defaultValue, minimum, isMinimumIncluded);
    assertEquals(minimum, parameter.getMinimum());
    assertEquals(isMinimumIncluded, parameter.isMinimumIncluded());
    Condition condition = new SimpleCondition(parameterName, defaultValue);
    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Grupo de parmetros");
    group.addParameter(parameter);
    configurator.addGroup(group);
    assertTrue(condition.evaluate(configurator));
    Double newMinimum = 31.5;
    boolean newIsMinimumIncluded = !isMinimumIncluded;
    ChangeMinimumForDoublesTrigger trigger = new ChangeMinimumForDoublesTrigger(parameter,
      condition, newMinimum, newIsMinimumIncluded);
    assertTrue(trigger.update(configurator));
    assertEquals(newMinimum, parameter.getMinimum());
    assertEquals(newIsMinimumIncluded, parameter.isMinimumIncluded());
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForDoublesTrigger#doAction()} com parmetro do
   * tipo {@link DoubleListParameter} com condio verdadeira.
   */
  public void testDoAction2() {
    String parameterName = "Nome do parmetro";
    Double minimum = 21.0;
    boolean isMinimumIncluded = true;
    DoubleListParameter parameter = createMockDoubleListParameter(
      parameterName, minimum, isMinimumIncluded);
    assertEquals(minimum, parameter.getMinimum());
    assertEquals(isMinimumIncluded, parameter.isMinimumIncluded());
    Condition condition = new SimpleCondition(parameterName, null);
    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Grupo de parmetros");
    group.addParameter(parameter);
    configurator.addGroup(group);
    assertTrue(condition.evaluate(configurator));
    Double newMinimum = 31.5;
    boolean newIsMinimumIncluded = !isMinimumIncluded;
    ChangeMinimumForDoublesTrigger trigger = new ChangeMinimumForDoublesTrigger(parameter,
      condition, newMinimum, newIsMinimumIncluded);
    assertTrue(trigger.update(configurator));
    assertEquals(newMinimum, parameter.getMinimum());
    assertEquals(newIsMinimumIncluded, parameter.isMinimumIncluded());
  }


  /**
   * Testando o mtodo {@link ChangeMinimumForDoublesTrigger#doAction()} com parmetro do
   * tipo {@link DoubleParameter} com condio falsa.
   */
  public void testDoAction3() {
    String parameterName = "Nome do parmetro";
    Double defaultValue = 10.5;
    Double minimum = 21.0;
    boolean isMinimumIncluded = true;
    DoubleParameter parameter = createMockDoubleParameter(parameterName,
      defaultValue, minimum, isMinimumIncluded);
    assertEquals(minimum, parameter.getMinimum());
    assertEquals(isMinimumIncluded, parameter.isMinimumIncluded());
    Condition condition = new SimpleCondition(parameterName, null);
    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Grupo de parmetros");
    group.addParameter(parameter);
    configurator.addGroup(group);
    assertFalse(condition.evaluate(configurator));
    Double newMinimum = 31.5;
    boolean newIsMinimumIncluded = !isMinimumIncluded;
    ChangeMinimumForDoublesTrigger trigger = new ChangeMinimumForDoublesTrigger(parameter,
      condition, newMinimum, newIsMinimumIncluded);
    assertFalse(trigger.update(configurator));
    assertEquals(minimum, parameter.getMinimum());
    assertEquals(isMinimumIncluded, parameter.isMinimumIncluded());
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForDoublesTrigger#doAction()} com parmetro do
   * tipo {@link DoubleListParameter} com condio falsa.
   */
  public void testDoAction4() {
    String parameterName = "Nome do parmetro";
    Double minimum = 21.0;
    boolean isMinimumIncluded = true;
    DoubleListParameter parameter = createMockDoubleListParameter(
      parameterName, minimum, isMinimumIncluded);
    assertEquals(minimum, parameter.getMinimum());
    assertEquals(isMinimumIncluded, parameter.isMinimumIncluded());
    Double value = 34.5;
    Condition condition = new SimpleCondition(parameterName, value);
    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Grupo de parmetros");
    group.addParameter(parameter);
    configurator.addGroup(group);
    assertFalse(condition.evaluate(configurator));
    Double newMinimum = 31.5;
    boolean newIsMinimumIncluded = !isMinimumIncluded;
    ChangeMinimumForDoublesTrigger trigger = new ChangeMinimumForDoublesTrigger(parameter,
      condition, newMinimum, newIsMinimumIncluded);
    assertFalse(trigger.update(configurator));
    assertEquals(minimum, parameter.getMinimum());
    assertEquals(isMinimumIncluded, parameter.isMinimumIncluded());
  }
  
  /**
   * Testando o mtodo {@link ChangeMinimumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleParameter} e com gatilhos iguais.
   */
  public void testEquals1() {
    String parameterName = "Nome do parmetro";
    DoubleParameter parameter = createMockDoubleParameter(parameterName);
    Double minimum = 10.5;
    boolean isMinimumIncluded = true;
    Condition condition1 = new SimpleCondition("Nome do parmetro 1", "Valor 1");
    Condition condition2 = new SimpleCondition("Nome do parmetro 2", "Valor 2");
    ChangeMinimumForDoublesTrigger trigger1 = new ChangeMinimumForDoublesTrigger(parameter,
      condition1, minimum, isMinimumIncluded);
    ChangeMinimumForDoublesTrigger trigger2 = new ChangeMinimumForDoublesTrigger(parameter,
      condition2, minimum, isMinimumIncluded);
    assertTrue(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleListParameter} e com gatilhos iguais.
   */
  public void testEquals2() {
    String parameterName = "Nome do parmetro";
    DoubleListParameter parameter = createMockDoubleListParameter(parameterName);
    Double minimum = 10.5;
    boolean isMinimumIncluded = true;
    Condition condition1 = new SimpleCondition("Nome do parmetro 1", "Valor 1");
    Condition condition2 = new SimpleCondition("Nome do parmetro 2", "Valor 2");
    ChangeMinimumForDoublesTrigger trigger1 = new ChangeMinimumForDoublesTrigger(parameter,
      condition1, minimum, isMinimumIncluded);
    ChangeMinimumForDoublesTrigger trigger2 = new ChangeMinimumForDoublesTrigger(parameter,
      condition2, minimum, isMinimumIncluded);
    assertTrue(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleParameter} e com nomes de parmetro diferentes.
   */
  public void testEquals3() {
    String parameterName1 = "Nome do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    DoubleParameter parameter1 = createMockDoubleParameter(parameterName1);
    DoubleParameter parameter2 = createMockDoubleParameter(parameterName2);
    Double minimum = 10.5;
    boolean isMinimumIncluded = true;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMinimumForDoublesTrigger trigger1 = new ChangeMinimumForDoublesTrigger(parameter1,
      condition, minimum, isMinimumIncluded);
    ChangeMinimumForDoublesTrigger trigger2 = new ChangeMinimumForDoublesTrigger(parameter2,
      condition, minimum, isMinimumIncluded);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleListParameter} e com nomes de parmetro diferentes.
   */
  public void testEquals4() {
    String parameterName1 = "Nome do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    DoubleListParameter parameter1 = createMockDoubleListParameter(parameterName1);
    DoubleListParameter parameter2 = createMockDoubleListParameter(parameterName2);
    Double minimum = 10.5;
    boolean isMinimumIncluded = true;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMinimumForDoublesTrigger trigger1 = new ChangeMinimumForDoublesTrigger(parameter1,
      condition, minimum, isMinimumIncluded);
    ChangeMinimumForDoublesTrigger trigger2 = new ChangeMinimumForDoublesTrigger(parameter2,
      condition, minimum, isMinimumIncluded);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleParameter} e com mnimos diferentes.
   */
  public void testEquals5() {
    String parameterName = "Nome do parmetro";
    DoubleParameter parameter = createMockDoubleParameter(parameterName);
    Double minimum1 = 11.1;
    Double minimum2 = 22.2;
    boolean isMinimumIncluded = true;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMinimumForDoublesTrigger trigger1 = new ChangeMinimumForDoublesTrigger(parameter,
      condition, minimum1, isMinimumIncluded);
    ChangeMinimumForDoublesTrigger trigger2 = new ChangeMinimumForDoublesTrigger(parameter,
      condition, minimum2, isMinimumIncluded);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleListParameter} e com mnimos diferentes.
   */
  public void testEquals6() {
    String parameterName = "Nome do parmetro";
    DoubleListParameter parameter = createMockDoubleListParameter(parameterName);
    Double minimum1 = 11.1;
    Double minimum2 = 22.2;
    boolean isMinimumIncluded = true;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMinimumForDoublesTrigger trigger1 = new ChangeMinimumForDoublesTrigger(parameter,
      condition, minimum1, isMinimumIncluded);
    ChangeMinimumForDoublesTrigger trigger2 = new ChangeMinimumForDoublesTrigger(parameter,
      condition, minimum2, isMinimumIncluded);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleParameter} e com incluir mnimo diferentes.
   */
  public void testEquals7() {
    String parameterName = "Nome do parmetro";
    DoubleParameter parameter = createMockDoubleParameter(parameterName);
    Double minimum = 10.5;
    boolean isMinimumIncluded1 = true;
    boolean isMinimumIncluded2 = false;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMinimumForDoublesTrigger trigger1 = new ChangeMinimumForDoublesTrigger(parameter,
      condition, minimum, isMinimumIncluded1);
    ChangeMinimumForDoublesTrigger trigger2 = new ChangeMinimumForDoublesTrigger(parameter,
      condition, minimum, isMinimumIncluded2);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleListParameter} e com incluir mnimo diferentes.
   */
  public void testEquals8() {
    String parameterName = "Nome do parmetro";
    DoubleListParameter parameter = createMockDoubleListParameter(parameterName);
    Double minimum = 10.5;
    boolean isMinimumIncluded1 = true;
    boolean isMinimumIncluded2 = false;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMinimumForDoublesTrigger trigger1 = new ChangeMinimumForDoublesTrigger(parameter,
      condition, minimum, isMinimumIncluded1);
    ChangeMinimumForDoublesTrigger trigger2 = new ChangeMinimumForDoublesTrigger(parameter,
      condition, minimum, isMinimumIncluded2);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForDoublesTrigger#equals(Object)} com parmetro
   * com parmetro de tipos diferentes.
   */
  public void testEquals9() {
    String parameterName = "Nome do parmetro";
    DoubleListParameter parameter1 = createMockDoubleListParameter(parameterName);
    DoubleParameter parameter2 = createMockDoubleParameter(parameterName);
    Double minimum = 10.5;
    boolean isMinimumIncluded = true;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMinimumForDoublesTrigger trigger1 = new ChangeMinimumForDoublesTrigger(parameter1,
      condition, minimum, isMinimumIncluded);
    ChangeMinimumForDoublesTrigger trigger2 = new ChangeMinimumForDoublesTrigger(parameter2,
      condition, minimum, isMinimumIncluded);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleParameter} e com argumento {@code null}.
   */
  public void testEquals10() {
    String parameterName = "Nome do parmetro";
    DoubleParameter parameter = createMockDoubleParameter(parameterName);
    Double minimum = 10.5;
    boolean isMinimumIncluded = true;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMinimumForDoublesTrigger trigger = new ChangeMinimumForDoublesTrigger(parameter,
      condition, minimum, isMinimumIncluded);
    assertFalse(trigger.equals(null));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleParameter} e com argumento de outra classe.
   */
  public void testEquals11() {
    String parameterName = "Nome do parmetro";
    DoubleParameter parameter = createMockDoubleParameter(parameterName);
    Double minimum = 10.5;
    boolean isMinimumIncluded = true;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMinimumForDoublesTrigger trigger = new ChangeMinimumForDoublesTrigger(parameter,
      condition, minimum, isMinimumIncluded);
    assertFalse(trigger.equals("Outra classe"));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleListParameter} e com argumento {@code null}.
   */
  public void testEquals12() {
    String parameterName = "Nome do parmetro";
    DoubleListParameter parameter = createMockDoubleListParameter(parameterName);
    Double minimum = 10.5;
    boolean isMinimumIncluded = true;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMinimumForDoublesTrigger trigger = new ChangeMinimumForDoublesTrigger(parameter,
      condition, minimum, isMinimumIncluded);
    assertFalse(trigger.equals(null));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleListParameter} e com argumento de outra classe.
   */
  public void testEquals13() {
    String parameterName = "Nome do parmetro";
    DoubleListParameter parameter = createMockDoubleListParameter(parameterName);
    Double minimum = 10.5;
    boolean isMinimumIncluded = true;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMinimumForDoublesTrigger trigger = new ChangeMinimumForDoublesTrigger(parameter,
      condition, minimum, isMinimumIncluded);
    assertFalse(trigger.equals("Outra classe"));
  }

  /**
   * Cria um configurador de algorithms qualquer.
   * 
   * @return .
   */
  private SimpleAlgorithmConfigurator createMockAlgorithmConfigurator() {
    AlgorithmInfo algorithmInfo = new AlgorithmInfo("ID Algoritmo",
      "Algoritmo", "DIR Algoritmo", new Hashtable<String, String>());
    AlgorithmVersionId algorithmVersionId = new AlgorithmVersionId(1, 2, 3);
    AlgorithmVersionInfo algorithmVersionInfo = new AlgorithmVersionInfo(
      algorithmInfo, algorithmVersionId,
      new Hashtable<String, List<FileInfo>>(),
      new Hashtable<String, String>());
    return new SimpleAlgorithmConfigurator(algorithmVersionInfo, null,
      ExecutionType.SIMPLE, null, false, "Comando", null, null, true);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link DoubleListParameter}.
   * 
   * @return .
   */
  private DoubleListParameter createMockDoubleListParameter() {
    String parameterName = "Nome do parmetro";
    return createMockDoubleListParameter(parameterName);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link DoubleListParameter}.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * 
   * @return .
   */
  private DoubleListParameter createMockDoubleListParameter(String parameterName) {
    Double minimum = null;
    boolean isMinimumIncluded = false;
    return createMockDoubleListParameter(parameterName, minimum,
      isMinimumIncluded);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link DoubleListParameter}.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * @param minimum O mnimo (Aceita {@code null}).
   * @param isMinimumIncluded Indica se o mnimo deve ser considerado um dos
   *        valores vlidos.
   * 
   * @return .
   */
  private DoubleListParameter createMockDoubleListParameter(
    String parameterName, Double minimum, boolean isMinimumIncluded) {
    return new DoubleListParameter(parameterName, "Rtulo do parmetro",
      "Descrio do parmetro", false, null, null, false, minimum, isMinimumIncluded);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link DoubleParameter}.
   * 
   * @return .
   */
  private DoubleParameter createMockDoubleParameter() {
    String parameterName = "Nome do parmetro";
    return createMockDoubleParameter(parameterName);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link DoubleParameter}.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * 
   * @return .
   */
  private DoubleParameter createMockDoubleParameter(String parameterName) {
    Double defaultValue = null;
    Double minimum = null;
    boolean isMinimumIncluded = false;
    return createMockDoubleParameter(parameterName, defaultValue, minimum,
      isMinimumIncluded);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link DoubleParameter}.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * @param defaultValue O valor-padro (Aceita {@code null}).
   * @param minimum O mnimo (Aceita {@code null}).
   * @param isMinimumIncluded Indica se o mnimo deve ser considerado um dos
   *        valores vlidos.
   * 
   * @return .
   */
  private DoubleParameter createMockDoubleParameter(String parameterName,
    Double defaultValue, Double minimum, boolean isMinimumIncluded) {
    return new DoubleParameter(parameterName, "Rtulo do parmetro",
      "Descrio do parmetro", defaultValue, false, null, null, false, minimum,
      isMinimumIncluded);
  }

  /**
   * Cria uma condio qualquer.
   * 
   * @return .
   */
  private Condition createMockCondition() {
    return new SimpleCondition("Nome do parmetro", "Valor qualquer");
  }

}
