/*
 * $Author:$
 * $Date:$
 * $Release:$
 */
package csbase.logic.algorithms.parameters.triggers;

import junit.framework.Assert;
import csbase.logic.algorithms.parameters.SimpleParameter;
import csbase.logic.algorithms.parameters.conditions.Condition;

/**
 * Gatilho de Verificao: permite verificar e proibir/permitir que mtodos do 
 * gatilho foram executados ou sejam executados. 
 *
 * @author lmoreira
 */
public final class CheckTrigger extends Trigger {
  /**
   * Indica se o mtodo {@link #doAction()} pode ser executado.
   */
  private boolean allowDo;
  
  /**
   * Indica se o mtodo {@link #undoAction()} pode ser executado.
   */
  private boolean allowUndo;
  
  /**
   * Indica se o mtodo {@link #doAction()} foi executado.
   */
  private boolean wasDone;
  
  /**
   * Indica se o mtodo {@link #undoAction()} foi executado.
   */
  private boolean wasUndone;

  /**
   * <p>Cria o gatilho.</p>
   * 
   * <p>O gatilho criado no permite que os mtodos {@link #doAction()} ou
   * {@link #undoAction()}.</p>
   * 
   * @param parameter O parmetro (No aceita {@code null}).
   * @param condition A condio (No aceita {@code null}).
   */
  public CheckTrigger(SimpleParameter<?> parameter, Condition condition) {
    super(parameter, condition);
    allowDo = false;
    allowUndo = false;
    wasDone = false;
    wasUndone = false;
  }
  
  /**
   * D permisso para executar o mtodo {@link #doAction()}.
   */
  public void allowDo() {
    allowDo = true;
    allowUndo = false;
  }

  /**
   * D permisso para executar o mtodo {@link #undoAction()}.
   */
  public void allowUndo() {
    allowUndo = true;
    allowDo = false;
  }


  /**
   * Remove a permisso para executar o mtodo {@link #undoAction()}.
   */
  public void prohibitDo() {
    allowDo = false;
  }

  /**
   * Remove a permisso para executar o mtodo {@link #undoAction()}.
   */
  public void prohibitUndo() {
    allowUndo = false;
  }
  
  /**
   * Indica se o mtodo {@link #doAction()} foi executado.
   * 
   * @return .
   */
  public boolean wasDone() {
    boolean wasDone = this.wasDone;
    this.wasDone = false;
    return wasDone;
  }

  /**
   * Indica se o mtodo {@link #undoAction()} foi executado.
   * 
   * @return .
   */
  public boolean wasUndone() {
    boolean wasUndone = this.wasUndone;
    this.wasUndone = false;
    return wasUndone;
  }

  @Override
  protected void doAction() {
    Assert.assertTrue(allowDo);
    wasDone = true;
  }

  @Override
  protected void undoAction() {
    Assert.assertTrue(allowUndo);
    wasUndone = true;
  }
}
