package csbase.logic.algorithms.parameters.triggers;

import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;

import junit.framework.TestCase;
import csbase.logic.FileInfo;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.ExecutionType;
import csbase.logic.algorithms.parameters.EnumerationItem;
import csbase.logic.algorithms.parameters.EnumerationParameter;
import csbase.logic.algorithms.parameters.ParameterGroup;
import csbase.logic.algorithms.parameters.SimpleAlgorithmConfigurator;
import csbase.logic.algorithms.parameters.conditions.Condition;
import csbase.logic.algorithms.parameters.conditions.SimpleCondition;

/**
 * Testes unitrios para a classe {@link SetVisibleEnumerationItemTrigger}.
 * 
 * @author lmoreira
 */
public final class SetVisibleEnumerationItemTriggerTest extends TestCase {

  /**
   * Testando criar um objeto da classe {@link SetVisibleEnumerationItemTrigger}:
   * Gatilho para habilitar um item.
   */
  public void testSetVisibleEnumerationItemTrigger1() {
    EnumerationParameter parameter = createMockEnumerationParameter();
    Condition condition = createMockCondition();
    EnumerationItem item = parameter.getDefaultValue();
    String itemId = item.getId();
    boolean isVisible = true;
    SetVisibleEnumerationItemTrigger trigger = new SetVisibleEnumerationItemTrigger(
      parameter, condition, itemId, isVisible);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(itemId, trigger.getItemId());
    assertEquals(isVisible, trigger.isVisible());
  }

  /**
   * Testando criar um objeto da classe {@link SetVisibleEnumerationItemTrigger}:
   * Gatilho para desabilitar um item.
   */
  public void testSetVisibleEnumerationItemTrigger2() {
    EnumerationParameter parameter = createMockEnumerationParameter();
    Condition condition = createMockCondition();
    EnumerationItem item = parameter.getDefaultValue();
    String itemId = item.getId();
    boolean isVisible = false;
    SetVisibleEnumerationItemTrigger trigger = new SetVisibleEnumerationItemTrigger(
      parameter, condition, itemId, isVisible);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(itemId, trigger.getItemId());
    assertEquals(isVisible, trigger.isVisible());
  }

  /**
   * Testando criar um objeto da classe {@link SetVisibleEnumerationItemTrigger}
   * com parmetro {@code null}.
   */
  public void testSetVisibleEnumerationItemTrigger3() {
    Condition condition = createMockCondition();
    EnumerationItem item = createMockEnumerationItem();
    String itemId = item.getId();
    try {
      new SetVisibleEnumerationItemTrigger(null, condition, itemId, false);
      fail();
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um objeto da classe {@link SetVisibleEnumerationItemTrigger}
   * com condiao {@code null}.
   */
  public void testSetVisibleEnumerationItemTrigger4() {
    EnumerationParameter parameter = createMockEnumerationParameter();
    EnumerationItem item = parameter.getDefaultValue();
    String itemId = item.getId();
    try {
      new SetVisibleEnumerationItemTrigger(parameter, null, itemId, false);
      fail();
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um objeto da classe {@link SetVisibleEnumerationItemTrigger}
   * com identificador do item {@code null}.
   */
  public void testSetVisibleEnumerationItemTrigger5() {
    EnumerationParameter parameter = createMockEnumerationParameter();
    Condition condition = createMockCondition();
    try {
      new SetVisibleEnumerationItemTrigger(parameter, condition, null, false);
      fail();
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando o mtodo {@link
   * SetVisibleEnumerationItemTrigger#update(SimpleAlgorithmConfigurator)} com
   * condio verdadeira e com um gatilho para habilitar.
   */
  public void testUpdate1() {
    String defaultItemId = "Identificador do item padro";
    EnumerationItem defaultItem = createMockEnumerationItem(defaultItemId);

    String otherItemId = "Identificador de outro item.";
    EnumerationItem otherItem = createMockEnumerationItem(otherItemId);

    assertTrue(otherItem.isVisible());

    String parameterName = "Nome do parmetro";
    EnumerationParameter parameter = createMockEnumerationParameter(
      parameterName, defaultItem, otherItem);

    assertEquals(defaultItem, parameter.getValue());

    Condition condition = new SimpleCondition(parameterName, defaultItem);

    ParameterGroup group = createMockParameterGroup();
    group.addParameter(parameter);

    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    configurator.addGroup(group);

    assertTrue(condition.evaluate(configurator));

    parameter.setVisibleItem(otherItemId, false);
    assertFalse(otherItem.isVisible());

    SetVisibleEnumerationItemTrigger trigger = new SetVisibleEnumerationItemTrigger(
      parameter, condition, otherItemId, true);

    assertTrue(trigger.update(configurator));

    assertTrue(otherItem.isVisible());
  }

  /**
   * Testando o mtodo {@link
   * SetVisibleEnumerationItemTrigger#update(SimpleAlgorithmConfigurator)} com
   * condio verdadeira e com um gatilho para desabilitar.
   */
  public void testUpdate2() {
    String defaultItemId = "Identificador do item padro";
    EnumerationItem defaultItem = createMockEnumerationItem(defaultItemId);

    String otherItemId = "Identificador de outro item.";
    EnumerationItem otherItem = createMockEnumerationItem(otherItemId);

    assertTrue(otherItem.isVisible());

    String parameterName = "Nome do parmetro";
    EnumerationParameter parameter = createMockEnumerationParameter(
      parameterName, defaultItem, otherItem);

    assertEquals(defaultItem, parameter.getValue());

    Condition condition = new SimpleCondition(parameterName, defaultItem);

    ParameterGroup group = createMockParameterGroup();
    group.addParameter(parameter);

    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    configurator.addGroup(group);

    assertTrue(condition.evaluate(configurator));

    SetVisibleEnumerationItemTrigger trigger = new SetVisibleEnumerationItemTrigger(
      parameter, condition, otherItemId, false);

    assertTrue(trigger.update(configurator));

    assertFalse(otherItem.isVisible());
  }

  /**
   * Testando o mtodo {@link
   * SetVisibleEnumerationItemTrigger#update(SimpleAlgorithmConfigurator)} com
   * condio falsa e com um gatilho para desabilitar.
   */
  public void testUpdate3() {
    String defaultItemId = "Identificador do item padro";
    EnumerationItem defaultItem = createMockEnumerationItem(defaultItemId);

    String otherItemId = "Identificador de outro item.";
    EnumerationItem otherItem = createMockEnumerationItem(otherItemId);

    assertTrue(otherItem.isVisible());

    String parameterName = "Nome do parmetro";
    EnumerationParameter parameter = createMockEnumerationParameter(
      parameterName, defaultItem, otherItem);

    assertEquals(defaultItem, parameter.getValue());

    parameter.setVisibleItem(otherItemId, false);
    assertFalse(otherItem.isVisible());

    Condition condition = new SimpleCondition(parameterName, otherItem);

    ParameterGroup group = createMockParameterGroup();
    group.addParameter(parameter);

    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    configurator.addGroup(group);

    assertFalse(condition.evaluate(configurator));

    SetVisibleEnumerationItemTrigger trigger = new SetVisibleEnumerationItemTrigger(
      parameter, condition, otherItemId, false);

    assertFalse(trigger.update(configurator));

    assertTrue(otherItem.isVisible());
  }

  /**
   * Testando o mtodo {@link
   * SetVisibleEnumerationItemTrigger#update(SimpleAlgorithmConfigurator)} com
   * condio falsa e com um gatilho para habilitar.
   */
  public void testUpdate4() {
    String defaultItemId = "Identificador do item padro";
    EnumerationItem defaultItem = createMockEnumerationItem(defaultItemId);

    String otherItemId = "Identificador de outro item.";
    EnumerationItem otherItem = createMockEnumerationItem(otherItemId);

    assertTrue(otherItem.isVisible());

    String parameterName = "Nome do parmetro";
    EnumerationParameter parameter = createMockEnumerationParameter(
      parameterName, defaultItem, otherItem);

    assertEquals(defaultItem, parameter.getValue());

    Condition condition = new SimpleCondition(parameterName, otherItem);

    ParameterGroup group = createMockParameterGroup();
    group.addParameter(parameter);

    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    configurator.addGroup(group);

    assertFalse(condition.evaluate(configurator));

    SetVisibleEnumerationItemTrigger trigger = new SetVisibleEnumerationItemTrigger(
      parameter, condition, otherItemId, true);

    assertFalse(trigger.update(configurator));

    assertFalse(otherItem.isVisible());
  }

  /**
   * Testando o mtodo {@link SetVisibleEnumerationItemTrigger#equals(Object)}
   * com gatilhos iguais (identificador do item e habilitar/desabilitar iguais).
   */
  public void testEquals1() {
    String parameterName = "Nome do parmetro";
    String itemId = "Identificador do item";
    EnumerationItem item = createMockEnumerationItem(itemId);
    EnumerationParameter parameter = createMockEnumerationParameter(
      parameterName, item);
    boolean isVisible = true;
    Condition condition1 = new SimpleCondition("Nome do parmetro 1",
      "Valor do parmetro 1");
    Condition condition2 = new SimpleCondition("Nome do parmetro 2",
      "Valor do parmetro 2");
    SetVisibleEnumerationItemTrigger trigger1 = new SetVisibleEnumerationItemTrigger(
      parameter, condition1, itemId, isVisible);
    SetVisibleEnumerationItemTrigger trigger2 = new SetVisibleEnumerationItemTrigger(
      parameter, condition2, itemId, isVisible);
    assertTrue(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link SetVisibleEnumerationItemTrigger#equals(Object)}
   * com gatilhos diferentes (identificador do item e habilitar/desabilitar
   * diferentes).
   */
  public void testEquals2() {
    Condition condition = new SimpleCondition("Nome do parmetro",
      "Valor do parmetro");
    String parameterName = "Nome do parmetro";
    String itemId1 = "Identificador do item 1";
    EnumerationItem item1 = createMockEnumerationItem(itemId1);
    String itemId2 = "Identificador do item 2";
    EnumerationItem item2 = createMockEnumerationItem(itemId2);
    EnumerationParameter parameter = createMockEnumerationParameter(
      parameterName, item1, item2);
    boolean isVisible1 = true;
    boolean isVisible2 = false;
    SetVisibleEnumerationItemTrigger trigger1 = new SetVisibleEnumerationItemTrigger(
      parameter, condition, itemId1, isVisible1);
    SetVisibleEnumerationItemTrigger trigger2 = new SetVisibleEnumerationItemTrigger(
      parameter, condition, itemId2, isVisible2);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link SetVisibleEnumerationItemTrigger#equals(Object)}
   * com argumento {@code null}.
   */
  public void testEquals4() {
    String itemId = "Identificador do item";
    EnumerationItem item = createMockEnumerationItem(itemId);
    String parameterName = "Nome do parmetro";
    EnumerationParameter parameter = createMockEnumerationParameter(
      parameterName, item);
    Condition condition = createMockCondition();
    boolean isVisible = true;
    SetVisibleEnumerationItemTrigger trigger = new SetVisibleEnumerationItemTrigger(
      parameter, condition, itemId, isVisible);
    assertFalse(trigger.equals(null));
  }

  /**
   * Testando o mtodo {@link SetVisibleEnumerationItemTrigger#equals(Object)}
   * com argumento um objeto de outra classe.
   */
  public void testEquals5() {
    String itemId = "Identificador do item";
    EnumerationItem item = createMockEnumerationItem(itemId);
    String parameterName = "Nome do parmetro";
    EnumerationParameter parameter = createMockEnumerationParameter(
      parameterName, item);
    Condition condition = createMockCondition();
    boolean isVisible = true;
    SetVisibleEnumerationItemTrigger trigger = new SetVisibleEnumerationItemTrigger(
      parameter, condition, itemId, isVisible);
    assertFalse(trigger.equals("Outra classe"));
  }

  /**
   * Cria um configurador de algorithms qualquer.
   * 
   * @return .
   */
  private SimpleAlgorithmConfigurator createMockAlgorithmConfigurator() {
    AlgorithmInfo algorithmInfo = new AlgorithmInfo("ID Algoritmo",
      "Algoritmo", "DIR Algoritmo", new Hashtable<String, String>());
    AlgorithmVersionId algorithmVersionId = new AlgorithmVersionId(1, 2, 3);
    AlgorithmVersionInfo algorithmVersionInfo = new AlgorithmVersionInfo(
      algorithmInfo, algorithmVersionId,
      new Hashtable<String, List<FileInfo>>(),
      new Hashtable<String, String>());
    return new SimpleAlgorithmConfigurator(algorithmVersionInfo, null,
      ExecutionType.SIMPLE, null, false, "Comando", null, null, true);
  }

  /**
   * Cria uma condio qualquer.
   * 
   * @return .
   */
  private Condition createMockCondition() {
    return new SimpleCondition("Nome do parmetro", "Valor qualquer");
  }

  /**
   * Cria um item de enumerao qualquer.
   * 
   * @return .
   */
  private EnumerationItem createMockEnumerationItem() {
    String itemId = "Identificador do item";
    return createMockEnumerationItem(itemId);
  }

  /**
   * Cria um item de enumerao qualquer.
   * 
   * @param itemId O identificador do item (No aceita {@code null}).
   * 
   * @return .
   */
  private EnumerationItem createMockEnumerationItem(String itemId) {
    return new EnumerationItem(itemId, "Nome do item " + itemId,
      "Valor do item " + itemId, "Descrio do item " + itemId);
  }

  /**
   * Cria um parmetro do tipo enumerao qualquer.
   * 
   * @return .
   */
  private EnumerationParameter createMockEnumerationParameter() {
    String parameterName = "Nome do parmetro";
    EnumerationItem item = createMockEnumerationItem();
    return createMockEnumerationParameter(parameterName, item);
  }

  /**
   * Cria um parmetro do tipo enumerao qualquer.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * @param defaultValue O valor-padro (No aceita {@code null}).
   * @param otherItems Outros itens (No aceita {@code null}).
   * 
   * @return .
   */
  private EnumerationParameter createMockEnumerationParameter(
    String parameterName, EnumerationItem defaultValue,
    EnumerationItem... otherItems) {
    List<EnumerationItem> items = new LinkedList<EnumerationItem>();
    items.add(defaultValue);
    for (EnumerationItem otherItem : otherItems) {
      items.add(otherItem);
    }
    return new EnumerationParameter(parameterName, "Rtulo do parmetro",
      "Descrio do parmetro", defaultValue, null, items, true);
  }

  /**
   * Cria um grupo de parmetros qualquer.
   * 
   * @return .
   */
  private ParameterGroup createMockParameterGroup() {
    return new ParameterGroup("Grupo de parmetros");
  }
}
