package csbase.logic.algorithms.parsers;

import java.io.StringReader;
import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

import tecgraf.javautils.configurationmanager.ConfigurationManager;

import csbase.exception.ParseException;

public final class XmlParserTest extends TestCase {
  static {
    try {
      ConfigurationManager.createInstance("csbase");
    }
    catch (IllegalStateException e) {
      // O ConfigurationManager j foi criado.
    }
  }

  /**
   * Testando o mtodo {@link XmlParser#parseDocument(java.io.Reader)} com um
   * caso de sucesso.
   * 
   * @throws ParseException Se o teste falhar.
   */
  public void testParseDocument1()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz>";
    xml += "<primeiro_filho/>";
    xml += "</raiz>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
  }

  /**
   * Testando o mtodo {@link XmlParser#parseDocument(java.io.Reader)}
   * com o parmetro <code>reader</code><code>null</code>.
   *
   * @throws ParseException Se o teste falhar.
   */
  public void testParseDocument2()
  throws ParseException {
    XmlParser parser = new XmlParser();
    try {
      parser.parseDocument(null);
      fail("Esperada IllegalArgumentException");
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando o mtodo {@link XmlParser#goToRoot()} com um caso de
   * sucesso.
   *
   * @throws ParseException Se o teste falhar
   */
  public void testGoToRoot1()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
  }

  /**
   * Testando o mtodo {@link XmlParser#goToRoot()} sem um documento
   * atual.
   */
  public void testGoToRoot2() {
    XmlParser parser = new XmlParser();
    try {
      parser.goToRoot();
      fail("Esperada IllegalStateException");
    }
    catch (IllegalStateException e) {
    }
  }

  /**
   * Testando o mtodo {@link XmlParser#goToFirstChild()} com um caso de
   * sucesso.
   *
   * @throws ParseException Se o teste falhar.
   */
  public void testGoToFirstChild1()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz>";
    xml += "<primeiro_filho/>";
    xml += "</raiz>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    assertTrue(parser.goToFirstChild());
  }

  /**
   * Testando o mtodo {@link XmlParser#goToFirstChild()} com o elemento
   * atual sem filhos.
   *
   * @throws ParseException Se o teste falhar.
   */
  public void testGoToFirstChild2()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz>";
    xml += "</raiz>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    assertFalse(parser.goToFirstChild());
  }

  /**
   * Testando o mtodo {@link XmlParser#goToFirstChild()} sem elemento
   * atual.
   *
   * @throws ParseException Se o teste falhar.
   */
  public void testGoToFirstChild3()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz>";
    xml += "</raiz>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    try {
      parser.goToFirstChild();
      fail("Esperada IllegalStateException");
    }
    catch (IllegalStateException e) {
    }
  }

  /**
   * Testando o mtodo {@link XmlParser#goToFirstChild()} sem um
   * documento atual.
   */
  public void testGoToFirstChild4() {
    XmlParser parser = new XmlParser();
    try {
      parser.goToFirstChild();
      fail("Esperada IllegalStateException");
    }
    catch (IllegalStateException e) {
    }
  }

  /**
   * Testando o mtodo {@link XmlParser#goToNextSibling()} com um caso de
   * sucesso.
   *
   * @throws ParseException Se o teste falhar.
   */
  public void testGoToNextSibling1()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz>";
    xml += "<primeiro_filho/>";
    xml += "<segundo_filho/>";
    xml += "</raiz>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    parser.goToFirstChild();
    assertTrue(parser.goToNextSibling());
  }

  /**
   * Testando o mtodo {@link XmlParser#goToNextSibling()} com um
   * elemento atual que no possui irmos.
   *
   * @throws ParseException Se o teste falhar.
   */
  public void testGoToNextSibling2()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz>";
    xml += "<primeiro_filho/>";
    xml += "</raiz>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    parser.goToFirstChild();
    assertFalse(parser.goToNextSibling());
  }

  /**
   * Testando o mtodo {@link XmlParser#goToNextSibling()} sem elemento
   * atual.
   *
   * @throws ParseException Se o teste falhar.
   */
  public void testGoToNextSibling3()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz>";
    xml += "<primeiro_filho/>";
    xml += "<segundo_filho/>";
    xml += "</raiz>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    try {
      parser.goToNextSibling();
      fail("Esperada IllegalStateException");
    }
    catch (IllegalStateException e) {
    }
  }

  public void testGoToNextSibling4() {
    XmlParser parser = new XmlParser();
    try {
      parser.goToNextSibling();
      fail("Esperada IllegalStateException");
    }
    catch (IllegalStateException e) {
    }
  }

  public void testEnsureElementName1()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    parser.ensureElementName("raiz");
  }

  public void testEnsureElementName2()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    try {
      parser.ensureElementName("elemento_inexistente");
      fail("Esperada ParseException");
    }
    catch (ParseException e) {
    }
  }

  public void testEnsureElementName3()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    try {
      parser.ensureElementName("elemento_inexistente");
      fail("Esperada IllegalStateException");
    }
    catch (IllegalStateException e) {
    }
  }

  public void testEnsureElementName4()
  throws ParseException {
    XmlParser parser = new XmlParser();
    try {
      parser.ensureElementName("elemento_inexistente");
      fail("Esperada IllegalStateException");
    }
    catch (IllegalStateException e) {
    }
  }

  public void testGetAttributeValueString1()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz atributo='valor'/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    String value = parser.getAttributeValue("atributo");
    assertNotNull(value);
    assertEquals("valor", value);
  }

  public void testGetAttributeValueString2()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz atributo='valor'/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    try {
      parser.getAttributeValue("outro_atributo");
      fail("Esperada ParseException");
    }
    catch (ParseException e) {
    }
  }

  public void testGetAttributeValueString3()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    try {
      parser.getAttributeValue("outro_atributo");
      fail("Esperada ParseException");
    }
    catch (ParseException e) {
    }
  }

  public void testGetAttributeValueString4()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz atributo='valor'/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    try {
      parser.getAttributeValue("outro_atributo");
      fail("Esperada IllegalStateException");
    }
    catch (IllegalStateException e) {
    }
  }

  public void testGetAttributeValueString5()
  throws ParseException {
    XmlParser parser = new XmlParser();
    try {
      parser.getAttributeValue("outro_atributo");
      fail("Esperada IllegalStateException");
    }
    catch (IllegalStateException e) {
    }
  }

  public void testGetAttributeValueStringString1()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz atributo='valor'/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    String defaultValue = "valor-padro";
    String value = parser.getAttributeValue("atributo", defaultValue);
    assertNotNull(value);
    assertEquals("valor", value);
  }

  public void testGetAttributeValueStringString2()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz atributo='valor'/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    String defaultValue = "valor-padro";
    String value = parser.getAttributeValue("outro_atributo", defaultValue);
    assertEquals(defaultValue, value);
  }

  public void testGetAttributeValueStringString3()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    String defaultValue = "valor-padro";
    String value = parser.getAttributeValue("atributo", defaultValue);
    assertEquals(defaultValue, value);
  }

  public void testGetAttributeValueStringString4()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz atributo='valor'/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    String value = parser.getAttributeValue("atributo", null);
    assertEquals("valor", value);
  }

  public void testGetAttributeValueStringString5()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz atributo='valor'/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    String value = parser.getAttributeValue("outro_atributo", null);
    assertNull(value);
  }

  public void testGetAttributeValueStringString6()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    String value = parser.getAttributeValue("atributo", null);
    assertNull(value);
  }

  public void testGetAttributeValueStringString7()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz atributo='valor'/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    try {
      parser.getAttributeValue("outro_atributo", null);
      fail("Esperada IllegalStateException");
    }
    catch (IllegalStateException e) {
    }
  }

  public void testGetAttributeValueStringString8() {
    XmlParser parser = new XmlParser();
    try {
      parser.getAttributeValue("outro_atributo", null);
      fail("Esperada IllegalStateException");
    }
    catch (IllegalStateException e) {
    }
  }

  public void testGetAttributeValueAsBooleanString1()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz atributo='false'/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    boolean value = parser.getAttributeValueAsBoolean("atributo");
    assertFalse(value);
  }

  public void testGetAttributeValueAsBooleanString2()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz atributo='true'/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    boolean value = parser.getAttributeValueAsBoolean("atributo");
    assertTrue(value);
  }

  public void testGetAttributeValueAsBooleanString3()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz atributo='valor no-booleano'/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    try {
      parser.getAttributeValueAsBoolean("atributo");
      fail("Esperada ParseException");
    }
    catch (ParseException e) {
    }
  }

  public void testGetAttributeValueAsBooleanString4()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz atributo='valor'/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    try {
      parser.getAttributeValueAsBoolean("outro_atributo");
      fail("Esperada ParseException");
    }
    catch (ParseException e) {
    }
  }

  public void testGetAttributeValueAsBooleanString5()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    try {
      parser.getAttributeValueAsBoolean("outro_atributo");
      fail("Esperada ParseException");
    }
    catch (ParseException e) {
    }
  }

  public void testGetAttributeValueAsBooleanString6()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz atributo='valor'/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    try {
      parser.getAttributeValueAsBoolean("outro_atributo");
      fail("Esperada IllegalStateException");
    }
    catch (IllegalStateException e) {
    }
  }

  public void testGetAttributeValueAsBooleanString7()
  throws ParseException {
    XmlParser parser = new XmlParser();
    try {
      parser.getAttributeValueAsBoolean("outro_atributo");
      fail("Esperada IllegalStateException");
    }
    catch (IllegalStateException e) {
    }
  }

  public void testGetAttributeValueAsBooleanStringBoolean1()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz atributo='false'/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    boolean value = parser.getAttributeValueAsBoolean("atributo", true);
    assertFalse(value);
  }

  public void testGetAttributeValueAsBooleanStringBoolean2()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz atributo='true'/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    boolean value = parser.getAttributeValueAsBoolean("atributo", false);
    assertTrue(value);
  }

  public void testGetAttributeValueAsBooleanStringBoolean3()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz atributo='false'/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    boolean value = parser.getAttributeValueAsBoolean("outro_atributo", true);
    assertTrue(value);
  }

  public void testGetAttributeValueAsBooleanStringBoolean4()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz atributo='true'/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    boolean value = parser.getAttributeValueAsBoolean("outro_atributo", false);
    assertFalse(value);
  }

  public void testGetAttributeValueAsBooleanStringBoolean5()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    boolean value = parser.getAttributeValueAsBoolean("outro_atributo", true);
    assertTrue(value);
  }

  public void testGetAttributeValueAsBooleanStringBoolean6()
  throws ParseException {
    XmlParser parser = new XmlParser();
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<raiz/>";
    StringReader reader = new StringReader(xml);
    parser.parseDocument(reader);
    parser.goToRoot();
    boolean value = parser.getAttributeValueAsBoolean("outro_atributo", false);
    assertFalse(value);
  }

  public void testGetAttributeValueAsBooleanStringBoolean7()
  throws ParseException {
    XmlParser parser = new XmlParser();
    try {
      parser.getAttributeValueAsBoolean("outro_atributo", true);
      fail("Esperada IllegalStateException");
    }
    catch (IllegalStateException e) {
    }
  }

  public void testReplaceElement()
  throws ParseException {
    String inputElementName = "entrada";
    String outputElementName = "saida";
    Map<String, String> replacementMap = new HashMap<String, String>();
    replacementMap.put(inputElementName, "<"+outputElementName+"/>");
    XmlParser parser = new XmlParser(replacementMap);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<"+inputElementName+"/>";
    parser.parseDocument(new StringReader(xml));
    parser.goToRoot();
    assertEquals(outputElementName, parser.getElementName());
  }

  public void testReplaceElementWithOutputAttributes()
  throws ParseException {
    String inputElementName = "entrada";
    String outputElementName = "saida";
    String attribute = "atributo";
    String attributeValue = "valor";
    Map<String, String> replacementMap = new HashMap<String, String>();
    replacementMap.put(
      inputElementName,
      "<"+outputElementName+" "+attribute+" = '"+attributeValue+"'/>");
    XmlParser parser = new XmlParser(replacementMap);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<"+inputElementName+"/>";
    parser.parseDocument(new StringReader(xml));
    parser.goToRoot();
    assertEquals(outputElementName, parser.getElementName());
    assertEquals(attributeValue, parser.getAttributeValue(attribute));
  }

  public void testReplaceElementWithInputAttributes()
  throws ParseException {
    String inputElementName = "entrada";
    String outputElementName = "saida";
    Map<String, String> replacementMap = new HashMap<String, String>();
    replacementMap.put(inputElementName, "<"+outputElementName+"/>");
    XmlParser parser = new XmlParser(replacementMap);
    String attribute = "atributo";
    String attributeValue = "valor";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<"+inputElementName+" "+attribute+" = '"+attributeValue+"'/>";
    parser.parseDocument(new StringReader(xml));
    parser.goToRoot();
    assertEquals(outputElementName, parser.getElementName());
    assertEquals(attributeValue, parser.getAttributeValue(attribute));
  }

  public void testReplaceElementWithInputAndOutputAttributes()
  throws ParseException {
    String inputElementName = "entrada";
    String outputElementName = "saida";
    String outputAttribute = "atributo_saida";
    String outputAttributeValue = "valor_saida";
    Map<String, String> replacementMap = new HashMap<String, String>();
    replacementMap.put(
      inputElementName,
      "<"+outputElementName+" "+outputAttribute+" = '"+outputAttributeValue+
      "'/>");
    XmlParser parser = new XmlParser(replacementMap);
    String inputAttribute = "atributo_entrada";
    String inputAttributeValue = "valor_entrada";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<"+inputElementName+" "+inputAttribute+" = '"+inputAttributeValue+
    "'/>";
    parser.parseDocument(new StringReader(xml));
    parser.goToRoot();
    assertEquals(outputElementName, parser.getElementName());
    assertEquals(
      outputAttributeValue, parser.getAttributeValue(outputAttribute));
    assertEquals(inputAttributeValue, parser.getAttributeValue(inputAttribute));
  }

  public void testReplaceElementWithInputAndOutputCommonAttributes()
  throws ParseException {
    String inputElementName = "entrada";
    String outputElementName = "saida";
    String attribute = "atributo_saida";
    String outputAttributeValue = "valor_saida";
    Map<String, String> replacementMap = new HashMap<String, String>();
    replacementMap.put(
      inputElementName,
      "<"+outputElementName+" "+attribute+" = '"+outputAttributeValue+"'/>");
    XmlParser parser = new XmlParser(replacementMap);
    String inputAttributeValue = "valor_entrada";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<"+inputElementName+" "+attribute+" = '"+inputAttributeValue+"'/>";
    parser.parseDocument(new StringReader(xml));
    parser.goToRoot();
    assertEquals(outputElementName, parser.getElementName());
    assertEquals(inputAttributeValue, parser.getAttributeValue(attribute));
  }

  public void testReplaceElementWithInputChildNodes()
  throws ParseException {
    String inputElementName = "entrada";
    String outputElementName = "saida";
    Map<String, String> replacementMap = new HashMap<String, String>();
    replacementMap.put(inputElementName, "<"+outputElementName+"/>");
    XmlParser parser = new XmlParser(replacementMap);
    String childElement1 = "filho_1";
    String childElement2 = "filho_2";
    String childElement3 = "filho_3";
    String grandchildElement = "neto";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<"+inputElementName+">\n";
    xml += "<"+childElement1+"/>\n";
    xml += "<"+childElement2+"/>\n";
    xml += "<"+childElement3+">\n";
    xml += "<"+grandchildElement+"/>\n";
    xml += "</"+childElement3+">\n";
    xml += "</"+inputElementName+">";
    parser.parseDocument(new StringReader(xml));
    parser.goToRoot();
    assertEquals(outputElementName, parser.getElementName());
    assertTrue(parser.goToFirstChild());
    assertEquals(childElement1, parser.getElementName());
    assertTrue(parser.goToNextSibling());
    assertEquals(childElement2, parser.getElementName());
    assertTrue(parser.goToNextSibling());
    assertEquals(childElement3, parser.getElementName());
    assertTrue(parser.goToFirstChild());
    assertEquals(grandchildElement, parser.getElementName());
  }

  public void testReplaceElementWithOutputChildNodes()
  throws ParseException {
    String inputElementName = "entrada";
    String outputElementName = "saida";
    String childElement1 = "filho_1";
    String childElement2 = "filho_2";
    String childElement3 = "filho_3";
    String grandchildElement = "neto";
    Map<String, String> replacementMap = new HashMap<String, String>();
    String outputXml = "";
    outputXml += "<"+outputElementName+">\n";
    outputXml += "<"+childElement1+"/>\n";
    outputXml += "<"+childElement2+"/>\n";
    outputXml += "<"+childElement3+">\n";
    outputXml += "<"+grandchildElement+"/>\n";
    outputXml += "</"+childElement3+">\n";
    outputXml += "</"+outputElementName+">";
    replacementMap.put(inputElementName, outputXml);
    XmlParser parser = new XmlParser(replacementMap);
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<"+inputElementName+"/>";
    parser.parseDocument(new StringReader(xml));
    parser.goToRoot();
    assertEquals(outputElementName, parser.getElementName());
    assertTrue(parser.goToFirstChild());
    assertEquals(childElement1, parser.getElementName());
    assertTrue(parser.goToNextSibling());
    assertEquals(childElement2, parser.getElementName());
    assertTrue(parser.goToNextSibling());
    assertEquals(childElement3, parser.getElementName());
    assertTrue(parser.goToFirstChild());
    assertEquals(grandchildElement, parser.getElementName());
  }

  public void testReplaceElementNotRootElement()
  throws ParseException {
    String inputElementName = "entrada";
    String outputElementName = "saida";
    Map<String, String> replacementMap = new HashMap<String, String>();
    replacementMap.put(inputElementName, "<"+outputElementName+"/>");
    XmlParser parser = new XmlParser(replacementMap);
    String rootElementName = "raiz";
    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>";
    xml += "<"+rootElementName+">";
    xml += "<"+inputElementName+"/>";
    xml += "</"+rootElementName+">";
    parser.parseDocument(new StringReader(xml));
    parser.goToRoot();
    assertEquals(rootElementName, parser.getElementName());
    assertTrue(parser.goToFirstChild());
    assertEquals(outputElementName, parser.getElementName());
  }
}
