package csbase.server.services.algorithmservice.niotransfer;

import java.io.IOException;
import java.security.DigestException;
import java.util.Map;

import csbase.remote.niotransfer.RequestTransferInfo;


/**
 * Objeto de teste representando o solicitante da transferncia. Um exemplo 
 * real de um objeto deste tipo seria o servio de algoritmos.
 *
 * @author Tecgraf
 */
public class MockFileTransferrer implements FileTransferrer {

  /** Referncia para o servidor de transferncia de arquivos. */
  private FileServer server;
  
  /**
   * Atribui a este objeto uma referncia para o servidor de transferncias. 
   * 
   * @param server servidor de transferncias de arquivos.
   */
  public void setServer(FileServer server) {
    this.server = server;
  }

  /**
   * Prepara o servidor para a exportao de um arquivo. Em um servio real,
   * o caminho de entrada estaria codificado na viso do cliente. Este mtodo
   * ento se encarregaria de traduzi-lo para um caminho no sistema de 
   * arquivos do servidor. Para efeitos de testes, este mtodo j espera
   * receber um caminho compreensvel para o servidor e simplesmente delega
   * a chamada.
   * 
   * @param sourcePath caminho de origem no servidor para o arquivo a ser
   *        exportado.
   *        
   * @return informaes que permitam que um cliente solicite posteriormente
   *         a exportao, de forma autenticada.
   * 
   * @throws DigestException
   * @throws IOException
   */
  public RequestTransferInfo prepareDownload(String sourcePath) 
  throws DigestException, IOException {
    return server.prepareDownload(sourcePath);
  }
  
  /**
   * Prepara o servidor para a importao de um arquivo. Em um servio real,
   * o caminho de destino estaria codificado na viso do cliente. Este mtodo
   * ento se encarregaria de traduzi-lo para um caminho no sistema de 
   * arquivos do servidor. Para efeitos de testes, este mtodo j espera
   * receber um caminho compreensvel para o servidor e simplesmente delega
   * a chamada.
   * 
   * @param targetPath caminho de destino no servidor para o arquivo a ser
   *        importado.
   * @param notifyArgs mapa de argumentos para que o servio possa reconhecer
   *        posteriormente a transferncia realizada quando for invocada a
   *        callback {@link #channelClosing(Map)} pelo servidor. Essa callback
   *        permite que este servio realize algum eventual procedimento antes
   *        de fechar a conexo com o cliente, o que indicaria para este 
   *        ltimo que a transferncia foi concluda.
   *        
   * @return informaes que permitam que um cliente solicite posteriormente
   *         a exportao, de forma autenticada.
   * 
   * @throws DigestException
   * @throws IOException
   */
  public RequestTransferInfo prepareUpload(String targetPath, Map notifyArgs) 
  throws DigestException, IOException {
    return server.prepareUpload(targetPath, notifyArgs);
  }
  
  /**
   * {@inheritDoc}
   */
  public void channelClosing(Map notifyArgs) {}

  /**
   * {@inheritDoc}
   */
  public void logSevereMessage(String message, Throwable error) {}
  
}
