package csbase.tools;

import Acme.JPM.Encoders.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

import javax.swing.*;

public class BuildVariants {
  private static Variant[] variants =
    new Variant[] {
      new Variant("_db", 255, 210, 210, 245),
      new Variant("_pub_me", 255, 230, 200, 245),
      new Variant("_pub_other", 225, 225, 255, 245),
      new Variant("_cut", 255, 255, 255, 050),
      new Variant("_uc", 205, 205, 205, 245)
    };
  private static float A = 0.6272F;
  private static float B = -0.3403F;
  private static float C = 0.1132F;
  private static float D = 0.6F;

  public static void main(String[] args) {
    if (args.length == 0) {
      while (true) {
        String path = chooseFile();
        if (path == null) {
          break;
        }
        buildVariants(path);
      }
    }
    else {
      for (int i = 0; i < args.length; i++) {
        buildVariants(args[i]);
      }
    }
    System.exit(0);
  }

  private static String chooseFile() {
    JFileChooser dialog = new JFileChooser(".");
    dialog.setFileFilter(
      new javax.swing.filechooser.FileFilter() {
        public boolean accept(File f) {
          String name = f.getName().toLowerCase();
          return name.endsWith(".gif") || name.endsWith(".jpg") ||
          name.endsWith(".jpeg") || name.endsWith(".png") || f.isDirectory();
        }

        public String getDescription() {
          return "Image files";
        }
      });

    int resp = dialog.showOpenDialog(null);
    if (resp == JFileChooser.APPROVE_OPTION) {
      return dialog.getSelectedFile().getAbsolutePath();
    }
    return null;
  }

  private static void buildVariants(String path) {
    ImageData imageData = loadImage(path);
    for (int i = 0; i < variants.length; i++) {
      System.out.println("Gerando " + variants[i].suffix + "...");
      buildVariant(imageData, variants[i]);
    }
  }

  private static ImageData loadImage(String path) {
    System.out.print("Carregando " + path);
    ImageData imageData = new ImageData();
    Observer observer = new Observer(imageData);
    imageData.path = path;
    imageData.img = Toolkit.getDefaultToolkit().getImage(path);
    imageData.width = imageData.img.getWidth(observer);
    imageData.height = imageData.img.getHeight(observer);
    while (!observer.isReady()) {
      try {
        Thread.sleep(1000);
        System.out.print(".");
      }
      catch (InterruptedException e) {
        e.printStackTrace();
      }
    }
    System.out.println();
    return imageData;
  }

  private static void buildVariant(ImageData imageData, Variant variant) {
    Image image = applyFilter(imageData, variant);
    String newPath = getVariantPath(imageData, variant);
    try {
      FileOutputStream fos = new FileOutputStream(newPath);
      BufferedOutputStream bos = new BufferedOutputStream(fos);
      GifEncoder encoder = new GifEncoder(image, bos);
      encoder.encode();
      bos.flush();
      bos.close();
    }
    catch (IOException e) {
      e.printStackTrace();
    }
  }

  private static Image applyFilter(ImageData imageData, Variant v) {
    BufferedImage img =
      getBufferedImage(imageData, BufferedImage.TYPE_INT_ARGB);
    BufferedImage gry =
      getBufferedImage(imageData, BufferedImage.TYPE_BYTE_GRAY);
    for (int y = 0; y < imageData.height; y++) {
      for (int x = 0; x < imageData.width; x++) {
        int argb = img.getRGB(x, y);
        float a = ((argb & 0xFF000000) >>> 24) / 255F;
        if (a == 0) {
          continue;
        }
        float f = (gry.getRGB(x, y) & 0xFF) / 255F;
        f = (A * f * f * f) + (B * f * f) + (C * f) + D;
        argb =
          (op(1, a) << 24) | (op(f, v.r) << 16) | (op(f, v.g) << 8) |
          op(f, v.b);
        img.setRGB(x, y, argb);
      }
    }
    return img;
  }

  private static int op(float f, float c) {
    return (int)(Math.min(1, f * c) * 255);
  }

  private static String getVariantPath(ImageData imageData, Variant variant) {
    int p = imageData.path.lastIndexOf('.');
    if (p == -1) {
      return imageData.path + variant.suffix;
    }
    return imageData.path.substring(0, p) + variant.suffix +
    imageData.path.substring(p);
  }

  private static BufferedImage getBufferedImage(ImageData imageData, int type) {
    BufferedImage img =
      new BufferedImage(imageData.width, imageData.height, type);
    Graphics2D g2d = img.createGraphics();
    g2d.drawImage(imageData.img, 0, 0, null);
    return img;
  }

  private static class Variant {
    String suffix;
    float r;
    float g;
    float b;
    float a;

    Variant(String suffix, int r, int g, int b, int a) {
      this.suffix = suffix;
      this.r = r / 255F;
      this.g = g / 255F;
      this.b = b / 255F;
      this.a = a / 255F;
    }
  }

  private static class ImageData {
    String path;
    Image img;
    int width;
    int height;
  }

  private static class Observer implements ImageObserver {
    private ImageData imageData;
    private boolean ready;

    public boolean imageUpdate(Image i, int f, int x, int y, int w, int h) {
      if ((f & WIDTH) == WIDTH) {
        imageData.width = w;
      }
      if ((f & HEIGHT) == HEIGHT) {
        imageData.height = h;
      }
      if ((f & ALLBITS) == ALLBITS) {
        synchronized (this) {
          ready = true;
        }
        return false;
      }
      return true;
    }

    synchronized boolean isReady() {
      return ready;
    }

    Observer(ImageData imageData) {
      this.imageData = imageData;
      ready = false;
    }
  }
}
