package csbase.tools;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;

/**
 * Gerador de proxies de servio do csbase
 * 
 * @author Jorge Alexandre
 * 
 * @deprecated use {@link NewProxyGenerator}
 */
@Deprecated
public class ProxyGenerator {
  protected StringBuffer buffer = new StringBuffer();
  private Class serviceClass;

  /**
   * Mtodo principal que ser executado quando o gerador for chamado da linha
   * de comando.
   * 
   * @param args args[0]: nome da interface do servio que ser implementada
   *        pelo proxy gerado, args[1]: nome da classe do servio que ser usado
   *        pelo proxy gerado, args[2]: nome da classe do proxy a ser gerado.
   */
  public static void main(String[] args) throws Exception {
    if (args.length != 3) {
      System.out.println("uso:");
      System.out.println("args[0]: nome da interface do servio que ser "
        + "implementada pelo proxy gerado");
      System.out.println("args[1]: nome da classe do servio que ser "
        + "usado pelo proxy gerado");
      System.out.println("args[2]: nome da classe do proxy a ser gerado");
      System.exit(1);
    }
    ProxyGenerator proxygen = new ProxyGenerator(args[0], args[1], args[2]);
    proxygen.flushToFile();
  }

  protected ProxyGenerator(String interfaceName, String serviceClassName,
    String proxyClassName) throws ClassNotFoundException {
    generate(interfaceName, serviceClassName, proxyClassName);
  }

  /**
   * Gera o cdigo fonte do proxy em uma <code>StringBuffer</code>
   * 
   * @param interfaceName nome da interface do servio que ser implementada
   *        pelo proxy gerado
   * @param serviceClassName nome da classe do servio que ser usado pelo proxy
   *        gerado.
   * @param proxyClassName nome da classe do proxy a ser gerado.
   */
  private final void generate(String interfaceName, String serviceClassName,
    String proxyClassName) throws ClassNotFoundException {
    Class interfaceClass = Class.forName(interfaceName);
    serviceClass = Class.forName(serviceClassName);
    buffer.append(serviceClass.getPackage());
    buffer.append(";\n");
    String[] imports = getImports();
    for (int i = 0; (imports != null) && (i < imports.length); i++) {
      buffer.append("import ");
      buffer.append(imports[i]);
      buffer.append(";\n");
    }
    buffer.append("public class ");
    String noPackagesProxyClassName =
      proxyClassName.substring(proxyClassName.lastIndexOf('.') + 1);
    buffer.append(noPackagesProxyClassName);
    buffer.append(" extends ");
    buffer.append(getSuperClasse().getName());
    buffer.append(" implements ");
    buffer.append(interfaceName);
    buffer.append(" {\n");
    Method[] methods = interfaceClass.getDeclaredMethods();
    for (int i = 0; i < methods.length; i++) {
      buffer.append("public ");
      buffer.append(typeToString(methods[i].getReturnType()));
      buffer.append(' ');
      buffer.append(methods[i].getName());
      buffer.append("(");
      Class[] parameters = methods[i].getParameterTypes();
      String[] paremeterNames = new String[parameters.length];
      if (parameters != null && parameters.length > 0) {
        for (int j = 0; j < parameters.length; j++) {
          buffer.append(typeToString(parameters[j]));
          paremeterNames[j] = "param_" + j;
          buffer.append(" " + paremeterNames[j]);
          if (j >= parameters.length - 1) {
            break;
          }
          buffer.append(", ");
        }
      }
      buffer.append(") ");
      Class[] throwses = methods[i].getExceptionTypes();
      if (throwses != null & throwses.length > 0) {
        buffer.append("throws ");
        for (int j = 0; j < throwses.length; j++) {
          buffer.append(throwses[j].getName());
          if (j >= throwses.length - 1) {
            break;
          }
          buffer.append(", ");
        }
      }
      buffer.append(" {\n");
      String[] methodLines = getMethodLines(methods[i], paremeterNames);
      if (methodLines != null & methodLines.length > 0) {
        for (int j = 0; j < methodLines.length; j++) {
          buffer.append(methodLines[j]);
          buffer.append("\n");
        }
      }
      buffer.append("}\n");
    }
    Constructor[] constructors = getSuperClasse().getDeclaredConstructors();
    if (constructors != null & constructors.length > 0) {
      for (int i = 0; i < constructors.length; i++) {
        buffer.append("public ");
        buffer.append(noPackagesProxyClassName);
        buffer.append("(");
        Class[] parameters = constructors[i].getParameterTypes();
        String[] paremeterNames = new String[parameters.length];
        if (parameters != null & parameters.length > 0) {
          for (int j = 0; j < parameters.length; j++) {
            buffer.append(typeToString(parameters[j]));
            paremeterNames[j] = "param_" + j;
            buffer.append(" " + paremeterNames[j]);
            if (j >= parameters.length - 1) {
              break;
            }
            buffer.append(", ");
          }
        }
        buffer.append(") ");
        Class[] exceptions = constructors[i].getExceptionTypes();
        if (exceptions != null & exceptions.length > 0) {
          buffer.append("throws ");
          for (int j = 0; j < exceptions.length; j++) {
            buffer.append(exceptions[j].getName());
            if (j >= exceptions.length - 1) {
              break;
            }
            buffer.append(", ");
          }
        }
        buffer.append(" {\n");
        buffer.append("super(");
        if (parameters != null & parameters.length > 0) {
          for (int j = 0; j < parameters.length; j++) {
            buffer.append(paremeterNames[j]);
            if (j >= parameters.length - 1) {
              break;
            }
            buffer.append(", ");
          }
        }
        buffer.append(");\n");
        String[] constructorLines = getConstructorLines(paremeterNames);
        if (constructorLines != null & constructorLines.length > 0) {
          for (int j = 0; j < constructorLines.length; j++) {
            buffer.append(constructorLines[j]);
            buffer.append("\n");
          }
        }
        buffer.append("}\n");
      }
    }
    buffer.append("}\n");
  }

  /**
   * Descarrega o buffer para um arquivo.
   */
  protected void flushToFile() {
    System.out.println(buffer);
  }

  /**
   * Obtm a classe que representa o servio que ser usado pelo proxy.
   */
  protected Class getServiceClass() {
    return serviceClass;
  }

  private final String typeToString(Class type) {
    String s = "";
    if (!type.isArray()) {
      s = type.getName();
      return s.replace('$', '.');
    }
    while (type != null && type.isArray()) {
      type = type.getComponentType();
      s += "[]";
    }
    s = type.getName() + s;
    return s.replace('$', '.');
  }

  /**
   * Obtm as linhas de cdigo que ficaram no contrutor do proxy.
   * 
   * @param paremeterNames nomes dos parmetros passados pelo contrutor.
   * 
   * @return um array contendo as linhas de cdigo em java(obs: as linhas tm
   *         que ser terminadas com ';')
   */
  protected String[] getConstructorLines(String[] paremeterNames) {
    String[] constructorLines =
      { "UnicastRemoteObject.exportObject(this, 0);" };
    return constructorLines;
  }

  /**
   * Obtm as linhas de cdigo que ficaro nos mtodos.
   * 
   * @param method
   * @param parameterNames .
   * 
   * @return um array contendo as linhas de cdigo em java(obs: as linhas tm
   *         que ser terminadas com ';')
   */
  protected String[] getMethodLines(Method method, String[] parameterNames) {
    String returnLine =
      getServiceClass().getName() + ".getInstance()." + method.getName() + "(";
    for (int i = 0; i < parameterNames.length; i++) {
      returnLine += parameterNames[i];
      if (i >= parameterNames.length - 1) {
        break;
      }
      returnLine += ", ";
    }
    returnLine += ");";
    if (!"void".equals(method.getReturnType().getName()))
      returnLine = "return " + returnLine;
    String[] methodLines = { "Service.setKey(this.key);", returnLine };
    return methodLines;
  }

  /**
   * Obtm a super classe do proxy.
   * 
   * @return a classe que o proxy ir extender.
   */
  protected Class getSuperClasse() {
    return csbase.server.ServiceProxy.class;
  }

  /**
   * Obtm os pacotes a serem importados pelo proxy. Ex: csbase.logic.*,
   * java.rmi.server.*
   * 
   * @return um array com strings que representam os pacotes a serem importados.
   */
  protected String[] getImports() {
    String[] imports =
      { "csbase.server.Service", "java.rmi.server.UnicastRemoteObject", };
    return imports;
  }

}
