/*
 * $Id$
 */

package csbase.util;

import java.io.File;
import java.security.AccessControlException;

/**
 * @author Tecgraf / PUC-Rio
 * 
 * Encapsula mtodos utilitrios do sistema de arquivos.
 */
public class FileSystemUtils {

  /**
   * Nome do diretrio de controle usado pelo sistema de controle de verso. 
   * Ex.: CVS = "CVS", Subversion = ".svn".
   */
  public static final String VERSION_CONTROL_DIR = ".svn";  
  
  // Indica se o sistema permite a leitura de arquivos.
  private static final boolean READ_ALLOWED = _canRead();

  /**
   * Verifica se o sistema permite a leitura de arquivos. <br>
   * Obs.: Esse valor  cacheado.
   * 
   * @return <tt>true</tt> se o sistema permite a leitura de arquivos.
   */
  public static final boolean canRead() {
    return READ_ALLOWED;
  }

  /**
   * Verifica se o sistema permite a leitura de arquivos.
   * 
   * @return <tt>true</tt> se o sistema permite a leitura de arquivos.
   */
  private static boolean _canRead() {
    try {
      new File(".").canRead();
      return true;
    }
    catch (AccessControlException ex) {
      return false;
    }
  }
}
