package csbase.util.parser;

import csbase.util.parser.exception.AutomatonException;
import csbase.util.parser.exception.InvalidStateException;
import csbase.util.parser.iterators.SymbolIterator;

import java.util.List;

/**
 * Representa um analisador lxico (Parser) que atua sobre iteradores de
 * smbolos ({@link csbase.util.parser.iterators.SymbolIterator}).
 */
public abstract class Parser {
  /** O autmato finito que ser utilizado para fazer efetivamente o parser. */
  private FiniteAutomaton automaton;

  /**
   * Cria um parser que utilizar o autmato finito recebido.
   *
   * @param automaton O autmato finito.
   *
   * @throws IllegalArgumentException Caso o autmato finito seja nulo (null).
   */
  protected Parser(FiniteAutomaton automaton) {
    if (automaton == null) {
      throw new IllegalArgumentException(
        "O autmato finito no pode ser nulo (null).");
    }
    this.automaton = automaton;
  }

  /**
   * Gera os tokens a partir dos smbolos recebidos.
   *
   * @param symbolIterator Um iterador para os smbolos.
   *
   * @return Uma {@link List} com os tokens da linha recebida. Cada elemento da
   *         lista  do tipo {@link Token}.
   *
   * @throws InvalidStateException Caso acabem os smbolos do iterador e o
   *         autmato esteja num estado no-final.
   * @throws AutomatonException Caso ocorra algum erro durante as iteraes.
   */
  protected final List parse(SymbolIterator symbolIterator)
  throws InvalidStateException, AutomatonException {
    return this.automaton.start(symbolIterator);
  }
}
