package csbase.util.parser.actions;

import csbase.util.parser.Session;
import csbase.util.parser.symbols.Symbol;

/**
 *  a ao responsvel por adicionar o smbolo atual  sesso.
 */
public final class AppendAction implements TransitionAction {
  /** A instncia nica. */
  private static AppendAction instance;

  /**
   * Construtor definido para impedir a criao de objetos desta classe
   * externamente.
   */
  private AppendAction() {}

  /**
   * Obtm a instncia nica.
   *
   * @return A instncia nica.
   */
  public static AppendAction getInstance() {
    if (instance == null) {
      instance = new AppendAction();
    }
    return instance;
  }

  public void execute(Symbol symbol, Session session) {
    if (symbol == null) {
      throw new IllegalArgumentException(
        "O smbolo que est sendo utilizado pelo autmato no pode ser nulo (null).");
    }
    if (session == null) {
      throw new IllegalArgumentException(
        "A sesso de execuo no pode ser nula (null).");
    }
    session.appendSymbol(symbol);
  }
}
