-- $Id: csalg.lua 37080 2006-08-02 22:30:41Z rodrigoh $

require "oo"
require "global"
require "csglobal"

-- --------------------------------------------------
-- classe Algorithm
--
-- executa operaes sobre algoritmos
-- assume que um dos ns da rvore de descrio do projeto foi marcado
-- como "repositrio de algoritmos"; neste caso, todos os seus
-- subdiretrios (com exceo do CVS) so considerados algoritmos

Algorithm = Module:new {
    params = "V:lLbCn",
    provides = MODULE.ALGORITHM,
    info = "operacoes sobre algoritmos",
    actionInfo = "algoritmos",
    requires = {
        make = "GNU",
        java = "*",
    },
}

cs_registerModule(Algorithm)

function Algorithm:check ()
    return false
end

-- retorna os nomes dos algoritmos, os diretrios e as verses
-- associadas aos mesmos
local function _listAlgsDirs (self)
    local masterDir = ConfigFile:getAlgsMasterDir()
    return coroutine.wrap(function ()
        if self.getopt:nargs() < 2 then
            -- nenhum algoritmo foi especificado, retorna todos
            for name, path in ConfigFile:listAlgs() do
                coroutine.yield(name, masterDir.."/"..name,
                    ConfigFile:getAlgVersion(name))
            end
        else
            -- retorna o diretrio associado a cada um dos algoritmos
            for _, name in self.getopt:argList(2) do
                coroutine.yield(name, masterDir.."/"..name,
                    ConfigFile:getAlgVersion(name))
            end
        end
    end)
end

function Algorithm:execute (args, params)
    params = string.concat(params, self.params)
    self.getopt:parse(args, params)

    -- lista algoritmos
    if self.getopt:opt("L") then
        cs_log:info("algoritmos disponiveis:\n")
        for alg, _, versions in _listAlgsDirs(Algorithm) do
            print(string.format("%-15s----------", alg))
            for _, v in ipairs(versions) do
                print(string.format("%25s", v))
            end
        end
        print()
        return
    end

    local simulate = self.getopt:opt("n")
    local localOnly = self.getopt:opt("l")
    local verbose = self.getopt:opt("v")
    local cmdMask
    local version = self.getopt:opt("V")

    local actionInfo

    if self.getopt:opt("C") then
        -- clean
        fileMask = "find %s -name \*.class -type f -print"
        cmdMask = "rm -f %s"
        actionInfo = "'limpando' algoritmo >>> "
    elseif self.getopt:opt("b") then
        -- build
        local opts = string.format("%s%s", simulate and "-n" or "",
            verbose and " -v" or "")
        if not localOnly then
            local act = Action:new(MODULE.UPDATE)
            act:execute(string.format("%s %s %s", MODULE.UPDATE,
                opts, ConfigFile:getAlgName()))
        end
        fileMask = "find %s -name \\*.java -type f -print"
        cmdMask = "javac -classpath .:..:../csbase/libs/javautils/javautils.jar %s"
        actionInfo = "gerando algoritmo >>> "
    end

    -- executa efetivamente a ao
    if cmdMask then
        if simulate then
            cs_log:warn("algoritmos: modo simulacao (operacoes nao serao efetuadas)")
        end
        for name, dir in _listAlgsDirs(Algorithm) do
            local cmd
            if version then
                abort(ConfigFile:isVersion(name, version), "versao invalida")
                cmd = string.format(fileMask, dir.."/versions/"..version)
            else
                cmd = string.format(fileMask, dir)
            end
            cs_log:info(actionInfo..name)
            local files = io.popen(cmd)
            for file in files:lines() do
                local v = version or string.gsub(file, ".*/versions/(.-)/.*", "%1")
                cmd = string.format(cmdMask, file)
                cs_log:info("    versao: "..v)
                if not simulate then
                    cs_log:debug(cmd)
                    local status = os.execute(cmd)
                    abort(tonumber(status) == 0, "nao foi possivel executar a operacao (status = "..status..")")
                end
            end
        end
    end
end

function Algorithm:help ()
    print([[

Algoritmos (alg):
----------
[a1 a2...]  : algoritmos a serem manipulados [*]
-V          : versao a ser manipulada [**]
-l          : gera versao local (nao faz update)
-L          : lista algoritmos disponiveis
-b          : gera algoritmos
-C          : remove binarios ("make clean")
-n          : apenas simula execucao

[*]  caso nao seja especificado nenhum algoritmo, atua sobre todos
[**] caso nao seja especificada nenhuma versao, atua sobre todas
     apenas uma versao pode ser especificada

OBS.: por default os fontes de TODOS os algoritmos sao atualizados (cvs
update) antes da compilacao
]])
end
