-- $Id: csapp.lua 34801 2004-11-26 15:50:35Z costa $

-- Mdulo responsvel pela execuo de "aplicaes" (runclient, runserver
-- etc.)

require "csglobal"

-- --------------------------------------------------
-- classe Runnable

local Runnable = OO:new {
    help = "",
    nargs = 0,
}

function Runnable:new (data)
    return OO.new(self, table.merge(Runnable, data or {}))
end

-- executa o comando em uma janela dedicada, verificando se a quantidade
-- mnima de argumentos est disponvel
function Runnable:run (name, args, nargs)
    abort(nargs >= self.nargs, self.cmd.." exige ao menos "..self.nargs.." parametro(s)")
    local cmd = string.format("%s %s %s", self.cmd, self.params, args)
    local windowTitle = ConfigFile.projName.." ("..name..")"
    cs_log:debug(cmd)
    local path = self.path or "./"
    os.execute("xterm -T '"..windowTitle.."' -n '"..windowTitle.."' -e "..path..cmd.." &")
end

-- --------------------------------------------------
-- classe Run

local Run = Module:new {
    actionInfo = "run",
    info = "execucao de aplicacoes/componentes",
    provides = MODULE.RUN,
    params = "",
    requires = {},
    apps = {
        client = Runnable:new {
            cmd = "runclient",
            params = "-p",
            help = "servidor:porta",
            nargs = 1,
        },
        server = Runnable:new {
            cmd = "runserver",
            params = "-p",
        },
        vgeclient = Runnable:new {
            cmd = "runvgeclient",
            params = "-p",
            help = "servidor:porta",
            nargs = 1,
        },
    },
}

cs_registerModule(Run)

function Run:check ()
    return false
end

function Run:help ()
    print([[
    
Run:
----------  
app [p1 p2 ...] : aplicacao/componente a ser executado

Aplicacoes disponiveis e seus respectivos parametros:
]]) 
    for appName, app in pairs(self.apps) do
        print(string.format("%-10s%s", appName, app.help or ""))
    end
    print()
end

function Run:execute (args, params)
    params = string.concat(params, self.params)
    self.getopt:parse(args, params)
    local nargs = self.getopt:nargs()
    abort(nargs > 1, "deve ser fornecido o nome da aplicacao a ser executada")
    local app = self.getopt:arg(2)
    abort(self.apps[app], "aplicacao nao suportada")
    cs_log:info("executando '"..app.."'")
    self.apps[app]:run(app, self.getopt:getArgsAsString(3))
end
