-- $Id: csbuild.lua 56785 2006-09-20 18:50:50Z costa $

require "oo"
require "global"
require "csglobal"

-- --------------------------------------------------
-- classe Make (local)

local Make = Module:new {
    requires = {
        make = "GNU",
    },
    cmdparams = ParamList:new(),
}

function Make:new (data)
    return OO.new(self, table.merge(Make, data or {}))
end

function Make:execute (args, params)
    if self.getopt:opt("n") then
        self.cmdparams:set("-n")
        cs_log:warn(self.actionInfo..": modo simulacao (operacoes nao serao efetuadas)")
    end
    local act
    if not self.localOnly then
        act = Action:new(MODULE.UPDATE)
    end
    local opts = self.getopt:getOptsAsString()
    -- no queremos repassar um eventual parmetro -a para o update,
    -- todo o controle da iterao ser feito aqui
    opts = self.getopt:remove(opts, "a")

    local didSomething
    local cmdparams = self.cmdparams:toString()

    -- arg(1)  a operao ('build' ou 'clean')
    local allModules = (self.getopt:nargs() == 1)
    for module, dir, level in self:moduleIterator(allModules, 2, 1) do
        if not cs_buildIgnore[module] then
            posix.putenv("CLEAN_MODULE_DIR="..ConfigFile:getModuleDir(module))
            if not self.localOnly or
                (allModules and level == 1 and not self.localOnly) then
                act:execute(MODULE.UPDATE.." "..opts.." "..module, params)
            end
            print()
            if module ~= "" then
                cs_log:info(self.executeMsg..module)
            end
            local cmd = string.format(self.cmdMask, dir, cmdparams)
            cs_log:debug(cmd)
            local status = os.execute(cmd)
            abort(tonumber(status) == 0, "nao foi possivel executar a operacao (status = "..status..")")
            didSomething = true
        else
            cs_log:warn("ignorando modulo >>> "..module)
        end
    end
end

-- --------------------------------------------------
-- classe Build (exportada)

local Build = Make:new {
    actionInfo = "build",
    info = "compila o codigo",
    params = "ln",
    provides = MODULE.BUILD,
    requires = {
        make = "GNU",
        java = "1.4.1",
        javac = "*",
    },
    cmdMask = "(cd %s; gmake %s)",
    executeMsg = "gerando modulo >>> ",
}

-- registra o mdulo
cs_registerModule(Build)

function Build:check ()
    return false
end

function Build:execute (args, params)
    params = string.concat(params, self.params)
    self.getopt:parse(args, params)
    self.localOnly = self.getopt:opt("l")
    Make.execute(self, args, params)
end

function Build:help ()
    print([[

Build:
----------
[m1 m2 ...] : modulos afetados pela operacao [*]
-l          : gera versao local (nao faz update)
-n          : apenas simula execucao

[*] caso nao sejam especificados modulos, atua sobre TODOS os modulos

OBS.: por default os fontes sao atualizados (cvs update) antes da
      compilacao
]])
end


-- --------------------------------------------------
-- classe Clean (exportada)

Clean = Make:new {
    actionInfo = "clean",
    info = "remove binarios",
    params = "n",
    provides = MODULE.CLEAN,
    requires = {
        make = "GNU",
    },
    cmdparams = ParamList:new(),
    cmdMask = "(cd %s; gmake clean %s)",
    executeMsg = '"limpando" modulo >>> ',
}

-- registra o mdulo
cs_registerModule(Clean)

function Clean:check ()
    return false
end

function Clean:execute (args, params)
    params = string.concat(params, self.params)
    self.getopt:parse(args, params)
    self.localOnly = true
    Make.execute(self, args, params)
end

function Clean:help ()
    print([[

Clean:
----------
[m1 m2 ...] : modulos afetados pela operacao [*]
-n          : apenas simula execucao

[*] caso nao sejam especificados modulos, atua sobre TODOS os modulos
]])
end

-- --------------------------------------------------
-- classe Docs (exportada)

Docs = Make:new {
    actionInfo = "doc",
    info = "gera documentacao",
    params = "n",
    provides = MODULE.DOCS,
    requires = {
        make = "GNU",
    },
    cmdparams = ParamList:new(),
    cmdMask = "(cd %s; gmake doc %s)",
    executeMsg = 'gerando documentacao >>> ',
}

-- registra o mdulo
cs_registerModule(Docs)

function Docs:check ()
    return false
end

function Docs:execute (args, params)
    params = string.concat(params, self.params)
    self.getopt:parse(args, params)
    self.localOnly = true
    Make.execute(self, args, params)
end

function Docs:help ()
    print([[

Doc:
----------
[m1 m2 ...] : modulos afetados pela operacao [*]
-n          : apenas simula execucao

[*] caso nao sejam especificados modulos, atua sobre TODOS os modulos
]])
end

-- --------------------------------------------------
-- classe SingleFile
--
-- compila apenas um nico arquivo

SingleFile = Module:new {
    provides = MODULE.SINGLEFILE,
    info = "compila um unico arquivo",
    actionInfo = "arquivo",
    params = "nm:",
    requires = {
        make = "GNU",
        java = "1.5",
        javac = "*",
    },
    cmdMask = "(cd %s; gmake %s %s)",
}

cs_registerModule(SingleFile)

function SingleFile:check ()
    return false
end

function SingleFile:help ()
    print([[

Class:
----------  
arq [arq2 ...]  : arquivos .java a serem compilados [*]
-n              : apenas simula execucao
-m modulo       : compila o arquivo no escopo do modulo, usando o makefile
                  do mesmo. [**]

[*]  pode ser fornecido paths relativos para os arquivos
[**] os paths para os arquivos devem ser relativos a raiz do modulo

OBS.: 
  * os arquivos devem ter extensao .java ou .class
  * caso o parametro -m nao seja usado, o makefile _corrente_ sera usado,
    mesmo que o path remeta a outro modulo
]])         
end

function SingleFile:execute (args, params)
    params = string.concat(params, self.params)
    self.getopt:parse(args, params)

    -- pegamos o dir do mdulo, caso este tenha sido definido, ou o dir
    -- corrente
    local module = self.getopt:opt("m")
    local moduleDir
    if module then
        moduleDir = ConfigFile:getModuleDir(module)
    else
        moduleDir = "."
    end

    -- arg[1]  o comando ('class')
    local files = self.getopt:getArgsAsString(2)
    files = string.gsub(files, "%.java", "%.class")
    abort(files ~= "", "nao foi fornecida a classe a ser compilada")

    local simulate = self.getopt:opt("n")
    local cmdparams = simulate and "-n " or ""
    local cmd = string.format(self.cmdMask, moduleDir, cmdparams, files)
    cs_log:info("compilando >>> "..files)
    cs_log:debug(cmd)
    local status = os.execute(cmd)

    abort(tonumber(status) == 0, "nao foi possivel executar a operacao (status = "..status..")")
end

-- --------------------------------------------------
-- classe Rebuild
--
-- equivale a clean + build

Rebuild = Module:new {
    provides = MODULE.REBUILD,
    info = "clean + build",
    actionInfo = "executando rebuild",
    params = "ln",
    requires = {
        make = "GNU",
        java = "1.4.1",
        javac = "*",
    },
}

cs_registerModule(Rebuild)

function Rebuild:check ()
    return false
end

function Rebuild:help ()
    print([[

Rebuild:
----------  
[m1 m2 ...] : modulos afetados pela operacao [*]
-l          : gera versao local (nao faz update)
-n          : apenas simula execucao

[*] caso nao sejam especificados modulos, atua sobre TODOS os modulos
]])         
end

function Rebuild:execute (args, params)
    params = string.concat(params, self.params)
    self.getopt:parse(args, params)
    local originalOpts = self.getopt:getOptsAsString()
    -- no queremos pegar o primeiro argumento, pois este  o prprio
    -- comando
    local originalArgs = self.getopt:getArgsAsString(2)
    local cleanAction = Action:new(MODULE.CLEAN)
    -- clean no aceita o parmetro -l, portanto estamos interessados
    -- apenas no -n
    cleanAction:execute(MODULE.CLEAN..
      " "..self.getopt:leaveOnly(originalOpts, "n").." "..originalArgs)
    -- para o build todos os parmetros originais devem ser considerados
    local buildAction = Action:new(MODULE.BUILD)
    buildAction:execute(MODULE.BUILD.." "..originalOpts.." "..originalArgs)
end

-- --------------------------------------------------
-- classe Test
--
-- executa "make run-tests"

Test = Module:new {
    provides = MODULE.TEST,
    info = "executa testes automatizados",
    actionInfo = "unit tests",
    params = "ln",
    requires = {
        make = "GNU",
        java = "1.4.1",
        javac = "*",
    },
    cmdparams = ParamList:new(),
    cmdMask = "(cd %s; gmake run-tests %s)",
    executeMsg = 'testando modulo >>> ',
}

cs_registerModule(Test)

function Test:check ()
    return false
end

function Test:help ()
    print([[

Test:
----------  
[m1 m2 ...] : modulos afetados pela operacao [*]
-l          : testa versao local (nao faz update)
-n          : apenas simula execucao

[*] caso nao sejam especificados modulos, atua sobre TODOS os modulos
]])         
end

function Test:execute (args, params)
    params = string.concat(params, self.params)
    self.getopt:parse(args, params)
    self.localOnly = self.getopt:opt("l")
    Make.execute(self, args, params)
end


