-- $Id: csglobal.lua 37080 2006-08-02 22:30:41Z rodrigoh $

require "oo"
require "log"
require "getopt"

-- --------------------------------------------------
-- variveis globais

-- mdulos a serem ignorados durante builds. Esta tabela  a inversa de
-- BUILD_IGNORE (definida nos arquivos de configurao), e  preenchida
-- pelo mdulo ConfigFile
cs_buildIgnore = {}

-- arquivos a serem removidos antes de um deployment (para que o 'update
-- -r' efetivamente altere a tag 'Name' do CVS)
cs_versionFiles = {}

-- --------------------------------------------------
-- funcoes globais

function cs_isVerboseModeOn ()
    return (cs_log:getLevel() == Log.level.DEBUG)
end

-- --------------------------------------------------

-- implementao default para mtodos obrigatrios
local function missingMethod (method)
    cs_log:fatal(string.format("metodo '%s' nao implementado", method))
    os.exit(1)
end

-- --------------------------------------------------
-- classe Module
-- superclasse para todos os mdulos
Module = OO:new()

function Module:new (tab)
    local data
    data = table.merge({
            check = function () missingMethod("check") end,
            execute = function () missingMethod("execute") end,
            help = function () missingMethod("help") end,
            getopt = GetOpt:new(),
        },
        tab)
    data.actionInfo = data.actionInfo or "N/A"
    return OO.new(self, data)
end

-- processa os parmetros usando o interpretador do prprio mdulo
function Module:parseArgs (args, params)
    self.getopt:parse(args, string.concat(params, self.params))
end

-- consulta o interpretador do mdulo a respeito de uma opo
function Module:opt (opt)
    return self.getopt:opt(opt)
end

-- "iterador universal": retorna um iterador para [a] percorrer todos os
-- mdulos ou [b] percorrer os mdulos a partir do argumento associado
-- ao ndice 'start'. A escolha por [a] ou [b]  determinada pelo
-- parmetro 'allModules'. No modo [a], apenas os nveis maiores ou
-- iguais a 'level' sero percorridos (o nvel  retornado como um dos
-- resultados); se level=nil, retorna todos os mdulos
function Module:moduleIterator (allModules, start, level)
    if allModules then
        if level then
            return coroutine.wrap(function ()
                for _, node, dir, module, level in ConfigFile:nodesAtLevel(level, true) do
                    if not ConfigFile:isAlg(module) then
                        coroutine.yield(module, dir..module, level)
                    end
                end
            end)
        else
            return coroutine.wrap(function ()
                for _, node, dir, module in ConfigFile:nodes() do
                    if not ConfigFile:isAlg(module) then
                        coroutine.yield(module, dir..module)
                    end
                end
            end)
        end
    else
        return coroutine.wrap(function ()
            local didSomething = false
            for _, module in self.getopt:argList(start) do
                coroutine.yield(module, ConfigFile:getModuleDir(module))
                didSomething = true
            end
            -- usurio no pediu por todos os mdulos, e nenhum foi
            -- especificado na linha de comando -- retorna nome de
            -- mdulo "vazio" e o dir corrente
            if not didSomething then
                coroutine.yield("", "./")
            end
        end)
    end
end

-- --------------------------------------------------
-- classe ParamList
-- gerencia listas de parmetros (pares (param, valor))

ParamList = OO:new()

function ParamList:new ()
    return OO.new(self, { plist = {}, pcache = {}, })
end

-- define/altera um parmetro. Caso o valor seja nil, o valor 
-- adicionado ao fim da lista de valores associados a ndices numricos
function ParamList:set(param, val)
    if val then
        self.plist[param] = val
    else
        -- o cache  usado apenas para que os parmetros associados a
        -- ndices numricos no se repitam
        if not self.pcache[param] then
            table.insert(self.plist, param)
            self.pcache[param] = true
        end
    end
end

-- retorna todos os parmetros sob a forma de uma string, separados por
-- espaos
function ParamList:toString ()
    local result = ""
    for p, v in pairs(self.plist) do
        if not tonumber(p) then
            result = string.format("%s %s %s", result, p, v)
        end
    end
    return result .. " " .. table.concat(self.plist, " ")
end

-- zera a lista de parmetros
function ParamList:reset ()
    self.plist = {}
    self.pcache = {}
end

-- armazena string representando o estado atual da lista de parmetros
function ParamList:save ()
    self.saved = table.merge(self.plist, {})
end

-- recupera estado previamente salvo
function ParamList:restore ()
    if self.saved then
        self.plist = table.merge(self.saved, {})
    end
end

