-- $Id: cspack.lua 73242 2008-02-28 21:38:31Z costa $

require "csglobal"
require "cssvn"

local Pack = Module:new {
    actionInfo = "pack",
    info = "geracao de um pacote de instalacao",
    params = "unm:e:",
    provides = MODULE.PACK,
    requires = {
        gtar = "*",
        gzip = "*",
        gunzip = "*",
    },
}

cs_registerModule(Pack)

local warning = [[
este comando apenas agrupa arquivos e diretorios.
        Certifique-se de que a versao correta foi devidamente gerada, e
        que o tag correspondente a mesma foi devidamente criado.]]


function Pack:check ()
    return false
end

function Pack:help ()
    print([[
    
Pack:
----------  

Gera um executaval (.bin) de instalacao.

Parametros:

-u          : faz update antes de executar os testes
-m path     : especifica arquivo com mensagem a ser exibida apos a instalacao
-e path     : especifica arquivo com definicoes extras para o script de deployment
-v          : exibe nomes dos arquivos durante processamento
-n          : apenas simula execucao

ATENCAO: ]] .. warning) 
end

function Pack:execute(args, params)
    params = string.concat(params, self.params)
    self.getopt:parse(args, params)

    local PRJ_PREFIX = ConfigFile:getVar("PRJ_PREFIX")
    local excludeList = cs_exedir.."/csmake_deployment_excludes"
    local tarExcludes = string.format("-X %s --exclude='%s*.tar.gz'", excludeList, PRJ_PREFIX)

    local version = SVN:getCurrentTagVersion()
    if not version then
        cs_log:warn("o diretorio corrente nao esta associado a uma tag")
        if not confirm("Continua? (s/n)", "s") then
            print()
            cs_log:fatal("operacao abortada pelo usuario")
            os.exit(1)
        end
        version = "XXX"
        print()
    end

    local originalOpts = self.getopt:getOptsAsString()
    -- aborta caso no exista o link simblico para o tomcat no dir pai
    abort(isLink("../tomcat") or isDir("../tomcat"), "nao existe link/dir 'tomcat' no diretorio ../")

    local originalOpts = self.getopt:leaveOnly(self.getopt:getOptsAsString(), "vn")
    local action = Action:new(MODULE.TEST)
    if not self.getopt:opt("u") then
        originalOpts = originalOpts .. " -l "
        cs_log:warn([[a versao corrente do projeto sera usada. Certifique-se de que voce
        esta usando a versao correta dos fontes, e que um build foi gerado.]])
    else
        cs_log:warn("os fontes do CSBase serao atualizados")
    end
    if not confirm("Continua? (s/n)", "s") then
      print()
      cs_log:fatal("operacao abortada pelo usuario")
      os.exit(1)
    end
    print()
    -- executa os testes
    action:execute(MODULE.TEST.." "..originalOpts)
    -- gera o tar
    local verbose = self.getopt:opt("v")
    -- ATENO: subimos um nvel no path
    pushd("..")
    local tarName = string.format("%s-%s-%s.tar.gz", PRJ_PREFIX,
        version, os.date("%Y_%m_%d"))
    print()
    cs_log:info("gerando arquivo "..tarName.." (pode demorar)")
    cs_log:warn(warning)
    local tarParams = "-czp"
    if not verbose then
        cs_log:info("DICA: para acompanhar o processo, use o parametro '-v'")
    else
        tarParams = tarParams.."v"
    end
    -- geramos o tar
    local tarCmd = string.format("gtar %s %s -f %s .", tarParams, tarExcludes, tarName)
    os.execute(tarCmd)

    -- parmetros do tar2exe:
    local tar2exeParams = ""
    local msgFilePath = self.getopt:opt("m")
    if msgFilePath then
        tar2exeParams = string.format(" %s -m %s", tar2exeParams, msgFilePath)
    end
    local extrasPath = self.getopt:opt("e")
    if extrasPath then
        tar2exeParams = string.format(" %s -e %s", tar2exeParams, extrasPath)
    end

    -- agora geramos o .bin e removemos o tar
    local tar2exe = "csbase/bin/tar2exe"
    os.execute(string.format("%s %s %s", tar2exe, tar2exeParams, tarName))
    print()
    os.remove(tarName)
    cs_log:info("o arquivo "..tarName.." foi removido.\n        Para recupera-lo execute o .bin com o parametro '-t'")
    -- retornamos ao diretrio de origem
    popd()
end

-- --------------------------------------------------
-- Classe Patch

local Patch = Module:new {
    actionInfo = "patch",
    info = "geracao de patches para deployment",
    params = "",
    provides = MODULE.PATCH,
    requires = {
        gtar = "*",
        gzip = "*",
        gunzip = "*",
    },
    -- campos especficos
    flist = "",
}

cs_registerModule(Patch)

function Patch:check ()
    return false
end

function Patch:help ()
    print([[
    
Pack:
----------  
(sem parametros)

OBS.: a identificacao do patch e os paths para os arquivos serao obtidos
      interativamente
]]) 
end

-- l a lista de nomes de arquivos/dirs de stdin, um nome por linha.
-- Encerra a execuo quando uma linha em branco  fornecida
function Patch:_readNameList ()
    print()
    cs_log:info([[Entre com os nomes dos arquivos, um por linha.
Digite apenas ENTER para encerrar a lista.

IMPORTANTE: os paths devem ser relativos ao diretorio-raiz do projeto
            (i.e. init/... src/... sgad/... etc.)
      ]])
    local fname
    while true do
        fname = io.read()
        if fname == "" then break end
        self.flist = self.flist .. fname .. "\n"
    end
    -- remove espaos em branco redudantes no final da linha (isto
    -- inclui linhas em branco)
    self.flist = string.gsub(self.flist, "%s+$", "")
end

-- verifica se os nomes fornecidos correspondem a arquivos/dirs que
-- existem. Aborta execuo do programa caso um dos testes falhe.
-- OBS.: assume que o dir corrente  o mesmo a partir do qual foram
-- especificados os arquivos
function Patch:_checkFiles ()
    cs_log:info("verificando arquivos")
    for fname in string.gfind(self.flist, "%S+") do
        if not fileExists(fname) then
            cs_log:fatal(fname.." nao existe")
            os.exit(1)
        end
    end
end

-- armazena a lista de arquivos/dirs em um arquivo temporrio. Retorna o
-- nome do arquivo temporrio
-- OBS.: assume que o arquivo ser removido posteriormente
function Patch:_storeList ()
    local tmpName = "/tmp/patch"..os.date("%Y%m%d_%H%M%S")..".tmp"
    local tmpFile = io.open(tmpName, "w")
    abort(tmpFile, "erro criando arquivo temporario")
    tmpFile:write(self.flist)
    tmpFile:close()
    return tmpName
end

function Patch:execute (args, params)
    local version = SVN:getCurrentTagVersion()
    if not version then
        cs_log:warn("o diretorio corrente nao esta associado a uma tag")
        if not confirm("Continua? (s/n)", "s") then
            print()
            cs_log:fatal("operacao abortada pelo usuario")
            os.exit(1)
        end
        version = "XXX"
        print()
    end

    local patchNum = ask("Entre com o numero sequencial do patch", 1)
    local PRJ_PREFIX = ConfigFile:getVar("PRJ_PREFIX")
    pushd("..")
    local tarName = string.format("%s-%s-patch_%02d-%s.tar.gz", PRJ_PREFIX,
      version, patchNum, os.date("%Y_%m_%d"))
    cs_log:info("gerando patch "..tarName)
    local proceed = confirm("Confirma? (s/n)", "n")
    if proceed then
        self:_readNameList()
        self:_checkFiles()
        local tmpName = self:_storeList()
        local tarCmd = string.format("gtar -czp -T %s -f %s", tmpName, tarName)
        os.execute(tarCmd)
        os.remove(tmpName)
    else
        cs_log:warn("operacao interrompida pelo usuario")
    end
    popd()
end
