-- $Id: cswebdeploy.lua 73243 2008-02-28 21:38:49Z costa $

require "csglobal"
require "cssvn"

local WebDeploy = Module:new {
    actionInfo = "webdeploy",
    info = "instalacao no servidor de aplicacoes web",
    params = "nbq",
    provides = MODULE.WEBDEPLOY,
    requires = {
        jar = "*",
        gtar = "*",
        gzip = "*",
        gunzip = "*",
    },
    -- se removeVersionFiles = true, arquivos definidos em VERSION_FILES
    -- no csmake.cfg sero removidos antes do update
    removeVersionFiles = false,
}

cs_registerModule(WebDeploy)

function WebDeploy:check ()
    return false
end

function WebDeploy:help ()
    print([[
    
WebDeploy:
----------  

Executado sem parametros, este comando apenas executa o script 'install'

Parametros:

-q          : 'quiet mode': assume defaults
-b          : faz update + build
-n          : apenas simula execucao
]]) 
end

function WebDeploy:execute(args, params)
    params = string.concat(params, self.params)
    self.getopt:parse(args, params)

    -- estamos interessados apenas em saber se o usurio especificou -v
    -- -n ou -q
    local originalOpts = self.getopt:leaveOnly(self.getopt:getOptsAsString(), "vnq")
    local verbose = cs_isVerboseModeOn()
    local simulate = self.getopt:opt("n")

    local version = SVN:getCurrentTagVersion()
    if not version then
      cs_log:warn("o diretorio corrente nao esta associado a uma tag")
      if not confirm("Continua? (s/n)", "s") then
        cs_log:fatal("operacao abortada pelo usuario")
        os.exit(1)
      end
    end

    if simulate then
        cs_log:warn(self.actionInfo..": MODO SIMULACAO (** OPERACOES NAO SERAO EFETUADAS **)")
    end

    if self.getopt:opt("b") then
        -- build (local)
        cs_log:info("gerando modulos")
        act = Action:new(MODULE.BUILD)
        act:execute(MODULE.BUILD..originalOpts)

        -- gera algoritmos
        if ConfigFile:hasAlgs() then
            cs_log:info("gerando algoritmos")
            act = Action:new(MODULE.ALGORITHM)
            act:execute(MODULE.ALGORITHM.." -l -b "..originalOpts)
        end
    end

    -- executa script 'install'
    print()
    cs_log:warn("e recomendavel interromper servidores em execucao\n")
    ask("Pressione ENTER para prosseguir com a instalacao")
    local installDir = getAbsolutePath(ConfigFile:getModuleDir("install"))
    cs_log:debug("executando script 'install'")
    if not simulate then
        pushd(installDir)
        os.execute("./install")
        popd()
    end
end

-- setter para removeVersionFiles
function WebDeploy:updateVersionFiles ()
    self.removeVersionFiles = true
end
