/**
 * @file
 *
 */

import sgaidl.*;

/**
 * A classe <code>SGABridgeCommand</code> implementa a ponte
 * para um commando SGA.
 */
public class SGABridgeCommand extends SGACommandPOA {

  private String id;
  private static Integer id_counter=new Integer(0);

  /**
   * Referncia para a implementao do servio SGABridgeCommand
   */
  private SGACommand service = null;
  private SGACommand servant = null;
  private SGAServer sga = null;

  /**
   * Constri um servant para a interface SGABridgeCommand
   *
   * @param srv referncia para a implementao do servio SGABridgeCommand
   * @param sga .
   */
  public SGABridgeCommand( SGACommand srv, SGAServer sga ) {
    this.service = srv;
    this.sga = sga;
  }

  /**
   * Identificao do processo principal.
   * @return identificao (pid) do processo principal do comando
   */
  public int getPid() {
    if (service == null) return -1;
    return service.getPid();
  }

  /**
   * Indica se o comando ainda est ativo (em execuo).
   * @return true se o comando est em execuo
   */
  public boolean isRunning() {
    if (service == null) return false;
    return service.isRunning();
  }

  /**
   * Fornece as informaes de monitorao de um comando.
   * @return informaes de monitorao de todos os processos 
   * que compem o comando.
   */
  public RunningCommandInfo getRunningCommandInfo() {
    if (service == null) return null;
    return service.getRunningCommandInfo();
  }

  /**
   * Fornece a referncia para o SGA.
   * @return referncia para o SGA responsvel pela execuo do comando
   */
  public SGAServer getSGAServer() {
    return sga;
  }

  /**
   * Termina a execuo do comando.
   */
  public void kill() {
    if (service == null) return;
    service.kill();
  }

  public void activate() throws Exception {
     synchronized (id_counter) {
       ++id_counter;
       this.id = "Command" + id_counter.toString();
     }
     SGABridge.getInstance().activateObjectAtServerORB(id, this);
     this.servant = SGACommandHelper.narrow(this._this());
  }

  public void deactivate() {
     try {
       SGABridge.getInstance().deactivateObjectAtServerORB(this.id);
     }
     catch (Exception e) {
       System.out.println("No foi possvel desativar o comando "+
         this.id);
     }
     this.servant = null;
  }

  public SGACommand getServant() {
     if (this.servant == null)
        this.servant = SGACommandHelper.narrow(this._this());
     return this.servant;
  }

  public SGACommand getService() {
     return this.service;
  }

}
