-------------------------------------------------------------------------
-- $Author: valeria $
-- $Revision: 88437 $  - $Date: 2009-02-16 13:21:45 -0300 (Mon, 16 Feb 2009) $
-------------------------------------------------------------------------

local verbose = require "sga.verbose"

-- Definio de modo verbose
verbose.level(1)
verbose.flag("top", false)


-------------------------------------------------------------------------
---------------------<:: CONFIGURAO AVANADA ::>-----------------------
-------------------------------------------------------------------------

-------------------------------------------------------------------------
-- Definio de intervalos de tempo no loop principal do SGA-Daemon
-- Os tempos comentados so os valores default.
-------------------------------------------------------------------------

-- Intervalo, em segundos, para buscar referencia do Servidor
SGAD_CONF.ssiref_time_seconds = 15

-- Intervalo, em segundos, para tentar registrar o SGA no Servidor
SGAD_CONF.sgaregistry_time_seconds = 30

-- Intervalo, em segundos, para leitura de uso de cpu/memria
SGAD_CONF.machine_time_seconds = 25

-- Intervalo, em segundos, para envio de informaes dinmicas ao servidor
SGAD_CONF.update_time_seconds = 30

-- Intervalo, em segundos, para consulta de processos
SGAD_CONF.process_time_seconds = 10

-- Intervalo, em segundos, para busca de comandos terminados
SGAD_CONF.completed_time_seconds = 10

-- Intervalo, em segundos, para dump de comandos em execuo (persistencia)
SGAD_CONF.dump_time_seconds = 60

-- Numero de vezes que uma consulta (cpu/memria) deve falhar para
-- que seja considerado um erro.
SGAD_CONF.disregard_times = 3

-- Nmero de vezes que tenta fazer o registro no Servidor quando ocorre
-- falha de comunicao. (OPCIONAL -- default  no ter limite)
--SGAD_CONF.retries_number = 5

-------------------------------------------------------------------------
-- Nome da mquina onde ser executado o sgad
-- (evite alterar as propriedades abaixo, configure as variveis de 
-- ambiente no arquivo sga-daemon de acordo com o necessrio)
-------------------------------------------------------------------------

local HOSTNAME = os.getenv("HOSTNAME")
if HOSTNAME then
   SGAD_CONF.domain = string.gsub(HOSTNAME, "^[^%.]+","")
   SGAD_CONF.localhost = string.gsub(HOSTNAME, "^([^%.]+).*","%1")
end

local HOST = os.getenv("HOST")
if not HOST then
   verbose.error("\n\nEnvironment variable $HOST is undefined!!!\n\n")
   os.exit(0)
end

HOST = string.gsub(HOST,"%..*","")
SGAD_CONF.nodes = getSGANodes(HOST)
SGAD_CONF.name = HOST

if not SGAD_CONF.nodes then
   verbose.error( "\n\nNo configuration found for host: "..tostring(HOST).."!!!\n\n")
   os.exit(0)
end

function setConf(field)
  SGAD_CONF[field] = SGAD_CONF[field] or SGAD_CONF.nodes[1][field]
end

SGAD_CONF.requirements = {}
function setRequirement(req)
  SGAD_CONF.requirements[req]=SGAD_CONF[req] or SGAD_CONF.nodes[1][req]
end

setRequirement("deny_node_selection")
setConf("benchmarks")
setConf("net_benchmark")
setConf("platform_id")
setConf("os_platform")
setConf("manage_csfs")
setConf("csfs_properties")
setConf("ssi_bin_directory")
setConf("enable_historic")
setRequirement("enable_historic")
setConf("partition_name")
setConf("net_interface_name")
SGAD_CONF.os_platform = SGAD_CONF.os_platform or SGAD_CONF.platform_id
SGAD_CONF.ssibindir = SGAD_CONF.ssi_bin_directory and 
  (SGAD_CONF.ssi_bin_directory .. SGAD_CONF.os_platform)

-- Poderia fazer com que os campos nao encontrados em SGAD_CONF fossem
-- procurados em SGAD_CONF.nodes[1]... XXX

---------------------------------------------------------------------
-- Funo que define o mecanismo de herana com o campo 'parent' 
--
-- Definida no arquivo sga-command.lua, mas s  carregada depois...
---------------------------------------------------------------------
--function inherit_from_parent( tab, index )
--   if type(tab) ~= type({}) then
--      error( "indexed expression is not a table.")
--   end
--
--   local parent = rawget(tab, "parent")
--   if parent then
--      return parent[ index ]
--   else
--      return nil
--   end
--end
--
-- SGAD_CONF.meta = { 
--  __index = inherit_from_parent
-- }
-- 
-- setmetatable( SGAD_CONF, SGAD_CONF.meta )
-- SGAD_CONF.parent = SGAD_CONF.nodes[1]
--

-------------------------------------------------------------------------
--------------------------<:::::::::::::::::::>--------------------------
-------------------------------------------------------------------------

-------------------------------------------------------------------------
-- Localizao do Servidor
-------------------------------------------------------------------------
SGAD_CONF.ssi_host = os.getenv("SSI_HOSTNAME") or "localhost"
SGAD_CONF.ssi_port = os.getenv("SSI_PORT") or "7778"
SGAD_CONF.ssi_printablename = SGAD_CONF.ssi_host..":"..SGAD_CONF.ssi_port
SGAD_CONF.sga_addr = os.getenv("SGA_ADDR")

SGAD_CONF.path = os.getenv("PWD") or ""
SGAD_CONF.home = os.getenv("HOME") or ""

SGAD_CONF.restarthour = tonumber(os.getenv("RESTARTHOUR") or "")
if not SGAD_CONF.restarthour  or
   SGAD_CONF.restarthour > 23 or
   SGAD_CONF.restarthour < 0  then
   SGAD_CONF.restarthour = nil
end

-- endereo de localizao do Servidor (via corbaloc)
SGAD_CONF.ssi_ior = "corbaloc::"..
   SGAD_CONF.ssi_host..":"..SGAD_CONF.ssi_port..
   "/SGAMgr"..
   "/SGAMgrPOA/"..
   "SGAManager"

-- Nome do arquivo para dump de comandos
SGAD_CONF.dump_file = "./commands/"..SGAD_CONF.name

-- Diretrio do binrio do SGA
local sgabindir = os.getenv("SGAD_DIR")
SGAD_CONF.sgabindir = sgabindir or
  string.format("%s/../bin/%s/", SGAD_CONF.path, SGAD_CONF.os_platform)

CSFS.command = [[ cd ../csfs; ./startCSFS ]]
CSFS.process_id = "CSFS_PROCESS"
CSFS.properties = SGAD_CONF.csfs_properties or {}
local logdir = os.getenv("SGALOG_DIR")
if logdir then
  local logname = "csfs-" ..SGAD_CONF.name..".log"
  CSFS.properties.SERVER_LOG = "../src/"..logdir.."/"..logname
end
CSFS.properties.OAIAddr = CSFS.properties.OAIAddr or
  (SGAD_CONF.manage_csfs == YES and SGAD_CONF.localhost..SGAD_CONF.domain)

local iperfbindir = string.format("../bin/%s", SGAD_CONF.os_platform)
IPERF.command = " cd "..iperfbindir.."; ./iperf -s "
IPERF.process_id = "IPERF_PROCESS"
